// Generated on 2013-10-28 using generator-angular 0.5.1
'use strict';

// # Globbing
// for performance reasons we're only matching one level down:
// 'test/spec/{,*/}*.js'
// use this if you want to recursively match all subfolders:
// 'test/spec/**/*.js'

module.exports = function (grunt) {
  require('load-grunt-tasks')(grunt);
  require('time-grunt')(grunt);

  grunt.initConfig({
    shell: {
      version: {
        command: 'git rev-list --full-history --all | wc -l',
        options: {
          callback: function log(err, stdout, stderr, cb) {
            // Update the 'version.json' to the location where you want the file written
            var pkg = grunt.file.readJSON('package.json');
            var version = grunt.file.read('../infrastructure/properties/applicationVersionPrefix');
            grunt.file.write('dist/version.json', JSON.stringify({
              name: pkg.name,
              version: version + '.' + stdout.trim(),
              date: grunt.template.today()
            }));
            cb();
          }
        }
      }
    },
    yeoman: {
      // configurable paths
      app: require('./bower.json').appPath || 'app',
      dist: 'dist'
    },
    watch: {
      styles: {
        files: ['<%= yeoman.app %>/styles/{,*/}*.css'],
        tasks: ['copy:styles', 'autoprefixer']
      },
      livereload: {
        options: {
          livereload: '<%= connect.options.livereload %>'
        },
        files: [
          'components/**/*',
          '<%= yeoman.app %>/{,*/}*.html',
          '.tmp/styles/{,*/}*.css',
          '{.tmp,<%= yeoman.app %>}/scripts/{,*/}*.js',
          '<%= yeoman.app %>/images/{,*/}*.{png,jpg,jpeg,gif,webp,svg}'
        ]
      }
    },
    autoprefixer: {
      options: ['last 1 version'],
      dist: {
        files: [{
          expand: true,
          cwd: '.tmp/styles/',
          src: '{,*/}*.css',
          dest: '.tmp/styles/'
        }]
      }
    },
    connect: {
      options: {
        port: 9000,
        // Change this to '0.0.0.0' to access the server from outside.
        hostname: 'localhost',
        livereload: 35729
      },
      livereload: {
        options: {
          open: true,
          base: [
            '.tmp',
            '<%= yeoman.app %>'
          ]
        }
      },
      test: {
        options: {
          port: 9001,
          base: [
            '.tmp',
            'test',
            '<%= yeoman.app %>'
          ]
        }
      },
      dist: {
        options: {
          base: '<%= yeoman.dist %>'
        }
      }
    },
    clean: {
      dist: {
        files: [{
          dot: true,
          src: [
            '.tmp',
            '<%= yeoman.dist %>/*',
            '!<%= yeoman.dist %>/.git*'
          ]
        }]
      },
      server: '.tmp'
    },
    jshint: {
      options: {
        jshintrc: '.jshintrc'
      },
      all: [
        'Gruntfile.js',
        '<%= yeoman.app %>/scripts/{,*/}*.js'
      ]
    },
    // not used since Uglify task does concat,
    // but still available if needed
    /*concat: {
      dist: {}
    },*/
    rev: {
      dist: {
        files: {
          src: [
            '<%= yeoman.dist %>/scripts/{,*/}*.js',
            '<%= yeoman.dist %>/styles/{,*/}*.css',
            '<%= yeoman.dist %>/images/{,*/}*.{png,jpg,jpeg,gif,webp,svg}',
            '<%= yeoman.dist %>/styles/fonts/*',
            '<%= yeoman.dist %>/components/{,*/}*.{css,png,jpg,jpeg,gif,webp,svg}'
          ]
        }
      }
    },
    useminPrepare: {
      html: '<%= yeoman.app %>/index.html',
      options: {
        dest: '<%= yeoman.dist %>'
      }
    },
    usemin: {
      html: ['<%= yeoman.dist %>/{,*/}*.html'],
      css: ['<%= yeoman.dist %>/styles/{,*/}*.css'],
      options: {
        basedir: '<%= yeoman.dist %>',
        dirs: ['<%= yeoman.dist %>']
      }
    },
    //  imagemin: {
    //    dist: {
    //      files: [{
    //        expand: true,
    //        cwd: '<%= yeoman.app %>/images',
    //        src: '{,*/}*.{png,jpg,jpeg}',
    //        dest: '<%= yeoman.dist %>/images'
    //      }]
    //    }
    //  },
    svgmin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= yeoman.app %>/images',
          src: '{,*/}*.svg',
          dest: '<%= yeoman.dist %>/images'
        }]
      }
    },
    cssmin: {
      // By default, your `index.html` <!-- Usemin Block --> will take care of
      // minification. This option is pre-configured if you do not wish to use
      // Usemin blocks.
      // dist: {
      //   files: {
      //     '<%= yeoman.dist %>/styles/main.css': [
      //       '.tmp/styles/{,*/}*.css',
      //       '<%= yeoman.app %>/styles/{,*/}*.css'
      //     ]
      //   }
      // }
    },
    htmlmin: {
      dist: {
        options: {
          /*removeCommentsFromCDATA: true,
          // https://github.com/yeoman/grunt-usemin/issues/44
          //collapseWhitespace: true,
          collapseBooleanAttributes: true,
          removeAttributeQuotes: true,
          removeRedundantAttributes: true,
          useShortDoctype: true,
          removeEmptyAttributes: true,
          removeOptionalTags: true*/
        },
        files: [{
          expand: true,
          cwd: '<%= yeoman.app %>',
          src: ['*.html', 'views/*.html', 'scripts/modules/**/*.html'],
          dest: '<%= yeoman.dist %>'
        }]
      }
    },
    // Put files not handled in other tasks here
    copy: {
      dist: {
        files: [{
          expand: true,
          dot: true,
          cwd: '<%= yeoman.app %>',
          dest: '<%= yeoman.dist %>',
          src: [
            '*.{ico,png,txt,json}',
            '.htaccess',
            '*.pdf',
            'components/**/*',
            '!components/**/*.js',
            'images/{,*/}*.{gif,webp,png}',
            'styles/fonts/*',
            'fonts/*'
          ]
        }, {
          expand: true,
          cwd: '.tmp/images',
          dest: '<%= yeoman.dist %>/images',
          src: [
            'generated/*',
            'images/{,*/}*.{gif,webp,png}'
          ]
        }]
      },
      styles: {
        expand: true,
        cwd: '<%= yeoman.app %>/styles',
        dest: '.tmp/styles/',
        src: '{,*/}*.css'
      }
    },
    concurrent: {
      server: [
        'copy:styles'
      ],
      test: [
        'copy:styles'
      ],
      dist: [
        'copy:styles',
        'svgmin',
        'htmlmin'
      ]
    },
    karma: {
      unit: {
        configFile: 'karma.conf.js',
        singleRun: true
      }
    },
    cdnify: {
      dist: {
        html: ['<%= yeoman.dist %>/*.html']
      }
    },
    ngmin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= yeoman.dist %>/scripts',
          src: '*.js',
          dest: '<%= yeoman.dist %>/scripts'
        }]
      }
    },
    uglify: {
      dist: {
        files: {
          '<%= yeoman.dist %>/scripts/scripts.js': [
            '<%= yeoman.dist %>/scripts/scripts.js'
          ]
        }
      }
    },
    htmlangular: {
      options: {
        reportpath: null,
        tmplext: 'tpl.html',
        customtags: [
          'sham-spinner',
          'pagination',
          'audit-trail',
          'admin-measurement',
          'admin-vital-message',
          'admin-schedule',
          'admin-motivational-message',
          'admin-error-message',
          'highchart',
          'checkbox-group'
        ],
        customattrs: [
          'sham-spinner',
          'datepicker-popup',
          'datepicker-options',
          'date-disabled',
          'close-text',
          'is-open',
          'min',
          'max',
          'main-content',
          'global-header-content',
          'admin-tabs-content',
          'collapse',
          'tabs-content',
          'justified',
          'role',
          'ng-form',
          'name',
          'numeric-only',
          'tooltip',
          'validlength'
        ],
        //ignores these errors
        relaxerror: [
          'Bad value X-UA-Compatible for attribute http-equiv on element meta.',
          'Duplicate ID templates[{{templateIndex}}].measurement.',
          'The first occurrence of ID templates[{{templateIndex}}].measurement was here.',
          'An ID must not contain whitespace',
          // clinicalAdminMeasure.tpl - duplicate IDs, for testing.
          'Duplicate ID error-list.',
          'The first occurrence of ID error-list was here.',
          // clinicalAdminProtocol.tpl.html
          'Duplicate ID protocol[{{ $index }}].focus-area',
          'The first occurrence of ID protocol[{{ $index }}].focus-area was here.',
          'Bad value #/clinicalAdmin/auditTrail/{{ getProtocolId(protocol) }} for attribute href on element a: Illegal character in fragment: not a URL code point.',
          'Duplicate ID protocol[{{ $index }}].enable.',
          'Duplicate ID protocol[{{ $index }}].disable.',
          'The first occurrence of ID protocol[{{ $index }}].enable was here.',
          'The first occurrence of ID protocol[{{ $index }}].disable was here.',
          // clinicalAdminErrorMessages.tpl.html
          'Duplicate ID templates[{{templateIndex}}].global-error.',
          'The first occurrence of ID templates[{{templateIndex}}].global-error was here.',
          // clinicalAdminVitalMessage.tpl.html
          'Element option without attribute label must not be empty.',
          // messageHistory.tpl.html
          'Duplicate ID message-page.',
          'The first occurrence of ID message-page was here.',
          // patientProtocolEdit.tpl.html
          'Duplicate ID templates[{{templateIndex}}].heading-message-settings.',
          'The first occurrence of ID templates[{{templateIndex}}].heading-message-settings was here.',
          'Duplicate ID templates[{{templateIndex}}].message-setting.',
          'The first occurrence of ID templates[{{templateIndex}}].message-setting was here.',
          'Duplicate ID save-bottom.',
          'The first occurrence of ID save-bottom was here.',
          // patientProtocolList.tpl.html
          'Bad value #/patient/account/{{currPatient._id}} for attribute href on element a: Illegal character in fragment: not a URL code point.',
          'Duplicate ID assign-protocol-btn.',
          'The first occurrence of ID assign-protocol-btn was here.',
          // patientRegistration.tpl.html
          'Duplicate ID heading-patient-registration.',
          'The first occurrence of ID heading-patient-registration was here.',
          'Duplicate ID select-patient-consent-form.',
          'The first occurrence of ID select-patient-consent-form was here.',

          // The action attribute is mandatory, and it must contain a valid URI. But according to the URI RFC, an empty URI is still a URI:
          // A URI reference that does not contain a URI is a reference to the current document. In other words, an empty URI reference within a document is interpreted as a reference to the start of that document, and a reference containing only a fragment identifier is a reference to the identified fragment of that document. Traversal of such a reference should not result in an additional retrieval action. However, if the URI reference occurs in a context that is always intended to result in a new request, as in the case of HTML's FORM element, then an empty URI reference represents the base URI of the current document and should be replaced by that URI when transformed into a request.
          'Bad value  for attribute action on element form: Must be non-empty.'
        ]
      },
      validateHTML: {
        // Target-specific file lists and/or options go here.
        src: [
          'app/**/*.html',
          '!app/bower_components/**'
        ],
      }
    }
  });

  grunt.registerTask('server', function (target) {
    if (target === 'dist') {
      return grunt.task.run(['build', 'connect:dist:keepalive']);
    }

    grunt.task.run([
      'clean:server',
      'concurrent:server',
      'autoprefixer',
      'connect:livereload',
      'watch'
    ]);
  });

  grunt.registerTask('test', [
    'clean:server',
    'concurrent:test',
    'autoprefixer',
    'connect:test',
    'karma'
  ]);

  grunt.registerTask('build', [
    'clean:dist',
    'useminPrepare',
    'concurrent:dist',
    'autoprefixer',
    'concat',
    'copy:dist',
    'cdnify',
    'ngmin',
    'cssmin',
    'uglify',
    'rev',
    'usemin'
  ]);

  grunt.registerTask('default', [
    'jshint',
    'test',
    'build'
  ]);

  grunt.registerTask( 'version', [
    'shell:version'
  ]);

  grunt.registerTask('artifactBuild', [
    'default',
    'version'
  ]);
};
