/*jslint node: true, unparam: true */
/*global angular, property */

'use strict';

describe('breadcrumbs', function () {

    beforeEach(module('broadcast'));
    beforeEach(module('MessagingAppApp'));


    var breadcrumbs,
        $httpBackend,
        $route,
        staffProfileMockService,
        facilityMsgResolvedMock;


    staffProfileMockService = {
        query: function (x) {
            return {
                $promise: {
                    then: function () {
                        return true;
                    }
                }
            };
        }
    };

    facilityMsgResolvedMock = {
        'enabled': true,
        'name': 'October Flu Shots',
        'assigningAuthority': 'dfn-688',
        'createDate': '10/02/2014',
        'createBy': 'One Vehu',
        '_id': '54492aa1c0e22f7e6fafd1b3',
        'message': 'CDC recommends an annual flu vaccine for everyone 6 months of age and older.',
        'startDate': '2014-10-23T04:00:00.000Z',
        'endDate': '2018-12-25',
        'schedule': {
            'acceptUnscheduledReading': false,
            'scheduleType': {
                '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany',
                'name': 'Custom',
                'description': 'Every 10 minute',
                'interval': {
                    'value': 10,
                    'unit': 'MINUTE'
                }
            },
            'scheduleTime': {
                'hour': 8,
                'min': 30,
                'meridian': 'AM',
                'description': '(At 8:30 AM)'
            },
            'isStartTimeRelative': false,
            'isEndTimeRelative': false
        },
        'progress': 'Delivered',
        'comments': {
            'content': null,
            'user': 'EIGHT VEHU',
            'time': '12:21 PM',
            'date': '2014-10-23',
            'changes': []
        },
        'createdDateUTC': '2014-10-02 16:05:57'
    };

    beforeEach(inject(function (_$route_) {
        $route = _$route_;
        $route.current = {
            params: { 
                id: 2,
                label: 'Add Message'
            },
            originalPath: '/broadcast/facility/edit-message/:id'
        };
    }));

    beforeEach(inject(function (_breadcrumbs_, _$httpBackend_) {
        breadcrumbs = _breadcrumbs_;
        $httpBackend = _$httpBackend_;
    }));

    describe('#get()', function () {
        it('should return a collection of breadcrumb objects with their default labels', function () {
            var crumbs = breadcrumbs.get();
            expect(crumbs.length).toEqual(2);
            expect(crumbs[0].label).toEqual('VA Facility Broadcast Messages');
            expect(crumbs[1].label).toEqual('Edit Message');
        });
    });

    

    describe('#get(options)', function () {
        it('should return a collection of breadcrumb objects with the specified labels', function () {
            var crumbs = breadcrumbs.get({
                'Edit Message': facilityMsgResolvedMock.name
            });
            expect(crumbs[0].label).toEqual('VA Facility Broadcast Messages');
            expect(crumbs[1].label).toEqual('October Flu Shots');
        });
    });
});