/*jslint node: true, unparam: true */
/*global angular, navigator */

'use strict';

angular.module('broadcast', [
    'ngCookies',
    'ngResource',
    'ngRoute',
    'ui.bootstrap',
    'broadcast.facility',
    'breadcrumbs'
]).config([
    '$routeProvider',
    function ($routeProvider) {
        $routeProvider.when('/broadcast/facility', {
            templateUrl: 'components/broadcast/facility/facility-message.tpl.html',
            controller: 'broadcastFacilityController',
            label: 'VA Facility Broadcast Messages',
            resolve: {
                facilityMsgList: ['$q', 'broadcastFacilityService', function ($q, broadcastFacilityService) {
                    var deferred = $q.defer();
                    broadcastFacilityService.query(
                        function (resp) {
                            deferred.resolve(resp);
                        });
                    return deferred.promise;
                }]
            },
            role: 'bcmView'
        }).when('/broadcast/facility/add-message', {
            templateUrl: 'components/broadcast/facility/facility-crud/facility-add-message.tpl.html',
            controller: 'broadcastFacilityAddController',
            label: 'Add Message',
            role: 'bcmView',
            resolve: {
                resolvedUser: ['$q', 'staffProfileService', function ($q, staffProfileService) {
                    var deferred = $q.defer();
                    staffProfileService.query().$promise.then(
                        function (resp) {
                            deferred.resolve(resp);
                        });
                    return deferred.promise;
                }],
                resolvedMessage: function () { 
                    return null; 
                }
            }
        }).when('/broadcast/facility/edit-message/:id', {
            templateUrl: 'components/broadcast/facility/facility-crud/facility-add-message.tpl.html',
            controller: 'broadcastFacilityAddController',
            label: 'Edit Message',
            role: 'bcmView',
            resolve: {
                resolvedUser: ['$q', 'staffProfileService', function ($q, staffProfileService) {
                    var deferred = $q.defer();

                    staffProfileService.query().$promise.then(
                        function (resp) {
                            deferred.resolve(resp);
                        }
                    );
                    return deferred.promise;
                }],
                resolvedMessage: ['$q', '$route', 'broadcastMessageService', function ($q, $route, broadcastMessageService) {
                    var deferred = $q.defer(),
                        msgId = $route.current.params.id;

                    broadcastMessageService.query({ id: msgId }, function (resp) {
                        deferred.resolve(resp);
                    });
                    return deferred.promise;
                }]
            }
        });
    }
]);

angular.module('broadcast').run([
    '$rootScope',
    'staffProfileService',
    '$location',
    function ($rootScope, staffProfileService, $location) {
        $rootScope.$on('$routeChangeStart', function () {
            var bcmProfile = staffProfileService.query();
            bcmProfile.$promise.then(function (result) {
                if (result.isBcmAdmin && !result.isAdmin && !result.isStaff) {
                    $location.path('/broadcast/facility');
                }
            });
        });
    }
]);

