/*jslint node: true, unparam: true */
/*global angular, navigator */

'use strict';

describe('broadcastFacilityController', function () {
    var scope,
        MainCtrl ,
        httpBackend,
        mockModal,
        mockModalInstance,
        modalInstance,
        mockService,
        mockUserInfo,
        $modal,
        rootScope,
        mockRouteService,
        staffProfileMockService,
        breadCrumbsMockService,
        mockFacilityMsgList;


    //need to indicate your module in a test
    beforeEach(module('broadcast.facility'));

    mockService = {
        query: function (x) {
            return 'ok';
        }
    };
    mockRouteService = {
        reload: function () {
            return 'ok';
        }
    };
    staffProfileMockService = {
        query: function (x) {
            return {$promise: {then: function(){}}};
        }
    };
    breadCrumbsMockService = {
        result: {
            then: function (confirmCallback, cancelCallback) {
                //Store the callbacks for later when the user clicks on the OK or Cancel button of the dialog
                this.confirmCallBack = confirmCallback;
                this.cancelCallback = cancelCallback;
            }
        },
        close: function (item) {
            //The user clicked OK on the modal dialog, call the stored confirm callback with the selected item
            this.result.confirmCallBack(item);
        },
        get: function () {
            //The user clicked cancel on the modal dialog, call the stored cancel callback
            return true;
        }
    };
    mockFacilityMsgList = [{
        '_id': '544526c00d76bfe7265ac7fe',
        'assigningAuthority': 'dfn-688',
        'comments': {
            'changes': {
                'length': 0
            },
            'content': 'Disable',
            'date': '2014-10-12',
            'time': '12:05 PM',
            'user': 'FOUR VEHU'
        },
        'createBy': 'One Vehu',
        'createDate': '10/12/2014',
        'createdDateUTC': '2014-10-12 16:05:57',
        'enabled': true,
        'endDate': '2015-10-25',
        'message': "The Veterans Health Administration is America's largest integrated health care system with over 1,700 sites of care, serving 8.76 million Veterans each year.",
        'name': 'VA Information',
        'progress': 'Pending Delivery',
        'schedule': {
            'acceptUnscheduledReading': false,
            'isEndTimeRelative': false,
            'isStartTimeRelative': false,
            'scheduleTime': {
                'description': '(At 8:30 AM)',
                'hour': 8,
                'meridian': 'AM',
                'min': 30,
            },
            'scheduleType': {
                '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany',
                'description': 'Every 10 minute',
                'interval': {
                    'name': 'Custom'
                }
            }
        },
        'startDate': '2014-10-23T04:00:00.000Z'
    }];
    mockUserInfo = {
        'firstName': 'FOUR', 
        'middleName': null, 
        'lastName': 'VEHU', 
        'isAdmin': true, 
        'isBcmAdmin': true
    };

    beforeEach(inject(function ($modal) {
        spyOn($modal, 'open').andReturn(mockModal);
    }));

    beforeEach(inject(function ($rootScope, $controller, $injector, _$httpBackend_, _$modal_) {
        modalInstance = { // Create a mock object using spies
            close: jasmine.createSpy('modalInstance.close'),
            dismiss: jasmine.createSpy('modalInstance.dismiss'),
            result: {
                then: jasmine.createSpy('modalInstance.result.then')
            }
        };
        scope = $injector.get('$rootScope');
        rootScope = $injector.get('$rootScope');
        $controller = $injector.get('$controller');
        httpBackend = _$httpBackend_;
        $modal = _$modal_;
        MainCtrl = $controller('broadcastFacilityController', {
            $scope: scope,
            broadcastFacilityService: mockService,
            staffProfileService: staffProfileMockService,
            facilityMsgList: mockFacilityMsgList,
            $modalInstance: modalInstance,
            breadcrumbs: breadCrumbsMockService,
            $route: mockRouteService
        });
        scope.userInfo = mockUserInfo;
    }));

    describe('Testing the Pagination', function () {
        it('Test the page number for pagination function', function () {
            scope.setPage(2);
            expect(scope.currentPage).toEqual(2);
        });
    });
    describe('Testing the Filtering and Table of Messages', function () {
        it('Test initing and resetting the initial table of messages', function () {
            expect(scope.facilityList.length).toEqual(1);
            scope.resetTable();
            expect(scope.progressFilter).toEqual('All');
            expect(scope.viewFilter).toEqual('All');
        });
        it('Test the filtering of messages - Enabled', function () {
            scope.filteredTable = [];
            scope.progressFilter = 'All';
            scope.viewFilter = 'Enabled';
            scope.filterMsgList();
            expect(scope.filteredTable).toEqual(scope.facilityList);
        });
        it('Test the filtering of messages - Disabled', function () {
            scope.filteredTable = [];
            scope.progressFilter = 'All';
            scope.viewFilter = 'Disabled';
            scope.filterMsgList();
            expect(scope.filteredTable).toEqual([]);
        });
        it('Test the filtering of messages - Created By Me', function () {
            scope.filteredTable = [];
            scope.progressFilter = 'All';
            scope.viewFilter = 'Created By Me';
            scope.filterMsgList();
            expect(scope.filteredTable).toEqual([]);
        });
        it('Test the filtering of messages - Pending Delivery', function () {
            scope.filteredTable = [];
            scope.progressFilter = 'Pending Delivery';
            scope.viewFilter = 'All';
            scope.filterMsgList();
            expect(scope.filteredTable).toEqual(scope.facilityList);
        });
    });
    describe('Testing the Enable/Disable Calls from Main Page', function () {
        it('Test calling a disabled function on an item', function () {
            scope.openEnableDisable(true, scope.facilityList[0]);
            expect($modal.open).toHaveBeenCalled();
        });
        it('Test calling a enabled function on an item', function () {
            scope.openEnableDisable(false, scope.facilityList[0]);
            expect($modal.open).toHaveBeenCalled();
        });
        it('Testing the $emit of confirmedDisable', function () {
            scope.filteredTable = '';
            rootScope.$emit('confirmedDisable', scope.facilityList[0], 0);
            expect(scope.filteredTable).toEqual(scope.facilityList);
        });
    });
});
