/*jslint node: true, unparam: true */
/*global angular */
/* jshint unused:false */

'use strict';

angular.module('broadcast.facility').controller('broadcastFacilityController', [
    '$scope',
    '$location',
    '$log',
    '$modal',
    '$rootScope',
    '$route',
    '$timeout',
    'facilityMsgList',  // from route resolve.
    'staffProfileService',
    'breadcrumbs',
    function ($scope, $location, $log, $modal, $rootScope, $route, $timeout, facilityMsgList, staffProfileService, breadcrumbs) {
        // ------------------------------------------------------------------------
        // All the Inits
        // ------------------------------------------------------------------------
        $scope.breadcrumbs = breadcrumbs;
        // For Angular-UI datepicker
        $scope.dtTo = null;
        $scope.dtFrom = null;

        // init the filter drop downs
        $scope.progressFilter = 'All';
        $scope.filteredTable = [];
        $scope.progressFilter = 'All';
        $scope.viewFilter = 'All';
        $scope.dateChanged = false;

        // Stroing a list of facility message after promise in routeprovider
        $scope.facilityList = facilityMsgList;

        // Code for pagination of facility messages
        $scope.currentPage = 1;
        $scope.itemsPerPage = 20;

        $scope.dateOptions = {
            'year-format': 'yy',
            'starting-day': 1
        };
        $scope.format = 'MM/dd/yyyy';


        // ------------------------------------------------------------------------
        // Watches
        // ------------------------------------------------------------------------
        // to display the success message on the parent scope page
        $rootScope.$on('msgSaveSuccess', function (event) {

            $rootScope.msgSaveSuccess = true;
        });

        // watch for the datepickers to close on selection and then filter the table
        $scope.$watch('[openedStart, openedEnd]', function (newValue) {
            // testing for false because if these variables changed and they are false, it means a datepicker was closed
            if (newValue.indexOf(false) !== -1) {
                $scope.filterDates();
            }
        }, true);

        // Watch for $emit of Disabled Modal Yes Click.
        $rootScope.$on('confirmedDisable', function (event, data, index) {
            $scope.facilityList[index] = data;
            $scope.resetTable();
            $scope.filterMsgList();
            $route.reload();
        });

        // Watch for $emit of Disabled Modal No Click.
        $rootScope.$on('cancelDisable', function (event, index) {
            $scope.filterMsgList();
        });




        // ------------------------------------------------------------------------
        // Functions
        // ------------------------------------------------------------------------

        // This function is only called when typing inside the datepicker input field...
        $scope.enter = function (event) {
            if (event !== undefined && event.keyCode === 13) {
                $scope.openedEnd = false;
                $scope.openedStart = false;
                $scope.filterDates();
            }
        };

        // ------------------------------------------------------------------------
        // Functions
        // ------------------------------------------------------------------------

        // Use a function to call the User Profile Service.
        // This is easier to test.
        $scope.getUserProfile = function () {
            // Getting the User Profile
            staffProfileService.query().$promise.then(function (data) {
                $scope.userInfo = data;
            }, function () {
                $scope.userInfo = null;
            });
        };

        // Function to format the scheduleTime ??
        $scope.formatScheduleTime = function (time) {
            var scheduleTime = time;
            scheduleTime = scheduleTime.replace(/[(]At |[)]/g, '');
            return scheduleTime;
        };

        // set the table to the full dataset
        $scope.resetTable = function () {
            // Init
            var len, // generic counter for facilityList length
                fromDate,
                toDate;

            // if there is data to reset...
            if ($scope.facilityList.length > 0) {
                // Set the initial table which can be altered by filters to the entire table since filters default to ALL
                $scope.filteredTable = angular.copy($scope.facilityList);

                toDate = new Date($scope.facilityList[0].createDate);
                $scope.dtTo = new Date(toDate.getUTCFullYear(), toDate.getUTCMonth(), toDate.getUTCDate());

                // get the first date from the message list
                len = $scope.facilityList.length;
                fromDate = new Date($scope.facilityList[len - 1].createDate);
                // convert it to UTC since we parsed the date from a literal string
                $scope.dtFrom = new Date(fromDate.getUTCFullYear(), fromDate.getUTCMonth(), fromDate.getUTCDate());

                $scope.progressFilter = 'All';
                $scope.viewFilter = 'All';
            }
        };

        $scope.setPage = function (pageNo) {
            $scope.currentPage = pageNo;
        };

        $scope.filterMsgList = function () {
            // Init
            var isProto,
                isStatus,
                isDateOk,
                date,
                dateUTC,
                msgListCopy,
                myName,
                i;

            if ($scope.progressFilter === 'All' && $scope.viewFilter === 'All' && $scope.dateChanged === false) {
                $scope.filteredTable = angular.copy($scope.facilityList);
            } else {
                // Since there can be so many combinations of the table we need to rebuild the table based on the 3 criteria
                // We need to make a copy and not set by reference via copy function...
                msgListCopy = angular.copy($scope.facilityList);
                myName = $scope.userInfo.firstName + ' ' + $scope.userInfo.lastName;
                $scope.filteredTable = [];

                // This is wrong.
                // http://stackoverflow.com/questions/1963102/what-does-the-jslint-error-body-of-a-for-in-should-be-wrapped-in-an-if-statemen
                for (i in msgListCopy) {
                    // re-initing
                    isProto = false;
                    isStatus = false;
                    isDateOk = false;

                    if ($scope.progressFilter === 'All' || msgListCopy[i].progress === $scope.progressFilter) {
                        isProto = true;
                    }

                    if ($scope.viewFilter === 'All' || ($scope.viewFilter === 'Created By Me' && msgListCopy[i].createBy.toLowerCase() === myName.toLowerCase()) || ($scope.viewFilter === 'Enabled' && msgListCopy[i].enabled === true) || ($scope.viewFilter === 'Disabled' && msgListCopy[i].enabled === false)) {
                        isStatus = true;
                    }

                    // now filter by dates...
                    date = new Date(msgListCopy[i].createDate);
                    dateUTC = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate());

                    if (dateUTC >= $scope.dtFrom && dateUTC <= $scope.dtTo) {
                        isDateOk = true;
                    }

                    if (isProto && isStatus && isDateOk) {
                        $scope.filteredTable.push(msgListCopy[i]);
                    }
                }
            }

            $scope.setPage(1);
            $scope.dateChanged = false;
        };

        $scope.filterDates = function () {
            $scope.dateChanged = true;
            $scope.filterMsgList();
        };


        // for calendars
        $scope.openStart = function ($event) {
            $timeout(function () {
                $event.preventDefault();
                $event.stopPropagation();
                $scope.openedStart = true;
            });
        };

        $scope.openEnd = function ($event) {
            $timeout(function () {
                $event.preventDefault();
                $event.stopPropagation();
                $scope.openedEnd = true;
            });
        };

        // for Enable/Disabled
        $scope.openEnableDisable = function (flag, message, messageIndex) {
            // flag is either true or false coming in.
            // true denotes disable.
            // This follows pattern in patientProtocolListController.js
            var modalInstance;
            if (flag === true) {
                modalInstance = $modal.open({
                    templateUrl: 'components/broadcast/facility/facility-crud/facility-crud-modal-disable.tpl.html',
                    controller: 'broadcastEnableDisableModalCtrl',
                    backdrop: 'static',
                    keyboard: false,
                    resolve: {
                        selectedMessage: function () {
                            message.enabled = false;
                            return message;
                        },
                        messageIndex : function () {
                            return messageIndex;
                        }
                    }
                });
            } else {
                modalInstance = $modal.open({
                    templateUrl: 'components/broadcast/facility/facility-crud/facility-crud-modal-enable.tpl.html',
                    controller: 'broadcastEnableDisableModalCtrl',
                    backdrop: 'static',
                    keyboard: false,
                    resolve: {
                        selectedMessage: function () {
                            message.enabled = true;
                            return message;
                        },
                        messageIndex : function () {
                            return messageIndex;
                        }
                    }
                });
            }
        };


        // ------------------------------------------------------------------------
        // Page Load Calls.
        // ------------------------------------------------------------------------
        $scope.getUserProfile();
        $scope.resetTable();

        if ($rootScope.msgSaveSuccess === true) {
            // Clear the succes message
            $timeout(function () {
                $rootScope.msgSaveSuccess = false;
            }, 10000);
        }
    }
]);
