/*jslint node: true, unparam: true */
/*global angular, property */

'use strict';

angular.module('broadcast.facility.crud').controller('broadcastFacilityAddController', [
    '$scope',
    '$http',
    '$log',
    '$rootScope',
    '$filter',
    '$modal',
    '$location',
    '$window',
    'breadcrumbs',
    'broadcastFacilityService',
    'resolvedMessage',
    'resolvedUser',
    function ($scope, $http, $log, $rootScope, $filter, $modal, $location, $window, breadcrumbs, broadcastFacilityService, resolvedMessage, resolvedUser) {
        // ------------------------------------------------------------------------
        // All the Inits
        // ------------------------------------------------------------------------
        $scope.breadcrumbs = breadcrumbs;
        $scope.facilityMsg = {};
        $rootScope.isEditingMsg = false;

        $scope.facilityMsgLocals = {
            'crudScreenTitle': 'Add Message',
            'title': '',
            'message': '',
            'schedule': '',
            'scheduleAllOptions': [
                {
                    'id': 'once',
                    'name': 'Once'
                }, {
                    'id': 'specified-days',
                    'name': 'Specified Days'
                }, {
                    'id': 'custom',
                    'name': 'Custom'
                }
            ],
            'hoursList': [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 ],
            'minutesList': [ '00', '01', '02', '03', '04', '05', '06', '07', '08', '09',
                           '10', '11', '12', '13', '14', '15', '16', '17', '18', '19',
                           '20', '21', '22', '23', '24', '25', '26', '27', '28', '29',
                           '30', '31', '32', '33', '34', '35', '36', '37', '38', '39',
                           '40', '41', '42', '43', '44', '45', '46', '47', '48', '49',
                           '50', '51', '52', '53', '54', '55', '56', '57', '58', '59' ],
            'startTime': {
                'hour': '',
                'minutes': '',
                'ampm': ''
            },
            'days': {
                'mon': '',
                'tues': '',
                'weds': '',
                'thurs': '',
                'fri': '',
                'sat': '',
                'sun': ''
            },
            'cycle': {
                'time': '',
                'custom': ''
            },
            'startDateOpen': false,
            'startMinDate': $filter('date')(new Date(), 'MM/dd/yyyy'),
            'startMaxDate': '',
            'startDateOptions': {
                'year-format': 'yy',
                'starting-day': 1
            },
            'endDateOpen': false,
            'endMinDate': $filter('date')(new Date(), 'MM/dd/yyyy'),
            'endDateOptions': {
                'year-format': 'yy',
                'starting-day': 1
            }
        };

        // scroll to top of page
        $window.scrollTo(0,0);

        // Set the Create Date and Created By
        $scope.facilityMsg.createDate = $scope.facilityMsgLocals.startMinDate;

        if(resolvedUser !== undefined && resolvedUser !== null)
            $scope.facilityMsg.createBy = resolvedUser.firstName + ' ' + resolvedUser.lastName;

        // We will have data in the resolvedMessage object if we got here via the edit button route
        if ( resolvedMessage !== undefined && resolvedMessage !== null ){
            $log.info('resolvedMessage = ');
            $log.info(resolvedMessage);

            // we have to convert start time minutes to a string because of the drop down options defined above
            $scope.msgTimeMin = resolvedMessage.schedule.scheduleTime.min.toString();
            if ($scope.msgTimeMin.length === 1) {
                $scope.msgTimeMin = '0' + $scope.msgTimeMin;
            }
            $scope.facilityMsg = resolvedMessage;
            $scope.facilityMsg.createDate = $filter('date')($scope.facilityMsg.createDate, 'MM/dd/yyyy');

            // Set the dynamic breadcrumb title
            breadcrumbs.options = {
                'Edit Message': $scope.facilityMsg.name
            };

            // Let the directive's validation know that we are editing because there are different rules
            $scope.facilityMsgLocals.crudScreenTitle = 'Edit Message';
        }

        // ------------------------------------------------------------------------
        // Watches
        // ------------------------------------------------------------------------
        $scope.$watch('facilityMsg.schedule.scheduleType.name', function (newValue, oldValue) {
            // Ignore initial setup.
            if (newValue === oldValue) {
                return;
            }
            // Loop through the Form schedule error elements.
            // Here we are setting the Validity to True, clearing out error.
            for (property in $scope.facilityMsgForm.schedule.$error) {
                $scope.facilityMsgForm.schedule.$setValidity(property, true);
            }
            // This is hack-y I think, but I'm very tired.
            // On change we're still populating the form object with the other child objects.
            // This resets it on change.  allowing it to populate; and then get validated on submit.
            $scope.facilityMsg.schedule.scheduleType = {};
            $scope.facilityMsg.schedule.scheduleType.name = newValue;
        });


        // ------------------------------------------------------------------------
        // Functions
        // ------------------------------------------------------------------------

        // Add Message Submit
        $scope.addMessageSubmit = function () {

            $scope.facilityMsg.schedule.scheduleType['@class'] = $scope.scheduleClass;
            // Always set an edited or added message as enabled.
            $scope.facilityMsg.enabled = true;

            $scope.facilityMsg.schedule.scheduleTime.min = $scope.msgTimeMin;

            // Save service. this service is from main module.s
            broadcastFacilityService.save($scope.facilityMsg,
                function () {
                    $log.info('success - form submitted');
                    $scope.successModal();
                },
                function () {
                    $log.info('failed');
                }
            );
        };

        // Updating the Min and Max Dates
        $scope.updateEndMinDate = function (newStartDate) {
            $scope.facilityMsgLocals.endMinDate = newStartDate;
        };
        $scope.updateStartMaxDate = function (newEndDate) {
            $scope.facilityMsgLocals.startMaxDate = newEndDate;
        };

        // Open Date via Widget-click
        $scope.openStartDate = function ($event) {
            $event.preventDefault();
            $event.stopPropagation();

            $scope.facilityMsgLocals.openedStartDate = true;
        };
        $scope.openEndDate = function ($event) {
            $event.preventDefault();
            $event.stopPropagation();

            $scope.facilityMsgLocals.openedEndDate = true;
        };

        // Cancel Modal
        $scope.cancelMessage = function () {
            var modalInstance = $modal.open({
                templateUrl: 'components/broadcast/facility/facility-crud/facility-crud-cancel-modal.tpl.html',
                controller: 'broadcastFacilityModalInstanceCtrl'
            });
            modalInstance.result.then(function () {
                $log.info('Modal dismissed at: ' + new Date());
            }, function () {
                $location.path('/broadcast/facility');
            });
        };
        // Sucess Modal, this redirects.
        $scope.successModal = function () {
            var modalInstance = $modal.open({
                templateUrl: 'components/broadcast/facility/facility-crud/facility-crud-save-modal.tpl.html',
                controller: 'broadcastFacilityModalInstanceCtrl'
            });
            modalInstance.result.then(function () {
                $location.path('/broadcast/facility');

                // to display the success message on the parent scope page
                $rootScope.$emit('msgSaveSuccess');
            }, function () {
                $location.path('/broadcast/facility');
            });
        };


    }
]);
