/*jslint node: true, unparam: true, nomen: true */
/*global angular, navigator, describe, beforeEach, it, inject, expect, jasmine */

'use strict';

describe('Testing Add Facility Messages in the broadcastFacilityAddController', function () {
    var dateFilter,
        scope,
        MainCtrl,
        httpBackend,
        mockService,
        mockMsg,
        staffProfileMockService,
        breadCrumbsMockService;

    //need to indicate your module in a test
    beforeEach(module('ui.bootstrap'));
    beforeEach(module('validation'));
    beforeEach(module('MessAppDirectives'));
    beforeEach(module('broadcast.facility.crud'));

    mockService = {
        query: function (x) {
            return {
                'enabled': true,
                'name': 'October Flu Shots',
                'assigningAuthority': 'dfn-688',
                'createDate': '10/02/2014',
                'createBy': 'One Vehu',
                '_id': '54492aa1c0e22f7e6fafd1b3',
                'message': 'CDC recommends an annual flu vaccine for everyone 6 months of age and older.',
                'startDate': '2014-10-23T04:00:00.000Z',
                'endDate': '2018-12-25',
                'schedule': {
                    'acceptUnscheduledReading': false,
                    'scheduleType': {
                        '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany',
                        'name': 'Custom',
                        'description': 'Every 10 minute',
                        'interval': {
                            'value': 10,
                            'unit': 'MINUTE'
                        }
                    },
                    'scheduleTime': {
                        'hour': 8,
                        'min': 30,
                        'meridian': 'AM',
                        'description': '(At 8:30 AM)'
                    },
                    'isStartTimeRelative': false,
                    'isEndTimeRelative': false
                },
                'progress': 'Delivered',
                'comments': {
                    'content': null,
                    'user': 'EIGHT VEHU',
                    'time': '12:21 PM',
                    'date': '2014-10-23',
                    'changes': []
                },
                'createdDateUTC': '2014-10-02 16:05:57'
            };
        }
    };
    breadCrumbsMockService = {
        get: function () {
            return true;
        }
    };


    mockMsg = mockService.query();

    beforeEach(inject(function ($rootScope, $controller, $injector, _$httpBackend_) {
        scope = $injector.get('$rootScope');
        $controller = $injector.get('$controller');
        dateFilter = $injector.get('$filter')('date');
        httpBackend = _$httpBackend_;
        MainCtrl = $controller('broadcastFacilityAddController', {
            $scope: scope,
            broadcastFacilityService: mockService,
            breadcrumbs: breadCrumbsMockService,
            resolvedMessage: mockService.query(),
            resolvedUser: mockService
        });
    }));

    it('Created Date Test', function () {
        var mockMsgCreateDate = mockMsg.createDate;
        expect(scope.facilityMsg.createDate).toEqual(mockMsgCreateDate);
    });

    describe('Start and End Date Calendar Controls', function () {
        it('Test Calendar Configuration with start and end dates', function () {
            var today = dateFilter(new Date(), 'MM/dd/yyyy'),
                todayEnd = dateFilter(new Date(), 'MM/dd/yyyy');

            // Here we are matching the Minimum Start Date to Today as MM/dd/yyy
            expect(dateFilter(scope.facilityMsgLocals.startMinDate)).toBe(today);
            // Here we are matching the Minimum End Date to Tomorrow as MM/dd/yyy
            expect(dateFilter(scope.facilityMsgLocals.endMinDate)).toBe(todayEnd);
        });

        it('Test changing the Start Date and update the Minimum End Date', function () {
            var futureStartDate = '04/23/2016';
            scope.updateEndMinDate(futureStartDate);
            expect(dateFilter(scope.facilityMsgLocals.endMinDate)).toBe(futureStartDate);
        });

        it('Test changing the End Date and update the Maximum Start Date', function () {
            var futureEndDate = '04/23/2014';
            scope.updateStartMaxDate(futureEndDate);
            expect(dateFilter(scope.facilityMsgLocals.startMaxDate)).toBe(futureEndDate);
        });

        it('Test opening the calendar widget by calling the clickevent function.', function () {
            var $event = jasmine.createSpyObj('$event', [ 'preventDefault', 'stopPropagation' ]);
            scope.openStartDate($event);
            expect(scope.facilityMsgLocals.openedStartDate).toBe(true);
            scope.openEndDate($event);
            expect(scope.facilityMsgLocals.openedEndDate).toBe(true);
        });
    });

    describe('Test View Existing Message Fields', function () {
        it('reads some resolved message fields in edit message context', function () {
            expect(scope.facilityMsg.name).toEqual(mockMsg.name);
            expect(scope.facilityMsg.createBy).toEqual(mockMsg.createBy);
            expect(scope.facilityMsg.message).toEqual(mockMsg.message);
            expect(scope.facilityMsg.progress).toEqual(mockMsg.progress);
            expect(scope.facilityMsg.schedule.scheduleType.name).toEqual(mockMsg.schedule.scheduleType.name);
        });
    });
});
