/*jslint node: true, unparam: true, plusplus: true */
/*global angular */

'use strict';

angular.module('broadcast.facility.crud').directive('messageSubmit', [
    '$parse',
    '$log',
    '$filter',
    'utility',
    function ($parse, $log, $filter, utility) {
        return {
            require: 'form',
            link: function (scope, element, attrs, form) {
                // INIT ALL THE VARS
                var formAction,
                    isSpecifiedDaysValid,
                    isCustomTimeValid,
                    isCustomCycleValid,
                    isStartDateValid,
                    startDateStatus,
                    isEndDateValid,
                    endDateStatus,
                    startDate,
                    endDate,
                    currentDate = new Date(),
                    regex = /^(0[1-9]|1[0-2])\/(0[1-9]|1\d|2\d|3[01])\/(19|20|21)\d{2}$/,
                    regexZeros = /^0{3}$/;
                form.attempted = false;
                currentDate.setHours(0, 0, 0, 0);

                // get a hold of the function that handles submission when form is valid
                formAction = $parse(attrs.messageSubmit);

                var isEditing = scope.$eval(attrs.isEditing);
                // -----------------------------------------------------
                // Validation Functions.
                // -----------------------------------------------------
                isSpecifiedDaysValid = function () {
                    var isValid,
                        diff,
                        days,
                        daySearch = '',
                        start,
                        end,
                        newDate,
                        k; // Counter;

                    // For calendar vs. typed in dates
                    if (angular.isDate(form.startDate.$viewValue)) {
                        start = new Date(form.startDate.$viewValue);
                    } else {
                        start = new Date(form.startDate.$modelValue);
                    }
                    if (angular.isDate(form.endDate.$viewValue)) {
                        end = new Date(form.endDate.$viewValue);
                    } else {
                        end = new Date(form.endDate.$modelValue);
                    }
                    // using validation service for this function.  Returns date difference in number of days.                    
                    diff = utility.dateDiffInDays(start, end);
                    days = diff;

                    // Cheap initial fix; checks to see if ANY checkboxes are checked.
                    if (form.mon.$viewValue === false && form.tues.$viewValue === false && form.weds.$viewValue === false && form.thurs.$viewValue === false && form.fri.$viewValue === false && form.sat.$viewValue === false && form.sun.$viewValue === false) {
                        isValid = false;
                    } else {
                        isValid = true;
                    }
                    if (days < 7) {
                        while (start <= end) {
                            daySearch = daySearch +  start.getDay() + ',';
                            newDate = start.setDate(start.getDate() + 1);

                            start = new Date(newDate);
                        }
                        // using 1 - 8 rather than 0-7 to match the daySearch object that's built.
                        // we are looping through it.  If any of the days doesn't match our 'daysearch' which is an array of days allowed in our time selection, we check to see if that is checked. if it is; tis invalid.
                        for (k = 1; k < 8; k++) {
                            if (daySearch.search(k) === -1) {
                                switch (k) {
                                case 1:
                                    if (form.mon.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 2:
                                    if (form.tues.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 3:
                                    if (form.weds.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 4:
                                    if (form.thurs.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 5:
                                    if (form.fri.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 6:
                                    if (form.sat.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                case 7:
                                    if (form.sun.$viewValue === true) {
                                        isValid = false;
                                        scope.specifiedDaysOutsideRange = true;
                                    }
                                    break;
                                default:
                                    break;
                                }
                            }
                        }
                    }
                    $log.info('isSpecifiedDaysValid isValid: ' + isValid);

                    return isValid;
                };
                isCustomTimeValid = function () {
                    var isValid;

                    if (!form.cycleTime.$viewValue || form.cycleTime.$invalid) {
                        isValid = false;
                    } else {
                        isValid = true;
                    }
                    $log.info('cycleTime isValid: ' + isValid);
                    return isValid;
                };
                isCustomCycleValid = function () {
                    var isValid;
                    if (!form.customCycleTime.$viewValue) {
                        isValid = false;
                    } else {
                        isValid = true;
                    }
                    $log.info('customCycleTime isValid: ' + isValid);
                    return isValid;
                };
                isStartDateValid = function () {
                    var isValid = false;
                    // Date Validation - Start Date Empty
                    if (!startDate || form.startDate.$viewValue === undefined || form.startDate.$viewValue === '') {
                        $log.info("startDateEmpty");
                        isValid = 'startDateEmpty';
                        return isValid;
                    }
                    // Date Validation - 3 Zeros?
                    // Checking here because we don't want it empty; throws and error.
                    if (regexZeros.test(startDate) === true) {
                    // if (form.startDate.$viewValue.indexOf('000') >= 0) {
                        $log.info('zeros');
                        $log.info("startDateFormatting");
                        isValid = 'startDateFormatting';
                        return isValid;
                    }

                    // Date Validation - Date Formatting - Regex
                    if (regex.test($filter('date')(startDate, "MM/dd/yyyy")) === false) {
                        $log.info('startDateFormatting - regex - filter');
                        $log.info(form.startDate.$viewValue);
                        isValid = 'startDateFormatting';
                        return isValid;
                    }
                    
                    // Date Validation - Start Date before Today
                    if (startDate < currentDate) {
                        $log.info("startDateEarly");
                        isValid = 'startDateEarly';
                        return isValid;
                    }
                    // Date Validation - Date After End Date
                    if (startDate > endDate && form.endDate.$viewValue !== undefined) {
                        $log.info("startDateAfter");
                        isValid = 'startDateAfter';
                        return isValid;
                    }
                    $log.info('startDate isValid: ' + isValid);
                    return isValid;
                };
                isEndDateValid = function () {
                    var isValid = false;
                    $log.info(endDate);
                    $log.info(form.endDate.$viewValue);

                    // Date Validation - End Date Empty
                    if (!endDate || form.endDate.$viewValue === undefined || form.endDate.$viewValue === '') {
                        $log.info('endDateEmpty');
                        isValid = 'endDateEmpty';
                        return isValid;
                    }
                    // Date Validation - 3 Zeros?
                    // Checking here because we don't want it empty; throws and error.
                    if (regexZeros.test(endDate) === true) {
                        // if (form.endDate.$viewValue.indexOf('000') >= 0) {
                        $log.info('zeros');
                        isValid = 'endDateFormatting';
                        return isValid;
                    }
                    // Date Validation - End Date before Start Date
                    if (endDate < startDate) {
                        $log.info('endDateEarly');
                        isValid = 'endDateEarly';
                        return isValid;
                    }

                    // Angular UI Widget inserts Date into $viewValue
                    if (regex.test($filter('date')(endDate, 'MM/dd/yyyy')) === false) {
                        $log.info('endDateFormatting - regex - filter');
                        $log.info(form.endDate.$viewValue);
                        isValid = 'endDateFormatting';
                        return isValid;
                    }
                    // Checking here that it's real dates in here, not words, like Strawberry
                    if (regex.test($filter('date')(endDate, "MM/dd/yyyy"))) {
                        // Date Validation - Date After End Date
                        if (endDate < startDate && startDate !== undefined) {
                            $log.info('regex test endate - endDateBefore');
                            isValid = 'endDateBefore';
                            return isValid;
                        }
                    }
                    $log.info('endDate isValid: ' + isValid);
                    return isValid;
                };

                // -----------------------------------------------------
                // Submit Functions.
                // -----------------------------------------------------
                // register DOM event handler and wire into Angular's lifecycle with scope.$apply
                element.on('submit', function (event) {

                    scope.$apply(function () {
                        // on submit event, set submitted to true (like the previous trick)
                        form.attempted = true;

                        // Check here for either widget or hand-typed date.
                        if (angular.isDate(form.startDate.$viewValue)) {
                            startDate = form.startDate.$viewValue;
                            $log.info("new startDate date: " + startDate);
                        } else {
                            startDate = new Date(form.startDate.$viewValue);
                            $log.info("new startDate non-date: " + startDate);
                        }
                        if (angular.isDate(form.endDate.$viewValue)) {
                            endDate = form.endDate.$viewValue;
                            $log.info("new end date: " + endDate);
                        } else {
                            endDate = new Date(form.endDate.$viewValue);
                            $log.info("new end date non-date: " + endDate);
                        }

                        // Setting these values here (for Date Validations) to prevent code duplication
                        // startDate = new Date(form.startDate.$viewValue);
                        // endDate = new Date(form.endDate.$viewValue);
                        startDate.setHours(0, 0, 0, 0);
                        endDate.setHours(0, 0, 0, 0);

                        // ----------------------------------
                        // Schedule Validation Block
                        // ----------------------------------
                        if (form.schedule.$viewValue === 'Once') {
                            // template handles this single input.
                            // setting the Java Classes here???
                            form.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce';
                            scope.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce';
                        } else if (form.schedule.$viewValue === 'Specified Days') {
                            form.mon.$setValidity('days', isSpecifiedDaysValid());
                            if (scope.specifiedDaysOutsideRange === true) {
                                $log.info('specifiedDaysOutsideRange');
                                form.scheduleErrorMessage = 'The day(s) selected must be within the Start Date and End Date range';
                            } else {
                                $log.info('!specifiedDaysOutsideRange');
                                form.scheduleErrorMessage = 'Please select Days that Apply';
                            }
                            // setting the Java Classes here???
                            form.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek';
                            scope.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek';
                        } else if (form.schedule.$viewValue === 'Custom') {
                            form.schedule.$setValidity('time', isCustomTimeValid());
                            form.schedule.$setValidity('cycle', isCustomCycleValid());
                            // setting the Java Classes here???
                            form.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany';
                            scope.scheduleClass = 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany';
                        }

                        // ----------------------------------
                        // START Date Validation Block
                        // ----------------------------------
                        startDateStatus = isStartDateValid();
                        if (startDateStatus === 'startDateEmpty') {
                            form.startDate.$setValidity('startDate', false);
                            form.startDateErrorMessage = 'Please select a Start Date';
                        } else if (startDateStatus === 'startDateEarly') {
                            form.startDate.$setValidity('startDate', false);
                            form.startDateErrorMessage = 'The Start Date CANNOT be before today\'s date';
                        } else if (startDateStatus === 'startDateFormatting') {
                            form.startDate.$setValidity('startDate', false);
                            form.startDateErrorMessage = 'Please enter a valid date format (MM/DD/YYYY)';
                        } else if (startDateStatus === 'startDateAfter') {
                            form.startDate.$setValidity('startDate', false);
                            form.startDateErrorMessage = 'The Start Date CANNOT be after the End Date';
                        } else if (startDateStatus === false) {
                            form.startDate.$setValidity('startDate', true);
                        }

                        // ----------------------------------
                        // END Date Validation Block
                        // ----------------------------------
                        endDateStatus = isEndDateValid();
                        if (endDateStatus === 'endDateEmpty') {
                            form.endDate.$setValidity('endDate', false);
                            form.endDateErrorMessage = 'Please select an End Date';
                        } else if (endDateStatus === 'endDateEarly') {
                            form.endDate.$setValidity('endDate', false);
                            form.endDateErrorMessage = 'The End Date CANNOT be before the Start Date';
                        } else if (endDateStatus === 'endDateFormatting') {
                            form.endDate.$setValidity('endDate', false);
                            form.endDateErrorMessage = 'Please enter a valid date format (MM/DD/YYYY)';
                        } else if (endDateStatus === 'endDateBefore') {
                            form.endDate.$setValidity('endDate', false);
                            form.endDateErrorMessage = 'The End Date CANNOT be before the Start Date';
                        } else  if (endDateStatus === false) {
                            form.endDate.$setValidity('endDate', true);
                            $log.info('default endDate');
                            $log.info(form.endDate.$error);
                        }


                        // ----------------------------------
                        // THE FORM, IT IS VALID
                        // ----------------------------------
                        // if form is valid, execute the submission handler function and reset form submission state
                        if (form.$valid) {
                            $log.info('FORM VALID');
                            formAction(scope, {
                                $event: event
                            });
                            form.attempted = false;
                        }
                    });
                });
            }
        };
    }
]);
