/*jslint node: true, unparam: true, nomen: true */
/*global angular, navigator, describe, beforeEach, it, inject, expect */

'use strict';

describe('Test the Facility Messages Add Validation Directive', function () {
    var scope,
        form,
        formElement,
        dateFilter,
        today;

    beforeEach(module('broadcast.facility.crud'));

    beforeEach(inject(function ($compile, $rootScope, $injector) {
        scope = $rootScope;
        dateFilter = $injector.get('$filter')('date');
        today = dateFilter(new Date(), 'MM/dd/yyyy');

        // Filling out Variables needed for form.
        scope.facilityMsgLocals = {
            'startDateOpen': false,
            'startMinDate': dateFilter(new Date(), 'MM/dd/yyyy'),
            'startMaxDate': '',
            'startDateOptions': {
                'year-format': 'yy',
                'starting-day': 1
            },
            'endDateOpen': false,
            'endMinDate': dateFilter(new Date(), 'MM/dd/yyyy'),
            'endDateOptions': {
                'year-format': 'yy',
                'starting-day': 1
            }
        };

        var element = angular.element(
            '<form name="facilityMsgForm" id="add-message-form" novalidate message-submit="addMessageSubmit()">' +
                '<input type="text" ' +
                '    class="form-control" ' +
                '    title="Please enter a title" ' +
                '    id="title"' +
                '    maxlength="40"' +
                '    name="title"' +
                '    ng-model="facilityMsg.name"' +
                '    ng-required="true" />' +
                '<textarea rows="3" ' +
                '    class="form-control" ' +
                '    title="Please enter a message" ' +
                '    id="message"' +
                '    name="message"' +
                '    maxlength="160"' +
                '    ng-model="facilityMsg.message"' +
                '    ng-required="true">' +
                '</textarea>' +
                '<select class="form-control" ' +
                '    id="schedule" ' +
                '    name="schedule" ' +
                '    ng-model="facilityMsg.schedule.scheduleType.name" ' +
                '    ng-required="true"' +
                '    title="Please select a Schedule">' +
                '    <option value=""></option>' +
                '    <option value="once">Once</option>' +
                '    <option value="specified-days">Specified Days</option>' +
                '    <option value="custom">Custom</option>' +
                '</select>' +
                '<input type="text"' +
                '    class="form-control"' +
                '    close-text="Close"' +
                '    datepicker-options="facilityMsgLocals.startDateOptions"' +
                '    datepicker-popup="MM/dd/yyyy"' +
                '    id="start-date"' +
                '    is-open="facilityMsgLocals.openedStartDate"' +
                '    init-date="facilityMsgLocals.startMinDate" ' +
                '    name="startDate" ' +
                '    ng-change="updateEndMinDate(facilityMsg.startDate)"' +
                '    min="facilityMsgLocals.startMinDate"' +
                '    max="facilityMsg.endDate"' +
                '    ng-model="facilityMsg.startDate"' +
                '    ng-required="true"' +
                '    title="Start Date" />' +
                '<input type="text" ' +
                '    class="form-control"' +
                '    close-text="Close"' +
                '    datepicker-options="facilityMsgLocals.endDateOptions" ' +
                '    datepicker-popup="MM/dd/yyyy"' +
                '    id="end-date"' +
                '    is-open="facilityMsgLocals.openedEndDate"' +
                '    min="facilityMsgLocals.endMinDate"' +
                '    name="endDate"' +
                '    title="End Date" ' +
                '    ng-change="updateStartMinDate(facilityMsg.startDate)"' +
                '    ng-required="true"' +
                '    ng-model="facilityMsg.endDate"' +
                '    title="End Date" />' +
                '<input class="btn btn-primary"' +
                '    id="save-top"' +
                '    name="submit"' +
                '    type="submit"' +
                '    value="Save" />' +
                '</form>'
        );
        scope.facilityMsg = {
            name: { 'facilityMsg.name': null },
            message: { 'facilityMsg.message': null },
            schedule: {
                scheduleType: {
                    name: { 'scope.facilityMsg.schedule.scheduleType.name': null }
                }
            },
            startDate: { 'facilityMsg.startDate': null },
            endDate: { 'facilityMsg.endDate': null }
        };

        $compile(element)(scope);
        scope.$digest();
        form = scope.facilityMsgForm;
        formElement = element.find('input#save-top');
    }));

    describe('Testing the Title Input', function () {
        it('should pass with integer or string', function () {
            form.title.$setViewValue('3');
            expect(scope.facilityMsg.name).toEqual('3');
            expect(form.title.$valid).toBe(true);

            form.title.$setViewValue('a');
            expect(scope.facilityMsg.name).toEqual('a');
            expect(form.title.$valid).toBe(true);
        });
        it('should not pass with null', function () {
            form.title.$setViewValue('');
            expect(form.title.$valid).toBe(false);
        });
    });

    describe('Testing the Message Input', function () {
        it('should pass with integer or string', function () {
            form.message.$setViewValue('3');
            expect(scope.facilityMsg.message).toEqual('3');
            expect(form.message.$valid).toBe(true);

            form.message.$setViewValue('a');
            expect(scope.facilityMsg.message).toEqual('a');
            expect(form.message.$valid).toBe(true);
        });
        it('should not pass with null', function () {
            form.message.$setViewValue('');
            expect(form.message.$valid).toBe(false);
        });
    });

    describe('Testing the Schedule Type Select', function () {
        it('should be populated', function () {
            form.schedule.$setViewValue('specified-days');
            expect(scope.facilityMsg.schedule.scheduleType.name).toBe('specified-days');
            expect(form.schedule.$valid).toBe(true);
        });
        it('should error out on null', function () {
            form.schedule.$setViewValue('');
            expect(form.schedule.$valid).toBe(false);
        });
    });

    describe('Testing the Start and End Dates', function () {
        it('should be populated', function () {
            form.startDate.$setViewValue(today);
            expect(dateFilter(scope.facilityMsg.startDate, 'MM/dd/yyyy')).toBe(today);
            expect(form.startDate.$valid).toBe(true);
        });
        it('should error out on null', function () {
            form.startDate.$setViewValue('');
            expect(form.startDate.$valid).toBe(false);
        });
        it('should error out on bad pattern', function () {
            form.startDate.$setViewValue('100/100/100');
            scope.$apply();
            expect(form.startDate.$valid).toBe(false);
        });
    });
});