/*jslint node: true, unparam: true */
/*global angular, navigator */

'use strict';

angular.module('validation').factory('utility', [
    '$log',
    function ($log) {
        return {
            isNumber: function (value) {
                var type = typeof value;
                return type === 'number' ||
                    value && type === 'object' && Object.prototype.toString.call(value) === '[object Number]' || false;
            },

            has: function (object, key) {
                return object ? Object.prototype.hasOwnProperty.call(object, key) : false;
            },

            /**
             * @param value the value
             * @returns {boolean} true if the given value is not null, not undefined, not an empty string, NaN returns false
             */
            notEmpty: function (value) {
                if (this.isNaN(value)) {
                    return false;
                }
                return angular.isDefined(value) && value !== '' && value !== null;
            },

            /**
             * @param value the value to validate
             * @returns {boolean} true if the given value is null, undefined, an empty string, NaN returns false
             */
            isEmpty: function (value) {
                if (this.isNaN(value)) {
                    return false;
                }
                return !this.notEmpty(value);
            },

            /**
             * @param fromDate the date object to validate
             * @param toDate the date object to subtract from.
             * @returns {time} Returns the days between a & b date objects...
             */
            dateDiffInDays: function (fromDate, toDate) {
                var millisecondsPerDay = 1000 * 60 * 60 * 24,
                    utcFromDate,
                    utcToDate;
                // Discard the time and time-zone information.
                utcFromDate = Date.UTC(fromDate.getFullYear(), fromDate.getMonth(), fromDate.getDate());
                utcToDate = Date.UTC(toDate.getFullYear(), toDate.getMonth(), toDate.getDate());
                return Math.floor((utcToDate - utcFromDate) / millisecondsPerDay);
            }

        };
    }
]);