
'use strict';

angular.module('MessagingAppApp', [
  'ngCookies',
  'ngResource',
  'ngRoute',
  'ui.bootstrap',
  'AuthStaff',
  'MessAppFilters',
  'MessAppDirectives',
  'highcharts-ng',
  'ui.mask.for.number',
  'angularShamSpinnerAnnie',
  'ngSanitize',
  'broadcast',
  'validation',
])
    .config(function ($routeProvider, $httpProvider, $locationProvider) {
      $routeProvider
      .when('/protocols/sendMessage', {
        templateUrl: 'views/sendMessage.tpl.html',
        controller: 'MessageSendController'
      })
      .when('/patient/list', {
        templateUrl: 'views/patientList.tpl.html',
        controller: 'PatientListCtrl',
        resolve: {
          resolvedPatients: ['$q', 'PatientListService', function ($q, PatientListService) {
            var deferred = $q.defer();
            PatientListService.query(
                function (resp) {
                  deferred.resolve(resp);
                });
            return deferred.promise;
          }]
        },
        role: 'staffView'
      })
        .when('/patient/protocolList', {
          templateUrl: 'views/patientProtocolList.tpl.html',
          controller: 'PatientProtocolListCtrl',
          role: 'staffView'
          /*resolve: {                                                        // TODO: move service call back into here
            resolvedProtocols: ['$q', '$route', 'PatientProtocolListService', function ($q, $route, PatientProtocolListService) {
                var deferred = $q.defer();
                PatientProtocolListService.query({id: $route.current.params.patientId},
                    function (resp) {
                      deferred.resolve(resp);
                    });
                return deferred.promise;
              }]
          } */
        })
        .when('/reports', {
          templateUrl: 'views/reports.tpl.html',
          controller: 'ReportsCtrl',
          role: 'staffView'
        })
        .when('/editProtocol/:protocolId/:isATemplate', {
          templateUrl: 'views/patientProtocolEdit.tpl.html',
          controller: 'PatientProtocolEditCtrl',
          resolve: {
            resolvedPatientProtocols: ['$q', '$route', 'ProtocolListService', 'ProtocolEditService', 'patientService',
              function ($q, $route, ProtocolListService, ProtocolEditService, patientService) {
                var deferred = $q.defer();
                var protocolId = $route.current.params.protocolId;
                var patientId = patientService.getCurrPatient()._id;
                if ($route.current.params.isATemplate === 'true') {
                  ProtocolListService.update({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                } else {
                  ProtocolEditService.get({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                }
                return deferred.promise;
              }]
          },
          role: 'staffView'
        })
        .when('/patient/protocolAssignList', {
          templateUrl: 'views/protocolAssignList.tpl.html',
          controller: 'ProtocolAssignCtrl',
          resolve: {
            resolvedProtocols: ['$q', 'TemplateListActiveOnlyService', function ($q, TemplateListActiveOnlyService) {
              var deferred = $q.defer();
              deferred.resolve({proto: TemplateListActiveOnlyService.query()});
              return deferred.promise;
            }]
          },
          role: 'staffView'
        })
        .when('/patient/messages', {
          templateUrl: 'views/messageHistory.tpl.html',
          controller: 'MessageHistoryCtrl',
          role: 'staffView'
        })
        .when('/auditTrail/:protoId', {
          templateUrl: 'views/staffAuditTrail.tpl.html',
          controller: 'AuditTrailCtrl',
          resolve: {
            resolvedProtocol: ['$q', '$route', 'ProtocolEditService', 'patientService',
              function ($q, $route, ProtocolEditService, patientService) {
                var deferred = $q.defer();
                var protocolId = $route.current.params.protoId;
                var patientId = patientService.getCurrPatient()._id;
                ProtocolEditService.get({protocolId: protocolId, patientId: patientId},
                  function (resp) {
                    deferred.resolve(resp);
                  });
                return deferred.promise;
              }]
          },
          role: 'staffView'
        })
        .when('/patientSearch/:name', {
          templateUrl: 'views/patientSearch.tpl.html',
          controller: 'PatientSearchCtrl',
          role: 'staffView',
          resolve: {
            resolvedSearchResults: ['$q', '$route', '$log', 'PatientSearchService', function($q, $route, $log, PatientSearchService){
              var deferred = $q.defer();
              var name = $route.current.params.name;
              var queryResult = PatientSearchService.queryPatient(name);
              if (queryResult !== null) {
                queryResult.then(function(res){
                  deferred.resolve(res);
                });
                return deferred.promise;
              } else {
                return null;
              }
            }]
          }
        })
        .when('/patientSearch', {
          templateUrl: 'views/patientSearch.tpl.html',
          controller: 'PatientSearchCtrl',
          role: 'staffView',
          resolve: {
            resolvedSearchResults: function () {
              return;
            }   // dummy function since the controller is shared by two views.
          }
        })
        .when('/patient/account/:patientid', {
          templateUrl: 'views/patientRegistration.tpl.html',
          controller: 'PatientProfileCtrl',
          role: 'staffView',
        })
        .when('/clinicalAdmin/protocols', {
          templateUrl: 'views/clinicalAdminProtocol.tpl.html',
          controller: 'TemplateListCtrl',
          resolve: {
            resolvedTemplates: ['$q', 'TemplateListService', function ($q, TemplateListService) {
              var deferred = $q.defer();
              TemplateListService.query(
                  function (resp) {
                    deferred.resolve(resp);
                  });
              return deferred.promise;
            }]
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add', {
          templateUrl: 'views/clinicalAdminAddProtocol.tpl.html',
          controller: 'AddProtocolCtrl',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/addOrCopyTemplate', {
          templateUrl: 'views/clinicalAdminAddOrCopyTemplate.tpl.html',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/addTemplate', {
          templateUrl: 'views/clinicalAdminAddTemplate.tpl.html',
          controller: 'AddProtocolCtrl',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add/vital', {
          templateUrl: 'views/clinicalAdminVitalTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: function () {
              return;
            }   // dummy function since view and new templates share a controller.
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add/motivational', {
          templateUrl: 'views/clinicalAdminMotivationalTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: function () {
              return;
            }   // dummy function since view and new templates share a controller.
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add/custom', {
          templateUrl: 'views/clinicalAdminCustomTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: function () {
              return;
            }   // dummy function since view and new templates share a controller.
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/template/view/:templateId', {
          templateUrl: 'views/clinicalAdminViewEditProtocol.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: ['$q', '$route', 'TemplateListService', function ($q, $route, TemplateListService) {
              var deferred = $q.defer();
              var templId = $route.current.params.templateId;
              TemplateListService.get({id: templId},
                  function (resp) {
                    deferred.resolve(resp);
                  });
              return deferred.promise;
            }]
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/auditTrail/:templateId', {
          templateUrl: 'views/clinicalAdminAuditTrail.tpl.html',
          controller: 'AuditTrailCtrl',
          resolve: {
            resolvedProtocol: ['$q', '$route', 'TemplateListService',
              function ($q, $route, TemplateListService) {
                var deferred = $q.defer();
                var templId = $route.current.params.templateId;
                TemplateListService.get({id: templId},
                    function (resp) {
                      deferred.resolve(resp);
                    });
                return deferred.promise;
              }]
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/reports', {
          templateUrl: 'views/clinicalAdminReports.tpl.html',
          role: 'clinicalAdmin'
        })
        .when('/vistaPatientResults/:query', {
          templateUrl: 'views/vistaPatientResults.tpl.html',
          controller: 'VistaPatientResultsCtrl',
          role: 'staffView'
        })
        .when('/patientRegistration', {
          templateUrl: 'views/patientRegistration.tpl.html',
          controller: 'PatientProfileCtrl',
          role: 'staffView'
        })
        .when('/emptyAnniePage', {
          templateUrl: 'views/emptyPage.tpl.html'
        })
        .when('/success', {
          templateUrl: 'views/registrationSuccess.tpl.html',
          role: 'staffView'
        })
        .when('/', {
          redirectTo: '/patientSearch'
        })
        .otherwise({
          redirectTo: '/patientSearch'
        });

      $httpProvider.interceptors.push('noCacheInterceptor');

      $locationProvider.html5Mode(false);

    });

// Preload templates
angular.module('MessagingAppApp').run(function($http, $templateCache, $rootScope, $location, AddProtocolFactory, connectionErrorService){

  // starting connectionErrorService
  connectionErrorService.run();
    
  // register listener to watch route changes
  $rootScope.$on('$routeChangeStart', function () {
    var path = $location.path();
    
    // clear data if the user navigates away from add/edit
    // protocol wizard
    if (path.indexOf('/clinicalAdmin/protocols/edit') === -1) {
      AddProtocolFactory.resetData();
    } else {
      // if it is not the start page of the wizard, and there is not data
      // saved in service, we need to redirect to start page
      // TODO: need to revisit for view/edit case
      if (path !== '/clinicalAdmin/protocols/edit/add') {
        var protocol = AddProtocolFactory.loadData();
        if (protocol.isNew === true) {
          $location.path('/clinicalAdmin/protocols/edit/add');
        }
      }
    }
  });
});



// Workaround for bug #1404
// https://github.com/angular/angular.js/issues/1404
// Source: http://jsfiddle.net/gstabel/Tzp7a/
// Credits: http://plnkr.co/edit/hSMzWC?p=preview
angular.module('MessagingAppApp').config([
  '$provide',
  function ($provide) {
    $provide.decorator('ngFormDirective', ['$delegate', function ($delegate) {
      var ngForm = $delegate[0], controller = ngForm.controller;
      ngForm.controller = [
        '$scope',
        '$element',
        '$attrs',
        '$injector',
        function (scope, element, attrs, $injector) {
          var $interpolate = $injector.get('$interpolate');
          attrs.$set('name', $interpolate(attrs.name || '')(scope));
          $injector.invoke(controller, this, {
            '$scope': scope,
            '$element': element,
            '$attrs': attrs
          });
        }
      ];
      return $delegate;
    }]);
    $provide.decorator('ngModelDirective', [
      '$delegate',
      function ($delegate) {
        var ngModel = $delegate[0], controller = ngModel.controller;
        ngModel.controller = [
          '$scope',
          '$element',
          '$attrs',
          '$injector',
          function (scope, element, attrs, $injector) {
            var $interpolate = $injector.get('$interpolate');
            attrs.$set('name', $interpolate(attrs.name || '')(scope));
            $injector.invoke(controller, this, {
              '$scope': scope,
              '$element': element,
              '$attrs': attrs
            });
          }
        ];
        return $delegate;
      }
    ]);
    $provide.decorator('formDirective', [
      '$delegate',
      function ($delegate) {
        var form = $delegate[0], controller = form.controller;
        form.controller = [
          '$scope',
          '$element',
          '$attrs',
          '$injector',
          function (scope, element, attrs, $injector) {
            var $interpolate = $injector.get('$interpolate');
            attrs.$set('name', $interpolate(attrs.name || attrs.ngForm || '')(scope));
            $injector.invoke(controller, this, {
              '$scope': scope,
              '$element': element,
              '$attrs': attrs
            });
          }
        ];
        return $delegate;
      }
    ]);
  }
]);