'use strict';

angular.module('MessagingAppApp')
    .controller('AddProtocolCtrl', function($scope, $log, $location, $timeout, $window, AddProtocolFactory) {

      // Const list of schedule type classes
      var templateTypeClasses = ['gov.va.mobilehealth.courier.core.protocol.Vital',
        'gov.va.mobilehealth.courier.core.protocol.Motivational',
        'gov.va.mobilehealth.courier.core.protocol.Reminder'];

      $scope.newProtocol = AddProtocolFactory.loadData();
      $scope.useVitals = false;
      $scope.useMotivations = false;
      $scope.useReminders = false;
      $scope.numVitals = 1;
      $scope.numMotivations = 1;
      $scope.numReminders = 1;
      $scope.vitalsError = false;
      $scope.motivationError = false;
      $scope.submitError = false;
      $scope.formSubmitted = false;
      $scope.quantityErrorMsg = 'Please enter a number between 1-20';

      $scope.saveDraft = function() {

        $scope.formSubmitted = true;

        if($scope.protocolForm.$valid) {
          AddProtocolFactory.saveDraft($scope.newProtocol);
          $location.path('/clinicalAdmin/protocols/edit/addOrCopyTemplate');
        }

      };
      
      // need to clear data if the user choose another template type
      $scope.selectTemplateTyps = function(type) {
        $scope.newProtocol.templateType = type;
        cleanTemplate($scope.newProtocol.templates);
        $scope.submitError = false;
        
        if (type === 'Vitals') {
          // initialize the structure
          initializeVitals(0);
          $scope.newProtocol.numVitals = 1;
          $scope.newProtocol.numReminders = 0;
          $scope.newProtocol.numMotivations = 0;
        } else if (type === 'Motivational/Educational') {
          // initialize the structure
          initializeMotivationals(0);
          $scope.newProtocol.numVitals = 0;
          $scope.newProtocol.numReminders = 0;
          $scope.newProtocol.numMotivations = 1;
        } else if (type === 'Custom') {
          if (!$scope.useVitals && !$scope.useMotivations) {
            $scope.submitError = true;
          }
          if ($scope.submitError ||
              ($scope.useVitals && $scope.vitalsError) ||
              ($scope.useMotivations && $scope.motivationError)) {
            return;
          }
          var ii;
          cleanTemplate($scope.newProtocol.templates);
          if (!$scope.useVitals) {
            $scope.numVitals = 0;
          }
          $scope.newProtocol.numVitals = $scope.numVitals;
          for (ii=0; ii<$scope.numVitals; ii++) {
            initializeVitals(ii);
          }

          if (!$scope.useMotivations) {
            $scope.numMotivations = 0;
          }
          $scope.newProtocol.numMotivations = $scope.numMotivations;
          for (ii=$scope.numVitals; ii<parseInt($scope.numVitals, 10)+parseInt($scope.numMotivations, 10); ii++){
            initializeMotivationals(ii);
          }
        }
        AddProtocolFactory.saveDraft($scope.newProtocol);
        
        // route
        if (type === 'Vitals') {
          $location.path('/clinicalAdmin/protocols/edit/add/vital');
        } else if (type === 'Motivational/Educational') {
          $location.path('/clinicalAdmin/protocols/edit/add/motivational');
        } else if (type === 'Custom') {
          $location.path('/clinicalAdmin/protocols/edit/add/custom');
        }
      };
      
      function initializeVitals(index){
        $scope.newProtocol.templates[index] = {};
        $scope.newProtocol.templates[index].service = {};
        $scope.newProtocol.templates[index].schedule = {};
        $scope.newProtocol.templates[index].acceptUnscheduledReading = true;
        $scope.newProtocol.templates[index]['@class'] = templateTypeClasses[0];
        $scope.newProtocol.templates[index].type = 'Vitals';
      }
      
      function initializeMotivationals(index){
        $scope.newProtocol.templates[index] = {};
        $scope.newProtocol.templates[index].schedule = {};
        $scope.newProtocol.templates[index]['@class'] = templateTypeClasses[1];
        $scope.newProtocol.templates[index].type = 'Motivational/Educational';
      }
      
      function cleanTemplate(){
        $scope.newProtocol.selectedMeasurement = [];
        $scope.newProtocol.templates = [];
      }
      
      $scope.checkQuantity = function(type, number) {
        var error = false;
        if (isNaN(number) || number < 1 || number > 20) {
          error = true;
        }
          
        if (type === 'Motivational/Educational') {
          $scope.motivationError = error;
        } else if (type === 'Vitals') {
          $scope.vitalsError = error;
        }
      };
      
    });
