'use strict';

angular.module('MessagingAppApp')
    .controller('MessageHistoryCtrl', function ($scope, $log, patientService, MessageHistoryService, $timeout, $filter, PatientProtocolListService) {

      $log.info('MessageHistoryCtrl fired');

      // init the filter drop downs
      $scope.statusFilter = 'All';
      $scope.protocolNameList = ['All'];
      $scope.filteredTable = [];
      $scope.dateChanged = false;
      $scope.searching = false;

      // For Angular-UI datepicker
      $scope.dtTo = null;
      $scope.dtFrom = null;

      $scope.currPatient = patientService.getCurrPatient();

      // set the table to the full dataset
      $scope.resetTable = function(){

        // if there is data to reset...
        if( $scope.patientMessageHistoryList.length > 0 ){
          // Set the initial table which can be altered by filters to the entire table since filters default to ALL
          $scope.filteredTable = angular.copy($scope.patientMessageHistoryList);

          $scope.dtTo = new Date();

          // get the first date from the message list
          var fromDate = new Date($scope.patientMessageHistoryList[0].date);
          // convert it to UTC since we parsed the date from a literal string
          $scope.dtFrom = new Date(fromDate.getUTCFullYear(), fromDate.getUTCMonth(), fromDate.getUTCDate());

          // reset the status and protocol dropdowns
          $scope.protocolFilter = $scope.protocolNameList[0];
          $scope.statusFilter = 'All';
        }
      };

      if ($scope.currPatient !== undefined && $scope.currPatient !== null ) {
        var patientMessageHistoryList = MessageHistoryService.query({patientId: $scope.currPatient._id},
          function () {
            // Keep a copy of the original list
            $scope.patientMessageHistoryList = patientMessageHistoryList;
            $scope.resetTable();
          }
        );

        // get the list of the patient's protocols for the drop down filter
        var list = PatientProtocolListService.query({id: $scope.currPatient._id},
          function(){
            for( var i = 0; i < list.length; i++ ){
              $scope.protocolNameList.push(list[i].protocol.name);
            }
            // set default protocol filter to All
            $scope.protocolFilter = $scope.protocolNameList[0];
          }
        );
      }

      // Function to adjust the table data based on the filter(s) selected
      $scope.filterMsgList = function(){

        // To display the searching div text
        $scope.searching = true;

        if( $scope.protocolFilter === 'All' && $scope.statusFilter === 'All' && $scope.dateChanged === false ){
          $scope.filteredTable = angular.copy($scope.patientMessageHistoryList);
        }
        else{
          // Since there can be so many combinations of the table we need to rebuild the table based on the 3 criteria
          // We need to make a copy and not set by reference via copy function...
          var msgListCopy = angular.copy($scope.patientMessageHistoryList);
          $scope.filteredTable = [];

          for( var i in msgListCopy ){

            var isProto = false;
            var isStatus = false;
            var isDateOk = false;

            if( $scope.statusFilter === 'All' || ($scope.TypeConverted(msgListCopy[i].type) === $scope.statusFilter) ){
              isProto = true;
            }

            if( $scope.protocolFilter === 'All' || (msgListCopy[i].serviceName === $scope.protocolFilter) ){
              isStatus = true;
            }

            // now filter by dates...
            var date = new Date(msgListCopy[i].date);
            var dateUTC = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate());

            if( dateUTC >= $scope.dtFrom && dateUTC <= $scope.dtTo ){
              isDateOk = true;
            }

            if( isProto && isStatus && isDateOk ){
              $scope.filteredTable.push(msgListCopy[i]);
            }

          }
        }

        $scope.setPage(1);
        $scope.dateChanged = false;
        $scope.searching = false;
      };

      $scope.filterDates = function(){
        $scope.dateChanged = true;
        $scope.filterMsgList();
      };

      // watch for the datepickers to close on selection and then filter the table
      $scope.$watch('[openedStart, openedEnd]', function(newValue){
        // testing for false because if these variables changed and they are false, it means a datepicker was closed
        if( newValue.indexOf(false) !== -1 ){
          $scope.filterDates();
        }
      }, true);

      // This function is only called when typing inside the datepicker input field...
      $scope.enter = function(event){
        if (event !== undefined && event.keyCode === 13){
          $scope.openedEnd = false;
          $scope.openedStart = false;
          $scope.filterDates();
        }
      };

      // Code for pagination of Message History list
      $scope.currentPage = 1;
      $scope.itemsPerPage = 20;
      $scope.ascending = true;

      $scope.setPage = function (pageNo) {
        $scope.currentPage = pageNo;
      };

      $scope.TypeConverted = function (type) {
        if (type === 'In') {
          return 'Received';
        }
        else {
          return 'Sent';
        }
      };

      // for calendars
      $scope.openStart = function ($event) {
        $timeout(function () {
          $event.preventDefault();
          $event.stopPropagation();
          $scope.openedStart = true;
        });
      };

      $scope.openEnd = function ($event) {
        $timeout(function () {
          $event.preventDefault();
          $event.stopPropagation();
          $scope.openedEnd = true;
        });
      };

      $scope.dateOptions = {
        'year-format': 'yy',
        'starting-day': 1
      };

      $scope.format = 'MM/dd/yyyy';
    });
