'use strict';

angular.module('MessagingAppApp')
    .controller('PatientProtocolEditCtrl', function ($scope, $routeParams, $location, $log, ProtocolListService, $rootScope,
                resolvedPatientProtocols, PatientProtocolListService, patientService, CommentService, $timeout, $filter, $modal,
                ProtocolValidationService, $window) {

      $scope.patientId = patientService.getCurrPatient()._id;
      $scope.currPatient = patientService.getCurrPatient();
      $scope.protocol = {};
      $scope.numParams = [];

      // This template variable is needed to fill in the template's description fields.
      $scope.startDate = [];
      $scope.endDate = [];
      $scope.openedEnd = [];
      $scope.openedStart = [];
      $scope.scheduleMin = [];

      // Flag for form validation
      $scope.formStatus = {};
      $scope.formSubmitted = false;
      $scope.readingErrorMsg= [];
      
      $scope.isAPatientProtocol = true; // a 'flag' used for shared html files
      // This varaible let's us know if we are assigning a new template to a patient.
      $scope.isATemplate = $routeParams.isATemplate;

      // Error message constant
      //var DUP_NAME = 'A patient protocol with that name already exists for this patient. Please update the Protocol Name.';

      // set defaults for accordion for each template 
      var initializeCollapsedDefault = function(template) {
        // Defaults for the accordion
        template.isScheduleCollapsed = true;
        template.isReadingsCollapsed = true;
        template.isMessageCollapsed = true;
        template.useReadingsNgErrorFlag = false;
        template.useMessageNgErrorFlag = false;
        template.useScheduleNgErrorFlag = false;
        template.useNgErrorFlag = false;
      };

      // remove all the accordion fields that don't need to go to the back end
      var removeAccordionFields = function(template){
        delete template.isScheduleCollapsed;
        delete template.isReadingsCollapsed;
        delete template.isMessageCollapsed;
        delete template.useReadingsNgErrorFlag;
        delete template.useMessageNgErrorFlag;
        delete template.useScheduleNgErrorFlag;
        delete template.useNgErrorFlag;
      };

      // Const list of schedule type classes
      var scheduleTypeClasses = ['gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce',
        'gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek',
        'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany'];

      // Set error state when updating protocol
      $scope.setUpdateError = function (status, msg) {
        $scope.updateError = status;
        $scope.errorMsg = msg;
      };

      $scope.errorMessage = function(index)
      {
        if($scope.protocolForm['scheduleForm' + index.toString()].option.$valid ){
          $scope.formSubmitted = false;
        }
      };

      $scope.updateProtocol = function(protocol){
        var i;
        $scope.formSubmitted = true;

        // set the dates to the templates so we can validate in the service
        for (i = 0; i < protocol.templates.length; i++) {
          $scope.protocol.protocol.templates[i].startDate = $filter('date')($scope.startDate[i], 'yyyy-MM-dd');
          $scope.protocol.protocol.templates[i].endDate = $filter('date')($scope.endDate[i], 'yyyy-MM-dd');
        }

        $scope.formStatus = ProtocolValidationService.validateForm(protocol);
        for (i = 0; i < protocol.templates.length; i++) {
          $scope.templates[i].isReadingsCollapsed = $scope.formStatus.errors.templates[i].isReadingsOk;
          $scope.templates[i].isScheduleCollapsed = $scope.formStatus.errors.templates[i].isScheduleOk;
          $scope.templates[i].isMessageCollapsed = true;
          $scope.templates[i].useReadingsNgErrorFlag = true;
          $scope.templates[i].useMessageNgErrorFlag = true;
          $scope.templates[i].useScheduleNgErrorFlag = true;
          $scope.templates[i].useNgErrorFlag = true;

          // Set the error on the messages area, but this makes sure new messages don't get the error automatically
          if ($scope.templates[i].type === 'Motivational/Educational') {
            $scope.templates[i].totalMessagesSubmitted = protocol.templates[i].serviceMessage.length;
          }
        }

        if ($scope.formStatus.ok === false || $scope.protocolForm.$invalid) {
          // force page to scroll to top when there is an error, to display the error message
          $window.scrollTo(0, 0);
        }

        if ($scope.protocolForm.$valid && $scope.formStatus.ok === true) {

          for (i = 0; i < $scope.protocol.protocol.templates.length; i++) {
            var selectedScheduleTypeIndex = $scope.scheduleAllOptions.indexOf($scope.protocol.protocol.templates[i].schedule.scheduleType.name);
            var scheduleClass = scheduleTypeClasses[selectedScheduleTypeIndex];

            $scope.protocol.protocol.templates[i].schedule.scheduleType['@class'] = scheduleClass;
            $scope.protocol.protocol.templates[i].schedule.scheduleTime.min = parseInt($scope.scheduleMin[i],10);

            if ($scope.protocol.protocol.templates[i].schedule.scheduleType.name !== $scope.scheduleAllOptions[2])
            {
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.interval;
            }

            // Remove the flag which saved the number of submitted messages so only the correct number of errors are displayed.
            if ($scope.templates[i].type === 'Motivational/Educational') {
              delete $scope.templates[i].totalMessagesSubmitted;
            }

            removeAccordionFields($scope.templates[i]);

            if ($scope.protocol.protocol.templates[i].schedule.scheduleType.name !== $scope.scheduleAllOptions[1])
            {
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.monday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.tuesday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.wednesday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.thursday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.friday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.saturday;
              delete $scope.protocol.protocol.templates[i].schedule.scheduleType.sunday;
            }
          }

          if( $scope.isATemplate === 'true' ) {
            $scope.protocol.assignedDate = new Date();
            $scope.protocol.lastEditedDate = new Date();
            $scope.protocol.status = 'Active';
          }else{
            $scope.protocol.lastEditedDate = new Date();
          }

          var patientComment = CommentService.getComment();
          if(patientComment !== null){
            var cmt = {};
            cmt.content = patientComment;
            cmt.time = $filter('date')(new Date(), 'yyyy-MM-dd');
            cmt.user = '';
            var comments = $scope.protocol.comments;
            comments.push(cmt);
          }
          if($scope.protocol.status === 'Removed' || $scope.protocol.status === 'Expired'){
            $scope.protocol.status = 'Active';
          }

          $scope.updateResponse = PatientProtocolListService.update({id: $scope.patientId}, $scope.protocol,
            function(){
              $log.info('got to success');
              // success
              patientService.clearProtocolStatus();
              CommentService.clearComment();

              // Open the modal window confirmation
              $scope.open();
            },
            function(resp){
              patientService.clearProtocolStatus();
              $log.info('PatientProtocolEditCtrl editing protocol failed.');

              $location.path('/patient/protocolList');

              var errorMsg = resp.data.message;
              patientService.setStatusError(true);

              if( $scope.isATemplate === 'true' ) {
                patientService.setProtocolStatus('Error assigning \"' + $scope.protocol.protocol.name + '\". ' + errorMsg);
              }
              else{
                patientService.setProtocolStatus('Error editing \"' + $scope.protocol.protocol.name + '\". ' + errorMsg);
              }
              $scope.setUpdateError(true, errorMsg);
              $log.info('PatientProtocolEditCtrl editing protocol failed.' + $scope.updateError);
            });
        }
      };

      // Cancel out of the edit page and return to the template list
      $scope.cancelEdit = function () {
        patientService.clearProtocolStatus();
        CommentService.clearComment();
        $location.path('/patient/protocolList');
      };

      $scope.setStatus = function (status) {
        $scope.status = status;
      };

      $scope.setTemplateScope = function (protocol) {

        $log.info('Setting template scope in PatientProtocolEditCtrl: ');
        $log.info(protocol);

        // TODO: do we want to just create an array based on the size of the templates instead? this is for ng-repeat.
        $scope.templates = protocol.protocol.templates;

        $scope.protocol = protocol;
        $scope.Id = protocol._id;

        // Save the original name so we can display it while editing the protocol
        $scope.origName = protocol.protocol.name;

        //$log.info('PatientProtocolEditCtrl Retrieved protocol = '); $log.info(protocol.protocol);

        $scope.scheduleAllOptions = ['Once', 'Specified Days', 'Custom'];

        $scope.cycleAllOptions = ['MINUTE', 'HOUR', 'DAY', 'WEEK', 'MONTH', 'YEAR' ];

        $scope.hoursList = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 ];

        $scope.minutesList = [ '00', '15', '30', '45' ];

        $scope.meridianTypeslist = [ 'AM', 'PM' ];

        $scope.timeUnitslist = [ 'MINUTE', 'HOUR', 'DAY', 'WEEK', 'MONTH', 'YEAR' ];

        var i;
        for (i = 0; i < $scope.protocol.protocol.templates.length; i++) {

          initializeCollapsedDefault($scope.protocol.protocol.templates[i]);

          if ($scope.numParams[i] === undefined) {
            $scope.numParams[i] = [1];
          }
          if (protocol.protocol.templates[i].type === 'Vitals') {
            // If the number of params is greater than the default then create the array based on the size
            if (protocol.protocol.templates[i].service.readings.length > $scope.numParams[i].length) {
              var idx = $scope.numParams[i].length + 1;
              while ($scope.numParams[i].push(idx++) < protocol.protocol.templates[i].service.readings.length) {
              }
            }
          }

          // Set dropdown to default if not already set
          $scope.scheduleMin[i] = $scope.protocol.protocol.templates[i].schedule.scheduleTime.min.toString();
          if ($scope.scheduleMin[i].length === 1) {
            $scope.scheduleMin[i] = '0' + $scope.scheduleMin[i];
          }
        }

        // Creates a range of numbers for the UI to use
        $scope.range = function (min, max, step) {
          step = (!step) ? 1 : step;
          var input = [];
          for (var i = min; i <= max; i += step) {
            input.push(i);
          }
          return input;
        };

        var parts;
        for (i = 0; i < $scope.protocol.protocol.templates.length; i++) {
          // Set the start and end dates inside the calendar widget
          // TODO: do we still need isATemplate?????     yes but not here...

          // TODO: waiting until timezone is implemented
          parts = $scope.protocol.protocol.templates[i].startDate.split('-');
          $scope.startDate[i] = new Date(parts[0], parts[1] - 1, parts[2]); // Note: months are 0-based
          parts = $scope.protocol.protocol.templates[i].endDate.split('-');
          $scope.endDate[i] = new Date(parts[0], parts[1] - 1, parts[2]); // Note: months are 0-based

        }
      };

      $scope.setTemplateScope(resolvedPatientProtocols);

      // Append another message for motivational template
      $scope.addMessage = function (index) {
        $scope.protocol.protocol.templates[index].serviceMessage.push('');
      };

      // Append message for motivational template
      $scope.removeMessage = function (index) {
        var len = $scope.protocol.protocol.templates[index].serviceMessage.length;
        $scope.protocol.protocol.templates[index].serviceMessage.splice(len - 1, 1);
        $scope.templates[index].totalMessagesSubmitted--;
      };

      // For Angular-UI datepicker
      $scope.openStart = function (idx, $event) {
        $timeout(function () {
          $event.preventDefault();
          $event.stopPropagation();
          $scope.openedStart[idx] = true;
        });
      };

      $scope.openEnd = function (idx, $event) {
        $timeout(function () {
          $event.preventDefault();
          $event.stopPropagation();
          $scope.openedEnd[idx] = true;
        });
      };

      $scope.dateOptions = {
        'year-format': 'yy',
        'starting-day': 1
      };

      // Default update error to false
      $scope.setUpdateError(false, '');

      // For save/cancel modal
      $scope.open = function () {

        var modalInstance = $modal.open({
          templateUrl: 'modalSaveTopContent',
          controller: 'saveModalCtrl'
        });

        modalInstance.result.then(function () {
        });

      };

      $scope.cancel = function () {

        var modalInstance = $modal.open({
          templateUrl: 'modalCancelTopContent',
          controller: 'saveModalCtrl'
        });

        modalInstance.result.then(function () {
        });

      };

      $scope.openRemoveMessageModal = function (index) {
        var modalInstanceRemove = $modal.open({
          templateUrl: 'modalRemoveParamContent'
        });
        
        modalInstanceRemove.result.then(function () {
          $scope.removeMessage(index);
        }, function () {
          $log.info('Modal dismissed');
        });
      };

      $scope.isOnlySpaces = function( str ){
        return ProtocolValidationService.testOnlySpaces(str);
      };

    });

angular.module('MessagingAppApp')
  .controller('saveModalCtrl', function ($scope, $modalInstance, $location, $rootScope) {

    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
      $location.path('/patient/protocolList');
      $rootScope.$broadcast('protocolSet');
    };

  });



