'use strict';

angular.module('MessagingAppApp')
    .controller('PatientProtocolListCtrl', function ($scope, $log, $routeParams, PatientProtocolListService, $modal, $http,
                /*resolvedProtocols,*/ patientService, PatientProtocolDeleteService, CommentService, $location, $route, $timeout, $filter, $rootScope, patientRegistrationService) {

    $log.info('PatientProtocolListCtrl fired.');

    $scope.protocolStatus = patientService.getProtocolStatus();
    $scope.isErrorStatus = patientService.getIsErrorStatus();
    $scope.accountMessages = patientService.getProtocolStatus();
    $scope.disableStatus = false;
    $scope.initCallComplete = false;
    $scope.newProtocolText = 'Protocol Assigned';
    $scope.assignProtocolEnabled = true;
    $scope.userConsented = false;

    // set the flag that capture from the parent scope
    $rootScope.$on('errorDuplicateMessage', function () {
      $rootScope.errorDuplicateMessage = true;
    });

    $log.info ('patientProtocolListController.js. protocol status = ' + $scope.protocolStatus);

    $rootScope.$on('cancelClicked', function(){
	    $log.info('cancelClicked on function called!');
	    $route.reload();
    });
    
    $scope.getProtocolId = function(proto){
      return proto._id;
    };

    $scope.assignProtocol = function(){
      $location.path('/patient/protocolAssignList');
    };

    $scope.currPatient = patientService.getCurrPatient();

    // This function is for start and stopped the protocols.
    if ($scope.currPatient !== undefined && $scope.currPatient !== null ) {
      var userProfile = patientRegistrationService.get({id: $scope.currPatient._id},
        function () {
            $scope.currPatientStoppedMessages = userProfile;
            if (userProfile.consent === 3) {
              $scope.userConsented = true;
            }
            if (userProfile.preference.messageStarted === false || userProfile.preference.messageStopped === true || userProfile.preference.messagePaused === true) {
              $scope.assignProtocolEnabled = false;
            }
          },
      function () {
        $scope.currPatientStoppedMessages = null;
      });
    }



    $scope.getProtocols = function(){     // TODO: make this NOT a scope variable

      $scope.currPatient = patientService.getCurrPatient();

      $scope.patientProtocolList = PatientProtocolListService.query({id: $scope.currPatient._id},
      function () {
        var activeList = [];
        var inactiveList = [];
        for (var p in $scope.patientProtocolList) {
          var patientProtocol = $scope.patientProtocolList[p];
          if (patientProtocol.status === 'Active' ||patientProtocol.status === 'Resumed' ) {
            activeList.push(patientProtocol);
          }
          else if (patientProtocol.status === 'Removed' || patientProtocol.status === 'Expired' || patientProtocol.status === 'Paused' ) {
            inactiveList.push(patientProtocol);
          }
        }
        $scope.activePatientProtocolList = activeList;
        $scope.inactivePatientProtocolList = inactiveList;
        $scope.initCallComplete = true;
      },
      function () {
        $scope.initCallComplete = true;
      });
    };

    if( patientService.getCurrPatient() ){
      $scope.getProtocols();
    }

    $rootScope.$on('protocolSet', function () {
      $scope.getProtocols();
    });

    // TODO: rootscope.on ??
    $scope.$on('patientSelected', function () {
      $scope.getProtocols();
    });

    // Timer to remove the status message after deleting/assigning/editing a protocol
    var timer = false;
    var delay = 0000;
    $scope.$watch('StatusModel', function () {
      if (timer) {
        $timeout.cancel(timer);
      }
      timer = $timeout(function () {
        $scope.clearProtocolResult();
      }, delay);
    });

    // If the user navigates away from the page, clear the results
    $scope.$on('$destroy', function () {
      $scope.clearProtocolResult();
    });

    $scope.$on('updateItems', function (e, protocol) {
      $scope.activePatientProtocolList.splice($scope.activePatientProtocolList.indexOf(protocol), 1);
      $scope.inactivePatientProtocolList.push(protocol);
      $scope.disableStatus = 'The Patient Protocol ' + protocol.protocol.name + ' is now inactive. The patient will no longer receive or send messages related to this protocol.';
    });

    $scope.clearProtocolResult = function () {
      $scope.protocolStatus = '';
      $scope.disableStatus = '';
      $scope.accountMessages = '';
      patientService.clearProtocolStatus();
      patientService.setStatusError(false);
    };

    $scope.isHidden = false;

    $scope.saveEnableComment = function (comment, protocol) {
      var cmt = {};
      cmt.content = comment;
      cmt.time = new Date();
      cmt.user = '';
      CommentService.setComment(comment);
      $location.path('/editProtocol/' + protocol._id + '/false');
      //$location.path('/patient/patientProtocolEdit/' + protocol._id + '/false');
    };

    $scope.getLastComment = function (comments) {
      var i;
      for (i=comments.length-1; i>0; i-- ) {
        if (comments[i].content!==null && comments[i].content!=='') {
          return comments[i];
        }
      }
      return null;
    };

    $scope.disableProtocol = function (comment, protocol) {

      protocol.status = 'Removed';
      var len = protocol.comments.length;
      protocol.comments[len] = {};
      protocol.comments[len].content = comment;
      PatientProtocolListService.update({id: $scope.currPatient._id}, protocol, function () {
        var result = PatientProtocolDeleteService.query({patientId: $scope.currPatient._id, protocolId: protocol._id},
          function () {
            protocol.lastEditedDate = result.lastEditedDate;
            protocol.comments = result.comments;
            $rootScope.$broadcast('updateItems', protocol);
          });
      },
      function (resp) {
        var DUP_NAME;   // TODO: what is this????
        var errorMsg = resp.status === 409 ? DUP_NAME : '';
        $log.info('PatientProtocolListCtrl failed deleting protocol.');
        $scope.disableStatus = 'Error deleting ' + protocol.protocol.name + '". ' + errorMsg;
      });
    };

    $scope.open = function (flag, protocol) {
      var modalInstance;
      if (flag === true) {
        modalInstance = $modal.open({
          templateUrl: 'myModalContent.html',
          controller: 'ModalDisableCtrl',
          resolve: {
            disableComment: function () {
              return $scope.disableComment;
            }
          }
        });
      } else {
        modalInstance = $modal.open({
          templateUrl: 'myModalContentEnable.html',
          controller: 'ModalEnableCtrl',
          resolve: {
            disableComment: function () {
              return $scope.disableComment;
            }
          }
        });
      }

      modalInstance.result.then(function (comment) {
        $scope.disableComment = comment;

        // flag true means disable
        if (flag === true) {
          $scope.disableProtocol($scope.disableComment, protocol);
        } else {
          $scope.saveEnableComment(comment, protocol);
        }
      }, function () {
        $log.info('Modal dismissed at: ' + new Date());
      });
    };
  });

// TODO: move to filter file
angular.module('MessagingAppApp').filter('commentFilter', function(FieldLabelsService) {
  return function(input){

    var filteredComments = '';
    var labels = FieldLabelsService.getLabels();

    if( input !== null && typeof input !== 'undefined'){

      var fieldName = input.fieldName;
      var fieldText = '';
      for (var j = 0; j < labels.length; j++){
        if (labels[j].fieldName === fieldName) {
          fieldText = labels[j].fieldText;
          break;
        }
      }
      // Test that there actually was an old value to display in the string...
      if( input.oldValue === null  || input.oldValue === '' ){
        filteredComments = 'Field: ' + fieldText + ' set to ' + input.newValue + '.  ';
      }
      else{
        filteredComments = 'Field: ' + fieldText + ' changed from ' + input.oldValue + ' to ' + input.newValue + '  ';
      }
    }
    return filteredComments;
  };
});



//may need to remove one of the modal controller since they are basically doing the same thing
angular.module('MessagingAppApp').controller('ModalEnableCtrl', function ($scope, $modalInstance, $rootScope) {
  $scope.comment = {};
  $scope.submitted = false;
  $scope.ok = function () {
    $scope.submitted = true;
    if ($scope.comment.content !== null && $scope.comment.content.length > 0) {
      $modalInstance.close($scope.comment.content);
    }
  };

  $scope.cancel = function () {
    $modalInstance.dismiss('cancel');
    $rootScope.$broadcast('cancelClicked');
  };
});

angular.module('MessagingAppApp').controller('ModalDisableCtrl', function ($scope, $modalInstance, $rootScope) {
  $scope.comment = {};
  $scope.submitted = false;
  $scope.ok = function () {
    $scope.submitted = true;
    if ($scope.comment.content !== null && $scope.comment.content.length > 0) {
      $modalInstance.close($scope.comment.content);
    }
  };

  $scope.cancel = function () {
    $modalInstance.dismiss('cancel');
    $rootScope.$broadcast('cancelClicked');
  };
  
});


