'use strict';

angular.module('MessagingAppApp').controller('PatientSearchCtrl', function ($scope, $log, $location, $http, $rootScope, PatientSearchService, patientService,
               orderByFilter, resolvedSearchResults, $routeParams) {

  // Init
  $scope.sortedPatients = null;
  $scope.patientNotFound = false;
  $scope.searchTab = true;
  $scope.protocolTab = false;
  $scope.patientList = null;                       // TODO : refactor
  $scope.currPatient = patientService.getCurrPatient();
  $scope.buttonDisabled = false;
  // For the loading icon/text
  $scope.initCallComplete = true;
  $scope.patientSearchText = '';
  $scope.registrationText = '';

  $log.info('PatientSearchCtrl. The current patient = ');
  $log.info($scope.currPatient);

  // Generic function for button to go to locations, can't use href on buttons.
  $scope.go = function (path) {
    $location.path(path);
  };

  // Test if we just assigned or edited a protocol, so show the protocol tab
  if( $scope.currPatient ){
    $scope.protocolTab = true;
  }

  // Process the patient search results from the resolve in the route or from the service
  var processResults = function(queryResult){
    if (queryResult !== null) {
      if( queryResult.length === 0 || queryResult === false ){
        setNotFound();
      }
      else{
        $scope.patientList = queryResult;
        $scope.sortedPatients = queryResult;
        $scope.sortedPatients = orderByFilter($scope.patientList, ['+lastName', '+firstName']);

        //console.log("the sorted patients = "); console.log($scope.sortedPatients);
      }
    }
    else {
      $scope.patientList = null;
      $log.info('PatientSearchCtrl, Error getting patient list from query: ');
    }
    $scope.buttonDisabled = false;
    $scope.initCallComplete = true;

  };

  // init some scope variables for the patient query
  var prepareResults = function(){
    // For the loading icon/text
    $scope.initCallComplete = false;

    // So go can't be clicked over and over
    if ($scope.buttonDisabled === true) {
      return;
    }
    $scope.buttonDisabled = true;

    // Clear the results
    $scope.patientNotFound = false;
    $scope.sortedPatients = [];
    $scope.patientList = [];         // TODO : ??? do we need this extra list? refactor..
  };

  // search function, called from the view
  $scope.search = function(query){

    // we need to clear the top search text so we can register the correct query
    $scope.$parent.patientSearchTextTop = '';

    // first fail on invalid query: with both numbers and characters
    if( query === ''|| query === null || typeof query === 'undefined'){
      setNotFound();
      return false;
    }

    // search for patient in the database
    var queryResult = PatientSearchService.queryPatient(query);
    if (queryResult !== null) {
      prepareResults();
      queryResult.then(function(res){
        processResults(res);
      });
    } else {
      setNotFound();
      return false;
    }

  };

  function setNotFound(){
    // if not found give the user the option to register
    if( $scope.patientSearchText !== '' ){
      $scope.registrationText = $scope.patientSearchText;
    }
    else{
      $scope.registrationText = $routeParams.name;
    }

    $scope.patientNotFound = true;
    $scope.buttonDisabled = false;
    $scope.initCallComplete = true;
    return;
  }

  // if the search was performed from the global search, just process the results
  if( resolvedSearchResults !== null && typeof resolvedSearchResults !== 'undefined'){
    // init some scope variables
    prepareResults();
    // process the routeProvider's resolve
    processResults(resolvedSearchResults);
  } else if (resolvedSearchResults === null){
    setNotFound();
  }

  $rootScope.$on('patientSearched', function(event, input){
    $log.info('patientSearched on function called!');
    $log.info(input);
    $scope.search(input);
  });

  $scope.setCurrPatient = function (patient) {
    $log.info('setCurrPatient function fired!! ' + patient._id);
    $scope.protocolTab = true;
    patientService.setCurrPatient(patient);
  };

  // Clear the status messages from the patient's pages
  patientService.clearProtocolStatus();

  // set the default sorting style, for the icon's direction
  $scope.ascending = true;

  // Code for pagination of patient list
  $scope.currentPage = 1;
  $scope.itemsPerPage = 20; // magic number for now. per CMS-303 12/13

  $scope.setPage = function (pageNo) {
    $scope.currentPage = pageNo;
  };

  $scope.getGenderShort = function (gender) {
    return patientService.getGenderShort(gender);
  };

});
