'use strict';

angular.module('MessagingAppApp')
    .controller('ProtocolTemplateCtrl', function ($scope, $routeParams, $location, $log, $http, MeasurementDictionaryService,
                NewProtocolTemplateService, patientService, resolvedTemplate, $route, AddProtocolFactory,
                $modal, CommentService, $filter, ProtocolValidationService, $window) {

      $scope.newProtocol = AddProtocolFactory.loadData();
      $scope.availMeasurements = [];
      $scope.templates = [];
      $scope.selectedMinute = [];
      $scope.selectedMeasurement = [];
      $scope.hideDropdown = [];
      $scope.numParams = [];
      $scope.protocol = {};
      $scope.protocol.templates = [];
      $scope.protocolId = $routeParams.templateId;
      // Flag for form validation
      $scope.formStatus = {};
      $scope.formSubmitted = false;
      $scope.readingErrorMsg= [];

      // Const list of schedule type classes
      var scheduleTypeClasses = ['gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce',
        'gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek',
        'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany'];

      if($scope.newProtocol.templateType !== 'Custom') {
        $scope.templateIndex=0;
      }
      
      // set defaults for accordion for each template 
      var initializeCollapsedDefault = function(template) {
        // Defaults for the accordion
        template.isScheduleCollapsed = true;
        template.isReadingsCollapsed = true;
        template.isMessageCollapsed = true;
        template.useReadingsNgErrorFlag = false;
        template.useMessageNgErrorFlag = false;
        template.useScheduleNgErrorFlag = false;
        template.useNgErrorFlag = false;
      };

      // remove all the accordion fields that don't need to go to the back end
      var removeAccordionFields = function(template){
        delete template.isScheduleCollapsed;
        delete template.isReadingsCollapsed;
        delete template.isMessageCollapsed;
        delete template.useReadingsNgErrorFlag;
        delete template.useMessageNgErrorFlag;
        delete template.useScheduleNgErrorFlag;
        delete template.useNgErrorFlag;
      };

      // default initialization
      $scope.templates[0] = {};
      initializeCollapsedDefault($scope.templates[0]);    // TODO:   delete me

      // Classes: TypeOnce,  TypeEverySoMany
      $scope.scheduleAllOptions = ['Once', 'Specified Days', 'Custom'];

      $scope.hoursList = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 ];
      $scope.minutesList = [ '00', '01', '02', '03', '04', '05', '06', '07', '08', '09',
                             '10', '11', '12', '13', '14', '15', '16', '17', '18', '19',
                             '20', '21', '22', '23', '24', '25', '26', '27', '28', '29',
                             '30', '31', '32', '33', '34', '35', '36', '37', '38', '39',
                             '40', '41', '42', '43', '44', '45', '46', '47', '48', '49',
                             '50', '51', '52', '53', '54', '55', '56', '57', '58', '59'];
      $scope.meridianTypeslist = [ 'AM', 'PM' ];
      $scope.timeUnitslist = [{ 'value': 'MINUTE', 'text': 'Minute' },
                              { 'value': 'HOUR', 'text': 'Hour' },
                              { 'value': 'DAY', 'text': 'Day' },
                              { 'value': 'WEEK', 'text': 'Week' },
                              { 'value': 'MONTH', 'text': 'Month' },
                              { 'value': 'YEAR', 'text': 'Year' }];
      //TODO: need to get rid of this later
      $scope.cycleAllOptions = $scope.timeUnitslist;

      // Get the measurements from the dictionary
      var i=0,
        response = MeasurementDictionaryService.get(
          function () {
            $scope.dictionary = response;

            // Test if this is an existing template.
            if ($scope.protocolId !== null && $scope.protocolId !== undefined) {
              $scope.protocol = resolvedTemplate;
              $scope.templates = resolvedTemplate.templates;
              $scope.setTemplatesData(resolvedTemplate.templates);

              for (i = 0; i < resolvedTemplate.templates.length; i++) {
                setTemplateSchedule(i);
                // Don't show the select dropdown since we can't change the measurement
                $scope.hideDropdown[i] = true;
              }
            } else {
              // Set of available measurements the user can choose from in the select dropdown.
              $scope.availMeasurements = $scope.dictionary.services;

              // add a field to enable/disable the vital in the dropdown, since it cannot be chosen more than once
              for( i = 0; i < $scope.availMeasurements.length; i++ ){
                $scope.availMeasurements[i].selectable = true;
              }

              // Else, we are creating a new protocol template
              // get saved data from service
              if ($scope.newProtocol.templates.length > 0) {
                // initialize. Temp solution for the page to match the new
                // protocol structure. Need to revisit when we refactor the template page.
                $scope.templates = $scope.newProtocol.templates;
                $scope.setTemplatesData($scope.newProtocol.templates);
                $scope.selectedMeasurement = $scope.newProtocol.selectedMeasurement;
                for ( i = 0; i < $scope.newProtocol.templates.length; i++) {
                  setTemplateSchedule(i);
                  // Don't show the select dropdown since we can't change the measurement
                  if ($scope.newProtocol.templates[i].type === 'Vitals') {
                    $scope.hideDropdown[i] = false;
                    if (($scope.newProtocol.selectedMeasurement[i] !== null) && ($scope.newProtocol.selectedMeasurement[i] !== undefined)) {
                      $scope.setMeasurement(i);
                    }
                  }
                }
              } else {
                // Default number of reading params is 1. This has to be an array so we can loop through it in ng-repeat
                $scope.numParams[0] = [1];
                // initialize. Temp solution for the page to match the new
                // protocol structure. Need to revisit when we refactor the template page.
                $scope.protocol.templates[0] = {};
                setTemplateSchedule(0);
                $scope.templates[0].schedule.acceptUnscheduledReading = true;
                // Show the select dropdown since we  change the measurement
                $scope.hideDropdown[0] = false;
                $scope.selectedMeasurement[0] = null;
              }
            }
          }
      );

      $scope.setTemplatesData = function (templates) {
        // There will only be one measurement per template, so get it from index 0.
        // need to do a loop for all templates
        for (var i = 0; i < $scope.templates.length; i++) {
          $scope.templates[i] = templates[i];
          if ($scope.protocol.templates[i] === undefined) {
            $scope.protocol.templates[i] = {};
          }
          initializeCollapsedDefault($scope.templates[i]);

          if (templates[i].type === 'Vitals') {
            $scope.selectedMeasurement[i] = null;
            $scope.templates[i].service = templates[i].service;

            if($scope.templates[i].schedule.acceptUnscheduledReading === undefined){
              $scope.templates[i].schedule.acceptUnscheduledReading = true;
            }

            // Default number of reading params is 2. This has to be an array so we can loop through it in ng-repeat
            $scope.numParams[i] = [1];
            // If the number of params is greater than the default then create the array based on the size
            if ($scope.templates[i].service !== undefined &&
                $scope.templates[i].service.readings !== undefined &&
                $scope.templates[i].service.readings.length !== $scope.numParams[i].length) {
              setNumParams(i);
            }
          } else if (templates[i].type === 'Motivational/Educational') {
            if (templates[i].serviceMessage === undefined) {
              templates[i].serviceMessage = [];
              templates[i].serviceMessage.push('');
            }
            $scope.protocol.templates[i].serviceMessage = templates[i].serviceMessage;
          }

          $scope.templates[i].schedule = templates[i].schedule;
        }
      };

      // Input: the measurement's number of parameters
      function setNumParams(idx) {
        var i = 1;
        $scope.numParams[idx] = [];
        while ($scope.numParams[idx].push(i++) < $scope.templates[idx].service.readings.length) {
        }
      }

      // Append another message for motivational template
      $scope.addMessage = function (index) {
        // test form and reset the form submitted flag if it is OK, so it doesn't show up on a new empty message field
        if($scope.protocolForm['motivationalForm' + index.toString()].$valid ){
          $scope.formSubmitted = false;
        }
        $scope.protocol.templates[index].serviceMessage.push('');
      };

      // Append message for motivational template
      $scope.removeMessage = function (index) {
        var len = $scope.protocol.templates[index].serviceMessage.length;
        $scope.protocol.templates[index].serviceMessage.splice(len - 1, 1);
        $scope.templates[index].totalMessagesSubmitted--;
      };

      function setTemplateSchedule(idx) {
        // Set dropdown to default if not already set
        if (($scope.protocolId !== null && $scope.protocolId !== undefined) ||
            ($scope.templates[idx].schedule !== undefined &&
                $scope.templates[idx].schedule.scheduleTime !== undefined &&
                $scope.templates[idx].schedule.scheduleTime.min !== undefined &&
                $scope.templates[idx].schedule.scheduleTime.min !== null)) {
          var min = $scope.templates[idx].schedule.scheduleTime.min.toString();
          if (min.length === 1) {
            min = '0' + min;
          }
          $scope.selectedMinute[idx] = $scope.minutesList[$scope.minutesList.indexOf(min)];
        }
        if (!$scope.templates[idx].schedule) {
          $scope.templates[idx].schedule = {};
        }
        if (!$scope.templates[idx].schedule.scheduleType) {
          $scope.templates[idx].schedule.scheduleType = {};
        }
        if (!$scope.templates[idx].schedule.scheduleType.interval) {
          $scope.templates[idx].schedule.scheduleType.interval = {};
        }
        if (!$scope.templates[idx].schedule.scheduleTime) {
          $scope.templates[idx].schedule.scheduleTime = {};
        }
        if (!$scope.templates[idx].schedule.relativeStartTime) {
          $scope.templates[idx].schedule.relativeStartTime = {};
        }
        if (!$scope.templates[idx].schedule.relativeEndTime) {
          $scope.templates[idx].schedule.relativeEndTime = {};
        }
      }

      // idx is the index of the template within the protocol
      $scope.setMeasurement = function (idx) {

        // make the measurement selectable in the dropdown again
        if( !isEmpty($scope.templates[idx].service) ){
          var i = $scope.availMeasurements.indexOf($scope.templates[idx].service);
          $scope.availMeasurements[i].selectable = true;
        }

        // The select is bound to the option's index, so selectedMeasurement will be a number
        $scope.templates[idx].service = $scope.availMeasurements[$scope.selectedMeasurement[idx]];
        $scope.protocol.templates[idx].confirmationMessage = '';

        //If the number of params is greater than the default then create the array based on the size
        if( $scope.templates[idx].service !== undefined && $scope.templates[idx].service !== null ){
          if ($scope.templates[idx].service.readings.length !== $scope.numParams[idx].length) {
            setNumParams(idx);
          }

          // Set this measurement to disabled so it cannot be chosen again for another template.
          $scope.availMeasurements[$scope.selectedMeasurement[idx]].selectable = false;
        }
      };

      // Since minutes is rendered as a string we have to handle it in by setting it
      $scope.changeMinutes = function (s, idx) {
        $scope.selectedMinute[idx] = s;
      };

      // Cancel out of the edit page and return to the template list
      $scope.cancelEdit = function () {
        patientService.clearProtocolStatus();
        if($route.current.role === 'staffView')
        {
          $location.path('/template/list/');
        }
        else
        {
          $location.path('/clinicalAdmin/protocols');
        }

      };

      $scope.$on('$destroy', function () {
        var prot = AddProtocolFactory.loadData();
        if (prot.isNew === false) {
          $scope.saveDraft($scope.protocol);
        }
      });

      $scope.saveDraft = function (protocol) {
        // convert the minute string to int
        if ($scope.protocolId === null || $scope.protocolId === undefined) {
          protocol.name = $scope.newProtocol.name;
          protocol.description = $scope.newProtocol.description;
          protocol.area = $scope.newProtocol.area;
        }

        for (var i = 0; i < $scope.templates.length; i++) {
          $scope.templates[i].schedule.scheduleTime.min = parseInt($scope.selectedMinute[i], '');
          protocol.templates[i].name = $scope.templates[i].name;
          protocol.templates[i].description = $scope.templates[i].description;
          protocol.templates[i]['@class'] = $scope.templates[i]['@class'];
          protocol.templates[i].schedule = $scope.templates[i].schedule;
          if ($scope.templates[i].type === 'Vitals') {
            protocol.templates[i].service = $scope.templates[i].service;
          }

          var selectedScheduleTypeIndex = $scope.scheduleAllOptions.indexOf($scope.templates[i].schedule.scheduleType.name);

          var scheduleClass = scheduleTypeClasses[selectedScheduleTypeIndex];
          protocol.templates[i].schedule.scheduleType['@class'] = scheduleClass;
        }

        $scope.newProtocol.templates = protocol.templates;
        $scope.newProtocol.selectedMeasurement = $scope.selectedMeasurement;

        AddProtocolFactory.saveDraft($scope.newProtocol);
      };

      $scope.saveTemplate = function (protocol) {
        var i = 0;
        $scope.saveDraft(protocol);
        $scope.formSubmitted = true;

        $scope.formStatus = ProtocolValidationService.validateForm(protocol);
        for (i = 0; i < protocol.templates.length; i++) {
          $scope.templates[i].isReadingsCollapsed = $scope.formStatus.errors.templates[i].isReadingsOk;
          $scope.templates[i].isMessageCollapsed = true;
          $scope.templates[i].isScheduleCollapsed = true;
          $scope.templates[i].useReadingsNgErrorFlag = true;
          $scope.templates[i].useMessageNgErrorFlag = true;
          $scope.templates[i].useScheduleNgErrorFlag = true;
          $scope.templates[i].useNgErrorFlag = true;

          // Set the error on the messages area, but this makes sure new messages don't get the error automatically
          if ($scope.templates[i].type === 'Motivational/Educational') {
            $scope.templates[i].totalMessagesSubmitted = protocol.templates[i].serviceMessage.length;
          }
        }

        if ($scope.formStatus.ok === false || $scope.protocolForm.$invalid) {
          // force page to scroll to top when there is an error, to display the error message
          $window.scrollTo(0, 0);
        }

        if ($scope.protocolForm.$valid && $scope.formStatus.ok === true) {

          protocol.status = true;

          for (i = 0; i < $scope.templates.length; i++) {

            // Remove the flag which saved the number of submitted messages so only the correct number of errors are displayed.
            if ($scope.templates[i].type === 'Motivational/Educational') {
              delete $scope.templates[i].totalMessagesSubmitted;
            }

            if ($scope.templates[i].schedule.scheduleType.name !== $scope.scheduleAllOptions[2]) {
              delete protocol.templates[i].schedule.scheduleType.interval;
            }

            if ($scope.templates[i].schedule.scheduleType.name !== $scope.scheduleAllOptions[1]) {
              delete protocol.templates[i].schedule.scheduleType.monday;
              delete protocol.templates[i].schedule.scheduleType.tuesday;
              delete protocol.templates[i].schedule.scheduleType.wednesday;
              delete protocol.templates[i].schedule.scheduleType.thursday;
              delete protocol.templates[i].schedule.scheduleType.friday;
              delete protocol.templates[i].schedule.scheduleType.saturday;
              delete protocol.templates[i].schedule.scheduleType.sunday;
            }

            removeAccordionFields($scope.templates[i]);

            // TODO: creste a new template object and copy over attributes instead of deleting attributes from the template object in the scope
            delete protocol.templates[i].timeZones;
          }

          var comment = CommentService.getComment();
          if (comment !== null && $scope.protocolId !== null && $scope.protocolId !== undefined) {
            var cmt = {};
            cmt.content = comment;
            // TODO need to get rid of this, since time and user is filled by backend
            cmt.time = $filter('date')(new Date(), 'yyyy-MM-dd');
            cmt.user = '';
            var comments = protocol.comments;
            comments.push(cmt);
          }

          NewProtocolTemplateService.update(protocol,
              function () {
                if ($scope.protocolId !== null && $scope.protocolId !== undefined) {
                  patientService.setProtocolStatus('Successfully edited protocol template "' + protocol.name + '."');
                } else {
                  patientService.setProtocolStatus('Successfully created protocol template "' + protocol.name + '."');
                }
                AddProtocolFactory.resetData();
                if($route.current.role === 'staffView')
                {
                  $location.path('/template/list/');
                }
                else
                {
                  $location.path('/clinicalAdmin/protocols');
                }
                CommentService.clearComment();
                $scope.open();
              },
              function () {
                if ($scope.protocolId !== null && $scope.protocolId !== undefined) {
                  $log.info('NewProtocolTemplateCtrl editing template failed.');
                  patientService.setProtocolStatus('Error creating protocol template "' + protocol.name + '."');   // TODO : add http response in the error messages
                } else {
                  $log.info('NewProtocolTemplateCtrl creating template failed.');
                  patientService.setProtocolStatus('Error creating protocol template "' + protocol.name + '."');
                }
              }
          );
        }
      };

      $scope.open = function () {

        var modalInstance = $modal.open({
          templateUrl: 'modalConfirmContent',
          controller: 'FinishInstanceCtrl',
        });

        modalInstance.result.then(function () {
        });

      };

      $scope.cancel = function () {

        CommentService.clearComment();
        var modalInstance = $modal.open({
          templateUrl: 'modalCancelTopContent',
        });

        modalInstance.result.then(function () {
        });
      };

      $scope.openRemoveMessageModal = function (index) {
        var modalInstanceRemove = $modal.open({
          templateUrl: 'modalRemoveParamContent',
        });
        
        modalInstanceRemove.result.then(function () {
          $scope.removeMessage(index);
        }, function () {
          $log.info('Modal dismissed');
        });
      };

      var isEmpty = function( o ) {
        for ( var p in o ) {
          if ( o.hasOwnProperty( p ) ) { return false; }
        }
        return true;
      };

      $scope.isOnlySpaces = function( str ){
        return ProtocolValidationService.testOnlySpaces(str);
      };
    });

angular.module('MessagingAppApp')
  .controller('FinishInstanceCtrl', function ($scope, $modalInstance) {

    $scope.okay = function () {
      $modalInstance.dismiss('cancel');
    };

  });
