'use strict';

angular.module('MessagingAppApp')
    .controller('PatientProfileCtrl', function ($scope, $log, $routeParams, $filter, $timeout, $location, patientService,
                patientRegistrationService, MeasurementDictionaryService, patientRegistrationSaveService, $modal, facilityService) {

  $scope.currPatient = patientService.getCurrPatient();
  $scope.patientConsent = false;

  var userProfile = null;

  // default for accordions is collapsed,except the consent one.
  $scope.consentCollapsed = false;
  $scope.apptCollapsed = true;
  $scope.preferencesCollapsed = true;
  $scope.patientInfoCollapsed = true;


  if( typeof $routeParams.patientid !== 'undefined'){
      userProfile = patientRegistrationService.get({id: $routeParams.patientid},
          function () {
            $scope.profile = userProfile; // TODO: fix this uneeded. move to setProfile
            $scope.patientExistInCms = $scope.profile.patientExistInCms;
            setProfile($scope.profile);
          },
          function () {
            $scope.profile = null;
          });
      }
      else{
      userProfile = patientService.getPatientToRegister();
      userProfile.patientExistInCms = false;
      setProfile(userProfile);
  }

	function setProfile(profile){
	    $scope.profile = profile;
	    $scope.profile.gender = $filter('lowercase')(profile.gender);
	    $scope.profile.dob = $filter('date')(profile.dob, 'MM/dd/yyyy');
        // default the checkbox to false

        if( profile.preference !==  null && profile.preference !== undefined ){

          // check the delivery preference radio button
          if( $scope.patientExistInCms === true ) {
              $scope.msgDeliveryPreference = $scope.profile.preference.messageDeliveryPreference.sendSms === true ? 'sendSms' : 'sendAnnie';
          }

          var homeNumber = profile.preference.homePhoneNumber;
	      if ( homeNumber !== null )
	      {
	        var homeNumberProfile = homeNumber.substr(1);
	        $scope.homePhoneNumber = homeNumberProfile;
	      }
	      else
	      {
	        $scope.homePhoneNumber = homeNumber;
	      }

	      var mobileNumber = profile.preference.phoneNumber.substr(1);
	      $scope.mobileNumber = mobileNumber;
	      if (profile.consent === 3) {
	        $scope.patientConsent = true;
	      }
	    }
        else {
            // new users from vista don't have a preferences object. initialize one.
            $scope.profile.preference = {};
            $scope.profile.preference.notificationDeliveryPreference = {};
            $scope.profile.preference.notificationDeliveryPreference.sendSms = false;
        }
	}

    $scope.timezone = [];
    var timeZoneResponse = MeasurementDictionaryService.get(
          function(){
            setTimeZone(timeZoneResponse);
          }
        );

        function setTimeZone(time){
            $scope.timezone = time.timeZones;
           //$log.info('Timezone object');$log.info($scope.timezone);
          }

    $scope.gender = [
            { name: 'Female', value: 'FEMALE' },
            { name : 'Male', value : 'MALE' }
          ];


    // set the flag that capture from the parent scope
    $scope.patientConsented = function () {
      if ($scope.patientConsent === true) {

        // Set the consent time now.
        var now = new Date();
        var meridiem = now.getHours() < 12 ? 'AM' : 'PM';
        var hours = now.getHours() % 12 || 12;
        var min = now.getMinutes() < 10 ? '0' + now.getMinutes() : now.getMinutes();
  
        $scope.profile.consentTime = hours + ':' + min + ' ' + meridiem;
  
        if ($scope.profile.consentDate === undefined) {
          $scope.profile.consentDate = (now.getMonth() + 1) + '/' + now.getDate() + '/' + now.getFullYear();
        }
      }
    };


    // TODO: this can ben done on the FE!!!
    // Flag for Facility Required dropdown
    $scope.facilityIsRequired = false;

    $scope.validationFacility = function () {
      if ($scope.profile.preference.receiveFacilityMessage === true) {
        $scope.facilityIsRequired = true;
      } else {
        $scope.facilityIsRequired = false;
        $scope.profile.facilityId = null;
      }
    };




    // Get all the dynamic list of facility from backend.
    facilityService.get().$promise.then(function (msg) {
        $scope.facilityList = msg;

    }, function () {
        $scope.facilityList = null;
    });


    // a test to see if there is an error on the form, and if so, open the appropriate accordions
    var testFormAccordions = function(){
      // Open the consent accordion if there was an error in that section
      if($scope.profileForm.consentForm !== undefined && $scope.profileForm.consentForm.$invalid){
        $scope.consentCollapsed = false;
      }

      // per acceptance criteria, open the accordion if the facility was required
      if($scope.profileForm.facility !== undefined && $scope.profileForm.facility.$invalid === true) {
        $scope.apptCollapsed = false;
      }

      if($scope.profileForm.profilePhonePref !== undefined && $scope.profileForm.profilePhonePref.$invalid) {
        $scope.preferencesCollapsed = false;
      }

      // this condition has to be on one line because of jshint error....
      if($scope.isDuplicatePhoneNumber === true || ($scope.profileForm.mobileNumber !== undefined && $scope.profileForm.mobileNumber.$invalid) || ($scope.profileForm.timezone !== undefined && $scope.profileForm.timezone.$invalid) || ($scope.profileForm.Email !== undefined && $scope.profileForm.Email.$invalid)) {
        $scope.patientInfoCollapsed = false;
      }
    };



    // Update or save new profile.
    $scope.updateProfile = function () {

      $scope.profileForm.submitted = true;

      if ($scope.profileForm.$valid) {

        // set message delivery preference
        if( $scope.patientExistInCms !== true ) {
            $scope.profile.preference.messageDeliveryPreference = {};
        }
        $scope.profile.preference.messageDeliveryPreference.sendSms = $scope.msgDeliveryPreference === 'sendSms' ? true : false ;


        // Format Home Phone Number
        var homeNumber = {};
        homeNumber = $scope.homePhoneNumber;

        // TODO: why do we need home number to be an object???
        if (typeof homeNumber === 'object' && homeNumber === null) {
          $scope.profile.preference.homePhoneNumber = homeNumber;
        } else if (homeNumber !== undefined) {
          // Format  Phone Number. Back end requires to append a 1
          $scope.profile.preference.homePhoneNumber = '1' + homeNumber;
        }


        // Format Mobile Phone Number. Back end requires to append a 1
        var mobileNumber = $scope.mobileNumber;
        $scope.profile.preference.phoneNumber = '1' + mobileNumber;
        if ($scope.patientConsent === true) {
          $scope.profile.consent = 3;
        }


        // Save the patient's profile
        patientRegistrationSaveService.update($scope.profile,
          function (resp) {
            // Success!
            patientService.setCurrPatient(resp);

            // clean up hackish success and phone numbers.
            $scope.isDuplicatePhoneNumber = null;
            $scope.theDuplicatePhoneNumber = null;
            $scope.duplicatePhoneNumberMessage = null;

            if ($scope.profile.patientExistInCms === true) {
              patientService.setProtocolStatus('Thank you. Account settings successfully saved.');
              $location.path('/patient/protocolList');
            } else if ($scope.profile.patientExistInCms === false) {
              $location.path('/success');
            }
          }, function (error) {

            // error returned
            $log.info('An error occurred:');
            $log.info(error);

            // set phone number error flag. parse other error codes. see error.configuration file in back end.
            // Code 1004 is passed back for duplicate Phone Number
            if (error.data.code === 1004) {
              $log.info('error 1004');
              // hack, should be part of global system, but since it's after form post we can't alter form validitiy
              // can't add into message scope; as that's for success only.
              $scope.isDuplicatePhoneNumber = true;
              $scope.theDuplicatePhoneNumber = $scope.mobileNumber;
              $scope.duplicatePhoneNumberMessage = error.data.message;
              // 'Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX.';
            }
          }
        );
      }
      else {
        testFormAccordions();
      } // end else
    };


    // cancel editing of the account info
    $scope.cancelEdit = function () {
      if( $scope.profile.patientExistInCms === true )
      {
        $location.path('/patient/protocolList');
      }
      else
      {
        $location.path('/patientSearch');
      }
    };

    // open the consent modal
    $scope.open = function () {

        var modalInstance = $modal.open({
            templateUrl: 'views/infoAndDisclosureForm.tpl.html',
            controller: 'consentModalCtrl'
        });

        modalInstance.result.then(function () {
        });
    };
});


angular.module('MessagingAppApp').controller('consentModalCtrl', function ($scope, $modalInstance) {

    $scope.cancel = function () {
        $modalInstance.dismiss('cancel');
    };

});
