'use strict';

angular.module('MessagingAppApp')
  .controller('VistaPatientResultsCtrl', function ($scope, $log, ResourceDirectoryService,
              $resource, $http, patientService, $location, $routeParams) {

    patientService.clearCurrPatient();

    $scope.vistaErrorMsg = 'We\'re sorry, the patient entered cannot be found, please search again.';
    $scope.noPatientsFoundHA = false;

    var searchHref, searchResource;
    searchHref = ResourceDirectoryService.getMhpSearchHref();

    searchResource = $resource(searchHref,{},{});
    searchResource.get({searchstring: $routeParams.query}, function(result) {
      if( result.size < 1 ){
        $scope.noPatientsFoundHA = true;
      } else {
        $scope.noPatientsFoundHA = false;
        $scope.patientList = result.patient;
        //console.log('the search = ');
        //console.log(result.patient);
      }
    },function (result) { /* This function handles all the error messages come from backend through $resource call */
      if( result.status === 400 ){
        $scope.noPatientsFoundHA = true;
        $scope.vistaErrorMsg = result.data.message;
      } else {
        $scope.noPatientsFoundHA = true;
      }
    });


    // need to parse the dob string to a certain format per mockups
    $scope.getDate = function(date){

      if(typeof date === 'undefined'){
        return null;
      }

      var month, day, year;
      month = day = year = 0;
      var months = ["jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"];
      var dateParts = date.split(' ');
      if( dateParts.length < 3 ){
        if( typeof dateParts[0] === 'string' ){
          month = (months.indexOf(dateParts[0].toLowerCase()) + 1);
          var commaIndex = dateParts[1].indexOf(',');
          if( commaIndex !== -1 ){
            day = dateParts[1].substring(0, commaIndex);
          }
          else{
            year = dateParts[1];
          }
        }
        else{
          // assume it's a year
          if( dateParts[0].length === 4 ){
            year = dateParts[0];
          }
        }
      }
      else{
        month = (months.indexOf(dateParts[0].toLowerCase()) + 1);
        var commaIndex = dateParts[1].indexOf(',');
        day = dateParts[1].substring(0, commaIndex);
        year = dateParts[2];
      }

      // append a 0 to a single digit number for back end parsing
      if( month.toString().length === 1 ){
        month = '0' + month;
      }

      return year + '-' + month + '-' + day;
    };

    $scope.selectPatient = function(patient){

      $http({method: 'GET', url: patient.link[0].href}).
        success(function(data, status, headers, config) {

          var first = data.firstName[0].toUpperCase() + data.firstName.slice(1).toLowerCase();
          var last = data.lastName[0].toUpperCase() + data.lastName.slice(1).toLowerCase();

          var patientObj = {};
          patientObj.firstName = first;
          patientObj.lastName = last;
          patientObj.ssn = data.ssn;
          patientObj.dob = $scope.getDate(data.dateOfBirth);
          patientObj.gender = data.gender;
          patientObj.age = data.age;
          patientObj.patientIdentifier = data.patientIdentifier;

          patientService.setPatientToRegister(patientObj);
          $location.path('/patientRegistration');
        }).
        error(function(data, status, headers, config) {
        $log.info('VistaPatientResultsCtrl. Error retrieving patient profile.');
      });
    };

    $scope.searchAgain = function(query){
      $location.path('/patientSearch/' + query);
    }

    // Code for pagination of patient list
    $scope.currentPage = 1;
    $scope.itemsPerPage = 20;

    $scope.setPage = function (pageNo) {
      $scope.currentPage = pageNo;
    };

  });
