'use strict';

angular.module('MessagingAppApp')
    .controller('ReportsCtrl', function ($scope, $log, $http, PatientsReportsService, $rootScope, patientService) {

    $log.info('ReportsCtrl fired.');

    // Some init
    $scope.currPatient = patientService.getCurrPatient();
    // error message const
    var CANT_FIND_MEAS_ERROR = 'Unable to locate reading for selected reading.';
    // the parsed data for the table
    $scope.tableData = [];
    // save a copy of the entire set of data so we can vary the dates in the table
    $scope.allTableData = [];
    $scope.tableName = '';
    $scope.unit = '';
    $scope.selectedServiceName = '';
    $scope.serviceNames = [];

    // Code for pagination
    $scope.currentPage = 1;
    $scope.itemsPerPage = 30;
    $scope.ascending = true;
    $scope.setPage = function (pageNo) {
      $scope.currentPage = pageNo;
    };

    // Get the patients readings from the service
    if( $scope.currPatient !== null && typeof $scope.currPatient !== 'undefined' ){
      var resp = PatientsReportsService.getReadings($scope.currPatient._id);
      $scope.readings = resp.readings;
      $scope.serviceNames = resp.serviceNames;
      $scope.patientReadings = resp.patientReadings;
      // set a default
      $scope.selectedServiceName = $scope.serviceNames[0];
    }

    // Update the chart based on the filter choice on the page.
    $scope.updateChart = function(){
      $scope.chartConfig = PatientsReportsService.getChartConfig();

      // a new vital was selected by the user in the filters
      $scope.tableData = [];

      // Blood pressure is a special case where it has two readings so name systolic & diastolic just blood pressure
      if( $scope.selectedServiceName.toUpperCase() === 'BLOOD PRESSURE' ){

        var indexSystolic = objExists('systolic');
        var indexDiastolic = objExists('diastolic');
        var series0 = {name: $scope.readings[indexSystolic].name,
                        data: $scope.readings[indexSystolic].data,
                        marker : { enabled : true, radius : 3, lineWidth: 2, lineColor: '#51324e', fillColor: {}},
                        color: '#51324e'
                       };
        var series1 = {name: $scope.readings[indexDiastolic].name,
                       data: $scope.readings[indexDiastolic].data,
                       marker : { enabled : true, radius : 3, lineWidth: 2, lineColor: '#51324e', fillColor: {}},
                       color: '#51324e'
                       };
        $scope.chartConfig.series = [series0, series1];

        // set up the chart
        $scope.allTableData = [];
        $scope.allTableData.push({'systolic': $scope.readings[indexSystolic].data, 'diastolic': $scope.readings[indexDiastolic].data});
        getReadingName($scope.selectedServiceName);
        $scope.tableName = $scope.selectedServiceName;
        $scope.unit = $scope.readings[indexSystolic].name + '/' + $scope.readings[indexDiastolic].name + ' ' + $scope.unit;
        $scope.chartConfig.plotOptions.line.pointStart = $scope.readings[indexSystolic].data[0][0];

        // if there are less than 7 entries (1 week's worth of data) the chart won't trigger an
        // event so we have to manually set the table range
        var len = $scope.readings[indexSystolic].data.length - 1;
        setTableRange([$scope.readings[indexSystolic].data[0][0], $scope.readings[indexSystolic].data[len][0]]);
      }
      else{

        var rName = getReadingName($scope.selectedServiceName);

        // get index of readings array for this measurement
        var index = objExists(rName);
        if( index === CANT_FIND_MEAS_ERROR ){
          $log.info('Reports Controller, ' + CANT_FIND_MEAS_ERROR);
        }

        if( $scope.chartConfig.series.length > 1 ){
          $scope.chartConfig.series.splice(1);
        }

        $scope.allTableData = $scope.readings[index].data.slice(0);
        $scope.chartConfig.series[0].name = $scope.readings[index].name;
        $scope.chartConfig.plotOptions.line.pointStart = $scope.readings[index].data[0][0];
        $scope.chartConfig.series[0].data = $scope.readings[index].data;
        $scope.tableName = $scope.readings[index].name;

        // if there are less than 7 entries (1 week's worth of data) the chart won't trigger an
        // event so we have to manually set the table range
        setTableRange([$scope.readings[index].data[0][0], $scope.readings[index].data[$scope.readings[index].data.length-1][0]]);
      }

      // TODO: why doesn't this show up???
      $scope.chartConfig.yAxis.title = $scope.tableName + ' (' + $scope.unit + ')';
      setChartTitle($scope.selectedServiceName);

    };    // end updateChart

    // change the chart when the zoom has changed
    $rootScope.$on('chartRangeChanged', function(event, input) {
      setTableRange(input);
    });

    // this function calculates which readings should be displayed on the table, based on what's showing in the chart
    var setTableRange = function(input){
      var data = $scope.allTableData.slice(0);
      var tmpTable = [];
      var i = 0, x, phase;

      if( input[0] === 'all' ){

        if( $scope.selectedServiceName.toUpperCase() === 'BLOOD PRESSURE' ){
          for( x = 0; x < data[0].systolic.length; x++ ){
            tmpTable.push({'date': data[0].systolic[x][0], 'value': data[0].systolic[x][1] + '/' + data[0].diastolic[x][1]});
          }
        }
        else{

          for( x = 0; x < data.length; x++ ){
            tmpTable.push({'date': data[x][0], 'value': data[x][1]});
          }

        }

        // for pagination
        $scope.setPage(1);
        $scope.totalReadings = tmpTable.length;

        // update table data.
        phase = $scope.$root.$$phase;
        if(phase !== '$apply' && phase !== '$digest'){
          $scope.$apply(function(){
            $scope.tableData = tmpTable;
          });
        }
        else{
          $scope.tableData = tmpTable;
        }
      }
      else{
        var max = input[1];
        var min = input[0];

        if( $scope.selectedServiceName.toUpperCase() === 'BLOOD PRESSURE' ){

          while( data[0].systolic[i] && data[0].systolic[i][0] < min ){
            i++;
          }

          // create the systolic/diastolic structure
          while( data[0].systolic[i] && data[0].systolic[i][0] <= max ){
            tmpTable.push({'date': data[0].systolic[i][0], 'value': data[0].systolic[i][1] + '/' + data[0].diastolic[i][1]});
            i++;
          }
        }
        else{
          while( data[i] && data[i][0] < min ){
            i++;
          }

          while( data[i] && data[i][0] <= max ){
            tmpTable.push({'date': data[i][0], 'value': data[i][1]});
            i++;
          }
        }

        // for pagination
        $scope.totalReadings = tmpTable.length;

        // update table data.
        phase = $scope.$root.$$phase;
        if(phase !== '$apply' && phase !== '$digest'){
          $scope.$apply(function(){
            $scope.tableData = tmpTable;
          });
        }
        else{
          $scope.tableData = tmpTable;
        }
      }
    };

    // loop through the patient's readings to check which measurement this reading param belongs to
    var getReadingName = function(meas){

      meas = meas.toUpperCase();
      for( var i = $scope.patientReadings.length - 1; i >= 0; i-- ){
        if( $scope.patientReadings[i].patientServiceReading.serviceName.toUpperCase() === meas ){
          $scope.unit = $scope.patientReadings[i].patientServiceReading.readings[0].unit;
          return $scope.patientReadings[i].patientServiceReading.readings[0].name;
        }
      }

      // not found. return error
      return CANT_FIND_MEAS_ERROR;
    };

    // function to test if object exists
    var objExists = function(name){

      for(var i = 0; i < $scope.readings.length; i++) {
        if( $scope.readings[i].name.toUpperCase() === name.toUpperCase() ) {
          return i;
        }
      }
      // else, not found
      return -1;
    };

    var setChartTitle= function(meas){
      // capitalize the first letters of the name
      var first = $scope.currPatient.firstName.charAt(0).toUpperCase() + $scope.currPatient.firstName.substr(1).toLowerCase();
      var last = $scope.currPatient.lastName.charAt(0).toUpperCase() + $scope.currPatient.lastName.substr(1).toLowerCase();
      $scope.chartConfig.title.text = first + ' ' + last + '\'s ' + meas ;
    };

    // conversion functions for table which add/subtracts date based on computer's time
    var toUTCDate = function(date){
      var _utc = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(),  date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
      return _utc;
    };

    var millisToUTCDate = function(millis){
      return toUTCDate(new Date(millis));
    };

    $scope.toUTCDate = toUTCDate;
    $scope.millisToUTCDate = millisToUTCDate;
  });
