'use strict';

angular.module('MessagingAppApp')
    .controller('TemplateListCtrl', function ($scope, $log, $location, $timeout, $window, TemplateListService,
        resolvedTemplates, patientService, CommentService, $rootScope, $route, NewProtocolTemplateService, $modal) {
      //$log.info("TemplateListCtrl fired");

      $rootScope.$on('cancelClicked', function(){
        $log.info('cancelClicked on function called!');
        $route.reload();
      });
    
      // When we come to this list of templates, clear the status messages that might be left over from the last page,
      // since this is a navigation bar link
      patientService.clearProtocolStatus();
      CommentService.clearComment();
      $scope.protocolStatus = patientService.getProtocolStatus();
      $scope.disableStatus = false;

      $scope.newProtocolText = 'Protocol Created';

      $scope.getProtocolId = function (protocol) {
        //$log.info("getProtocolId = " + protocol._id);    // TODO: why is this called 3 times when listing available protocols.???
        return protocol._id;
      };

      $scope.$on('updateItems', function (e, protocol) {
        $scope.disableStatus = 'The Protocol ' + protocol.name + ' is now inactive.';
      });

      $scope.protocolsList = resolvedTemplates;

      // Function for the Tooltip directive if protocl name is more than 50
      var i;
      $scope.fullProtocolName = [];
      $scope.protocolName = [];
      $scope.fullProtocolDesc = [];
      $scope.protocolDesc = [];
      $scope.fullProtocolArea = [];
      $scope.protocolArea = [];

      for (i = 0; i < $scope.protocolsList.length; i++)
      {
        var maxSizeProtocol = 50;
        $scope.fullProtocolName[i]  = $scope.protocolsList[i].name;
        $scope.protocolName[i] = $scope.fullProtocolName[i];
        if( $scope.fullProtocolName[i].length > maxSizeProtocol ){
          $scope.protocolName[i] = $scope.fullProtocolName[i].substring(0, maxSizeProtocol);
          $scope.protocolName[i] += '...';
        }

        var maxSize = 100;
        $scope.fullProtocolDesc[i]  = $scope.protocolsList[i].description;
        $scope.protocolDesc[i] = $scope.fullProtocolDesc[i];
        if( $scope.fullProtocolDesc[i].length > maxSize ){
          $scope.protocolDesc[i] = $scope.fullProtocolDesc[i].substring(0, maxSize);
          $scope.protocolDesc[i] += '...';
        }

        $scope.fullProtocolArea[i]  = $scope.protocolsList[i].area;
        $scope.protocolArea[i] = $scope.fullProtocolArea[i];
        if( $scope.fullProtocolArea[i].length > maxSize ){
          $scope.protocolArea[i] = $scope.fullProtocolArea[i].substring(0, maxSize);
          $scope.protocolArea[i] += '...';
        }

      }


      // If the user navigates away from the page, clear the results
      $scope.$on('$destroy', function () {
        $scope.clearProtocolResult();
      });

      $scope.clearProtocolResult = function () {
        $scope.protocolStatus = '';
        $scope.disableStatus = '';
        patientService.clearProtocolStatus();
      };

      $scope.saveEnableComment = function (comment, protocol) {
        var cmt = {};
        cmt.content = comment;
        cmt.time = new Date();
        cmt.user = '';
        CommentService.setComment(comment);
        $location.path('/clinicalAdmin/template/view/' + protocol._id);
      };

      $scope.getLastComment = function (comments) {
        var i;
        for (i=comments.length-1; i>=0; i-- ) {
          if (comments[i].content!==null && comments[i].content!=='') {
            return comments[i];
          }
        }
        return null;
      };

      // Timer to remove the status message after deleting/assigning/editing a template
      var timer = false;
      var delay = 0000;
      $scope.$watch('StatusModel', function () {
        if (timer) {
          $timeout.cancel(timer);
        }
        timer = $timeout(function () {
          $scope.clearProtocolResult();
        }, delay);
      });
      
      $scope.disableProtocol = function (comment, protocol) {

        protocol.status = false;
        var len = protocol.comments.length;
        protocol.comments[len] = {};
        protocol.comments[len].content = comment;
        NewProtocolTemplateService.update(protocol, function () {
          var result = TemplateListService.get({id: protocol._id},
            function () {
              protocol.comments = result.comments;
              $rootScope.$broadcast('updateItems', protocol);
            });
        },
        function (resp) {
          var DUP_NAME;   // TODO: what is this????
          var errorMsg = resp.status === 409 ? DUP_NAME : '';
          $log.info('ProtocolListCtrl failed deleting protocol.');
          $scope.disableStatus = 'Error deleting ' + protocol.name + '". ' + errorMsg;
        });
      };

      $scope.open = function (flag, protocol) {
        var modalInstance;
        if (flag === true) {
          modalInstance = $modal.open({
            templateUrl: 'myModalContent.html',
            controller: 'ModalDisableCtrl',
            resolve: {
              disableComment: function () {
                return $scope.disableComment;
              }
            }
          });
        } else {
          modalInstance = $modal.open({
            templateUrl: 'myModalContentEnable.html',
            controller: 'ModalEnableCtrl',
            resolve: {
              disableComment: function () {
                return $scope.disableComment;
              }
            }
          });
        }

        modalInstance.result.then(function (comment) {
          $scope.disableComment = comment;

          // flag true means disable
          if (flag === true) {
            $scope.disableProtocol($scope.disableComment, protocol);
          } else {
            $scope.saveEnableComment(comment, protocol);
          }
        }, function () {
          $log.info('Modal dismissed at: ' + new Date());
        });
      };
    });

