'use strict';
angular.module('MessAppDirectives', ['ngCookies','ngResource']).directive('placeholder', function ($timeout) {
  var i = document.createElement('input');
  if ('placeholder' in i) {
    return {};
  }
  return {
    link: function (scope, elm, attrs) {
      $timeout(function () {
        elm.val(attrs.placeholder);
        elm.bind('focus', function () {
          if (elm.val() === attrs.placeholder) {
            elm.val('');
          }
        }).bind('blur', function () {
          if (elm.val() === '') {
            elm.val(attrs.placeholder);
          }
        });
      });
    }
  };
}).directive('maxlength', function () {
  return {
    require: 'ngModel',
    link: function (scope, element, attrs, ngModelCtrl) {
      var maxlength = Number(attrs.maxlength);

      function fromUser(text) {
        if (text === undefined) {
          return '';
        }
        if (text.length > maxlength) {
          var transformedInput = text.substring(0, maxlength);
          ngModelCtrl.$setViewValue(transformedInput);
          ngModelCtrl.$render();
          return transformedInput;
        }
        return text;
      }
      ngModelCtrl.$parsers.push(fromUser);
    }
  };
}).directive('ngBlur', function () {
  return {
    restrict: 'A',
    require: 'ngModel',
    link: function (scope, el, attrs, ctrl) {
      var BLUR_CLASS = 'ng-blurred';
      ctrl.$blurred = false;
      el.on('keydown', function () {
        el.removeClass(BLUR_CLASS);
        scope.$apply(function () {
          ctrl.$blurred = false;
        });
      });
      el.on('blur', function () {
        el.addClass(BLUR_CLASS);
        scope.$apply(function () {
          ctrl.$blurred = true;
        });
      });
    }
  };
}).directive('numericOnlyWithoutZero', function () {
  return {
    require: 'ngModel',
    link: function (scope, element, attrs, modelCtrl) {
      modelCtrl.$parsers.push(function (inputValue) {
        // this next if is necessary for when using ng-required on your input. 
        // In such cases, when a letter is typed first, this parser will be called
        // again, and the 2nd time, the value will be undefined
        if (inputValue === undefined) {
          return '';
        }
        var transformedInput = inputValue.replace(/(^0)|([^0-9])+/g, '');
        if (transformedInput !== inputValue) {
          modelCtrl.$setViewValue(transformedInput);
          modelCtrl.$render();
        }
        return transformedInput;
      });
    }
  };
}).directive('numericOnly', function () {
  return {
    require: 'ngModel',
    link: function (scope, element, attrs, modelCtrl) {
      modelCtrl.$parsers.push(function (inputValue) {
        // this next if is necessary for when using ng-required on your input. 
        // In such cases, when a letter is typed first, this parser will be called
        // again, and the 2nd time, the value will be undefined
        if (inputValue === undefined) {
          return '';
        }
        var transformedInput = inputValue.replace(/[^0-9]+/g, '');
        if (transformedInput !== inputValue) {
          modelCtrl.$setViewValue(transformedInput);
          modelCtrl.$render();
        }
        return transformedInput;
      });
    }
  };
}).directive('numericOnlyWithPeriod', function () {
  return {
    require: 'ngModel',
    link: function (scope, element, attrs, modelCtrl) {
      modelCtrl.$parsers.push(function (inputValue) {
        // this next if is necessary for when using ng-required on your input. 
        // In such cases, when a letter is typed first, this parser will be called
        // again, and the 2nd time, the value will be undefined
        if (inputValue === undefined) {
          return '';
        }
        var transformedInput = inputValue.replace(/[^0-9\.]+/g, '');
        if( transformedInput.split('.').length>2)
        {
          transformedInput = inputValue.replace(/\.+$/,'');
        }
        if (transformedInput !== inputValue) {
          modelCtrl.$setViewValue(transformedInput);
          modelCtrl.$render();
        }
        return transformedInput;
      });
    }
  };
}).directive('alphaNumericOnly', function () {
  return {
    require: '?ngModel',
    link: function (scope, element, attrs, ngModelCtrl) {
      if (!ngModelCtrl) {
        return;
      }
      ngModelCtrl.$parsers.push(function (val) {
        var clean = val.replace( /[^a-z0-9]/gi, '');
        if (val !== clean) {
          ngModelCtrl.$setViewValue(clean);
          ngModelCtrl.$render();
        }
        return clean;
      });
      element.bind('keypress', function (event) {
        if (event.keyCode === 32) {
          event.preventDefault();
        }
      });
    }
  };
}).directive('checkboxGroup', function() {
  return {
    restrict: 'E',
    controller: function($scope, $attrs) {
      var self = this;
      var ngModels = [];
      var minRequired;
      self.validate = function() {
        var checkedCount = 0;
        angular.forEach(ngModels, function(ngModel) {
          if ( ngModel.$modelValue ) {
            checkedCount++;
          }
        });
        var minRequiredValidity = checkedCount >= minRequired;
        angular.forEach(ngModels, function(ngModel) {
          ngModel.$setValidity('checkboxGroup-minRequired', minRequiredValidity, self);
        });
      };
      
      self.register = function(ngModel) {
        ngModels.push(ngModel);
      };
      
      self.deregister = function(ngModel) {
        var index = ngModels.indexOf(ngModel);
        if ( index !== -1 ) {
          ngModels.splice(index, 1);
        }
      };
        
      $scope.$watch($attrs.minRequired, function(value) {
        minRequired = parseInt(value, 10);
        self.validate();
      });
    }
  };
}).directive('input', function() {
  return {
    restrict: 'E',
    require: ['?^checkboxGroup','?ngModel'],
    link: function(scope, element, attrs, controllers) {
      var checkboxGroup = controllers[0];
      var ngModel = controllers[1];
      if ( attrs.type==='checkbox' && checkboxGroup && ngModel ) {
        checkboxGroup.register(ngModel);
        scope.$watch(function() { return ngModel.$modelValue; }, checkboxGroup.validate );
        // In case we are adding and removing checkboxes dynamically we need to tidy up after outselves.
        scope.$on('$destroy', function() { checkboxGroup.deregister(ngModel); });
      }
    }
  };
}).directive('datepickerPattern',function() {
  return {
    restrict: 'A',
    require: 'ngModel',
    link: function(scope,elem,attrs,ngModelCtrl) {
      var dRegex, isValid;
      dRegex = new RegExp(attrs.datepickerPattern);
      
      ngModelCtrl.$parsers.unshift(function(value) {
       
        if (typeof value === 'string') {

          isValid = dRegex.test(value);
          ngModelCtrl.$setValidity('date',isValid);
          if (!isValid) {
            return undefined;
            // Set the value to undefined if date is valid
          }
        }
        return value;
      });
      
    }
  };
});