'use strict';

angular.module('AuthStaff')
  .factory('AuthorizationInterceptor', function ($q, $log, $injector, $location) {

    //which errors require us to login
    var errorRequiresLogin = function (responseStatus) {
      return (responseStatus === 403 || responseStatus === 401);
    };

    return {
      // optional method
      'request': function (config) {
        //Need to utilize injector to gain access to userService so we avoid circular dependency at app start
        // http://stackoverflow.com/questions/14681654/i-need-two-instances-of-angularjs-http-service-or-what
        //see 2nd solution by thaumant

        $injector.invoke(['$log', 'userService', function ($log, userService) {
          // do something on success
          // $log.debug('request= ' + config.url + '. authenticated user= ' + userService.isAuthenticated() + '. token= ' + userService.getAuthorizationToken());

          //Inline function in attempt to resolve circular dependency issues based upon instantiating userService at creation
          //if we are authenticated, add the authorization header to the request
          if (userService.isAuthenticated()) {
            config.headers.Authorization = userService.getAuthorizationToken();
          }
        }]);

        return config;// || $q.when(config);
      },
      'responseError': function (rejection) {
        //we received an error, so check the status and make sure we are not in process of logging out
        if (errorRequiresLogin(rejection.status)) {
          // start login process
          $injector.invoke(['userService', function (userService) {
            userService.forceLogin();
          }]);
        }
        return $q.reject(rejection);
      }
    };
  });
