'use strict';

angular.module('AuthStaff').factory('userService', [
  '$window',
  '$location',
  '$resource',
  '$timeout',
  'ResourceDirectoryService',
  function ($window, $location, $resource, $timeout, ResourceDirectoryService) {
    var clearStorage;

    clearStorage = function () {
      // clear session storage, if it exists.
      $window.sessionStorage.clear();
    };

    return {
      isAuthenticated: function () {
        return $window.sessionStorage.getItem('token') !== null;
      },
      getAuthorizationToken: function () {
        return 'Bearer ' + $window.sessionStorage.token;
      },
      setAuthenticationToken: function (token) {
        $window.sessionStorage.token = token;
      },
      logout: function () {
        //invalidate the token via authServices
        //we do need to manage the headers here since it is call to external site
        var tokenHref = ResourceDirectoryService.getTokenHref();

        var OAuthBearerToken = $resource(tokenHref, {}, {});

        OAuthBearerToken.delete(function() {
          clearStorage();

          $window.location = ResourceDirectoryService.getLogoutHref();
        });
      },
      forceLogin: function () {
        // this short timeout prevents the service from being called before fully instantiated.
        // It, in effects, puts the call into the Angular digest cycle.
        $timeout(function() {
          clearStorage();

          var loc = 'https://' + $location.host() + '/cms/service/oauthlogin?redirect_uri=https://' + $location.host() + '/cms/#';
          $window.location.href = loc;
        }, 500);
      }
    };
  }
]);
