/*jslint node: true, unparam: true */
/*global angular, navigator */

'use strict';

angular.module('MessagingAppApp').config(['$provide', '$httpProvider', function ($provide, $httpProvider) {
  $provide.factory('httpInterceptor', [
    '$q',
    '$injector',
    '$log',
    function ($q, $injector, $log) {
      return {
        // optional method
        'request': function(config) {
          return config || $q.when(config);
        },
        // optional method
        'requestError': function (rejection) {
          $injector.invoke(['connectionErrorService', function (connectionErrorService) {
            $log.info('HTTP Interceptor caught Request Error: ' + rejection.status);
            connectionErrorService.openServerErrorModal(rejection);
          }]);
          return $q.reject(rejection);
        },
        // optional method
        'response': function (response) {
          return response || $q.when(response);
        },
        // optional method
        'responseError': function (rejection) {
          // we don't repress log statements in production, so I'm commenting out these; but they're important for debugging and finding requests for the 'whitelist' of errors.
          // $log.info('HTTP Interceptor caught Response Error: ' + rejection.status);
          // $log.info(rejection);
          var newPromise;

          // Is the local browser online?  If so check for remote 'upness'
          if (navigator.onLine || window.navigator.onLine) {
            switch (rejection.status) {
            // This is for odd IE events.
            case 0:
              $injector.invoke(['connectionErrorService', '$rootScope', function (connectionErrorService, $rootScope) {
                $log.info('Caught a 0 responseError');
                $rootScope.$broadcast('offline');
                connectionErrorService.showConnectionErrorMsg();
              }]);
              break;
            // IE Pre-login issue with resource directory
            case 200:
              $log.info('Caught a 200 responseError');
              break;
            // For the prior login stuff?
            // have all the logs to track down what responses are where.
            // should have global suppression on app.js for logs.
            case 302:
              $log.info('Caught a 302 responseError');
              break;
            // For the prior login stuff?
            case 304:
              $log.info('Caught a 304 responseError');
              break;
            // For Vista Patient Results
            case 400:
              $log.info('Caught a 400 responseError');
              break;
            case 401:
                $log.info('Caught a 401 responseError');
                break;
            // For the prior login stuff?
            case 403:
              $log.info('Caught a 403 responseError');
              break;
            // 409 is duplicate protocol name
            case 409:
                $log.info('Caught a 409 responseError');
                break;
            default:
              // This checks for a status of above 100, which is normal for most browsers.
              // the undefined check, which is oddly for this app only, is for IE 9 offline mode.
              // took away the (|| rejection.status === undefined) check in the if statement per tester
              // that didn't want to see the odd connection message post login due to a bad request.
              if (rejection.status) { 
                newPromise = $injector.invoke(['connectionErrorService', function (connectionErrorService) {
                  $log.info('Caught a non 400 responseError');
                  $log.info(rejection);
                  return connectionErrorService.showServerErrorMsg(rejection);
                }]);
              }
              break;
            }            

          // otherwise show the local connection down modal.
          } else {
            // need to do the invoke to grab the service and fire it.
            $injector.invoke(['connectionErrorService', '$rootScope', function (connectionErrorService, $rootScope) {
              $log.info('Caught a window.onLine false event');
              connectionErrorService.showConnectionErrorMsg();
            }]);
          }


          return newPromise || $q.reject(rejection);
        }
      };
    }
  ]);
  $httpProvider.interceptors.push('httpInterceptor');
}
]);
