'use strict';

angular.module('MessagingAppApp')
    .factory('PatientSearchService', function ($http, $resource, $log, $q) {

  return {

    queryPatient: function(queryStr) {

      // Some constants
      var type;
      var numbers = /^([\d]+[-\d])*$/;
      // back end is expecting an integer representation of the type
      var TypeEnum = {
        NAME: 1,
        SSN: 2,
        MOBILE: 3,
        LAST_NAME_FIRST_INITIAL_SSN_LAST_4: 4
      };

      //
      // process the string before passing it to the back end
      //
      if(/^[a-zA-Z]\d{4}$/.test(queryStr)) {
        // the query is a first initial of last name, space, 4 last digits of SSN
        type = TypeEnum.LAST_NAME_FIRST_INITIAL_SSN_LAST_4;
      } else if( numbers.test(queryStr) ){
        // the query is a phone number
        // test for hyphens and remove
        if( /-/.test(queryStr) ){
          queryStr = queryStr.replace(/-/g,'');
        }

        // test if its size matches a phone number
        if( queryStr.length === 10 || queryStr.length === 11 ){
          queryStr = queryStr.length === 10 ? '1' + queryStr : queryStr;
          type = TypeEnum.MOBILE;
        }
        // test for ssn
        else if( queryStr.length === 9 ){
          type = TypeEnum.SSN;
        }
        else{
          $log.info('Invalid number of digits: ');
          $log.info(queryStr);
          return null;
        }
      } else {
        // else the query is a name
        type = TypeEnum.NAME;

        // The back end is expecting two commas in the name string
        var names = queryStr.split(',');
        var newQuery = '';
        if( names.length > 3) {
          $log.info('Invalid number of commas: ');
          $log.info(queryStr);
          return null;
        } else {
          for (var i=0; i < names.length-1; i++) {
            // We need to remove the spaces in the text before sending to back end
            names[i] = names[i].trim();
            newQuery = newQuery + names[i] + ',';
          }
          // add the last parameter without ','
          names[i] = names[i].trim();
          newQuery = newQuery + names[i];
          queryStr = newQuery;
        }
      }


      var SearchResource = $resource('service/patients/search',
        {type: type, value: queryStr}
      );

      var d = $q.defer();
      SearchResource.query(function(data){
        d.resolve(data);
      });
      return d.promise;

    }
  };

});


