'use strict';
angular.module('MessagingAppApp').factory('ProtocolValidationService', function () {

  var isOnlySpaces = function( str ){
    var regex = /^\s*$/;
    if( regex.test(str) ){
      return true;
    }
    else{
      return false;
    }
  };

  return {
    testOnlySpaces: function( str ){
      return isOnlySpaces(str);
    },
    validateForm: function (protocol) {
      var status = {};
      status.errors = {};
      status.errors.templates = [];
      status.ok = true;

      // This is global function to check empty and undfined object
      var globalCheckEmpty = function (element) {
        // Don't need typeof with triple equals 
        if (element === undefined || element === '') {
          return true;
        } else {
          return false;
        }
      };

      // This is global function to check days
      var globalCheckErrorSchedule = function (i, days) {
        if (days) {
          status.errors.templates[i].schedule.scheduleType.option = { 'errorMsg' : 'The day(s) selected must be within the Template Start Date and Template End Date range'};
          return true;
        }
        else
        {
          return false;
        }
      };

      for (var i = 0; i < protocol.templates.length; i++) {
        status.errors.templates.push({});
        status.errors.templates[i].service = {};
        status.errors.templates[i].schedule = {};
        status.errors.templates[i].schedule.reminderInterval = {};
        status.errors.templates[i].schedule.scheduleType = {};
        status.errors.templates[i].service.readings = [];
        status.errors.templates[i].isScheduleOk = true;
        status.errors.templates[i].isReadingsOk = true;
        status.errors.templates[i].schedule.scheduleType.option = {};
        status.errors.templates[i].startDate = [];
        status.errors.templates[i].endDate = [];

        //Service readings
        // fire error if valid confirmation is empty
        // confirmation message is always required
        if (protocol.templates[i].confirmationMessage !== undefined) {
          var confirmationMessage = protocol.templates[i].confirmationMessage.trim().length === 0;
          if (globalCheckEmpty(protocol.templates[i].confirmationMessage) || (confirmationMessage === true)) {
            status.errors.templates[i].service.conf = {
              'errorMsg' : 'Please enter Message for Valid Reading'
            };
            status.errors.templates[i].isReadingsOk = false;
            status.ok = false;
          }
        }

        if ((protocol.templates[i].service !== undefined ) && ( protocol.templates[i].service.readings !== undefined )) {
          var criticalRequired = false, alertRequired = false, j = 0;

          for (j = 0; j < protocol.templates[i].service.readings.length; j++) {
            status.errors.templates[i].service.readings.push({});
            status.errors.templates[i].service.readings[j].validLow = [];
            status.errors.templates[i].service.readings[j].validHigh = [];
            status.errors.templates[i].service.readings[j].criticalLow = [];
            status.errors.templates[i].service.readings[j].criticalHigh = [];
            status.errors.templates[i].service.readings[j].alertLow = [];
            status.errors.templates[i].service.readings[j].alertHigh = [];

            if ((protocol.templates[i].service.criticalMessage !== undefined && !isOnlySpaces(protocol.templates[i].service.criticalMessage.below) && (!globalCheckEmpty(protocol.templates[i].service.criticalMessage.below) || !globalCheckEmpty(protocol.templates[i].service.criticalMessage.above))) || (protocol.templates[i].service.readings[j].critical !== undefined && ( !globalCheckEmpty(protocol.templates[i].service.readings[j].critical.low) ||  !globalCheckEmpty(protocol.templates[i].service.readings[j].critical.high)))) {
              criticalRequired = true;
            }

            // Set a Flag to check undefined and defined object
            if ((protocol.templates[i].service.firstLevelMessage !== undefined && !isOnlySpaces(globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.below)) && !isOnlySpaces(globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.below)) && (!globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.above) || !globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.above))) || (protocol.templates[i].service.readings[j].alert !== undefined && (!globalCheckEmpty(protocol.templates[i].service.readings[j].alert.low) ||  !globalCheckEmpty(protocol.templates[i].service.readings[j].alert.high)))) {
              alertRequired = true;
            }
          
            // Fire the error message if Lowest valid Reading is empty
            if (globalCheckEmpty(protocol.templates[i].service.readings[j].valid) || globalCheckEmpty(protocol.templates[i].service.readings[j].valid.low)) {
              status.errors.templates[i].service.readings[j].validLow.push(
                {'errorMsg' : 'Please enter a Lowest Valid Reading'}
              );
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }
            
            // Fire the error message if Lowest valid Reading is greater than highest valid reading
            // It throws the error message on Lowest valid and highest valid too.
            if(protocol.templates[i].service.readings[j].valid !== undefined)
            {
              if (parseFloat(protocol.templates[i].service.readings[j].valid.low) >= parseFloat(protocol.templates[i].service.readings[j].valid.high)) {
                status.errors.templates[i].service.readings[j].validLow.push({
                    'errorMsg' : 'Lowest Valid Reading CANNOT be the same value and must be lower than the Highest Valid Reading'
                  });
                status.errors.templates[i].service.readings[j].validHigh.push({
                  'errorMsg' : 'Highest Valid Reading CANNOT be the same value and must be higher than the Lowest Valid Reading'
                });
                status.errors.templates[i].isReadingsOk = false;
                status.ok = false;
              }
            }
           

            if (criticalRequired === true)
            {
              if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
              {
                // Fire the error message if Lowest valid Reading is greater than critical low level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.low) >= parseFloat(protocol.templates[i].service.readings[j].critical.low))
                {
                  status.errors.templates[i].service.readings[j].validLow.push({
                    'errorMsg' : 'Lowest Valid Reading CANNOT be the same value and must be lower than the Critical Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }

                // Fire the error message if Lowest valid Reading is greater than critical high level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.low) >= parseFloat(protocol.templates[i].service.readings[j].critical.high))
                {
                  status.errors.templates[i].service.readings[j].validLow.push({
                    'errorMsg' : 'Lowest Valid Reading CANNOT be the same value and must be lower than the Critical High Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }
            }

            if (alertRequired === true)
            {
              if(protocol.templates[i].service.readings[j].alert !== undefined && protocol.templates[i].service.readings[j].valid !== undefined){
                // Fire the error message if Lowest valid Reading is greater than low level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.low) >= parseFloat(protocol.templates[i].service.readings[j].alert.low))
                {
                  status.errors.templates[i].service.readings[j].validLow.push({
                    'errorMsg' : 'Lowest Valid Reading CANNOT be the same value and must be lower than the Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                
                // Fire the error message if Lowest valid Reading is greater than high level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.low) >= parseFloat(protocol.templates[i].service.readings[j].alert.high))
                {
                  status.errors.templates[i].service.readings[j].validLow.push({
                    'errorMsg' : 'Lowest Valid Reading CANNOT be the same value and must be lower than the High Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }
            }


            // Fire the error message if Highest valid Reading is empty
            if (globalCheckEmpty(protocol.templates[i].service.readings[j].valid) || globalCheckEmpty(protocol.templates[i].service.readings[j].valid.high)) {
              status.errors.templates[i].service.readings[j].validHigh.push({
                'errorMsg' : 'Please enter a Highest Valid Reading'
              });
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }

            if (criticalRequired === true)
            {
              if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
              {
                // Fire the error message if Highest valid Reading is less than critical low level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.high) <= parseFloat(protocol.templates[i].service.readings[j].critical.low)) {
                  status.errors.templates[i].service.readings[j].validHigh.push({
                    'errorMsg' : 'Highest Valid Reading CANNOT be the same value and must be higher than the Critical Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }

                // Fire the error message if Highest valid Reading is less than critical high level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.high) <= parseFloat(protocol.templates[i].service.readings[j].critical.high)) {
                  status.errors.templates[i].service.readings[j].validHigh.push({
                    'errorMsg' : 'Highest Valid Reading CANNOT be the same value and must be higher than the Critical High Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }

            }

            if (alertRequired === true)
            {
              if(protocol.templates[i].service.readings[j].alert !== undefined && protocol.templates[i].service.readings[j].valid !== undefined){
                 // Fire the error message if Highest valid Reading is less than Low level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.high) <= parseFloat(protocol.templates[i].service.readings[j].alert.low)) {
                  status.errors.templates[i].service.readings[j].validHigh.push({
                    'errorMsg' : 'Highest Valid Reading CANNOT be the same value and must be higher than the Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }

                // Fire the error message if Highest valid Reading is less than Low level alert
                if (parseFloat(protocol.templates[i].service.readings[j].valid.high) <= parseFloat(protocol.templates[i].service.readings[j].alert.high)) {
                  status.errors.templates[i].service.readings[j].validHigh.push({
                    'errorMsg' : 'Highest Valid Reading CANNOT be the same value and must be higher than the High Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }
            }

            
          }

          for (j = 0; j < protocol.templates[i].service.readings.length; j++){
            if (criticalRequired === true) {
              // Fire the error message if Critical low and high Level Alert are empty
              if (protocol.templates[i].service.criticalMessage !== undefined && protocol.templates[i].service.readings[j].critical === undefined) {
                status.errors.templates[i].service.readings[j].criticalHigh.push({
                  'errorMsg' : 'Please enter a Critical High Level Alert'
                });
                status.errors.templates[i].service.readings[j].criticalLow.push({
                  'errorMsg' : 'Please enter a Critical Low Level Alert'
                });
                status.errors.templates[i].isReadingsOk = false;
                status.ok = false;
              }

              // Test if there is a message and readings, but not both readings were entered...
              if ( (protocol.templates[i].service.criticalMessage !== undefined && protocol.templates[i].service.readings[j].critical !== undefined) || (protocol.templates[i].service.readings[j].critical !== undefined) ) {

                if( globalCheckEmpty(protocol.templates[i].service.readings[j].critical.high) ){
                  status.errors.templates[i].service.readings[j].criticalHigh.push({
                    'errorMsg' : 'Please enter a Critical High Level Alert'
                  });

                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }

                if( globalCheckEmpty(protocol.templates[i].service.readings[j].critical.low) ){
                  status.errors.templates[i].service.readings[j].criticalLow.push({
                      'errorMsg' : 'Please enter a Critical Low Level Alert'
                    });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }

              if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
              {
                // Fire the error message if Critical low Level Alert is less than lowest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].critical.low) <= parseFloat(protocol.templates[i].service.readings[j].valid.low)) {
                  status.errors.templates[i].service.readings[j].criticalLow.push({'errorMsg' : 'Critical Low Level Alert CANNOT be the same value or lower than the Lowest Valid Reading'});
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                // Fire the error message if Critical low Level Alert is greater than highest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].critical.low) >= parseFloat(protocol.templates[i].service.readings[j].valid.high)) {
                  status.errors.templates[i].service.readings[j].criticalLow.push({'errorMsg' : 'Critical Low Level Alert CANNOT be the same value or higher than the Highest Valid Reading'});
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }

              if(protocol.templates[i].service.readings[j].critical !== undefined)
              {
                // Fire the error message if Critical low Level Alert is greater than critical high level alert
                // It throws the error message on critical low and critical high level alert.
                if (parseFloat(protocol.templates[i].service.readings[j].critical.low) >= parseFloat(protocol.templates[i].service.readings[j].critical.high)) {
                  status.errors.templates[i].service.readings[j].criticalLow.push({
                    'errorMsg' : 'Critical Low Level Alert CANNOT be the same value and must be lower than Critical High Level Alert'
                  });
  
                  status.errors.templates[i].service.readings[j].criticalHigh.push({
                    'errorMsg' : 'Critical High Level Alert CANNOT be the same value and must be higher than Critical Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }

              

              if (alertRequired === true)
              {
                if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
                {
                  // Fire the error message if Critical low Level Alert is greater than Low level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].critical.low) >= parseFloat(protocol.templates[i].service.readings[j].alert.low)) {
                    status.errors.templates[i].service.readings[j].criticalLow.push({
                      'errorMsg' : 'Critical Low Level Alert CANNOT be the same value or higher than the Low Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                  }

                  // Fire the error message if Critical low Level Alert is greater than high level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].critical.low) >= parseFloat(protocol.templates[i].service.readings[j].alert.high)) {
                    status.errors.templates[i].service.readings[j].criticalLow.push({
                      'errorMsg' : 'Critical Low Level Alert CANNOT be the same value or higher than the High Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                }
              }

              if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
              {
                // Fire the error message if Critical High Level Alert is less than lowest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].critical.high) <= parseFloat(protocol.templates[i].service.readings[j].valid.low)) {
                  status.errors.templates[i].service.readings[j].criticalHigh.push({
                    'errorMsg' : 'Critical High Level Alert CANNOT be the same value or higher than the Lowest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                // Fire the error message if Critical High Level Alert is less than highest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].critical.high) >= parseFloat(protocol.templates[i].service.readings[j].valid.high)) {
                  status.errors.templates[i].service.readings[j].criticalHigh.push({
                    'errorMsg' : 'Critical High Level Alert CANNOT be the same value or higher than the Highest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }
              

              if( alertRequired === true){
                if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
                {
                  // Fire the error message if Critical High Level Alert is less than low level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].critical.high) <= parseFloat(protocol.templates[i].service.readings[j].alert.low)) {
                    status.errors.templates[i].service.readings[j].criticalHigh.push({
                      'errorMsg' : 'Critical High Level Alert CANNOT be the same value or higher than the Low Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                  }

                  // Fire the error message if Critical High Level Alert is less than high level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].critical.high) <= parseFloat(protocol.templates[i].service.readings[j].alert.high)) {
                    status.errors.templates[i].service.readings[j].criticalHigh.push({
                      'errorMsg' : 'Critical High Level Alert CANNOT be the same value or lower than the High Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                }

                
              }
              
            }
            
            if (alertRequired === true) {
              // Fire the error message if Low level Alert is empty
              if (protocol.templates[i].service.readings[j].alert === undefined  || globalCheckEmpty(protocol.templates[i].service.readings[j].alert.low)) {
                status.errors.templates[i].service.readings[j].alertLow.push({
                  'errorMsg' : 'Please enter a Low Level Alert'
                });
                status.errors.templates[i].isReadingsOk = false;
                status.ok = false;
              }

              if(protocol.templates[i].service.readings[j].alert !== undefined && protocol.templates[i].service.readings[j].valid !== undefined){
                // Fire the error message if Low level Alert is less than lowest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].alert.low) <= parseFloat(protocol.templates[i].service.readings[j].valid.low)) {
                  status.errors.templates[i].service.readings[j].alertLow.push({
                    'errorMsg' : 'Low Level Alert CANNOT be the same value or lower than the Lowest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                // Fire the error message if Low level Alert is greater than highest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].alert.low) >= parseFloat(protocol.templates[i].service.readings[j].valid.high)) {
                  status.errors.templates[i].service.readings[j].alertLow.push({
                    'errorMsg' : 'Low Level Alert CANNOT be the same value or higher than the Highest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                // Fire the error message if Alert low Level Alert is greater than  high level alert
                // It throws the error message on  low and  high level alert.
                if (parseFloat(protocol.templates[i].service.readings[j].alert.low) >= parseFloat(protocol.templates[i].service.readings[j].alert.high))
                {
                  status.errors.templates[i].service.readings[j].alertLow.push({
                    'errorMsg' : 'Low Level Alert CANNOT be the same value and must be lower than  High Level Alert'
                  });
                  status.errors.templates[i].service.readings[j].alertHigh.push({
                    'errorMsg' : 'High Level Alert CANNOT be the same value and must be higher than Low Level Alert'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              
              }

  
              if (criticalRequired === true) {

                if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
                {
              
                  // Fire the error message if Alert low Level Alert is less than critical low level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].alert.low) <= parseFloat(protocol.templates[i].service.readings[j].critical.low)) {
                    status.errors.templates[i].service.readings[j].alertLow.push({
                      'errorMsg' : 'Low Level Alert CANNOT be the same value or lower than the Critical Low Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                  // Fire the error message if Alert low Level Alert is less than critical high level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].alert.low) >= parseFloat(protocol.templates[i].service.readings[j].critical.high)) {
                    status.errors.templates[i].service.readings[j].alertLow.push({
                      'errorMsg' : 'Low Level Alert CANNOT be the same value or higher than the Critical High Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                }
              }

              // Fire the error message if High Level Alert is empty
              if (protocol.templates[i].service.readings[j].alert === undefined  || globalCheckEmpty(protocol.templates[i].service.readings[j].alert.high)) {
                status.errors.templates[i].service.readings[j].alertHigh.push({
                  'errorMsg' : 'Please enter a High Level Alert'
                });
                status.errors.templates[i].isReadingsOk = false;
                status.ok = false;
              }

              if(protocol.templates[i].service.readings[j].alert !== undefined && protocol.templates[i].service.readings[j].valid !== undefined)
              {                // Fire the error message if High Level Alert is less than lowest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].alert.high) <= parseFloat(protocol.templates[i].service.readings[j].valid.low)) {
                  status.errors.templates[i].service.readings[j].alertHigh.push({
                    'errorMsg' : 'High Level Alert CANNOT be the same value or lower than the Lowest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
                // Fire the error message if High Level Alert is greater than highest valid reading
                if (parseFloat(protocol.templates[i].service.readings[j].alert.high) >= parseFloat(protocol.templates[i].service.readings[j].valid.high)) {
                  status.errors.templates[i].service.readings[j].alertHigh.push({
                    'errorMsg' : 'High Level Alert CANNOT be the same value or higher than the Highest Valid Reading'
                  });
                  status.errors.templates[i].isReadingsOk = false;
                  status.ok = false;
                }
              }
              

              if (criticalRequired === true) {
                if(protocol.templates[i].service.readings[j].critical !== undefined && protocol.templates[i].service.readings[j].valid !== undefined) {
                  // Fire the error message if High Level Alert is greater than critical low level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].alert.high) >= parseFloat(protocol.templates[i].service.readings[j].critical.high)) {
                    status.errors.templates[i].service.readings[j].alertHigh.push({
                      'errorMsg' : 'High Level Alert CANNOT be the same value or higher than the Critical High Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                  // Fire the error message if High Level Alert is less than than critical low level alert
                  if (parseFloat(protocol.templates[i].service.readings[j].alert.high) <= parseFloat(protocol.templates[i].service.readings[j].critical.low)) {
                    status.errors.templates[i].service.readings[j].alertHigh.push({
                      'errorMsg' : 'High Level Alert CANNOT be the same value or lower than the Critical Low Level Alert'
                    });
                    status.errors.templates[i].isReadingsOk = false;
                    status.ok = false;
                  }
                }
              }
            }
          }

          if (criticalRequired === true) {
            var criticalLowMessage, criticalHighMessage;

            if( protocol.templates[i].service.criticalMessage !== undefined && protocol.templates[i].service.criticalMessage.below !== undefined )
            {
              criticalLowMessage = protocol.templates[i].service.criticalMessage.below.trim().length === 0;

            }

            // fire error if critical low level alert message is empty
            if ( (protocol.templates[i].service.criticalMessage === undefined  || globalCheckEmpty(protocol.templates[i].service.criticalMessage.below) ) || ( criticalLowMessage === true)  ) {

              status.errors.templates[i].service.criticalBelowMsg = {
                'errorMsg' : 'Please enter a Message for Critical Low Level Alert'
              };
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }

            if( protocol.templates[i].service.criticalMessage !== undefined &&  protocol.templates[i].service.criticalMessage.above !== undefined )
            {
              criticalHighMessage = protocol.templates[i].service.criticalMessage.above.trim().length === 0;

            }
        
            // fire error if critical high level alert message is empty
            if ( (protocol.templates[i].service.criticalMessage === undefined ||  globalCheckEmpty(protocol.templates[i].service.criticalMessage.above) ) || (criticalHighMessage === true) ) {
              status.errors.templates[i].service.criticalAboveMsg = {
                'errorMsg' : 'Please enter a Message for Critical High Level Alert'
              };
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }
          }

          if (alertRequired === true) {
            var alertLowMessage, alertHighMessage;

            if( protocol.templates[i].service.firstLevelMessage !== undefined && protocol.templates[i].service.firstLevelMessage.below !== undefined )
            {
              alertLowMessage = protocol.templates[i].service.firstLevelMessage.below.trim().length === 0;

            }
            // fire error if low level message is empty
            if ( (protocol.templates[i].service.firstLevelMessage === undefined  || globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.below) ) || (alertLowMessage === true) ) {
              status.errors.templates[i].service.alertBelowMsg = {
                'errorMsg' : 'Please enter a Message for Low Level Alert'
              };
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }

            if( protocol.templates[i].service.firstLevelMessage !== undefined && protocol.templates[i].service.firstLevelMessage.above !== undefined )
            {
              alertHighMessage = protocol.templates[i].service.firstLevelMessage.above.trim().length === 0;

            }

            // fire error if high level message is empty
            if ( (protocol.templates[i].service.firstLevelMessage === undefined ||  globalCheckEmpty(protocol.templates[i].service.firstLevelMessage.above) ) || (alertHighMessage === true) ) {
              status.errors.templates[i].service.alertAboveMsg = {
                'errorMsg' : 'Please enter a Message for High Level Alert'
              };
              status.errors.templates[i].isReadingsOk = false;
              status.ok = false;
            }
          }
        }

        // Start and End dates are always required. Test that the start date is not after the end date and vice versa
        if (protocol.templates[i].startDate !== undefined && protocol.templates[i].endDate !== undefined) {

          // first convert the string dates for comparison, ignore timezone of computer...
          var startWithTimezone = new Date(protocol.templates[i].startDate);
          var startDate = new Date(startWithTimezone.getUTCFullYear(), startWithTimezone.getUTCMonth(), startWithTimezone.getUTCDate());

          var end = new Date(protocol.templates[i].endDate);
          var endDate = new Date(end.getUTCFullYear(), end.getUTCMonth(), end.getUTCDate());

          var now = new Date();
          var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate());

          if(endDate < today){
            status.errors.templates[i].endDate.push({
              'errorMsg' : 'The Template End Date CANNOT be in the past'
            });
            status.ok = false;
            status.errors.templates[i].isScheduleOk = false;
          }

          if (endDate < startDate) {
            status.errors.templates[i].startDate.push({
              'errorMsg' : 'The Template Start Date CANNOT be after the Template End Date'
            });
            status.errors.templates[i].endDate.push({
              'errorMsg' : 'The Template End Date CANNOT be before the Template Start Date'
            });
            status.ok = false;
            status.errors.templates[i].isScheduleOk = false;
          }

          // Test that verifies that specfied days should fall under strat and End date
          if(protocol.templates[i].schedule.scheduleType.name === 'Specified Days'){
            if( protocol.templates[i].schedule.scheduleType.monday !== undefined || protocol.templates[i].schedule.scheduleType.tuesday !== undefined || protocol.templates[i].schedule.scheduleType.wednesday !== undefined || protocol.templates[i].schedule.scheduleType.thursday !== undefined || protocol.templates[i].schedule.scheduleType.friday !== undefined || protocol.templates[i].schedule.scheduleType.saturday !== undefined || protocol.templates[i].schedule.scheduleType.sunday !== undefined )
            {
              var  diff  = new Date(endDate - startDate);
              var days  = diff/1000/60/60/24+1;
              var daySearch = '';

              if (days < 7)
              {
                var start = startDate;
                while(start <= endDate)
                {

                  daySearch = daySearch +  start.getDay() + ',';

                  var newDate = start.setDate(start.getDate() + 1);
                  start = new Date(newDate);
                }

                for ( var k = 0; k < 7; k++ )
                {

                  if (daySearch.search( k + ',' ) ===-1)
                  {
                    switch(k) {
                    case 0:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.monday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 1:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.tuesday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 2:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.wednesday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 3:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.thursday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 4:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.friday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 5:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.saturday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    case 6:
                      if (globalCheckErrorSchedule(i, protocol.templates[i].schedule.scheduleType.sunday))
                      {
                        status.ok = false;
                        status.errors.templates[i].isScheduleOk = false;
                      }
                      break;
                    default:
                      break;
                    }
                  }
                }
              }
            }
          }
        }
      }
      return status;
    }
  };
});