'use strict';

// Service retrieves all readings for a patient
angular.module('MessagingAppApp')
    .factory('PatientsReportsService', function ($http, $resource, $log, ReportFilterService) {

  return {

    // get and parse the patient's readings
    getReadings: function(patientId){

      // the readings array which will contain the parsed readings
      var readings = [];
      readings.data = {};
      var serviceNames = ['Select Reading'];

      // parse date from string function TODO:
      var parseDate = function(date){
        var daytime = date.split(' ');
        var timeParts = daytime[1].split(':');
        var dateParts = daytime[0].split('-');

        // create a UTC date so we can add/subtract the patient's timezone offset
        var d = new Date(Date.UTC(dateParts[0], dateParts[1]-1, dateParts[2], timeParts[0], timeParts[1], timeParts[2]));
        return d.getTime();
      };

      // function to test if object exists
      var objExists = function(name){

        for(var i = 0; i < readings.length; i++) {
          if( readings[i].name.toUpperCase() === name.toUpperCase() ) {
            return i;
          }
        }
        // else, not found
        return -1;
      };

      var Resource = $resource('service/patient/:id/readings', {}, {});

      var patientReadings = Resource.query({id: patientId},
          function(){
            // success
            setupChart();
          },
          function(resp){
            $log.info('PatientsReportsService retrieving patient readings failed.');
            $log.info(resp);
          }
        );

        // After the service call to retrieve the patient's readings, set up the chart
      var setupChart = function(){

        // Get the names of the readings
        angular.forEach(patientReadings, function(s){
          var name = s.patientServiceReading.serviceName;
          if(name && (serviceNames.indexOf(name) === -1)){
            serviceNames.push(name);

          }
        });


        // index for looping through the patient's readings.
        var i = 0;

        while( patientReadings[i] ){

          // get date of last reading
          var lastDate = parseDate(patientReadings[i].patientResponseReceivedTime);

          // Test if the reading has more than one parameter
          var readingsLength = patientReadings[i].patientServiceReading.readings.length;

          if( readingsLength > 1 ){

            for( var j = 0; j < readingsLength; j++ ){

              var readingName1 = patientReadings[i].patientServiceReading.readings[j].name;
              var index1 = objExists(readingName1);

              // test if the reading is already an object in the array so append the data
              if( index1 !== -1 ){
                var newData1 = [lastDate, patientReadings[i].patientServiceReading.readings[j].value];
                readings[index1].data.push(newData1);
              }
              else{
                var newReading1 = {name: readingName1, data: [[lastDate, patientReadings[i].patientServiceReading.readings[j].value]]};
                readings.push(newReading1);
              }
            }
          }
          else{
            // just grab the one reading
            var readingName = patientReadings[i].patientServiceReading.readings[0].name;
            var index = objExists(readingName);

            // test if the reading is already an object in the array so append the data
            if( index !== -1 ){
              var newData = [lastDate, patientReadings[i].patientServiceReading.readings[0].value];
              readings[index].data.push(newData);
            }
            else{
              var newReading = {name: readingName, data: [[lastDate, patientReadings[i].patientServiceReading.readings[0].value]]};
              readings.push(newReading);
            }
          }

          // next reading index
          i++;

        }

      }; // end setupChart()

      return {'readings': readings, 'serviceNames': serviceNames, 'patientReadings': patientReadings};
    }, // getReadings

    getChartConfig: function(){

      return {
        'plotOptions': {
          'line': {
            'cursor': 'pointer',
            'marker': {
              'lineWidth': 1.5
            },
            'lineWidth': 1
          }
        },
        'chart': {
          'alignTicks': true,
          'type': 'spline'
        },
        'options': {
            'rangeSelector': {
              'inputEnabled': true,
              'selected': 0,
              'buttons': []
            },
            'navigator': {
              'enabled': true
            },
            'colors': ['#51324e', '#6e6e6e', '#3d2b3b', '#515151', '#342d3d', '#3e3e3e', '#413451', '#4f4b4b', '#351032', '#3e3e3e']
          },
          'xAxis': {
              'type': 'datetime',
              'minTickInterval': 8640000,
              'title': {
                'text': 'Date/Time',
                'style': {
                  'color': '#51324e'
                }
              },
              'events': {
                'afterSetExtremes': function(event){
                  ReportFilterService.setZoomRange(event);
                },
                'setExtremes': function(e) {
                  if(typeof(e.rangeSelectorButton)!== 'undefined') {
                    //var c = e.rangeSelectorButton.count;
                    var t = e.rangeSelectorButton.type;
                    var event = {};
                    event.min = {};
                    event.min = t;

                    if(t === 'ytd'){
                      ReportFilterService.setZoomRange(event);
                    }
                    else if(t === 'all'){
                      ReportFilterService.setZoomRange(event);
                    }
                  }
                }
              }
            },
            'yAxis': {
              'title': {
                'text': '',
                'style': {
                  'color': '#51324e'
                }
              }
            },
            'tooltip': {
              'crosshairs': true,
              'dateTimeLabelFormats': '%m-%d-%Y %H:%M:%S'
            },
            'stackLabels': {
              'enabled': true,
              'style': {
                'fontWeight': 'bold'
              }
            },
            'useHighStocks': true,
            'series': [{
              'marker' : { enabled : true, radius : 4, lineWidth: 3, lineColor: '#51324e', fillColor: {}},
              'color': '#51324e',
              'data': [[
                0,0
              ]],
              'dataGrouping': {
                'units': [[
                  'week',
                  [1]
                ], [
                  'month',
                  [1, 2, 3, 4, 6]
                ]]
              }
            }],
            'title': {
              'text': '',
              'style': {
                'color': '#51324e'
              }
            },
            'credits': {
              'enabled': false
            }
          };
    }// getChartConfig()
  };
});

// Service returns an object for a particular reading
angular.module('MessagingAppApp')
    .factory('ReportService', function ($http, $resource) {

    return $resource('service/patient/:id/readings/:serviceId', {}, {
    });
  });

// Service returns an object for a particular reading
angular.module('MessagingAppApp')
    .factory('ReportFilterService', function ($rootScope) {

    return {
      setZoomRange: function(event){
        $rootScope.$broadcast('chartRangeChanged', [event.min, event.max]);
      }
    };
  });