'use strict';

angular.module('MessagingAppApp').factory('SessionTimeoutService', [
  '$http',
  '$resource',
  'ResourceDirectoryService',
  'AngularShamNotification',
  '$modal',
  '$timeout',
  '$location',
  '$log',
  function ($http, $resource, ResourceDirectoryService, angularShamNotification, $modal, $timeout, $location, $log) {
    var service = {},
      timeToExpire,
      timeUnitsText = 'minutes',
      timeUnitText = 'minute',
      displayTimeUnits = 60000, // one minute. $timeout requires milliseconds
      warningThreshold = 3 * displayTimeUnits,
      conversionFactor = 1000,
      sessionEnded = false,
      timeoutPromise,
      modalInstance,
      sessionInfo = {
        value: null,
        dismissed: true
      },
      timeoutDialogController,
      dismissModal,
      openModal,
      showTimeoutWarning;

    timeoutDialogController = function () {
      var waitTime,
        lastAccessedResource = $resource(ResourceDirectoryService.getLastAccessedHref(), {}, {});

      // disable spinner -do not reset automatically
      angularShamNotification.setDisabled(true, -1);

      // From ResourceDirectoryService
      // Not exactly sure what that does.
      // This starts the countdown though.
      lastAccessedResource.get({}, function (result) {
        timeToExpire = result.timeToExpireInSeconds * conversionFactor;
        if (timeToExpire <= warningThreshold) {
          showTimeoutWarning(timeToExpire);
          waitTime = displayTimeUnits;
        } else {
          dismissModal();
          waitTime = timeToExpire - warningThreshold;
        }

        if (!sessionEnded) {
          $timeout.cancel(timeoutPromise);
          timeoutPromise = $timeout(function () {
            timeoutDialogController();
          }, waitTime, false);
        }
        // disable spinner -do not reset automatically
        angularShamNotification.setDisabled(false);
      });
    };

    dismissModal = function () {
      if (angular.isDefined(modalInstance) && !sessionInfo.dismissed) {
        sessionInfo.dismissed = true;
        modalInstance.dismiss();
      }
    };

    openModal = function (template, controller, sessionInfo) {
      dismissModal();
      // This seems like the place to put the check for offline, but in another module??
      if (navigator.onLine) {
        modalInstance = $modal.open({
          templateUrl: template,
          controller: controller,
          backdrop: 'static',
          keyboard: false,
          resolve: {
            sessionInfo: function () {
              return sessionInfo;
            }
          }
        });
      } else {
        // Have a log statement here as a placeholder for better integration point for offline session.
        $log.info('offline and session fired');
      }
    };

    showTimeoutWarning = function (timeToExpire) {
      var minutesToExpire = Math.round(parseInt(timeToExpire, 10) / displayTimeUnits);
      if (minutesToExpire <= 0) {
        sessionEnded = true;
        $location.path('/emptyAnniePage');
        openModal('views/sessionEnded.tpl.html', 'SessionModalInstanceCtrl', sessionInfo);
      } else {
        sessionInfo.value = minutesToExpire;
        sessionInfo.unit = sessionInfo.value !== 1 ? timeUnitsText : timeUnitText;
        openModal('views/sessionEnding.tpl.html', 'SessionModalInstanceCtrl', sessionInfo);
      }
    };
    service.run = function () {
      timeoutDialogController();
    };
    return service;
  }
]);

angular.module('MessagingAppApp').controller('SessionModalInstanceCtrl', [
  '$scope',
  '$modalInstance',
  'sessionInfo',
  '$resource',
  '$http',
  'SessionTimeoutService',
  'userService',
  'userSessionService',
  function ($scope, $modalInstance, sessionInfo, $resource, $http, SessionTimeoutService, userService, userSessionService) {
    // set dismissed to false because we just opened the modal hence it hasn't yet been dismissed
    sessionInfo.dismissed = false;

    $scope.logout = function () {
      $modalInstance.dismiss();
      // userService logout method also clears the session storage
      userService.logout();
    };
    $scope.login = function () {
      $scope.logout();
    };
    $scope.continues = function () {
      $modalInstance.dismiss();

      userSessionService.continueSession(function () {
        // now, reset the wait time
        SessionTimeoutService.run();
      });
      sessionInfo.dismissed = true;

    };
    $scope.sessionInfo = sessionInfo;
  }
]);
