'use strict';

describe('PatientListCtrl', function () {
    var scope = null,
        ctrl = null,
        httpBackend = null,
        mockService,
        $controller,
        baseUrl,
        regex,
        patientList = [{
            "firstName": "John",
            "lastName": "Smith",
            "age":47,
            "dob": "1966-01-01",
            "preference": {
                "phoneNumber": "1-703-111-2222"
            },
            "_id": "528a739eda06d04179851fa9"
        }, {
            "firstName": "Mary",
            "lastName": "Jones",
            "age": 64,
            "dob": "1949-05-01",
            "preference": {
                "phoneNumber": "1-703-222-3333"
            },
            "_id": "528a739eda06d04179852357"
        }, {
            "firstName": "Bob",
            "lastName": "Johnson",
            "age": 33,
            "dob": "1980-07-04",
            "preference": {
                "phoneNumber": "1-703-333-4444"
            },
            "_id": "528a739eda06d0417985698a"
        }];
    
    //need to indicate your module in a test
    beforeEach(module('MessagingAppApp'));

    mockService = {
        query: function (x) {
            return 'ok';
        }
    }

    beforeEach(inject(function ($injector, _$httpBackend_, $rootScope, $controller) {
        scope = $injector.get('$rootScope');
        $controller = $injector.get('$controller');
        //httpBackend = $injector.get('$httpBackend');
        httpBackend = _$httpBackend_;
        ctrl = $controller('PatientListCtrl', {
            '$scope': scope,
            PatientListService: mockService,
            resolvedPatients: mockService
        });
        scope = $injector.get('$rootScope');

        //set response for resource-directory, use regexp to work around the noCache param
        baseUrl = "service/resource-directory/tokenurls";
        regex = new RegExp(baseUrl + "*", '');
        httpBackend.when('GET', regex).respond(204);

        // For the new templateCaching
        httpBackend.when('GET', 'views/patientProtocol.tpl.html').respond(204);
        httpBackend.when('GET', 'views/protocolTemplate.tpl.html').respond(204);
        httpBackend.when('GET', 'views/templateDescription.tpl.html').respond(204);
        httpBackend.when('GET', 'views/templateMeasurement.tpl.html').respond(204);
        httpBackend.when('GET', 'views/templateScheduleSettings.tpl.html').respond(204);
        httpBackend.when('GET', 'views/templateListSidebar.tpl.html').respond(204);
        httpBackend.when('GET', 'views/templateMessages.tpl.html').respond(204);
        httpBackend.when('GET', 'views/patientScheduleSettings.tpl.html').respond(204);
        httpBackend.when('GET', 'views/patientCalendar.tpl.html').respond(204);
        httpBackend.when('GET', 'views/patientMeasurement.tpl.html').respond(204);
        httpBackend.when('GET', 'views/patientMessages.tpl.html').respond(204);
        httpBackend.when('GET', 'views/protocolStatus.tpl.html').respond(204);
        httpBackend.when('GET', 'views/patientSearch.tpl.html').respond(204);
        httpBackend.when('GET', 'scripts/modules/connection/connection_error.tmpl.html').respond(204);
        httpBackend.when('GET', 'scripts/modules/connection/connection_server.tmpl.html').respond(204);
    }));

    afterEach(function () {
        httpBackend.verifyNoOutstandingExpectation();
    });

    it('should create the controller', function () {
        expect(ctrl).not.toBeNull();
        expect(scope).not.toBeNull();
    });

    it('should verify initial set up of controller is performed', function () {
        expect(scope.ascending).toEqual(true);
    });

    it('should populate the controllers patient list variable when changed', function () {
        scope.patientList = patientList;
        //$apply the change to trigger the $watch.
        scope.$apply();
        //assert
        expect(scope.patientList).toEqual(patientList);
        //console.log("patientList = ");  console.log(scope.patientList[0]);
    });

    it('should test the current patient ID is returned', function () {
        scope.patientList = patientList;
        //$apply the change to trigger the $watch.
        scope.$apply();
        //assert
        expect(scope.getPatientId(scope.patientList[0])).toEqual('528a739eda06d04179851fa9');
        expect(scope.getPatientId(scope.patientList[1])).toEqual('528a739eda06d04179852357');
        expect(scope.getPatientId(scope.patientList[2])).toEqual('528a739eda06d0417985698a');
        //console.log(scope.getPatientId(scope.patientList[0]));
    });

    it('should test setting the page number for pagination function', function () {
        scope.setPage(3);
        //$apply the change to trigger the $watch.
        scope.$apply();
        //assert
        expect(scope.currentPage).toEqual(3);
        //console.log(scope.currentPage);
    });

});
