'use strict';

describe('PatientProfileCtrl Unit Test', function () {
    var x,
        $timeout,
        scope = null,
        ctrl = null,
        httpBackend = null,
        protocolId = null,
        baseUrl = "service/resource-directory/tokenurls",
        regex = new RegExp(baseUrl + "*", ''),
        mockResolvedTemplate = {
            'templates': [{}]
        },
        mockMeasurementDictionaryService = {
            get: function (x) {
                return 'ok';
            }
        },
        mockPatientService = {
            getCurrPatient: function (x) {
                return 'ok';
            },
            getPatientToRegister: function (x) {
                return 'ok';
            }
        },
        mockPatientRegistrationService = {
            update: function (x) {
                return 'ok';
            }
        },
        patientRegistrationSaveService,
        mockSaveErrorReturn = {
            "data": {
                "code": 1004,
                "message": "Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX."
            }
        },
        profileForm = {
            '$dirty'             : true,
            '$error'             : {},
            'invalid'            : false,
            '$name'              : 'profileForm',
            '$pristine'          : false,
            '$valid'             : true,
            'firstName'          : {},
            'middlename'         : {},
            'lastName'           : {},
            'ssn'                : {},
            'dob'                : {},
            'gender'             : {},
            'mobileNumber'       : {},
            'timezone'           : {},
            'homePhone'          : {},
            'Email'              : {},
            'phonePref'          : {},
            'profile-phone-pref' : {},
            'preference'         : {},
            'consentForm'        : {}
        },
        profile = {
            '_id'              : '52b9f1519d16ccbfab2fcff4',
            'firstName'        : 'zztest',
            'middleName'       : null,
            'lastName'         : 'patient01',
            'dob'              : '05/05/1900',
            'ssn'              : '999-99-9901',
            'gender'           : 'male',
            'patientExistInCms': true,
            'preference':  { 
                'messageStarted'     : true,
                'messageStopped'     : false,
                'messagePaused'      : false,
                'phoneNumber'        : 1571P0O0R0T,
                'homePhoneNumber'    : null,
                'messageDeliveryPreference': { 'sendSms' : true },
                'email'              : null,
                'timeZone'           : 'America/Chicago',
                'phoneType'          : 'smart',
                'notificationDeliveryPreference': { 'sendSms' : false }
            },
            'patientIdentifier':  { 
                'uniqueId'          : 'D123401',
                'assigningAuthority': 'EDIPI'
            },
            'lastNameInitial': 'P',
            'last4SSN'       : 9901,
            'consent'        : true,
            'consentDate'    : '2014-04-30',
            'age'            : 114
        };

    beforeEach(function ($provide) {
        // We load all the module dependancies up front.
        module('MessagingAppApp');

        //Use the angular $provide service to mock any dependancies;
        module(function ($provide) {
            $provide.value('resolvedTemplate', mockResolvedTemplate);
            $provide.value('MeasurementDictionaryService', mockMeasurementDictionaryService);
            $provide.value('patientService', mockPatientService);
            $provide.value('patientRegistrationService', mockPatientRegistrationService);
        });

        inject(function ($injector, _$httpBackend_, $rootScope, $controller, _$log_, _$routeParams_, _$filter_, _$timeout_, _$location_, _$modal_, _patientRegistrationSaveService_) {

            httpBackend = _$httpBackend_;
            patientRegistrationSaveService = _patientRegistrationSaveService_;

            httpBackend.when('GET', regex).respond(204);
            // For the new templateCaching
            httpBackend.when('GET', 'views/patientProtocol.tpl.html').respond(204);
            httpBackend.when('GET', 'views/protocolTemplate.tpl.html').respond(204);
            httpBackend.when('GET', 'views/templateDescription.tpl.html').respond(204);
            httpBackend.when('GET', 'views/templateMeasurement.tpl.html').respond(204);
            httpBackend.when('GET', 'views/templateScheduleSettings.tpl.html').respond(204);
            httpBackend.when('GET', 'views/templateListSidebar.tpl.html').respond(204);
            httpBackend.when('GET', 'views/templateMessages.tpl.html').respond(204);
            httpBackend.when('GET', 'views/patientScheduleSettings.tpl.html').respond(204);
            httpBackend.when('GET', 'views/patientCalendar.tpl.html').respond(204);
            httpBackend.when('GET', 'views/patientMeasurement.tpl.html').respond(204);
            httpBackend.when('GET', 'views/patientMessages.tpl.html').respond(204);
            httpBackend.when('GET', 'views/protocolStatus.tpl.html').respond(204);
            httpBackend.when('GET', 'views/patientSearch.tpl.html').respond(204);

            httpBackend.whenPUT('service/patients').respond(401, mockSaveErrorReturn);

            scope = $rootScope.$new();
            scope.profileForm = profileForm;
            scope.profile = profile;
            scope.profile.preference = profile.preference;
            ctrl = $controller('PatientProfileCtrl', {
                $scope: scope,
                $log: _$log_,
                $routeParams: _$routeParams_,
                $filter: _$filter_,
                $timeout: _$timeout_,
                $location: _$location_,
                $modal: _$modal_,
                patientRegistrationSaveService: _patientRegistrationSaveService_
            });
        });
    });

//    it('should allow updating of profile', function () {
//        expect(scope.updateProfile).not.toBeUndefined();
//        scope.profile = profile;
//        scope.updateProfile();
//        expect(scope.profile.preference.phoneNumber).toContain(scope.profile.preference.phoneNumber);
//        expect(scope.profileForm.submitted).toBe(true);
//    });
//    it('should hit the patientRegistrationSaveService and error', function () {
//        scope.profile = profile;
//        patientRegistrationSaveService.update(scope.profile);
//        scope.updateProfile();
//        expect(scope.updateResponse).not.toBeUndefined();
//        expect(scope.isDuplicatePhoneNumber).toBe(undefined);
//    });
    // this doesn't work becuase it's a private function. Doesn't need to be.
    // it('should set a profile', function () {
    //     scope.profile = profile;
    //     this.setProfile(profile);
    //     expect(scope.profile.gender).toBe('male');
    //     expect(scope.profile.dob).toBe('05/05/1900');
    // });
});
