'use strict';

describe('ProtocolTemplateCtrl', function () {
    var scope = null,
        ctrl = null,
        httpBackend = null,
        protocolId = null,
        routeParams = {},
        dictionary = {
            'services': [{
                '_id' : '52e829f9bf890cf9ee685d85',
                'name': 'Blood Pressure',
                'readings': [{
                    'name': 'Systolic',
                    'valid': {
                        'low' : 70,
                        'high': 262
                    },
                    'alert': {
                        'low' : 80,
                        'high': 140
                    },
                    'unit': 'mmHg'
                }, {
                    'name': 'Diastolic',
                    'valid': {
                        'low' : 50,
                        'high': 120
                    },
                    'alert': {
                        'low'  : 60,
                        'high' : 90
                    },
                    'unit': 'mmHg'
                }],
                'pattern': ['(?i)\\s*(bp|blood)\\s+[0-9]{2,3}/[0-9]{2,3}\\s*']
            }],
            'schedule': {
                'acceptUnscheduledReading': false,
                'scheduleType': [{
                    '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce',
                    'name'  : 'Once'
                }, {
                    '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany',
                    'name': 'Custom',
                    'interval': {
                        'value': 0,
                        'unit' : 'MINUTE'
                    }
                }, {
                    '@class': 'gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek',
                    'name' : 'Specified Days',
                    'scheduleDayOfWeek': {
                        'monday'   : false,
                        'tuesday'  : false,
                        'wednesday': false,
                        'thursday' : false,
                        'friday'   : false,
                        'saturday' : false,
                        'sunday'   : false
                    }
                }]
            }
        },
        protocol = {
            'templates': [{
                'confirmationMessage': 'Test - should not cause failure',
                '@class': 'gov.va.mobilehealth.courier.core.protocol.Vital',
                'schedule': {
                    'scheduleType': {
                        'interval': {
                            'name': 'Specified Days'
                        }
                    },
                    'service': {
                        'criticalMessage': {
                            'below': '10'
                        },
                        'readings': [{
                            'valid': {
                                'low': '5'
                            },
                            'alert': {
                                'low': '10'
                            }
                        }]
                    },
                    'scheduleTime': {
                        'min': null
                    },
                    'relativeStartTime':{},
                    'relativeEndTime':{},
                    'acceptUnscheduledReading':true
                },
                'service': {
                    'criticalMessage': {},
                    'readings': []
                },
            }],
            'length'     : 1,
            'name'       : 'asdf',
            'description': 'asdf',
            'area'       : 'asdf'
        },
        templates = [{
            'service':{},
            'schedule':{
                'conf': {},
                'scheduleType':{
                    'interval': {}
                },
                'scheduleTime': {
                    'min':null
                },
                'relativeStartTime':{},
                'relativeEndTime':{},
                'acceptUnscheduledReading':true
            },
            'acceptUnscheduledReading':true,
            '@class':'gov.va.mobilehealth.courier.core.protocol.Vital','type':'Vitals',
            'isScheduleCollapsed':true,
            'isReadingsCollapsed':true,
            'isMessageCollapsed':true,
            'useReadingsNgErrorFlag':false,
            'useMessageNgErrorFlag':false,
            'useScheduleNgErrorFlag':false,
            'useNgErrorFlag':false
        }],
        protocolForm = { 
            '$dirty'                : true,
            '$error'                : [{
                'checkboxGroup-minRequired': {},
                '$required'         : {}
            }],
            'invalid'               : true,
            '$name'                 : 'protocolForm',
            '$pristine'             : false,
            '$valid'                : false,

            'scheduleForm0':  { 
                'scheduleType0'     : {
                    'selected'      : 0,
                    '$dirty'        : true,
                    '$modelValue'   : 'Specified Days',
                    '$valid'        : false
                },
                'startTimeHours0'   : { },
                'startTimeMinutes0' : { },
                'startTimeMeridiem0': { },
                'enterStartPeriod0' : { },
                'selectStartPeriod0': { },
                'enterEndPeriod0'   : { },
                'selectEndPeriod0'  : { },
                'unscheduled0'      : { },
            } ,
            'readingsForm0':  { 
                'measurement0': { }
            }, 
            'messageForm0':  { 
                'serviceMessage0'      : { },
                'reminderMessage0'     : { },
                'timeBetweenReminders0': { },
                'reminderTimeSchedule0': { },
                'timesReminderMessage0': { }
            } 
        },
        ProtocolValidationService,
        patientService = null,
        mockPatientService = null,
        mockService = {
            get: function (x) {
                return 'ok';
            }
        },
        mockResolvedTemplate = {
            'templates': [{}]
        },
        mockMeasurementDictionaryService = {
            get: function (x) {
                return 'ok';
            }
        },
        mockNewProtocolTemplateService = {
            update: function (x) {
                return 'ok';
            }
        };

    beforeEach(function ($provide) {
        // We load all the module dependancies up front.
        module('MessagingAppApp');

        //Use the angular $provide service to mock any dependancies;
        module(function ($provide) {
            $provide.value('resolvedTemplate', mockResolvedTemplate);
            $provide.value('MeasurementDictionaryService', mockMeasurementDictionaryService);
            $provide.value('NewProtocolTemplateService', mockNewProtocolTemplateService);
            $provide.value('patientService', mockPatientService);
        });

        inject(function ($injector, $httpBackend, $rootScope, $controller, _ProtocolValidationService_) {
            // var $controller = $injector.get('$controller');
            // httpBackend = $injector.get('$httpBackend');
            // scope = $injector.get('$rootScope');
            scope = $rootScope.$new();
            ProtocolValidationService = _ProtocolValidationService_;

            ctrl = $controller('ProtocolTemplateCtrl', {
                ProtocolListService         : mockService,
                patientService              : mockPatientService,
                $scope                      : scope,
                $routeParams                : routeParams,
                ProtocolValidationService   : ProtocolValidationService
            });
        });
    });

    it('should have base functions', function () {
        expect(scope.setTemplatesData).not.toBeUndefined();
    });


    it('should have save the form', inject(['ProtocolValidationService', function () {
        scope.templates = templates;
        scope.protocolForm = protocolForm;
        scope.protocol = protocol;
        scope.formSubmitted = false;
        scope.formStatus.ok = false;

        // Making this null will cause an error
        scope.protocol.templates[0].confirmationMessage = '';

        scope.formStatus = ProtocolValidationService.validateForm(scope.protocol);
        expect(scope.formStatus.ok).toEqual(false);

        // Now proving the service will turn the okay back on.
        scope.protocol.templates[0].confirmationMessage = 'Test - this should not cause an error';

        scope.formStatus = ProtocolValidationService.validateForm(scope.protocol);
        expect(scope.formStatus.ok).toEqual(true);
    }]));

    it('should have Fire the error message if Critical low Level Alert is empty', inject(['ProtocolValidationService', function () {
        scope.templates = templates;
        scope.protocolForm = protocolForm;
        scope.protocol = protocol;
        scope.formSubmitted = false;
        scope.formStatus.ok = false;
        // This causes the form to fire error. 
        // Sets a validHigh message with no High amount.
        scope.protocol.templates[0].service = {
            'criticalMessage': {
                'below': '10'
            },
            'readings': [{
                'valid': {
                    'low': '5'
                },
                'alert': {
                    'low': '10'
                }
            }]
        };
        // Test the service.
        scope.formStatus = ProtocolValidationService.validateForm(scope.protocol);
        expect(scope.formStatus.ok).toEqual(false);
        expect(scope.formStatus.errors.templates[0].service.readings[0].validHigh[0].errorMsg).toEqual('Please enter a Highest Valid Reading');

        // Now Fixing all the validation errors.
        scope.protocol.templates[0].service = {
            'criticalMessage': {
                'above': '30',
                'below': '10'
            },
            'readings': [{
                'critical': {
                    'low': '10',
                    'high': '20'
                },
                'valid': {
                    'low': '5',
                    'high': '25'
                },
                'alert': {
                    'low': '12',
                    'high': '18'
                }
            }],
            'firstLevelMessage': {
                'above': 'asdf',
                'below': 'asdf'
            }
        };
        // Test the service.
        scope.formStatus = {};
        scope.formStatus = ProtocolValidationService.validateForm(scope.protocol);
        expect(scope.formStatus.ok).toEqual(true);
    }]));
});
