'use strict';

angular.module('PatientApp')
    .controller('ProfileController', function ($scope, $rootScope, $timeout, ProfileService, profileHelperService, $log,
                                               $location, $filter, dictionaryService, userProfileService, $modal, facilityService, $window) {

    // default for accordions is collapsed.
    $scope.apptCollapsed = true;
    $scope.preferencesCollapsed = true;
    $scope.patientInfoCollapsed = true;
    $scope.patientConsent = false;

    $scope.timezone = [];

    //Clear the form when leaving the page via any link
    $scope.$on('$locationChangeStart', function () {
      profileHelperService.resetCurrProfile();
    });

    $scope.gender = [
      { name: 'Female', value: 'FEMALE' },
      { name: 'Male', value: 'MALE' }
    ];

    // Cancel out of the edit page and return to the template list
    $scope.cancelEdit = function () {
      profileHelperService.resetCurrProfile();
      $scope.phoneFormatStatus = '';
      $location.path('/success');
    };

    // set the flag that capture from the parent scope
    $rootScope.$on('patientConsented', function () {
        $scope.patientConsent = true;

        // Set the consent time now.
        var now = new Date();
        var meridiem = now.getHours() < 12 ? 'AM' : 'PM';
        var hours = now.getHours() % 12 || 12;
        var min = now.getMinutes() < 10 ? '0' + now.getMinutes() : now.getMinutes();

        $scope.profile.consentTime = hours + ':' + min + ' ' + meridiem;
      }
    );

    // set the flag that capture from the parent scope
    $rootScope.$on('patientNotConsented', function () {
        $scope.patientConsent = false;
      }
    );

    var timeZoneResponse = dictionaryService.get(
      function () {
        setTimeZone(timeZoneResponse);
      }
    );

    function setTimeZone(time) {
      $scope.timezone = time.timeZones;
      //$log.info('Timezone object');$log.info($scope.timezone);
    }

    // Flag for Facility Required dropdown
    $scope.facilityIsRequired = false;

    $scope.validationFacility = function () {
      if ($scope.profile.preference.receiveFacilityMessage === true) {
        $scope.facilityIsRequired = true;
      } else {
        $scope.facilityIsRequired = false;
        $scope.profile.facilityId = null;
      }
    };

    // Get all the dynamic list of facility from backend.
    facilityService.get().$promise.then(function (msg) {
        $scope.facilityList = msg;
      }, function () {
        $scope.facilityList = null;
      });


    // Display a pre-propulated on the registeration page form HS
    var userProfile = userProfileService.query(
      function () {
        $scope.profile = userProfile;
        setProfile($scope.profile);
      },
      function () {
        $scope.profile = null;
      });

    function setProfile(profile) {
        $scope.profile = profile;
        $scope.profile.gender = $filter('lowercase')(profile.gender);
        $scope.profile.dob = $filter('date')(profile.dob, 'MM/dd/yyyy');
        
        if (profile.consent >= 1) {
          $scope.patientConsent = true;
        }

        if (profile.preference !== null) {

          // check the delivery preference radio button
          if( $scope.profile.patientExistInCms === true ) {
            $scope.msgDeliveryPreference = $scope.profile.preference.messageDeliveryPreference.sendSms === true ? 'sendSms' : 'sendAnnie';
          }

          var homeNumber = profile.preference.homePhoneNumber;
          if (homeNumber !== null) {
            var homeNumberProfile = homeNumber.substr(1);
            $scope.homePhoneNumber = homeNumberProfile;
          }
          else {
            $scope.homePhoneNumber = homeNumber;
          }

          var mobileNumber = profile.preference.phoneNumber;
          if (mobileNumber !== null) {
            var mobileNumberProfile = mobileNumber.substr(1);
            $scope.mobileNumber = mobileNumberProfile;
          }
          else {
            $scope.mobileNumber = homeNumber;
          }
        }
      }

    // a test to see if there is an error on the form, and if so, open the appropriate accordions
    var testFormAccordions = function(){
      // per acceptance criteria, open the accordion if the facility was required
      if($scope.profileForm.facility !== undefined && $scope.profileForm.facility.$invalid === true) {
        $scope.apptCollapsed = false;
      }

      if($scope.profileForm.profilePhonePref !== undefined && $scope.profileForm.profilePhonePref.$invalid) {
        $scope.preferencesCollapsed = false;
      }

      // this condition has to be on one line because of jshint error....
      if($scope.isDuplicatePhoneNumber === true || ($scope.profileForm.mobileNumber !== undefined && $scope.profileForm.mobileNumber.$invalid) || ($scope.profileForm.timezone !== undefined && $scope.profileForm.timezone.$invalid) || ($scope.profileForm.Email !== undefined && $scope.profileForm.Email.$invalid)) {
        $scope.patientInfoCollapsed = false;
      }
    };


    // TODO: do we really need s saveProfile AND updateProfile??? see staff app...
    $scope.saveProfile = function () {

      $scope.profileForm.submitted = true;

      if ($scope.profileForm.$valid) {
        if (($scope.patientConsent === true) && ($scope.profile.consent === 0)) {
          $scope.profile.consent = 1;
        }

        // Format Home Phone Number
        var homeNumber = {};
        homeNumber = $scope.homePhoneNumber;

        // set message delivery preference
        $scope.profile.preference.messageDeliveryPreference.sendSms = $scope.msgDeliveryPreference === 'sendSms' ? true : false ;

        if (typeof homeNumber === 'object' && homeNumber === null) {
          $scope.profile.preference.homePhoneNumber = homeNumber;
        }
        else if (homeNumber !== undefined) {
          $scope.profile.preference.homePhoneNumber = '1' + homeNumber;
        }

        // Format Mobile Phone Number
        var mobileNumber = $scope.mobileNumber;

        $scope.profile.preference.phoneNumber = '1' + mobileNumber;

        // This actuall submits the form, via a service.
        $scope.updateResponse = ProfileService.save($scope.profile, function () {
            // set profilestatus in helper...
            var userProfile = userProfileService.query(
              // success
              function () {
                // clean up hackish success and phone numbers.
                $scope.isDuplicatePhoneNumber = null;
                $scope.theDuplicatePhoneNumber = null;
                $rootScope.userInfo = userProfile;
              },
              // error
              function () {
                $rootScope.userInfo = null;
              });

            $location.path('/success');
          }, function (error) {
            // error returned
            $log.info(error);
            // set phone number error flag. parse other error codes. see error.configuration file in back end.
            // Code 1004 is passed back for duplicate Phone Number
            if (error.data.code === 1004) {
              $log.info('error 1004');
              // hack, should be part of global system, but since it's after form post we can't alter form validity
              // can't add into message scope; as that's for success only.
              $scope.isDuplicatePhoneNumber = true;
              // Storing the old number to if you zero out the old input and type in the duplicate we'd put the message back.
              $scope.theDuplicatePhoneNumber = $scope.mobileNumber;
            }
          }
        );
      }
      else {
        testFormAccordions();
      } // end else
    };

    $scope.updateProfile = function () {

      $scope.profileForm.submitted = true;

      if ($scope.profileForm.$valid) {

        // set message delivery preference
        $scope.profile.preference.messageDeliveryPreference.sendSms = $scope.msgDeliveryPreference === 'sendSms' ? true : false ;

          // Format Home Phone Number
        var homeNumber = {},
          mobileNumber;
        homeNumber = $scope.homePhoneNumber;
        if (typeof homeNumber === 'object' && homeNumber === null) {
          $scope.profile.preference.homePhoneNumber = homeNumber;
        } else if (homeNumber !== undefined) {
          $scope.profile.preference.homePhoneNumber = '1' + homeNumber;
        }

        // Format Mobile Phone Number
        mobileNumber = $scope.mobileNumber;
        $scope.profile.preference.phoneNumber = '1' + mobileNumber;

        // This actuall submits the form, via a service.
        $scope.updateResponse = ProfileService.save($scope.profile, function () {
          // success
          // scroll to top of page
          $window.scrollTo(0,0);

          $scope.messages = 'Your changes are saved.';
          // clean up hackish success and phone numbers.
          $scope.isDuplicatePhoneNumber = null;
          $scope.theDuplicatePhoneNumber = null;
          $scope.duplicatePhoneNumberMessage = null;
          // clears success after 10 seconds.
          $timeout(function () {
            $scope.messages = null;
          }, 10000);
        }, function (error) {
          // error handler
          // Code 1004 is passed back for duplicate Phone Number
          if (error.data.code === 1004) {
            $log.info('error 1004');
            // hack, should be part of global system, but since it's after form post we can't alter form validitiy
            // can't add into message scope; as that's for success only.
            $scope.isDuplicatePhoneNumber = true;
            $scope.theDuplicatePhoneNumber = $scope.mobileNumber;
            $scope.duplicatePhoneNumberMessage = error.data.message;
            //'Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX.';
          }
        });
      }
      else {
        testFormAccordions();
      } // end else
    };

    $scope.open = function () {

      var modalInstance = $modal.open({
        templateUrl: 'views/infoAndDisclosureForm.tpl.html',
        controller: 'consentModalCtrl',
        resolve: {
          notConsented: function () {
            return !$scope.patientConsent || $scope.profile.consent===0;
          }
        }
      });

      modalInstance.result.then(function () {
      });
    };

  }).directive('placeholder', function ($timeout) {
    var i = document.createElement('input');
    if ('placeholder' in i) {
      return {};
    }
    return {
      link: function (scope, elm, attrs) {
        if (attrs.type === 'password') {
          return;
        }
        $timeout(function () {
          elm.val(attrs.placeholder);
          elm.bind('focus', function () {
            if (elm.val() === attrs.placeholder) {
              elm.val('');
            }
          }).bind('blur', function () {
            if (elm.val() === '') {
              elm.val(attrs.placeholder);
            }
          });
        });
      }
    };
  }).directive('ngBlur', function () {
    return {
      restrict: 'A',
      require: 'ngModel',
      link: function (scope, el, attrs, ctrl) {
        var BLUR_CLASS = 'ng-blurred';
        ctrl.$blurred = false;
        el.on('keydown', function () {
          el.removeClass(BLUR_CLASS);
          scope.$apply(function () {
            ctrl.$blurred = false;
          });
        });
        el.on('blur', function () {
          el.addClass(BLUR_CLASS);
          scope.$apply(function () {
            ctrl.$blurred = true;
          });
        });
      }
    };
  });

angular.module('PatientApp').controller('consentModalCtrl', function($scope, $modalInstance, $rootScope, notConsented){

  $scope.notConsented = notConsented;
  $scope.scrolledBottom = false;

  $scope.closeConsent = function(){
    $modalInstance.dismiss('cancel');
    $rootScope.$broadcast('patientConsented');
  };

  $scope.notConsent = function () {
    $modalInstance.dismiss('cancel');
    $rootScope.$broadcast('patientNotConsented');
  };

  $rootScope.$on('scrolledToBottom', function () {
    $scope.scrolledBottom = true;
  });

  $scope.cancel = function () {
    $modalInstance.dismiss('cancel');
  };
});

// this directive is to set focus on the modal text so 508 users can scroll down to the bottom of the page
angular.module('PatientApp').directive('autoFocus', function($timeout) {
  return {
    restrict: 'AC',
    link: function(_scope, _element) {
      $timeout(function(){
        _element[0].focus();
      }, 10);
    }
  };
});

// this directive is to grey out the consent button until the user has scrolled down the form
// NOTE: if this ever stops working it is because you upgraded to Angular 2.0+
angular.module('PatientApp').directive('scroll', function ($rootScope) {
  return function(scope, element) {
    element.bind('scroll', function() {
      scope.height = element.prop('offsetHeight');

      if (element.prop('scrollTop') >= element.prop('scrollHeight') - scope.height) {
        $rootScope.$broadcast('scrolledToBottom');
      }
      scope.$apply();

    });
  };
});




