'use strict';

angular.module('AuthVet')
  .factory('AuthorizationInterceptor', function ($q, $log, $injector) {

    //which errors require us to login
    var errorRequiresLogin = function (responseStatus) {
      return (responseStatus === 403 || responseStatus === 401);
    };

    return {
      // optional method
      'request': function (config) {
        //Need to utilize injector to gain access to userService so we avoid circular dependency at app start
        // http://stackoverflow.com/questions/14681654/i-need-two-instances-of-angularjs-http-service-or-what
        //see 2nd solution by thaumant
        $injector.invoke(['$log', 'userService', function ($log, userService) {
          // do something on success
          $log.debug('request= ' + config.url + '. authenticated user= ' + userService.isAuthenticated() + '. token= ' + userService.getAuthorizationToken());

          //Inline function in attempt to resolve circular dependency issues based upon instantiating userService at creation
          //if we are authenticated, add the authorization header to the request
          if (userService.isAuthenticated()) {
            config.headers.Authorization = userService.getAuthorizationToken();
          }
        }]);

        return config;// || $q.when(config);
      },

      'responseError': function (rejection) {
        // if it Eula page, don't do authentication yet
        var eulaAccepted = false;
        $injector.invoke(['eulaService', function (eulaService) {
          if (eulaService.isAccepted()) {
            eulaAccepted = true;
          } else {
            return;
          }
        }]);
        //we received an error, so check the status
        if ((errorRequiresLogin(rejection.status)) && eulaAccepted) {
          // start login process
          $injector.invoke(['userService', function (userService) {
            userService.forceLogin();
          }]);
        }
        return $q.reject(rejection);
      }
    };
  });
