/*jslint node: true, unparam: true */
/*global angular, window */
/* jshint unused:false */

'use strict';
angular.module('PatientApp').service('connectionErrorService', [
  '$http',
  '$resource',
  '$q',
  '$log',
  '$modal',
  'AngularShamNotification',
  function ($http, $resource, $q, $log, $modal, angularShamNotification) {
    var service = {},
      modalInfo = {value: null, dismissed: true, resendHash: {}},
      modalInstance,
      uniqueId = 0,
      openModal,
      dismissModal,
      cacheHtmlTemplates,
      connectionErrorTemplate,
      serverErrorTemplate,
      ModalInstanceCtrl;

    openModal = function (template, controller, modalInfo) {
      if (angular.isDefined(modalInstance) && !modalInfo.dismissed) {
        modalInfo.dismissed = true;
        modalInstance.dismiss();
      }
      modalInstance = $modal.open({
        templateUrl: template,
        controller: 'ConnectionModalInstanceCtrl',
        backdrop: 'static',
        keyboard: false,
        resolve: {
          modalInfo: function () {
            return modalInfo;
          }
        }
      });
    };

    dismissModal = function () {
      if (angular.isDefined(modalInstance) && !modalInfo.dismissed) {
        modalInfo.dismissed = true;
        modalInstance.dismiss();
      }
    };

    // This function caused problems, served up templates as a string I think.
    cacheHtmlTemplates = function () {
      // Silly for now, but the templates all get copies into the views directory
      $http.get('scripts/modules/connection/connection_error.tmpl.html').then(
        function (promise) {
          connectionErrorTemplate = promise.data;
        }
      );
      $http.get('scripts/modules/connection/connection_server.tmpl.html').then(
        function (promise) {
          serverErrorTemplate = promise.data;
        }
      );
    };

    service.showConnectionErrorMsg = function () {
      // disable spinner -do not reset automatically
      angularShamNotification.setDisabled(true, -1);

      dismissModal();
      openModal('scripts/modules/connection/connection_error.tmpl.html', ModalInstanceCtrl, modalInfo);
    };


    service.showServerErrorMsg = function (rejection) {
      var deferred = $q.defer();

      // disable spinner -do not reset automatically
      angularShamNotification.setDisabled(true, -1);

      dismissModal();
      // Here we seem to be adding to the previous attempts.
      // going to try and zero it out first
      // This way is not ideal, seems cumbersome, but need to erase all objects in array.
      for (var hash in modalInfo.resendHash) {
        delete modalInfo.resendHash[hash];
      }
      modalInfo.resendHash[(uniqueId++).toString()] = {rejection: rejection, deferred: deferred};

      if (modalInfo.dismissed) {
        openModal('scripts/modules/connection/connection_server.tmpl.html', ModalInstanceCtrl, modalInfo);
      }

      return deferred.promise;
    };

    service.run = function () {
      window.addEventListener('offline', function () {
        service.showConnectionErrorMsg();
      });
      window.addEventListener('online', function () {
        dismissModal();
      });
      cacheHtmlTemplates();
    };

    // service.run();
    return service;
  }
]);

