'use strict';
angular.module('PatientApp')
    .factory('mhpUserService', [
    '$resource',
    'ResourceDirectoryService',
    '$log',
    '$q',
    function ($resource, ResourceDirectoryService, $log, $q) {

        var mhpLinks,
            getMhpUserLinks;

        getMhpUserLinks = function(){
          var isReady;

          if (isReady) {
            return isReady.promise;
          }

          isReady = $q.defer();

          var mhpLinksResource = $resource(ResourceDirectoryService.getVetUserLinksHref(), {}, {});
          mhpLinks = mhpLinksResource.get({}, function (result) {
                $log.info('Got MHP links');
                isReady.resolve(result);
              },
              function (result) {
                  $log.info('Error retrieving mhpUserLinks');
                  $log.info(result);
                  isReady.reject(result);
                  isReady = null;
                }
          );

          return isReady.promise;
        };

        return {

          getMhpUser: function() {
              var mhpUser,
                  mhpUserLink,
                  isReady;

              if (isReady) {
                return isReady.promise;
              }

              isReady = $q.defer();

              var promise = getMhpUserLinks();

              promise.then(function() {
                if( mhpLinks !== undefined ){
                  for( var i = 0; i < mhpLinks.link.length; i++ ){
                    if( mhpLinks.link[i].title === 'mhpuser' ){
                      mhpUserLink = mhpLinks.link[i].href;
                      break;
                    }
                  }

                  // test link is not undefined
                  if( mhpUserLink !== undefined ){

                    // TODO...undo this once Mike L has fixed prop file
                    //var links = mhpUserLink.split(':');
                    //mhpUserLink = 'https:' + links[1];

                    var mhpUserLinkResource = $resource(mhpUserLink, {}, {});
                    mhpUser = mhpUserLinkResource.get({}, function () {
                        isReady.resolve(mhpUser);
                      },
                      function (result) {
                          $log.info('Error retrieving mhp user');
                          $log.info(result);
                          isReady.reject(result);
                          isReady = null;
                        });
                  }
                }

              },
              function(reason) {
                $log.info('Failed to get mhp user: ');
                $log.info(reason);
              },
              function(update) {
                $log.info('Got mhp user notification: ');
                $log.info(update);
              });

              return isReady.promise;
            }
        };
      }
  ]
);

angular.module('PatientApp')
    .factory('roaService', [
    '$resource',
    'mhpUserService',
    '$log',
    '$q',
    'ResourceDirectoryService',
    '$window',
    function ($resource, mhpUserService, $log, $q, ResourceDirectoryService, $window) {


      var getUserRoaStatus = function () {

        var isReady;

        if (isReady) {
          return isReady.promise;
        }

        isReady = $q.defer();

        var promise = mhpUserService.getMhpUser();

        promise.then(function(mphUser) {
          isReady.resolve(mphUser.rightOfAccessAccepted);
        },
        function(reason) {
          $log.info('ROA Failed: ' + reason);
        },
        function(update) {
          $log.info('Got ROA notification: ' + update);
        });

        return isReady.promise;
      };

      return {
        checkRoaStatus: function() {

          var isReady;

          if (isReady) {
            return isReady.promise;
          }

          isReady = $q.defer();

          var promise = getUserRoaStatus();

          promise.then(function(roaStatus) {

            if(roaStatus === true){
              $log.info('ROA status is true');
              isReady.resolve(true);

              return true;
            }
            else if( roaStatus === false){
              $log.info('ROA status is false');

              var vetRoaPageLink = ResourceDirectoryService.getVetRoaPage();
              $window.location.href = (vetRoaPageLink + '?url=' + window.location.origin + window.location.pathname);

            }
            else {
              $log.info('we dont have roa status');
              isReady.resolve(false);
            }

            return true;
          },
          function(reason) {
            $log.info('Failed getting ROA status: ');
            $log.info(reason);
            isReady.resolve(false);
          },
          function(update) {
            $log.info('Got ROA status notification: ' + update);
            isReady.resolve(false);
          });
          
          return isReady.promise;

        }
      };
    }
  ]
);
