'use strict';

angular.module('MessagingAppApp', [
  'ngCookies',
  'ngResource',
  'ngRoute',
  'ui.bootstrap',
  'AuthStaff',
  'MessAppFilters',
  'highcharts-ng'
])
    .config(function ($routeProvider, $httpProvider, $locationProvider) {
      $routeProvider
      .when('/template/list', {
        templateUrl: 'views/templateList.tpl.html',
        controller: 'TemplateListCtrl',
        resolve: {
          resolvedTemplates: ['$q', 'TemplateListService', function ($q, TemplateListService) {
            var deferred = $q.defer();
            TemplateListService.query(
                function (resp) {
                  deferred.resolve(resp);
                });
            return deferred.promise;
          }]
        },
        role: 'staffView'
      })
      .when('/template/edit/:templateId', {
        templateUrl: 'views/templateEdit.tpl.html',
        controller: 'TemplateEditCtrl',
        resolve: {
          resolvedTemplate: ['$q', '$route', 'TemplateListService', function ($q, $route, TemplateListService) {
            var deferred = $q.defer();
            TemplateListService.get({id: $route.current.params.templateId},
                function (resp) {
                  deferred.resolve(resp);
                });
            return deferred.promise;
          }]
        },
        role: 'staffView'
      })
      .when('/protocols/sendMessage', {
        templateUrl: 'views/sendMessage.html',
        controller: 'MessageSendController'
      })
      .when('/patient/list', {
        templateUrl: 'views/patientList.tpl.html',
        controller: 'PatientListCtrl',
        resolve: {
          resolvedPatients: ['$q', 'PatientListService', function ($q, PatientListService) {
            var deferred = $q.defer();
            PatientListService.query(
                function (resp) {
                  deferred.resolve(resp);
                });
            return deferred.promise;
          }]
        },
        role: 'staffView'
      })
        .when('/patient/protocolList', {
          templateUrl: 'views/patientProtocolList.tpl.html',
          controller: 'PatientProtocolListCtrl',
          role: 'staffView'
          /*resolve: {                                                        // TODO: move service call back into here
            resolvedProtocols: ['$q', '$route', 'PatientProtocolListService', function ($q, $route, PatientProtocolListService) {
                var deferred = $q.defer();
                PatientProtocolListService.query({id: $route.current.params.patientId},
                    function (resp) {
                      deferred.resolve(resp);
                    });
                return deferred.promise;
              }]
          } */
        })
        .when('/reports', {
          templateUrl: 'views/reports.tpl.html',
          controller: 'ReportsCtrl',
          role: 'staffView'
        })
        .when('/editProtocol/:protocolId/:isATemplate', {
          templateUrl: 'views/patientProtocolEdit.tpl.html',
          controller: 'PatientProtocolEditCtrl',
          resolve: {
            resolvedPatientProtocols: ['$q', '$route', 'ProtocolListService', 'ProtocolEditService', 'patientService',
              function ($q, $route, ProtocolListService, ProtocolEditService, patientService) {
                var deferred = $q.defer();
                var protocolId = $route.current.params.protocolId;
                var patientId = patientService.getCurrPatient()._id;
                if ($route.current.params.isATemplate === 'true') {
                  ProtocolListService.update({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                } else {
                  ProtocolEditService.get({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                }
                return deferred.promise;
              }]
          },
          role: 'staffView'
        })
        .when('/patient/patientProtocolEdit/:protocolId/:isATemplate', {      // TODO: this will be replaced with the route above...CMS-1404
          templateUrl: 'views/patientProtocol.tpl.html',
          controller: 'PatientProtocolEditCtrl',
          resolve: {
            resolvedPatientProtocols: ['$q', '$route', 'ProtocolListService', 'ProtocolEditService', 'patientService',
              function ($q, $route, ProtocolListService, ProtocolEditService, patientService) {
                var deferred = $q.defer();
                var protocolId = $route.current.params.protocolId;
                var patientId = patientService.getCurrPatient()._id;
                if ($route.current.params.isATemplate === 'true') {
                  ProtocolListService.update({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                } else {
                  ProtocolEditService.get({protocolId: protocolId, patientId: patientId},
                      function (resp) {
                        deferred.resolve(resp);
                      });
                }
                return deferred.promise;
              }]
          },
          role: 'staffView'
        })
        .when('/patient/protocolAssignList', {
          templateUrl: 'views/protocolAssignList.tpl.html',
          controller: 'ProtocolAssignCtrl',
          resolve: {
            resolvedProtocols: ['$q', 'TemplateListActiveOnlyService', function ($q, TemplateListActiveOnlyService) {
              var deferred = $q.defer();
              deferred.resolve({proto: TemplateListActiveOnlyService.query()});
              return deferred.promise;
            }]
          },
          role: 'staffView'
        })
        .when('/patient/messages', {
          templateUrl: 'views/messageHistory.tpl.html',
          controller: 'MessageHistoryCtrl',
          role: 'staffView'
        })
        .when('/auditTrail/:protoId', {
          templateUrl: 'views/auditTrail.tpl.html',
          controller: 'AuditTrailCtrl',
          resolve: {
            resolvedProtocol: ['$q', '$route', 'ProtocolEditService', 'patientService',
              function ($q, $route, ProtocolEditService, patientService) {
                var deferred = $q.defer();
                var protocolId = $route.current.params.protoId;
                var patientId = patientService.getCurrPatient()._id;
                ProtocolEditService.get({protocolId: protocolId, patientId: patientId},
                  function (resp) {
                    deferred.resolve(resp);
                  });
                return deferred.promise;
              }]
          },
          role: 'staffView'
        })
        .when('/template/new', {
          templateUrl: 'views/protocolTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: function () {
              return;
            }   // dummy function since view and new templates share a controller.
          },
          role: 'staffView'
        })
        .when('/template/view/:templateId', {
          templateUrl: 'views/protocolTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: ['$q', '$route', 'TemplateListService', function ($q, $route, TemplateListService) {
              var deferred = $q.defer();
              var templId = $route.current.params.templateId;
              TemplateListService.get({id: templId},
                  function (resp) {
                    deferred.resolve(resp);
                  });
              return deferred.promise;
            }]
          },
          role: 'staffView'
        })
        .when('/patientSearch/:name', {
          templateUrl: 'views/patientSearch.tpl.html',
          controller: 'PatientSearchCtrl',
          role: 'staffView'
        })
        .when('/patientSearch', {
          templateUrl: 'views/patientSearch.tpl.html',
          controller: 'PatientSearchCtrl',
          role: 'staffView'
        })
        .when('/clinicalAdmin/protocols', {
          templateUrl: 'views/clinicalAdminProtocol.tpl.html',
          controller: 'TemplateListCtrl',
          resolve: {
            resolvedTemplates: ['$q', 'TemplateListService', function ($q, TemplateListService) {
              var deferred = $q.defer();
              TemplateListService.query(
                  function (resp) {
                    deferred.resolve(resp);
                  });
              return deferred.promise;
            }]
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add', {
          templateUrl: 'views/clinicalAdminAddProtocol.tpl.html',
          controller: 'AddProtocolCtrl',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/addOrCopyTemplate', {
          templateUrl: 'views/clinicalAdminAddOrCopyTemplate.tpl.html',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/addTemplate', {
          templateUrl: 'views/clinicalAdminAddTemplate.tpl.html',
          controller: 'AddProtocolCtrl',
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/protocols/edit/add/vital', {
          templateUrl: 'views/clinicalAdminVitalTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: function () {
              return;
            }   // dummy function since view and new templates share a controller.
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/template/view/:templateId', {
          templateUrl: 'views/protocolTemplate.tpl.html',
          controller: 'ProtocolTemplateCtrl',
          resolve: {
            resolvedTemplate: ['$q', '$route', 'TemplateListService', function ($q, $route, TemplateListService) {
              var deferred = $q.defer();
              var templId = $route.current.params.templateId;
              TemplateListService.get({id: templId},
                  function (resp) {
                    deferred.resolve(resp);
                  });
              return deferred.promise;
            }]
          },
          role: 'clinicalAdmin'
        })
        .when('/clinicalAdmin/reports', {
          templateUrl: 'views/clinicalAdminReports.tpl.html',
          role: 'clinicalAdmin'
        })
        .when('/', {
          redirectTo: '/patientSearch' // TODO: route to the /patientSearch/:name
        })
        .otherwise({
          redirectTo: '/patientSearch'
        });

      $httpProvider.interceptors.push('noCacheInterceptor');

      $locationProvider.html5Mode(false);
    });

// Preload templates
angular.module('MessagingAppApp').run(function($http, $templateCache, $rootScope, $location, AddProtocolFactory){

  $http.get( 'views/protocolTemplate.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientProtocol.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/templateDescription.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/templateMeasurement.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/templateScheduleSettings.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/templateListSidebar.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/templateMessages.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientScheduleSettings.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientCalendar.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientMeasurement.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientMessages.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/protocolStatus.tpl.html', { cache: $templateCache }).then(function () {
  });

  $http.get( 'views/patientSearch.tpl.html', { cache: $templateCache }).then(function () {
  });

  // register listener to watch route changes
  $rootScope.$on('$routeChangeStart', function () {
    var path = $location.path();
    // clear data if the user navigates away from add/edit
    // protocol wizard
    if (path.indexOf('/clinicalAdmin/protocols/edit') === -1) {
      AddProtocolFactory.resetData();
    } else {
      // if it is not the start page of the wizard, and there is not data
      // saved in service, we need to redirect to start page
      // TODO: need to revisit for view/edit case
      if (path !== '/clinicalAdmin/protocols/edit/add') {
        var protocol = AddProtocolFactory.loadData();
        if (protocol.isNew === true) {
          $location.path('/clinicalAdmin/protocols/edit/add');
        }
      }
    }
  });
});



