package gov.va.mobilehealth.courier.smsgateway.communication;

import com.google.common.base.Objects;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;

import java.io.Serializable;

public class CMSMessage implements Serializable {

  private static final long serialVersionUID = 1L;
  private String incomingMessageId;
  private String patientId;
  private String type;
  private String source;
  private String destination;
  private String text;
  /* time the message is received in UTC */
  private String networkSubmitTime;
  private String patientProtocolId;
  private int templateIndex;
  private String alertLevel;
  private String serviceName;
  private boolean forceSendViaSMS = false;
  private boolean forceSendViaAnnie = true;
  /* time zone the patient belongs to */
  private String timeZone;
  /* time the message is received in patient time zone */
  private String adjustedSubmitTime;
  private boolean messageHelp = false;

  //TODO: Get away from the default constructor and setters and create immutable class, values should be known at time of sending, otherwise it is probably
  //business logic that needs to be determined (protocol specific, etc)
  public CMSMessage() {
  }

  public CMSMessage(CMSMessage other) {
    this.incomingMessageId = other.incomingMessageId;
    this.patientId = other.patientId;
    this.type = other.type;
    this.source = other.source;
    this.destination = other.destination;
    this.text = other.text;
    this.networkSubmitTime = other.networkSubmitTime;
    this.patientProtocolId = other.patientProtocolId;
    this.templateIndex = other.templateIndex;
    this.alertLevel = other.alertLevel;
    this.serviceName = other.serviceName;
    this.forceSendViaSMS = other.forceSendViaSMS;
    this.timeZone = other.timeZone;
    this.adjustedSubmitTime = other.adjustedSubmitTime;
    this.messageHelp = other.messageHelp;
  }

  public CMSMessage(String source, Patient patient, String messageText, MessageType messageType) {

    this.setSource(source);
    this.setDestination(patient.getPreference().getPhoneNumber());
    this.setText(messageText);
    this.setType(messageType.toString());
    this.setPatientId(patient.getId());
    this.setTimeZone(patient.getPreference().getTimeZone());
  }

  public CMSMessage(PatientIncomingMessage patientResponse, String messageText, MessageType messageType) {
    this.setIncomingMessageId(patientResponse.getId());
    this.setSource(patientResponse.getDestination());
    this.setDestination(patientResponse.getSource());
    this.setText(messageText);
    this.setType(messageType.toString());
    this.setTimeZone(patientResponse.getTimeZone());
  }

  public String getIncomingMessageId() {
    return incomingMessageId;
  }

  public void setIncomingMessageId(String incomingMessageId) {
    this.incomingMessageId = incomingMessageId;
  }

  public String getPatientId() {
    return patientId;
  }

  public void setPatientId(String patientId) {
    this.patientId = patientId;
  }

  public String getType() {
    return type;
  }

  public void setType(String type) {
    this.type = type;
  }

  public String getSource() {
    return source;
  }

  public void setSource(String source) {
    this.source = source;
  }

  public String getDestination() {
    return destination;
  }

  public void setDestination(String destination) {
    this.destination = destination;
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
  }

  public String getAlertLevel() {
    return alertLevel;
  }

  public void setAlertLevel(String status) {
    this.alertLevel = status;
  }

  public String getNetworkSubmitTime() {
    return networkSubmitTime;
  }

  public void setNetworkSubmitTime(String networkSubmitTime) {
    this.networkSubmitTime = networkSubmitTime;
  }

  public String getPatientProtocolId() {
    return patientProtocolId;
  }

  public void setPatientProtocolId(String patientProtocolId) {
    this.patientProtocolId = patientProtocolId;
  }

  public int getTemplateIndex() {
    return templateIndex;
  }

  public void setTemplateIndex(int templateIndex) {
    this.templateIndex = templateIndex;
  }

  public String getServiceName() {
    return serviceName;
  }

  public void setServiceName(String serviceName) {
    this.serviceName = serviceName;
  }

  public void setForceSendViaSMS(boolean forceSendViaSMS) {
    this.forceSendViaSMS = forceSendViaSMS;
  }

  public boolean isForceSendViaSMS() {
    return forceSendViaSMS;
  }

  public void setForceSendViaAnnie(boolean forceSendViaAnnie) {
      this.forceSendViaAnnie = forceSendViaAnnie;
    }

    public boolean isForceSendViaAnnie() {
      return forceSendViaAnnie;
    }

  public String getTimeZone() {
    return this.timeZone;
  }

  public void setTimeZone(String timeZone) {
    this.timeZone = timeZone;
  }

  public String getAdjustedSubmitTime() {
    return this.adjustedSubmitTime;
  }

  public void setAdjustedSubmitTime(String adjustedSubmitTime) {
    this.adjustedSubmitTime = adjustedSubmitTime;
  }

  public void setMessageHelp(boolean messageHelp) {
    this.messageHelp = messageHelp;
  }

  public boolean isMessageHelp() {
    return messageHelp;
  }


  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof CMSMessage)) {
      return false;
    }

    CMSMessage that = (CMSMessage) o;

    if (templateIndex != that.templateIndex) {
      return false;
    }
    if (alertLevel != null ? !alertLevel.equals(that.alertLevel) : that.alertLevel != null) {
      return false;
    }
    if (destination != null ? !destination.equals(that.destination) : that.destination != null) {
      return false;
    }
    if (incomingMessageId != null ? !incomingMessageId.equals(that.incomingMessageId) : that.incomingMessageId != null) {
      return false;
    }
    if (networkSubmitTime != null ? !networkSubmitTime.equals(that.networkSubmitTime) : that.networkSubmitTime != null) {
      return false;
    }
    if (patientId != null ? !patientId.equals(that.patientId) : that.patientId != null) {
      return false;
    }
    if (patientProtocolId != null ? !patientProtocolId.equals(that.patientProtocolId) : that.patientProtocolId != null) {
      return false;
    }
    if (serviceName != null ? !serviceName.equals(that.serviceName) : that.serviceName != null) {
      return false;
    }
    if (source != null ? !source.equals(that.source) : that.source != null) {
      return false;
    }
    if (text != null ? !text.equals(that.text) : that.text != null) {
      return false;
    }
    if (type != null ? !type.equals(that.type) : that.type != null) {
      return false;
    }

    return true;
  }

  @Override
  public int hashCode() {
    int result = incomingMessageId != null ? incomingMessageId.hashCode() : 0;
    result = 31 * result + (patientId != null ? patientId.hashCode() : 0);
    result = 31 * result + (type != null ? type.hashCode() : 0);
    result = 31 * result + (source != null ? source.hashCode() : 0);
    result = 31 * result + (destination != null ? destination.hashCode() : 0);
    result = 31 * result + (text != null ? text.hashCode() : 0);
    result = 31 * result + (networkSubmitTime != null ? networkSubmitTime.hashCode() : 0);
    result = 31 * result + (patientProtocolId != null ? patientProtocolId.hashCode() : 0);
    result = 31 * result + templateIndex;
    result = 31 * result + (alertLevel != null ? alertLevel.hashCode() : 0);
    result = 31 * result + (serviceName != null ? serviceName.hashCode() : 0);
    return result;
  }

  @Override
  public String toString() {
    return Objects.toStringHelper(this)
      .add("incomingMessageId", incomingMessageId)
      .add("patientId", patientId)
      .add("type", type)
      .add("source", source)
      .add("destination", destination)
      .add("text", text)
      .add("networkSubmitTime", networkSubmitTime)
      .add("patientProtocolId", patientProtocolId)
      .add("templateIndex", templateIndex)
      .add("alertLevel", alertLevel)
      .add("serviceName", serviceName)
      .toString();
  }
}
