package gov.va.mobilehealth.courier.smsgateway.communication;

import gov.va.mobilehealth.courier.smsgateway.parser.SendMessageParser;
import gov.va.mobilehealth.courier.smsgateway.types.Message;
import gov.va.mobilehealth.courier.smsgateway.types.Response;
import org.apache.commons.codec.binary.Base64;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.utils.URIBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import javax.xml.bind.JAXBException;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;


public class MessageProcessor extends MessageProcessorBase {

  private static final Logger LOGGER = LoggerFactory.getLogger(MessageProcessor.class);

  protected MessageDeliver md;

  public MessageProcessor() {
    md = new MessageDeliver();
  }

  @Override
  public void sendSingleMessage(CMSMessage msg, MessageHandler mh) throws ParserConfigurationException, SAXException {

    System.out.println(" in the MessageProcessor");

    HttpRequest httpRequest;

    try {
      Response response = null;
      if (msg != null) {

        if (msg.isForceSendViaSMS()) {
          httpRequest = createSingleMessageRequest(msg.getDestination(), msg.getText());
          LOGGER.debug("Request: {}", httpRequest.getRequestUrl());
          response = (Response) md.sendMessage(httpRequest, new SendMessageParser());

          LOGGER.debug("Response: ");

          LOGGER.debug("Success Code: {}", response.getMessageList().get(0).getSuccess_code());
          LOGGER.debug("Response Code: {}", response.getMessageList().get(0).getResponse_code());
          LOGGER.debug("Response String: {}", response.getMessageList().get(0).getResponse_string());
          LOGGER.debug("MessageId: {}", response.getMessageList().get(0).getMessage_id());
        }

      }
      if (mh != null) {
        mh.processSingleMessageResult(msg, response);
      }

    } catch (ClientProtocolException e) {
      LOGGER.error("ClientProtocolException", e);
    } catch (IOException e) {
      LOGGER.error("IOException", e);
    } catch (JAXBException e) {
      LOGGER.error("JAXBException", e);
    }
  }

  protected HttpRequest createSingleMessageRequest(String mdn, String message)
    throws IOException {

    HttpRequest newRequest;

    try {
      URI uri = new URIBuilder(smsConfig.serverUrlSendMessage())
        .setParameter("text", message)
        .setParameter("destination", mdn)
        .setParameter("responseType", "1")
        .build();

      newRequest = new HttpRequest(HttpRequest.Method.GET, uri.toString(), null, null);

    } catch (URISyntaxException e) {
      throw new RuntimeException(e);
    }

    // add header
    //TODO: Pre-encode the values in the properties file so we don't recompute per message
    if (smsConfig.serverUsername() != null) {
      byte[] encodedBytes = Base64.encodeBase64(
        (smsConfig.serverUsername() + ":"
          + smsConfig.serverPassword()).getBytes()
      );
      newRequest.addHeader("Authorization", "Basic " + new String(encodedBytes));
    }
    newRequest.addHeader("Accept", "application/xml");

    return newRequest;
  }

  protected void processSingleMessageResult(Response response) {

    ArrayList<Message> list = response.getMessageList();
    for (Message msg : list) {
      try {
        LOGGER.debug("{} {} {} {}", msg.getSuccess_code(),
          msg.getResponse_code(),msg.getResponse_string(),
          msg.getMessage_id());
      } catch (Exception e) {
        LOGGER.error("Exception", e);
      }
      LOGGER.debug("{} {} {} {}", msg.getSuccess_code(),
        msg.getResponse_code(), msg.getResponse_string(),
        msg.getMessage_id());
    }
    // update db
  }

  public void getHeader(HttpURLConnection conn) {
  }

}
