class ClinicalStaffErrorMessages
  def initialize
    @default_separator = "\n"

    @error_summary = {
      :heading => "Error",
      :contents => "Please correct errors on the form below."
    }

    @readings = {
      :valid_reading => {
        :lowest => {
          :empty_field => "Please enter a Lowest Valid Reading",
          :highest_valid_violation => "Lowest Valid Reading CANNOT be the same value and must be lower than the Highest Valid Reading",
          :critical_low_violation => "Lowest Valid Reading CANNOT be the same value and must be lower than the Critical Low Level Alert",
          :critical_high_violation => "Lowest Valid Reading CANNOT be the same value and must be lower than the Critical High Level Alert",
          :alert_low_violation => "Lowest Valid Reading CANNOT be the same value and must be lower than the Low Level Alert",
          :alert_high_violation => "Lowest Valid Reading CANNOT be the same value and must be lower than the High Level Alert"
        },
        :highest => {
          :empty_field => "Please enter a Highest Valid Reading",
          :lowest_valid_violation => "Highest Valid Reading CANNOT be the same value and must be higher than the Lowest Valid Reading",
          :critical_low_violation => "Highest Valid Reading CANNOT be the same value and must be higher than the Critical Low Level Alert",
          :critical_high_violation => "Highest Valid Reading CANNOT be the same value and must be higher than the Critical High Level Alert",
          :alert_low_violation => "Highest Valid Reading CANNOT be the same value and must be higher than the Low Level Alert",
          :alert_high_violation => "Highest Valid Reading CANNOT be the same value and must be higher than the High Level Alert"
        },
        :empty_message_field => "Please enter Message for Valid Reading"
      },
      :critical_low => {
        :empty_field => "Please enter a Critical Low Level Alert",
        :lowest_valid_violation => "Critical Low Level Alert CANNOT be the same value or lower than the Lowest Valid Reading",
        :highest_valid_violation => "Critical Low Level Alert CANNOT be the same value or higher than the Highest Valid Reading",
        :critical_high_violation => "Critical Low Level Alert CANNOT be the same value and must be lower than Critical High Level Alert",
        :alert_low_violation => "Critical Low Level Alert CANNOT be the same value or higher than the Low Level Alert",
        :alert_high_violation => "Critical Low Level Alert CANNOT be the same value or higher than the High Level Alert",
        :empty_message_field => "Please enter a Message for Critical Low Level Alert"
      },
      :critical_high => {
        :empty_field => "Please enter a Critical High Level Alert",
        :lowest_valid_violation => "Critical High Level Alert CANNOT be the same value or higher than the Lowest Valid Reading",
        :highest_valid_violation => "Critical High Level Alert CANNOT be the same value or higher than the Highest Valid Reading",
        :critical_low_violation => "Critical High Level Alert CANNOT be the same value and must be higher than Critical Low Level Alert",
        :alert_low_violation => "Critical High Level Alert CANNOT be the same value or higher than the Low Level Alert",
        :alert_high_violation => "Critical High Level Alert CANNOT be the same value or lower than the High Level Alert",
        :empty_message_field => "Please enter a Message for Critical High Level Alert"
      },
      :alert_low => {
        :empty_field => "Please enter a Low Level Alert",
        :lowest_valid_violation => "Low Level Alert CANNOT be the same value or lower than the Lowest Valid Reading",
        :highest_valid_violation => "Low Level Alert CANNOT be the same value or higher than the Highest Valid Reading",
        :critical_low_violation => "Low Level Alert CANNOT be the same value or lower than the Critical Low Level Alert",
        :critical_high_violation => "Low Level Alert CANNOT be the same value or higher than the Critical High Level Alert",
        :alert_high_violation => "Low Level Alert CANNOT be the same value and must be lower than High Level Alert",
        :empty_message_field => "Please enter a Message for Low Level Alert"
      },
      :alert_high => {
        :empty_field => "Please enter a High Level Alert",
        :lowest_valid_violation => "High Level Alert CANNOT be the same value or lower than the Lowest Valid Reading",
        :highest_valid_violation => "High Level Alert CANNOT be the same value or higher than the Highest Valid Reading",
        :critical_low_violation => "High Level Alert CANNOT be the same value or lower than the Critical Low Level Alert",
        :critical_high_violation => "High Level Alert CANNOT be the same value or higher than the Critical High Level Alert",
        :alert_low_violation => "High Level Alert CANNOT be the same value and must be higher than Low Level Alert",
        :empty_message_field => "Please enter a Message for High Level Alert"
      },
      :separator => @default_separator
    }

    @schedule = {
      :schedule_template_empty_field => "Please enter a Template Description",
      :schedule_empty_field => "Please select a Schedule",
      :service_message_start_time_empty_field => "Please select a time for Service Message Start Time",
      :start_date_empty_field => "Please enter a valid date format (MM/DD/YYYY)",
      :end_date_empty_field => "Please enter a valid date format (MM/DD/YYYY)",
      :measurement_empty_field => "Please select a Measurement",
      :service_message_empty_field => "Please enter a Service Message",
      :time_between_each_cycle_empty_field => "Please enter a Time Between Each Cycle",
      :select_days_none_selected => "Please select Days that Apply",
      :separator => @default_separator
    }

    @message = {
      :empty_service_message => "Please enter a Service Message",
      :empty_reminder_message => "Please enter a Reminder Message",
      :empty_time_between_reminders => "Please enter in the Amount of time between reminders",
      :empty_time_schedule => "Please select a Time Schedule",
      :empty_reminder_message_count => "Please enter the number of times of reminder message is sent",
      :empty_motivational_message => "Please enter a Motivational/Education Message",
      :separator => @default_separator
    }
  end

  def get_error_messages(options = {})
    final_message = ""

    # Error summary
    if options[:error_summary]
      final_message << @error_summary[:heading] << @default_separator << @error_summary[:contents] << @default_separator
    end

    # Lowest valid reading error messages
    if options[:lowest_valid_reading_empty_field]
      final_message << @readings[:valid_reading][:lowest][:empty_field] << @readings[:separator]
    end
    if options[:lowest_valid_reading_highest_valid_violation]
      final_message << @readings[:valid_reading][:lowest][:highest_valid_violation] << @readings[:separator]
    end
    if options[:lowest_valid_reading_critical_low_violation]
      final_message << @readings[:valid_reading][:lowest][:critical_low_violation] << @readings[:separator]
    end
    if options[:lowest_valid_reading_critical_high_violation]
      final_message << @readings[:valid_reading][:lowest][:critical_high_violation] << @readings[:separator]
    end
    if options[:lowest_valid_reading_alert_low_violation]
      final_message << @readings[:valid_reading][:lowest][:alert_low_violation] << @readings[:separator]
    end
    if options[:lowest_valid_reading_alert_high_violation]
      final_message << @readings[:valid_reading][:lowest][:alert_high_violation] << @readings[:separator]
    end

    # Highest valid reading error messages
    if options[:highest_valid_reading_empty_field]
      final_message << @readings[:valid_reading][:highest][:empty_field] << @readings[:separator]
    end
    if options[:highest_valid_reading_lowest_valid_violation]
      final_message << @readings[:valid_reading][:highest][:lowest_valid_violation] << @readings[:separator]
    end
    if options[:highest_valid_reading_critical_low_violation]
      final_message << @readings[:valid_reading][:highest][:critical_low_violation] << @readings[:separator]
    end
    if options[:highest_valid_reading_critical_high_violation]
      final_message << @readings[:valid_reading][:highest][:critical_high_violation] << @readings[:separator]
    end
    if options[:highest_valid_reading_alert_low_violation]
      final_message << @readings[:valid_reading][:highest][:alert_low_violation] << @readings[:separator]
    end
    if options[:highest_valid_reading_alert_high_violation]
      final_message << @readings[:valid_reading][:highest][:alert_high_violation] << @readings[:separator]
    end

    # Valid reading message error messages
    if options[:valid_reading_empty_message_field]
      final_message << @readings[:valid_reading][:empty_message_field] << @readings[:separator]
    end

    # Critical low reading error messages
    if options[:critical_low_reading_empty_field]
      final_message << @readings[:critical_low][:empty_field] << @readings[:separator]
    end
    if options[:critical_low_reading_lowest_valid_violation]
      final_message << @readings[:critical_low][:lowest_valid_violation] << @readings[:separator]
    end
    if options[:critical_low_reading_highest_valid_violation]
      final_message << @readings[:critical_low][:highest_valid_violation] << @readings[:separator]
    end
    if options[:critical_low_reading_critical_high_violation]
      final_message << @readings[:critical_low][:critical_high_violation] << @readings[:separator]
    end
    if options[:critical_low_reading_alert_low_violation]
      final_message << @readings[:critical_low][:alert_low_violation] << @readings[:separator]
    end
    if options[:critical_low_reading_alert_high_violation]
      final_message << @readings[:critical_low][:alert_high_violation] << @readings[:separator]
    end
    if options[:critical_low_reading_empty_message_field]
      final_message << @readings[:critical_low][:empty_message_field] << @readings[:separator]
    end

    # Critical high reading error messages
    if options[:critical_high_reading_empty_field]
      final_message << @readings[:critical_high][:empty_field] << @readings[:separator]
    end
    if options[:critical_high_reading_critical_low_violation]
      final_message << @readings[:critical_high][:critical_low_violation] << @readings[:separator]
    end
    if options[:critical_high_reading_lowest_valid_violation]
      final_message << @readings[:critical_high][:lowest_valid_violation] << @readings[:separator]
    end
    if options[:critical_high_reading_highest_valid_violation]
      final_message << @readings[:critical_high][:highest_valid_violation] << @readings[:separator]
    end
    if options[:critical_high_reading_alert_low_violation]
      final_message << @readings[:critical_high][:alert_low_violation] << @readings[:separator]
    end
    if options[:critical_high_reading_alert_high_violation]
      final_message << @readings[:critical_high][:alert_high_violation] << @readings[:separator]
    end
    if options[:critical_high_reading_empty_message_field]
      final_message << @readings[:critical_high][:empty_message_field] << @readings[:separator]
    end

    # Alert low reading error messages
    if options[:alert_low_reading_empty_field]
      final_message << @readings[:alert_low][:empty_field] << @readings[:separator]
    end
    if options[:alert_low_reading_lowest_valid_violation]
      final_message << @readings[:alert_low][:lowest_valid_violation] << @readings[:separator]
    end
    if options[:alert_low_reading_highest_valid_violation]
      final_message << @readings[:alert_low][:highest_valid_violation] << @readings[:separator]
    end
    if options[:alert_low_reading_alert_high_violation]
      final_message << @readings[:alert_low][:alert_high_violation] << @readings[:separator]
    end
    if options[:alert_low_reading_critical_low_violation]
      final_message << @readings[:alert_low][:critical_low_violation] << @readings[:separator]
    end
    if options[:alert_low_reading_critical_high_violation]
      final_message << @readings[:alert_low][:critical_high_violation] << @readings[:separator]
    end
    if options[:alert_low_reading_empty_message_field]
      final_message << @readings[:alert_low][:empty_message_field] << @readings[:separator]
    end

    # Alert high reading error messages
    if options[:alert_high_reading_empty_field]
      final_message << @readings[:alert_high][:empty_field] << @readings[:separator]
    end
    if options[:alert_high_reading_alert_low_violation]
      final_message << @readings[:alert_high][:alert_low_violation] << @readings[:separator]
    end
    if options[:alert_high_reading_lowest_valid_violation]
      final_message << @readings[:alert_high][:lowest_valid_violation] << @readings[:separator]
    end
    if options[:alert_high_reading_highest_valid_violation]
      final_message << @readings[:alert_high][:highest_valid_violation] << @readings[:separator]
    end
    if options[:alert_high_reading_critical_low_violation]
      final_message << @readings[:alert_high][:critical_low_violation] << @readings[:separator]
    end
    if options[:alert_high_reading_critical_high_violation]
      final_message << @readings[:alert_high][:critical_high_violation] << @readings[:separator]
    end
    if options[:alert_high_reading_empty_message_field]
      final_message << @readings[:alert_high][:empty_message_field] << @readings[:separator]
    end

    # Schedule error messages
    if options[:schedule_start_date_empty_field]
      final_message << @schedule[:start_date_empty_field] << @schedule[:separator]
    end
    if options[:schedule_end_date_empty_field]
      final_message << @schedule[:end_date_empty_field] << @schedule[:separator]
    end

    # Message error messages
    if options[:service_message_empty_field]
      final_message << @message[:empty_service_message] << @message[:separator]
    end
    if options[:reminder_message_empty_field]
      final_message << @message[:empty_reminder_message] << @message[:separator]
    end
    if options[:time_between_reminders_empty_field]
      final_message << @message[:empty_time_between_reminders] << @message[:separator]
    end
    if options[:reminder_message_count_empty_field]
      final_message << @message[:empty_reminder_message_count] << @message[:separator]
    end

    # Strip any leading or trailing whitespace off of the return string
    final_message.strip
  end
end