require 'data_models/template_reading'  # Provides the TemplateReading class
require 'data_models/template_messages'  # Provides the TemplateMessages class
require 'data_models/template_schedule'  # Provides the TemplateSchedule class
require 'data_models/utility_models'  # Provides the ValueUnits & TimeOfDay structs and the RangeLimit class

class TemplateModel
  attr_accessor :measurement  # String object
  attr_accessor :keywords  # String object
  attr_accessor :messages  # TemplateMessages object
  attr_accessor :schedule  # TemplateSchedule object
  attr_accessor :time_between_reminders  # ValueUnit object
  attr_accessor :reminder_message_count  # String object
  attr_accessor :template_description  # String object

  def initialize(options = {})
    # Initalize an empty array for the readings
    @readings = Array.new

    # Initialize string objects
    @measurement = options[:measurement]
    @keywords = options[:keywords]
    @reminder_message_count = options[:reminder_message_count]
    @template_description = options[:template_description]

    # Initialize ValueUnit objects
    if options[:time_between_reminders] && options[:time_between_reminders].is_a?(ValueUnits)
      @time_between_reminders = options[:time_between_reminders]
    end

    # Initialize readings
    if options[:readings]
      if options[:readings].is_a?(TemplateReading)  # A single TemplateReading object has been received
        @readings.push(options[:readings])
      elsif options[:readings].is_a?(Array)  # An array of TemplateReading objects has been received
        options[:readings].each do |reading_object|
          @readings.push(reading_object)
        end
      end
    end

    # Initialize messages
    if options[:messages] && options[:messages].is_a?(TemplateMessages)
      @messages = options[:messages]
    end

    # Initialize schedule
    if options[:schedule] && options[:schedule].is_a?(TemplateSchedule)
      @schedule = options[:schedule]
    end
  end

  def get_reading(index = 0)
    @readings[index]
  end

  def count_readings()
    @readings.length
  end
end
