require 'data_models/utility_models'

class TemplateSchedule
  attr_accessor :schedule  # The frequency at which the schedule is triggered
  attr_accessor :service_message_start_time  # Expects a TimeOfDay object
  attr_accessor :time_between_each_cycle  # Expects a ValueUnit object
  attr_accessor :accept_unscheduled_readings  # Expects a boolean value
  attr_accessor :start_date  # Expects a string value
  attr_accessor :end_date  # Expects a string value
  attr_accessor :schedule_days  # Expects a DaysOfTheWeek object
  attr_accessor :start_period  # Expects a ValueUnit object
  attr_accessor :end_period  # Expects a ValueUnit object

  def initialize(options = {})
    @schedule = options[:schedule]
    @start_date = options[:start_date]
    @end_date = options[:end_date]

    if options[:service_message_start_time] && options[:service_message_start_time].is_a?(TimeOfDay)
      @service_message_start_time = options[:service_message_start_time]
    end

    if options[:time_between_each_cycle] && options[:time_between_each_cycle].is_a?(ValueUnits)
      @time_between_each_cycle = options[:time_between_each_cycle]
    end

    if options[:accept_unscheduled_readings]
      @accept_unscheduled_readings = options[:accept_unscheduled_readings]
    else
      @accept_unscheduled_readings = false
    end

    if options[:schedule_days] && options[:schedule_days].is_a?(DaysOfTheWeek)
      @schedule_days = options[:schedule_days]
    end

    if options[:start_period] && options[:start_period].is_a?(ValueUnits)
      @start_period = options[:start_period]
    end

    if options[:end_period] && options[:end_period].is_a?(ValueUnits)
      @end_period = options[:end_period]
    end
  end

  def accept_unscheduled_readings?
    @accept_unscheduled_readings
  end
end