require 'page-object'

class ListTemplatesPage < AdminPage
  include PageObject

  #Many of the elements here do not appear to describe the ListTemplatesPage
  page_url PROVIDER_APP_URL #+ 'template/list/nav'

  link(:add_protocol, link_text: 'Add Protocol')

  indexed_property(:protocol_list, [
    [:h2, :name, {id: 'protocol[%s].name'}],
    [:h3, :created_info, {id: 'created-[%s].info'}],
    [:paragraph, :description, {id: 'protocol[%s].description'}],
    [:paragraph, :focus_area, {id: 'protocol[%s].focus-area'}],
    [:paragraph, :last_comment, {id: 'protocol[%s].last-comment'}],
    [:link, :audit_trail, {id: 'protocol[%s].audit-trail-btn'}],
    [:radio_button, :enable, {id: 'protocol[%s].enable'}],
    [:radio_button, :disable, {id: 'protocol[%s].disable'}],
    [:link, :edit_view, {id: 'protocol[%s].edit-view-btn'}]
  ])

  paragraph(:success_message, id: 'success-message')

  text_area(:disable_comment, id: 'disable-comment')

  button(:disable_ok, id: 'ok')

  button(:disable_cancel, id: 'cancel')

  span(:disable_error, id: 'comment-error')

  text_area(:enable_comment, id: 'enable-comment')

  button(:enable_ok, id: 'ok1')

  button(:enable_cancel, id: 'cancel1')

  span(:enable_error, id: 'comment-error1')

  list_item(:create_new_template, id: 'new-tpl-link')

  div(:patient_search, id: 'patient-search')

  link(:list_templates, :link_text => 'List Templates')

  link(:glucose_template, :link_text => 'GlucoseXXX')

  link(:new_template, :link_text => 'BP Measurement')

  link(:new_bloodPressureTemplate, :link_text => 'BP Measurement')

  link(:new_oxygenTemplate, :link_text => 'Oxygen Measurement')

  link(:new_bloodGlucoseTemplate, :link_text => 'Glucose Measurement')

  link(:new_glucoseBeforeEatingTemplate, :link_text => 'Before Measurement')

  link(:new_glucoseAfterEatingTemplate, :link_text => 'After Measurement')

  link(:new_weightLossTemplate, :link_text => 'Weight Measurement')

  link(:new_caloricIntakeTemplate, :link_text => 'Caloric Measurement')

  link(:new_amountOfExerciseTemplate, :link_text => 'Exercise Measurement')

  link(:new_temperatureTemplate, :link_text => 'Temperature Measurement')

  link(:new_pulseTemplate, :link_text => 'Pulse Measurement')

  link(:new_otherTemplate, :link_text => 'Other Measurement')

  link(:create_new_template_link, :link_text => 'Create Protocol Template')

  link(:list_patients, :link_text => 'List Patients')

  span(:userdetails, id: 'user-details')


  indexed_property(:template_list, [
    [:link, :templ_name, {id: 'protocol[%s].name'}]
  ])

  table(:template_table, id: 'assign')
  link(:gl_bp_protocol, :link_text => 'GL BP Protocol')
  link(:gl_bp_protocol2, :link_text => 'GL BP Protocol2')

  link(:about, id: 'nav-about')
  link(:more_info, id: 'more-info')
  button(:about_ok, id: 'about-ok')
  paragraph(:version, id: 'version')
  paragraph(:clinical_admin, id: 'clinical_admin')
  paragraph(:clinical_staff, id: 'clinical_staff')
  paragraph(:veteran, id: 'veteran')
  list_item(:clinical_admin_info, id: 'clinical_admin_info')
  list_item(:clinical_staff_info, id: 'clinical_staff_info')
  list_item(:veteran_info, id: 'veteran_info')


  indexed_property(:user_info, [
    [:link, :user_info_dropdown, {id: 'userinfo[{{$index}}]'}]
  ])

  def create_new_template
    self.create_new_template_link_element.when_visible.click
    self.waitForLoadingToClear

    TemplateEditPage.new(@browser)
  end

  def select_protocol_template(template_name)
    self.get_link_by_text(template_name).click
    self.waitForLoadingToClear

    TemplateEditPage.new(@browser)
  end

  def create_new_protocol(protocol)
    self.add_protocol

    page = AddNewProtocolPage.new(@browser)
    page.protocol_name = protocol.newTemplateName
    page.protocol_description = protocol.description
    page.protocol_focus_area = protocol.focusArea
    page.next
    page.back_element.visible?
    page.add_template_element.click
    page.vitals_element.click
  end

  def create_motivational_educational_protocol(protocol)
    self.add_protocol

    page = AddNewProtocolPage.new(@browser)
    page.protocol_name = protocol.newTemplateName
    page.protocol_description = protocol.description
    page.protocol_focus_area = protocol.focusArea
    page.next
    page.back_element.visible?
    page.add_template_element.click
    page.motivational_educational_element.click
  end

  def create_custom_protocol(protocol)
    self.add_protocol

    page = AddNewProtocolPage.new(@browser)
    page.protocol_name = protocol.newTemplateName
    page.protocol_description = protocol.description
    page.protocol_focus_area = protocol.focusArea
    page.next
    page.back_element.visible?
    page.add_template_element.click
    page.custom_element.click
  end

  def navigateToAuditTrail(name)
    list = self.protocol_list
    index = self.get_index_by_text(list, name)
    self.protocol_list[index].audit_trail
  end

end
