class GlobalObjectsPage < RubySelenium
  include PageObject

  #Session Time Out Warning Objects
  h3(:sessionTimeOutWarningHeading, id: 'login-session-ending')
  paragraph(:timer, id: 'timer')
  div(:warningText, id: 'session-warning-text')
  button(:continue, id: 'continue')
  button(:logout, id: 'logout')

  #Login Session Ended Objects
  h3(:sessionTimeOutHeading, id: 'login-session-ended')
  div(:timeOutText, id: 'session-timeout-text')
  button(:login, id: 'login')

  #Loading Animation
  paragraph(:loading_text, :id => 'loading-text')

  #About Page info
  div(:about_modal_body, :id => 'about-modal-body')
  link(:more_info, :id => 'more-info')
  button(:about_ok, :id => 'about-ok')
  paragraph(:version, :id => 'version')
  paragraph(:clinical_admin, :id => 'clinical_admin')
  paragraph(:clinical_staff, :id => 'clinical_staff')
  paragraph(:veteran, :id => 'veteran')
  list_item(:clinical_admin_info, :id => 'clinical_admin_info')
  list_item(:clinical_staff_info, :id => 'clinical_staff_info')
  list_item(:veteran_info, :id => 'veteran_info')

  #External Link Modal
  h3(:modal_header, id:'heading-enable-protocol')
  paragraph(:modal_text, id: 'modal-text')
  button(:cancel_modal, id: 'cancel-btn')
  link(:continue_modal, id: 'continue-btn')
  paragraph(:training_text, id: 'training-text')

  #Function that calls the wait and validation of the session time out warning functions below
  def waitAndValidate(waitTime, validateValue)
    self.waitForSessionWarningMessage(waitTime)
    self.validateSessionWarningMessage(validateValue)
  end


#Function waits a set amount of time to trigger the session time out warning dialog message
  def waitForSessionWarningMessage(time)
    self.wait_until(time, "Sessions Warning Message did not appear") do
      self.sessionTimeOutWarningHeading_element.visible?
    end

  end

  #Function validates the session time out warning dialog message
  def validateSessionWarningMessage(time)
    self.sessionTimeOutWarningHeading == "Warning: Login Session Ending"
    self.warningText == "To help ensure protect your information, your user session times out after 15 minutes.\nIf you are using your mobile apps, simply select Continue to reset the session. You can also select Log Out to end your user session now."
    self.continue_element.visible? == true
    self.logout_element.visible? == true

    if time > 1 then
      self.timer == "Your user session will time out in #{time} minutes."
    else
      self.timer == "Your user session will time out in #{time} minute."
    end

  end

  #Function that calls the wait and validation of the session time out functions below
  def waitAndValidateTimeout(waitTime)
    self.waitForSessionTimeoutMessage(waitTime)
    self.validateSessionTimeOutMessage()
  end

  #Function waits a set amount of time to trigger the session time out dialog message
  def waitForSessionTimeoutMessage(time)
    self.wait_until(time, "Sessions Warning Message did not appear") do
    self.sessionTimeOutHeading_element.visible?
    end

  end

  #Function validates the session time out dialog message
  def validateSessionTimeOutMessage()
    self.sessionTimeOutHeading == "Login Session Ended"
    self.timeOutText == "Click close to close the application. Click Login to log back into the application you were using"
    self.login_element.visible? == true
  end
end
