class PatientProtocolListPage < StaffPage
  include PageObject

  h2(:active_protocols_heading, id: 'heading-active-protocols')
  h2(:inactive_protocols_heading, id: 'heading-inactive-protocols')
  h2(:no_protocols_heading, id: 'heading-no-protocols')

  link(:account_settings, :id => 'account-settings')
  button(:New_Protocol, :id => 'assign-protocol-btn')

  indexed_property(:protocol_list, [
      [:span, :name, {:id => 'protocol[%s].name'}]
  ])

  indexed_property(:patient_protocol_list, [
      [:span, :name, {:id => 'patient-protocol[%s].name'}],
      [:button, :activeprotocolaudit, {:id => 'patient-protocol[%s].audit'}],
      [:radio_button, :activeprotocolenable, {:id => 'patient-protocol[%s].enable'}],
      [:radio_button, :activeprotocoldisable, {:id => 'patient-protocol[%s].disable'}],
      [:span, :assigned_date, {:id => 'patient-protocol[%s].assigned-date'}],
      [:span, :edited_date, {:id => 'patient-protocol[%s].lastedit-date'}],
      [:span, :edited_by, {:id => 'patient-protocol[%s].edit-by'}], 
      [:span, :last_comment, {:id => 'patient-protocol[%s].comment'}],
      [:link, :edit, {:id => 'patient-protocol[%s].edit'}]
  ])

  indexed_property(:inactivepatient_protocol_list, [
      [:span, :name, {:id => 'inactive-patient-protocol[%s].name'}],
      [:button, :activeprotocolaudit, {:id => 'inactive-patient-protocol[%s].audit'}],
      [:radio_button, :inactiveprotocolenable, {:id => 'inactive-patient-protocol[%s].enable'}],
      [:radio_button, :inactiveprotocoldisable, {:id => 'inactive-patient-protocol[%s].disable'}],
      [:span, :assigned_date, {:id => 'inactive-patient-protocol[%s].assigned-date'}],
      [:span, :edited_date, {:id => 'inactive-patient-protocol[%s].lastedit-date'}],
      [:span, :edited_by, {:id => 'inactive-patient-protocol[%s].edited-by'}],
      [:span, :inactivetype, {:id => 'inactive-patient-protocol[%s].inactive-type'}],
      [:span, :last_comment, {:id => 'inactive-patient-protocol[%s].inactive-last-comment'}],
      [:span, :expired, {:id => 'inactive-patient-protocol[%s].lastedit-date'}]
  ])

  span(:patient_details_name, :id => 'patient-details-name')
  div(:protocol_status, {:id => 'protocol-status'})

  span(:no_assigned_protocols_message, :id => 'no-assigned-protocols-message')
  span(:no_active_protocols_message, :id => 'no-active-protocols-message')
  span(:no_inactive_protocols_message, :id => 'no-inactive-protocols-message')

  text_area(:text_area, :name => 'commentField')
  button(:comment_okay, :id => 'ok')
  button(:enable_okay, :id => 'ok1')
  button(:comment_cancel, :id => 'cancel')
  button(:enable_cancel, :id => 'cancel1')
  span(:error_validation, :id => 'comment-error')
  form(:comment_form, :name => 'commentForm')

  link(:protocol_blood_pressure1, :link_text => 'Blood Pressure Protocol1')
  link(:protocol_blood_pressure2, :link_text => 'Blood Pressure Protocol2')
  table(:template_table, :id => 'assign')
  link(:listTemplatesLink, :link_text => 'List Templates')
  link(:listPatientLink, :link_text => 'List Patients')

  span(:loading, :id=>'patient-protocol-loading')

  # Disable a protocol
  def disableProtocol(protocolName)
    patientProtocol = getProtocolByName(protocolName)

    # disable protocol
    patientProtocol.select_activeprotocoldisable
    self.text_area = "comment"
    self.comment_okay
  end

  # Enable a protocol
  def enableProtocol(protocolName)
    patientProtocol = getProtocolByName(protocolName)

    # disable protocol
    patientProtocol.select_activeprotocolenable
    self.text_area = "comment"
    self.comment_okay
  end

  # get a reference to the protocol list item
  def getProtocolByName(protocolName)
    index = self.get_index_by_text(self.patient_protocol_list, protocolName)

    # if element not found, return nil
    if index == -1
      return nil
    end

    self.patient_protocol_list[index]
  end
  
  # get a reference to the protocol list item
  def getProtocolByName(protocolName)
    @list = self.inactivepatient_protocol_list
    @index = self.get_index_by_text(@list, protocolName)
    return self.inactivepatient_protocol_list[@index]
  end

  def wait_for_page_to_load
    #Allow a fairly long time for initial page load (mae to agilexhealth takes a long time for first secured request)
    self.wait_until(60, "Protocols never loaded") do
      !self.loading_element.visible?
    end
  end


  # Enable an inactive protocol
  def enableInactiveProtocol(protocolName)
    patientProtocol = getInactiveProtocolByName(protocolName)

    # disable protocol
    patientProtocol.select_inactiveprotocolenable
    self.text_area = "comment"
    self.enable_okay
  end

  # get a reference to the protocol list item
  def getInactiveProtocolByName(protocolName)
    index = self.get_index_by_text(self.inactivepatient_protocol_list, protocolName)

    # if element not found, return nil
    if index == -1
      return nil
    end

    self.inactivepatient_protocol_list[index]
  end

end
