class PatientSearchPage < StaffPage
  include PageObject

  page_url PROVIDER_APP_URL

  link(:get_templates, :link_text => 'Templates')
  link(:list_patients, :link_text => 'List Patients')

  text_field(:patient_search_text, :id => 'input-sm-search-bar')
  button(:patient_go, :id => 'input-content-search-bar')

  # search result found
  indexed_property(:patient_list, [
      [:link, :arrow, {:id => 'patient[%s].arrow'}],
      [:h2, :name, {:id => 'patient[%s].name'}],
      [:span, :dob, {:id => 'patient[%s].dob'}],
      [:span, :age, {:id => 'patient[%s].age'}],
      [:span, :mobile, {:id => 'patient[%s].mobile'}],
      [:span, :gender, {:id => 'patient[%s].gender'}],
      [:span, :ssn, {:id => 'patient[%s].ssn'}]
  ])

  unordered_list(:patients_table, :id => 'patient-list')
  unordered_list(:pagination, :id => 'protocol-page')

  paragraph(:search_again_text, :id => 'search-again-text')

  link(:account_settings, :id => 'nav-account-settings')
  link(:sign_out, :id => 'nav-sign-out')

  # search result not found
  h2(:heading_not_found, :id => 'heading-patient-not-found')
  text_field(:patient_not_found, :id => 'no-patient-found-text')
  button(:register, :id => 'register-btn')

  button(:template_save, :id => 'tmpl-save')

  link(:user_nav, id: 'nav-user')
  # this about element is duplicated on admin page, should just be on global
  link(:about, :id => 'nav-about')

  span(:loading, :id=>'patient-search-loading')

#********************************************************************************************************
#     Global Patient Search functions
#********************************************************************************************************
  def global_search(search_param)
    self.main_search = search_param
    self.main_go_element.when_visible.click
    self.waitForLoadingToClear
  end

  def global_search_patient_and_select(patient_name, search_result, wait_to_load = true)
    self.global_search_patient(patient_name)

    index = -1
    self.wait_until(60, "Patient search result not found") do
      index = self.get_index_by_text(self.patient_list, search_result)
      index != -1
    end

    patient = self.patient_list[index]
    patient.name_element.when_visible.click
    self.waitForPageToLoad

    page = UpdatePatientProtocolListPage.new(@browser)
    if wait_to_load
      page.waitForLoadingToClear
    end

    page
  end

  def global_search_patient(patient_name)
    self.patient_search_element.when_visible.click
    self.global_search patient_name

    return self
  end
#********************************************************************************************************

#********************************************************************************************************
#     Main Patient Search functions
#********************************************************************************************************
  def search(search_param)
    self.patient_search_text = search_param
    self.patient_go_element.when_visible.click
    self.waitForLoadingToClear
  end

  def search_patient_and_select(patient_name, search_result, wait_for_protocols_to_load = true)
    self.search_patient(patient_name)

    self.waitForLoadingToClear
    index = self.get_index_by_text(self.patient_list, search_result)

    patient = self.patient_list[index]
    patient.name_element.when_visible.click

    page = UpdatePatientProtocolListPage.new(@browser)
    if wait_for_protocols_to_load
      page.waitForLoadingToClear
    end

    page
  end

  def search_patient(patient_name)
    self.patient_search_element.when_visible.click
    self.search patient_name

    return self
  end
#********************************************************************************************************

  def waitForPatientsToLoad(search_result)
    #Allow a fairly long time for initial page load (mae to agilexhealth takes a long time for first secured request)
    pageLoadWait = Selenium::WebDriver::Wait.new(:timeout => 60) # seconds
    pageLoadWait.until { self.get_link_by_text(search_result) }
  end

  def authentication_delay
    page = PatientSearchPage.new(browser)
  end

  def gotoProtocolTemplates
    self.templates_element.click
    templatesPage = ListTemplatesPage.new(@browser)
    templatesPage.waitForLoadingToClear
    return templatesPage
  end

  def waitForPageToLoad
    #Allow a fairly long time for initial page load (mae to agilexhealth takes a long time for first secured request)
    pageLoadWait = Selenium::WebDriver::Wait.new(:timeout => 120) # seconds
    # pageLoadWait.until { self.get_link_by_text('nav-about') }
    pageLoadWait.until { self.patient_go_element }
    # pageLoadWait.until { self.about_element }
  end
end
