class StaffPage < GlobalObjectsPage
  include PageObject

  # user navigation module
  link(:user_nav, id: 'nav-user')
  link(:about_nav, id: 'nav-about')
  link(:switch_admin_view, id: 'nav-admin-view')
  link(:account_settings, :id => 'nav-account-settings')
  link(:sign_out, :id => 'nav-sign-out')

  # top search module
  text_field(:main_search, id: 'input-main-search-bar')
  button(:main_go, id: 'go-btn-main')

  # Navigation "tabs"
  div(:patient_search, id: 'patient-search')
  div(:templates, id: 'template-list')
  div(:protocols_tab, id: 'protocols-list')
  div(:message_history, id: 'message-history')
  div(:reports, id: 'reports')

  # Making no assumptions about current location, perform navigations to go to patient search
  # Returns PatientSearchPage
  def goto_patient_search_page
    page = PatientSearchPage.new(@browser)
    page.patient_search_element.click
    page
  end

  # Making no assumptions about current location, perform navigations to start the creation of a new template
  # Returns TemplateEditPage
  def goto_new_template_page
    page = PatientSearchPage.new(@browser)
    page = page.gotoProtocolTemplates
    page.create_new_template

    page = TemplateEditPage.new(@browser)
    page.waitForLoadingToClear
    page
  end

  def goto_admin_page
    self.user_nav_element.when_visible.click
    self.switch_admin_view_element.when_visible.click

    AdminPage.new(@browser)
  end

  def waitForLoadingToClear
    # allowTimeoutError { self.loading_text_element.when_visible} #make sure it is showing
    allowTimeoutError {
      self.loading_text_element.when_visible(3)
    } #make sure it is showing

    page_wait = Selenium::WebDriver::Wait.new(:timeout => 120)
    page_wait.until { !self.loading_text_element.visible? }

    # self.loading_text_element.when_not_visible #and wait for it to disappear
  end


  def allowTimeoutError(&blk)
    yield
  rescue Selenium::WebDriver::Error::TimeOutError
    false
  end

  #********************************************************
  def navigate_to_protocols_tab
    self.protocols_tab_element.click

    page = PatientProtocolListPage.new(@browser)
    page.waitForLoadingToClear

    page
  end

  #********************************************************
  def goto_staff_signout
    self.sign_out_element.when_present.click
  end

end
