require 'page-object'

class UpdatePatientProtocolEditPage < StaffPage
  include PageObject

  h1(:protocol_name, id: 'heading-protocols')
  h1(:description, id: 'template-description')
  span(:template_name, id: 'heading-template')
  span(:template_schedule_heading, id: 'heading-template-schedule')
  h3(:schedule_settings, id: 'heading-schedule-settings')
  h3(:reading_settings, id: 'heading-readings-settings')
  h3(:message_settings, id: 'heading-message-settings')
  hidden_field(:protocol_keyword, id: 'keywords')
  select_list(:protocol_schedtype, id: 'p-schedule-type')
  select_list(:protocol_whattimehour, id: 'p-hour')
  select_list(:protocol_whattimemin, id: 'p-min')
  select_list(:protocol_whattimeampm, id: 'p-am-pm')
  radio_button(:protocol_unscheduled, id: 'p-unscheduled')
  radio_button(:protocol_unscheduledno, id: 'p-unscheduledNo')
  text_field(:protocol_timebtwneachcycle, id: 'new-tmpl-number-entry')
  select_list(:protocol_timebtwneachcycleunit, id: 'new-tmpl-cycle-unit')
  text_field(:protocol_startdate, id: 'p-start-date')
  text_field(:protocol_enddate, id: 'p-end-date')
  hidden_field(:measurement, id: 'measurement')
  text_field(:lowest_validreading_one, id: 'lowest-valid-reading-1')
  text_field(:highest_validreading_one, id: 'highest-valid-reading-1')
  text_field(:message_validreading_one, id: 'normal-limit-msg')
  text_field(:lowlevel_alert_one, id: 'low-alert-1')
  text_field(:highlevel_alert_one, id: 'high-alert-1')
  text_field(:critical_lowlevel_one, id: 'low-critical-1')
  text_field(:critical_highlevel_one, id: 'high-critical-1')
  text_field(:message_criticallowalert, id: 'below-critical-level-msg')
  text_field(:message_criticalhighalert, id: 'above-critical-level-msg')
  text_field(:below_desirable_level_msgone, id: 'below-desirable-level-msg1')
  text_field(:above_desirable_level_msgone, id: 'above-desirable-level-msg1')
  text_field(:lowest_validreading_two, id: 'lowest-valid-reading-2')
  text_field(:highest_validreading_two, id: 'highest-valid-reading-2')
  text_field(:critical_lowlevel_two, id: 'low-critical-2')
  text_field(:critical_highlevel_two, id: 'high-critical-2')
  text_field(:lowlevel_alert_two, id: 'low-alert-2')
  text_field(:highlevel_alert_two, id: 'high-alert-2')
  checkbox(:protocol_mon, id: 'p-mon')
  checkbox(:protocol_tues, id: 'p-tues')
  checkbox(:protocol_wed, id: 'p-wed')
  checkbox(:protocol_thurs, id: 'p-thurs')
  checkbox(:protocol_fri, id: 'p-fri')
  checkbox(:protocol_sat, id: 'p-sat')
  checkbox(:protocol_sun, id: 'p-sun')
  button(:protocol_save_top, id: 'save-top')
  button(:protocol_cancel_top, id: 'p-cancel')
  button(:protocol_save_bottom, id: 'save-bottom')
  button(:protocol_cancel_bottom, id: 'cancel-bottom')
  text_field(:protocol_servicemessage, id: 'first-prompt-msg')
  text_field(:protocol_remindermessage, id: 'reminder-prompt-msg')
  text_field(:protocol_timebtwn, id: 'time-btw-reminders-value')
  select_list(:protocol_remindertime, id: 'time-btw-reminders-unit')
  text_field(:protocol_remindertimesent, id: 'readingreminders')
  indexed_property(:message_list, [
    [:text_field, :motivational_service_message, {id: 'motivational-education[%s].message'}],
    [:paragraph, :motivational_characters_remaining, {id: 'motivational-education[%s].remaining-characters'}]
  ])
  link(:add_message, id: 'add-message')
  link(:remove_message, id: 'remove-message')
  paragraph(:remove_message_text, id: 'remove-message-confirm-text')
  button(:remove_message_yes, id: 'yes_remove_message')
  button(:remove_message_no, id: 'no_remove_message')
  div(:save_msg, id: 'okay-text')
  button(:modalwindow_ok, id: 'okay')
  button(:modalwindow_cancelyes, id: 'cancel-yes')
  button(:modalwindow_cancelno, id: 'cancel-no')
  span(:errormessage, id: 'error_msg')

  link(:save_button_top, id: 'save-top')
  link(:cancel_button_top, id: 'p-cancel')
  link(:save_button_bottom, id: 'save-bottom')
  link(:cancel_button_bottom, id: 'cancel-bottom')

  #IDs for the custom template type:
  indexed_property(:custom_list, [
    [:text_field, :custom_description, {id: 'templates[%s].template-description'}],
    [:span, :custom_template_name, {id: 'templates[%s].heading-template'}],
    [:span, :custom_template_schedule_heading, {id: 'templates[%s].heading-template-schedule'}],

    # Template accordions
    [:h3, :custom_schedule_settings, {id: 'templates[%s].heading-schedule-settings'}],
    [:h3, :custom_reading_settings, {id: 'templates[%s].heading-readings-settings'}],
    [:h3, :custom_message_settings, {id: 'templates[%s].heading-message-settings'}],

    # Schedule settings
    [:hidden_field, :custom_protocol_keyword, {id: 'templates[%s].keywords'}],
    [:select_list, :custom_protocol_schedtype, {id: 'templates[%s].p-schedule-type'}],
    [:select_list, :custom_protocol_whattimehour, {id: 'templates[%s].p-hour'}],
    [:select_list, :custom_protocol_whattimemin, {id: 'templates[%s].p-min'}],
    [:select_list, :custom_protocol_whattimeampm, {id: 'templates[%s].p-am-pm'}],
    [:radio_button, :custom_protocol_unscheduled, {id: 'templates[%s].p-unscheduled'}],
    [:radio_button, :custom_protocol_unscheduledno, {id: 'templates[%s].p-unscheduledNo'}],
    [:text_field, :custom_protocol_timebtwneachcycle, {id: 'templates[%s].new-tmpl-number-entry'}],
    [:select_list, :custom_protocol_timebtwneachcycleunit, {id: 'templates[%s].new-tmpl-cycle-unit'}],
    [:text_field, :custom_protocol_startdate, {id: 'templates[%s].p-start-date'}],
    [:text_field, :custom_protocol_enddate, {id: 'templates[%s].p-end-date'}],

    # Reading settings
    [:hidden_field, :custom_measurement, {id: 'templates[%s].measurement'}],
    [:text_field, :custom_lowest_validreading_one, {id: 'templates[%s].lowest-valid-reading-1'}],
    [:text_field, :custom_highest_validreading_one, {id: 'templates[%s].highest-valid-reading-1'}],
    [:text_field, :custom_message_validreading_one, {id: 'templates[%s].normal-limit-msg'}],
    [:text_field, :custom_lowlevel_alert_one, {id: 'templates[%s].low-alert-1'}],
    [:text_field, :custom_highlevel_alert_one, {id: 'templates[%s].high-alert-1'}],
    [:text_field, :custom_critical_lowlevel_one, {id: 'templates[%s].low-critical-1'}],
    [:text_field, :custom_critical_highlevel_one, {id: 'templates[%s].high-critical-1'}],
    [:text_field, :custom_message_criticallowalert, {id: 'templates[%s].below-critical-level-msg'}],
    [:text_field, :custom_message_criticalhighalert, {id: 'templates[%s].above-critical-level-msg'}],
    [:text_field, :custom_below_desirable_level_msgone, {id: 'templates[%s].below-desirable-level-msg1'}],
    [:text_field, :custom_above_desirable_level_msgone, {id: 'templates[%s].above-desirable-level-msg1'}],
    [:text_field, :custom_lowest_validreading_two, {id: 'templates[%s].lowest-valid-reading-2'}],
    [:text_field, :custom_highest_validreading_two, {id: 'templates[%s].highest-valid-reading-2'}],
    [:text_field, :custom_critical_lowlevel_two, {id: 'templates[%s].low-critical-2'}],
    [:text_field, :custom_critical_highlevel_two, {id: 'templates[%s].high-critical-2'}],
    [:text_field, :custom_lowlevel_alert_two, {id: 'templates[%s].low-alert-2'}],
    [:text_field, :custom_highlevel_alert_two, {id: 'templates[%s].high-alert-2'}],
    [:checkbox, :custom_protocol_mon, {id: 'templates[%s].p-mon'}],
    [:checkbox, :custom_protocol_tues, {id: 'templates[%s].p-tues'}],
    [:checkbox, :custom_protocol_wed, {id: 'templates[%s].p-wed'}],
    [:checkbox, :custom_protocol_thurs, {id: 'templates[%s].p-thurs'}],
    [:checkbox, :custom_protocol_fri, {id: 'templates[%s].p-fri'}],
    [:checkbox, :custom_protocol_sat, {id: 'templates[%s].p-sat'}],
    [:checkbox, :custom_protocol_sun, {id: 'templates[%s].p-sun'}],
    [:text_field, :custom_protocol_servicemessage, {id: 'templates[%s].first-prompt-msg'}],
    [:text_field, :custom_protocol_remindermessage, {id: 'templates[%s].reminder-prompt-msg'}],
    [:text_field, :custom_protocol_timebtwn, {id: 'templates[%s].time-btw-reminders-value'}],
    [:select_list, :custom_protocol_remindertime, {id: 'templates[%s].time-btw-reminders-unit'}],
    [:text_field, :custom_protocol_remindertimesent, {id: 'templates[%s].readingreminders'}],
    [:text_field, :custom_motivational_service_message1, {id: 'templates[%s].motivational-education[0].message'}],
    [:paragraph, :custom_motivational_characters_remaining1, {id: 'templates[%s].motivational-education[0].remaining-characters'}],
    [:text_field, :custom_motivational_service_message2, {id: 'templates[%s].motivational-education[1].message'}],
    [:paragraph, :custom_motivational_characters_remaining2, {id: 'templates[%s].motivational-education[1].remaining-characters'}],
    [:text_field, :custom_motivational_service_message3, {id: 'templates[%s].motivational-education[2].message'}],
    [:paragraph, :custom_motivational_characters_remaining3, {id: 'templates[%s].motivational-education[2].remaining-characters'}],
    [:text_field, :custom_motivational_service_message4, {id: 'templates[%s].motivational-education[3].message'}],
    [:paragraph, :custom_motivational_characters_remaining4, {id: 'templates[%s].motivational-education[3].remaining-characters'}],
    [:text_field, :custom_motivational_service_message5, {id: 'templates[%s].motivational-education[4].message'}],
    [:paragraph, :custom_motivational_characters_remaining5, {id: 'templates[%s].motivational-education[4].remaining-characters'}],
    [:link, :custom_add_message, {id: 'templates[%s].add-message'}],
    [:link, :custom_remove_message, {id: 'templates[%s].remove-message'}],

    # Validation error summary
    [:div, :error_summary, {:id => 'templates[%s].error-summary'}],


    # Validation error schedule settings elements
    [:div, :error_template_startdate_one, {:id => 'templates[%s].start-date-1-error'}],
    [:div, :error_template_enddate_one, {:id => 'templates[%s].end-date-1-error'}],
    [:div, :error_template_startdate_two, {:id => 'templates[%s].start-date-2-error'}],
    [:div, :error_template_enddate_two, {:id => 'templates[%s].end-date-2-error'}],
    [:paragraph, :custom_select_days_error, {:id => 'templates[%s].days-error'}],


    # Validation error reading settings elements
    [:div, :error_lowest_validreading_one, {:id => 'templates[%s].lowest-valid-reading-1-error'}],
    [:div, :error_highest_validreading_one, {:id => 'templates[%s].highest-valid-reading-1-error'}],
    [:paragraph, :error_message_validreading_one, {:id => 'templates[%s].valid-reading-message-error'}],

    [:div, :error_critical_lowlevel_one, {:id => 'templates[%s].low-critical-level-alert-1-error'}],
    [:div, :error_critical_highlevel_one, {:id => 'templates[%s].high-critical-level-alert-1-error'}],
    [:paragraph, :error_message_criticallowalert, {:id => 'templates[%s].critical-low-alert-message-error'}],
    [:paragraph, :error_message_criticalhighalert, {:id => 'templates[%s].critical-high-alert-message-error'}],

    [:div, :error_lowlevel_alert_one, {:id => 'templates[%s].low-level-alert-1-error'}],
    [:div, :error_highlevel_alert_one, {:id => 'templates[%s].high-level-alert-1-error'}],
    [:paragraph, :error_below_desirable_level_msgone, {:id => 'templates[%s].low-alert-message-error'}],
    [:paragraph, :error_above_desirable_level_msgone, {:id => 'templates[%s].high-alert-message-error'}],

    [:div, :error_lowest_validreading_two, {id: 'templates[%s].lowest-valid-reading-2-error'}],
    [:div, :error_highest_validreading_two, {id: 'templates[%s].highest-valid-reading-2-error'}],

    [:div, :error_critical_lowlevel_two, {id: 'templates[%s].low-critical-level-alert-2-error'}],  # Element does not appear on the page
    [:div, :error_critical_highlevel_two, {id: 'templates[%s].high-critical-level-alert-2-error'}],  # Element does not appear on the page

    [:div, :error_lowlevel_alert_two, {id: 'templates[%s].low-level-alert-2-error'}],  # Element does not appear on the page
    [:div, :error_highlevel_alert_two, {id: 'templates[%s].high-level-alert-2-error'}],  # Element does not appear on the page

    # Validation error message settings elements
    [:paragraph, :error_protocol_servicemessage, {:id => 'templates[%s].service-message-error'}],
    [:paragraph, :error_protocol_remindermessage, {:id => 'templates[%s].reminder-message-error'}],
    [:paragraph, :error_protocol_timebtwn, {:id => 'templates[%s].reminder-time-input-error'}],
    [:paragraph, :error_protocol_remindertimesent, {:id => 'templates[%s].reminder-time-sent-error'}],

    #Validation error motivational message elements
    [:paragraph, :custom_motivational_service_message1_error, {:id => 'templates[%s].motivational-education[0].message-error'}],
    [:paragraph, :custom_motivational_service_message2_error, {:id => 'templates[%s].motivational-education[1].message-error'}],
    [:paragraph, :custom_motivational_service_message3_error, {:id => 'templates[%s].motivational-education[2].message-error'}],
    [:paragraph, :custom_motivational_service_message4_error, {:id => 'templates[%s].motivational-education[3].message-error'}],
    [:paragraph, :custom_motivational_service_message5_error, {:id => 'templates[%s].motivational-education[4].message-error'}]
  ])


  def saveProtocol
    self.protocol_save_bottom_element.when_visible.click
    self.modalwindow_ok_element.when_visible.click
    self.waitForLoadingToClear

    UpdatePatientProtocolListPage.new(@browser)
  end

  def waitForLoadingToClear
    super
  end

  alias_method :saveTemplate, :saveProtocol
  alias_method :waitForEditsToLoad, :waitForLoadingToClear

  def dismiss_modal
    self.modalwindow_ok
    self.waitForLoadingToClear
  end


end
