class UpdatePatientProtocolListPage < StaffPage
  include PageObject

  h2(:active_protocols_heading, id: 'heading-active-protocols')
  h2(:inactive_protocols_heading, id: 'heading-inactive-protocols')
  h2(:no_protocols_heading, id: 'heading-no-protocols')

  button(:New_Protocol, id: 'assign-protocol-btn')
  link(:update_protocol_list_account_settings, :id => 'account-settings')

  indexed_property(:protocol_list, [
      [:span, :name, {:id => 'protocol[%s].name'}],
      [:button, :assign, {:id => 'protocol[%s].assign'}],
      [:span, :desc, {:id => 'protocol[%s].desc'}],
      [:span, :template, {:id => 'protocol[%s].template'}],
      [:span, :created, {:id => 'protocol[%s].created'}],
      [:span, :focus, {:id => 'protocol[%s].focus'}],
      [:span, :status, {:id => 'protocol[%s].status'}]
  ])

  indexed_property(:patient_protocol_list, [
      [:span, :name, {:id => 'patient-protocol[%s].name'}],
      [:button, :activeprotocolaudit, {:id => 'patient-protocol[%s].audit'}],
      [:radio_button, :activeprotocolenable, {:id => 'patient-protocol[%s].enable'}],
      [:radio_button, :activeprotocoldisable, {:id => 'patient-protocol[%s].disable'}],
      [:span, :assigned_date, {:id => 'patient-protocol[%s].assigned-date'}],
      [:span, :edited_date, {:id => 'patient-protocol[%s].lastedit-date'}],
      [:span, :edited_by, {:id => 'patient-protocol[%s].edit-by'}],
      [:span, :last_comment, {:id => 'patient-protocol[%s].comment'}],
      [:link, :edit_temp, {:id => 'patient-protocol[%s].edit_temp'}],
      [:link, :edit, {:id => 'patient-protocol[%s].edit_temp'}]
  ])

  indexed_property(:inactivepatient_protocol_list, [
      [:span, :name, {:id => 'inactive-patient-protocol[%s].name'}],
      [:button, :activeprotocolaudit, {:id => 'inactive-patient-protocol[%s].audit'}],
      [:radio_button, :inactiveprotocolenable, {:id => 'inactive-patient-protocol[%s].enable'}],
      [:radio_button, :inactiveprotocoldisable, {:id => 'inactive-patient-protocol[%s].disable'}],
      [:span, :assigned_date, {:id => 'inactive-patient-protocol[%s].assigned-date'}],
      [:span, :edited_date, {:id => 'inactive-patient-protocol[%s].lastedit-date'}],
      [:span, :edited_by, {:id => 'inactive-patient-protocol[%s].edited-by'}],
      [:span, :inactivetype, {:id => 'inactive-patient-protocol[%s].inactive-type'}],
      [:span, :last_comment, {:id => 'inactive-patient-protocol[%s].inactive-last-comment'}],
      [:span, :expired, {:id => 'inactive-patient-protocol[%s].lastedit-date'}]
  ])

  span(:patient_details_name, :id => 'patient-details-name')
  div(:protocol_status, {:id => 'protocol-status'})

  span(:no_assigned_protocols_message, :id => 'no-assigned-protocols-message')
  span(:no_active_protocols_message, :id => 'no-active-protocols-message')
  span(:no_inactive_protocols_message, :id => 'no-inactive-protocols-message')

  text_area(:text_area, :name => 'commentField')
  button(:comment_okay, :id => 'ok')
  button(:enable_okay, :id => 'ok1')
  button(:comment_cancel, :id => 'cancel')
  button(:enable_cancel, :id => 'cancel1')
  span(:error_validation, :id => 'comment-error')
  form(:comment_form, :name => 'commentForm')

  link(:protocol_blood_pressure1, :link_text => 'Blood Pressure Protocol1')
  link(:protocol_blood_pressure2, :link_text => 'Blood Pressure Protocol2')
  table(:template_table, :id => 'assign')
  link(:listTemplatesLink, :link_text => 'List Templates')
  link(:listPatientLink, :link_text => 'List Patients')

  button(:customizeprotocol_yes_button, :id => 'yes')

  button(:customizeprotocol_no, :id => 'no')

  button(:customizeprotocol_cancel, :id => 'cancel')
  span(:loading, :id=>'patient-protocol-loading')


  def assignProtocolByName(protocolName)
    auditTemplateProtocol = nil
    self.wait_until(120, "Protocol never loaded") do
      auditTemplateProtocol = self.getAvailableProtocolByName(protocolName)
    end
    auditTemplateProtocol.assign
  end

  # Disable a protocol
  def disableProtocol(protocolName)
    patientProtocol = getProtocolByName(protocolName)

    # disable protocol
    patientProtocol.select_activeprotocoldisable
    self.text_area = "comment"
    self.comment_okay
    self.wait_until(60, "Protocols never disabled") do
      self.getInactiveProtocolByName(protocolName)
    end

  end

  # Enable a protocol
  def enableProtocol(protocolName)
    patientProtocol = getInactiveProtocolByName(protocolName)

    # enable protocol
    patientProtocol.select_inactiveprotocolenable
    self.text_area = "comment"
    self.enable_okay
  end


  # get a reference to the protocol list item
  def getAvailableProtocolByName(protocolName)
    wait_for_available_protocols_to_load
    index = self.get_index_by_text(self.protocol_list, protocolName)
    # if element not found, return nil
    if index == -1
      return nil
    end

    self.protocol_list[index]
  end

  # get a reference to the protocol list item
  def getProtocolByName(protocolName)
    index = self.get_index_by_text(self.patient_protocol_list, protocolName)
    # if element not found, return nil
    if index == -1
      return nil
    end

    self.patient_protocol_list[index]
  end

  # get a reference to the protocol list item
  def getInactiveProtocolByName(protocolName)
    index = self.get_index_by_text(self.inactivepatient_protocol_list, protocolName)
    # if element not found, return nil
    if index == -1
      return nil
    end

    self.inactivepatient_protocol_list[index]
  end

  def customizeprotocol_yes
    self.customizeprotocol_yes_button
    self.waitForLoadingToClear

    UpdatePatientProtocolEditPage.new(@browser)
  end

  def wait_for_page_to_load
    self.waitForLoadingToClear
  end

  def wait_for_available_protocols_to_load
    self.waitForLoadingToClear
  end

  def wait_for_active_protocols_to_load
    self.waitForLoadingToClear
  end

  def navigateToStaffAuditTrail(name)
    @list = self.patient_protocol_list
    @index = self.get_index_by_text(@list, name)
    self.patient_protocol_list[@index].activeprotocolaudit
  end

end

