# PatientApp  profile.tpl.html

class VetProfileEditPage < VetRegistrationPage
  include PageObject

  page_url PATIENT_APP_URL

  #****************    Section Headers   **************************

  link(:heading_appointment_info_messages, :id => 'heading-appointment-info-messages')
  link(:heading_preferences, :id => 'heading-preferences')
  link(:heading_patient_info, :id => 'heading-patient-info')

  #****************    Patient Info   **************************

  paragraph(:success_msg, :id => 'success-msg')

  text_field(:firstname, :id => 'profile-firstname')  # Read-only field
  text_field(:middlename, :id => 'profile-middlename')  # Read-only field
  text_field(:lastname, :id => 'profile-lastname')  # Read-only field

  text_field(:dob, :id => 'profile-dob-dob')  # Read-only field
  text_field(:gender, :id => 'profile-gender')  # Read-only field
  text_field(:mobile_phone, :id => 'profile-mobile-phone')
  span(:mobile_phone_notice_error, :id => 'mobile-notice')

  select_list(:time_zone, :id => 'profile-timezone')
  text_field(:home_phone, :id => 'profile-homephone')
  text_field(:email, :id => 'profile-email')

  #****************    Preferences   **************************

  radio_button(:basic_cell, :id => 'profile-pref-basic-cell')
  radio_button(:smart_phone, :id => 'profile-pref-smart-cell')
  span(:select_phone_format_error, :id => 'select-phone-error-msg')

  checkbox(:receive_nonsecure_msgs, :id => 'profile-pref-nonsecure-msg-sms')
  span(:basic_phone_acceptance_error, :id => 'select-profile-pref-nonsecure-msg-sms-error-msg')

  button(:submit, :id => 'profile-submit')
  button(:save, :id => 'profile-save')
  button(:cancel, :id => 'profile-cancel-submit')

  #****************    Consent Form   **************************

  checkbox(:select_patient_consent_chkbox, :id => 'select-patient-consent-form')
  link(:consent_form, :id => 'consent-form')

  span(:consent_date, :id => 'consent-date')
  span(:consent_time, :id => 'consent-time')

  #***********  Consent Modal Window  ******************

  h3(:heading_consent_form, :id => 'heading-consent-form')
  div(:modal_consent_form, :id => 'modal-consent-form')

  button(:button_consent_yes, :id => 'consent-yes')
  button(:button_consent_no, :id => 'consent-no')
  button(:button_consent_dismiss, :id => 'consent-dismiss')

  #**********************************************

  span(:home_phone_format_error, :id => 'profile-homephone-error')
  span(:phone_format_error, :id => 'phone-format-error')
  span(:phone_numeric_error, :id => 'phone-numeric_error')
  span(:timezone_error, :id => 'profile_time_zone_error')
  span(:email_error, :id => 'profile-email-error')
  span(:preferences_error, :id => 'profile_preferences_error')

  paragraph(:consent_form_error, :id => 'select-patient-consent-form-error-msg')

  button(:eula_accept, :id => 'eula-accept')

  paragraph(:duplicate_mobile_phone_error, :id => 'profile-mobile-phone-error-msg')

  #***************   Broadcast Messaging Subscriptions   *******

  checkbox(:facility_checkbox, :id => 'vamc-facility')
  checkbox(:appointment_reminders_checkbox, :id => 'appointment-reminders')
  select_list(:facility, :id => 'facility')
  paragraph(:facility_error, :id => 'select-facility-error-msg')

  #**************************************************************

  def wait_for_eula_page_to_load
    self.wait_until(20, "EULA Page did not load") do
      self.eula_accept?
    end
  end

  #********************************************************
  def wait_for_page_to_load
    self.wait_until(120, "Patient Profile Account Edit Page did not finish loading") do
      !self.firstname.empty?
    end
  end

  #********************************************************
  def wait_for_timezone_to_load
    self.wait_until do
      self.time_zone_options.length > 1
    end
  end

  #********************************************************
  def saveRegistration
    self.save_element.when_visible.click
  end

  #********************************************************
  def submitRegistration
    self.submit_element.when_visible.click
  end

  # Hit the down arrow enough times to scroll to the bottom of the consent form
  # Pre:  The consent modal dialog is open
  # Post: The consent modal dialog is open and the content is scrolled all the way down
  def scroll_consent_form_down
    press_down_arrow_key(self.modal_consent_form_element, 50)
  end
end
