#########################################################################################
# Stories:   CMS-112, CMS-1934, CMS-1957, CMS-1662, CMS-1370, CMS-2002, CMS-283, CMS-632, CMS-1091, CMS-1129, CMS-1168
#
# Tests:   CMS-2122, CMS-2123, CMS-2124, CMS-2356, CMS-3347, CMS-1342
#
# Acceptance Criteria: CMS-154, CMS-1935, CMS-1954, CMS-1955, CMS-1956, CMS-1958, CMS-1959, CMS-1960, CMS-1962
#                       CMS-1963, CMS-1965, CMS-1977, CMS-1979, CMS-1980, CMS-1981, CMS-1982,
#                       CMS-1432, CMS-1433, CMS-1434, CMS-1435, CMS- 1436, CMS-1437, CMS-1440, CMS-1441,
#                       CMS-1442, CMS-2193, CMS-2194, CMS-2195, CMS-392, CMS-393, CMS-808, CMS-809, CMS-1265,
#                       CMS-1266, CMS-1712, CMS-1713
#
#########################################################################################

require 'spec_helper'
require 'time'
require 'date'

# Set up variables
date_format = "%m/%d/%Y"
today_date = DateTime.now.strftime(date_format)
pulse_ox_protocol = ProtocolModel.new(
    :name => "Pulse OX",
    :focus_area => "Pulse, OX, Focus, Area",
    :description => "This is a new protocol created for Pulse OX",
    :templates => TemplateModel.new(
        :template_description => "This is a new protocol created for Pulse OX",
        :measurement => "Pulse Oximetry (SpO2)",
        :readings => TemplateReading.new(
            :valid_range => RangeLimit.new(60, 140),
            :critical_range => RangeLimit.new(65, 95),
            :alert_range => RangeLimit.new(70, 90)
        ),
        :messages => TemplateMessages.new(
            :service_message => "Hello, just a quick reminder that your readings are due. (e.g., OX70). Thanks.",
            :reminder_message => "Hi again. This is your reminder message. Could you please text them in. Kind regards.",
            :valid_reading_message => "Your Pulse OX protocol reading is fine.",
            :alert_low_message => "Your Pulse OX is a little low today. Take it again, and follow the advice in your management plan. Take care.",
            :alert_high_message => "Your Pulse OX is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.",
            :critical_low_message => "Your Pulse OX is CRITICALLY low  today. Take it again, and follow the advice in your management plan. Take care.",
            :critical_high_message => "Your Pulse OX is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks."
        ),
        :schedule => TemplateSchedule.new(
            :schedule => "Custom",
            :time_between_each_cycle => ValueUnits.new("15", "Minute"),
            :service_message_start_time => TimeOfDay.new("11", "30", "PM"),
            :start_date => today_date,
            :end_date => "04/10/2015",
            :start_period => ValueUnits.new("2", "Hour"),
            :end_period => ValueUnits.new("5", "Month")
        ),
        :time_between_reminders => ValueUnits.new("10", "Minute"),
        :reminder_message_count => "4"
    )
)

updated_pulse_ox_protocol = ProtocolModel.new(
    :name => pulse_ox_protocol.name,
    :focus_area => "Pulse, OX, Focus, Area, EDITED!",
    :description => pulse_ox_protocol.description,
    :templates => TemplateModel.new(
        :readings => TemplateReading.new(
            :valid_range => pulse_ox_protocol.get_template.get_reading.valid_range,
            :critical_range => RangeLimit.new(64, 96),
            :alert_range => pulse_ox_protocol.get_template.get_reading.alert_range
        ),
        :messages => TemplateMessages.new(
            :service_message => pulse_ox_protocol.get_template.messages.service_message,
            :reminder_message => pulse_ox_protocol.get_template.messages.reminder_message,
            :valid_reading_message => "Your EDITED Pulse OX protocol reading is fine.",
            :alert_low_message => "Your EDITED Pulse OX is a little low today. Take it again, and follow the advice in your management plan. Take care.",
            :alert_high_message => "Your EDITED Pulse OX is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.",
            :reminder_message => "EDITED! Hi again. This is your reminder message. Could you please text them in. Kind regards. EDITED!",
            :critical_low_message => pulse_ox_protocol.get_template.messages.critical_low_message,
            :critical_high_message => pulse_ox_protocol.get_template.messages.critical_high_message
        ),
        :schedule => TemplateSchedule.new(
            :schedule => pulse_ox_protocol.get_template.schedule.schedule,
            :time_between_each_cycle => pulse_ox_protocol.get_template.schedule.time_between_each_cycle,
            :service_message_start_time => pulse_ox_protocol.get_template.schedule.service_message_start_time,
            :start_date => pulse_ox_protocol.get_template.schedule.start_date,
            :end_date => pulse_ox_protocol.get_template.schedule.end_date,
            :start_period => pulse_ox_protocol.get_template.schedule.start_period,
            :end_period => pulse_ox_protocol.get_template.schedule.end_period
        ),
        :time_between_reminders => pulse_ox_protocol.get_template.time_between_reminders,
        :reminder_message_count => pulse_ox_protocol.get_template.reminder_message_count
    )
)

weight_temperature_protocol = ProtocolModel.new(
    :name => "Weight Temperature Custom",
    :description => "Custom Protocol of Weight and Temperature",
    :focus_area => "Weight, Temperature, Temp",
    :templates => [
        TemplateModel.new(
            :measurement => "Weight Loss",
            :keywords => "WE, WEIGHT",
            :schedule => TemplateSchedule.new(
                :schedule => "Once",
                :service_message_start_time => TimeOfDay.new("8", "17", "PM"),
                :start_period => ValueUnits.new("1", "Minute"),
                :end_period => ValueUnits.new("4", "Year")
            ),
            :readings => TemplateReading.new(
                :valid_range => RangeLimit.new(50, 250),
                :critical_range => RangeLimit.new(55, 245),
                :alert_range => RangeLimit.new(70, 200)
            ),
            :messages => TemplateMessages.new(
                :valid_reading_message => "Your Weight Loss reading is valid",
                :critical_low_message => "Your Weight Loss is CRITICALLY LOW!",
                :critical_high_message => "Your Weight Loss is CRITICALLY HIGH!",
                :alert_low_message => "This is your Weight Loss LOW LEVEL ALERT!",
                :alert_high_message => "This is your Weight Loss HIGH LEVEL ALERT!",
                :service_message => "This is your Weight Loss Service Message.",
                :reminder_message => "This is your Weight Loss Reminder Message."
            ),
            :time_between_reminders => ValueUnits.new("15", "Hour"),
            :reminder_message_count => "122"
        ),
        TemplateModel.new(
            :measurement => "Temperature",
            :keywords => "TEMP",
            :schedule => TemplateSchedule.new(
                :schedule => "Specified Days",
                :service_message_start_time => TimeOfDay.new("6", "13", "AM"),
                :schedule_days => DaysOfTheWeek.new(
                    :monday => true,
                    :wednesday => true,
                    :friday => true,
                    :saturday => true
                ),
                :start_period => ValueUnits.new("3", "Day"),
                :end_period => ValueUnits.new("9", "Week")
            ),
            :readings => TemplateReading.new(
                :valid_range => RangeLimit.new(95, 106),
                :critical_range => RangeLimit.new(97, 104),
                :alert_range => RangeLimit.new(98, 103)
            ),
            :messages => TemplateMessages.new(
                :valid_reading_message => "Your Temperature reading is valid",
                :critical_low_message => "Your Temperature is CRITICALLY LOW!",
                :critical_high_message => "Your Temperature is CRITICALLY HIGH!",
                :alert_low_message => "This is your Temperature LOW LEVEL ALERT!",
                :alert_high_message => "This is your Temperature HIGH LEVEL ALERT!",
                :service_message => "This is your Temperature Service Message.",
                :reminder_message => "This is your Temperature Reminder Message."
            ),
            :time_between_reminders => ValueUnits.new("2", "Minute"),
            :reminder_message_count => "22"
        )
    ]
)

glucose_motivational_protocol = ProtocolModel.new(
    :name => "Glucose Motivational Protocol",
    :description => "Custom Protocol Measuring Glucose and Providing Motivational Messages",
    :focus_area => "Glucose, Motivation",
    :templates => [
        TemplateModel.new(
            :measurement => "Blood Glucose",
            :keywords => "BG, GLUCOSE, SUGAR",
            :schedule => TemplateSchedule.new(
                :schedule => "Specified Days",
                :schedule_days => DaysOfTheWeek.new(
                    :monday => true,
                    :tuesday => true,
                    :wednesday => true,
                    :thursday => true,
                    :friday => true,
                    :saturday => true,
                    :sunday => true
                ),
                :service_message_start_time => TimeOfDay.new("4", "43", "AM"),
                :start_period => ValueUnits.new("3", "Day"),
                :end_period => ValueUnits.new("9", "Week")
            ),
            :readings => TemplateReading.new(
                :valid_range => RangeLimit.new(30, 300),
                :critical_range => RangeLimit.new(35, 280),
                :alert_range => RangeLimit.new(40, 200)
            ),
            :messages => TemplateMessages.new(
                :valid_reading_message => "Your Blood Glucose reading is valid",
                :critical_low_message => "Your Blood Glucose is CRITICALLY LOW!",
                :critical_high_message => "Your Blood Glucose is CRITICALLY HIGH!",
                :alert_low_message => "This is your Blood Glucose LOW LEVEL ALERT!",
                :alert_high_message => "This is your Blood Glucose HIGH LEVEL ALERT!",
                :service_message => "This is your Blood Glucose Service Message.",
                :reminder_message => "This is your Blood Glucose Reminder Message."
            ),
            :time_between_reminders => ValueUnits.new("3", "Minute"),
            :reminder_message_count => "333"
        ),
        TemplateModel.new(
            :template_description => "This is the custom template motivational description.",
            :schedule => TemplateSchedule.new(
                :schedule => "Custom",
                :service_message_start_time => TimeOfDay.new("1", "59", "AM"),
                :time_between_each_cycle => ValueUnits.new("6", "Week"),
                :start_period => ValueUnits.new("1", "Hour"),
                :end_period => ValueUnits.new("7", "Year")
            ),
            :messages => TemplateMessages.new(
                :motivational_message => [
                    "This is my first service message.",
                    "This is my second service message.",
                    "This is my Third service message.",
                    "This is my fourth service message."
                ]
            )
        )
    ]
)

bp_protocol_1 = ProtocolModel.new(
    :name => "Blood Pressure Protocol1",
    :description => "This is blood pressure protocol with once schedule",
    :focus_area => "Blood"
)

bp_protocol_2 = ProtocolModel.new(
    :name => "Blood Pressure Protocol2"
)

bp_protocol_3 = ProtocolModel.new(
    :name => "Blood Pressure Protocol3"
)


# Validate the specified audit trail entry
def validate_audit_trail(audit_page, comments, *alternate_comments, row_index, row_count, user)
  date_obj = DateTime.now
  date_format = "%m/%d/%Y"
  today_date = date_obj.strftime(date_format)
  yesterday_date = (date_obj - 1).strftime(date_format)

  if audit_page == "admin"
    page = AdminAuditTrailPage.new(@browser)
  elsif audit_page == "staff"
    page = AuditTrailPage.new(@browser)
  end

  page.waitForLoadingToClear
  expect(page.audittrail_table_element.rows).to eq(row_count)

  # expect(page.audit_table_edited_time_element(table_index).text).to match(/\d*:\d{2} (AM|PM)/)
  # expect(page.audit_table_edited_by_element(table_index).text).to eq(user)
  expect(page.audit_trail[row_index - 1].editedtime_element.text).to match(/\d*:\d{2} (AM|PM)/)
  expect(page.audit_trail[row_index - 1].editedby_element.text).to eq(user)
  if page.audit_trail[row_index - 1].editedon_element.text != today_date
    expect(page.audit_trail[row_index - 1].editedon_element.text).to eq(yesterday_date)
  end

  if comments.is_a?(String)
    comment_on_page = page.audit_table_audit_trail_comment_element(row_index).text
    if comment_on_page != comments
      expect(comment_on_page).to eq(alternate_comments[0])
    end
  elsif comments.is_a?(Array)
    comments.each_with_index do |current_comment, current_comment_index|
      comment_on_page = page.audit_table_audit_trail_comment_element(row_index + current_comment_index).text
      if comment_on_page != current_comment
        expect(comment_on_page).to eq(alternate_comments[current_comment_index])
      end
    end
  end
end

# Validate an admin audit trail entry
def validate_admin_audit_trail(comments, *alternate_comments, row_index, row_count, user)
  if alternate_comments.empty?
    validate_audit_trail("admin", comments, row_index, row_count, user)
  else
    validate_audit_trail("admin", comments, alternate_comments[0], row_index, row_count, user)
  end
end

# Validate a staff audit trail entry
def validate_staff_audit_trail(comments, *alternate_comments, row_index, row_count, user)
  if alternate_comments.empty?
    validate_audit_trail("staff", comments, row_index, row_count, user)
  else
    validate_audit_trail("staff", comments, alternate_comments[0], row_index, row_count, user)
  end
end


describe 'Create Multiple Protocols to Assign to a user including a Custom Template (CMS-1370A, CMS2002)' do
  # Validate the data for the indicated protocol
  def validate_protocol(test_protocol, test_comment)
    # Set up local variables
    date_obj = DateTime.now
    date_format = "%m/%d/%Y"
    today_date = date_obj.strftime(date_format)
    yesterday_date = (date_obj - 1).strftime(date_format)

    # Start on the List Protocols page
    page = ListTemplatesPage.new(@browser)

    # Find the desired protocol
    protocol_list = page.protocol_list
    protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
    protocol_on_page = page.protocol_list[protocol_index]

    # Verify the protocol data
    expect(protocol_on_page.name).to eq(test_protocol.name)
    expect(protocol_on_page.description).to eq(test_protocol.description)
    if protocol_on_page.created_info != "Created By: #{@full_name}\nCreated On: #{today_date}"
      expect(protocol_on_page.created_info).to eq ("Created By: #{@full_name}\nCreated On: #{yesterday_date}")
    end
    expect(protocol_on_page.focus_area).to eq(test_protocol.focus_area)
    expect(protocol_on_page.last_comment).to eq(test_comment)
  end

  # Method to get the character remaining text for a motivational message field
  def get_motivational_message_characters_remaining_text(message_text)
    "#{160 - message_text.length} characters remaining"
  end


  context 'Create multiple protocols' do
    it 'Update design clinical admin home page (CMS-1957)' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)

      # Navigate to the Clinical Admin page
      page.goto_admin_page
      page = AdminPage.new(@browser)
      page.protocols_tab_element.visible?
      page.add_protocol_element.visible?
    end

    it 'View list of protocols (CMS-154)' do
      # Set up variables
      test_protocol_list =[
          {
              :protocol => bp_protocol_1,
              :created_info => "Created By: One Vehu\nCreated On: 01/05/2014",
              :comment => "Enable protocol"
          },
          { :protocol => bp_protocol_2 },
          { :protocol => bp_protocol_3 },
      ]

      # Start on the Templates page
      page = ListTemplatesPage.new(@browser)

      # Get the list of protocols
      protocol_list = page.protocol_list

      # Verify that the expected protocols are present
      page.get_index_by_text(protocol_list, test_protocol_list[0][:protocol].name).should_not be_nil
      page.get_index_by_text(protocol_list, test_protocol_list[1][:protocol].name).should_not be_nil
      page.get_index_by_text(protocol_list, test_protocol_list[2][:protocol].name).should_not be_nil

      protocol_index = page.get_index_by_text(protocol_list, test_protocol_list[0][:protocol].name)
      protocol_on_page = page.protocol_list[protocol_index]

      expect(protocol_on_page.name).to eq(test_protocol_list[0][:protocol].name)
      expect(protocol_on_page.description).to eq(test_protocol_list[0][:protocol].description)
      expect(protocol_on_page.focus_area).to eq(test_protocol_list[0][:protocol].focus_area)
      expect(protocol_on_page.created_info).to eq(test_protocol_list[0][:created_info])
      expect(protocol_on_page.last_comment).to eq(test_protocol_list[0][:comment])
    end

    it 'Validates that the user can create a vitals only protocol' do
      # Set up variables
      test_protocol = pulse_ox_protocol

      # Start on the Templates page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Add a new protocol
      page.add_protocol
      page = AddNewProtocolPage.new(@browser)
      page.waitForLoadingToClear

      # Fill in the Add Protocol form
      page.protocol_name = test_protocol.name
      page.protocol_description = test_protocol.description
      page.protocol_focus_area = test_protocol.focus_area

      # Move to the next step
      page.next
      page.waitForLoadingToClear
      expect(page.back_element.visible?).to eq(true)

      # Add a vitals template
      page.add_template_element.click
      page.vitals_element.when_visible.click
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear
      vital_template_form = page.custom_list[0]

      # Expand the template Schedule Settings section
      vital_template_form.custom_schedule_settings_element.click
      pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90)
      pageLoadWait.until { vital_template_form.custom_schedule_options != '' }

      # Fill in the Schedule Settings values
      vital_template_form.custom_schedule = test_protocol.get_template.schedule.schedule
      vital_template_form.custom_start_time_hour = test_protocol.get_template.schedule.service_message_start_time.hours
      vital_template_form.custom_start_time_minutes = test_protocol.get_template.schedule.service_message_start_time.minutes
      vital_template_form.custom_am_pm = test_protocol.get_template.schedule.service_message_start_time.am_pm
      vital_template_form.custom_time_between_each_cycle = test_protocol.get_template.schedule.time_between_each_cycle.value
      vital_template_form.custom_time_between_each_cycle_units = test_protocol.get_template.schedule.time_between_each_cycle.units
      vital_template_form.custom_start_period = test_protocol.get_template.schedule.start_period.value
      vital_template_form.custom_select_start_period = test_protocol.get_template.schedule.start_period.units
      vital_template_form.custom_end_period = test_protocol.get_template.schedule.end_period.value
      vital_template_form.custom_select_end_period = test_protocol.get_template.schedule.end_period.units

      # Expand the template Readings/Alert Settings section
      vital_template_form.custom_readings_alerts_settings_element.click

      # Fill in the Readings/Alert Settings section values
      vital_template_form.custom_template_measurement = test_protocol.get_template.measurement

      vital_template_form.custom_lowest_valid_reading = test_protocol.get_template.get_reading.valid_range.lowest
      vital_template_form.custom_highest_valid_reading = test_protocol.get_template.get_reading.valid_range.highest
      vital_template_form.custom_valid_reading_message = test_protocol.get_template.messages.valid_reading_message

      vital_template_form.custom_critical_low_level_alert = test_protocol.get_template.get_reading.critical_range.lowest
      vital_template_form.custom_critical_high_level_alert = test_protocol.get_template.get_reading.critical_range.highest
      vital_template_form.custom_critical_low_level_alert_message = test_protocol.get_template.messages.critical_low_message
      vital_template_form.custom_critical_high_level_alert_message = test_protocol.get_template.messages.critical_high_message

      vital_template_form.custom_low_level_alert = test_protocol.get_template.get_reading.alert_range.lowest
      vital_template_form.custom_high_level_alert = test_protocol.get_template.get_reading.alert_range.highest
      vital_template_form.custom_low_level_alert_message = test_protocol.get_template.messages.alert_low_message
      vital_template_form.custom_high_level_alert_message = test_protocol.get_template.messages.alert_high_message

      # Expand the template Message Settings section
      vital_template_form.custom_message_settings_element.click

      # Fill in the Message Settings section values
      vital_template_form.custom_service_message = test_protocol.get_template.messages.service_message
      vital_template_form.custom_reminder_message = test_protocol.get_template.messages.reminder_message
      vital_template_form.custom_reminder_time_between = test_protocol.get_template.time_between_reminders.value
      vital_template_form.custom_reminder_time_schedule = test_protocol.get_template.time_between_reminders.units
      vital_template_form.custom_reminder_time_sent = test_protocol.get_template.reminder_message_count

      # Submit the form
      page.finish_form

      # Verify success and dismiss the success dialog
      expect(page.congrats_message).to eq('Congratulations! You successfully created a new template for your new protocol.')
      page.success_ok
    end

    it 'Validates that the user can create a custom protocol with 20 vitals and 20 motivational/educational' do
      # Set variables
      test_protocol = weight_temperature_protocol
      vitals_template_quantity = 20
      motivational_template_quantity = 20

      # Start on the Templates page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Begin to add a new protocol
      page.add_protocol
      page = AddNewProtocolPage.new(@browser)
      page.waitForLoadingToClear

      # Fill in the new protocol form
      page.protocol_name = test_protocol.name
      page.protocol_description = test_protocol.description
      page.protocol_focus_area = test_protocol.focus_area

      # Continue to the next protocol creation page
      page.next
      page.waitForLoadingToClear
      page.back_element.visible?

      # Add a custom template
      page.add_template_element.click
      page.waitForLoadingToClear
      page.custom_element.click
      page= AddNewProtocolPage.new(@browser)
      page.waitForLoadingToClear

      # Set up the custom templates
      page.check_vitals_checkbox
      page.vitals_quantity = vitals_template_quantity
      page.check_motivational_educational_checkbox
      page.motivational_educational_quantity = motivational_template_quantity

      # Continue to the template edit page
      page.next
      page = TemplateEditPage.new(@browser)

      # Count the vitals templates available for editing
      index = 0
      while page.custom_list[index].custom_vitals? do
        index += 1
      end

      # Count the motivational templates available for editing
      index += 1  # Increment by 1 before moving to the motivational templates
      while page.custom_list[index].custom_motivational_educational? do
        index += 1
      end

      # Verify that the expected number of templates are available for editing
      expect(index).to eq(vitals_template_quantity + motivational_template_quantity)

      # Go back to the previous page
      page.back
      page.waitForLoadingToClear
    end

    it 'Validates that the user can create a custom protocol with 2 vitals templates' do
      # Set variables
      test_protocol = weight_temperature_protocol

      # Start on the Add New Protocol page
      page= AddNewProtocolPage.new(@browser)

      # Create a custom protocol with two vitals templates
      page.custom_element.click
      page.check_vitals_checkbox
      page.vitals_quantity = test_protocol.count_templates
      page.next
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      for template_index in 0..1 do
        # Expand the template
        page.custom_list[template_index].custom_vitals_element.click

        # Expand the template's schedule settings
        page.custom_list[template_index].custom_schedule_settings_element.click
        pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
        pageLoadWait.until { page.custom_list[template_index].custom_schedule_options != '' }

        # Fill in the template's schedule settings
        page.custom_list[template_index].custom_schedule = test_protocol.get_template(template_index).schedule.schedule

        if test_protocol.get_template(template_index).schedule.schedule == "Specified Days"
          page.custom_list[template_index].check_custom_protocol_mon if test_protocol.get_template(template_index).schedule.schedule_days.monday
          page.custom_list[template_index].check_custom_protocol_tues if test_protocol.get_template(template_index).schedule.schedule_days.tuesday
          page.custom_list[template_index].check_custom_protocol_wed if test_protocol.get_template(template_index).schedule.schedule_days.wednesday
          page.custom_list[template_index].check_custom_protocol_thurs if test_protocol.get_template(template_index).schedule.schedule_days.thursday
          page.custom_list[template_index].check_custom_protocol_fri if test_protocol.get_template(template_index).schedule.schedule_days.friday
          page.custom_list[template_index].check_custom_protocol_sat if test_protocol.get_template(template_index).schedule.schedule_days.saturday
          page.custom_list[template_index].check_custom_protocol_sun if test_protocol.get_template(template_index).schedule.schedule_days.sunday
        end

        if test_protocol.get_template(template_index).schedule.schedule == "Custom"
          page.custom_list[template_index].custom_time_between_each_cycle = test_protocol.get_template(template_index).schedule.time_between_each_cycle.value
          page.custom_list[template_index].custom_time_between_each_cycle_units = test_protocol.get_template(template_index).schedule.time_between_each_cycle.units
        end

        page.custom_list[template_index].custom_start_time_hour = test_protocol.get_template(template_index).schedule.service_message_start_time.hours
        page.custom_list[template_index].custom_start_time_minutes = test_protocol.get_template(template_index).schedule.service_message_start_time.minutes
        page.custom_list[template_index].custom_am_pm = test_protocol.get_template(template_index).schedule.service_message_start_time.am_pm
        page.custom_list[template_index].custom_start_period = test_protocol.get_template(template_index).schedule.start_period.value
        page.custom_list[template_index].custom_select_start_period = test_protocol.get_template(template_index).schedule.start_period.units
        page.custom_list[template_index].custom_end_period = test_protocol.get_template(template_index).schedule.end_period.value
        page.custom_list[template_index].custom_select_end_period = test_protocol.get_template(template_index).schedule.end_period.units

        # Expand the template's readings and alerts settings
        page.custom_list[template_index].custom_readings_alerts_settings_element.click

        # Fill in the template's readings and alerts settings and verify pre-populated values
        page.custom_list[template_index].custom_template_measurement = test_protocol.get_template(template_index).measurement
        expect(page.custom_list[template_index].custom_template_keyword).to eq(test_protocol.get_template(template_index).keywords)

        page.custom_list[template_index].custom_lowest_valid_reading = test_protocol.get_template(template_index).get_reading.valid_range.lowest
        page.custom_list[template_index].custom_highest_valid_reading = test_protocol.get_template(template_index).get_reading.valid_range.highest
        page.custom_list[template_index].custom_valid_reading_message = test_protocol.get_template(template_index).messages.valid_reading_message

        page.custom_list[template_index].custom_critical_low_level_alert = test_protocol.get_template(template_index).get_reading.critical_range.lowest
        page.custom_list[template_index].custom_critical_high_level_alert = test_protocol.get_template(template_index).get_reading.critical_range.highest
        page.custom_list[template_index].custom_critical_low_level_alert_message = test_protocol.get_template(template_index).messages.critical_low_message
        page.custom_list[template_index].custom_critical_high_level_alert_message = test_protocol.get_template(template_index).messages.critical_high_message

        page.custom_list[template_index].custom_low_level_alert = test_protocol.get_template(template_index).get_reading.alert_range.lowest
        page.custom_list[template_index].custom_high_level_alert = test_protocol.get_template(template_index).get_reading.alert_range.highest
        page.custom_list[template_index].custom_low_level_alert_message = test_protocol.get_template(template_index).messages.alert_low_message
        page.custom_list[template_index].custom_high_level_alert_message = test_protocol.get_template(template_index).messages.alert_high_message

        # Expand the first template's message settings
        page.custom_list[template_index].custom_message_settings_element.click

        # Fill in the first template's message settings
        page.custom_list[template_index].custom_service_message = test_protocol.get_template(template_index).messages.service_message
        page.custom_list[template_index].custom_reminder_message = test_protocol.get_template(template_index).messages.reminder_message
        page.custom_list[template_index].custom_reminder_time_between = test_protocol.get_template(template_index).time_between_reminders.value
        page.custom_list[template_index].custom_reminder_time_schedule = test_protocol.get_template(template_index).time_between_reminders.units
        page.custom_list[template_index].custom_reminder_time_sent = test_protocol.get_template(template_index).reminder_message_count
      end

      # Submit the form and dismiss the success modal dialog
      page.finish_form
      page.dismiss_modal
    end

    it 'Validates that the user can create a custom protocol with 1 vitals template and one motivational/education template' do
      # Set up variables
      test_protocol = glucose_motivational_protocol
      vitals_template_count = 1
      motivational_template_count = 1

      # Start on the Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Begin to add a new protocol
      page.add_protocol
      page = AddNewProtocolPage.new(@browser)
      page.waitForLoadingToClear

      # Fill in the new protocol form
      page.protocol_name = test_protocol.name
      page.protocol_description = test_protocol.description
      page.protocol_focus_area = test_protocol.focus_area

      # Continue to the next protocol creation page
      page.next
      page.waitForLoadingToClear
      page.back_element.visible?

      # Create a custom protocol one of each type of template
      page.add_template_element.click
      page.custom_element.click
      page = AddNewProtocolPage.new(@browser)
      page.waitForLoadingToClear
      page.check_vitals_checkbox
      page.vitals_quantity = vitals_template_count
      page.check_motivational_educational_checkbox
      page.motivational_educational_quantity = motivational_template_count
      page.next
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      # Expand the vitals template
      page.custom_list[0].custom_vitals_element.click

      # Expand the vitals template's schedule settings
      page.custom_list[0].custom_schedule_settings_element.click
      pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
      pageLoadWait.until { page.custom_list[0].custom_schedule_options != '' }

      # Fill in the vitals template's schedule settings
      page.custom_list[0].custom_schedule = test_protocol.get_template(0).schedule.schedule

      if test_protocol.get_template(0).schedule.schedule == "Specified Days"
        page.custom_list[0].check_custom_protocol_mon if test_protocol.get_template(0).schedule.schedule_days.monday
        page.custom_list[0].check_custom_protocol_tues if test_protocol.get_template(0).schedule.schedule_days.tuesday
        page.custom_list[0].check_custom_protocol_wed if test_protocol.get_template(0).schedule.schedule_days.wednesday
        page.custom_list[0].check_custom_protocol_thurs if test_protocol.get_template(0).schedule.schedule_days.thursday
        page.custom_list[0].check_custom_protocol_fri if test_protocol.get_template(0).schedule.schedule_days.friday
        page.custom_list[0].check_custom_protocol_sat if test_protocol.get_template(0).schedule.schedule_days.saturday
        page.custom_list[0].check_custom_protocol_sun if test_protocol.get_template(0).schedule.schedule_days.sunday
      end

      if test_protocol.get_template(0).schedule.schedule == "Custom"
        page.custom_list[0].custom_time_between_each_cycle = test_protocol.get_template(0).schedule.time_between_each_cycle.value
        page.custom_list[0].custom_time_between_each_cycle_units = test_protocol.get_template(0).schedule.time_between_each_cycle.units
      end

      page.custom_list[0].custom_start_time_hour = test_protocol.get_template(0).schedule.service_message_start_time.hours
      page.custom_list[0].custom_start_time_minutes = test_protocol.get_template(0).schedule.service_message_start_time.minutes
      page.custom_list[0].custom_am_pm = test_protocol.get_template(0).schedule.service_message_start_time.am_pm
      page.custom_list[0].custom_start_period = test_protocol.get_template(0).schedule.start_period.value
      page.custom_list[0].custom_select_start_period = test_protocol.get_template(0).schedule.start_period.units
      page.custom_list[0].custom_end_period = test_protocol.get_template(0).schedule.end_period.value
      page.custom_list[0].custom_select_end_period = test_protocol.get_template(0).schedule.end_period.units

      # Expand the vitals template's readings and alerts settings
      page.custom_list[0].custom_readings_alerts_settings_element.click

      # Fill in the vitals template's readings and alerts settings and the pre-populated values
      page.custom_list[0].custom_template_measurement = test_protocol.get_template(0).measurement
      expect(page.custom_list[0].custom_template_keyword).to eq(test_protocol.get_template(0).keywords)

      page.custom_list[0].custom_lowest_valid_reading = test_protocol.get_template(0).get_reading.valid_range.lowest
      page.custom_list[0].custom_highest_valid_reading = test_protocol.get_template(0).get_reading.valid_range.highest
      page.custom_list[0].custom_valid_reading_message = test_protocol.get_template(0).messages.valid_reading_message

      page.custom_list[0].custom_critical_low_level_alert = test_protocol.get_template(0).get_reading.critical_range.lowest
      page.custom_list[0].custom_critical_high_level_alert = test_protocol.get_template(0).get_reading.critical_range.highest
      page.custom_list[0].custom_critical_low_level_alert_message = test_protocol.get_template(0).messages.critical_low_message
      page.custom_list[0].custom_critical_high_level_alert_message = test_protocol.get_template(0).messages.critical_high_message

      page.custom_list[0].custom_low_level_alert = test_protocol.get_template(0).get_reading.alert_range.lowest
      page.custom_list[0].custom_high_level_alert = test_protocol.get_template(0).get_reading.alert_range.highest
      page.custom_list[0].custom_low_level_alert_message = test_protocol.get_template(0).messages.alert_low_message
      page.custom_list[0].custom_high_level_alert_message = test_protocol.get_template(0).messages.alert_high_message

      # Expand the vitals template's message settings
      page.custom_list[0].custom_message_settings_element.click

      # Fill in the vitals template's message settings
      page.custom_list[0].custom_service_message = test_protocol.get_template(0).messages.service_message
      page.custom_list[0].custom_reminder_message = test_protocol.get_template(0).messages.reminder_message
      page.custom_list[0].custom_reminder_time_between = test_protocol.get_template(0).time_between_reminders.value
      page.custom_list[0].custom_reminder_time_schedule = test_protocol.get_template(0).time_between_reminders.units
      page.custom_list[0].custom_reminder_time_sent = test_protocol.get_template(0).reminder_message_count

      # Expand the motivational template
      page.custom_list[1].custom_motivational_educational_element.click

      # Expand the motivational template's schedule settings
      page.custom_list[1].custom_schedule_settings_element.click
      pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
      pageLoadWait.until { page.custom_list[1].custom_schedule_options != '' }

      # Fill in the motivational template's schedule settings
      page.custom_list[1].custom_schedule = test_protocol.get_template(1).schedule.schedule

      if test_protocol.get_template(1).schedule.schedule == "Specified Days"
        page.custom_list[1].check_custom_protocol_mon if test_protocol.get_template(1).schedule.schedule_days.monday
        page.custom_list[1].check_custom_protocol_tues if test_protocol.get_template(1).schedule.schedule_days.tuesday
        page.custom_list[1].check_custom_protocol_wed if test_protocol.get_template(1).schedule.schedule_days.wednesday
        page.custom_list[1].check_custom_protocol_thurs if test_protocol.get_template(1).schedule.schedule_days.thursday
        page.custom_list[1].check_custom_protocol_fri if test_protocol.get_template(1).schedule.schedule_days.friday
        page.custom_list[1].check_custom_protocol_sat if test_protocol.get_template(1).schedule.schedule_days.saturday
        page.custom_list[1].check_custom_protocol_sun if test_protocol.get_template(1).schedule.schedule_days.sunday
      end

      if test_protocol.get_template(1).schedule.schedule == "Custom"
        page.custom_list[1].custom_time_between_each_cycle = test_protocol.get_template(1).schedule.time_between_each_cycle.value
        page.custom_list[1].custom_time_between_each_cycle_units = test_protocol.get_template(1).schedule.time_between_each_cycle.units
      end

      page.custom_list[1].custom_start_time_hour = test_protocol.get_template(1).schedule.service_message_start_time.hours
      page.custom_list[1].custom_start_time_minutes = test_protocol.get_template(1).schedule.service_message_start_time.minutes
      page.custom_list[1].custom_am_pm = test_protocol.get_template(1).schedule.service_message_start_time.am_pm
      page.custom_list[1].custom_start_period = test_protocol.get_template(1).schedule.start_period.value
      page.custom_list[1].custom_select_start_period = test_protocol.get_template(1).schedule.start_period.units
      page.custom_list[1].custom_end_period = test_protocol.get_template(1).schedule.end_period.value
      page.custom_list[1].custom_select_end_period = test_protocol.get_template(1).schedule.end_period.units
      page.custom_list[1].custom_motivational_description = test_protocol.get_template(1).template_description

      # Expand the motivational template's message settings
      page.custom_list[1].custom_message_settings_element.click

      # Fill in the motivational template's message settings
      page.custom_list[1].custom_motivational_service_message1 = test_protocol.get_template(1).messages.get_motivational_message(0)
      expect(page.custom_list[1].custom_motivational_characters_remaining1).to eq(get_motivational_message_characters_remaining_text(test_protocol.get_template(1).messages.get_motivational_message(0)))
      page.custom_list[1].custom_add_message
      page.custom_list[1].custom_motivational_service_message2 = test_protocol.get_template(1).messages.get_motivational_message(1)
      expect(page.custom_list[1].custom_motivational_characters_remaining2).to eq(get_motivational_message_characters_remaining_text(test_protocol.get_template(1).messages.get_motivational_message(1)))
      page.custom_list[1].custom_add_message
      page.custom_list[1].custom_motivational_service_message3 = test_protocol.get_template(1).messages.get_motivational_message(2)
      page.custom_list[1].custom_add_message
      page.custom_list[1].custom_motivational_service_message4 = test_protocol.get_template(1).messages.get_motivational_message(3)

      # Submit the form and verify the success message
      page.finish_form
      expect(page.congrats_message).to eq("Congratulations! You successfully created a new template for your new protocol.")
      page.dismiss_modal
    end

    it 'Validates the newly created protocol with Custom template on the Protocol List page' do
      # Set up variables
      default_comment = "Protocol Created"
      test_protocols = [
          { :protocol => pulse_ox_protocol, :comment => default_comment },
          { :protocol => weight_temperature_protocol, :comment => default_comment },
          { :protocol => glucose_motivational_protocol, :comment => default_comment }
      ]

      # Start on the List Protocols page
      page = ListTemplatesPage.new(@browser)

      # Verify the protocol data
      test_protocols.each do |protocol_entry|
        page.waitForLoadingToClear
        validate_protocol(protocol_entry[:protocol], protocol_entry[:comment])
      end
    end

    it 'Validates the admin is able to disable the protocol' do
      # Set up variables
      test_protocol = glucose_motivational_protocol
      disable_comment = "my disable text."

      # Start on the List Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Find the desired protocol on the page
      protocol_list = page.protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
      protocol_on_page = page.protocol_list[protocol_index]
      protocol_on_page.enable_selected?

      # Start to disable the protocol and cancel the disabling
      protocol_on_page.select_disable
      page.disable_ok
      expect(page.disable_error).to eq("Please enter in comment")
      page.disable_cancel
      page.waitForLoadingToClear

      # Start to disable the protocol and complete the disabling
      protocol_on_page.enable_selected?
      protocol_on_page.select_disable
      page.disable_comment = disable_comment
      page.disable_ok

      # Verify that the protocol was disabled and that the comment was saved
      protocol_on_page.disable_selected?
      expect(protocol_on_page.last_comment).to eq(disable_comment)
    end

    it 'Validates the admin is able to re-enable the protocol' do
      # Set up variables
      test_protocol = glucose_motivational_protocol
      enable_comment = "my enable text."

      # Start on the List Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Grab the desired protocol on the page
      protocol_list = page.protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
      protocol_on_page = page.protocol_list[protocol_index]

      # Start to enable the desired protocol and cancel the enabling
      protocol_on_page.disable_selected?
      protocol_on_page.select_enable
      page.enable_ok
      expect(page.enable_error).to eq("Please enter in comment")
      page.enable_cancel
      page.waitForLoadingToClear

      # Start to enable the desired protocol and proceed with the enabling
      protocol_on_page.select_enable
      page.enable_comment = enable_comment
      page.enable_ok
      page.waitForLoadingToClear
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      # Save the re-enabled protocol
      expect(page.protocol_name).to eq(test_protocol.name)
      page.save_form_with_bottom_button
      page.modal_ok
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Verify the protocol was enabled
      protocol_list = page.protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
      protocol_on_page = page.protocol_list[protocol_index]
      protocol_on_page.enable_selected?
      expect(protocol_on_page.last_comment).to eq(enable_comment)
    end

    it 'Validates the changes to the newly created protocol with Custom template on the Audit page.' do
      # Set up variables
      audit_protocols = [pulse_ox_protocol, weight_temperature_protocol, glucose_motivational_protocol]
      audit_user = @full_name
      audit_list = [
          {
              :name => audit_protocols[0].name,
              :audit_trail_comment => "Field: Status changed from false to true\nField: Template Name set to #{audit_protocols[0].name}.\nField: Description set to #{audit_protocols[0].description}.\nField: Focus Area set to #{audit_protocols[0].focus_area}.\nField: Status set to Active.\nField: Template Description set to Pulse Oximetry (SpO2).\nField: Service Message set to Hello, just a quick reminder that your readings are due. (e.g., OX70). Thanks..\nField: Reminder Message set to Hi again. This is your reminder message. Could you please text them in. Kind regards..\nField: Message for Valid Reading set to Your Pulse OX protocol reading is fine..\nField: Measurement set to Pulse Oximetry (SpO2).\nField: Lowest Valid Reading set to 60.\nField: Highest Valid Reading set to 140.\nField: Low Level Alert set to 70.\nField: High Level Alert set to 90.\nField: Critical Low Level Alert set to 65.\nField: Critical High Level Alert set to 95.\nField: Reading Parameter Unit set to % Sat.\nField: Below Desirable Level Level Message set to Your Pulse OX is a little low today. Take it again, and follow the advice in your management plan. Take care..\nField: Above Desirable Level Message set to Your Pulse OX is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks..\nField: Message for Critical Low Level Alert set to Your Pulse OX is CRITICALLY low today. Take it again, and follow the advice in your management plan. Take care..\nField: Message for Critical High Level Alert set to Your Pulse OX is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks..\nField: Keywords set to OX,OXYGEN.\nField: Accept Unscheduled readings? changed from false to true\nField: Schedule Type set to Custom.\nField: Time Between Each Cycle set to 15.\nField: Time Between Each Cycle (unit) set to MINUTE.\nField: Service Message Start Time (hour) changed from 0 to 11\nField: Service Message Start Time (minute) changed from 0 to 30\nField: Service Message Start Time (am/pm) set to PM.\nField: Relative Start Time set to 2.\nField: Relative Start Time (unit) set to HOUR.\nField: Relative End Time set to 5.\nField: Relative End Time (unit) set to MONTH.\nField: Number of times reminder message is sent set to 4.\nField: Amount of time between reminders set to 10.\nField: Amount of time between reminders (unit) set to MINUTE.",
              :audit_trail_index => 1,
              :audit_trail_row_count => 2
          },
          {
              :name => audit_protocols[1].name,
              :audit_trail_comment => "Field: Status changed from false to true\nField: Template Name set to Weight Temperature Custom.\nField: Description set to Custom Protocol of Weight and Temperature.\nField: Focus Area set to Weight, Temperature, Temp.\nField: Status set to Active.\nField: Template Description set to Weight Loss.\nField: Service Message set to This is your Weight Loss Service Message..\nField: Reminder Message set to This is your Weight Loss Reminder Message..\nField: Message for Valid Reading set to Your Weight Loss reading is valid.\nField: Measurement set to Weight Loss.\nField: Lowest Valid Reading set to 50.\nField: Highest Valid Reading set to 250.\nField: Low Level Alert set to 70.\nField: High Level Alert set to 200.\nField: Critical Low Level Alert set to 55.\nField: Critical High Level Alert set to 245.\nField: Reading Parameter Unit set to lbs.\nField: Below Desirable Level Level Message set to This is your Weight Loss LOW LEVEL ALERT!.\nField: Above Desirable Level Message set to This is your Weight Loss HIGH LEVEL ALERT!.\nField: Message for Critical Low Level Alert set to Your Weight Loss is CRITICALLY LOW!.\nField: Message for Critical High Level Alert set to Your Weight Loss is CRITICALLY HIGH!.\nField: Keywords set to WE,WEIGHT.\nField: Accept Unscheduled readings? changed from false to true\nField: Service Message Start Time (hour) changed from 0 to 8\nField: Service Message Start Time (minute) changed from 0 to 17\nField: Service Message Start Time (am/pm) set to PM.\nField: Relative Start Time set to 1.\nField: Relative Start Time (unit) set to MINUTE.\nField: Relative End Time set to 4.\nField: Relative End Time (unit) set to YEAR.\nField: Number of times reminder message is sent set to 122.\nField: Amount of time between reminders set to 15.\nField: Amount of time between reminders (unit) set to HOUR.\nField: Status set to Active.\nField: Template Description set to Temperature.\nField: Service Message set to This is your Temperature Service Message..\nField: Reminder Message set to This is your Temperature Reminder Message..\nField: Message for Valid Reading set to Your Temperature reading is valid.\nField: Measurement set to Temperature.\nField: Lowest Valid Reading set to 95.\nField: Highest Valid Reading set to 106.\nField: Low Level Alert set to 98.\nField: High Level Alert set to 103.\nField: Critical Low Level Alert set to 97.\nField: Critical High Level Alert set to 104.\nField: Reading Parameter Unit set to Fahrenheit.\nField: Below Desirable Level Level Message set to This is your Temperature LOW LEVEL ALERT!.\nField: Above Desirable Level Message set to This is your Temperature HIGH LEVEL ALERT!.\nField: Message for Critical Low Level Alert set to Your Temperature is CRITICALLY LOW!.\nField: Message for Critical High Level Alert set to Your Temperature is CRITICALLY HIGH!.\nField: Keywords set to TEMP.\nField: Accept Unscheduled readings? changed from false to true\nField: Schedule Type set to Specified Days.\nField: Days that Apply (Monday) changed from false to true\nField: Days that Apply (Wednesday) changed from false to true\nField: Days that Apply (Friday) changed from false to true\nField: Days that Apply (Saturday) changed from false to true\nField: Service Message Start Time (hour) changed from 0 to 6\nField: Service Message Start Time (minute) changed from 0 to 13\nField: Service Message Start Time (am/pm) set to AM.\nField: Relative Start Time set to 3.\nField: Relative Start Time (unit) set to DAY.\nField: Relative End Time set to 9.\nField: Relative End Time (unit) set to WEEK.\nField: Number of times reminder message is sent set to 22.\nField: Amount of time between reminders set to 2.\nField: Amount of time between reminders (unit) set to MINUTE.",
              :audit_trail_index => 1,
              :audit_trail_row_count => 2
          },
          {
              :name => audit_protocols[2].name,
              :audit_trail_comment => "Field: Status changed from false to true\nField: Template Name set to Glucose Motivational Protocol.\nField: Description set to Custom Protocol Measuring Glucose and Providing Motivational Messages.\nField: Focus Area set to Glucose, Motivation.\nField: Status set to Active.\nField: Template Description set to Blood Glucose.\nField: Service Message set to This is your Blood Glucose Service Message..\nField: Reminder Message set to This is your Blood Glucose Reminder Message..\nField: Message for Valid Reading set to Your Blood Glucose reading is valid.\nField: Measurement set to Blood Glucose.\nField: Lowest Valid Reading set to 30.\nField: Highest Valid Reading set to 300.\nField: Low Level Alert set to 40.\nField: High Level Alert set to 200.\nField: Critical Low Level Alert set to 35.\nField: Critical High Level Alert set to 280.\nField: Reading Parameter Unit set to mg/dl.\nField: Below Desirable Level Level Message set to This is your Blood Glucose LOW LEVEL ALERT!.\nField: Above Desirable Level Message set to This is your Blood Glucose HIGH LEVEL ALERT!.\nField: Message for Critical Low Level Alert set to Your Blood Glucose is CRITICALLY LOW!.\nField: Message for Critical High Level Alert set to Your Blood Glucose is CRITICALLY HIGH!.\nField: Keywords set to BG,GLUCOSE,SUGAR.\nField: Accept Unscheduled readings? changed from false to true\nField: Schedule Type set to Specified Days.\nField: Days that Apply (Monday) changed from false to true\nField: Days that Apply (Tuesday) changed from false to true\nField: Days that Apply (Wednesday) changed from false to true\nField: Days that Apply (Thursday) changed from false to true\nField: Days that Apply (Friday) changed from false to true\nField: Days that Apply (Saturday) changed from false to true\nField: Days that Apply (Sunday) changed from false to true\nField: Service Message Start Time (hour) changed from 0 to 4\nField: Service Message Start Time (minute) changed from 0 to 43\nField: Service Message Start Time (am/pm) set to AM.\nField: Relative Start Time set to 3.\nField: Relative Start Time (unit) set to DAY.\nField: Relative End Time set to 9.\nField: Relative End Time (unit) set to WEEK.\nField: Number of times reminder message is sent set to 333.\nField: Amount of time between reminders set to 3.\nField: Amount of time between reminders (unit) set to MINUTE.\nField: Status set to Active.\nField: Template Description set to This is the custom template motivational description..\nField: Service Message set to This is my first service message..\nField: Service Message set to This is my second service message..\nField: Service Message set to This is my Third service message..\nField: Service Message set to This is my fourth service message..\nField: Schedule Type set to Custom.\nField: Time Between Each Cycle set to 6.\nField: Time Between Each Cycle (unit) set to WEEK.\nField: Service Message Start Time (hour) changed from 0 to 1\nField: Service Message Start Time (minute) changed from 0 to 59\nField: Service Message Start Time (am/pm) set to AM.\nField: Relative Start Time set to 1.\nField: Relative Start Time (unit) set to HOUR.\nField: Relative End Time set to 7.\nField: Relative End Time (unit) set to YEAR.",
              :audit_trail_index => 3,
              :audit_trail_row_count => 4
          }
      ]

      audit_list.each do |audit_entry|
        # Start on the Admin Audit Trail page
        page = AdminAuditTrailPage.new(@browser)
        page.waitForLoadingToClear

        # Navigate to the Protocols page
        page.protocols_tab_element.click
        page = ListTemplatesPage.new(@browser)
        page.waitForLoadingToClear

        # Open the audit trail for the desired protocol
        page.navigateToAuditTrail(audit_entry[:name])

        # Validate the protocol's audit trail
        validate_admin_audit_trail(audit_entry[:audit_trail_comment], audit_entry[:audit_trail_index], audit_entry[:audit_trail_row_count], audit_user)
      end
    end

    it 'Validates the pagination on the Audit Trail page.' do
      # Set up variables
      test_protocol = ProtocolModel.new(
          :name => "Blood Pressure Protocol4"
      )
      audit_trail_entries_per_page = 20
      audit_trail_entries_in_protocol = 23

      # Start on the Admin Audit Trail page
      page = AdminAuditTrailPage.new(@browser)
      page.waitForLoadingToClear

      # Navigate to the List Protocols page
      page.protocols_tab_element.click
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Find the desired protocol and open its audit trail
      protocol_list = page.protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
      page.protocol_list[protocol_index].audit_trail
      page = AdminAuditTrailPage.new(@browser)
      page.waitForLoadingToClear

      # Verify that the page does not contain more audit log entries than allowed per page
      expect(page.audittrail_table_element.rows).to eq(audit_trail_entries_per_page + 1)  # Add 1 for the header
      expect(page.pagination_top_element.list_item_element(:text => '1').class_name).to include('active')

      # Navigate to the second page
      page.pagination_bottom_element.list_item_element(:text => '2').link_element.click
      expect(page.pagination_top_element.list_item_element(:text => '2').class_name).to include('active')

      # Verify that the page contains the correct number of audit log entries
      if audit_trail_entries_in_protocol < (audit_trail_entries_per_page * 2)
        expect(page.audittrail_table_element.rows).to be >= (audit_trail_entries_in_protocol % audit_trail_entries_per_page)
      else
        expect(page.audittrail_table_element.rows).to eq(audit_trail_entries_per_page)
      end

      # Navigate back to the Protocols page
      page.protocols_tab_element.click
    end

    it 'Validates a protocol can be edited on the admin side' do
      original_protocol = pulse_ox_protocol
      updated_protocol = updated_pulse_ox_protocol

      # Start on the Protocols page
      page = ListTemplatesPage.new(@browser)

      # Find and edit the desired protocol
      protocol_list = page.protocol_list
      protocol_index = page.get_index_by_text(protocol_list, original_protocol.name)
      page.protocol_list[protocol_index].edit_view
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      # Edit the protocol
      page.custom_list[0].custom_template_name_element.click
      page.custom_list[0].custom_readings_alerts_settings_element.click
      page.custom_list[0].custom_valid_reading_message = updated_protocol.get_template.messages.valid_reading_message
      page.custom_list[0].custom_critical_low_level_alert = updated_protocol.get_template.get_reading.critical_range.lowest
      page.custom_list[0].custom_critical_high_level_alert = updated_protocol.get_template.get_reading.critical_range.highest
      page.custom_list[0].custom_low_level_alert_message = updated_protocol.get_template.messages.alert_low_message
      page.custom_list[0].custom_high_level_alert_message = updated_protocol.get_template.messages.alert_high_message
      page.custom_list[0].custom_message_settings_element.click
      page.custom_list[0].custom_reminder_message = updated_protocol.get_template.messages.reminder_message
      page.protocol_focus_area = updated_protocol.focus_area

      # Save the changes to the protocol and dismiss the success modal
      page.save_form_with_bottom_button
      page.dismiss_modal
    end

    it 'Validates the edited protocol in the admin protocol list' do
      # Set up variables
      default_comment = "Protocol Created"
      test_protocols = [
          { :protocol => updated_pulse_ox_protocol, :comment => default_comment },
          { :protocol => weight_temperature_protocol, :comment => default_comment },
          { :protocol => glucose_motivational_protocol, :comment => "my enable text." }
      ]

      # Start on the List Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Verify the protocol data
      test_protocols.each do |protocol_entry|
        page.waitForLoadingToClear
        validate_protocol(protocol_entry[:protocol], protocol_entry[:comment])
      end
    end

    it 'Validates the edited protocol changes are present in the audit trail' do
      # Set up variables
      original_audit_protocol = pulse_ox_protocol
      updated_audit_protocol = updated_pulse_ox_protocol
      audit_user = @full_name
      audit_entry = {
          :name => updated_audit_protocol.name,
          :comments => [
              "Field: Focus Area changed from #{original_audit_protocol.focus_area} to #{updated_audit_protocol.focus_area}\nField: Reminder Message changed from #{original_audit_protocol.get_template.messages.reminder_message} to #{updated_audit_protocol.get_template.messages.reminder_message}\nField: Message for Valid Reading changed from #{original_audit_protocol.get_template.messages.valid_reading_message} to #{updated_audit_protocol.get_template.messages.valid_reading_message}\nField: Critical Low Level Alert changed from #{original_audit_protocol.get_template.get_reading.critical_range.lowest} to #{updated_audit_protocol.get_template.get_reading.critical_range.lowest}\nField: Critical High Level Alert changed from #{original_audit_protocol.get_template.get_reading.critical_range.highest} to #{updated_audit_protocol.get_template.get_reading.critical_range.highest}\nField: Below Desirable Level Level Message changed from #{original_audit_protocol.get_template.messages.alert_low_message} to #{updated_pulse_ox_protocol.get_template.messages.alert_low_message}\nField: Above Desirable Level Message changed from #{original_audit_protocol.get_template.messages.alert_high_message} to #{updated_audit_protocol.get_template.messages.alert_high_message}",
              "Field: Status changed from false to true\nField: Template Name set to Pulse OX.\nField: Description set to This is a new protocol created for Pulse OX.\nField: Focus Area set to Pulse, OX, Focus, Area.\nField: Status set to Active.\nField: Template Description set to Pulse Oximetry (SpO2).\nField: Service Message set to Hello, just a quick reminder that your readings are due. (e.g., OX70). Thanks..\nField: Reminder Message set to Hi again. This is your reminder message. Could you please text them in. Kind regards..\nField: Message for Valid Reading set to Your Pulse OX protocol reading is fine..\nField: Measurement set to Pulse Oximetry (SpO2).\nField: Lowest Valid Reading set to 60.\nField: Highest Valid Reading set to 140.\nField: Low Level Alert set to 70.\nField: High Level Alert set to 90.\nField: Critical Low Level Alert set to 65.\nField: Critical High Level Alert set to 95.\nField: Reading Parameter Unit set to % Sat.\nField: Below Desirable Level Level Message set to Your Pulse OX is a little low today. Take it again, and follow the advice in your management plan. Take care..\nField: Above Desirable Level Message set to Your Pulse OX is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks..\nField: Message for Critical Low Level Alert set to Your Pulse OX is CRITICALLY low today. Take it again, and follow the advice in your management plan. Take care..\nField: Message for Critical High Level Alert set to Your Pulse OX is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks..\nField: Keywords set to OX,OXYGEN.\nField: Accept Unscheduled readings? changed from false to true\nField: Schedule Type set to Custom.\nField: Time Between Each Cycle set to 15.\nField: Time Between Each Cycle (unit) set to MINUTE.\nField: Service Message Start Time (hour) changed from 0 to 11\nField: Service Message Start Time (minute) changed from 0 to 30\nField: Service Message Start Time (am/pm) set to PM.\nField: Relative Start Time set to 2.\nField: Relative Start Time (unit) set to HOUR.\nField: Relative End Time set to 5.\nField: Relative End Time (unit) set to MONTH.\nField: Number of times reminder message is sent set to 4.\nField: Amount of time between reminders set to 10.\nField: Amount of time between reminders (unit) set to MINUTE."
          ],
          :audit_trail_index => 1,
          :audit_trail_row_count => 3
      }

      # Start on the Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Navigate to the audit trail for the desired protocol
      page.navigateToAuditTrail(audit_entry[:name])

      # Validate the comments in the audit trail
      validate_admin_audit_trail(audit_entry[:comments], audit_entry[:audit_trail_index], audit_entry[:audit_trail_row_count], audit_user)
    end


    it 'Validates the newly created protocol with Custom template on the Patient Protocol List Page' do
      def validate_patient_protocol(protocol, template_list, status = "Active")
        # Set up local variables
        date_obj = DateTime.now
        date_format = "%m/%d/%Y"
        today_date = date_obj.strftime(date_format)
        yesterday_date = (date_obj - 1).strftime(date_format)

        # Start on the Patient Protocol List page
        page = UpdatePatientProtocolListPage.new(@browser)
        page.waitForLoadingToClear

        # Find the desired protocol on the page
        protocol_list = page.protocol_list
        protocol_index = page.get_index_by_text(protocol_list, protocol.name)
        protocol_on_page = page.protocol_list[protocol_index]

        # Verify the protocol data
        expect(protocol_on_page.name).to eq(protocol.name)
        expect(protocol_on_page.desc).to eq(protocol.description)
        expect(protocol_on_page.template).to eq(template_list)
        if protocol_on_page.created != today_date
          expect(protocol_on_page.created).to eq(yesterday_date)
        end
        expect(protocol_on_page.focus).to eq(protocol.focus_area)
        expect(protocol_on_page.status).to eq(status)
      end

      # Set up variables
      protocol_list = [
          {
              :protocol => updated_pulse_ox_protocol,
              :template_list => "Vitals: Pulse Oximetry (SpO2) - Every 15 minute (At 11:30 PM)",
              :customize => false
          },
          {
              :protocol => weight_temperature_protocol,
              :template_list => "Vitals: Weight Loss - Once (At 8:17 PM)\nVitals: Temperature - Mon., Wed., Fri., Sat. (At 6:13 AM)",
              :customize => false
          },
          {
              :protocol => glucose_motivational_protocol,
              :template_list => "Vitals: Blood Glucose - Sun., Mon., Tue., Wed., Thu., Fri., Sat. (At 4:43 AM)\nMotivational/Educational: This is the custom template motivational description. - Every 6 week (At 1:59 AM)",
              :customize => true
          }
      ]
      test_patient = {
          :display_name => @patient18Name,
          :search_string => "P9918"
      }

      # Start on the Admin Protocols page
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Navigate to the Staff Patient Search page
      page.GoToStaffSearch
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad
      page.patient_search_element.click

      # Search for the desired patient
      page = page.search_patient_and_select(test_patient[:search_string], test_patient[:display_name])
      page = UpdatePatientProtocolListPage.new(@browser)

      # Add a new protocol to the patient

      # Validate that the expected protocols are available and assign them
      protocol_list.each do |protocol_entry|
        # Add a new protocol
        page.New_Protocol
        page.waitForLoadingToClear

        # Validate the protocol entry
        validate_patient_protocol(protocol_entry[:protocol], protocol_entry[:template_list])

        # Assign the protocol
        page.assignProtocolByName(protocol_entry[:protocol].name)
        if protocol_entry[:customize]
          page.customizeprotocol_yes
        else
          page.customizeprotocol_no
        end
        page.waitForLoadingToClear
      end
    end


    it 'Validates the newly created protocol with Custom template can be edited on the Patient Protocol Edit page.' do
      # Set up variables
      date_obj = DateTime.now
      date_format = "%m/%d/%Y"
      today_date = date_obj.strftime(date_format)
      one_year_date = (date_obj >> 12).strftime(date_format)
      reference_protocol = glucose_motivational_protocol
      test_protocol = ProtocolModel.new(
          :name => reference_protocol.name,
          :description => reference_protocol.description,
          :focus_area => reference_protocol.focus_area,
          :templates => [
              TemplateModel.new(
                  :measurement => reference_protocol.get_template(0).measurement,
                  :keywords => reference_protocol.get_template(0).keywords,
                  :schedule => TemplateSchedule.new(
                      :schedule => reference_protocol.get_template(0).schedule.schedule,
                      :schedule_days => reference_protocol.get_template(0).schedule.schedule_days,
                      :service_message_start_time => reference_protocol.get_template(0).schedule.service_message_start_time,
                      :start_date => today_date,
                      :end_date => one_year_date
                  ),
                  :readings => TemplateReading.new(
                      :valid_range => RangeLimit.new(29, reference_protocol.get_template(0).get_reading.valid_range.highest),
                      :critical_range => RangeLimit.new(reference_protocol.get_template(0).get_reading.critical_range.lowest, 281),
                      :alert_range => RangeLimit.new(39, reference_protocol.get_template(0).get_reading.alert_range.highest)
                  ),
                  :messages => reference_protocol.get_template(0).messages,
                  :time_between_reminders => ValueUnits.new(reference_protocol.get_template(0).time_between_reminders.value, "Year"),
                  :reminder_message_count => reference_protocol.get_template(0).reminder_message_count
              ),
              TemplateModel.new(
                  :template_description => reference_protocol.get_template(1).template_description,
                  :schedule => TemplateSchedule.new(
                      :schedule => reference_protocol.get_template(1).schedule.schedule,
                      :service_message_start_time => reference_protocol.get_template(1).schedule.service_message_start_time,
                      :time_between_each_cycle => ValueUnits.new(reference_protocol.get_template(1).schedule.time_between_each_cycle.value, "Day"),
                      :start_period => reference_protocol.get_template(1).schedule.start_period,
                      :end_period => reference_protocol.get_template(1).schedule.end_period
                  ),
                  :messages => reference_protocol.get_template(1).messages
              )
          ]
      )

      # Start on the Update Patient Protocol Edit page
      page = UpdatePatientProtocolEditPage.new(@browser)

      # Verify that some protocol data is correct
      expect(page.protocol_name).to eq("Protocol: #{test_protocol.name}")
      expect(page.custom_list[0].custom_template_schedule_heading).to eq("Sun., Mon., Tue., Wed., Thu., Fri., Sat.\n(At 4:43 AM)")
      expect(page.custom_list[1].custom_template_schedule_heading).to eq("Every 6 week\n(At 1:59 AM)")

      # Verify and customize the protocol's vitals template
      page.custom_list[0].custom_template_name_element.click
      page.custom_list[0].custom_schedule_settings_element.click
      page.custom_list[0].custom_protocol_startdate = test_protocol.get_template(0).schedule.start_date
      page.custom_list[0].custom_protocol_enddate = test_protocol.get_template(0).schedule.end_date
      page.custom_list[0].custom_reading_settings_element.click
      expect(page.custom_list[0].custom_measurement).to eq(test_protocol.get_template(0).measurement)
      page.custom_list[0].custom_lowest_validreading_one = test_protocol.get_template(0).get_reading.valid_range.lowest
      page.custom_list[0].custom_lowlevel_alert_one = test_protocol.get_template(0).get_reading.alert_range.lowest
      page.custom_list[0].custom_critical_highlevel_one = test_protocol.get_template(0).get_reading.critical_range.highest
      page.custom_list[0].custom_message_settings_element.click
      expect(page.custom_list[0].custom_protocol_servicemessage).to eq(test_protocol.get_template(0).messages.service_message)
      page.custom_list[0].custom_protocol_remindertime = test_protocol.get_template(0).time_between_reminders.units

      # Verify and customize the protocol's motivational template
      page.custom_list[1].custom_template_name_element.click
      page.custom_list[1].custom_schedule_settings_element.click
      expect(page.custom_list[1].custom_description).to eq(test_protocol.get_template(1).template_description)
      page.custom_list[1].custom_protocol_timebtwneachcycleunit = test_protocol.get_template(1).schedule.time_between_each_cycle.units
      page.custom_list[1].custom_message_settings_element.click
      expect(page.custom_list[1].custom_motivational_service_message1).to eq(test_protocol.get_template(1).messages.get_motivational_message(0))
      page.custom_list[1].custom_remove_message
      expect(page.remove_message_text).to eq("Are you sure you want to remove a message?")
      page.remove_message_yes
      expect(page.custom_list[1].custom_motivational_characters_remaining1).to eq(get_motivational_message_characters_remaining_text(test_protocol.get_template(1).messages.get_motivational_message(0)))

      # Save the changes to the protocol and dismiss the success modal dialog
      page.protocol_save_top
      expect(page.save_msg).to eq("Thank you. Your changes are saved.")
      page.dismiss_modal
    end


    it 'Validates the newly edited protocol with Custom template can be viewed with changes on the Patient Protocol List page.' do
      # Set up variables
      date_obj = DateTime.now
      date_format = "%m/%d/%Y"
      today_date = date_obj.strftime(date_format)
      yesterday_date = (date_obj - 1).strftime(date_format)
      protocol_list = [pulse_ox_protocol, weight_temperature_protocol, glucose_motivational_protocol]

      # Start on the Update Patient Protocol List page
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Verify that each of the protocols are assigned
      protocol_list.each do |protocol_entry|
        protocol_index = page.get_index_by_text(page.patient_protocol_list, protocol_entry.name)
        protocol_on_page = page.patient_protocol_list[protocol_index]

        expect(protocol_on_page.name).to eq(protocol_entry.name)
        if protocol_on_page.assigned_date != today_date
          expect(protocol_on_page.assigned_date).to eq(yesterday_date)
        end
        if protocol_on_page.edited_date != today_date
          expect(protocol_on_page.edited_date).to eq(yesterday_date)
        end
        expect(protocol_on_page.edited_by).to eq(@full_name)
        expect(protocol_on_page.last_comment).to eq('Protocol Assigned')
        expect(protocol_on_page.activeprotocolenable_selected?).to be_true
      end
    end


    it 'Validates the assigned and edited protocols in the audit trail page.' do
      # Set up variables
      date_now = DateTime.now
      date_format = "%Y-%m-%d"
      comment_dates = { :date_object => date_now }
      comment_dates[:base] = comment_dates[:date_object].strftime(date_format)
      comment_dates[:five_months] = (comment_dates[:date_object] >> 5).strftime(date_format)
      comment_dates[:three_days] = (comment_dates[:date_object] + 3).strftime(date_format)
      comment_dates[:sixty_six_days] = (comment_dates[:date_object] + 66).strftime(date_format)
      comment_dates[:fourteen_sixty_one_days] = (comment_dates[:date_object] + 1461).strftime(date_format)
      comment_dates[:one_year] = (comment_dates[:date_object] >> 12).strftime(date_format)
      alternate_comment_dates = { :date_object => date_now - 1 }
      alternate_comment_dates[:base] = alternate_comment_dates[:date_object].strftime(date_format)
      alternate_comment_dates[:five_months] = (alternate_comment_dates[:date_object] >> 5).strftime(date_format)
      alternate_comment_dates[:three_days] = (alternate_comment_dates[:date_object] + 3).strftime(date_format)
      alternate_comment_dates[:sixty_six_days] = (alternate_comment_dates[:date_object] + 66).strftime(date_format)
      alternate_comment_dates[:fourteen_sixty_one_days] = (alternate_comment_dates[:date_object] + 1461).strftime(date_format)
      alternate_comment_dates[:one_year] = (alternate_comment_dates[:date_object] >> 12).strftime(date_format)
      audit_user = @full_name
      audit_list = [
          {
              :protocol => updated_pulse_ox_protocol,
              :comment => "Field: Status set to Active.\nField: Start Date set to #{comment_dates[:base]}.\nField: End Date set to #{comment_dates[:five_months]}.",
              :alternate_comment => "Field: Status set to Active.\nField: Start Date set to #{alternate_comment_dates[:base]}.\nField: End Date set to #{alternate_comment_dates[:five_months]}."
          },
          {
              :protocol => weight_temperature_protocol,
              :comment => "Field: Status set to Active.\nField: Start Date set to #{comment_dates[:base]}.\nField: End Date set to #{comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{comment_dates[:three_days]}.\nField: End Date set to #{comment_dates[:sixty_six_days]}.",
              :alternate_comment => "Field: Status set to Active.\nField: Start Date set to #{alternate_comment_dates[:base]}.\nField: End Date set to #{alternate_comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{alternate_comment_dates[:three_days]}.\nField: End Date set to #{alternate_comment_dates[:sixty_six_days]}."
          },
          {
              :protocol => glucose_motivational_protocol,
              :comment => "Field: Status set to Active.\nField: Lowest Valid Reading changed from 30 to 29\nField: Low Level Alert changed from 40 to 39\nField: Critical High Level Alert changed from 280 to 281\nField: Amount of time between reminders (unit) changed from MINUTE to YEAR\nField: Start Date set to #{comment_dates[:base]}.\nField: End Date set to #{comment_dates[:one_year]}.\nField: Service Message changed from This is my fourth service message. to\nField: Time Between Each Cycle (unit) changed from WEEK to DAY",
              :alternate_comment => "Field: Status set to Active.\nField: Lowest Valid Reading changed from 30 to 29\nField: Low Level Alert changed from 40 to 39\nField: Critical High Level Alert changed from 280 to 281\nField: Amount of time between reminders (unit) changed from MINUTE to YEAR\nField: Start Date set to #{alternate_comment_dates[:base]}.\nField: End Date set to #{alternate_comment_dates[:one_year]}.\nField: Service Message changed from This is my fourth service message. to\nField: Time Between Each Cycle (unit) changed from WEEK to DAY"
          }
      ]
      
      # Start on the Update Patient Protocol List page
      page = UpdatePatientProtocolListPage.new(@browser)

      # Validate the audit trail entries
      audit_list.each do |audit_entry|
        page.navigateToStaffAuditTrail(audit_entry[:protocol].name)
        validate_staff_audit_trail(audit_entry[:comment], audit_entry[:alternate_comment], 1, 2, audit_user)
        page = AuditTrailPage.new(@browser)
        page.protocols_element.click
        page = UpdatePatientProtocolListPage.new(@browser)
        page.waitForLoadingToClear
      end
    end
  end
end


describe 'Load Veteran Readings (CMS-1370B)' do
  context "Create Message Data To Test Filters in CMS-1370B" do
    it "Logout with current user and log back in with required user" do
      # Start on the Messages page
      page = VetMessagesPage.new(@veteran_browser)

      # Sign out
      page.sign_out
    end

    it "log in with correct user and send in multiple readings, then log out" do
      # Set up variables
      test_patient = {
          :username => @patient18Name.split(',')[1] + "." + @patient18Name.split(', ')[0],
          :password => "pass"
      }
      vitals_readings = ["OXYGEN61", "WE51", "TEMP96", "BG31", "OXYGEN65", "WE56", "TEMP100", "BG36", "OXYGEN71", "WE71", "TEMP97", "BG41", "OXYGEN70", "WE105", "TEMP99", "BG35", "OXYGEN66", "WE110", "TEMP101", "BG40"]

      # Start on the veteran login page
      page = VeteranLoginPage.new(@veteran_browser)

      # Sign in
      page.login_with(test_patient[:username], test_patient[:password])
      page = VetMessagesPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Send each of the specified readings
      vitals_readings.each do |vital_reading|
        page.send_message(vital_reading)
      end

      # Sign out
      page.sign_out
    end
  end
end


describe 'Message History Page Including Filters (CMS-1370C)' do
  date_obj = DateTime.now
  date_format = "%m/%d/%Y"
  today_date = date_obj.strftime(date_format)
  date_time_stamp = Regexp.new("#{today_date} \\d*:\\d{2}:\\d{2}")

  def go_to_patient_message_history(search_param, patient_name)
    # Start on the Patient Search page
    page = PatientSearchPage.new(@browser)
    page.waitForPageToLoad

    # Search for and select the desired patient
    page.search_patient_and_select(search_param, patient_name)

    # Navigate to the patient's message history
    page.message_history_element.click
    page = PatientMessageHistoryPage.new(@browser)
    page.waitForLoadingToClear
    page.wait_for_messages_to_load
  end


  context 'Validate Message History Filters' do
    # Default to the staff Message History page in each test
    let(:page) { PatientMessageHistoryPage.new(@browser) }

    it 'displayed no message history message' do
      # Switch from the admin view to the staff view if necessary
      if !page.patient_search_element.visible? then
        page = AdminPage.new(@browser)
        page.GoToStaffSearch
      end

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page

      # Go to the desired patient's message history
      go_to_patient_message_history(@patient06Name.split(',')[0], @patient06Name)
      page = PatientMessageHistoryPage.new(@browser)

      # Verify that the message history is empty for this patient
      expect(page.no_message_history).not_to be_empty
      expect(page.message_history_table_element.visible?).to be_false
      expect(page.no_message_history).to eq('This patient currently does not have any data to display. Please verify that the selected patient is assigned to a protocol, and that the patient is receiving messages from Annie and is sending their appropriate readings.')
      expect(page.message_history_pagination_element.visible?).to be_false
    end
  end

  context 'when there is message history available for the patient' do
    # Default to the staff Message History page in each test
    let(:page) { PatientMessageHistoryPage.new(@browser) }

    it 'displayed message history messages' do
      # Go to the desired patient's message history
      go_to_patient_message_history(@patient01Name.split(',')[0], @patient01Name)

      # Verify that the message history is populated for this patient
      expect(page.message_history_table_element.visible?).to be_true
      expect(page.message_history_pagination?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Thank you. You are not subscribed to any protocols at this time.').exists?).to be_true
    end

    it 'verify date filter operations' do
      # Set up variables
      date_format = "%m/%d/%Y"
      today_date = DateTime.now.strftime(date_format)

      # Verify the pre-populated date filter values
      expect(page.msghistory_date_from).to eq('11/16/2013')
      expect(page.msghistory_date_to).to eq(today_date)

      # Verify filter dates that should not return any results
      page.msghistory_date_from = "11/16/2012\r"
      page.msghistory_date_to = "06/27/2013\r"
      expect(page.no_results).to eq("Data Not Found\nPlease search again. We could not find any data based on your search criteria.")
      page.reset

      # Verify filter dates that should return limited results
      page.msghistory_date_from = "01/07/2014\r"
      expect(page.message_history_table_element.cell_element(text: '01/08/2014').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: '01/07/2014').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: '11/16/2013').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(16)
      page.reset

      page.msghistory_date_to = "01/06/2014\r"
      expect(page.message_history_table_element.cell_element(text: '11/16/2013').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: '01/08/2014').exists?).to be_false
      expect(page.message_history_table_element.cell_element(text: '01/07/2014').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(17)
      page.reset

      page.msghistory_date_from = "01/07/2014\r"
      page.msghistory_date_to = "01/07/2014\r"
      expect(page.message_history_table_element.cell_element(text: '01/07/2014').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: '01/08/2014').exists?).to be_false
      expect(page.message_history_table_element.cell_element(text: '11/16/2013').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(6)

      page.msghistory_date_to = "01/06/2014\r"
      expect(page.no_results).to eq("Data Not Found\nPlease search again. We could not find any data based on your search criteria.")
      page.reset
    end

    it 'default sort order by date/time descending' do
      # Grab the message history table from the page
      message_history_table_on_page = page.message_history_table_element
      row_index = 0

      # Verify that the messages are in the correct order
      while message_history_table_on_page[row_index + 2].exists? do
        # Get the dates from the table rows & verify they are in the correct order
        current_row_date = Date.strptime(page.message_history_list[row_index].date_element.text, "%m/%d/%Y")
        next_row_date = Date.strptime(page.message_history_list[row_index + 1].date_element.text, "%m/%d/%Y")
        expect(current_row_date).to be >= next_row_date

        # Get the times from the table rows & verify they are in the correct order if the dates are the same
        if current_row_date == next_row_date
          current_row_time = Time.parse(page.message_history_list[row_index].time_element.text)
          next_row_time = Time.parse(page.message_history_list[row_index + 1].time_element.text)
          expect(current_row_time).to be >= next_row_time
        end

        # Move to the next table row
        row_index += 1
      end
    end

    it 'sort order by date/time ascending' do
      # Bring the sorting icons into view
      page.wait_for_messages_to_load
      page.message_sort_icon_element.focus
      page.message_sort_icon_element.scroll_into_view
      wait = Selenium::WebDriver::Wait.new(:timeout => 120)        # seconds
      wait.until {page.message_sort_icon_element.visible?}

      # Sort the messages by date
      page.message_sort_icon

      # Grab the message history table from the page
      message_history_table_on_page = page.message_history_table_element
      row_index = 0

      # Verify that the messages are in the correct order
      while message_history_table_on_page[row_index + 2].exists? do
        # Get the dates from the table rows & verify they are in the correct order
        current_row_date = Date.strptime(page.message_history_list[row_index].date_element.text, "%m/%d/%Y")
        next_row_date = Date.strptime(page.message_history_list[row_index + 1].date_element.text, "%m/%d/%Y")
        expect(current_row_date).to be <= next_row_date

        # Get the times from the table rows & verify they are in the correct order if the dates are the same
        if current_row_date == next_row_date
          current_row_time = Time.parse page.message_history_list[row_index].time_element.text
          next_row_time = Time.parse page.message_history_list[row_index + 1].time_element.text
          expect(current_row_time).to be <= next_row_time
        end

        # Move to the next table row
        row_index += 1
      end

      # sleep 10  # No clue why this sleep is here, remove if all is well
    end

    it 'Verify presence of critical alert icon' do
      # Verify that the critical icon is found where it is expected
      row_index = 14
      expect(page.message_history_list[row_index].critical_icon_element.visible?).to be_true

      # Navigate back to the Patient Search page
      page.patient_search_element.click
    end
  end

  context 'Verify the filters' do
    # Verify that the message log contents are consistent with the protocol filter field value
    def verify_message_log_protocol_filter_results(possible_protocol_names, filter_value = "All")
      # Set up variables
      message_index = 0

      # Start on the patient's Message History page
      page = PatientMessageHistoryPage.new(@browser)

      # Verify that the messages shown match the filters set
      if filter_value == 'All' then
        while message_index <= (page.message_history_table_element.count - 2)
          # Reset the flag
          is_message_protocol_value_valid = false

          # Get the message protocol value
          message_protocol_value = page.message_history_list[message_index].protocol_element.text

          # Check the message protocol value against each of the possible protocol names
          possible_protocol_names.each do |protocol_name|
            if message_protocol_value == protocol_name
              is_message_protocol_value_valid = true
            end
          end

          # Verify that the message protocol value matched one of the possible protocol names
          expect(is_message_protocol_value_valid).to be_true

          # Increment the index
          message_index += 1
        end
      else
        while message_index <= (page.message_history_table_element.count - 2)
          expect(page.message_history_list[message_index].protocol_element.text).to eq(filter_value)
          message_index += 1
        end
      end
    end

    # Set up context variables
    no_message_log_filter_results_message = "Data Not Found\nPlease search again. We could not find any data based on your search criteria."
    filter_all_value = "All"
    expected_report_entries = ["Select Reading", "Pulse Oximetry (SpO2)", "Weight Loss", "Temperature", "Blood Glucose"]

    # Start each test on the staff Message History page
    let(:page) { PatientMessageHistoryPage.new(@browser) }

    it 'search for the patient and navigate to message history page' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page

      # Navigate to the patient's message history
      go_to_patient_message_history(@patient18Name.split(',')[0], @patient18Name)
      page = PatientMessageHistoryPage.new(@browser)
      page.waitForLoadingToClear
    end

    it 'validate contents of filter drop down lists' do
      # Set up variables
      expected_protocol_values = [updated_pulse_ox_protocol.name, weight_temperature_protocol.name, glucose_motivational_protocol.name]
      expected_status_values = ["Sent", "Received"]

      # Verify the protocol list
      page.msghistory_protocols = filter_all_value
      expect(page.msghistory_protocols_element.selected_options[0]).to eq(filter_all_value)
      expected_protocol_values.each do |protocol_value|
        expect(page.msghistory_protocols_element).to include(protocol_value)
      end

      # Verify the status list
      page.msghistory_status = filter_all_value
      expect(page.msghistory_status_element.selected_options[0]).to eq(filter_all_value)
      expected_status_values.each do |status_value|
        expect(page.msghistory_status_element).to include(status_value)
      end
    end

    it 'second page should display 20 messages' do
      # Navigate to the second page of messages
      page.message_history_pagination_element.list_item_element(:text => '2').link_element.click

      # Verify the number of messages on the page
      expect(page.message_history_table_element.count).to eq(21)

      # Navigate back to the first page of messages
      page.message_history_pagination_element.list_item_element(:text => '1').link_element.click
    end

    it 'validate message filters' do
      # Set up variables
      date_format = "%m/%d/%Y"
      today_date = DateTime.now.strftime(date_format)
      possible_protocol_names = [updated_pulse_ox_protocol.name, weight_temperature_protocol.name, glucose_motivational_protocol.name]

      # Verify message filtering and counts for the first protocol
      page.msghistory_protocols = possible_protocol_names[0]
      verify_message_log_protocol_filter_results(possible_protocol_names, possible_protocol_names[0])
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_true
      expect(page.message_history_table_element.count).to eq(11)

      # Verify sent message count for the first protocol
      page.msghistory_status = 'Sent'
      verify_message_log_protocol_filter_results(possible_protocol_names, possible_protocol_names[0])
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(6)

      # Verify date filtering for the first protocol
      page.msghistory_date_from = "01/07/" + (Date.today.year + 1).to_s + "\r"
      page.msghistory_date_to = "01/07/" + (Date.today.year + 1).to_s + "\r"
      expect(page.no_results).to eq(no_message_log_filter_results_message)

      # Reset the filter fields
      page.reset

      # Verify the received message count for all protocols
      page.msghistory_status = 'Received'
      verify_message_log_protocol_filter_results(possible_protocol_names)
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_false
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_true
      expect(page.message_history_table_element.count).to eq(21)

      # Verify the received message count for the second protocol
      page.msghistory_protocols = possible_protocol_names[1]
      verify_message_log_protocol_filter_results(possible_protocol_names, possible_protocol_names[1])
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(11)

      # Verify date filtering for the second protocol
      page.msghistory_date_from = "01/07/" + (Date.today.year + 1).to_s + "\r"
      page.msghistory_date_to = "01/07/" + (Date.today.year + 1).to_s + "\r"
      expect(page.no_results).to eq(no_message_log_filter_results_message)

      # Reset the filter fields
      page.reset

      # Verify the message counts for the third protocol
      page.msghistory_protocols = possible_protocol_names[2]
      verify_message_log_protocol_filter_results(possible_protocol_names, possible_protocol_names[2])
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_true
      expect(page.message_history_table_element.count).to eq(12)

      # Verify the received message count within the date range for the third protocol
      page.msghistory_status = 'Received'
      page.msghistory_date_from = today_date
      page.msghistory_date_to = "01/07/" + (Date.today.year + 1).to_s + "\r"
      verify_message_log_protocol_filter_results(possible_protocol_names, possible_protocol_names[2])
      expect(page.message_history_table_element.cell_element(text: 'Received').exists?).to be_true
      expect(page.message_history_table_element.cell_element(text: 'Sent').exists?).to be_false
      expect(page.message_history_table_element.count).to eq(6)
    end

    it 'validate filters return to their default state when reset is clicked' do
      # Set up variables
      date_format = "%m/%d/%Y"
      today_date = DateTime.now.strftime(date_format)

      # Reset the filter fields
      page.reset

      # Verify that the filter fields have the expected values
      expect(page.msghistory_protocols_element.selected_options[0]).to eq(filter_all_value)
      expect(page.msghistory_status_element.selected_options[0]).to eq(filter_all_value)
      expect(page.msghistory_date_from).to eq(today_date)
      expect(page.msghistory_date_to).to eq(today_date)
    end

    it "Navigate to reports and validate content of Readings drop down list" do
      # Navigate to the Reports page
      page.reports_element.click
      page = StaffReportsPage.new(@browser)
      page.waitForLoadingToClear

      # Verify that the expected reports are available
      expected_report_entries.each do |report_entry|
        expect(page.vitals_element).to include(report_entry)
      end
    end

    it "Validate content of reports tables" do
      def generate_chart_title(patient_name, vital_name)
        patient_name + "'s " + vital_name
      end

      # Set variables
      patient_name = "Zztest Patient18"
      report_values = [
          {
              :report_name => expected_report_entries[1],
              :reading_count => 6,
              :reading_values => ["61", "65", "71", "70", "66"],
              :chart_title => generate_chart_title(patient_name, expected_report_entries[1])
          },
          {
              :report_name => expected_report_entries[2],
              :reading_count => 6,
              :reading_values => ["51", "56", "71", "105", "110"],
              :chart_title => generate_chart_title(patient_name, expected_report_entries[2])
          },
          {
              :report_name => expected_report_entries[3],
              :reading_count => 6,
              :reading_values => ["96", "100", "97", "99", "101"],
              :chart_title => generate_chart_title(patient_name, expected_report_entries[3])
          },
          {
              :report_name => expected_report_entries[4],
              :reading_count => 6,
              :reading_values => ["31", "36", "41", "35", "40"],
              :chart_title => generate_chart_title(patient_name, expected_report_entries[4])
          }
      ]

      report_values.each do |report_entry|
        reading_index = 0

        # Start on the Reports page
        page = StaffReportsPage.new(@browser)
        page.waitForLoadingToClear

        # Select the report and verify the reading count
        page.vitals = report_entry[:report_name]
        expect(page.reading_table_element.rows).to eq(report_entry[:reading_count])

        # Verify the readings
        report_entry[:reading_values].each do |current_reading_value|
          expect(page.readings[reading_index].date_element.text).to match(date_time_stamp)
          expect(page.reading_table_value_element(reading_index + 1).text).to eq(current_reading_value)

          reading_index += 1
        end

        expect(page.chart_title).to eq(report_entry[:chart_title])
      end
    end

    it "Navigate to protocol list page and disable a protocol" do
      @templateName = "Weight Temperature Custom"
      page = StaffReportsPage.new(@browser)
      page.protocols_tab_element.click

      page = UpdatePatientProtocolListPage.new(@browser)
      @list = page.patient_protocol_list
      @index = page.get_index_by_text(@list, @templateName)
      protocol = page.patient_protocol_list[@index]
      protocol.select_activeprotocoldisable
      page.text_area = "disabling Weight Temperature Custom Protocol"
      page.comment_okay

    end


    it "Validate the disabled protocol in the audit trail" do
      # Set up variables
      date_obj_today = DateTime.now
      date_obj_yesterday = date_obj_today - 1
      date_format = "%Y-%m-%d"
      comment_dates = {
          :base => date_obj_today.strftime(date_format),
          :three_days => (date_obj_today + 3).strftime(date_format),
          :fourteen_sixty_one_days => (date_obj_today + 1461).strftime(date_format),
          :sixty_six_days => (date_obj_today + 66).strftime(date_format)
      }
      alternate_comment_dates = {
          :base => date_obj_yesterday.strftime(date_format),
          :three_days => (date_obj_yesterday + 3).strftime(date_format),
          :fourteen_sixty_one_days => (date_obj_yesterday + 1461).strftime(date_format),
          :sixty_six_days => (date_obj_yesterday + 66).strftime(date_format)
      }
      current_user = @full_name
      default_audit_user = "System"
      test_protocol = weight_temperature_protocol
      audit_row_count = 8
      audit_list = [
          { :expected_comment => "disabling Weight Temperature Custom Protocol\nField: Status changed from Active to Removed\nField: Status changed from Expired to Removed\nField: Status changed from Active to Removed", :user => current_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Active to Expired", :user => default_audit_user },
          {
              :expected_comment => "Field: Status set to Active.\nField: Start Date set to #{comment_dates[:base]}.\nField: End Date set to #{comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{comment_dates[:three_days]}.\nField: End Date set to #{comment_dates[:sixty_six_days]}.",
              :alternate_comment => "Field: Status set to Active.\nField: Start Date set to #{alternate_comment_dates[:base]}.\nField: End Date set to #{alternate_comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{alternate_comment_dates[:three_days]}.\nField: End Date set to #{alternate_comment_dates[:sixty_six_days]}.",
              :user => current_user
          }
      ]

      # Start on the Update Patient Protocol List page
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Open the audit trail for the desired protocol
      inactive_protocol_list = page.inactivepatient_protocol_list
      protocol_index = page.get_index_by_text(inactive_protocol_list, test_protocol.name)
      protocol_on_page = page.inactivepatient_protocol_list[protocol_index]
      protocol_on_page.activeprotocolaudit

      # Verify that the expected audit trail entries are present
      audit_list.each_with_index do |audit_entry, audit_index|
        if audit_entry[:alternate_comment]
          validate_staff_audit_trail(audit_entry[:expected_comment], audit_entry[:alternate_comment], audit_index + 1, audit_row_count, audit_entry[:user])
        else
          validate_staff_audit_trail(audit_entry[:expected_comment], audit_index + 1, audit_row_count, audit_entry[:user])
        end
      end
    end

    it "navigate back to protocol list and re-enable protocol" do
      # Set up variables
      test_protocol = weight_temperature_protocol

      # Start on the Audit Trail page
      page = AuditTrailPage.new(@browser)
      page.waitForLoadingToClear

      # Navigate to the Protocols page
      page.protocols_element.click
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Re-enable an inactive protocol
      inactive_protocol_list = page.inactivepatient_protocol_list
      protocol_index = page.get_index_by_text(inactive_protocol_list, test_protocol.name)
      protocol_on_page = page.inactivepatient_protocol_list[protocol_index]
      protocol_on_page.select_inactiveprotocolenable

      # Enter an enabling comment
      page.text_area = "Re-enable #{test_protocol.name} Protocol"
      page.enable_okay
      page = UpdatePatientProtocolEditPage.new(@browser)
      page.waitForLoadingToClear

      # Save the protocol without any changes
      page.protocol_save_top
      page.modalwindow_ok
    end

    it "Validate Re-enabled protocol on the Audit page" do
      # Set up variables
      date_obj_today = DateTime.now
      date_obj_yesterday = date_obj_today - 1
      date_format = "%Y-%m-%d"
      comment_dates = {
          :base => date_obj_today.strftime(date_format),
          :three_days => (date_obj_today + 3).strftime(date_format),
          :fourteen_sixty_one_days => (date_obj_today + 1461).strftime(date_format),
          :sixty_six_days => (date_obj_today + 66).strftime(date_format)
      }
      alternate_comment_dates = {
          :base => date_obj_yesterday.strftime(date_format),
          :three_days => (date_obj_yesterday + 3).strftime(date_format),
          :fourteen_sixty_one_days => (date_obj_yesterday + 1461).strftime(date_format),
          :sixty_six_days => (date_obj_yesterday + 66).strftime(date_format)
      }
      current_user = @full_name
      default_audit_user = "System"
      test_protocol = weight_temperature_protocol
      audit_row_count = 9
      audit_list = [
          { :expected_comment => "Re-enable Weight Temperature Custom Protocol\nField: Status changed from Removed to Active\nField: Status changed from Removed to Active\nField: Status changed from Removed to Active", :user => current_user },
          { :expected_comment => "disabling Weight Temperature Custom Protocol\nField: Status changed from Active to Removed\nField: Status changed from Expired to Removed\nField: Status changed from Active to Removed", :user => current_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Expired to Expired", :user => default_audit_user },
          { :expected_comment => "Field: Status changed from Active to Expired", :user => default_audit_user },
          {
              :expected_comment => "Field: Status set to Active.\nField: Start Date set to #{comment_dates[:base]}.\nField: End Date set to #{comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{comment_dates[:three_days]}.\nField: End Date set to #{comment_dates[:sixty_six_days]}.",
              :alternate_comment => "Field: Status set to Active.\nField: Start Date set to #{alternate_comment_dates[:base]}.\nField: End Date set to #{alternate_comment_dates[:fourteen_sixty_one_days]}.\nField: Start Date set to #{alternate_comment_dates[:three_days]}.\nField: End Date set to #{alternate_comment_dates[:sixty_six_days]}.",
              :user => current_user
          }
      ]

      # Start on the patient Protocols page
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Open the audit trail for the desired protocol
      protocol_list = page.patient_protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol.name)
      protocol_on_page = page.patient_protocol_list[protocol_index]
      protocol_on_page.activeprotocolaudit

      # Verify that the expected audit trail entries are present
      audit_list.each_with_index do |audit_entry, audit_index|
        if audit_entry[:alternate_comment]
          validate_staff_audit_trail(audit_entry[:expected_comment], audit_entry[:alternate_comment], audit_index + 1, audit_row_count, audit_entry[:user])
        else
          validate_staff_audit_trail(audit_entry[:expected_comment], audit_index + 1, audit_row_count, audit_entry[:user])
        end
      end
    end
  end
end

