#########################################################################################
# Stories:   CMS-2115
#
# Tests:
#
# Acceptance Criteria:  CMS-2869, CMS-2870
#
#
#########################################################################################
require 'spec_helper'
require 'htmlentities'
require 'yaml'

consent_form_data = YAML.load File.open('consent_form_data.yml')

describe 'Consent Form (CMS-2869, CMS-2870)' do
  context 'Validate consent form for patient registration' do
    it 'Validate the contents of the consent form' do
      # Set variables
      test_patient = {
          :user_name => 'zztest.patient47',
          :password => 'pass'
      }

      # Start on the Messages page
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      wait = Selenium::WebDriver::Wait.new(:timeout => 20)
      wait.until{page.messages_element.visible?}

      # Verify the navigation links
      expect(page.messages_element.visible?).to be_true
      expect(page.reports_element.visible?).to be_true
      expect(page.schedule_element.visible?).to be_true
      expect(page.account_element.visible?).to be_true
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true

      # Sign out
      page.sign_out

      # Sign back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear
      expect(page.user_nav).not_to be_nil

      # Bring up the consent form
      page.consent_form
      page.waitForLoadingToClear

      # Verify the page contents
      expect(page.heading_consent_form).to eq(consent_form_data["title"])
      expect(page.modal_consent_form).to eq(consent_form_data["content"])
      expect(page.consent_date_element.visible?).to be_true
      expect(page.consent_time_element.visible?).to be_true

      # Dismiss the consent form
      page.button_consent_dismiss
    end
  end
end
