#########################################################################################
# Stories:   CMS-614
#
# Tests:  CMS-1342
#
# Acceptance Criteria:  CMS-1189, CMS-1199, CMS-1259
#
#
#########################################################################################

require 'spec_helper'

describe 'About Link Messaging Staff Application Information (CMS-614 Part B)' do
  wait = Selenium::WebDriver::Wait.new(:timeout => 10)

  context 'About link launched from messaging app' do
    it 'About link application information using messaging app' do
      # Start from the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad
      if !page.patient_search_element.visible? then
        page = AdminPage.new(@browser)
        page.GoToStaffSearch
        page = PatientSearchPage.new(@browser)
        page.patient_search_element.click
        page.waitForPageToLoad
      end
      page.waitForLoadingToClear

      # Navigate to the About modal
      page.about
      page.wait_until(2, "wait for about modal to display") do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Verify that elements that should not be visible are not present
      expect(page.version).not_to eq('Annie - XXX')
      expect(page.clinical_admin).not_to eq('Clinical XXX')
      expect(page.clinical_admin_info).not_to eq('Create protocols and XXX')
      expect(page.clinical_staff).not_to eq('Clinical Subordinates')
      expect(page.clinical_staff_info).not_to eq('Assign/edit protocols, XXXXX')
      expect(page.veteran).not_to eq('Veteran XXX')
      expect(page.veteran_info).not_to eq('View messages history and XXXXXX')

      # Verify that expected page content is displayed
      expect(page.version).to eq('Annie - v1.0.0')
      expect(page.clinical_admin).to eq('Clinical Administrators')
      expect(page.clinical_admin_info).to eq('Create protocols and protocol templates')
      expect(page.clinical_staff).to eq('Clinical Staff')
      expect(page.clinical_staff_info).to eq('Assign/edit protocols, view messages history and reports for patients')
      expect(page.veteran).to eq('Veteran')
      expect(page.veteran_info).to eq('View messages history and reports')
      wait.until {page.get_link_by_text('Training Materials')}
      expect(page.more_info_element.text).to eq('Training Materials')

      # Dismiss the modal dialog
      page.about_ok
      page.wait_until(2, "wait for about modal to disappear") do
        begin
          wait.until {!(page.about_modal_body_element.visible?)}
        rescue
          sleep(1)
        end
      end

      # Open the About modal dialog
      page.about
      page.wait_until(2, "wait for about modal to display") do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Verify the expected page content is displayed
      expect(page.training_text).to eq("Please view Training Materials to access User Guides and additional information about Annie.")

      # Navigate to the Training Materials info
      page.more_info

      # Verify the expected modal dialog messages are present
      expect(page.modal_header_element.text).to eq('Accessing External Link')
      expect(page.modal_text).to eq('You are about to leave the Annie application. Please select \'Cancel\' to return to the application, or \'Continue\' to proceed to the external link.')

      # Select cancel to dismiss the modal
      page.cancel_modal

      # Navigate to the About modal dialog
      page.about
      page.wait_until(2, "wait for about modal to display") do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Navigate to the Training Materials info
      page.more_info

      # Verify that the page URL is correct
      expect(page.continue_modal_element.href).to eq("https://mobilehealth.domain/training/annie-providers")
    end
  end
end

describe 'About Link Messaging Admin Application Information (CMS-614 Part B)' do
  wait = Selenium::WebDriver::Wait.new(:timeout => 10)

  context 'About link launched from admin messaging app' do
    it 'AboutLink_Application_Information using Admin messaging App' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad

      # Navigate to the Staff Admin page
      page.goto_admin_page
      page = AdminPage.new(@browser)
      page.waitForLoadingToClear

      # Navigate to the About modal dialog
      page.about_nav
      page.wait_until(2, "wait for about modal to display") do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Verify the elements that should not be present are not displayed
      expect(page.version).not_to eq('Annie - XXX')
      expect(page.clinical_admin).not_to eq('Clinical XXX')
      expect(page.clinical_admin_info).not_to eq('Create protocols and XXX')
      expect(page.clinical_staff).not_to eq('Clinical Subordinates')
      expect(page.clinical_staff_info).not_to eq('Assign/edit protocols, XXXXX')
      expect(page.veteran).not_to eq('Veteran XXX')
      expect(page.veteran_info).not_to eq('View messages history and XXXXXX')

      # Verify the expected content is displayed
      expect(page.version).to eq('Annie - v1.0.0')
      expect(page.clinical_admin).to eq('Clinical Administrators')
      expect(page.clinical_admin_info).to eq('Create protocols and protocol templates')
      expect(page.clinical_staff).to eq('Clinical Staff')
      expect(page.clinical_staff_info).to eq('Assign/edit protocols, view messages history and reports for patients')
      expect(page.veteran).to eq('Veteran')
      expect(page.veteran_info).to eq('View messages history and reports')
      wait.until {page.get_link_by_text('Training Materials')}
      expect(page.more_info_element.text).to eq('Training Materials')

      # Dismiss the About modal dialog
      page.about_ok
      page.wait_until(2, 'wait for about modal to disappear') do
        begin
          wait.until {!(page.about_modal_body_element.visible?)}
        rescue
          sleep(1)
        end
      end

      # Navigate to the About modal dialog
      page.about_nav
      page.wait_until(2, 'wait for about modal to display') do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Verify the expected page content is displayed
      expect(page.training_text).to eq('Please view Training Materials to access User Guides and additional information about Annie.')

      # Navigate to the Training Materials more info page
      page.more_info

      # Verify the expected page content is displayed
      expect(page.modal_header_element.text).to eq('Accessing External Link')
      expect(page.modal_text).to eq('You are about to leave the Annie application. Please select \'Cancel\' to return to the application, or \'Continue\' to proceed to the external link.')

      # Select the Cancel button to dismiss the button
      page.cancel_modal

      # Navigate to the About modal dialog
      page.about_nav
      page.wait_until(2, 'wait for about modal to display') do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Navigate to the Training Materials more info page
      page.more_info

      # Verify the page URL is correct
      expect(page.continue_modal_element.href).to eq('https://mobilehealth.domain/training/annie-providers')
    end
  end
end
