#########################################################################################
# Stories:   CMS-1125, CMS-36, CMS-1861
#
# Tests:   CMS-2234
#
# Acceptance Criteria:  CMS-807, CMS-1871, CMS-1873, CMS-1871, CMS-1873
#
#
#########################################################################################

require 'spec_helper'
require 'selenium-webdriver'

describe 'Validate Welcome Message (CMS-807)' do
  def patient_message_history(search_param, patient_name)
    page = PatientSearchPage.new(@browser)
    page.waitForPageToLoad

    page.search_patient_and_select(search_param, patient_name)

    page.message_history_element.click
    page = PatientMessageHistoryPage.new(@browser)
    page.waitForLoadingToClear

    page.wait_for_messages_to_load
  end

  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "Validate welcome message for messaging app" do
    it 'Validate messaging app welcome message and sign out' do
      # Set variables
      current_staff_user = {
          :full_name => @full_name,
          :user_name => @current_staff_username,
          :password => @current_staff_password
      }
      current_staff_user[:first_name] = current_staff_user[:full_name].split(' ')[0].strip

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      if !page.patient_search_element.visible? then
        page = AdminPage.new(@browser)
        page.GoToStaffSearch
      end
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page
      page.waitForPageToLoad
      wait = Selenium::WebDriver::Wait.new(:timeout => 120)
      wait.until {page.patient_go_element.visible?}
      expect(page.user_nav).not_to be_nil

      # Verify the expected welcome message is displayed
      page.wait_until(60, 'Welcome Message not populated') do
        page.user_nav_element.text != 'Welcome,'
      end
      expect(page.user_nav_element.text).to eq ('Welcome, ' + current_staff_user[:first_name])

      # Sign out
      page.wait_until(60, 'Sign Out Button Not Visible!') do
        page.sign_out_element.visible?
      end
      page.sign_out

      # Sign back in
      page = VALoginPage.new(@browser)
      page.login_with current_staff_user[:user_name], current_staff_user[:password]
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Switch to the admin view
      page.user_nav
      expect(page.switch_admin_view_element.text).to eq('Switch To Admin View')
      page.switch_admin_view
      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      # Switch to staff view
      page.user_nav
      expect(page.switch_staff_view_element.text).to eq('Switch To Staff View')

      # Sign out
      page.sign_out
    end

    it 'Validate staff user doesn\'t have access to clinical admin dropdown (CMS-3298)' do
      # Set variables
      test_user = {
          :user_name => 'vehu015',
          :password => 've015hu!'
      }

      # Sign in
      page = VALoginPage.new(@browser)
      page.login_with test_user[:user_name], test_user[:password]
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Verify the expected user menu options
      expect(page.user_nav).not_to be_nil
      expect(page.switch_admin_view_element).not_to exist

      # Sign out
      page.sign_out
    end
  end
end
