#########################################################################################
# Stories:   CMS-614
#
# Tests:  CMS-1342
#
# Acceptance Criteria:  CMS-1189, CMS-1199, CMS-1259
#
#
#########################################################################################

require 'spec_helper'

describe 'About Link Patient Registration Application Information (CMS-614 Part A)' do
  context 'About link launched from patient registration' do
    it 'About link application information using patient registration app' do
      # Start on the Messages page
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.wait_for_page_to_load
      page.wait_for_timezone_to_load
      wait = Selenium::WebDriver::Wait.new(:timeout => 20)
      wait.until{page.messages_element.visible?}

      # Navigate to the About modal dialog
      page.about
      page.wait_until(2, 'wait for about modal to display') do
        wait.until {page.about_modal_body_element.visible?}
      end
      wait = Selenium::WebDriver::Wait.new(:timeout => 120)
      wait.until {page.version == 'VA Annie - v1.0.0'}

      # Verify the expected page content is displayed
      expect(page.version).to eq('VA Annie - v1.0.0')
      expect(page.send_reading).to eq('Send readings')
      expect(page.view_messages).to eq('View messages')
      expect(page.view_reports).to eq('View reports')
      expect(page.view_schedule).to eq('View schedule')
      expect(page.veteran).to eq('Veteran')
      expect(page.change_preference).to eq('Change preferences')
      expect(page.more_info_element.text).to eq('Training Materials')

      # Dismiss the About modal dialog
      page.about_ok
      page.wait_until(2, 'wait for about modal to disappear') do
        begin
          wait.until {!(page.about_modal_body_element.visible?)}
        rescue
          sleep(1)
        end
      end

      # Navigate to the About modal dialog
      page.about
      page.wait_until(2, 'wait for about modal to display') do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Verify the expected page content is displayed
      expect(page.training_text).to eq('Please view Training Materials to access User Guides and additional information about Annie.')

      # Navigate to the Training Materials more info page
      page.more_info

      # Verify the expected page content is displayed
      expect(page.modal_header_element.text).to eq('Accessing External Link')
      expect(page.modal_text).to eq('You are about to leave the Annie application. Please select \'Cancel\' to return to the application, or \'Continue\' to proceed to the external link.')

      # Select the Cancel button to dismiss the modal dialog
      page.cancel_modal
      page.wait_until(2, 'wait for about modal to disappear') do
        begin
          wait.until {!(page.about_modal_body_element.visible?)}
        rescue
          sleep(1)
        end
      end

      # Navigate to the About modal dialog
      page.about
      page.wait_until(2, 'wait for about modal to display') do
        wait.until {page.about_modal_body_element.visible?}
      end

      # Navigate to the Training Materials more info page
      page.more_info

      # Verify the page URL is correct
      expect(page.continue_modal_element.href).to eq('https://mobilehealth.domain/training/annie-providers')
    end
  end
end
