#########################################################################################
# Stories:   CMS-1203
#
# Tests:
#
# Acceptance Criteria:  CMS-1244, CMS-1270, CMS-1272, CMS-1653, CMS-1654, CMS-1655, CMS-1667, CMS-1672
#
#
#########################################################################################

require 'spec_helper'

# Set variables
date_today = DateTime.now.strftime('%m/%d/%Y')
test_protocol_title = 'Blood Pressure Protocol1'

patient5 = {
    :first_name => 'zztest',
    :last_name => 'patient05'
}
patient5[:display_name] = patient5[:last_name] + ', ' + patient5[:first_name]

describe 'No Protocols Messages' do
  context 'No active or inactive protocols assigned (CMS-1655)' do
    it 'displays correct message' do
      # Set variables
      test_patient = patient5

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad
      page.waitForLoadingToClear

      # Search for and select a patient
      page.patient_search_element.click
      page = page.search_patient_and_select(test_patient[:last_name], test_patient[:display_name])

      # Verify the expected page messages
      expect(page.no_assigned_protocols_message).to eq 'This patient does not have any assigned protocols. Select "Assign New Protocol" to assign a protocol.'
    end
  end

  context 'No inactive protocols assigned (CMS-1654)' do
    it "displays correct message" do
      # Set variables
      test_patient = patient5

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Search for and select a patient
      page.patient_search_element.click
      page = page.search_patient_and_select(test_patient[:last_name], test_patient[:display_name])
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Begin to assign a new protocol
      page.New_Protocol_element.when_visible.click
      page.waitForLoadingToClear

      # Select the protocol to assign and customize
      page.assignProtocolByName(test_protocol_title)
      page.customizeprotocol_yes
      page = UpdatePatientProtocolEditPage.new(@browser)
      page.waitForLoadingToClear

      # Customize and save the protocol
      page.protocol_save_top
      page.modalwindow_ok
      page = UpdatePatientProtocolListPage.new(@browser)
      page.wait_for_page_to_load

      # Verify the expected page messages
      expect(page.no_inactive_protocols_message).to eq('This patient does not have any inactive protocols.')

      # Get the list of active protocols
      protocol_list = page.patient_protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol_title)
      protocol_active_list = page.patient_protocol_list[protocol_index]

      # Verify the protocol data
      expect(protocol_active_list.name_element.text).to eq(test_protocol_title)
      expect(protocol_active_list.assigned_date_element.text).to eq(date_today)
      expect(protocol_active_list.edited_date_element.text).to eq(date_today)
      expect(protocol_active_list.edited_by_element.text).to eq(@full_name)
      expect(protocol_active_list.last_comment_element.when_visible.text).to eq('Protocol Assigned')
    end
  end

  context 'No active protocols (CMS-1653, CMS-1667)' do
    it 'displays correct message' do
      # Start on the Patient Protocol page
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Disable the active protocol
      page.disableProtocol(test_protocol_title)

      # Verify the expected page messages are displayed
      expect(page.no_active_protocols_message).to eq 'This patient does not have any active protocols. Select "Assign New Protocol" to assign a protocol.'
      expect(page.protocol_status_element.when_visible(20).text).to eq 'The Patient Protocol ' + test_protocol_title + ' is now inactive. The patient will no longer receive or send messages related to this protocol.'

      # Get the list of inactive protocols
      protocol_list = page.inactivepatient_protocol_list
      protocol_index = page.get_index_by_text(protocol_list, test_protocol_title)
      protocol_inactive_list = page.inactivepatient_protocol_list[protocol_index]

      # Verify the protocol data
      expect(protocol_inactive_list.name_element.text).to eq(test_protocol_title)
      expect(protocol_inactive_list.assigned_date_element.text).to eq(date_today)
      expect(protocol_inactive_list.edited_date_element.text).to eq(date_today)
      expect(protocol_inactive_list.inactivetype_element.text).to eq('Removed')
      expect(protocol_inactive_list.edited_by_element.text).to eq(@full_name)
      expect(protocol_inactive_list.last_comment_element.when_visible.text).to eq('comment')
    end
  end
end
