#########################################################################################
# Stories:   CMS-1909, CMS-3977
#
# Tests:    CMS-
#
# Acceptance Criteria:  CMS-4144
#
#
#########################################################################################

require 'spec_helper'

# Set variables
mobile_phone_duplicate_formatted = '571-050-0001'
mobile_phone_duplicate_raw = mobile_phone_duplicate_formatted.delete '-'
duplicate_mobile_phone_number_error_message = 'Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX.'

patient_imagepatient_ten = {
    :first_name => 'imagepatient',
    :middle_name => '',
    :last_name => 'ten',
    :dob => '04/15/1953',
    :mobile_phone_formatted => '900-824-5174',
    :gender => 'male',
    :ssn => '666-06-1010',
    :time_zone => 'Hawaii-Aleutian (GMT-10:00)',
    :home_phone_formatted => '888-976-2384',
    :email => 'laughingstock@joker.net',
    :facility => 'WASHINGTON'
}
patient_imagepatient_ten[:display_name] = patient_imagepatient_ten[:last_name] + ', ' + patient_imagepatient_ten[:first_name]
patient_imagepatient_ten[:display_name_formatted] = patient_imagepatient_ten[:last_name].capitalize + ', ' + patient_imagepatient_ten[:first_name].capitalize
patient_imagepatient_ten[:age] = calculate_age(patient_imagepatient_ten[:dob])
patient_imagepatient_ten[:mobile_phone_raw] = patient_imagepatient_ten[:mobile_phone_formatted].delete '-'
patient_imagepatient_ten[:home_phone_raw] = patient_imagepatient_ten[:home_phone_formatted].delete '-'
patient_imagepatient_ten[:search_code] = patient_imagepatient_ten[:last_name].capitalize[0, 1] + patient_imagepatient_ten[:ssn][-4, 4]

patient_ann_roger = {
    :first_name => 'ann',
    :middle_name => '',
    :last_name => 'roger',
    :dob => '05/11/1977',
    :mobile_phone_formatted => '903-636-5174',
    :home_phone_formatted => '987-676-2384',
    :gender => 'female',
    :ssn => '005-11-1977',
    :time_zone => 'Chamorro (GMT+10:00)',
    :email => 'riddlemethis@riddles.org'
}
patient_ann_roger[:display_name] = patient_ann_roger[:last_name] + ', ' + patient_ann_roger[:first_name]
patient_ann_roger[:display_name_formatted] = patient_ann_roger[:last_name].capitalize + ', ' + patient_ann_roger[:first_name].capitalize
patient_ann_roger[:age] = calculate_age(patient_ann_roger[:dob])
patient_ann_roger[:mobile_phone_raw] = patient_ann_roger[:mobile_phone_formatted].delete '-'
patient_ann_roger[:home_phone_raw] = patient_ann_roger[:home_phone_formatted].delete '-'
patient_ann_roger[:search_code] = patient_ann_roger[:last_name].capitalize[0, 1] + patient_ann_roger[:ssn][-4, 4]

describe 'Registration of Veteran by Clinical Staff/Admin (CMS-1909)' do
  # Start on the Patient Search page
  let(:page) { PatientSearchPage.new(@browser)}

  context 'when you search for a patient who is not registered in the system' do
    it 'Handle Search for Patient in VistA That Produces To Large of a Result Set' do
      # Search for a patient
      page.goto_patient_search_page
      page.patient_search_element.click
      page.patient_search_text = 'five'
      page.patient_go
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq('We\'re sorry, the patient entered is not found. Please search again or register this patient.')

      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      page.wait_until(60, 'Patient results not returned') do
        page.search_again_text == 'Please refine your search criteria to decrease the size of the result set'
      end
    end

    it 'handle "Invalid Search Criteria" returned from PVS - Clinical Staff Registration' do
      # Search for a patient
      page.goto_patient_search_page
      page.patient_search_element.click
      page.patient_search_text = 'patient72'
      page.patient_go
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq('We\'re sorry, the patient entered is not found. Please search again or register this patient.')

      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      page.wait_until(60, 'Patient results not returned') do
        page.search_again_text == 'Invalid Search Criteria'
      end
    end

    it 'no patient found in Vista' do
      # Search for a patient
      page.goto_patient_search_page
      page.patient_search_element.click
      page.patient_search_text = 'sam'
      page.patient_go
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq('We\'re sorry, the patient entered is not found. Please search again or register this patient.')

      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      page.wait_until(60, 'Patient results not returned') do
        page.search_again_text == 'We\'re sorry, the patient entered cannot be found, please search again.'
      end
    end

    it 'no patient information should be present' do
      # Search for a patient
      page.goto_patient_search_page
      page.patient_search_element.click
      page.patient_search_text = patient_imagepatient_ten[:last_name]
      page.patient_go
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq('We\'re sorry, the patient entered is not found. Please search again or register this patient.')
    end

    it 'click the registration button and bring results back from vista.' do
      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Locate the desired patient in the list of available results
      index = page.get_index_by_text(page.patient_list, patient_imagepatient_ten[:display_name])
      patient_result = page.patient_list[index]
      expect(patient_result.name).to eq(patient_imagepatient_ten[:display_name])

      # Select the desired patient
      patient_result.name_element.when_visible.click
      page.waitForLoadingToClear
    end

    it 'verify the patient registration information displays and complete registration' do
      # Start on the staff registration page
      page = StaffRegistrationPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Accept the patient consent form
      page.check_select_patient_consent_chkbox

      # Verify pre-populated patient data
      page.firstName.should eq(patient_imagepatient_ten[:first_name].capitalize)
      page.middleName.should eq(patient_imagepatient_ten[:middle_name].capitalize)
      page.lastName.should eq(patient_imagepatient_ten[:last_name].capitalize)
      page.ssn.should eq(patient_imagepatient_ten[:ssn])
      page.dob.should eq(patient_imagepatient_ten[:dob])
      page.gender.should eq(patient_imagepatient_ten[:gender])

      # Set the time zone, home & mobile phone, and email fields
      page.time_zone = patient_imagepatient_ten[:time_zone]
      page.home_phone = patient_imagepatient_ten[:home_phone_raw]
      page.mobile_phone = mobile_phone_duplicate_raw
      page.email = patient_imagepatient_ten[:email]

      # Set message preferences
      page.select_smart_phone

      # Set broadcast message preferences
      expect(page.facility_checkbox_checked?).to eq(false)
      page.check_facility_checkbox
      expect(page.facility_checkbox_checked?).to eq(true)
      expect(page.appointment_reminders_checkbox_checked?).to eq(false)
      page.check_appointment_reminders_checkbox
      expect(page.appointment_reminders_checkbox_checked?).to eq(true)

      # Submit the registration form
      page.submit

      # Verify that the expected error messages are displayed
      expect(page.facility_error).to eq('Please select a Facility')

      # Set the Facility field
      page.facility = patient_imagepatient_ten[:facility]

      # Submit the registration form
      page.submit
      page.waitForLoadingToClear

      # Verify that the expected error messages are displayed
      expect(page.duplicate_mobile_phone_error).to eq(duplicate_mobile_phone_number_error_message)

      # Set the mobile phone field to the correct value
      page.mobile_phone = patient_imagepatient_ten[:mobile_phone_raw]

      # Submit the registration form
      page.submit
    end

    it 'verify the success message and navigate to search again' do
      # Start on the Staff Registration page
      page = StaffRegistrationPage.new(@browser)
      page.waitForLoadingToClear
      expect(page.success_heading_element.text).to eq('Registration Success!')

      # Navigate to the Patient Search page
      page.patient_search_element.click
      page = PatientDetailsPageType.new(@browser)
      page.waitForLoadingToClear

      # Verify the current patient information
      expect(page.patient_details_name).to eq(patient_imagepatient_ten[:display_name_formatted])
      expect(page.patient_details_dob).to eq(patient_imagepatient_ten[:dob])
      expect(page.patient_details_age).to eq(patient_imagepatient_ten[:age])
      expect(page.patient_details_ssn).to eq(patient_imagepatient_ten[:ssn])
    end

    it 'search for another unregistered patient and cancel registration' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_text = patient_ann_roger[:ssn]
      page.patient_go
      page.waitForLoadingToClear

      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Locate the desired patient in the list of search results
      index = page.get_index_by_text(page.patient_list, patient_ann_roger[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(patient_ann_roger[:display_name])
      expect(patient_result.dob).to eq('DOB: ' + patient_ann_roger[:dob])
      expect(patient_result.age).to eq('Age: ' + patient_ann_roger[:age])
      expect(patient_result.ssn).to eq('SSN: ' + patient_ann_roger[:ssn])

      # Select the desired patient from the list of search results
      patient_result.name_element.when_visible.click
      page = StaffRegistrationPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Cancel the patient registration
      page.cancel
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Click the Patient Search element
      page.patient_search_element.click
    end

    it 'search for a second unregistered patient and continue registration' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_element.click
      page.patient_search_text = patient_ann_roger[:search_code]
      page.patient_go
      page.waitForLoadingToClear

      # Click the Register button
      page.register
      page.waitForLoadingToClear

      # Locate the desired patient in the list of search results
      index = page.get_index_by_text(page.patient_list, patient_ann_roger[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(patient_ann_roger[:display_name])
      expect(patient_result.dob).to eq('DOB: ' + patient_ann_roger[:dob])
      expect(patient_result.age).to eq('Age: ' + patient_ann_roger[:age])
      expect(patient_result.ssn).to eq('SSN: ' + patient_ann_roger[:ssn])

      # Select the desired patient from the list of search results
      patient_result.name_element.when_visible.click
      page = StaffRegistrationPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated patient data
      expect(page.firstName).to eq(patient_ann_roger[:first_name].capitalize)
      expect(page.middleName).to eq(patient_ann_roger[:middle_name].capitalize)
      expect(page.lastName).to eq(patient_ann_roger[:last_name].capitalize)
      expect(page.ssn).to eq(patient_ann_roger[:ssn])
      expect(page.dob).to eq(patient_ann_roger[:dob])
      expect(page.gender).to eq(patient_ann_roger[:gender])

      # Set the time zone, home & mobile phone, and email fields
      page.time_zone = patient_ann_roger[:time_zone]
      page.home_phone = patient_ann_roger[:home_phone_raw]
      page.mobile_phone = patient_ann_roger[:mobile_phone_raw]
      page.email = patient_ann_roger[:email]

      # Set the message preferences
      page.select_smart_phone

      # Accept the patient consent form
      page.check_select_patient_consent_chkbox

      # Submit the registration form
      page.submit
      page.waitForLoadingToClear

      # Navigate to the Patient Search page
      page.patient_search_element.click
      page.waitForLoadingToClear
      page = PatientDetailsPageType.new(@browser)

      # Verify the current patient information
      expect(page.patient_details_name).to eq(patient_ann_roger[:display_name_formatted])
      expect(page.patient_details_dob).to eq(patient_ann_roger[:dob])
      expect(page.patient_details_age).to eq(patient_ann_roger[:age])
      expect(page.patient_details_ssn).to eq(patient_ann_roger[:ssn])
      expect(page.patient_details_gender).to eq(patient_ann_roger[:gender].capitalize)
    end

    it 'validate unregistered patient pagination' do
      # Start on Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_text = 'Sev'
      page.patient_go
      page.waitForLoadingToClear

      # Click the Register button
      page.register
      page.waitForLoadingToClear
    end

    it 'patient list should have at least 2 pages' do
      # check for pagination links
      expect(page.pagination_element.items).to be >= 2
    end

    it 'first pagination link should be selected' do
      expect(page.pagination_element.list_item_element(:text => '1').class_name).to include('active')
    end

    it 'first page should display 20 patients' do
      expect(page.patients_table_element.items).to eq(20)
    end

    it 'second page should display 20 patients' do
      page.pagination_element.list_item_element(:text => '2').link_element.click
      page.waitForLoadingToClear
      expect(page.patients_table_element.items).to be <= 20
    end

    it 'second pagination link should be selected' do
      expect(page.pagination_element.list_item_element(:text => '2').class_name).to include('active')
      page.patient_search_element.click
      page.waitForLoadingToClear
    end

    it 'Verify that a staff assisted registered patient can be searched for by last initial and last 4 of ssn' do
      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_element.click
      page.patient_search_text = patient_imagepatient_ten[:search_code]
      page.patient_go
      page.waitForLoadingToClear

      # Locate and select the desired patient from the search results
      index = page.get_index_by_text(page.patient_list, patient_imagepatient_ten[:display_name_formatted])
      patient_result = page.patient_list[index]
      patient_result.name_element.when_visible.click
      page = PatientDetailsPageType.new(@browser)
      page.waitForLoadingToClear

      # Verify the patient data
      expect(page.patient_details_name).to eq(patient_imagepatient_ten[:display_name_formatted])
      expect(page.patient_details_dob).to eq(patient_imagepatient_ten[:dob])
      expect(page.patient_details_age).to eq(patient_imagepatient_ten[:age])
      expect(page.patient_details_ssn).to eq(patient_imagepatient_ten[:ssn])

      # Go back to the Patient Search page
      page = StaffPage.new(@browser)
      page.waitForLoadingToClear
      page.patient_search_element.click
      page.waitForLoadingToClear
    end
  end

  context 'Duplicate mobile phone validation on update' do
    it 'Verify that duplicate mobile phone numbers are not allowed when updating veteran information' do
      # Search for a patient
      page.patient_search_text = patient_imagepatient_ten[:search_code]
      page.patient_go
      page.waitForLoadingToClear

      # Locate and select the desired patient from the search results
      index = page.get_index_by_text(page.patient_list, patient_imagepatient_ten[:display_name_formatted])
      patient_result = page.patient_list[index]
      patient_result.name_element.when_visible.click
      page = UpdatePatientProtocolListPage.new(@browser)

      # Click the Account Settings button
      page.update_protocol_list_account_settings
      page = StaffVetProfileEditPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the patient data
      expect(page.facility).to eq(patient_imagepatient_ten[:facility])
      expect(page.facility_checkbox_checked?).to eq(true)
      expect(page.appointment_reminders_checkbox_checked?).to eq(true)

      # Verify that the mobile phone field cannot be set to a duplicate phone number
      page.mobile_phone = mobile_phone_duplicate_raw
      page.saveRegistration
      page.waitForLoadingToClear
      expect(page.duplicate_mobile_phone_error).to eq(duplicate_mobile_phone_number_error_message)
    end
  end
end
