#########################################################################################
# Stories:   CMS-156, CMS-1560, CMS-1596, CMS-1298
#
# Tests:    CMS-2229, CMS-2236, CMS-2118
#
# Acceptance Criteria:  CMS-157, CMS-1975, CMS-1976, CMS-2011, CMS2012
#
#
#########################################################################################

require 'spec_helper'

# Set variables
patient1 = {
    :first_name => 'zztest',
    :last_name => 'patient01',
    :dob => '05/05/1900',
    :mobile_phone_formatted => '1-571-050-0001',
    :gender => 'Male',
    :ssn => '999-99-9901'
}
patient1[:display_name] = patient1[:last_name] + ', ' + patient1[:first_name]
patient1[:mobile_phone_raw] = patient1[:mobile_phone_formatted].delete '-'
patient1[:age] = calculate_age(patient1[:dob])
patient1[:search_code] = patient1[:last_name].capitalize[0, 1] + patient1[:ssn][-4, 4]

patient3 = {
    :first_name => 'zztest',
    :last_name => 'patient03',
    :dob => '10/05/1900',
    :mobile_phone_formatted => '1-571-050-0003',
    :gender => 'Male',
    :ssn => '999-99-9903'
}
patient3[:display_name] = patient3[:last_name] + ', ' + patient3[:first_name]
patient3[:mobile_phone_raw] = patient3[:mobile_phone_formatted].delete '-'
patient3[:age] = calculate_age(patient3[:dob])
patient3[:search_code] = patient3[:last_name].capitalize[0, 1] + patient3[:ssn][-4, 4]

patient55 = {
    :first_name => 'zztest',
    :last_name => 'patient55',
    :dob => '05/05/1900',
    :mobile_phone_formatted => '1-703-111-0055',
    :gender => 'Male',
    :ssn => '999-99-9955'
}
patient55[:display_name] = patient55[:last_name] + ', ' + patient55[:first_name]
patient55[:mobile_phone_raw] = patient55[:mobile_phone_formatted].delete '-'
patient55[:age] = calculate_age(patient55[:dob])
patient55[:search_code] = patient55[:last_name].capitalize[0, 1] + patient55[:ssn][-4, 4]

patient83 = {
    :first_name => 'zztest',
    :last_name => 'patient83',
    :dob => '01/09/1900',
    :mobile_phone_formatted => '1-571-294-3179',
    :gender => 'Male',
    :ssn => '999-99-9983'
}
patient83[:display_name] = patient83[:last_name] + ', ' + patient83[:first_name]
patient83[:mobile_phone_raw] = patient83[:mobile_phone_formatted].delete '-'
patient83[:age] = calculate_age(patient83[:dob])
patient83[:search_code] = patient83[:last_name].capitalize[0, 1] + patient83[:ssn][-4, 4]

describe 'Display Patient List (CMS-157)' do
  let(:page) { PatientSearchPage.new(@browser)}

  context 'when you global search for a patient by last name' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient1

      # Make sure the Patient Search page is loaded
      page.goto_patient_search_page
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:last_name][0, 7])
      page.waitForLoadingToClear

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you global search for a patient by ssn' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient55

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:ssn].delete '-')
      page.waitForLoadingToClear

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you global search for a patient by mobile phone number' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient83

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:mobile_phone_raw])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you global search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For registered patient information should match expected (upper case)' do
      # Set variables
      test_patient = patient83

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:search_code])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you global search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For registered patient information should match expected (lower case)' do
      # Set variables
      test_patient = patient3

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:search_code].downcase)

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you global search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For NON registered patient information should match expected' do
      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient('P9972')
      page.waitForPageToLoad
      page.waitForLoadingToClear

      # Verify the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq('We\'re sorry, the patient entered is not found. Please search again or register this patient.')
    end
  end

  context 'when you global search for a patient by first name' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient1

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.global_search_patient(test_patient[:first_name])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])

      # Reset the search field
      page.main_search = ''
    end
  end

  context 'when you search for a patient by last name' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient1

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:last_name][0, 7])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you search for a patient by ssn' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient55

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:ssn].delete '-')

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you search for a patient by mobile phone number' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient83

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:mobile_phone_raw])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For registered patient information should match expected (upper case)' do
      # Set variables
      test_patient = patient83

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:search_code])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For registered patient information should match expected (lower case)' do
      # Set variables
      test_patient = patient3

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:search_code].downcase)

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end

  context 'when you search for a patient by 1st char of Last name & last 4 of SSN' do
    it 'For NON registered patient information should match expected' do
      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient('P9972')
      page.waitForPageToLoad
      page.waitForLoadingToClear

      # Verify the expected error messages are displayed
      expect(page.heading_not_found_element.when_visible.text).to eq('Patient Not Found')
      expect(page.patient_not_found_element.text).to eq("We're sorry, the patient entered is not found. Please search again or register this patient.")
    end
  end

  context 'when you search for a patient by first name' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient1

      # Make sure the Patient Search page is loaded
      page.waitForPageToLoad

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:first_name])

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end


    it 'patient list should have at least 4 pages' do
      # check for pagination links
      expect(page.pagination_element.items).to be >= 4
    end

    it 'first pagination link should be selected' do
      expect(page.pagination_element.list_item_element(:text => '1').class_name).to include('active')
    end

    it 'first page should display 20 patients' do
      expect(page.patients_table_element.items).to eq(20)
    end
  end

  context 'when you click on second pagination link' do
    it 'second page should display 20 patients' do
      page.pagination_element.list_item_element(:text => '2').link_element.click
      page.waitForLoadingToClear
      expect(page.patients_table_element.items).to eq(20)
    end

    it 'second pagination link should be selected' do
      expect(page.pagination_element.list_item_element(:text => '2').class_name).to include('active')
    end
  end

  context 'From Message History, Global Search redirects to Patient Search page' do
    it 'patient information should match expected' do
      # Set variables
      test_patient = patient1

      # Go to Patient Messages page
      page.message_history_element.click
      page = PatientMessageHistoryPage.new(@browser)

      # Search for a patient
      page.main_search = patient1[:first_name]
      page.main_go
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad

      # Locate the desired patient in the search results
      index = page.get_index_by_text(page.patient_list, test_patient[:display_name])
      patient_result = page.patient_list[index]

      # Verify the patient data
      expect(patient_result.name).to eq(test_patient[:display_name])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq(test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end
end
