#########################################################################################
# Stories:   CMS-2174
#
# Tests:
#
# Acceptance Criteria:
#
#
#
#########################################################################################

require 'spec_helper'

# Set variables
patient16 = {
    :first_name => 'zztest',
    :middle_name => '',
    :last_name => 'patient16',
    :ssn => '999-99-9916',
    :dob => '02/02/1900',
    :gender => 'female',
    :home_phone_formatted => '703-765-9834',
    :mobile_phone_formatted => '703-111-2223',
    :email => 'zpatient@testmail.com',
    :facility => 'WASHINGTON'
}
patient16[:display_name] = patient16[:last_name] + ', ' + patient16[:first_name]
patient16[:home_phone_raw] = patient16[:home_phone_formatted].delete '-'
patient16[:mobile_phone_raw] = patient16[:mobile_phone_formatted].delete '-'

describe "Clinical Staff Account Settings (CMS-2174)" do
  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "when you click on Account Settings for the patient" do
    it "Modify Veteran Account Settings" do
      # Set variables
      test_patient = patient16

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear

      # Search for and select a patient
      page = page.search_patient_and_select(test_patient[:last_name], test_patient[:display_name])

      # Verify the expected page messages
      expect(page.no_protocols_heading).to eq('Protocols Not Found')
      expect(page.no_assigned_protocols_message).to eq('This patient does not have any assigned protocols. Select "Assign New Protocol" to assign a protocol.')

      # # Navigate to the Account Settings page
      page.update_protocol_list_account_settings
      page = StaffVetProfileEditPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated patient data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.ssn).to eq(test_patient[:ssn])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])

      # Fill in the mobile & home phone and email fields
      page.home_phone = test_patient[:home_phone_raw]
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      page.mobile_phone = test_patient[:mobile_phone_raw]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      page.email = test_patient[:email]
      expect(page.email).to eq(test_patient[:email])

      # Verify the message preferences
      page.select_basic_cell
      expect(page.basic_cell_selected?).to eq(true)
      expect(page.smart_phone_selected?).to eq(false)
      page.select_smart_phone
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      page.check_receive_nonsecure_msgs
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(true)

      # Cancel the changes
      page.cancel
      page = PatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Verify that the expected page messages are displayed
      expect(page.no_protocols_heading).to eq('Protocols Not Found')
      expect(page.no_assigned_protocols_message).to eq('This patient does not have any assigned protocols. Select "Assign New Protocol" to assign a protocol.')

      # # Navigate to the Account Settings page
      page.account_settings
      page = StaffVetProfileEditPage.new(@browser)
      page.waitForLoadingToClear
      page.wait_for_timezone_to_load

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated patient data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.ssn).to eq(test_patient[:ssn])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])

      # Fill in the mobile & home phone and email fields
      page.home_phone = test_patient[:home_phone_raw]
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      page.mobile_phone = test_patient[:mobile_phone_raw]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      page.email = test_patient[:email]
      expect(page.email).to eq(test_patient[:email])

      # Verify the message preferences
      page.select_basic_cell
      expect(page.basic_cell_selected?).to eq(true)
      expect(page.smart_phone_selected?).to eq(false)
      page.select_smart_phone
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      page.uncheck_receive_nonsecure_msgs
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(false)

      # Select to receive facility messages
      page.check_facility_checkbox

      # Save the changes
      page.saveRegistration

      # Verify the expected error messages are received
      expect(page.facility_error).to eq('Please select a Facility')

      # Set the Facility field
      page.facility = test_patient[:facility]

      # Select to receive appointment reminders
      page.appointment_reminders_checkbox_checked?.should eq(false)
      page.check_appointment_reminders_checkbox
      page.appointment_reminders_checkbox_checked?.should eq(true)

      # Save the changes
      page.saveRegistration
      page.waitForLoadingToClear
      page = PatientProtocolListPage.new(@browser)
      (page.protocol_status_element.when_visible.text).should eq('Thank you. Account settings successfully saved.')
      sleep(5)

      # Verify the expected page messages are displayed
      expect(page.no_protocols_heading).to eq('Protocols Not Found')
      expect(page.no_assigned_protocols_message).to eq('This patient does not have any assigned protocols. Select "Assign New Protocol" to assign a protocol.')

      # Navigate to the Account Settings page
      page.account_settings
      page = StaffVetProfileEditPage.new(@browser)
      page.waitForLoadingToClear

      # Expand the Appointment/Info Messages, Preferences, and Patient Info sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated patient data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.ssn).to eq(test_patient[:ssn])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.email).to eq(test_patient[:email])
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(false)
      expect(page.facility).to eq(test_patient[:facility])
      expect(page.facility_checkbox_checked?).to eq(true)
      expect(page.appointment_reminders_checkbox_checked?).to eq(true)

      # Cancel the changes
      page.cancel
      page = PatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear

      # Verify the expected page messages are displayed
      expect(page.no_protocols_heading).to eq('Protocols Not Found')
      expect(page.no_assigned_protocols_message).to eq('This patient does not have any assigned protocols. Select "Assign New Protocol" to assign a protocol.')
    end
  end
end
