#########################################################################################
# Stories:   CMS-346, CMS-1934
#
# Tests:  CMS-595
#
# Acceptance Criteria:  CMS-520, CMS-1955
#
#
#########################################################################################

require 'spec_helper'

def verify_patient_data(page, patient)
  expect(page.patient_details_name).to eq(patient[:display_name])
  expect(page.patient_details_dob).to eq(patient[:dob])
  expect(page.patient_details_age).to eq(patient[:age])
  expect(page.patient_details_gender).to eq(patient[:gender])
  expect(page.patient_details_ssn).to eq(patient[:ssn])
end

# Set variables
patient1 = {
    :first_name => 'zztest',
    :last_name => 'patient01',
    :dob => '05/05/1900',
    :gender => 'Male',
    :ssn => '999-99-9901'
}
patient1[:display_name] = patient1[:last_name] + ', ' + patient1[:first_name]
patient1[:age] = calculate_age(patient1[:dob])

describe "Patient Details Information (CMS-520)" do
  wait = Selenium::WebDriver::Wait.new(:timeout => 30) # seconds

  context "when page should contain patient details" do
    it "confirm patient details displayed" do
      # Set variables
      test_patient = patient1

      # Start on the Patient Search page
      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad

      # Search for and select a patient
      page.patient_search_element.click
      page = page.search_patient_and_select(patient1[:last_name], patient1[:display_name])
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)

      # Navigate to the Admin page
      page = PatientSearchPage.new(@browser)
      page.goto_admin_page
      page = AdminPage.new(@browser)

      # Navigate to the Staff Search page
      page.GoToStaffSearch
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)

      # Navigate to the Protocols page and click the Add a Protocol button
      page = PatientSearchPage.new(@browser)
      page.protocols_tab_element.click
      page = UpdatePatientProtocolListPage.new(@browser)
      page.waitForLoadingToClear
      page.New_Protocol
      page.waitForLoadingToClear
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)

      # Assign a protocol
      page = UpdatePatientProtocolListPage.new(@browser)
      page.assignProtocolByName('Blood Pressure Protocol1')
      page.customizeprotocol_yes
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)

      # Navigate to the patient Messages page
      page = UpdatePatientProtocolListPage.new(@browser)
      page.message_history_element.click
      page.waitForLoadingToClear
      page = PatientMessageHistoryPage.new(@browser)
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)

      # Navigate to the patient Reports page
      page = PatientMessageHistoryPage.new(@browser)
      page.reports_element.click
      page.waitForLoadingToClear
      page = PatientDetailsPageType.new(@browser)

      # Verify the patient data
      verify_patient_data(page, test_patient)
    end
  end
end
