#########################################################################################
# Stories:   CMS-3850, CMS-3851
#
# Tests:    CMS-4269, , CMS4477
#
# Acceptance Criteria:  CMS-4148, CMS-4149, CMS-4150, CMS-4151, CMS-4152, CMS4458, CMS4459, CMS4460, CMS4461, CMS4462
#
#########################################################################################

require 'spec_helper'
require 'error_messages'

describe "CMS-3943 Validation of fields when adding or editing a Motivational/Educational template" do
  date_obj = DateTime.now
  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "As an Admin I can Validate The Schedule Settings Fields" do

    today_date = DateTime.now.strftime('%m/%d/%Y')

    it "Navigate to the Admin home page" do
      page = PatientSearchPage.new(@browser)

      page.goto_admin_page
      page = AdminPage.new(@browser)
    end

    it "Validate Motivational/Educational with no fields entered" do
      error_messages = Error_Messages.new
      @protocolName = 'Motivational Educational Validation Protocol'
      @focusArea = 'Motivational, Educational, Validation, Focus, Area'
      @description = 'This is a new protocol created for Motivational and Educational field Validation'

      protocol = Protocol.new
      protocol.focusArea = @focusArea
      protocol.description = @description
      protocol.newTemplateName = @protocolName

      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      page.create_motivational_educational_protocol(protocol)

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      motivational = page.custom_list[0]
      page.waitForLoadingToClear
      motivational.custom_schedule_settings_element.click
      motivational.custom_message_settings_element.click

      page.finish
      page.waitForLoadingToClear

      expect(motivational.custom_empty_error_summary).to eq("Error\nWe're sorry, there was an error with the protocol submission. This form cannot be empty. Please fill in the required fields.")
      expect(motivational.custom_motivational_description_error).to eq(error_messages.get_schedule_error_message(:show_template_desc_error => true))
      expect(motivational.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
    end

    it 'Validate Motivational/Educational Schedule Field' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      motivational = page.custom_list[0]
      motivational.custom_motivational_description='Educational'
      page.finish
      expect(motivational.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
      expect(motivational.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Custom'
      page.finish
      expect(motivational.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
      expect(motivational.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_time_between_each_cycle = '6'
      page.finish

      expect(motivational.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_time_between_each_cycle_units = 'Week'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Once'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Specified Days'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.check_custom_protocol_mon
      motivational.check_custom_protocol_wed
      motivational.check_custom_protocol_fri
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

     end

    it 'Validate Motivational/Educational Schedule Time Settings Fields' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)

      motivational = page.custom_list[0]
      @whattimehour = '9'
      @whattimemin = '15'
      @whattimeampm = 'PM'
      motivational.custom_start_time_hour = @whattimehour
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_start_time_minutes = @whattimemin
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_am_pm = @whattimeampm
      page.finish

      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Start/End Time Settings Fields' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)

      motivational = page.custom_list[0]

      motivational.custom_start_period = '2'
      page.finish

      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_end_period = '5'
      page.finish

      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_select_start_period = 'Hour'
      page.finish

      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_select_end_period = 'Month'
      page.finish

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

  end

  context "As an Admin I can Validate The Message Settings Fields" do

    it 'Validate Motivational/Educational Message Settings Fields' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      messages = ['this is my test',
                'this is my test 2345667890-',
                'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']


      motivational = page.custom_list[0]
      motivational.custom_motivational_service_message1 = messages[0]
      motivational.custom_add_message
      
      page.finish

      expect(motivational.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message2 = messages[1]
      motivational.custom_add_message
      page.finish

      expect(motivational.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message3 = messages[2]
      motivational.custom_motivational_service_message1 = ''
      motivational.custom_add_message

      page.finish

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message1 = messages[0]
      page.finish

      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message4 = '          '
      #page.finish
      #expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.custom_motivational_service_message4 = messages[3]
      page.finish
      page.modal_ok
    end
  end
  context "As an Admin I can Validate The Schedule Settings Fields While Editing a Template" do
    it 'Select newly created protocol and edit' do
      page = ListTemplatesPage.new(@browser)

      @list = page.protocol_list
      @index = page.get_index_by_text(@list, 'Motivational Educational Validation Protocol')
      protocol = page.protocol_list[@index].edit_view

    end

    it ' Validate Schedule Settings fields when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      motivational = page.custom_list[0]

      page.waitForLoadingToClear

      motivational.custom_template_name_element.click
      motivational.custom_schedule_settings_element.click

      motivational.uncheck_custom_protocol_mon
      motivational.uncheck_custom_protocol_wed
      motivational.uncheck_custom_protocol_fri
      motivational.custom_start_period = ''
      motivational.custom_end_period = ''

      page.save_bottom
      
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      motivational.check_custom_protocol_tues
      motivational.check_custom_protocol_sat
      page.save_bottom

      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational.custom_start_period = '8'
      page.save_bottom

      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational.custom_end_period = '9'

    end
  end
  context "As an Admin I can Validate The Message Settings Fields When Editing" do

    it 'Validate Motivational/Educational Message Settings Fields When Editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear


      messages = ['this is my test',
                  'this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']


      motivational = page.custom_list[0]
      motivational.custom_message_settings_element.click
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_motivational_service_message1 = messages[0]
      motivational.custom_add_message

      page.save_bottom

      expect(motivational.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message2 = messages[1]
      motivational.custom_add_message
      page.save_bottom

      expect(motivational.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message3 = messages[2]
      motivational.custom_motivational_service_message1 = ''
      motivational.custom_add_message

      page.save_bottom

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message1 = messages[0]
      page.save_bottom

      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message4 = '          '
      #page.save_bottom
      #expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.custom_motivational_service_message4 = messages[3]
      page.save_bottom
      page.modal_ok
    end
  end
  context "As an Admin I can Validate The Schedule Settings Fields in a Custom Protocol With Two Motivational/Educational Templates" do

    # ??? this is never used???
    today_date = DateTime.now.strftime('%m/%d/%Y')

    it "Create Custom Template With Two Motivational Templates" do
      error_messages = Error_Messages.new
      @templateName = 'Custom Motivational Protocol'
      @description = 'Custom Protocol to validate two motivational templates'
      @focusArea = 'Motivational, Custom, Field Validation'

      protocol = Protocol.new
      protocol.focusArea = @focusArea
      protocol.description = @description
      protocol.newTemplateName = @templateName

      page = ListTemplatesPage.new(@browser)

      page.create_custom_protocol(protocol)

      page= AddNewProtocolPage.new(@browser)
      page.check_motivational_educational_checkbox
      page.motivational_educational_quantity = '2'

      page.next

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      motivational = page.custom_list[0]
      motivational.custom_motivational_educational_element.click
      motivational.custom_schedule_settings_element.click
      motivational.custom_message_settings_element.click

      motivational2 = page.custom_list[1]
      motivational2.custom_motivational_educational_element.click
      motivational2.custom_schedule_settings_element.click
      motivational2.custom_message_settings_element.click

      page.finish

      expect(motivational.custom_empty_error_summary).to eq("Error\nWe're sorry, there was an error with the protocol submission. This form cannot be empty. Please fill in the required fields.")
      expect(motivational.custom_motivational_description_error).to eq(error_messages.get_schedule_error_message(:show_template_desc_error => true))
      expect(motivational.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      expect(motivational2.custom_empty_error_summary).to eq("Error\nWe're sorry, there was an error with the protocol submission. This form cannot be empty. Please fill in the required fields.")
      expect(motivational2.custom_motivational_description_error).to eq(error_messages.get_schedule_error_message(:show_template_desc_error => true))
      expect(motivational2.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
    end

    it 'Validate Motivational/Educational Schedule Field in the First Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      motivational = page.custom_list[0]
      motivational.custom_motivational_description='Educational'
      page.finish
      expect(motivational.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
      expect(motivational.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Custom'
      page.finish
      expect(motivational.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
      expect(motivational.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_time_between_each_cycle = '6'
      page.finish

      expect(motivational.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_time_between_each_cycle_units = 'Week'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Once'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_schedule='Specified Days'
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.check_custom_protocol_mon
      motivational.check_custom_protocol_wed
      motivational.check_custom_protocol_fri
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Schedule Time Settings Fields in the First Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)

      motivational = page.custom_list[0]
      @whattimehour = '9'
      @whattimemin = '15'
      @whattimeampm = 'PM'
      motivational.custom_start_time_hour = @whattimehour
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_start_time_minutes = @whattimemin
      page.finish

      expect(motivational.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_am_pm = @whattimeampm
      page.finish

      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Start/End Time Settings Fields in the First Template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)

      motivational = page.custom_list[0]

      motivational.custom_start_period = '2'
      page.finish

      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_end_period = '5'
      page.finish

      expect(motivational.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_select_start_period = 'Hour'
      page.finish

      expect(motivational.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_select_end_period = 'Month'
      page.finish

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Schedule Field in the Second Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      motivational2 = page.custom_list[1]
      motivational2.custom_motivational_description='Educational Two'
      page.finish
      expect(motivational2.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
      expect(motivational2.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_schedule='Custom'
      page.finish
      expect(motivational2.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
      expect(motivational2.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_time_between_each_cycle = '6'
      page.finish

      expect(motivational2.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_time_between_each_cycle_units = 'Week'
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_schedule='Once'
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_schedule='Specified Days'
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.check_custom_protocol_tues
      motivational2.check_custom_protocol_thurs
      motivational2.check_custom_protocol_sat
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Schedule Time Settings Fields in the Second Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)

      motivational2 = page.custom_list[1]
      @whattimehour = '6'
      @whattimemin = '52'
      @whattimeampm = 'AM'
      motivational2.custom_start_time_hour = @whattimehour
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_start_time_minutes = @whattimemin
      page.finish

      expect(motivational2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_am_pm = @whattimeampm
      page.finish

      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

    it 'Validate Motivational/Educational Start/End Time Settings Fields in the Second Template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)

      motivational2 = page.custom_list[1]

      motivational2.custom_start_period = '2'
      page.finish

      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_end_period = '5'
      page.finish

      expect(motivational2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_select_start_period = 'Hour'
      page.finish

      expect(motivational2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_select_end_period = 'Month'
      page.finish

      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

    end

  end

  context "As an Admin I can Validate The Message Settings Fields in a Custom Protocol With Two Motivational/Educational Templates" do

    it 'Validate Motivational/Educational Message Settings Fields in the First Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      messages = ['this is my test',
                  'this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']


      motivational = page.custom_list[0]
      motivational.custom_motivational_service_message1 = messages[0]
      motivational.custom_add_message

      page.finish

      expect(motivational.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message2 = messages[1]
      motivational.custom_add_message
      page.finish

      expect(motivational.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message3 = messages[2]
      motivational.custom_motivational_service_message1 = ''
      motivational.custom_add_message

      page.finish

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message1 = messages[0]
      page.finish

      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message4 = '          '
      # page.finish
      # expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.custom_motivational_service_message4 = messages[3]

    end
    it 'Validate Motivational/Educational Message Settings Fields in the Second Template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      messages = ['this is my test of the second template',
                  'this is my test 2345667890- of the second template',
                  'this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template',
                  'this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template']


      motivational2 = page.custom_list[1]
      motivational2.custom_motivational_service_message1 = messages[0]
      motivational2.custom_add_message

      page.finish

      expect(motivational2.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message2 = messages[1]
      motivational2.custom_add_message
      page.finish

      expect(motivational2.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message3 = messages[2]
      motivational2.custom_motivational_service_message1 = ''
      motivational2.custom_add_message

      page.finish

      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message1 = messages[0]
      page.finish

      expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message4 = '          '
      # page.finish
      # expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message4 = messages[3]
      page.finish
      page.modal_ok
    end
  end

  context "As an Admin I can Validate The Schedule Settings Fields in a Custom Protocol With Two Motivational/Educational Templates When Editing" do

    today_date = DateTime.now.strftime('%m/%d/%Y')

    it "Validate Motivational/Educational Schedule Field in the First Template When Editing" do
      error_messages = Error_Messages.new

      page = ListTemplatesPage.new(@browser)

      @list = page.protocol_list
      @index = page.get_index_by_text(@list, 'Custom Motivational Protocol')
      protocol = page.protocol_list[@index].edit_view

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      motivational = page.custom_list[0]
      motivational.custom_template_name_element.click
      motivational.custom_schedule_settings_element.click
      motivational.custom_message_settings_element.click

      motivational.uncheck_custom_protocol_mon
      motivational.uncheck_custom_protocol_wed
      motivational.uncheck_custom_protocol_fri
      motivational.custom_start_period = ''
      motivational.custom_end_period = ''

      page.save_bottom

      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      motivational.check_custom_protocol_tues
      motivational.check_custom_protocol_sat
      page.save_bottom

      expect(motivational.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational.custom_start_period = '8'
      page.save_bottom

      expect(motivational.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational.custom_end_period = '9'
    end

    it 'Validate Motivational/Educational Schedule Field in the Second Template When Editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      motivational2 = page.custom_list[1]

      page.waitForLoadingToClear
      motivational2.custom_template_name_element.click
      motivational2.custom_schedule_settings_element.click
      motivational2.custom_message_settings_element.click

      motivational2.uncheck_custom_protocol_tues
      motivational2.uncheck_custom_protocol_thurs
      motivational2.uncheck_custom_protocol_sat
      motivational2.custom_start_period = ''
      motivational2.custom_end_period = ''

      page.save_bottom

      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(motivational2.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      motivational2.check_custom_protocol_mon
      motivational2.check_custom_protocol_fri
      page.save_bottom

      expect(motivational2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational2.custom_start_period = '4'
      page.save_bottom

      expect(motivational2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      motivational2.custom_end_period = '1'

    end

  end

  context "As an Admin I can Validate The Message Settings Fields in a Custom Protocol With Two Motivational/Educational Templates When Editing" do

    it 'Validate Motivational/Educational Message Settings Fields in the First Template When Editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      messages = ['this is my test',
                  'this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                  'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']


      motivational = page.custom_list[0]
      motivational.custom_message_settings_element.click
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_remove_message
      page.remove_message_yes
      motivational.custom_motivational_service_message1 = messages[0]
      motivational.custom_add_message

      page.save_bottom

      expect(motivational.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message2 = messages[1]
      motivational.custom_add_message
      page.save_bottom

      expect(motivational.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message3 = messages[2]
      motivational.custom_motivational_service_message1 = ''
      motivational.custom_add_message

      page.save_bottom

      expect(motivational.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message1 = messages[0]
      page.save_bottom

      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational.custom_motivational_service_message4 = '          '
      page.save_bottom
      expect(motivational.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))


      motivational.custom_motivational_service_message4 = messages[3]

    end
    it 'Validate Motivational/Educational Message Settings Fields in the Second Template When Editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      messages = ['this is my test of the second template',
                  'this is my test 2345667890- of the second template',
                  'this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template',
                  'this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template  this is my test 2345667890- of the second template']


      motivational2 = page.custom_list[1]
      motivational2.custom_message_settings_element.click
      motivational2.custom_remove_message
      page.remove_message_yes
      motivational2.custom_remove_message
      page.remove_message_yes
      motivational2.custom_remove_message
      page.remove_message_yes
      motivational2.custom_motivational_service_message1 = messages[0]
      motivational2.custom_add_message

      page.save_bottom

      expect(motivational2.custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message2 = messages[1]
      motivational2.custom_add_message
      page.save_bottom

      expect(motivational2.custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message3 = messages[2]
      motivational2.custom_motivational_service_message1 = ''
      motivational2.custom_add_message

      page.save_bottom

      expect(motivational2.custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
      expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message1 = messages[0]
      page.save_bottom

      expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message4 = '          '
      page.save_bottom
      expect(motivational2.custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

      motivational2.custom_motivational_service_message4 = messages[3]
      page.save_bottom
      page.modal_ok

      page = ListTemplatesPage.new(@browser)
      page.GoToStaffSearch
    end

  end

  context "As a staff user verify that the validation error messages are correct" do

    it 'assign an existing protocol with a single motivational template to a patient' do
      is_protocol_assigned = false
      is_protocol_edited = false
      patient_search_string = 'P9907'
      patient_result_name = 'patient07, zztest'
      protocol_to_assign = 'Motivational Educational Validation Protocol'
      invalid_date_strings = ['bananaphone', '13/1/2014', '1/32/2014', '*', '##/##/####', 'MM/DD/YYYY']
      page = PatientSearchPage.new(@browser)

      # Set up the template models
      motivational_template = TemplateModel.new(
          :template_description => 'Educational',
          :schedule => TemplateSchedule.new(
              :schedule => 'Once',
              :service_message_start_time => TimeOfDay.new('8', '30', 'AM'),
              :accept_unscheduled_readings => false,
              :start_date => (date_obj + 1).strftime('%m/%d/%Y'),
              :end_date => (date_obj + 450).strftime('%m/%d/%Y')
          )
      )

      page.patient_search_element.click
      page.waitForPageToLoad
      page = page.search_patient_and_select(patient_search_string, patient_result_name)

      page = UpdatePatientProtocolListPage.new(@browser)
      while !(is_protocol_assigned && is_protocol_edited) do
        if !is_protocol_assigned then
          puts("Assigning Protocol")
          page.New_Protocol
          page.waitForLoadingToClear

          # Assign the protocol and elect to customize it
          page.assignProtocolByName(protocol_to_assign)
          page.customizeprotocol_yes
          page.waitForLoadingToClear
          is_protocol_assigned = true
        elsif !is_protocol_edited then
          puts("Editing Protocol")
          page = UpdatePatientProtocolListPage.new(@browser)
          @list = page.patient_protocol_list
          @index = page.get_index_by_text(@list, protocol_to_assign)
          protocol = page.patient_protocol_list[@index]
          protocol.edit_temp
          is_protocol_edited = true
        end

        page = UpdatePatientProtocolEditPage.new(@browser)

        # Expand the "Vitals: Blood Pressure" template and the Schedule Settings accordion
        page.custom_list[0].custom_template_name_element.click

        page.custom_list[0].custom_schedule_settings_element.click


        error_messages = Error_Messages.new

        #########################################################################
          ## Verify error messages for schedule settings
        #########################################################################

        messages = ['this is my test',
                    'this is my test for motivational template',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']

        # Empty field
        expect(page.custom_list[0].custom_description).to eq(motivational_template.template_description)
        page.custom_list[0].custom_protocol_startdate = ""
        page.custom_list[0].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq("Error\nPlease correct errors on the form below.")
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
        expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))

        # Calendar field negative testing
        for invalid_date in invalid_date_strings do
          page.custom_list[0].custom_protocol_startdate = invalid_date
          page.custom_list[0].custom_protocol_enddate = invalid_date
          page.save_button_bottom
          page.waitForLoadingToClear
          expect(page.custom_list[0].error_summary).to eq("Error\nPlease correct errors on the form below.")
          expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
          expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))
        end

        page.custom_list[0].custom_protocol_startdate = '09/03/2014'
        page.custom_list[0].custom_protocol_enddate = '09/02/2014'
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_comparison_field => true))
        end_date_error_message = error_messages.get_schedule_error_message(:end_date_past_field => true) + "\n" + 
                                  error_messages.get_schedule_error_message(:schedule_end_date_comparison_field => true)
        expect(page.custom_list[0].error_template_enddate_one).to eq(end_date_error_message)

        expect(page.custom_list[0].custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_template_error => true))

        # Restore the start date value
        page.custom_list[0].custom_protocol_startdate = motivational_template.schedule.start_date
        page.custom_list[0].custom_protocol_enddate = motivational_template.schedule.end_date

        #########################################################################
          ## Verify error messages for message settings
        #########################################################################

        page.custom_list[0].custom_message_settings_element.click
        page.custom_list[0].custom_remove_message
        page.remove_message_yes
        page.custom_list[0].custom_remove_message
        page.remove_message_yes
        page.custom_list[0].custom_remove_message
        page.remove_message_yes

        page.custom_list[0].custom_motivational_service_message1 = messages[0]
        page.custom_list[0].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message2 = messages[1]
        page.custom_list[0].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message3 = messages[2]
        page.custom_list[0].custom_motivational_service_message1 = ''
        page.custom_list[0].custom_add_message

        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message1 = messages[0]
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message4 = '          '
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message4 = messages[3]
        page.save_button_bottom
        page.waitForLoadingToClear
        page.modalwindow_ok
        page.waitForLoadingToClear
      end
    end

  end

  context "As a staff user verify that the validation error messages are correct in a custom protocol" do
    it 'assign an existing custom protocol with two motivational templates to a patient (first template)' do
      is_protocol_assigned = false
      is_protocol_edited = false
      protocol_to_assign = 'Custom Motivational Protocol'
      invalid_date_strings = ['bananaphone', '13/1/2014', '1/32/2014', '*', '##/##/####', 'MM/DD/YYYY']

      # Set up the template models
      motivational_template1 = TemplateModel.new(
          :template_description => 'Educational',
          :schedule => TemplateSchedule.new(
              :schedule => 'Once',
              :service_message_start_time => TimeOfDay.new('8', '30', 'AM'),
              :accept_unscheduled_readings => false,
              :start_date => (date_obj + 1).strftime('%m/%d/%Y'),
              :end_date => (date_obj + 450).strftime('%m/%d/%Y')
          )
      )
      motivational_template2 = TemplateModel.new(
          :template_description => 'Educational Two',
          :schedule => TemplateSchedule.new(
              :schedule => 'Once',
              :service_message_start_time => TimeOfDay.new('9', '47', 'PM'),
              :accept_unscheduled_readings => false,
              :start_date => (date_obj + 1).strftime('%m/%d/%Y'),
              :end_date => (date_obj + 450).strftime('%m/%d/%Y')
          )
      )
      page = UpdatePatientProtocolListPage.new(@browser)
      while !(is_protocol_assigned && is_protocol_edited) do
        if !is_protocol_assigned then
          puts("Assigning Protocols")
          page.New_Protocol
          page.waitForLoadingToClear

          # Assign the protocol and elect to customize it
          page.assignProtocolByName(protocol_to_assign)
          page.customizeprotocol_yes
          page.waitForLoadingToClear
          is_protocol_assigned = true
        elsif !is_protocol_edited then
          puts("Editing Protocols")
          page = UpdatePatientProtocolListPage.new(@browser)
          @list = page.patient_protocol_list
          @index = page.get_index_by_text(@list, protocol_to_assign)
          protocol = page.patient_protocol_list[@index]
          protocol.edit_temp
          is_protocol_edited = true
        end

        page = UpdatePatientProtocolEditPage.new(@browser)
        # Expand the "Vitals: Blood Pressure" template and the Schedule Settings accordion
        page.custom_list[0].custom_template_name_element.click

        page.custom_list[0].custom_schedule_settings_element.click


        error_messages = Error_Messages.new

        #########################################################################
        ## Verify error messages for schedule settings
        #########################################################################

        messages = ['this is my test',
                    'this is my test for motivational template',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']

        # Empty field
        expect(page.custom_list[0].custom_description).to eq(motivational_template1.template_description)
        page.custom_list[0].custom_protocol_startdate = ""
        page.custom_list[0].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq("Error\nPlease correct errors on the form below.")
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
        expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))

        # Calendar field negative testing
        for invalid_date in invalid_date_strings do
          page.custom_list[0].custom_protocol_startdate = invalid_date
          page.custom_list[0].custom_protocol_enddate = invalid_date
          page.save_button_bottom
          page.waitForLoadingToClear
          expect(page.custom_list[0].error_summary).to eq("Error\nPlease correct errors on the form below.")
          expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
          expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))
        end

        page.custom_list[0].custom_protocol_startdate = '09/03/2014'
        page.custom_list[0].custom_protocol_enddate = '09/02/2014'
        page.save_button_bottom
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_schedule_error_message(:schedule_start_date_comparison_field => true))
        end_date_error_message = error_messages.get_schedule_error_message(:end_date_past_field => true) + "\n" + 
                                  error_messages.get_schedule_error_message(:schedule_end_date_comparison_field => true)
        expect(page.custom_list[0].error_template_enddate_one).to eq(end_date_error_message)
        expect(page.custom_list[0].custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_template_error => true))

        # Restore the start date value
        page.custom_list[0].custom_protocol_startdate = motivational_template1.schedule.start_date
        page.custom_list[0].custom_protocol_enddate = motivational_template1.schedule.end_date

        #########################################################################
        ## Verify error messages for message settings
        #########################################################################

        page.custom_list[0].custom_message_settings_element.click
        page.custom_list[0].custom_remove_message
        page.remove_message_yes
        page.custom_list[0].custom_remove_message
        page.remove_message_yes
        page.custom_list[0].custom_remove_message
        page.remove_message_yes

        page.custom_list[0].custom_motivational_service_message1 = messages[0]
        page.custom_list[0].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message2 = messages[1]
        page.custom_list[0].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message3 = messages[2]
        page.custom_list[0].custom_motivational_service_message1 = ''
        page.custom_list[0].custom_add_message

        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message1 = messages[0]
        page.save_button_bottom

        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message4 = '          '
        page.save_button_bottom
        expect(page.custom_list[0].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[0].custom_motivational_service_message4 = messages[3]

        # Expand the "Vitals: Blood Pressure" template and the Schedule Settings accordion
        page.custom_list[1].custom_schedule_settings_element.click

        error_messages = Error_Messages.new

        #########################################################################
        ## Verify error messages for schedule settings
        #########################################################################

        messages = ['this is my test',
                    'this is my test for motivational template',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-',
                    'this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890- this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-  this is my test 2345667890-']

        # Empty field
        expect(page.custom_list[1].custom_description).to eq(motivational_template2.template_description)
        page.custom_list[1].custom_protocol_startdate = ""
        page.custom_list[1].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq("Error\nPlease correct errors on the form below.")
        expect(page.custom_list[1].error_template_startdate_two).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
        expect(page.custom_list[1].error_template_enddate_two).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))

        # Calendar field negative testing
        for invalid_date in invalid_date_strings do
          page.custom_list[1].custom_protocol_startdate = invalid_date
          page.custom_list[1].custom_protocol_enddate = invalid_date
          page.save_button_bottom
          page.waitForLoadingToClear
          expect(page.custom_list[1].error_summary).to eq("Error\nPlease correct errors on the form below.")
          expect(page.custom_list[1].error_template_startdate_two).to eq(error_messages.get_schedule_error_message(:schedule_start_date_empty_field => true))
          expect(page.custom_list[1].error_template_enddate_two).to eq(error_messages.get_schedule_error_message(:schedule_end_date_empty_field => true))
        end

        page.custom_list[1].custom_protocol_startdate = '09/03/2014'
        page.custom_list[1].custom_protocol_enddate = '09/02/2014'
        page.save_button_bottom
        expect(page.custom_list[1].error_template_startdate_two).to eq(error_messages.get_schedule_error_message(:schedule_start_date_comparison_field => true))
        expect(page.custom_list[1].error_template_enddate_two).to eq(end_date_error_message)
       # expect(page.custom_list[1].custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_template_error => true))

        # Restore the start date value
        page.custom_list[1].custom_protocol_startdate = motivational_template2.schedule.start_date
        page.custom_list[1].custom_protocol_enddate = motivational_template2.schedule.end_date

        #########################################################################
        ## Verify error messages for message settings
        #########################################################################

       page.custom_list[1].custom_message_settings_element.click
        page.custom_list[1].custom_remove_message
        page.remove_message_yes
        page.custom_list[1].custom_remove_message
        page.remove_message_yes
        page.custom_list[1].custom_remove_message
        page.remove_message_yes

        page.custom_list[1].custom_motivational_service_message1 = messages[0]
        page.custom_list[1].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[1].custom_motivational_service_message2_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[1].custom_motivational_service_message2 = messages[1]
        page.custom_list[1].custom_add_message
        page.save_button_bottom

        expect(page.custom_list[1].custom_motivational_service_message3_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[1].custom_motivational_service_message3 = messages[2]
        page.custom_list[1].custom_motivational_service_message1 = ''
        page.custom_list[1].custom_add_message

        page.save_button_bottom

        expect(page.custom_list[1].custom_motivational_service_message1_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))
        expect(page.custom_list[1].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[1].custom_motivational_service_message1 = messages[0]
        page.save_button_bottom

        expect(page.custom_list[1].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[1].custom_motivational_service_message4 = '          '
        page.save_button_bottom
        page.waitForLoadingToClear

        expect(page.custom_list[1].custom_motivational_service_message4_error).to eq(error_messages.get_message_error_message(:show_motivational_messages_error => true))

        page.custom_list[1].custom_motivational_service_message4 = messages[3]
        page.save_button_bottom
        page.waitForLoadingToClear
        page.modalwindow_ok
        page.waitForLoadingToClear
      end
    end
  end
end
