#########################################################################################
# Stories: CMS-4160
#
# Tests:
#
# Acceptance Criteria:
#
#########################################################################################

require 'spec_helper'
require 'clinical_staff_error_messages'
require 'data_models/template_model'
require 'data_models/template_schedule'
require 'data_models/template_reading'
require 'data_models/template_schedule'
require 'data_models/utility_models'


describe "Validation of fields when assigning and customizing multiple vitals templates" do
  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "Verify that the validation error messages are correct as a staff user" do
    today_date = DateTime.now.strftime('%m/%d/%Y')

    it 'Assign an existing protocol with multiple templates to a patient' do
      ## NOTE: This test must be executed after the CMS2010_Add_Edit_Vitals_Verification_spec.rb test

      # Set up test variables
      error_messages = ClinicalStaffErrorMessages.new
      patient_search_string = 'P9918'
      patient_result_name = 'patient18, zztest'
      protocol_to_assign = 'Custom Validation Protocol'
      is_protocol_assigned = false
      is_protocol_edited = false
      page = PatientSearchPage.new(@browser)

      # Set up the template models
      pulse_oximetry_template = TemplateModel.new(
          :measurement => 'Pulse Oximetry (SpO2)',
          :keywords => 'OX, OXYGEN',
          :time_between_reminders => ValueUnits.new(4, 'Minute'),
          :reminder_message_count => 8,
          :schedule => TemplateSchedule.new(
              :schedule => 'Specified Days',
              :service_message_start_time => TimeOfDay.new('9', '15', 'PM'),
              :schedule_days => DaysOfTheWeek.new(
                  :tuesday => true,
                  :saturday => true
              ),
              :accept_unscheduled_readings => true,
              :start_date => '09/12/2014',
              :end_date => '06/12/2015'
          ),
          :readings => TemplateReading.new(
              :valid_range => RangeLimit.new(60, 100),
              :critical_range => RangeLimit.new(65, 95),
              :alert_range => RangeLimit.new(70, 90)
          ),
          :messages => TemplateMessages.new(
              :service_message => 'Here is the EDITED service message for this protocol',
              :reminder_message => 'Edited Reminder Message',
              :valid_reading_message => 'Valid Blood Pressure Protocol Readings',
              :critical_low_message => 'Critical Low Level Alert Message',
              :critical_high_message => 'Critical High Level Alert Message',
              :alert_low_message => 'Low Level Alert Message',
              :alert_high_message => 'High Level Alert Message'
          )
      )
      blood_glucose_template = TemplateModel.new(
          :measurement => 'Blood Glucose',
          :keywords => 'BG, GLUCOSE, SUGAR',
          :time_between_reminders => ValueUnits.new(2, 'Minute'),
          :reminder_message_count => 22,
          :schedule => TemplateSchedule.new(
              :schedule => 'Specified Days',
              :service_message_start_time => TimeOfDay.new('6', '45', 'AM'),
              :schedule_days => DaysOfTheWeek.new(
                  :monday => true,
                  :friday => true
              ),
              :accept_unscheduled_readings => true,
              :start_date => '09/12/2014',
              :end_date => '09/12/2023'
          ),
          :readings => TemplateReading.new(
              :valid_range => RangeLimit.new(30, 300),
              :alert_range => RangeLimit.new(40, 200)
          ),
          :messages => TemplateMessages.new(
              :service_message => 'Here is the EDITED service message for this protocol',
              :reminder_message => 'Edited Reminder Message',
              :valid_reading_message => 'Valid Blood Pressure Protocol Readings',
              :alert_low_message => 'Low Level Alert Message',
              :alert_high_message => 'High Level Alert Message'
          )
      )

      #########################################################################
      ## Get to customization page with the protocol loaded and expanded
      #########################################################################

      # Search for the desired patient
      page.patient_search_element.click
      page.waitForPageToLoad
      page = page.search_patient_and_select(patient_search_string, patient_result_name)

      while !(is_protocol_assigned && is_protocol_edited)
        page = UpdatePatientProtocolListPage.new(@browser)

        if !is_protocol_assigned then
          puts "Assigning and customizing a new protocol"

          page.New_Protocol
          page.waitForLoadingToClear

          # Assign the protocol and elect to customize it
          page.assignProtocolByName(protocol_to_assign)
          page.customizeprotocol_yes

          is_protocol_assigned = true
        elsif !is_protocol_edited then
          puts "Editing a previously-assigned protocol"

          # Assign the protocol and elect to customize it
          @list = page.patient_protocol_list
          @index = page.get_index_by_text(@list, protocol_to_assign)
          protocol = page.patient_protocol_list[@index]
          protocol.edit_temp

          is_protocol_edited = true
        end

        page.waitForLoadingToClear
        page = UpdatePatientProtocolEditPage.new(@browser)
        page.waitForLoadingToClear
        #########################################################################
        ## Expand the first template and the Schedule Settings accordion
        #########################################################################
        page.custom_list[0].custom_template_name_element.click
        page.custom_list[0].custom_schedule_settings_element.click

        #########################################################################
        ## Verify the first template start date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_startdate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_error_messages(
                                                                       :schedule_start_date_empty_field => true
                                                                   ))

        # Restore the start date value
        page.custom_list[0].custom_protocol_startdate = pulse_oximetry_template.schedule.start_date

        #########################################################################
        ## Verify the first template end date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_error_messages(
                                                                     :schedule_end_date_empty_field => true
                                                                 ))

        # Restore the end date value
        page.custom_list[0].custom_protocol_enddate = pulse_oximetry_template.schedule.end_date

        #########################################################################
        ## Verify the first template first lowest valid reading error messages
        #########################################################################

        # Expand the Readings Settings accordion
        page.custom_list[0].custom_reading_settings_element.click

        # Empty field
        page.custom_list[0].custom_lowest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_empty_field => true
                                                                        ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).critical_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true
                                                                        ))

        # Violate alert lower limit
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true
                                                                        ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_highest_valid_violation => true,
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Restore lowest valid value
        page.custom_list[0].custom_lowest_validreading_one = pulse_oximetry_template.get_reading(0).valid_range.lowest

        #########################################################################
        ## Verify the first template first highest valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_empty_field => true
                                                                         ))

        # Violate critical upper limit
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).critical_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true
                                                                         ))

        # Violate alert upper limit
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate alert lower limit
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate critical lower limit
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate valid lower limit
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_lowest_valid_violation => true,
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Restore highest valid value
        page.custom_list[0].custom_highest_validreading_one = pulse_oximetry_template.get_reading(0).valid_range.highest

        #########################################################################
        ## Verify the first template first message for valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :valid_reading_empty_message_field => true
                                                                         ))

        # Restore lowest valid value
        page.custom_list[0].custom_message_validreading_one = pulse_oximetry_template.messages.valid_reading_message

        #########################################################################
        ## Verify the first template first critical low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_lowlevel_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_empty_field => true
                                                                      ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_lowest_valid_violation => true
                                                                      ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true
                                                                      ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate critical upper limit
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_highest_valid_violation => true,
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Restore highest valid value
        page.custom_list[0].custom_critical_lowlevel_one = pulse_oximetry_template.get_reading(0).critical_range.lowest

        #########################################################################
        ## Verify the first template first critical high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_highlevel_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_empty_field => true
                                                                       ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_highest_valid_violation => true
                                                                       ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate critical lower limit
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_lowest_valid_violation => true,
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Restore critical high value
        page.custom_list[0].custom_critical_highlevel_one = pulse_oximetry_template.get_reading(0).critical_range.highest

        #########################################################################
        ## Verify the first template first message for critical low level alert error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_criticallowalert = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_criticallowalert).to eq(error_messages.get_error_messages(
                                                                             :critical_low_reading_empty_message_field => true
                                                                         ))

        # Restore the critical low level message value
        page.custom_list[0].custom_message_criticallowalert = pulse_oximetry_template.messages.critical_low_message

        #########################################################################
        ## Verify the first template first message for critical high level alert error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_criticalhighalert = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_criticalhighalert).to eq(error_messages.get_error_messages(
                                                                              :critical_high_reading_empty_message_field => true
                                                                          ))

        # Restore the critical high level message value
        page.custom_list[0].custom_message_criticalhighalert = pulse_oximetry_template.messages.critical_high_message

        #########################################################################
        ## Verify the first template first alert low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_lowlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_empty_field => true
                                                                   ))

        # Violate valid lower limit
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_lowest_valid_violation => true,
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_highest_valid_violation => true,
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Restore the alert lowest value
        page.custom_list[0].custom_lowlevel_alert_one = pulse_oximetry_template.get_reading(0).alert_range.lowest

        #########################################################################
        ## Verify the first template first alert high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_empty_field => true
                                                                    ))

        # Violate valid upper limit
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_highest_valid_violation => true,
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate critical upper limit
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate alert lower limit
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Violate critical lower limit
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true,
                                                                        :alert_high_reading_critical_low_violation => true
                                                                    ))

        # Violate valid lower limit
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true,
                                                                        :alert_high_reading_lowest_valid_violation => true,
                                                                        :alert_high_reading_critical_low_violation => true
                                                                    ))

        # Restore the alert highest value
        page.custom_list[0].custom_highlevel_alert_one = pulse_oximetry_template.get_reading(0).alert_range.highest

        #########################################################################
        ## Verify the first template first message for alert low level error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_below_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_below_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_low_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[0].custom_below_desirable_level_msgone = pulse_oximetry_template.messages.alert_low_message

        #########################################################################
        ## Verify the first template first message for alert high level error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_above_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_above_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_high_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[0].custom_above_desirable_level_msgone = pulse_oximetry_template.messages.alert_high_message

        #########################################################################
        ## Verify the first template service message error messages
        #########################################################################

        # Expand the Message Settings accordion
        page.custom_list[0].custom_message_settings_element.click

        # Empty field
        page.custom_list[0].custom_protocol_servicemessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_servicemessage).to eq(error_messages.get_error_messages(
                                                                            :service_message_empty_field => true
                                                                        ))

        # Restore the service message value
        page.custom_list[0].custom_protocol_servicemessage = pulse_oximetry_template.messages.service_message

        #########################################################################
        ## Verify the first template reminder message error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_remindermessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_remindermessage).to eq(error_messages.get_error_messages(
                                                                             :reminder_message_empty_field => true
                                                                         ))

        # Restore the reminder message value
        page.custom_list[0].custom_protocol_remindermessage = pulse_oximetry_template.messages.reminder_message

        #########################################################################
        ## Verify the first template amount of time between reminders error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_timebtwn = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_timebtwn).to eq(error_messages.get_error_messages(
                                                                      :time_between_reminders_empty_field => true
                                                                  ))

        # Restore the time between reminders value
        page.custom_list[0].custom_protocol_timebtwn = pulse_oximetry_template.time_between_reminders.value

        #########################################################################
        ## Verify the first template reminder message count error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_remindertimesent = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_remindertimesent).to eq(error_messages.get_error_messages(
                                                                              :reminder_message_count_empty_field => true
                                                                          ))

        # Restore the time between reminders value
        page.custom_list[0].custom_protocol_remindertimesent = pulse_oximetry_template.reminder_message_count

        #########################################################################
        ## Expand the second template and the Schedule Settings accordion
        #########################################################################
        page.custom_list[1].custom_template_name
        page.custom_list[1].custom_template_name
        page.custom_list[1].custom_schedule_settings_element.click

        #########################################################################
        ## Verify the second template start date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[1].custom_protocol_startdate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_template_startdate_two).to eq(error_messages.get_error_messages(
                                                                       :schedule_start_date_empty_field => true
                                                                   ))

        # Restore the start date value
        page.custom_list[1].custom_protocol_startdate = blood_glucose_template.schedule.start_date

        #########################################################################
        ## Verify the second template end date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[1].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_template_enddate_two).to eq(error_messages.get_error_messages(
                                                                     :schedule_end_date_empty_field => true
                                                                 ))

        # Restore the end date value
        page.custom_list[1].custom_protocol_enddate = blood_glucose_template.schedule.end_date

        #########################################################################
        ## Verify the second template first lowest valid reading error messages
        #########################################################################

        # Expand the Readings Settings accordion
        page.custom_list[1].custom_reading_settings_element.click

        # Empty field
        page.custom_list[1].custom_lowest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_empty_field => true
                                                                        ))

        # Violate alert lower limit
        page.custom_list[1].custom_lowest_validreading_one = blood_glucose_template.get_reading(0).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_alert_low_violation => true
                                                                        ))

        # Violate alert upper limit
        page.custom_list[1].custom_lowest_validreading_one = blood_glucose_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate valid upper limit
        page.custom_list[1].custom_lowest_validreading_one = blood_glucose_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_highest_valid_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Restore lowest valid value
        page.custom_list[1].custom_lowest_validreading_one = blood_glucose_template.get_reading(0).valid_range.lowest

        #########################################################################
        ## Verify the second template first highest valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_highest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_empty_field => true
                                                                         ))

        # Violate alert upper limit
        page.custom_list[1].custom_highest_validreading_one = blood_glucose_template.get_reading(0).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate alert lower limit
        page.custom_list[1].custom_highest_validreading_one = blood_glucose_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate valid lower limit
        page.custom_list[1].custom_highest_validreading_one = blood_glucose_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_lowest_valid_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Restore highest valid value
        page.custom_list[1].custom_highest_validreading_one = blood_glucose_template.get_reading(0).valid_range.highest

        #########################################################################
        ## Verify the second template first message for valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_message_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_message_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :valid_reading_empty_message_field => true
                                                                         ))

        # Restore lowest valid value
        page.custom_list[1].custom_message_validreading_one = blood_glucose_template.messages.valid_reading_message

        # Empty field
        page.custom_list[1].custom_lowlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_empty_field => true
                                                                   ))

        # Violate valid lower limit
        page.custom_list[1].custom_lowlevel_alert_one = blood_glucose_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_lowest_valid_violation => true
                                                                   ))

        # Violate alert upper limit
        page.custom_list[1].custom_lowlevel_alert_one = blood_glucose_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate valid upper limit
        page.custom_list[1].custom_lowlevel_alert_one = blood_glucose_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_highest_valid_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Restore the alert lowest value
        page.custom_list[1].custom_lowlevel_alert_one = blood_glucose_template.get_reading(0).alert_range.lowest

        #########################################################################
        ## Verify the second template first alert high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_highlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_empty_field => true
                                                                    ))

        # Violate valid upper limit
        page.custom_list[1].custom_highlevel_alert_one = blood_glucose_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_highest_valid_violation => true
                                                                    ))

        # Violate alert lower limit
        page.custom_list[1].custom_highlevel_alert_one = blood_glucose_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Violate valid lower limit
        page.custom_list[1].custom_highlevel_alert_one = blood_glucose_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true,
                                                                        :alert_high_reading_lowest_valid_violation => true
                                                                    ))

        # Restore the alert highest value
        page.custom_list[1].custom_highlevel_alert_one = blood_glucose_template.get_reading(0).alert_range.highest

        #########################################################################
        ## Verify the second template first message for alert low level error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_below_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_below_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_low_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[1].custom_below_desirable_level_msgone = blood_glucose_template.messages.alert_low_message

        #########################################################################
        ## Verify the second template first message for alert high level error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_above_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_above_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_high_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[1].custom_above_desirable_level_msgone = blood_glucose_template.messages.alert_high_message

        #########################################################################
        ## Verify the second template service message error messages
        #########################################################################

        # Expand the Message Settings accordion
        page.custom_list[1].custom_message_settings_element.click

        # Empty field
        page.custom_list[1].custom_protocol_servicemessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_protocol_servicemessage).to eq(error_messages.get_error_messages(
                                                                            :service_message_empty_field => true
                                                                        ))

        # Restore the service message value
        page.custom_list[1].custom_protocol_servicemessage = blood_glucose_template.messages.service_message

        #########################################################################
        ## Verify the second template reminder message error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_protocol_remindermessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_protocol_remindermessage).to eq(error_messages.get_error_messages(
                                                                             :reminder_message_empty_field => true
                                                                         ))

        # Restore the reminder message value
        page.custom_list[1].custom_protocol_remindermessage = blood_glucose_template.messages.reminder_message

        #########################################################################
        ## Verify the second template amount of time between reminders error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_protocol_timebtwn = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_protocol_timebtwn).to eq(error_messages.get_error_messages(
                                                                      :time_between_reminders_empty_field => true
                                                                  ))

        # Restore the time between reminders value
        page.custom_list[1].custom_protocol_timebtwn = blood_glucose_template.time_between_reminders.value

        #########################################################################
        ## Verify the second template reminder message count error messages
        #########################################################################

        # Empty field
        page.custom_list[1].custom_protocol_remindertimesent = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[1].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[1].error_protocol_remindertimesent).to eq(error_messages.get_error_messages(
                                                                              :reminder_message_count_empty_field => true
                                                                          ))

        # Restore the time between reminders value
        page.custom_list[1].custom_protocol_remindertimesent = blood_glucose_template.reminder_message_count
        page.save_button_bottom
        page.waitForLoadingToClear
        page.modalwindow_ok
        page.waitForLoadingToClear
      end
    end
  end
end