#########################################################################################
# Stories: CMS-4160, CMS-4177, CMS-4194, CMS-477, CMS-4211, CMS-4228
#
# Tests: CMS-4424, CMS-4422, CMS-4420, CMS-4475, CMS-4473, CMS-4471
#
# Acceptance Criteria: CMS-4409, CMS-4411, CMS-4245, CMS-4412, CMS-4414, CMS-4415, CMS-4416,
#                      CMS-4417, CMS-4449, CMS-4450, CMS-4451, CMS-4452, CMS-4453, CMS-4454,
#                      CMS-4455, CMS-4456, CMS-4457, CMS-4458, CMS-4459, CMS-4460, CMS-4461,
#                      CMS-4462
#
#########################################################################################

require 'spec_helper'
require 'clinical_staff_error_messages'
require 'data_models/template_model'
require 'data_models/template_schedule'
require 'data_models/template_reading'
require 'data_models/template_schedule'
require 'data_models/utility_models'


describe "Validation of fields when assigning and customizing a single vitals template" do
  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "Verify that the validation error messages are correct as a staff user" do
    today_date = DateTime.now.strftime('%m/%d/%Y')

    it 'Assign an existing protocol with a single template to a patient and then edit it' do
      # Set up test variables
      error_messages = ClinicalStaffErrorMessages.new
      patient_search_string = 'P9918'
      patient_result_name = 'patient18, zztest'
      protocol_to_assign = 'Blood Pressure Protocol1'
      is_protocol_assigned = false
      is_protocol_edited = false
      page = PatientSearchPage.new(@browser)

      # Set up the template models
      blood_pressure_template = TemplateModel.new(
          :measurement => 'Blood Pressure',
          :keywords => 'BP, BLOOD',
          :time_between_reminders => ValueUnits.new(240, 'Minute'),
          :reminder_message_count => 2,
          :schedule => TemplateSchedule.new(
              :schedule => 'Once',
              :service_message_start_time => TimeOfDay.new('8', '30', 'AM'),
              :accept_unscheduled_readings => false,
              :start_date => '09/03/2014',
              :end_date => '07/03/2015'
          ),
          :readings => [
              TemplateReading.new(
                  :valid_range => RangeLimit.new(70, 262),
                  :critical_range => RangeLimit.new(75, 230),
                  :alert_range => RangeLimit.new(80, 140)
              ),
              TemplateReading.new(
                  :valid_range => RangeLimit.new(50, 120),
                  :critical_range => RangeLimit.new(55, 110),
                  :alert_range => RangeLimit.new(60, 90)
              )
          ],
          :messages => TemplateMessages.new(
              :service_message => 'Hello, just a quick reminder that your readings are due. Thanks.',
              :reminder_message => 'Hi again, I have noticed that you have not send in your readings today. Could you please text them in. Kind regards.',
              :valid_reading_message => 'Your reading is fine',
              :critical_low_message => 'this is low breach message',
              :critical_high_message => 'this is high breach message',
              :alert_low_message => 'Your blood pressure is a little low today. Take it again, and follow the advice in your management plan. Take care.',
              :alert_high_message => 'Your blood pressure is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
          )
      )

      #########################################################################
      ## Get to customization page with the protocol loaded and expanded
      #########################################################################

      # Search for the desired patient
      page.patient_search_element.click
      page.waitForPageToLoad
      page = page.search_patient_and_select(patient_search_string, patient_result_name)

      while !(is_protocol_assigned && is_protocol_edited) do
        page = UpdatePatientProtocolListPage.new(@browser)

        if !is_protocol_assigned then
          puts "Assigning and customizing a new protocol"

          page.New_Protocol
          page.waitForLoadingToClear

          # Assign the protocol and elect to customize it
          page.assignProtocolByName(protocol_to_assign)
          page.customizeprotocol_yes
          page.waitForLoadingToClear
          page = UpdatePatientProtocolEditPage.new(@browser)

          is_protocol_assigned = true
        elsif !is_protocol_edited then
          puts "Editing a previously-assigned protocol"
          # Assign the protocol and elect to customize it
          @list = page.patient_protocol_list
          @index = page.get_index_by_text(@list, 'Blood Pressure Protocol1')
          protocol = page.patient_protocol_list[@index]
          protocol.edit_temp
          page.waitForLoadingToClear
          page = UpdatePatientProtocolEditPage.new(@browser)

          is_protocol_edited = true
        end

        # Expand the "Vitals: Blood Pressure" template and the Schedule Settings accordion
        page.custom_list[0].custom_template_name_element.click
        page.custom_list[0].custom_schedule_settings_element.click

        #########################################################################
        ## Verify the start date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_startdate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_template_startdate_one).to eq(error_messages.get_error_messages(
                                                                       :schedule_start_date_empty_field => true
                                                                   ))

        # Restore the start date value
        page.custom_list[0].custom_protocol_startdate = blood_pressure_template.schedule.start_date

        #########################################################################
        ## Verify the end date schedule settings error message
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_enddate = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_template_enddate_one).to eq(error_messages.get_error_messages(
                                                                     :schedule_end_date_empty_field => true
                                                                 ))

        # Restore the end date value
        page.custom_list[0].custom_protocol_enddate = blood_pressure_template.schedule.end_date

        #########################################################################
        ## Verify the first lowest valid reading error messages
        #########################################################################

        # Expand the Readings Settings accordion
        page.custom_list[0].custom_reading_settings_element.click

        # Empty field
        page.custom_list[0].custom_lowest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_empty_field => true
                                                                        ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).critical_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true
                                                                        ))

        # Violate alert lower limit
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true
                                                                        ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_one).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_highest_valid_violation => true,
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Restore lowest valid value
        page.custom_list[0].custom_lowest_validreading_one = blood_pressure_template.get_reading(0).valid_range.lowest

        #########################################################################
        ## Verify the first highest valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highest_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_empty_field => true
                                                                         ))

        # Violate critical upper limit
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).critical_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true
                                                                         ))

        # Violate alert upper limit
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate alert lower limit
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate critical lower limit
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate valid lower limit
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_lowest_valid_violation => true,
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Restore highest valid value
        page.custom_list[0].custom_highest_validreading_one = blood_pressure_template.get_reading(0).valid_range.highest

        #########################################################################
        ## Verify the first message for valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_validreading_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_validreading_one).to eq(error_messages.get_error_messages(
                                                                             :valid_reading_empty_message_field => true
                                                                         ))

        # Restore lowest valid value
        page.custom_list[0].custom_message_validreading_one = blood_pressure_template.messages.valid_reading_message

        #########################################################################
        ## Verify the first critical low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_lowlevel_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_empty_field => true
                                                                      ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_lowest_valid_violation => true
                                                                      ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true
                                                                      ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate critical upper limit
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_one).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_highest_valid_violation => true,
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Restore highest valid value
        page.custom_list[0].custom_critical_lowlevel_one = blood_pressure_template.get_reading(0).critical_range.lowest

        #########################################################################
        ## Verify the first critical high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_highlevel_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_empty_field => true
                                                                       ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_highest_valid_violation => true
                                                                       ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate critical lower limit
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_one).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_lowest_valid_violation => true,
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Restore critical high value
        page.custom_list[0].custom_critical_highlevel_one = blood_pressure_template.get_reading(0).critical_range.highest

        #########################################################################
        ## Verify the first message for critical low level alert error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_criticallowalert = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_criticallowalert).to eq(error_messages.get_error_messages(
                                                                             :critical_low_reading_empty_message_field => true
                                                                         ))

        # Restore the critical low level message value
        page.custom_list[0].custom_message_criticallowalert = blood_pressure_template.messages.critical_low_message

        #########################################################################
        ## Verify the first message for critical high level alert error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_message_criticalhighalert = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_message_criticalhighalert).to eq(error_messages.get_error_messages(
                                                                              :critical_high_reading_empty_message_field => true
                                                                          ))

        # Restore the critical high level message value
        page.custom_list[0].custom_message_criticalhighalert = blood_pressure_template.messages.critical_high_message

        #########################################################################
        ## Verify the first alert low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_lowlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_empty_field => true
                                                                   ))

        # Violate valid lower limit
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_lowest_valid_violation => true,
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_highest_valid_violation => true,
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Restore the alert lowest value
        page.custom_list[0].custom_lowlevel_alert_one = blood_pressure_template.get_reading(0).alert_range.lowest

        #########################################################################
        ## Verify the first alert high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highlevel_alert_one = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_empty_field => true
                                                                    ))

        # Violate valid upper limit
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_highest_valid_violation => true,
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate critical upper limit
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate alert lower limit
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Violate critical lower limit
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true,
                                                                        :alert_high_reading_critical_low_violation => true
                                                                    ))

        # Violate valid lower limit
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_one).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true,
                                                                        :alert_high_reading_lowest_valid_violation => true,
                                                                        :alert_high_reading_critical_low_violation => true
                                                                    ))

        # Restore the alert highest value
        page.custom_list[0].custom_highlevel_alert_one = blood_pressure_template.get_reading(0).alert_range.highest

        #########################################################################
        ## Verify the first message for alert low level error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_below_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_below_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_low_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[0].custom_below_desirable_level_msgone = blood_pressure_template.messages.alert_low_message

        #########################################################################
        ## Verify the first message for alert high level error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_above_desirable_level_msgone = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_above_desirable_level_msgone).to eq(error_messages.get_error_messages(
                                                                                 :alert_high_reading_empty_message_field => true
                                                                             ))

        # Restore the alert low level message value
        page.custom_list[0].custom_above_desirable_level_msgone = blood_pressure_template.messages.alert_high_message

        #########################################################################
        ## Verify the second lowest valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_lowest_validreading_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_empty_field => true
                                                                        ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).critical_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true
                                                                        ))

        # Violate alert lower limit
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true
                                                                        ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowest_validreading_two).to eq(error_messages.get_error_messages(
                                                                            :lowest_valid_reading_highest_valid_violation => true,
                                                                            :lowest_valid_reading_critical_low_violation => true,
                                                                            :lowest_valid_reading_critical_high_violation => true,
                                                                            :lowest_valid_reading_alert_low_violation => true,
                                                                            :lowest_valid_reading_alert_high_violation => true
                                                                        ))

        # Restore lowest valid value
        page.custom_list[0].custom_lowest_validreading_two = blood_pressure_template.get_reading(1).valid_range.lowest

        #########################################################################
        ## Verify the second highest valid reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highest_validreading_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_empty_field => true
                                                                         ))

        # Violate critical upper limit
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).critical_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true
                                                                         ))

        # Violate alert upper limit
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate alert lower limit
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate critical lower limit
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Violate valid lower limit
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highest_validreading_two).to eq(error_messages.get_error_messages(
                                                                             :highest_valid_reading_lowest_valid_violation => true,
                                                                             :highest_valid_reading_critical_low_violation => true,
                                                                             :highest_valid_reading_critical_high_violation => true,
                                                                             :highest_valid_reading_alert_low_violation => true,
                                                                             :highest_valid_reading_alert_high_violation => true
                                                                         ))

        # Restore the highest valid value
        page.custom_list[0].custom_highest_validreading_two = blood_pressure_template.get_reading(1).valid_range.highest

        #########################################################################
        ## Verify the second critical low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_lowlevel_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_empty_field => true
                                                                      ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_lowest_valid_violation => true
                                                                      ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).alert_range.lowest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true
                                                                      ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate critical upper limit
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_lowlevel_two).to eq(error_messages.get_error_messages(
                                                                          :critical_low_reading_highest_valid_violation => true,
                                                                          :critical_low_reading_critical_high_violation => true,
                                                                          :critical_low_reading_alert_low_violation => true,
                                                                          :critical_low_reading_alert_high_violation => true
                                                                      ))

        # Restore the critical low level value
        page.custom_list[0].custom_critical_lowlevel_two = blood_pressure_template.get_reading(1).critical_range.lowest

        #########################################################################
        ## Verify the second critical high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_critical_highlevel_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_empty_field => true
                                                                       ))

        # Violate valid upper limit
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_highest_valid_violation => true
                                                                       ))

        # Violate alert upper limit
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).alert_range.highest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate alert lower limit
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate critical lower limit
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Violate valid lower limit
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_critical_highlevel_two).to eq(error_messages.get_error_messages(
                                                                           :critical_high_reading_lowest_valid_violation => true,
                                                                           :critical_high_reading_critical_low_violation => true,
                                                                           :critical_high_reading_alert_low_violation => true,
                                                                           :critical_high_reading_alert_high_violation => true
                                                                       ))

        # Restore the critical high level value
        page.custom_list[0].custom_critical_highlevel_two = blood_pressure_template.get_reading(1).critical_range.highest

        #########################################################################
        ## Verify the second alert low level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_lowlevel_alert_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_empty_field => true
                                                                   ))

        # Violate valid lower limit
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_lowest_valid_violation => true,
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate critical lower limit
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_low_violation => true
                                                                   ))

        # Violate alert upper limit
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).alert_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate critical upper limit
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Violate valid upper limit
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_lowlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                       :alert_low_reading_highest_valid_violation => true,
                                                                       :alert_low_reading_critical_high_violation => true,
                                                                       :alert_low_reading_alert_high_violation => true
                                                                   ))

        # Restore the alert low level value
        page.custom_list[0].custom_lowlevel_alert_two = blood_pressure_template.get_reading(1).alert_range.lowest

        #########################################################################
        ## Verify the second alert high level reading error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_highlevel_alert_two = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_empty_field => true
                                                                    ))

        # Violate valid upper limit
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).valid_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_highest_valid_violation => true,
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate critical upper limit
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).critical_range.highest + 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_critical_high_violation => true
                                                                    ))

        # Violate alert lower limit
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).alert_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Violate critical lower limit
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).critical_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_critical_low_violation => true,
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Violate valid lower limit
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).valid_range.lowest - 1
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_highlevel_alert_two).to eq(error_messages.get_error_messages(
                                                                        :alert_high_reading_lowest_valid_violation => true,
                                                                        :alert_high_reading_critical_low_violation => true,
                                                                        :alert_high_reading_alert_low_violation => true
                                                                    ))

        # Restore the alert high level value
        page.custom_list[0].custom_highlevel_alert_two = blood_pressure_template.get_reading(1).alert_range.highest

        #########################################################################
        ## Verify the service message error messages
        #########################################################################

        # Expand the Message Settings accordion
        page.custom_list[0].custom_message_settings_element.click

        # Empty field
        page.custom_list[0].custom_protocol_servicemessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_servicemessage).to eq(error_messages.get_error_messages(
                                                                            :service_message_empty_field => true
                                                                        ))

        # Restore the service message value
        page.custom_list[0].custom_protocol_servicemessage = blood_pressure_template.messages.service_message

        #########################################################################
        ## Verify the reminder message error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_remindermessage = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_remindermessage).to eq(error_messages.get_error_messages(
                                                                             :reminder_message_empty_field => true
                                                                         ))

        # Restore the reminder message value
        page.custom_list[0].custom_protocol_remindermessage = blood_pressure_template.messages.reminder_message

        #########################################################################
        ## Verify the amount of time between reminders error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_timebtwn = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_timebtwn).to eq(error_messages.get_error_messages(
                                                                      :time_between_reminders_empty_field => true
                                                                  ))

        # Restore the time between reminders value
        page.custom_list[0].custom_protocol_timebtwn = blood_pressure_template.time_between_reminders.value

        #########################################################################
        ## Verify the reminder message count error messages
        #########################################################################

        # Empty field
        page.custom_list[0].custom_protocol_remindertimesent = ""
        page.save_button_bottom
        page.waitForLoadingToClear
        expect(page.custom_list[0].error_summary).to eq(error_messages.get_error_messages(:error_summary => true))
        expect(page.custom_list[0].error_protocol_remindertimesent).to eq(error_messages.get_error_messages(
                                                                              :reminder_message_count_empty_field => true
                                                                          ))

        # Restore the time between reminders value
        page.custom_list[0].custom_protocol_remindertimesent = blood_pressure_template.reminder_message_count
        page.save_button_bottom
        page.waitForLoadingToClear
        page.modalwindow_ok
        page.waitForLoadingToClear
      end
    end
  end
end