#########################################################################################
# Stories:   CMS-557, CMS-1404, CMS-1414, CMS-1933
#
# Tests:    CMS-2119
#
# Acceptance Criteria:  CMS-559, CMS-1407, CMS-1408, CMS-1684, CMS-1690, CMS-1691, CMS-1702, CMS-1703, CMS-1704
#                       CMS-1706, CMS-1707, CMS-CMS-1708, CMS-1709, CMS-1710, CMS-1999, CMS-2003, CMS-2005, CMS-2006
#                       CMS-2008, CMS-2009
#
#
#########################################################################################


require 'spec_helper'

def updateProtocolScheduleInterval(expectedUnit, newUnit)
  page = UpdatePatientProtocolListPage.new(@browser)
  page.wait_for_page_to_load

  @list = page.patient_protocol_list
  @index = page.get_index_by_text(@list, @templateName)
  page.patient_protocol_list[@index].edit_temp

  page = UpdatePatientProtocolEditPage.new(@browser)
  template = page.custom_list[0]
  wait = Selenium::WebDriver::Wait.new(:timeout => 120)        # seconds
  wait.until {template.custom_template_name_element.visible?}

  template.custom_template_name_element.click
  template.custom_schedule_settings_element.click

  wait = Selenium::WebDriver::Wait.new(:timeout => 120)        # seconds
  wait.until {template.custom_protocol_schedtype_element.visible?}

  (template.custom_protocol_schedtype).should eq(@scheduleType)
  (template.custom_protocol_timebtwneachcycleunit).should eq(expectedUnit)
  template.custom_protocol_timebtwneachcycleunit=newUnit
  page.protocol_save_bottom
  page.waitForLoadingToClear
  page.save_msg.should eq 'Thank you. Your changes are saved.'
  page.modalwindow_ok
end

describe "Protocol Custom schedule type" do

     context "when assigning a new patient protocol and not selelcting the option to customize it" do
       it "The Patient Protocol values will match the Protocol Template values for Custom" do

         #**********************************************************************************************************************
         #***********  Variable Definitions
         #**********************************************************************************************************************

         @readingParamOne = 'Systolic' #will not need this after story cms 1404
         @highestNormalOne = '270' #will not need this after story cms 1404
         @lowestNormalOne = '70' #will not need this after story cms 1404
         @highestPossibleOne = '140' #will not need this after story cms 1404
         @lowestPossibleOne = '80'
         @readingParamTwo = 'Diastolic'
         @highestNormalTwo = '90'
         @lowestNormalTwo = '60'
         @highestPossibleTwo = '140'
         @lowestPossibleTwo = '40'
         @criticallowlevelone = '75'
         @criticalhighlevelone = '200'
         @criticallowleveltwo = '55'
         @criticalhighleveltwo = '100'


         @templateName = 'BloodPressure ProtocolTemplate-Custom_spec' #will not need this after story cms 1404
         @description = 'This is a new protocol created for Blood Pressure'
         @focusArea = 'Template Days, Templates, Days'
         @measurement = 'Blood Pressure'
         @firstPrompt = 'Hello, just a quick reminder that your readings are due. (e.g., BP 140 80). Thanks.'
         @reminderPrompt = 'Hi again. I have noticed that you have not sent in your readings today. Could you please text them in. Kind regards.'
         @aboveDesireOne = 'Your blood pressure is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
         @belowDesireOne = 'Your blood pressure is a little low today. Take it again, and follow the advice in your management plan. Take care.'
         @breach = 'Your BP is outside the safe range - so contact a doctor today as agreed in your management plan. Take care.'
         @scheduleType = 'Custom'
         @timebetween = '10'
         @timebetweenunit = 'Minute'
         @timebetweenunit2 = 'Hour'
         @timebetweenunit3 = 'Day'
         @timebetweenunit4 = 'Week'
         @timebetweenunit5 = 'Month'
         @timebetweenunit6 = 'Year'
         @formattedtimebetweenunit = 'Minute'
         @whattimehour = '9'
         @whattimemin = '15'
         @whattimeampm = 'PM'
         @rstarttime = '0'
         @rendtime = '10'
         @rstartselect = 'Hour'
         @rendselect = 'Month'

         @breachalert = '5'
         @howmnydays = '4'

         @breachtriggers = '40'
         @readingreminders = '1'
         @timebtwremindersvalue = '10'
         @timebtwremindersunit = 'Hour'

         @criticalHigh= 'Your BP is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
         @criticialLow = 'Your BP is CRITICALLY low  today. Take it again, and follow the advice in your management plan. Take care.'
         @rstartselect = 'Hour'
         @howmnydays = '4'
         @enddateB= '04/10/2015'
         @normalLimit = 'Your protocol reading is fine.'
         @charactersRemaining = '130 characters remaining'


         page = PatientSearchPage.new(@browser)
         page.waitForPageToLoad #will not need this after story cms 1404

         if !page.patient_search_element.visible? then
           page = AdminPage.new(@browser)
           page.GoToStaffSearch
           page = PatientSearchPage.new(@browser)
           page.patient_search_element.click
           page.waitForPageToLoad
         end

         page.goto_admin_page

         protocol = Protocol.new
         protocol.focusArea = @focusArea
         protocol.description = @description
         protocol.newTemplateName = @templateName

         page = ListTemplatesPage.new(@browser)

         page.waitForLoadingToClear
         page.add_protocol

         page = AddNewProtocolPage.new(@browser)
         page.protocol_name = @templateName
         page.protocol_description = @description
         page.protocol_focus_area = @focusArea
         page.next
         page.add_template_element.click
         page.vitals_element.click

         page = TemplateEditPage.new(@browser)
        template = page.custom_list[0]
         #Enter values on page

         page.schedule_protocol_once('9:15:PM', 0)
         template.custom_schedule = @scheduleType
         template.custom_time_between_each_cycle = @timebetween
         template.custom_time_between_each_cycle_units = @timebetweenunit
         template.custom_unscheduled_yes_selected?
         protocol = Protocol.new
         protocol.measurement = @measurement
         protocol.lowestNormalOne = @lowestNormalOne
         protocol.highestNormalOne = @highestNormalOne
         protocol.normalLimit = @normalLimit
         protocol.criticallowlevelone = @criticallowlevelone
         protocol.criticalhighlevelone = @criticalhighlevelone
         protocol.criticialLow = @criticialLow
         protocol.criticalHigh = @criticalHigh
         protocol.lowestPossibleOne = @lowestPossibleOne
         protocol.highestPossibleOne = @highestPossibleOne
         protocol.belowDesireOne = @belowDesireOne
         protocol.aboveDesireOne = @aboveDesireOne

         page.create_schedule_settings(protocol, 0)

         (template.custom_character_count).should eq(@charactersRemaining)

         protocol = Protocol.new
         protocol.firstPrompt = @firstPrompt
         protocol.reminderPrompt = @reminderPrompt
         protocol.readingreminders = @readingreminders
         protocol.rstartselect = @rstartselect
         protocol.howmnydays = @howmnydays


         page.create_message_settings(protocol, 0)

         page.finish
         page.success_ok

#*********************************************************************************************************************
        page = ListTemplatesPage.new(@browser)
         page.GoToStaffSearch

         page = PatientSearchPage.new(@browser)
         page.patient_search_element.click
         page.waitForLoadingToClear
         page = page.search_patient_and_select('patient03', @patient03Name)

         page = UpdatePatientProtocolListPage.new(@browser)
         page.wait_for_page_to_load

         page.New_Protocol
         page.waitForLoadingToClear

         page.assignProtocolByName(@templateName)
         page.customizeprotocol_yes

         page = UpdatePatientProtocolEditPage.new(@browser)
         template = page.custom_list[0]
         template.custom_template_name_element.click

         template.custom_schedule_settings_element.click

         pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
         pageLoadWait.until { template.custom_protocol_schedtype_options != '' }
         pageLoadWait.until { template.custom_protocol_whattimehour_options != '' }
         pageLoadWait.until { template.custom_protocol_whattimemin_options != '' }
         pageLoadWait.until { template.custom_protocol_whattimeampm_options != '' }

         (template.custom_protocol_schedtype).should eq(@scheduleType)
         (template.custom_protocol_timebtwneachcycle).should eq(@timebetween)
         (template.custom_protocol_timebtwneachcycleunit).should eq(@timebetweenunit)
         (template.custom_protocol_whattimehour).should eq(@whattimehour)
         (template.custom_protocol_whattimemin).should eq(@whattimemin)
         (template.custom_protocol_whattimeampm).should eq(@whattimeampm)

         template.custom_reading_settings_element.click
         (template.custom_highest_validreading_one).should eq(@highestNormalOne)
         (template.custom_lowest_validreading_one).should eq(@lowestNormalOne)
         (template.custom_message_validreading_one).should eq(@normalLimit)
         (template.custom_highlevel_alert_one).should eq(@highestPossibleOne)
         (template.custom_lowlevel_alert_one).should eq(@lowestPossibleOne)
         (template.custom_message_criticallowalert).should eq(@criticialLow)
         (template.custom_message_criticalhighalert).should eq(@criticalHigh)
         (template.custom_critical_lowlevel_one).should eq(@criticallowlevelone)
         (template.custom_critical_highlevel_one).should eq(@criticalhighlevelone)
         (template.custom_below_desirable_level_msgone).should eq(@belowDesireOne)
         (template.custom_above_desirable_level_msgone).should eq(@aboveDesireOne)
         (template.custom_highest_validreading_two).should eq(@highestPossibleTwo)
         (template.custom_lowest_validreading_two).should eq(@lowestPossibleTwo)
         (template.custom_highlevel_alert_two).should eq(@highestNormalTwo)
         (template.custom_lowlevel_alert_two).should eq(@lowestNormalTwo)
         (template.custom_critical_lowlevel_two).should eq(@criticallowleveltwo)
         (template.custom_critical_highlevel_two).should eq(@criticalhighleveltwo)

         template.custom_message_settings_element.click

         wait = Selenium::WebDriver::Wait.new(:timeout => 120)        # seconds
         wait.until {template.custom_protocol_servicemessage_element.visible?}

         (template.custom_protocol_servicemessage).should eq(@firstPrompt)
         (template.custom_protocol_remindermessage).should eq(@reminderPrompt)
         (template.custom_protocol_remindertime).should eq(@timebtwremindersunit)
         (template.custom_protocol_timebtwn).should eq(@readingreminders)
         (template.custom_protocol_remindertimesent).should eq(@howmnydays)

         page.protocol_save_bottom
         page.save_msg.should eq 'Thank you. Your changes are saved.'
         page.modalwindow_ok

         updateProtocolScheduleInterval(@timebetweenunit, @timebetweenunit2)
         updateProtocolScheduleInterval(@timebetweenunit2, @timebetweenunit3)
         updateProtocolScheduleInterval(@timebetweenunit3, @timebetweenunit4)
         updateProtocolScheduleInterval(@timebetweenunit4, @timebetweenunit5)
         updateProtocolScheduleInterval(@timebetweenunit5, @timebetweenunit6)
         # no need to change from existing on the last round
         updateProtocolScheduleInterval(@timebetweenunit6, @timebetweenunit6)
#**********************************************************************************************************************


       end
     end
   end
