#########################################################################################
# Stories:   CMS-202, CMS-2227
#
# Tests:    CMS-2354
#
# Acceptance Criteria:  CMS-203, CMS-204, CMS-237, CMS-2246, CMS-2254
#
#
#########################################################################################


require 'spec_helper'

describe "Protocol Specify Days schedule type" do

  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "when assigning a new patient protocol and not selecting the option to customize it" do
    it "The Patient Protocol values will match the Protocol Template values for Specified Days" do


#**********************************************************************************************************************
#***********  Variable Definitions
#**********************************************************************************************************************

      @readingParamOne = 'Systolic' #will not need this after story cms 1404
      @highestNormalOne = '270' #will not need this after story cms 1404
      @lowestNormalOne = '70' #will not need this after story cms 1404
      @highestPossibleOne = '140' #will not need this after story cms 1404
      @lowestPossibleOne = '80'
      @readingParamTwo = 'Diastolic'
      @highestNormalTwo = '90'
      @lowestNormalTwo = '60'
      @highestPossibleTwo = '140'
      @lowestPossibleTwo = '40'
      @criticallowlevelone = '75'
      @criticalhighlevelone = '200'
      @criticallowleveltwo = '55'
      @criticalhighleveltwo = '100'

      @measurement = 'Blood Pressure'
      @firstPrompt = 'Hello, just a quick reminder that your readings are due. (e.g., BP 140 80). Thanks.'
      @reminderPrompt = 'Hi again. I have noticed that you have not sent in your readings today. Could you please text them in. Kind regards.'
      @aboveDesireOne = 'Your blood pressure is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
      @belowDesireOne = 'Your blood pressure is a little low today. Take it again, and follow the advice in your management plan. Take care.'
      @breach = 'Your BP is outside the safe range - so contact a doctor today as agreed in your management plan. Take care.'
      @scheduleType = 'Specified Days'
      @whattimehour = '9'
      @whattimemin = '15'
      @whattimeampm = 'PM'
      @rstarttime = '0'
      @rendtime = '10'
      @rendselect = 'Minute'

      @breachalert = '5'
      @howmnydays = '4'

      @breachtriggers = '40'
      @readingreminders = '1'
      @timebtwremindersvalue = '10'
      @timebtwremindersunit = 'Minute'
      @formattedtimebtwremindersunit = 'Minute'

      @criticalHigh= 'Your BP is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
      @criticialLow = 'Your BP is CRITICALLY low  today. Take it again, and follow the advice in your management plan. Take care.'
      @readingreminders = '1'
      @rstartselect = 'Minute'
      @howmnydays = '4'
      @enddateB= '04/10/2015'
      @normalLimit = 'Your protocol reading is fine.'
      @templateScheduleName = 'Vitals: Blood Pressure'
      @templateSchedule = "Mon., Wed., Fri.\n(At 9:15 PM)"

      page = PatientSearchPage.new(@browser)
      page.waitForPageToLoad #will not need this after story cms 1404

      page.goto_admin_page

      @templateName = 'BloodPressure ProtocolTemplate-Days_spec'
      @description = 'Blood Pressure'
      @focusArea = 'Template Days, Templates, Days'

      protocol = Protocol.new
      protocol.focusArea = @focusArea
      protocol.description = @description
      protocol.newTemplateName = @templateName

      page = ListTemplatesPage.new(@browser)

      page.create_new_protocol(protocol)

      page = TemplateEditPage.new(@browser)
      template = page.custom_list[0]

#**********************************************************************************************************************
#***********  Create newProtocol Template #edit specified days
#**********************************************************************************************************************

      page.schedule_protocol_once('9:15:PM',0)
      template.custom_schedule='Specified Days'
      template.check_custom_protocol_mon
      template.check_custom_protocol_wed
      template.check_custom_protocol_fri

      protocol = Protocol.new
      protocol.measurement = @measurement
      protocol.lowestNormalOne = @lowestNormalOne
      protocol.highestNormalOne = @highestNormalOne
      protocol.normalLimit = @normalLimit
      protocol.criticallowlevelone = @criticallowlevelone
      protocol.criticalhighlevelone = @criticalhighlevelone
      protocol.criticialLow = @criticialLow
      protocol.criticalHigh = @criticalHigh
      protocol.lowestPossibleOne = @lowestPossibleOne
      protocol.highestPossibleOne = @highestPossibleOne
      protocol.belowDesireOne = @belowDesireOne
      protocol.aboveDesireOne = @aboveDesireOne

      page.create_schedule_settings(protocol,0)

      protocol = Protocol.new
      protocol.firstPrompt = @firstPrompt
      protocol.reminderPrompt = @reminderPrompt
      protocol.readingreminders = @readingreminders
      protocol.rstartselect = @rstartselect
      protocol.howmnydays = @howmnydays


      page.create_message_settings(protocol,0)

      page.finish
      page.success_ok
#**********************************************************************************************************************
#***********  Verify newly created Protocol Template values
#**********************************************************************************************************************

      page = ListTemplatesPage.new(@browser)
#This will work until the new view edit page is hooked up
      @list = page.protocol_list
      @index = page.get_index_by_text(@list, @templateName)

      page.protocol_list[@index].edit_view

      #This code will go away when the new view edit page comes on line.
      page = TemplateEditPage.new(@browser)
      template = page.custom_list[0]

      page.waitForLoadingToClear

      pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
      pageLoadWait.until { template.custom_template_name != '' }

      template.custom_template_name_element.click
      template.custom_schedule_settings_element.click
       template.custom_protocol_mon_checked?.should be_true
      template.custom_protocol_wed_checked?.should be_true
      template.custom_protocol_fri_checked?.should be_true
      template.custom_readings_alerts_settings_element.click

      (template.custom_lowest_valid_reading).should eq(@lowestNormalOne)
      (template.custom_highest_valid_reading).should eq(@highestNormalOne)
      (template.custom_valid_reading_message).should eq(@normalLimit)
      (template.custom_critical_low_level_alert).should eq(@criticallowlevelone)
      (template.custom_critical_high_level_alert).should eq(@criticalhighlevelone)
      (template.custom_critical_low_level_alert_message).should eq(@criticialLow)
      (template.custom_critical_high_level_alert_message).should eq(@criticalHigh)
      (template.custom_low_level_alert).should eq(@lowestPossibleOne)
      (template.custom_high_level_alert).should eq(@highestPossibleOne)
      (template.custom_low_level_alert_message).should eq(@belowDesireOne)
      (template.custom_high_level_alert_message).should eq(@aboveDesireOne)
      (template.custom_lowest_valid_reading2).should eq(@lowestPossibleTwo)
      (template.custom_highest_valid_reading2).should eq(@highestPossibleTwo)
      (template.custom_critical_low_level_alert2).should eq(@criticallowleveltwo)
      (template.custom_critical_high_level_alert2).should eq(@criticalhighleveltwo)
      (template.custom_low_level_alert2).should eq(@lowestNormalTwo)
      (template.custom_high_level_alert2).should eq(@highestNormalTwo)

      template.custom_message_settings_element.click
      (template.custom_service_message).should eq(@firstPrompt)
      (template.custom_reminder_message).should eq(@reminderPrompt)
      (template.custom_reminder_time_between).should eq(@readingreminders)
      (template.custom_reminder_time_schedule).should eq(@rstartselect)
      (template.custom_reminder_time_sent).should eq(@howmnydays)

      page.protocols_tab_element.click


#*******************************************************************************************************************************
#***********  Assign Protocol to Patient and Verify Patient Protocol values with Protocol Template values. #edit specified days
#*******************************************************************************************************************************
      page = ListTemplatesPage.new(@browser)
      page.GoToStaffSearch

      page = PatientSearchPage.new(@browser)
      page = page.search_patient_and_select('patient01', @patient01Name)

      page = UpdatePatientProtocolListPage.new(@browser)
      page.New_Protocol
      page.waitForLoadingToClear

      page.assignProtocolByName(@templateName)
      page.customizeprotocol_yes

      patient_protocol_edit_page = UpdatePatientProtocolEditPage.new(@browser)
      template = patient_protocol_edit_page.custom_list[0]

      (template.custom_template_name).should eq(@templateScheduleName)
      (template.custom_template_schedule_heading).should eq(@templateSchedule)
      (patient_protocol_edit_page.description).should eq "Description: " + (@description)

      template.custom_template_name_element.click
      template.custom_schedule_settings_element.click

     # patient_protocol_edit_page.protocol_unscheduled_selected?.should be_true
      (template.custom_protocol_schedtype).should eq(@scheduleType)
      template.custom_protocol_mon_checked?.should be_true
      template.custom_protocol_wed_checked?.should be_true
      template.custom_protocol_fri_checked?.should be_true

      (template.custom_protocol_whattimehour).should eq(@whattimehour)
      (template.custom_protocol_whattimemin).should eq(@whattimemin)
      (template.custom_protocol_whattimeampm).should eq(@whattimeampm)

      template.custom_reading_settings_element.click

      (template.custom_highest_validreading_one).should eq(@highestNormalOne)
      (template.custom_lowest_validreading_one).should eq(@lowestNormalOne)
      (template.custom_message_validreading_one).should eq(@normalLimit)
      (template.custom_highlevel_alert_one).should eq(@highestPossibleOne)
      (template.custom_lowlevel_alert_one).should eq(@lowestPossibleOne)
      (template.custom_message_criticallowalert).should eq(@criticialLow)
      (template.custom_message_criticalhighalert).should eq(@criticalHigh)
      (template.custom_critical_lowlevel_one).should eq(@criticallowlevelone)
      (template.custom_critical_highlevel_one).should eq(@criticalhighlevelone)
      (template.custom_below_desirable_level_msgone).should eq(@belowDesireOne)
      (template.custom_above_desirable_level_msgone).should eq(@aboveDesireOne)
      (template.custom_highest_validreading_two).should eq(@highestPossibleTwo)
      (template.custom_lowest_validreading_two).should eq(@lowestPossibleTwo)
      (template.custom_highlevel_alert_two).should eq(@highestNormalTwo)
      (template.custom_lowlevel_alert_two).should eq(@lowestNormalTwo)
      (template.custom_critical_lowlevel_two).should eq(@criticallowleveltwo)
      (template.custom_critical_highlevel_two).should eq(@criticalhighleveltwo)

      template.custom_message_settings_element.click
      (template.custom_protocol_servicemessage).should eq(@firstPrompt)
      (template.custom_protocol_remindermessage).should eq(@reminderPrompt)
      (template.custom_protocol_remindertime).should eq(@formattedtimebtwremindersunit)
      (template.custom_protocol_timebtwn).should eq(@readingreminders)
      (template.custom_protocol_remindertimesent).should eq(@howmnydays)

      #Click on Cancel Button
      patient_protocol_edit_page.protocol_cancel_bottom
      patient_protocol_edit_page.modalwindow_cancelyes

      page = PatientSearchPage.new(@browser)
      page.patient_search_element.click
      page.waitForLoadingToClear
    end
  end
end
