#########################################################################################
# Stories:   CMS-1175
#
# Tests:
#
# Acceptance Criteria:  CMS-1176
#
#
#########################################################################################


require 'spec_helper'

describe "ProtocolTemplate_Once_Protocol template page" do

  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds
  context "when creating a new protocol template" do
    today_date = DateTime.now.strftime('%m/%d/%Y')

    it "Validate the field validations on the Add Protocol page" do

      page = PatientSearchPage.new(@browser)
      page.patient_search_element.click

      page.goto_admin_page
      page = ListTemplatesPage.new(@browser)
      page.add_protocol

      page = AddNewProtocolPage.new(@browser)
      page.next

      expect(page.name_error).to eq("Please enter a Protocol Name")
      expect(page.description_error).to eq("Please enter a Protocol Description")
      expect(page.focus_area_error).to eq("Please enter a Focus Area")

      # enter 310 random characters; limit 300
      #page.protocol_name= "name"
      page.protocol_name= SecureRandom.hex(310)

      expect(page.protocol_name.length).to be <= 300

      page.next

      expect(page.description_error).to eq("Please enter a Protocol Description")
      expect(page.focus_area_error).to eq("Please enter a Focus Area")

      # enter 310 random characters; limit 300
      #page.protocol_description= "description"
      page.protocol_description= SecureRandom.hex(310)

      expect(page.protocol_description.length).to be <= 300

      page.next

      expect(page.focus_area_error).to eq("Please enter a Focus Area")

      # enter 310 random characters; limit 300
      #page.protocol_focus_area= "focus area"
      page.protocol_focus_area= SecureRandom.hex(310)

      expect(page.protocol_focus_area.length).to be <= 300

      page.next

      expect(page.add_template_element.visible?).to be_true
      page.protocols_tab_element.click
    end

    it "Define Protocol Template Unscheduled Readings" do

      #Enter values on page

      @protocolName = 'Blood Pressure Protocol1 Template - Once'
      @templateName = 'Vitals: Blood Pressure'
      @file_stamp = DateTime.now.strftime('%m-%d-%Y-%l%M%P')
      @newTemplateName = @templateName + ' - ' + @file_stamp
      @focusArea = 'BP, Blood Pressure, Hypertension'
      @keywords = 'BP, Blood Pressure'
      @readingParamOne = 'Systolic'
      @highestNormalOne = '140'
      @lowestNormalOne = '80'
      @highestPossibleOne = '95'
      @lowestPossibleOne = '90'
      @readingParamTwo = 'Diastolic'
      @highestNormalTwo = '85'
      @lowestNormalTwo = '15'
      @highestPossibleTwo = '70'
      @lowestPossibleTwo = '60'
      @description = 'This is a new protocol created for Blood Pressure'
      @measurement = 'Blood Pressure'
      @firstPrompt = 'Hello, just a quick reminder that your readings are due. (e.g., BP 140 80). Thanks.'
      @reminderPrompt = 'Hi again. I have noticed that you have not sent in your readings today. Could you please text them in. Kind regards.'
      @normalLimit = 'Your protocol reading is fine.'
      @aboveDesireOne = 'Your blood pressure is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
      @belowDesireOne = 'Your blood pressure is a little low today. Take it again, and follow the advice in your management plan. Take care.'
      @criticalHigh = 'Your blood pressure is CRITICALLY high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'
      @criticialLow = 'Your blood pressure is CRITICALLY low  today. Take it again, and follow the advice in your management plan. Take care.'
      @breach = 'Your BP is outside the safe range - so contact a doctor today as agreed in your management plan. Take care.'
      @scheduleType = 'Once'
      @whattimehour = '9'
      @whattimemin = '15'
      @whattimeampm = 'PM'
      @rstarttime = '0'
      @rendtime = '10'
      @rstartselect = 'Minute'
      @rendselect = 'Minute'
      @startDate = today_date
      @enddate = '04/10/2015'
      @breachalert = '5'
      @howmnydays = '4'
      @breachtriggers = '40'
      @readingreminders = '1'
      @timebtwremindersvalue = '10'
      @timebtwremindersunit = 'Minute'
      @criticallowlevelone = '85'
      @criticalhighlevelone = '135'
      @criticallowleveltwo = '55'
      @criticalhighleveltwo = '80'

      protocol = Protocol.new
      protocol.focusArea = @focusArea
      protocol.description = @description
      protocol.newTemplateName = @protocolName

      page = ListTemplatesPage.new(@browser)
      page.create_new_protocol(protocol)

      page = TemplateEditPage.new(@browser)

      page.schedule_protocol_once('9:15:PM',0)

      protocol = Protocol.new
      protocol.measurement = @measurement
      protocol.lowestNormalOne = @lowestNormalOne
      protocol.highestNormalOne = @highestNormalOne
      protocol.normalLimit = @normalLimit
      protocol.criticallowlevelone = @criticallowlevelone
      protocol.criticalhighlevelone = @criticalhighlevelone
      protocol.criticialLow = @criticialLow
      protocol.criticalHigh = @criticalHigh
      protocol.lowestPossibleOne = @lowestPossibleOne
      protocol.highestPossibleOne = @highestPossibleOne
      protocol.belowDesireOne = @belowDesireOne
      protocol.aboveDesireOne = @aboveDesireOne

      page.create_schedule_settings(protocol,0)

      protocol = Protocol.new
      protocol.lowestNormalTwo = @lowestNormalTwo
      protocol.highestNormalTwo = @highestNormalTwo
      protocol.criticallowleveltwo = @criticallowleveltwo
      protocol.criticalhighleveltwo = @criticalhighleveltwo
      protocol.lowestPossibleTwo = @lowestPossibleTwo
      protocol.highestPossibleTwo = @highestPossibleTwo

      page.create_schedule_settings_2(protocol,0)

      protocol = Protocol.new
      protocol.firstPrompt = @firstPrompt
      protocol.reminderPrompt = @reminderPrompt
      protocol.readingreminders = @readingreminders
      protocol.rstartselect = @rstartselect
      protocol.howmnydays = @howmnydays


      page.create_message_settings(protocol,0)

      page.finish
      page.success_ok

#**********************************************************************************************************************

      page = ListTemplatesPage.new(@browser)
      @list = page.protocol_list
      @index = page.get_index_by_text(@list, @protocolName)

      page.protocol_list[@index].edit_view
      page.waitForLoadingToClear
      page = TemplateEditPage.new(@browser)
      template = page.custom_list[0]
      pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
      pageLoadWait.until { template.custom_template_name != '' }
      (page.protocol_name).should eq(@protocolName)
      page.protocol_name = @protocolName + " EDITED"
      (page.protocol_description).should eq(@description)
      page.protocol_description = @description + " EDITED"
      (page.protocol_focus_area).should eq @focusArea
      page.protocol_focus_area = @focusArea + " EDITED"
      (template.custom_template_name).should eq(@templateName)
      (template.custom_template_schedule).should eq("Once\n(At 9:15 PM)")
      template.custom_template_name_element.click
      template.custom_readings_alerts_settings_element.click
      #pageLoadWait = Selenium::WebDriver::Wait.new(timeout: 90) # seconds
      #pageLoadWait.until { template.custom_template_measurement != '' }
      (template.custom_lowest_valid_reading).should eq(@lowestNormalOne)
      (template.custom_highest_valid_reading).should eq(@highestNormalOne)
      (template.custom_valid_reading_message).should eq(@normalLimit)
      (template.custom_critical_low_level_alert).should eq(@criticallowlevelone)
      (template.custom_critical_high_level_alert).should eq(@criticalhighlevelone)
      (template.custom_critical_low_level_alert_message).should eq(@criticialLow)
      (template.custom_critical_high_level_alert_message).should eq(@criticalHigh)
      (template.custom_low_level_alert).should eq(@lowestPossibleOne)
      (template.custom_high_level_alert).should eq(@highestPossibleOne)
      (template.custom_low_level_alert_message).should eq(@belowDesireOne)
      (template.custom_high_level_alert_message).should eq(@aboveDesireOne)

      (template.custom_lowest_valid_reading2).should eq(@lowestNormalTwo)
      (template.custom_highest_valid_reading2).should eq(@highestNormalTwo)
      (template.custom_critical_low_level_alert2).should eq(@criticallowleveltwo)
      (template.custom_critical_high_level_alert2).should eq(@criticalhighleveltwo)
      (template.custom_low_level_alert2).should eq(@lowestPossibleTwo)
      (template.custom_high_level_alert2).should eq(@highestPossibleTwo)

      template.custom_message_settings_element.click
      (template.custom_service_message).should eq(@firstPrompt)
      (template.custom_reminder_message).should eq(@reminderPrompt)
      (template.custom_reminder_time_between).should eq(@readingreminders)
      (template.custom_reminder_time_schedule).should eq(@rstartselect)
      (template.custom_reminder_time_sent).should eq(@howmnydays)

      page.protocols_tab_element.click

      page = AdminPage.new(@browser)
      page.GoToStaffSearch
      page = PatientSearchPage.new(@browser)
    end
  end
end
