########################################################################################################
# Stories:   CMS-630, CMS-804, CMS-911, CMS-913, CMS-925, CMS-1089, CMS-36, CMS-1398, CMS-1661, CMS-3977
#
# Tests:   CMS-2136
#
# Acceptance Criteria:  CMS-964, CMS-965, CMS-966, CMS-1087, CMS- 1131, CMS-1133, CMS-1134, CMS-1135,
# CMS-1679, CMS-1850, CMS-1851, CMS-1757, CMS-1697, CMS-1679, CMS-1699, CMS-1700, CMS-1701, CMS-4141
#
########################################################################################################
require 'spec_helper'

# Set variables
mobile_phone_duplicate_formatted = '571-050-0001'
mobile_phone_duplicate_raw = mobile_phone_duplicate_formatted.delete '-'

duplicate_mobile_number_error_message = 'Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX.'
no_message_preference_error_message = 'Please make a selection'
account_changes_saved_message = 'Your changes are saved.'

patient26_username = 'zztest.patient26'
patient56_username = 'zztest.patient56'
default_password = 'pass'

patient26 = {
    :user_name => 'zztest.patient26',
    :password => default_password,
    :home_phone_formatted => '703-765-9834',
    :mobile_phone_formatted => '703-111-2222',
    :email => 'zpatient@testmail.com',
    :time_zone => 'Eastern (GMT-05:00)',
    :facility => 'LONG BEACH VAMC',
    :mobile_phone_updated_formatted => '727-897-5678',
    :email_updated => 'vehu09@email.com'
}
patient26[:home_phone_raw] = patient26[:home_phone_formatted].delete '-'
patient26[:mobile_phone_raw] = patient26[:mobile_phone_formatted].delete '-'
patient26[:mobile_phone_updated_raw] = patient26[:mobile_phone_updated_formatted].delete '-'

patient56 = {
    :user_name => 'zztest.patient56',
    :password => default_password,
    :home_phone_formatted => '703-765-9834',
    :mobile_phone_formatted => '727-897-5678',
    :email => 'zpatient@testmail.com',
    :time_zone => 'Eastern (GMT-05:00)'
}
patient56[:home_phone_raw] = patient56[:home_phone_formatted].delete '-'
patient56[:mobile_phone_raw] = patient56[:mobile_phone_formatted].delete '-'

describe 'New Patient Self-Registration (CMS-913 Part A)' do
  context "New patient registration welcome message" do
    it "Validate the welcome message on the Patient Registration page (CMS-1679)" do
      # Set variables
      test_patient = patient26
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']

      # Open a new veteran browser
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Sign out of the Veteran app
      page.sign_out

      # Log back into the Veteran app with an unregistered patient
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetProfileEditPage.new(@veteran_browser)
      expect(page.user_nav).not_to be_nil

      # Verify the welcome message with the patient's first name
      expect(page.user_nav_element.text).to eq('Welcome, ' << test_patient[:first_name].capitalize)
    end
  end

  context 'New patient registration and form validation' do
    it 'Register the new patient account' do
      # Set variables
      test_patient = patient26
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:middle_name] = @patient_data[test_patient[:user_name]]['middle_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:ssn] = @patient_data[test_patient[:user_name]]['ssn']
      test_patient[:dob] = @patient_data[test_patient[:user_name]]['dob']
      test_patient[:gender] = @patient_data[test_patient[:user_name]]['gender']

      # Launch a new browser for patient registration
      page = VetProfileEditPage.new(@veteran_browser)

      # Expand the Patient Info section
      page.heading_patient_info

      # Verify pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])

      # Verify the home phone field formatting restrictions
      invalid_phone_numbers = ['111222','1','12','123','1234','12345','123456','1234567','12345678','123456789','123456-789']
      invalid_phone_numbers.each { |invalid_phone_number|
        page.home_phone_element.value = invalid_phone_number
        page.home_phone_element.fire_event('blur')
        expect(page.home_phone_format_error_element.text).to eq('Please enter a valid Home Phone')
      }

      # Set the home phone field back to the correct value
      page.home_phone_element.value = test_patient[:home_phone_raw]
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])

      # Submit the registration form
      page.submitRegistration

      # Verify the expected validation error messages
      expect(page.phone_format_error_element.when_visible.text).to eq('Please enter a Mobile Phone')
      expect(page.timezone_error_element.when_visible.text).to eq('Please select a Time Zone')
      expect(page.select_phone_format_error_element.when_visible.text).to eq(no_message_preference_error_message)
      expect(page.consent_form_error_element.when_visible.text).to eq('Please make a selection')

      # Verify the expected option values are present in the Time Zone select element
      time_zone_options = ['Alaska (GMT-09:00)', 'Atlantic (GMT-04:00)', 'Central (GMT-06:00)', 'Chamorro (GMT+10:00)', 'Eastern (GMT-05:00)', 'Hawaii-Aleutian (GMT-10:00)', 'Mountain (GMT-07:00)', 'Pacific (GMT-08:00)', 'Samoa (GMT-11:00)']
      page.time_zone = patient26[:time_zone]
      expect(page.time_zone_element.selected_options[0]).to eq(test_patient[:time_zone])
      time_zone_options.each { |current_time_zone|
        expect(page.time_zone_element).to include current_time_zone
      }

      # Verify the mobile phone field formatting restrictions
      invalid_phone_numbers = ['111222','12','123','1234','12345','123456','1234567','12345678','123456789','123456-789']
      invalid_phone_numbers.each { |invalid_phone_number|
        page.mobile_phone_element.value = invalid_phone_number
        page.mobile_phone_element.fire_event('blur')
        expect(page.phone_numeric_error_element.text).to eq('Please enter a valid Mobile Phone')
      }

      # Set the mobile phone field back to the correct value
      page.mobile_phone_element.value = test_patient[:mobile_phone_raw]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.mobile_phone_notice_error_element.when_visible.text).to eq('(Used to receive messages)')

      # Verify the email field formatting restrictions
      invalid_emails = ['&','t','tbecket','tbecketsomewhere','tbecketsomewhere.','tbecketsomewhere.xxxxxxx','tbecket@']
      invalid_emails.each { |invalid_email|
        page.email_element.value = invalid_email
        page.email_element.fire_event('blur')
        expect(page.email_error_element.text).to eq('Please enter a valid Email Address')
      }

      # Set the email field back to the correct value
      page.email = test_patient[:email]

      # Submit the registration form
      page.submitRegistration

      # Verify the expected validation error messages
      expect(page.select_phone_format_error_element.when_visible.text).to eq(no_message_preference_error_message)

      # Verify the basic cell phone message settings
      page.select_basic_cell
      expect(page.basic_cell_selected?).to eq(true)
      expect(page.smart_phone_selected?).to eq(false)

      # Verify the smart phone message settings
      page.select_smart_phone
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(false)
      page.check_receive_nonsecure_msgs
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(true)

      # Accept the patient consent form
      expect(page.select_patient_consent_chkbox_checked?).to be_false
      page.consent_form
      page.scroll_consent_form_down
      page.button_consent_yes_element.when_visible.click
      sleep(2)  # Allow the modal dialog to close
      expect(page.select_patient_consent_chkbox_checked?).to be_true

      # Expand the Appointment/Info Messages section
      page.heading_appointment_info_messages

      # Select the facility messages checkbox
      expect(page.facility_checkbox_checked?).to eq(false)
      expect(page.facility).to eq('')
      page.check_facility_checkbox
      expect(page.facility_checkbox_checked?).to eq(true)
      expect(page.facility).to eq('')

      # Submit the registration form
      page.submitRegistration

      # Verify the expected validation error messages
      expect(page.facility_error).to eq('Please select a Facility')

      # Set the facility field to the correct value
      page.facility = test_patient[:facility]

      # Verify the appointment reminders settings
      expect(page.appointment_reminders_checkbox_checked?).to eq(false)
      page.check_appointment_reminders_checkbox
      expect(page.appointment_reminders_checkbox_checked?).to eq(true)

      # Verify that duplicate phone numbers are not allowed
      page.mobile_phone = mobile_phone_duplicate_raw
      page.submitRegistration
      page.waitForLoadingToClear
      expect(page.duplicate_mobile_phone_error).to eq(duplicate_mobile_number_error_message)

      # Reset the mobile phone field to the correct value
      page.mobile_phone_element.value = test_patient[:mobile_phone_raw]

      # Submit the registration form
      page.submitRegistration
      page = VetSuccessPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Sign out
      page.sign_out
      sleep(1)

      # Log back in as the newly-registered patient
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Expand all the section accordions
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.email).to eq(test_patient[:email])
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.facility).to eq(test_patient[:facility])
      expect(page.facility_checkbox_checked?).to eq(true)
      expect(page.appointment_reminders_checkbox_checked?).to eq(true)

      # Update the mobile phone and email values
      page.mobile_phone = test_patient[:mobile_phone_updated_raw]
      page.email = test_patient[:email_updated]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_updated_formatted])
      expect(page.email).to eq(test_patient[:email_updated])

      # Save the account changes
      page.saveRegistration
      page.waitForLoadingToClear
      expect(page.success_msg_element.when_visible.text).to eq(account_changes_saved_message)

      # Sign out
      page.sign_out

      # Log back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Expand all the section accordions
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_updated_formatted])
      expect(page.time_zone).to eq(test_patient[:time_zone])
      expect(page.email).to eq(test_patient[:email_updated])
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)

      # Revert the mobile phone and email field values
      page.mobile_phone = test_patient[:mobile_phone_raw]
      page.email = test_patient[:email]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.email).to eq(test_patient[:email])

      # Save the account changes
      page.saveRegistration
      page.waitForLoadingToClear
      expect(page.success_msg_element.when_visible.text).to eq(account_changes_saved_message)

      # Sign out
      page.sign_out

      # Log back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Expand all the page sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.time_zone).to eq(test_patient[:time_zone])
      expect(page.email).to eq(test_patient[:email])

      # Cancel any account changes
      page.cancel_element.when_visible.click
      page = VetSuccessPage.new(@veteran_browser)
      page.wait_for_successpage_to_load

      # Sign out
      page.sign_out
      sleep(5)
    end
  end

  context 'Register a second new patient account' do
    it 'Register the new patient account' do
      # Set variables
      test_patient = patient56
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:middle_name] = @patient_data[test_patient[:user_name]]['middle_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:ssn] = @patient_data[test_patient[:user_name]]['ssn']
      test_patient[:dob] = @patient_data[test_patient[:user_name]]['dob']
      test_patient[:gender] = @patient_data[test_patient[:user_name]]['gender']

      # Log into the Veteran app
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetProfileEditPage.new(@veteran_browser)

      # Verify the visible page elements
      expect(page.submit_element.visible?).to be_true
      expect(page.cancel_element.visible?).to be_false
      expect(page.messages_element.visible?).to be_false
      expect(page.reports_element.visible?).to be_false
      expect(page.schedule_element.visible?).to be_false
      expect(page.account_element.visible?).to be_false
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true
      expect(page.user_nav).not_to be_nil

      # Verify the welcome message
      expect(page.user_nav_element.text).to eq('Welcome, ' << test_patient[:first_name].capitalize)

      # Expand the Patient Info section
      page.heading_patient_info

      # Verify the pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])

      # Set the mobile phone, home phone, time zone, and email fields
      page.home_phone_element.value = test_patient[:home_phone_raw]
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      page.mobile_phone_element.value = test_patient[:mobile_phone_raw]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      page.time_zone = test_patient[:time_zone]
      expect(page.time_zone_element.selected_options[0]).to eq(test_patient[:time_zone])
      page.email = test_patient[:email]
      expect(page.email).to eq(test_patient[:email])

      # Expand the Preferences section
      page.heading_preferences

      # Validate the basic cell phone settings
      page.select_basic_cell
      expect(page.basic_cell_selected?).to eq(true)
      expect(page.smart_phone_selected?).to eq(false)

      # Validate the smart phone settings
      page.select_smart_phone
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(false)
      page.check_receive_nonsecure_msgs
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(true)

      # Accept the patient consent form
      expect(page.select_patient_consent_chkbox_checked?).to be_false
      page.consent_form
      page.scroll_consent_form_down
      page.button_consent_yes_element.when_visible.click
      sleep(2)  # Allow the modal dialog to close
      expect(page.select_patient_consent_chkbox_checked?).to be_true

      # Save the registration form
      page.submitRegistration
      page = VetSuccessPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify that all the navigation links are now visible
      expect(page.messages_element.visible?).to be_true
      expect(page.reports_element.visible?).to be_true
      expect(page.schedule_element.visible?).to be_true
      expect(page.account_element.visible?).to be_true
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true

      # Sign out
      page.sign_out
      sleep(1)

      # Log back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Expand all the page sections
      page.heading_appointment_info_messages
      page.heading_preferences
      page.heading_patient_info

      # Verify the pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      expect(page.email).to eq(test_patient[:email])
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.receive_nonsecure_msgs_checked?).to eq(true)

      # Sign out
      page.sign_out
    end
  end

  context 'Duplicate mobile phone validation when self-updating veteran account information' do
    it 'Verify that duplicate mobile phone numbers are not allowed when self-updating veteran account information' do
      # Set variables
      test_patient = patient56

      # Open a new veteran browser
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Expand the Patient Info section
      page.heading_patient_info

      # Attempt to set a duplicate mobile phone number
      page.mobile_phone = mobile_phone_duplicate_raw
      page.saveRegistration
      page.waitForLoadingToClear
      expect(page.duplicate_mobile_phone_error).to eq(duplicate_mobile_number_error_message)

      # Sign out
      page.sign_out
    end
  end
end

describe "Self-Registered Patients in Patient Search Results (CMS-913 Part B)" do
  context "Search for newly-registered patients as a clinical staff user" do
    it "Searching for a newly-registered patient and validate the search results" do
      # Set variables
      test_patient = patient56
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:ssn] = @patient_data[test_patient[:user_name]]['ssn']
      test_patient[:dob] = @patient_data[test_patient[:user_name]]['dob']
      test_patient[:gender] = @patient_data[test_patient[:user_name]]['gender']
      test_patient[:full_name_display] = test_patient[:last_name] + ', ' + test_patient[:first_name]
      test_patient[:age] = calculate_age(test_patient[:dob])

      # Open new Staff browser
      page = PatientSearchPage.new(@browser)
      if !page.patient_search_element.visible?
        page = AdminPage.new(@browser)
        page.GoToStaffSearch
        page = PatientSearchPage.new(@browser)
        page.patient_search_element.click
        page.waitForPageToLoad
      end
      page.waitForLoadingToClear

      # Search for a patient
      page.patient_search_element.click
      page.search_patient(test_patient[:last_name])

      # Find the desired patient in the search results
      list = page.patient_list
      index = page.get_index_by_text(list, test_patient[:full_name_display])
      patient_result = page.patient_list[index]
      patient_result.wait_until(60, "Patient name not populated in 60 seconds.") do
        patient_result.name_element.text == test_patient[:full_name_display]
      end

      # Verify the displayed patient data
      expect(patient_result.name_element.text).to eq(test_patient[:full_name_display])
      expect(patient_result.dob.strip).to eq(test_patient[:dob])
      expect(patient_result.age.strip).to eq(test_patient[:age])
      expect(patient_result.mobile.strip).to eq('1-' + test_patient[:mobile_phone_formatted])
      expect(patient_result.gender.strip).to eq(test_patient[:gender])
      expect(patient_result.ssn.strip).to eq(test_patient[:ssn])
    end
  end
end