#########################################################################################
# Stories:   CMS-1174, CMS-2323
#
# Tests:   CMS-2136
#
# Acceptance Criteria:  CMS-2722, CMS-2723, CMS-2724, CMS-2726, CMS-2727
#
#
#########################################################################################
require 'spec_helper'

def validate_new_protocol_status(patient_name, search_result, is_enabled)
  page = PatientSearchPage.new(@browser)

  page.search_patient_and_select(patient_name, search_result)
  page = UpdatePatientProtocolListPage.new(@browser)

  expect(page.New_Protocol_element.enabled?).to eq(is_enabled)
end

def verify_schedule_is_removed
  page = VetMessagesPage.new(@veteran_browser)
  page = page.goto_schedule_page

  expect(page.schedule_heading).to eq('Schedule')
  expect(page.no_schedule_msg).to eq("I'm sorry, you currently do not have a schedule. I will contact you to let you know as soon as you are assigned a schedule.")

  page.goto_message_page
end

def verify_schedule_is_enabled
  page = VetMessagesPage.new(@veteran_browser)
  page = page.goto_schedule_page

  expect(page.schedule_heading).to eq('Schedule')
  expect(page.view_schedule_msg).to eq('View your schedule below and select to send a reading. If you forget I will remind you.')

  schedule = page.get_schedule_by_name('Blood Pressure')

  # Parse the due_at page element into an array for the purpose of validating the first element value
  schedule_due_at = schedule.due_at.split("\n")

  expect(schedule.name).to eq('Blood Pressure')
  expect(schedule_due_at[0]).to eq('Due at:')
  expect(schedule.time_element.visible?).to be_true
  expect(schedule.schedule_element.visible?).to be_true
  expect(schedule.date_element.visible?).to be_true

  page.goto_message_page
end

def validate_messages(messages)
  page = VetMessagesPage.new(@veteran_browser)
  page_wait = Selenium::WebDriver::Wait.new(:timeout => 200)

  index = 0
  messages.each do |message|
    message_text = message[:sent] ? message[:sent] : message[:received]
    schedule = page.message_list[index]

    page_wait.until { schedule.incoming_time_element.visible? }
    expect(schedule.incoming_time_element.text).to match(/\d+:\d{2}:\d{2}/)

    page_wait.until { schedule.incoming_message_element.visible? }
    expect(schedule.incoming_message).to eq(message_text)
    index += 1
  end
end

default_veteran_password = 'pass'
patient21 = {
    :user_name => 'zztest.patient21',
    :password => default_veteran_password,
    :home_phone_formatted => '703-765-9834',
    :mobile_phone_formatted => '703-111-2121',
    :time_zone => 'Eastern (GMT-05:00)',
    :email => 'zpatient@testmail.com'
}
patient21[:home_phone_raw] = patient21[:home_phone_formatted].delete '-'
patient21[:mobile_phone_raw] = patient21[:mobile_phone_formatted].delete '-'
patient55 = {
    :user_name => 'zztest.patient55',
    :password => default_veteran_password
}

describe 'Registration Messages (CMS-1174, CMS-2323)' do
  context 'as a veteran' do
    it 'register and select secure messaging preference' do
      # Set variables
      test_patient = patient21
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:middle_name] = @patient_data[test_patient[:user_name]]['middle_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:dob] = @patient_data[test_patient[:user_name]]['dob']
      test_patient[:gender] = @patient_data[test_patient[:user_name]]['gender']

      # Sign out
      page = VetMessagesPage.new(@veteran_browser)
      if page.user_nav_element.exists? then
        page.sign_out_element.when_present.click
      end
      
      # Sign in as a non-registered user
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify welcome message
      expect(page.user_nav_element.text).to eq('Welcome, ' << @patient_data[test_patient[:user_name]]['first_name'].capitalize)

      # Verify that expected navigation links and buttons are visible
      expect(page.submit_element.visible?).to be_true
      expect(page.messages_element.visible?).to be_false
      expect(page.reports_element.visible?).to be_false
      expect(page.schedule_element.visible?).to be_false
      expect(page.account_element.visible?).to be_false
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true
      expect(page.user_nav).to_not be_nil

      # Expand the Patient Info section
      page.heading_patient_info

      # Verify pre-populated data
      expect(page.firstname).to eq(test_patient[:first_name])
      expect(page.middlename).to eq(test_patient[:middle_name])
      expect(page.lastname).to eq(test_patient[:last_name])
      expect(page.dob).to eq(test_patient[:dob])
      expect(page.gender).to eq(test_patient[:gender])

      # Fill in the home & mobile phone, time zone, and email fields
      page.home_phone = test_patient[:home_phone_raw]
      expect(page.home_phone).to eq(test_patient[:home_phone_formatted])
      page.mobile_phone = test_patient[:mobile_phone_raw]
      expect(page.mobile_phone).to eq(test_patient[:mobile_phone_formatted])
      page.time_zone = test_patient[:time_zone]
      expect(page.time_zone_element.selected_options[0]).to eq(test_patient[:time_zone])
      page.email = test_patient[:email]
      expect(page.email).to eq(test_patient[:email])

      # Expand the Preferences section
      page.heading_preferences

      # Set the messaging preferences
      page.select_smart_phone
      expect(page.smart_phone_selected?).to eq(true)
      expect(page.basic_cell_selected?).to eq(false)
      expect(page.receive_nonsecure_msgs_checked?).to eq(false)

      # Accept the patient consent form
      expect(page.select_patient_consent_chkbox_checked?).to be_false
      page.consent_form
      page.scroll_consent_form_down
      page.button_consent_yes_element.when_visible.click
      sleep(2)  # Allow the modal dialog to close
      expect(page.select_patient_consent_chkbox_checked?).to be_true

      # Submit the registration page
      page.submitRegistration
      page = VetSuccessPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify top menu navigation items available for a registered user are visible
      expect(page.messages_element.visible?).to be_true
      expect(page.reports_element.visible?).to be_true
      expect(page.schedule_element.visible?).to be_true
      expect(page.account_element.visible?).to be_true
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true
    end
  end

  context 'as a clinical staff' do
    it 'search for a patient' do
      # Set variables
      test_patient = patient21
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Validate protocols
      validate_new_protocol_status(test_patient[:last_name], test_patient[:display_name], false)
    end
  end

  context 'as a veteran' do
    messages = [
      # The following line is temporarily commented out in order to resolve other issues
      # {received: 'Hi, this is Annie. Please text the word "START" to begin receiving messages from me. I look forward to hearing from you. http://bit.ly/1l4mk2m'},
      {sent: 'Help'},
      {received: %(These are commands to communicate with Annie\nStart = Start Annie\nStop = Stop Annie\nPause = Temporarily pause messages from Annie\nResume = Resume messages from Annie)},
      {sent: 'BP 164/93'},
      {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
      {sent: 'start'},
      {received: 'Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word "HELP".'},
      {sent: 'BP 164/93'},
      {received: 'Thank you. You are not subscribed to any protocols at this time.'},
      {sent: 'Stop'},
      {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
      {sent: 'BP 164/93'},
      {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
      {sent: 'Start'},
      {received: 'Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word "HELP".'},
      {sent: 'Pause'},
      {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting "Resume". I look forward to hearing from you.)},
      {sent: 'Stop'},
      {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
      {sent: 'Resume'},
      {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)}
    ]

    it 'send messages' do
      # Start on the Registration Success page
      page = VetSuccessPage.new(@veteran_browser)

      # Navigate to the Messages page
      page = page.goto_message_page

      # Send all messages
      messages.each do |message|
        if message[:sent]
          page.send_message(message[:sent])
        end
      end

      # Navigate away from and back to the Messages page
      page.schedule_element.click
      page.messages_element.click
    end

    it 'validate responses' do
      # Validate that the expected messages are present
      validate_messages(messages)
    end
  end


  context 'Registered veteran user with an assigned protocol' do
    it 'sign out' do
        # Log out to login as a non-registered user
        page = VetMessagesPage.new(@veteran_browser)
        page.sign_out_element.when_present.click
    end

    it 'Registration Messages - Registered User - Secure Messages (CMS-1174, CMS-2323)' do
        # Set variables
        test_patient = patient55

        # Login as a registered user
        page = VeteranLoginPage.new(@veteran_browser)
        page.login_with test_patient[:user_name], test_patient[:password]
        page = VetMessagesPage.new(@veteran_browser)
        page.waitForLoadingToClear

        # Validate schedule
        verify_schedule_is_enabled
    end
  end

  context 'as a clinical staff' do
    it "search for a patient #patient_username" do
      # Set variables
      test_patient = patient55
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Verify newly-assigned protocol is disabled since no START command was sent
      validate_new_protocol_status(test_patient[:last_name], test_patient[:display_name], true)
    end
  end

  context 'as a veteran' do
    messages = [
      {sent: 'Help'},
      {received: %(These are commands to communicate with Annie\nStart = Start Annie\nStop = Stop Annie\nPause = Temporarily pause messages from Annie\nResume = Resume messages from Annie)},
      {sent: 'BP 164/93'},
      {received: 'Your blood pressure is a little high today. Follow the advice in your management plan, and take the readings again at your usual time. Thanks.'},
      {sent: 'resume'},
      {received: %(I'm sorry; I don't recognize your command. Remember that you need to text me the "PAUSE" command before I can "RESUME" sending you messages. Thank you, Annie.)},
      {sent: 'pause'},
      {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting "Resume". I look forward to hearing from you.)},
      {sent: 'BP 164/93'},
      {received: %(Thank you. You are not subscribed to any protocols at this time.)},
      {sent: 'Pause'},
      {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting \"Resume\". I look forward to hearing from you.)},
      {sent: 'Resume'},
      {received: %(Welcome back! It is good to know that you have chosen to resume your messages. Let's get ready to get back on track.)},
      {sent: 'Stop'},
      {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
      {sent: 'BP 164/93'},
      {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
      {sent: 'Start'},
      {received: 'Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word "HELP".'}
    ]

    it "send and receive messages" do
      # Set variables
      test_patient = patient55
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Start on the Schedule page
      page = VetSchedulePage.new(@veteran_browser)

      # Navigate to the Messages page
      page = page.goto_message_page

      # Send all messages
      messages.each do |message|
        if message[:sent]
          page.send_message(message[:sent])

          # Verify Schedule(Protocol) is removed
          if (message[:sent] == 'pause') || (message[:sent] == 'Stop') then
            verify_schedule_is_removed
          end

          if (message[:sent] == 'pause') then
            # START: for PAUSE - validate Staff Assign New Protocol disabled
            validate_new_protocol_status(test_patient[:last_name], test_patient[:display_name], false)
          end

          #Verify Schedule(Protocol) is restored
          if (message[:sent] == 'Resume') then
            verify_schedule_is_enabled

            # START: for RESUME - validate Staff Assign New Protocol re-enabled
            validate_new_protocol_status(test_patient[:last_name], test_patient[:display_name], true)
          end
        end
      end

      # Navigate away from and back to the Messages page
      page.schedule_element.click
      page.messages_element.click
    end

    it 'validate responses' do
      # validate all sent and received messages
      validate_messages(messages)
    end

    it 'Re-enable Protocol' do
      # Set variables
      test_patient = patient55
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Verify the status of the new protocol
      validate_new_protocol_status(test_patient[:last_name], test_patient[:display_name], true)
      page = UpdatePatientProtocolListPage.new(@browser)

      # Enable a protocol
      page.enableProtocol('Blood Pressure Protocol1')
      page = UpdatePatientProtocolEditPage.new(@browser)

      # Save the changes to the protocol and dismiss the confirmation modal dialog
      page.protocol_save_top
      page.modalwindow_ok_element.when_visible.click
      page.waitForLoadingToClear

      # Navigate to the Patient Search page
      page.goto_patient_search_page
    end
  end
end

describe 'Message History (CMS-1174)' do
  def patient_message_history(search_param, patient_name)
    page = PatientSearchPage.new(@browser)
    page.search_patient_and_select(search_param, patient_name)
    page.message_history_element.click
    page = PatientMessageHistoryPage.new(@browser)
    page.waitForLoadingToClear
  end

  def validate_messages(messages)
    page = PatientMessageHistoryPage.new(@browser)

    index = 0
    pass = 0

    messages.each do |message|
      message_text = message[:sent] ? message[:sent] : message[:received]
      schedule = page.message_history_list[index]
      if (pass == 0)
        expect(schedule.message).to eq(message_text)
        pass += 1
      else
        expect(schedule.status).to eq(message_text)
        index += 1
        pass = 0
      end
    end
  end

  context 'Check messages in staff generated in Vet test 1174' do
    messages = [
        {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
        {sent: 'Sent'},
        {sent: 'Resume'},
        {sent: 'Received'},
        {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
        {sent: 'Sent'},
        {sent: 'Stop'},
        {sent: 'Received'},
        {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting "Resume". I look forward to hearing from you.)},
        {sent: 'Sent'},
        {sent: 'Pause'},
        {sent: 'Received'},
        {received: %(Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word "HELP".)},
        {sent: 'Sent'},
        {sent: 'Start'},
        {sent: 'Received'},
        {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
        {sent: 'Sent'},
        {received: %(BP 164/93)},
        {sent: 'Received'},
        {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
        {sent: 'Sent'},
        {received: %(Stop)},
        {sent: 'Received'},
        {received: %(Thank you. You are not subscribed to any protocols at this time.)},
        {sent: 'Sent'}
    ]

    it 'validate message history message for newly registered user' do
      # Set variables
      test_patient = patient21
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Navigate to the patient Message History page
      patient_message_history(test_patient[:last_name], test_patient[:display_name])

      # Validate all sent and received messages
      validate_messages(messages)

      # Navigate back to the Patient Search page
      page = PatientMessageHistoryPage.new(@browser)
      page.goto_patient_search_page
      page = PatientSearchPage.new(@browser)
      page.waitForLoadingToClear
    end
  end

  context 'Check messages in staff generated in previous test steps' do
    messages = [
        {received: %(Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word "HELP".)},
        {sent: 'Sent'},
        {sent: 'Start'},
        {sent: 'Received'},
        {received: %(I'm sorry; I cannot recognize your message. Please remember that you need to text me the word "START" in order for me to appropriately respond to your messages.)},
        {sent: 'Sent'},
        {received: %(BP 164/93)},
        {sent: 'Received'},
        {received: %(I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text "START" to receive messages again. Thank you, Annie.)},
        {sent: 'Sent'},
        {received: %(Stop)},
        {sent: 'Received'},
        {received: %(Welcome back! It is good to know that you have chosen to resume your messages. Let's get ready to get back on track.)},
        {sent: 'Sent'},
        {received: %(Resume)},
        {sent: 'Received'},
        {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting "Resume". I look forward to hearing from you.)},
        {sent: 'Sent'},
        {received: %(Pause)},
        {sent: 'Received'},
        {received: %(Thank you. You are not subscribed to any protocols at this time.)},
        {sent: 'Sent'},
        {received: %(BP 164/93)},
        {sent: 'Received'},
        {received: %(You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting "Resume". I look forward to hearing from you.)},
        {sent: 'Sent'}
    ]

    it 'validate message history message for registered user' do
      # Set variables
      test_patient = patient55
      test_patient[:first_name] = @patient_data[test_patient[:user_name]]['first_name']
      test_patient[:last_name] = @patient_data[test_patient[:user_name]]['last_name']
      test_patient[:display_name] = test_patient[:last_name] + ', ' + test_patient[:first_name]

      # Verify the patient message history
      patient_message_history(test_patient[:last_name], test_patient[:display_name])

      # Validate all sent and received messages
      validate_messages(messages)
    end
  end

  context 'Reset patient55 vet schedule' do
    it 'sign out' do
      # Sign out
      page = VetMessagesPage.new(@veteran_browser)
      page.sign_out_element.when_present.click
    end

    it 'Login as patient55' do
      # Set variables
      test_patient = patient55

      # Sign in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with test_patient[:user_name], test_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Send the START command
      page.message_text = 'start'
      page.message_send
    end
  end
end
