#########################################################################################
# Stories:   CMS-1174, CMS-2323
#
# Tests:   CMS-2136
#
# Acceptance Criteria:  CMS-2722, CMS-2723, CMS-2724, CMS-2726, CMS-2727
#
#
#########################################################################################
require 'spec_helper'

default_veteran_password = 'pass'
patient55 = {
    :user_name => 'zztest.patient55',
    :password => default_veteran_password
}
patient57 = {
    :user_name => 'zztest.patient57',
    :password => default_veteran_password
}

describe 'Registration Schedule (CMS-2291)' do
  context "Registration schedule for patient registration app" do
    it "Verify the registration schedule" do
      # Start on the Messages page
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the navigation header
      expect(page.messages_element.visible?).to be_true
      expect(page.reports_element.visible?).to be_true
      expect(page.schedule_element.visible?).to be_true
      expect(page.account_element.visible?).to be_true
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true

      # Sign out
      page.sign_out

      # Sign back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with patient57[:user_name], patient57[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Schedule page
      page.schedule_element.when_visible.click
      page = VetSchedulePage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the expected page text
      expect(page.schedule_heading).to eq('Schedule')
      expect(page.no_schedule_msg).to eq('I\'m sorry, you currently do not have a schedule. I will contact you to let you know as soon as you are assigned a schedule.')

      # Sign out
      page.sign_out

      # Sign in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with patient55[:user_name], patient55[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Schedule page
      page.schedule_element.when_visible.click
      page = VetSchedulePage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the expected page text
      expect(page.schedule_heading).to eq('Schedule')
      expect(page.view_schedule_msg).to eq('View your schedule below and select to send a reading. If you forget I will remind you.')

      # Grab the desired schedule
      schedule = page.get_schedule_by_name('Blood Pressure')

      # Verify the schedule details
      expect(schedule.name).to eq('Blood Pressure')
      expect(schedule.due_at.split("\n")[0]).to eq('Due at:')
      expect(schedule.time_element.visible?).to be_true
      expect(schedule.schedule_element.visible?).to be_true
      expect(schedule.date_element.visible?).to be_true
    end
  end
end
