#########################################################################################
# Stories:   CMS-1174, CMS-2323
#
# Tests:   CMS-2136
#
# Acceptance Criteria:  CMS-2722, CMS-2723, CMS-2724, CMS-2726, CMS-2727
#
#
#########################################################################################
require 'spec_helper'

default_patient_password = 'pass'

describe 'Registration Reports (CMS-2293, CMS-2476)' do
  context "Registration reports for patient registration app" do
    it "Open the Reports page" do
      # Set variables
      current_patient = {
          :user_name => @current_patient_username,
          :password => default_patient_password
      }
      patient1 = {
          :user_name => 'zztest.patient01',
          :password => default_patient_password
      }

      # Start on the Messages page
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Account page
      page.account_element.when_visible.click
      page = VetProfileEditPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the navigation links
      expect(page.messages_element.visible?).to be_true
      expect(page.reports_element.visible?).to be_true
      expect(page.schedule_element.visible?).to be_true
      expect(page.account_element.visible?).to be_true
      expect(page.about_element.visible?).to be_true
      expect(page.sign_out_element.visible?).to be_true

      # Sign out
      page.sign_out
      sleep(5)

      # Sign back in
      page = VeteranLoginPage.new(@veteran_browser)
      page.login_with current_patient[:user_name], current_patient[:password]
      page = VetMessagesPage.new(@veteran_browser)

      # Navigate to the Reports page
      page.reports_element.when_visible.click
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the expected page title is displayed
      expect(page.reports_heading).to eq('Reports')

      # Sign out
      page.sign_out
      page = VeteranLoginPage.new(@veteran_browser)

      # Sign back in as a different user
      page.login_with patient1[:user_name], patient1[:password]
      page = VetMessagesPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Navigate to the Reports page
      page.reports_element.when_visible.click
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the expected reports are displayed
      expect(page.reports_list[0].report_link_element.text).to eq('Weight Loss')
      expect(page.reports_list[1].report_link_element.text).to eq('Pulse Oximetry (SpO2)')
      expect(page.reports_list[2].report_link_element.text).to eq('Blood Pressure')
    end

    it "Validate the contents of the vet reports table for Weight Loss" do
      # Start on the Reports page
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # View the first report
      page.reports_list[0].report_link
      page = VetReportsPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the data on the first page of the report
      expect(page.reading_table_element.rows).to eq(31)
      expect(page.readings[0].date_element.text).to eq('03/05/2014 11:30:00')
      expect(page.reading_table_value_element(1).text).to eq('183')
      expect(page.readings[1].date_element.text).to eq('03/05/2014 12:30:00')
      expect(page.reading_table_value_element(2).text).to eq('182')
      expect(page.readings[2].date_element.text).to eq('03/05/2014 13:30:00')
      expect(page.reading_table_value_element(3).text).to eq('181')
      expect(page.readings[3].date_element.text).to eq('03/05/2014 14:30:00')
      expect(page.reading_table_value_element(4).text).to eq('180')
      expect(page.readings[4].date_element.text).to eq('03/05/2014 15:30:00')
      expect(page.reading_table_value_element(5).text).to eq('179')
      expect(page.readings[5].date_element.text).to eq('03/05/2014 16:30:00')
      expect(page.reading_table_value_element(6).text).to eq('178')

      # Navigate to the second page of the report
      page.pagination_element.list_item_element(:text => '2').link_element.click

      # Verify the data on the second page of the report
      expect(page.reading_table_element.rows).to eq(8)
      expect(page.readings[0].date_element.text).to eq('03/08/2014 21:30:00')
      expect(page.reading_table_value_element(1).text).to eq('177')
      expect(page.readings[1].date_element.text).to eq('03/08/2014 22:30:00')
      expect(page.reading_table_value_element(2).text).to eq('175')
      expect(page.readings[2].date_element.text).to eq('03/08/2014 23:30:00')
      expect(page.reading_table_value_element(3).text).to eq('176')
      expect(page.readings[3].date_element.text).to eq('03/09/2014 09:30:00')
      expect(page.reading_table_value_element(4).text).to eq('175')
      expect(page.readings[4].date_element.text).to eq('03/10/2014 13:30:00')
      expect(page.reading_table_value_element(5).text).to eq('172')
      expect(page.readings[5].date_element.text).to eq('03/11/2014 20:30:00')
      expect(page.reading_table_value_element(6).text).to eq('173')
      expect(page.readings[6].date_element.text).to eq('03/12/2014 11:30:00')
      expect(page.reading_table_value_element(7).text).to eq('174')

      # Navigate back to the Reports page
      page.back_button
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear
    end

    it "Validate the contents of the vet reports table for Pulse Oximetry (SPO2)" do
      # Start on the Reports page
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # View the second report
      page.reports_list[1].report_link
      page = VetReportsPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the data in the report
      expect(page.reading_table_element.rows).to eq(5)
      expect(page.readings[0].date_element.text).to eq('03/19/2014 10:30:00')
      expect(page.reading_table_value_element(1).text).to eq('70')
      expect(page.readings[1].date_element.text).to eq('03/19/2014 10:45:00')
      expect(page.reading_table_value_element(2).text).to eq('77')
      expect(page.readings[2].date_element.text).to eq('03/19/2014 12:30:00')
      expect(page.reading_table_value_element(3).text).to eq('99')
      expect(page.readings[3].date_element.text).to eq('03/21/2014 10:30:00')
      expect(page.reading_table_value_element(4).text).to eq('89')

      # Navigate back to the Reports page
      page.back_button
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear
    end

    it "Validate the contents of the vet reports table for Blood Pressure" do
      # Start on the Reports page
      page = VetReportsListPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # View the third report
      page.reports_list[2].report_link
      page = VetReportsPage.new(@veteran_browser)
      page.waitForLoadingToClear

      # Verify the data in the report
      expect(page.reading_table_element.rows).to eq(8)
      expect(page.readings[0].date_element.text).to eq('03/19/2014 10:00:00')
      expect(page.reading_table_value_element(1).text).to eq('130/85')
      expect(page.readings[1].date_element.text).to eq('03/20/2014 15:00:00')
      expect(page.reading_table_value_element(2).text).to eq('132/83')
      expect(page.readings[2].date_element.text).to eq('03/21/2014 08:12:00')
      expect(page.reading_table_value_element(3).text).to eq('116/90')
      expect(page.readings[3].date_element.text).to eq('03/21/2014 09:00:00')
      expect(page.reading_table_value_element(4).text).to eq('145/87')
      expect(page.readings[4].date_element.text).to eq('03/21/2014 10:41:00')
      expect(page.reading_table_value_element(5).text).to eq('137/91')
      expect(page.readings[5].date_element.text).to eq('03/22/2014 07:00:00')
      expect(page.reading_table_value_element(6).text).to eq('139/80')
      expect(page.readings[6].date_element.text).to eq('03/22/2014 11:00:00')
      expect(page.reading_table_value_element(7).text).to eq('120/77')
    end
  end
end

