#########################################################################################
# Stories:   CMS-2244
#
# Tests:  CMS-3737
#
# Acceptance Criteria:  CMS-3327
#
#
#########################################################################################
require 'spec_helper'

describe 'Validate Staff/AdminSession Time Out Warnings and Logout' do

  context 'Validate the session timeout warning messages on the staff/admin application' do

    it 'login to the staff/admin application' do
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page
    end

    it 'wait for 12 minutes to validate first session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(740, 3)
    end


    it 'wait for another minute (13) to validate second session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      #The application needs to wait a set amount of time in order to verify the dynamic portion of the dialog box changes value
      sleep(70)
      page.validateSessionWarningMessage(2)
    end

    it 'wait for another minute (14) to validate the final session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      #The application needs to wait a set amount of time in order to verify the dynamic portion of the dialog box changes value
      sleep(70)
      page.validateSessionWarningMessage(1)
    end

    it 'click the continue button and resume session' do
      page = GlobalObjectsPage.new(@browser)
      page.continue

    end

    it 'wait for another 12 minutes to validate first session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(745, 3)
    end

    it 'click the logout button and log back in' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(1, 3)
      page.logout

      page = VALoginPage.new(@browser)

      page.login_with @current_staff_username, @current_staff_password

    end

    it 'wait for the logout message to display on the staff side, verify and click login' do
      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page

      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidateTimeout(915)
      page.login

      page = VALoginPage.new(@browser)

      page.login_with @current_staff_username, @current_staff_password

    end

    it 'switch to admin view' do
      page = PatientSearchPage.new(@browser)
      page.goto_admin_page

      page = AdminPage.new(@browser)
      page.protocols_tab_element.visible?

    end

    it 'wait for 12 minutes on the admin side to validate first session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(740, 3)
    end

    it 'wait for another minute (13) on the admin side to validate second session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      #The application needs to wait a set amount of time in order to verify the dynamic portion of the dialog box changes value
      sleep(70)
      page.validateSessionWarningMessage(2)
    end

    it 'wait for another minute (14) on the admin side to validate the final session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      #The application needs to wait a set amount of time in order to verify the dynamic portion of the dialog box changes value
      sleep(70)
      page.validateSessionWarningMessage(1)
    end

    it 'click the continue button and resume session' do
      page = GlobalObjectsPage.new(@browser)
      page.continue

      page = AdminPage.new(@browser)

    end

    it 'wait for another 12 minutes on the admin side to validate first session timeout warning message' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(745, 3)
    end

    it 'click the logout button and log back in' do
      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidate(1, 3)
      page.logout

      page = VALoginPage.new(@browser)

      page.login_with @current_staff_username, @current_staff_password

      page = PatientSearchPage.new(@browser)
      page.goto_patient_search_page

    end

    it 'switch to admin view again' do
      page = PatientSearchPage.new(@browser)

      if page.patient_search_element.visible? then
        page.goto_admin_page
      end

      page = AdminPage.new(@browser)
      page.protocols_tab_element.visible?

    end

    it 'wait for the logout message to display on the admin side, verify and click login' do
      page = AdminPage.new(@browser)
      page.protocols_tab_element.visible?

      page = GlobalObjectsPage.new(@browser)
      page.waitAndValidateTimeout(915)
      page.login

      page = VALoginPage.new(@browser)

      page.login_with @current_staff_username, @current_staff_password

    end
  end
end
