# -*- encoding : utf-8 -*-
# CMS UI tests
#
# bootstrap code and helper functions
# File Utils is for folder items
require 'rubygems'
require 'bundler/setup'
require 'fileutils'
require 'date'


$LOAD_PATH.unshift(File.dirname(__FILE__))
$LOAD_PATH.unshift(File.join(File.dirname(__FILE__), ',,', 'lib'))

require 'rspec'
require 'rspec/expectations'
require 'yaml'
require 'selenium-webdriver'
require 'page-object'
require 'page-object/page_factory'
require 'benchmark'
require 'browser_helper'
require 'require_all'

require_all 'lib'

configs = YAML::load File.open('configuration.yml')
browser = nil
veteran_browser = nil

test_data = YAML.load File.open('test-data.yml')

RSpec.configure do |config|

  if configs['DEFAULT_BROWSER']
    browser_type = configs['DEFAULT_BROWSER']
  else
    raise %q[Configuration error: No default browser defined. Please define DEFAULT_BROWSER in your local configuration.yml!]
  end

  if configs['STAFF_USERNAME']
    username = configs['STAFF_USERNAME']
  else
    raise %q[Configuration error: Staff user username is not defined.]
  end

  if configs['STAFF_PASSWORD']
    password = configs['STAFF_PASSWORD']
  else
    raise %q[Configuration error: Staff user password is not defined.]
  end

  if configs['STAFF_FULL_NAME']
    full_name = configs['STAFF_FULL_NAME']
  else
    raise %q[Configuration error: Staff user full name is not defined.]
  end

  if configs['PATIENT_USERNAME']
    patient_username = configs['PATIENT_USERNAME']
  else
    raise %q[Configuration error: Patient user full name is not defined.]
  end

  if configs['PATIENT_PASSWORD']
    patient_password = configs['PATIENT_PASSWORD']
  else
    raise %q[Configuration error: Patient user full name is not defined.]
  end

  config.profile_examples = configs['CONSOLE_LOG']

  config.include PageObject::PageFactory

  if browser_type == 'firefox'
    firefox_paths = %x[mdfind "kMDItemFSName = Firefox.app"]
    firefox_path = firefox_paths.split('\n').first.chomp
    firefox_executable_path = File.join(firefox_path, '/Contents/MacOS/firefox')

    Selenium::WebDriver::Firefox::Binary.path = firefox_executable_path
  end

  config.before(:all) do
    # default timeout is 5 sec, use a longer timeout especially for MAE environment
    PageObject.default_page_wait = 60
    PageObject.default_element_wait = 90

    test_helper = BrowserHelper.new
    test_helper.console_log = configs['CONSOLE_LOG']
    test_helper.browser_type = browser_type

    test_helper.username = patient_username
    test_helper.password = patient_password

    veteran_browser = test_helper.open_veteran_browser
    # time = Benchmark.realtime { veteran_browser = test_helper.open_veteran_browser }
    #
    # puts "Open veteran browser: time elapsed #{time} seconds" if configs['CONSOLE_LOG']


    test_helper.username = username
    test_helper.password = password
    browser = test_helper.open_staff_browser
    # time = Benchmark.realtime { browser = test_helper.open_staff_browser }
    #
    # puts "Open staff browser: time elapsed #{time} seconds" if configs['CONSOLE_LOG']
  end

  config.before(:each) do
    @patient_data = test_data['patients']
    @staff_data = test_data['staff']
    @current_patient_username = patient_username
    @current_staff_username = username
    @current_staff_password = password
    @browser = browser
    @veteran_browser = veteran_browser

    #Patient Data Info
    @full_name = full_name
    @patient01Name = 'patient01, zztest'
    @patient01DOB = '05/05/1900'
    @patient01Age = calculate_age(@patient01DOB)
    @patient01Mobile = '1-571-050-0001'
    @Patient01Gender = 'Male'
    @patient01SSN = '999-99-9901'

    @patient02Name = 'patient02, zztest'
    @patient02DOB = '10/05/1900'
    @patient02Age = calculate_age(@patient02DOB)
    @patient02Mobile = '1-703-333-4444'
    @Patient02Gender = 'Male'
    @patient02SSN = '999-99-9902'

    @patient03Name = 'patient03, zztest'
    @patient03DOB = '10/05/1900'
    @patient03Age = calculate_age(@patient03DOB)
    @patient03Mobile = '1-571-050-0003'
    @Patient03Gender = 'Male'
    @patient03SSN = '999-99-9903'

    @patient04Name = 'patient04, zztest'
    @patient05Name = 'patient05, zztest'
    @patient06Name = 'patient06, zztest'
    @patient18Name = 'patient18, zztest'
    @patient20Name = 'patient20, zztest'
    @patient21Name = 'patient21, zztest'
    @patient22Name = 'patient22, zztest'
    @patient23Name = 'patient23, zztest'
    @patient24Name = 'patient24, zztest'

    @patient55Name = 'patient55, zztest'
    @patient55DOB = '05/05/1900'
    @patient55Age = calculate_age(@patient55DOB)
    @patient55Mobile = '1-703-111-0055'
    @Patient55Gender = 'Male'
    @patient55SSN = '999-99-9955'

    #Patients with active mobile numbers
    @patient83Name = 'patient83, zztest'
    @patient83DOB = '01/09/1900'
    @patient83Age = calculate_age(@patient83DOB)
    @patient83Mobile = '1-571-294-3179'
    @Patient83Gender = 'Male'
    @patient83SSN = '999-99-9983'

    @patient84Name = 'patient84, zztest'
    @patient84DOB = '01/10/1900'
    @patient84Age = calculate_age(@patient84DOB)
    @patient84Mobile = '1-571-239-6757'
    @Patient84Gender = 'Male'
    @patient84SSN = '999-99-9984'
  end

  config.after(:each) do
    if example.exception

      # Clear out Errors Folder
        # FileUtils.rm_rf 'errors'
      # make new Errors Folder
        # Dir.mkdir('errors')
      # Formatting to remove colons from date
      # errorOutputTimeStamp = Time.now.strftime("%Y%m%d%H")
      # if Dir.exists?("errors/#{errorOutputTimeStamp}")
      # else
      #   Dir.mkdir("errors/#{errorOutputTimeStamp}")
      # end

      meta = example.metadata
      filename = File.basename(meta[:file_path])
      line_number = meta[:line_number]
      screenshot_name = "screenshot-#{filename}-#{line_number}"
      staff_screenshot_path = "#{screenshot_name}-staff.png" #errors/#{errorOutputTimeStamp}/
      veteran_screenshot_path = "#{screenshot_name}-vet.png" #errors/#{errorOutputTimeStamp}/


      browser.save_screenshot(staff_screenshot_path)
      veteran_browser.save_screenshot(veteran_screenshot_path)

      puts meta[:full_description] + "\n  Screenshot: #{staff_screenshot_path}  #{veteran_screenshot_path}"
    end
  end

  config.after(:all) do
    veteran_browser.quit if veteran_browser
    browser.quit if browser
  end

end

##############################################################################
##### This function subtracts two date fields (System date & Patient DOB)
##### and returns an integer value.
##############################################################################
def difference_in_years(s1, s2)
  d1, d2 = Date.parse(s1), Date.parse(s2)
  ((d1 - d2) / 365.0).to_i
end

# Calculates age based on date of birth
# Pre:  dob is a date string of format MM/DD/YYYY
# Post: A string is returned with the age in years
def calculate_age(dob)
  age = Date.today.year - dob.split('/')[2].to_i
  if (Date.today.month < dob.split('/')[0].to_i) || ((Date.today.month == dob.split('/')[0].to_i) && (Date.today.day < dob.split('/')[1].to_i)) then
    age = age - 1
  end

  age.to_s
end

# Simulates pressing the down arrow key on the keyboard the specified number of times focused on the specified target
# Pre:  target_object is a valid browser element
# Post: The down arrow key has been pressed count times focused on the target_object element
def press_down_arrow_key(target_object, count=1)
  count.times {
    target_object.send_keys :arrow_down
    sleep(0.25)
  }
end