package com.agilex.bcm.framework.base;

import com.agilex.bcm.framework.pages.external.StaffLogin;
import com.agilex.bcm.framework.pages.staff.AdminProtocols;
import com.agilex.bcm.framework.pages.staff.BcmVaFacilityMessages;
import com.agilex.bcm.framework.pages.staff.StaffPatientSearch;
import com.agilex.common.selenium.DriverUtil;
import com.agilex.common.selenium.WebSite;

public class AnnieStaffPage extends BcmWebPage {
    private static final String PAGE_TITLE = "Annie";

    // Navigation links
    private static final String LINK_WELCOME = "id=nav-user";
    private static final String LINK_WELCOME_BROADCAST_ADMIN = "id=nav-bcm-view";
    private static final String LINK_WELCOME_CLINICAL_ADMIN = "id=nav-admin-view";
    private static final String LINK_WELCOME_CLINICAL_STAFF = "id=nav-staff-view";
    private static final String LINK_WELCOME_LOG_OUT = "id=nav-sign-out";
    private static final String LINK_ABOUT = "id=nav-about";

    public AnnieStaffPage(DriverUtil driverutil, WebSite webSite) {
        super(driverutil, webSite);
    }

    @Override
    public String getPageTitle() {
        return PAGE_TITLE;
    }

//-------------------------Page Class Methods-----------------------------------

    public void clickWelcomeMenu() {
        driverutil.click(LINK_WELCOME);
        driverutil.waitForVisibleElement(LINK_WELCOME_LOG_OUT, defaultTimeOutSeconds);
    }

    public boolean isBroadcastAdminLinkVisible() {
        return driverutil.isElementVisible(LINK_WELCOME_BROADCAST_ADMIN);
    }

    public boolean isClinicalAdminLinkVisible() {
        return driverutil.isElementVisible(LINK_WELCOME_CLINICAL_ADMIN);
    }

    public boolean isClinicalStaffLinkVisible() {
        return driverutil.isElementVisible(LINK_WELCOME_CLINICAL_STAFF);
    }

    public void clickBroadcastAdminLink() {
        driverutil.click(LINK_WELCOME_BROADCAST_ADMIN);
    }

    public void clickClinicalAdminLink() {
        driverutil.click(LINK_WELCOME_CLINICAL_ADMIN);
    }

    public void clickClinicalStaffLink() {
        driverutil.click(LINK_WELCOME_CLINICAL_STAFF);
    }

    public StaffLogin clickLogOutLink() {
        driverutil.click(LINK_WELCOME_LOG_OUT);
        return (StaffLogin) webSite.setCurrentPage(StaffLogin.class);
    }

    public BcmVaFacilityMessages switchToBroadcastAdminView() {
        clickWelcomeMenu();
        driverutil.waitForVisibleElement(LINK_WELCOME_BROADCAST_ADMIN, defaultTimeOutSeconds);
        clickBroadcastAdminLink();
        return (BcmVaFacilityMessages) webSite.setCurrentPage(BcmVaFacilityMessages.class);
    }

    public AdminProtocols switchToClinicalAdminView() {
        clickWelcomeMenu();
        driverutil.waitForVisibleElement(LINK_WELCOME_CLINICAL_ADMIN, defaultTimeOutSeconds);
        clickClinicalAdminLink();
        return (AdminProtocols) webSite.setCurrentPage(AdminProtocols.class);
    }

    public StaffPatientSearch switchToClinicalStaffView() {
        clickWelcomeMenu();
        driverutil.waitForVisibleElement(LINK_WELCOME_CLINICAL_STAFF, defaultTimeOutSeconds);
        clickClinicalStaffLink();
        return (StaffPatientSearch) webSite.setCurrentPage(StaffPatientSearch.class);
    }

    public StaffLogin logOut() {
        clickWelcomeMenu();
        driverutil.waitForVisibleElement(LINK_WELCOME_LOG_OUT, defaultTimeOutSeconds);
        return clickLogOutLink();
    }

    public BcmVaFacilityMessages kickToVaFacilityMessagesPage() {
        return (BcmVaFacilityMessages) webSite.setCurrentPage(BcmVaFacilityMessages.class);
    }

//-------------------------Internal Helper Methods------------------------------

 
}