package com.agilex.bcm.framework.base;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.NoSuchElementException;
import java.util.concurrent.TimeUnit;
import com.agilex.common.selenium.DriverUtil;
import com.agilex.common.selenium.WebPage;
import com.agilex.common.selenium.WebSite;

public abstract class BcmWebPage extends WebPage {
	public static BcmDeployment currentEnvironment = new BcmDeployment();
    public final static int defaultTimeOutSeconds = 30;

    private static final String LOADING_ANIMATION = "id=loading-text";
	
	public BcmWebPage(DriverUtil driverutil, WebSite webSite) {
		super(driverutil, webSite);
	}


    public void waitForLoadingToClear() {
        try {
            while(driverutil.isElementVisible(LOADING_ANIMATION)) {
                waitSeconds(1);
            }
        } catch(Throwable t) {
            // Do nothing
        }
    }
	
	public void waitSeconds(int seconds) {
		try {
			TimeUnit.SECONDS.sleep(seconds);
		} catch(InterruptedException ex) {
			Thread.currentThread().interrupt();
		}
	}

    public boolean isDateWithinRange(String value, String lowerLimit, String upperLimit) {
        String expectedDateFormat = "MM/dd/yyyy";
        String parseError = String.format("The date string '%s' could not be parsed, must be of format '%s'", value, expectedDateFormat);
        SimpleDateFormat dateFormatter = new SimpleDateFormat(expectedDateFormat);
        Date dateValue, dateLowerLimit, dateUpperLimit;

        // Parse the date to be validated
        try { dateValue = dateFormatter.parse(value); }
        catch(ParseException pe) { throw new RuntimeException(parseError); }

        // Parse the lower limit date
        try { dateLowerLimit = dateFormatter.parse(lowerLimit); }
        catch(ParseException pe) { throw new RuntimeException(parseError); }

        // Parse the upper limit date
        try { dateUpperLimit = dateFormatter.parse(upperLimit); }
        catch(ParseException pe) { throw new RuntimeException(parseError); }

        return isDateWithinRange(dateValue, dateLowerLimit, dateUpperLimit);
    }

    public boolean isDateWithinRange(Date value, Date lowerLimit, Date upperLimit) {
        return (value.compareTo(lowerLimit) >= 0) && (value.compareTo(upperLimit) <= 0);
    }
}
