package com.agilex.bcm.framework.pages.external;

import com.agilex.bcm.framework.base.BcmWebPage;
import com.agilex.bcm.framework.pages.staff.StaffPatientSearch;
import com.agilex.common.selenium.DriverUtil;
import com.agilex.common.selenium.WebSite;

public class StaffLogin extends BcmWebPage {
    private static final String PAGE_TITLE = "Department of Veterans Affairs - Provider Login Page";

    // Login form elements
    private static final String FIELD_ACCESS_CODE = "id=name-c";
    private static final String FIELD_VERIFY_CODE = "id=password";
    private static final String FIELD_STATION = "id=va-hospital";
    private static final String LIST_STATION_AUTOCOMPLETE_LIST = "domclass=autocomplete-dropdown-wrapper";
    private static final String LISTITEM_STATION_AUTOCOMPLETE_ATTRIBUTE = "data-facility-name";
    private static final String BUTTON_CLEAR = "id=resetLogin";
    private static final String BUTTON_SIGN_IN = "id=loginButton";

    public StaffLogin(DriverUtil driverutil, WebSite webSite) {
        super(driverutil, webSite);
    }

    @Override
    public String getPageTitle() {
        return PAGE_TITLE;
    }

//-------------------------Public Class Methods-----------------------------------

    public void selectStation(String station) {
        driverutil.setElementValue(FIELD_STATION, station);
        driverutil.waitForVisibleElement(LIST_STATION_AUTOCOMPLETE_LIST, 120);
        driverutil.click(createElementLocator(LISTITEM_STATION_AUTOCOMPLETE_ATTRIBUTE, station));
    }

    public StaffPatientSearch logInAs(String accessCode, String verifyCode, String station) {
        waitForLoginForm();
        driverutil.setElementValue(FIELD_ACCESS_CODE, accessCode);
        driverutil.setElementValue(FIELD_VERIFY_CODE, verifyCode);
        selectStation(station);
        driverutil.click(BUTTON_SIGN_IN);
        return (StaffPatientSearch) webSite.setCurrentPage(StaffPatientSearch.class);
    }

    public void waitForLoginForm() {
        driverutil.waitForVisibleElement(FIELD_ACCESS_CODE, defaultTimeOutSeconds);
    }

    public void clickClearButton() {
        driverutil.click(BUTTON_CLEAR);
    }

//-------------------------Internal Helper Methods------------------------------

 
}