package com.agilex.bcm.framework.pages.staff;

import com.agilex.bcm.framework.base.AnnieStaffPage;
import com.agilex.bcm.framework.base.BcmAdminPage;
import com.agilex.bcm.framework.base.FacilityMessageModel;
import com.agilex.bcm.framework.modals.BcmFacilityMessageCancellation;
import com.agilex.bcm.framework.modals.BcmFacilityMessageConfirmation;
import com.agilex.common.helpers.TimeOfDay;
import com.agilex.common.helpers.ValueUnits;
import com.agilex.common.selenium.DriverUtil;
import com.agilex.common.selenium.WebSite;

import java.util.*;

public class BcmAddMessage extends BcmAdminPage {
    private static final String PAGE_TITLE = "Annie";

    public static final String OPTION_NONE = "?";
    public static final String OPTION_UNDEFINED = "? undefined:undefined ?";

    // Welcome header elements
    private static final String HEADER_TITLE = "id=header-add-message";
    public static final String HEADER_TITLE_ADD_TEXT = "Add Message";
    public static final String HEADER_TITLE_EDIT_TEXT = "Edit Message";

    // Message form content elements
    private static final String ELEMENT_CREATED_BY_CREATED_ON = "id=add-message-createdBy";
    private static final String ELEMENT_CREATED_BY = "id=message[]-createdBy";
    private static final String ELEMENT_CREATED_ON = "id=message[]-createdDate";
    private static final String FIELD_TITLE = "id=title";
    private static final String FIELD_MESSAGE = "id=message";

    // Message form scheduling elements
    private static final String SELECT_SCHEDULE = "id=schedule";
    private static final Map<String, String> OPTIONS_SCHEDULE = new LinkedHashMap<String, String>(0);
    static {
        OPTIONS_SCHEDULE.put(EMPTY_STRING, OPTION_NONE);
        OPTIONS_SCHEDULE.put("Once", "0");
        OPTIONS_SCHEDULE.put("Specified Days", "1");
        OPTIONS_SCHEDULE.put("Custom", "2");
    }

    private static final Map<String, String> CHECKS_SCHEDULE_DAYS = new HashMap<String, String>(0);
    static {
        CHECKS_SCHEDULE_DAYS.put("Monday", "id=monday");
        CHECKS_SCHEDULE_DAYS.put("Tuesday", "id=tuesday");
        CHECKS_SCHEDULE_DAYS.put("Wednesday", "id=wednesday");
        CHECKS_SCHEDULE_DAYS.put("Thursday", "id=thursday");
        CHECKS_SCHEDULE_DAYS.put("Friday", "id=friday");
        CHECKS_SCHEDULE_DAYS.put("Saturday", "id=saturday");
        CHECKS_SCHEDULE_DAYS.put("Sunday", "id=sunday");
    }

    private static final String FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE = "id=time-between-each-cycle";
    private static final String SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS = "id=custom-schedule";
    private static final List<String> OPTIONS_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS = Arrays.asList(new String[] {
            OPTION_UNDEFINED,
            "MINUTE",
            "HOUR",
            "DAY",
            "WEEK",
            "MONTH",
            "YEAR"
    });

    private static final String SELECT_SCHEDULE_SEND_MESSAGE_TIME_HOURS = "id=start-time-hour";
    private static final String SELECT_SCHEDULE_SEND_MESSAGE_TIME_MINUTES = "id=minutes";
    private static final String SELECT_SCHEDULE_SEND_MESSAGE_TIME_AMPM = "id=AMPM";
    private static final ArrayList<String> OPTIONS_SCHEDULE_SEND_MESSAGE_TIME_AMPM = new ArrayList<String>(0);
    static {
        OPTIONS_SCHEDULE_SEND_MESSAGE_TIME_AMPM.add(OPTION_UNDEFINED);
        OPTIONS_SCHEDULE_SEND_MESSAGE_TIME_AMPM.add("AM");
        OPTIONS_SCHEDULE_SEND_MESSAGE_TIME_AMPM.add("PM");
    }

    private static final String FIELD_START_DATE = "id=start-date";
    private static final String FIELD_END_DATE = "id=end-date";

    // Message form buttons
    private static final String BUTTON_SAVE = "id=save-top";
    private static final String BUTTON_CANCEL = "id=cancel";

    // Message form validation message elements
    private static final String VALIDATION_HEADER = "//*[contains(@class, 'error-summary')]";
    private static final String VALIDATION_ERROR_HEADER = "We're sorry, there was an error with the broadcast message submission. Please fill in the required fields.";

    private static final String VALIDATION_FIELD_TITLE = "id=title-error";
    private static final String VALIDATION_ERROR_FIELD_TITLE_EMPTY = "Please enter a Title";

    private static final String VALIDATION_FIELD_MESSAGE = "id=message-error";
    private static final String VALIDATION_ERROR_FIELD_MESSAGE_EMPTY = "Please enter a Message";

    private static final String VALIDATION_SELECT_SCHEDULE = "id=schedule-error";
    private static final String VALIDATION_ERROR_SELECT_SCHEDULE_EMPTY = "Please select a Schedule";

    private static final String VALIDATION_FIELD_SCHEDULE_SEND_MESSAGE_TIME = "id=send-message-time-error";
    private static final String VALIDATION_ERROR_SCHEDULE_FIELD_SEND_MESSAGE_TIME_EMPTY = "Please select a time for Send Message Time";

    private static final String VALIDATION_CHECKS_SCHEDULE_DAYS = "id=select-days-error";
    private static final String VALIDATION_ERROR_CHECKS_SCHEDULE_DAYS_EMPTY = "Please select Days that Apply";
    private static final String VALIDATION_ERROR_CHECKS_SCHEDULE_DAYS_OUT_OF_RANGE = "The day(s) selected must be within the Start Date and End Date range";

    private static final String VALIDATION_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE = "id=time-between-each-cycle-error";
    private static final String VALIDATION_ERROR_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE_EMPTY = "Please enter Time Between Each Cycle";

    private static final String VALIDATION_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS = "id=time-between-cycletime-error";
    private static final String VALIDATION_ERROR_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS_EMPTY = "Please select Time Between Each Cycle";

    private static final String VALIDATION_FIELD_START_DATE = "id=start-date-error";
    private static final String VALIDATION_ERROR_FIELD_START_DATE_EMPTY = "Please select a Start Date";
    private static final String VALIDATION_ERROR_FIELD_START_DATE_BEFORE_TODAY = "The Start Date CANNOT be before today's date";
    private static final String VALIDATION_ERROR_FIELD_START_DATE_AFTER_END_DATE = "The Start Date CANNOT be after the End Date";
    private static final String VALIDATION_ERROR_FIELD_START_DATE_INVALID_FORMAT = "Please enter a valid date format (MM/DD/YYYY)";

    private static final String VALIDATION_FIELD_END_DATE = "id=end-date-error";
    private static final String VALIDATION_ERROR_FIELD_END_DATE_EMPTY = "Please select an End Date";
    private static final String VALIDATION_ERROR_FIELD_END_DATE_BEFORE_START_DATE = "The End Date CANNOT be before the Start Date";
    private static final String VALIDATION_ERROR_FIELD_END_DATE_INVALID_FORMAT = "Please enter a valid date format (MM/DD/YYYY)";

    public BcmAddMessage(DriverUtil driverutil, WebSite webSite) {
        super(driverutil, webSite);
    }

    @Override
    public String getPageTitle() {
        return PAGE_TITLE;
    }

    // ------------------------------ Page Class Methods ------------------------------

    public void waitForAddTitleHeader() {
        driverutil.waitForVisibleText(HEADER_TITLE, HEADER_TITLE_ADD_TEXT, defaultTimeOutSeconds);
    }

    public void waitForEditTitleHeader() {
        driverutil.waitForVisibleText(HEADER_TITLE, HEADER_TITLE_EDIT_TEXT, defaultTimeOutSeconds);
    }

    // ------------------------------ Message form field methods ------------------------------




    // Message Created By and Created On methods
    public String getMessageCreatedBy() {
        driverutil.waitForVisibleElement(ELEMENT_CREATED_BY, defaultTimeOutSeconds);
        return driverutil.getText(ELEMENT_CREATED_BY);
    }

    public String getMessageCreatedOn() {
        driverutil.waitForVisibleElement(ELEMENT_CREATED_ON, defaultTimeOutSeconds);
        return driverutil.getText(ELEMENT_CREATED_ON);
    }

    public boolean isMessageCreatedByValueCorrect(String value) {
        return getMessageCreatedBy().toUpperCase().equals(value.toUpperCase());
    }

    public boolean isMessageCreatedOnValueCorrect(String value) {
        return getMessageCreatedOn().equals(value);
    }


    // Message Title field methods
    public void setMessageTitleValue(String value) {
        driverutil.setElementValue(FIELD_TITLE, value);
    }

    public String getMessageTitleValue() { return driverutil.getElementValue(FIELD_TITLE); }

    public boolean isMessageTitleValueCorrect(String value) {
        return getMessageTitleValue().equals(value);
    }


    // Message Message field methods
    public void setMessageMessageValue(String value) {
        driverutil.clearElementValue(FIELD_MESSAGE);
        driverutil.setElementValue(FIELD_MESSAGE, value);
    }

    public String getMessageMessageValue() { return driverutil.getElementValue(FIELD_MESSAGE); }

    public boolean isMessageMessageValueCorrect(String value) {
        return getMessageMessageValue().equals(value);
    }


    // Message schedule methods
    public boolean areMessageScheduleOptionsCorrect() {
        return areExpectedSelectOptionsPresent(SELECT_SCHEDULE, OPTIONS_SCHEDULE);
    }

    public String getMessageScheduleValue() { return driverutil.getElementValue(SELECT_SCHEDULE); }

    public void setMessageScheduleByValue(String value) {
        driverutil.selectOptionByValue(SELECT_SCHEDULE, OPTIONS_SCHEDULE.get(value));
        waitSeconds(1);
    }

    public boolean isMessageScheduleValueCorrect(String value) {
        String expectedValue;

        if(value.equals("?")) { expectedValue = value; }
        else { expectedValue = OPTIONS_SCHEDULE.get(value); }

        return driverutil.getElementValue(SELECT_SCHEDULE).equals(expectedValue);
    }

    public boolean areScheduleSpecifiedDaysCheckboxesVisible() {
        for(String dayCheckbox : CHECKS_SCHEDULE_DAYS.values()) {
            try { if(!driverutil.isElementVisible(dayCheckbox)) { return false; } }
            catch(NoSuchElementException nsee) { return false; }
        }

        return true;
    }

    public void setScheduleSpecifiedDaysCheckboxes(Map<String, Boolean> daysToCheck) {
        if(areScheduleSpecifiedDaysCheckboxesVisible()) {  // The specified days checkboxes are visible
            for(Map.Entry<String, Boolean> dayEntry : daysToCheck.entrySet()) {
                // Check to see if the day selection is different from that day's checkbox state
                if(driverutil.isElementSelected(CHECKS_SCHEDULE_DAYS.get(dayEntry.getKey())) != dayEntry.getValue()) {
                    // Toggle the checkbox state to match the day selection
                    driverutil.click(CHECKS_SCHEDULE_DAYS.get(dayEntry.getKey()));
                }
            }
        } else {  // The specified days checkboxes are NOT visible
            throw new RuntimeException("The specified days cannot be set because the fields are not visible");
        }
    }

    public boolean areScheduleSpecifiedDaysCheckboxesCorrect(Map<String, Boolean> daysToCheck) {
        if(areScheduleSpecifiedDaysCheckboxesVisible()) {
            for(Map.Entry<String, Boolean> dayEntry : daysToCheck.entrySet()) {
                if(driverutil.isElementSelected(CHECKS_SCHEDULE_DAYS.get(dayEntry.getKey())) != dayEntry.getValue()) {
                    return false;
                }
            }
        } else {
            throw new RuntimeException("The specified days cannot be validated because the fields are not visible");
        }

        return true;
    }

    public boolean areScheduleTimeBetweenCyclesFieldsVisible() {
        try {
            return (driverutil.isElementVisible(FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE) &&
                    driverutil.isElementVisible(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS));
        } catch(NoSuchElementException nsee) {
            return false;
        }
    }

    public String getScheduleTimeBetweenCycleValue() {
        return driverutil.getElementValue(FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE);
    }

    public String getScheduleTimeBetweenCycleUnit() {
      return driverutil.getElementValue(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS);
    }

    public boolean areScheduleTimeBetweenCyclesUnitsOptionsCorrect() {
        return areExpectedSelectOptionsPresent(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS,
                OPTIONS_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS);
    }

    public void setScheduleTimeBetweenCyclesValues(ValueUnits timeBetweenCycles) {
        String unitsOptionValue;

        // Make sure the Schedule Time Between Each Cycle fields are displayed
        driverutil.waitForVisibleElement(FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE, defaultTimeOutSeconds);
        driverutil.waitForVisibleElement(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS, defaultTimeOutSeconds);

        // Set the Time Between Each Cycle Value value
        driverutil.setElementValue(FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE, timeBetweenCycles.getValue());

        // Set the Time Between Each Cycle Units value
        if(timeBetweenCycles.getUnits().equals(EMPTY_STRING)) {
            driverutil.selectOptionByIndex(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS, 0);
        } else {
            driverutil.selectOptionByValue(SELECT_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS, timeBetweenCycles.getUnits().toUpperCase());
        }
    }

    public boolean areScheduleTimeBetweenCyclesCorrect(ValueUnits timeBetweenCycles) {
        if(areScheduleTimeBetweenCyclesFieldsVisible()) {
            if(!getScheduleTimeBetweenCycleValue().equals(timeBetweenCycles.getValue())) { return false; }
            if(!getScheduleTimeBetweenCycleUnit().equals(timeBetweenCycles.getUnits())) { return false; }
        } else {
            throw new RuntimeException("The time between cycles cannot be validated because the fields are not visible");
        }

        return true;
    }


    // Message send time hours methods
    public boolean areMessageSendMessageTimeHoursOptionsCorrect() {
        ArrayList<String> expectedOptions = new ArrayList<String>(0);

        // Generate the list of expected options
        expectedOptions.add(OPTION_NONE);
        for(int i=0; i<12; i++) {
            expectedOptions.add(Integer.toString(i));
        }

        return areExpectedSelectOptionsPresent(SELECT_SCHEDULE_SEND_MESSAGE_TIME_HOURS, expectedOptions);
    }

    public void setMessageSendMessageTimeHoursByValue(String value) {
        driverutil.selectOptionByValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_HOURS, Integer.toString(Integer.parseInt(value) - 1));
    }

    public String getMessageSendMessageTimeHoursValue() {
        String gatheredValue = driverutil.getElementValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_HOURS);

        if(gatheredValue.equals(OPTION_NONE)) {
            return gatheredValue;
        } else {
            return Integer.toString(Integer.parseInt(gatheredValue) + 1);
        }
    }

    public boolean isMessageSendMessageTimeHoursValueCorrect(String value) {
        return getMessageSendMessageTimeHoursValue().equals(value);
    }


    // Message send time minutes methods
    public boolean areMessageSendMessageTimeMinutesOptionsCorrect() {
        ArrayList<String> expectedOptions = new ArrayList<String>(0);

        // Generate the list of expected options
        expectedOptions.add(OPTION_NONE);
        for(int i=0; i<=59; i++) {
            expectedOptions.add(Integer.toString(i));
        }

        return areExpectedSelectOptionsPresent(SELECT_SCHEDULE_SEND_MESSAGE_TIME_MINUTES, expectedOptions);
    }

    public void setMessageSendMessageTimeMinutesByValue(String value) {
        driverutil.selectOptionByValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_MINUTES, value);
    }

    public String getMessageSendMessageTimeMinutesValue() {
        return driverutil.getElementValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_MINUTES);
    }

    public boolean isMessageSendMessageTimeMinutesValueCorrect(String value) {
        return getMessageSendMessageTimeMinutesValue().equals(value);
    }


    // Message send time AM/PM methods
    public boolean areMessageSendMessageTimeAmPmOptionsCorrect() {
        return areExpectedSelectOptionsPresent(SELECT_SCHEDULE_SEND_MESSAGE_TIME_AMPM,
                OPTIONS_SCHEDULE_SEND_MESSAGE_TIME_AMPM);
    }

    public void setMessageSendMessageTimeAmPmByValue(String value) {
        driverutil.selectOptionByValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_AMPM, value);
    }

    public String getMessageSendMessageTimeAmPmValue() {
        return driverutil.getElementValue(SELECT_SCHEDULE_SEND_MESSAGE_TIME_AMPM);
    }

    public boolean isMessageSendMessageTimeAmPmValueCorrect(String value) {
        return getMessageSendMessageTimeAmPmValue().equals(value);
    }


    // Message send time collection
    public boolean isMessageSendMessageTimeCorrect(TimeOfDay value) {
        if(!isMessageSendMessageTimeHoursValueCorrect(value.getHours())) { return false; }
        if(!isMessageSendMessageTimeMinutesValueCorrect(value.getMinutes())) { return false; }
        if(!isMessageSendMessageTimeAmPmValueCorrect(value.getAmPm())) { return false; }

        return true;
    }


    // Message start date field methods
    public void setMessageStartDateValue(String value) {
        driverutil.setElementValue(FIELD_START_DATE, value);
    }

    public String getMessageStartDateValue() {
        return driverutil.getElementValue(FIELD_START_DATE);
    }

    public boolean isMessageStartDateValueCorrect(String value) {
        return getMessageStartDateValue().equals(value);
    }


    // Message end date field methods
    public void setMessageEndDateValue(String value) { driverutil.setElementValue(FIELD_END_DATE, value); }

    public String getMessageEndDateValue() { return driverutil.getElementValue(FIELD_END_DATE); }

    public boolean isMessageEndDateValueCorrect(String value) { return getMessageEndDateValue().equals(value); }


    // Form save button methods
    public void clickSaveButton() { driverutil.click(BUTTON_SAVE); }

    public BcmFacilityMessageConfirmation saveMessageExpectSuccess() {
        clickSaveButton();
        return (BcmFacilityMessageConfirmation) webSite.setCurrentPage(BcmFacilityMessageConfirmation.class);
    }

    public BcmAddMessage saveMessageExpectValidationError() {
        clickSaveButton();
        return (BcmAddMessage) webSite.setCurrentPage(BcmAddMessage.class);
    }


    // Form cancel button methods
    public void clickCancelButton() { driverutil.click(BUTTON_CANCEL); }

    public BcmFacilityMessageCancellation cancelMessageCreation() {
        clickCancelButton();
        return (BcmFacilityMessageCancellation) webSite.setCurrentPage(BcmFacilityMessageCancellation.class);
    }


    // ------------------------------ Form completion methods ------------------------------

    public void setMessageSendMessageTimeValues(TimeOfDay timeToSet) {
        setMessageSendMessageTimeHoursByValue(timeToSet.getHours());
        setMessageSendMessageTimeMinutesByValue(timeToSet.getMinutes());
        setMessageSendMessageTimeAmPmByValue(timeToSet.getAmPm());
    }

    public void fillOutMessageFields(FacilityMessageModel messageToAdd) {
        // Set non-conditional values
        setMessageTitleValue(messageToAdd.title);
        setMessageMessageValue(messageToAdd.message);
        setMessageScheduleByValue(messageToAdd.schedule);
        setMessageStartDateValue(messageToAdd.startDate);
        setMessageEndDateValue(messageToAdd.endDate);
        setMessageSendMessageTimeHoursByValue(messageToAdd.sendMessageTime.getHours());
        setMessageSendMessageTimeMinutesByValue(messageToAdd.sendMessageTime.getMinutes());
        setMessageSendMessageTimeAmPmByValue(messageToAdd.sendMessageTime.getAmPm());

        // Set the schedule values
        if(messageToAdd.schedule.equals("Specified Days")) {
            setScheduleSpecifiedDaysCheckboxes(messageToAdd.scheduleSpecifiedDays);
        } else if(messageToAdd.schedule.equals("Custom")) {
            setScheduleTimeBetweenCyclesValues(messageToAdd.scheduleTimeBetweenCycles);
        }
    }


    // ------------------------------ Message form validation message methods ------------------------------

    public boolean isMessageHeaderValidationShown() {
        return driverutil.getText(VALIDATION_HEADER).contains(VALIDATION_ERROR_HEADER);
    }

    public boolean isMessageTitleEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_TITLE).contains(VALIDATION_ERROR_FIELD_TITLE_EMPTY);
    }

    public boolean isMessageMessageEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_MESSAGE).contains(VALIDATION_ERROR_FIELD_MESSAGE_EMPTY);
    }

    public boolean isMessageScheduleEmptyValidationShown() {
        return driverutil.getText(VALIDATION_SELECT_SCHEDULE).contains(VALIDATION_ERROR_SELECT_SCHEDULE_EMPTY);
    }

    public boolean isMessageScheduleSendMessageTimeEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_SCHEDULE_SEND_MESSAGE_TIME).contains(VALIDATION_ERROR_SCHEDULE_FIELD_SEND_MESSAGE_TIME_EMPTY);
    }

    public boolean isMessageScheduleDaysEmptyValidationShown() {
        return driverutil.getText(VALIDATION_CHECKS_SCHEDULE_DAYS).contains(VALIDATION_ERROR_CHECKS_SCHEDULE_DAYS_EMPTY);
    }

    public boolean isMessageScheduleDaysOutOfRangeValidationShown() {
        return driverutil.getText(VALIDATION_CHECKS_SCHEDULE_DAYS).contains(VALIDATION_ERROR_CHECKS_SCHEDULE_DAYS_OUT_OF_RANGE);
    }

    public boolean isMessageScheduleTimeBetweenCyclesValueEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE).contains(
                VALIDATION_ERROR_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_VALUE_EMPTY);
    }

    public boolean isMessageScheduleTimeBetweenCyclesUnitsEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS).contains(
                VALIDATION_ERROR_FIELD_SCHEDULE_TIME_BETWEEN_CYCLES_UNITS_EMPTY);
    }

    public boolean isMessageStartDateEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_START_DATE).contains(VALIDATION_ERROR_FIELD_START_DATE_EMPTY);
    }

    public boolean isMessageStartDateBeforeTodayValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_START_DATE).contains(VALIDATION_ERROR_FIELD_START_DATE_BEFORE_TODAY);
    }

    public boolean isMessageStartDateAfterEndDateValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_START_DATE).contains(VALIDATION_ERROR_FIELD_START_DATE_AFTER_END_DATE);
    }

    public boolean isMessageStartDateInvalidFormatValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_START_DATE).contains(VALIDATION_ERROR_FIELD_START_DATE_INVALID_FORMAT);
    }

    public boolean isMessageEndDateEmptyValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_END_DATE).contains(VALIDATION_ERROR_FIELD_END_DATE_EMPTY);
    }

    public boolean isMessageEndDateBeforeStartDateValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_END_DATE).contains(VALIDATION_ERROR_FIELD_END_DATE_BEFORE_START_DATE);
    }

    public boolean isMessageEndDateInvalidFormatValidationShown() {
        return driverutil.getText(VALIDATION_FIELD_END_DATE).contains(VALIDATION_ERROR_FIELD_END_DATE_INVALID_FORMAT);
    }

//-------------------------Internal Helper Methods------------------------------

 
}
