package com.agilex.bcm.framework.pages.staff;

import com.agilex.bcm.framework.base.AnnieStaffPage;
import com.agilex.bcm.framework.base.BcmAdminPage;
import com.agilex.common.selenium.DriverUtil;
import com.agilex.common.selenium.WebSite;
import com.agilex.bcm.framework.modals.BcmFacilityMessageDisable;
import com.agilex.bcm.framework.modals.BcmFacilityMessageEnable;
import com.agilex.common.selenium.WebTestingException;
import java.util.*;

public class BcmVaFacilityMessages extends BcmAdminPage {
    private static final String PAGE_TITLE = "Annie";

    // Welcome header elements
    private static final String HEADER_TITLE = "id=header-vamc-facility-messages";
    public static final String HEADER_TITLE_TEXT = "VA Facility Broadcast Messages";

    // Navigation button elements
    private static final String BUTTON_ADD_MESSAGE = "id=vamc-add-message";

    // Message filtering elements
    private static final String FIELD_FROM = "id=message-date-from";
    private static final String FIELD_TO = "id=message-date-to";
    private static final String SELECT_PROGRESS = "id=filter-status";
    private static final List<String> OPTIONS_PROGRESS = Arrays.asList(new String[] { "0", "1", "2", "3" });
    private static final String SELECT_VIEW = "id=filter-view";
    private static final List<String> OPTIONS_VIEW = Arrays.asList(new String[] { "0", "1", "2", "3" });
    private static final String BUTTON_RESET = "id=reset";

    // Message list entries
    private static final String MESSAGE_ENTRY_BASE = "id=message[";
    private static final String MESSAGE_ENTRY_SUBSTRING_TITLE = "]-title";
    private static final String MESSAGE_ENTRY_SUBSTRING_CREATED_BY = "]-createdBy";
    private static final String MESSAGE_ENTRY_SUBSTRING_CREATED_ON = "]-createdOn";
    private static final String MESSAGE_ENTRY_SUBSTRING_PROGRESS = "]-status";
    private static final String MESSAGE_ENTRY_SUBSTRING_EDITED_BY = "]-editedBy";
    private static final String MESSAGE_ENTRY_SUBSTRING_EDITED_ON = "]-editedOn";
    private static final String MESSAGE_ENTRY_SUBSTRING_EDITED_TIME = "]-editedTime";
    private static final String MESSAGE_ENTRY_SUBSTRING_EDIT_VIEW_BUTTON = "]-edit";
    private static final String MESSAGE_ENTRY_SUBSTRING_ENABLED_RADIO = "].enable";
    private static final String MESSAGE_ENTRY_SUBSTRING_DISABLED_RADIO = "].disable";

    // No messages listed elements
    private static final String ELEMENT_DATA_NOT_FOUND = "id=no-results";
    private static final String ELEMENT_DATA_NOT_FOUND_TITLE = "Data Not Found";
    private static final String ELEMENT_DATA_NOT_FOUND_MESSAGE = "Please search again. We could not find any data based on your search criteria.";

    // Pagination
    private static final String LINK_PAGE_BASE = "//*[@ng-click='selectPage(page.number)' and contains(text(), '";
    private static final String LINK_PAGE_TAIL = "')]";

    // Page constants
    private static final int MAX_MESSAGES_PER_PAGE = 20;

    public BcmVaFacilityMessages(DriverUtil driverutil, WebSite webSite) {
        super(driverutil, webSite);
    }

    @Override
    public String getPageTitle() {
        return PAGE_TITLE;
    }

//-------------------------Page Class Methods-----------------------------------

    public void waitForTitleHeader() {
        driverutil.waitForVisibleText(HEADER_TITLE, HEADER_TITLE_TEXT, defaultTimeOutSeconds);
    }

    public BcmAddMessage clickAddMessageButton() {
        driverutil.click(BUTTON_ADD_MESSAGE);
        return (BcmAddMessage) webSite.setCurrentPage(BcmAddMessage.class);
    }

    // Filtering form element methods
    public void setCreatedFromValue(String valueToSet) {
        driverutil.click(FIELD_FROM);
        driverutil.setElementValue(FIELD_FROM, valueToSet);
        driverutil.hitEnterKey();
    }

    public String getCreatedFromValue() { return driverutil.getElementValue(FIELD_FROM); }

    public void setCreatedToValue(String valueToSet) {
        driverutil.click(FIELD_TO);
        driverutil.setElementValue(FIELD_TO, valueToSet);
        driverutil.hitEnterKey();
    }

    public String getCreatedToValue() { return driverutil.getElementValue(FIELD_TO); }

    public boolean areProgressOptionsCorrect() {
        return areExpectedSelectOptionsPresent(SELECT_PROGRESS, OPTIONS_PROGRESS);
    }

    public void setProgressByValue(String optionToSelect) {
        int optionIndex;

        if(optionToSelect.equals("All")) { optionIndex = 0; }
        else if(optionToSelect.equals("Pending Delivery")) { optionIndex = 1; }
        else if(optionToSelect.equals("Delivery In Progress")) { optionIndex = 2; }
        else if(optionToSelect.equals("Delivered")) { optionIndex = 3; }
        else { throw new RuntimeException(String.format("Invalid option of '%d'", optionToSelect)); }

        driverutil.selectOptionByValue(SELECT_PROGRESS, OPTIONS_PROGRESS.get(optionIndex));
    }

    public String getProgressValue() {
        String optionValue = driverutil.getElementValue(SELECT_PROGRESS), parsedOptionValue;

        if(optionValue.equals("0")) { parsedOptionValue = "All"; }
        else if(optionValue.equals("1")) { parsedOptionValue = "Pending Delivery"; }
        else if(optionValue.equals("2")) { parsedOptionValue = "Delivery In Progress"; }
        else if(optionValue.equals("3")) { parsedOptionValue = "Delivered"; }
        else { throw new RuntimeException(String.format("Invalid option of '%s'", optionValue)); }

        return parsedOptionValue;
    }

    public boolean areViewOptionsCorrect() {
        return areExpectedSelectOptionsPresent(SELECT_VIEW, OPTIONS_VIEW);
    }

    public void setViewByValue(String optionToSelect) {
        int optionIndex;

        if(optionToSelect.equals("All")) { optionIndex = 0; }
        else if(optionToSelect.equals("Created By Me")) { optionIndex = 1; }
        else if(optionToSelect.equals("Enabled")) { optionIndex = 2; }
        else if(optionToSelect.equals("Disabled")) { optionIndex = 3; }
        else { throw new RuntimeException(String.format("Invalid option of '%d'", optionToSelect)); }

        driverutil.selectOptionByValue(SELECT_VIEW, OPTIONS_VIEW.get(optionIndex));
    }

    public String getViewValue() { return driverutil.getElementValue(SELECT_VIEW); }

    public void clickResetButton() {
        driverutil.click(BUTTON_RESET);
    }


    // Message entry element locator methods
    public String getMessageEntryFieldLocator(int messageIndex, String fieldSubstring) {
        return String.format("%s%d%s", MESSAGE_ENTRY_BASE, messageIndex, fieldSubstring);
    }

    public String getMessageEntryTitleLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_TITLE);
    }

    public String getMessageEntryCreatedByLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_CREATED_BY);
    }

    public String getMessageEntryCreatedOnLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_CREATED_ON);
    }

    public String getMessageEntryProgressLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_PROGRESS);
    }

    public String getMessageEntryEditedByLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_EDITED_BY);
    }

    public String getMessageEntryEditedOnLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_EDITED_ON);
    }

    public String getMessageEntryEditedTimeLocator(int messageIndex) {
        return getMessageEntryFieldLocator(messageIndex, MESSAGE_ENTRY_SUBSTRING_EDITED_TIME);
    }

    public String getMessageEntryEditViewButtonLocator(int messageIndex) {
        return String.format("%s%d%s", MESSAGE_ENTRY_BASE, messageIndex, MESSAGE_ENTRY_SUBSTRING_EDIT_VIEW_BUTTON);
    }

    public String getMessageEntryEnabledRadioLocator(int messageIndex) {
        return String.format("%s%d%s", MESSAGE_ENTRY_BASE, messageIndex, MESSAGE_ENTRY_SUBSTRING_ENABLED_RADIO);
    }

    public String getMessageEntryDisabledRadioLocator(int messageIndex) {
        return String.format("%s%d%s", MESSAGE_ENTRY_BASE, messageIndex, MESSAGE_ENTRY_SUBSTRING_DISABLED_RADIO);
    }


    // Message entry element methods
    public boolean doesMessageEntryExist(int messageIndex) {
        try { return driverutil.isElementVisible(getMessageEntryTitleLocator(messageIndex)); }
        catch(NoSuchElementException nsee) { return false; }
    }

    public String getMessageEntryTitle(int messageIndex) {
        return driverutil.getText(getMessageEntryTitleLocator(messageIndex));
    }

    public String getMessageEntryCreatedBy(int messageIndex) {
        return driverutil.getText(getMessageEntryCreatedByLocator(messageIndex));
    }

    public String getMessageEntryCreatedOn(int messageIndex) {
        return driverutil.getText(getMessageEntryCreatedOnLocator(messageIndex));
    }

    public String getMessageEntryProgress(int messageIndex) {
        return driverutil.getText(getMessageEntryProgressLocator(messageIndex));
    }

    public int getMessageEntryIndexByTitle(String messageTitle) {
        int messageIndex = 0, pageIndex = 1;
        RuntimeException messageNotFoundException = new RuntimeException(String.format("The message with title '%s' was not found", messageTitle));

        if(doesMessageEntryExist(messageIndex)) {
            if(getMessageEntryTitle(messageIndex).equals(messageTitle)) { return messageIndex; }
            else { messageIndex++; }
        } else {
            if(messageIndex == MAX_MESSAGES_PER_PAGE) {
                pageIndex++;
                if(doesPageLinkExist(pageIndex)) {
                    goToPage(pageIndex);
                    messageIndex = 0;
                } else {
                    throw messageNotFoundException;
                }
            } else {
                throw messageNotFoundException;
            }
        }

        // Shouldn't ever reach here, but return an invalid message index if we do
        return -1;
    }

    public boolean isMessageEntryEnabled(int messageIndex) {
        return driverutil.isElementSelected(getMessageEntryEnabledRadioLocator(messageIndex));
    }

    public boolean isMessageEntryDisabled(int messageIndex) {
        return driverutil.isElementSelected(getMessageEntryDisabledRadioLocator(messageIndex));
    }

    public BcmFacilityMessageDisable disableMessage(int messageIndex) {
        driverutil.click(getMessageEntryDisabledRadioLocator(messageIndex));
        return (BcmFacilityMessageDisable) webSite.setCurrentPage(BcmFacilityMessageDisable.class);
    }

    public BcmFacilityMessageEnable enableMessage(int messageIndex) {
        driverutil.click(getMessageEntryEnabledRadioLocator(messageIndex));
        return (BcmFacilityMessageEnable) webSite.setCurrentPage(BcmFacilityMessageEnable.class);
    }

    public BcmAddMessage editMessage(int messageIndex) {
        driverutil.click(getMessageEntryEditViewButtonLocator(messageIndex));
        return (BcmAddMessage) webSite.setCurrentPage(BcmAddMessage.class);
    }


    // Message count methods
    public int countMessageEntriesOnPage() {
        int messageIndex = 0;

        while(doesMessageEntryExist(messageIndex)) {
            messageIndex++;
        }

        return messageIndex;
    }

    public boolean isMessageEntryCountWithinBounds() {
        int messageCount = countMessageEntriesOnPage();
        if((messageCount >= 0) && (messageCount <= MAX_MESSAGES_PER_PAGE)) { return true; }
        else { return false; }
    }


    // Message entry validations against filtering options
    public boolean doMessageCreatedOnValuesMatchFiltering() {
        int messageIndex = 0, pageIndex = 1;
        boolean areAllMessagesChecked = false;

        while(!areAllMessagesChecked) {
            if(doesMessageEntryExist(messageIndex)) {
                if(isDateWithinRange(getMessageEntryCreatedOn(messageIndex), getCreatedFromValue(), getCreatedToValue())) {
                    messageIndex++;
                } else {
                    return false;
                }
            } else {
                if(messageIndex == MAX_MESSAGES_PER_PAGE) {
                    pageIndex++;
                    if(doesPageLinkExist(pageIndex)) {
                        goToPage(pageIndex);
                        messageIndex = 0;
                    } else {
                        areAllMessagesChecked = true;
                    }
                } else {
                    areAllMessagesChecked = true;
                }
            }
        }

        return areAllMessagesChecked;
    }

    public boolean doMessageProgressValuesMatchFiltering() {
        int messageIndex = 0, pageIndex = 1;
        boolean areAllMessagesChecked = false;

        if(!getProgressValue().equals("All")) {
            while(!areAllMessagesChecked) {
                if(doesMessageEntryExist(messageIndex)) {
                    if(getMessageEntryProgress(messageIndex).equals(getProgressValue())) { messageIndex++; } else {
                        return false;
                    }
                } else {
                    if(messageIndex == MAX_MESSAGES_PER_PAGE) {
                        pageIndex++;
                        if(doesPageLinkExist(pageIndex)) {
                            goToPage(pageIndex);
                            messageIndex = 0;
                        } else {
                            areAllMessagesChecked = true;
                        }
                    } else {
                        areAllMessagesChecked = true;
                    }
                }
            }
        } else {
            // All Progress values are valid
            return true;
        }

        // Shouldn't ever reach here, but filtering is correct if we do
        return areAllMessagesChecked;
    }

    public boolean doMessageCreatedByValuesMatchUser(String userName) {
        int messageIndex = 0, pageIndex = 1;
        boolean areAllMessagesChecked = false;

        while(!areAllMessagesChecked) {
            if(doesMessageEntryExist(messageIndex)) {
                if(getMessageEntryCreatedBy(messageIndex).toUpperCase().equals(userName.toUpperCase())) {
                    messageIndex++;
                } else {
                    return false;
                }
            } else {
                if(messageIndex == MAX_MESSAGES_PER_PAGE) {
                    pageIndex++;
                    if(doesPageLinkExist(pageIndex)) {
                        goToPage(pageIndex);
                        messageIndex = 0;
                    } else {
                        areAllMessagesChecked = true;
                    }
                } else {
                    areAllMessagesChecked = true;
                }
            }
        }

        // Filtering is correct
        return true;
    }


    // No messages match filters message
    public boolean isDataNotFoundMessageShown() {
        driverutil.waitForVisibleElement(ELEMENT_DATA_NOT_FOUND, defaultTimeOutSeconds);
        String textFromPage = driverutil.getText(ELEMENT_DATA_NOT_FOUND);

        try {
            return textFromPage.contains(ELEMENT_DATA_NOT_FOUND_TITLE) && textFromPage.contains(ELEMENT_DATA_NOT_FOUND_MESSAGE);
        } catch(WebTestingException wte) {
            return false;
        }
    }


    // Pagination methods
    public String getPageLinkElement(int pageIndex) {
        return String.format("%s%d%s", LINK_PAGE_BASE, pageIndex, LINK_PAGE_TAIL);
    }

    public boolean doesPageLinkExist(int pageIndex) {
        try { return driverutil.isElementVisible(getPageLinkElement(pageIndex)); }
        catch(NoSuchElementException nsee) { return false; }
    }

    public BcmVaFacilityMessages goToPage(int pageIndex) {
        driverutil.click(getPageLinkElement(pageIndex));
        return (BcmVaFacilityMessages) webSite.setCurrentPage(BcmVaFacilityMessages.class);
    }

//-------------------------Internal Helper Methods------------------------------

 
}