package com.agilex.bcm.tests.structure;

import com.agilex.bcm.framework.base.BcmTest;
import com.agilex.bcm.framework.base.FacilityMessageModel;
import com.agilex.bcm.framework.base.StaffUserModel;
import com.agilex.bcm.framework.pages.staff.BcmAddMessage;
import com.agilex.bcm.framework.pages.staff.BcmVaFacilityMessages;
import com.agilex.common.helpers.DateRange;
import com.agilex.common.helpers.TimeOfDay;
import com.agilex.common.helpers.ValueUnits;
import org.junit.Assert;
import org.junit.Test;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public class Breadcrumbs extends BcmTest {
    @Test
    public void verifyBreadcrumbs() {
        // Set up test variables
        StaffUserModel testUser = new StaffUserModel(deployment.staffUser4);
        FacilityMessageModel testMessage = new FacilityMessageModel(testUser.getFullName(), generateDateString(),
                "Breadcrumb Test", "Breadcrumb test message", "Once", new TimeOfDay(5, 0, TimeOfDay.PM),
                generateDateString(), generateDateString(Calendar.YEAR, 1));


        // Open the Annie staff app, log in as a clinical staff/admin, and switch to the Broadcast Admin view
        bcm.openAnnieStaffApp();
        bcm.staffLogin().logInAs(testUser.accessCode, testUser.verifyCode, testUser.station);
        bcm.staffPatientSearch().switchToBroadcastAdminView();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Verify that the current breadcrumb is the proper page
        Assert.assertTrue(bcm.bcmVaFacilityMessages().isCurrentBreadcrumbCorrect(BcmVaFacilityMessages.HEADER_TITLE_TEXT));

        // Begin to add a new message, verifying that the breadcrumbs are accurate
        bcm.bcmVaFacilityMessages().clickAddMessageButton();
        bcm.bcmAddMessage().waitForAddTitleHeader();
        Assert.assertTrue(bcm.bcmAddMessage().isCurrentBreadcrumbCorrect(BcmAddMessage.HEADER_TITLE_ADD_TEXT));
        Assert.assertTrue(bcm.bcmAddMessage().isParentBreadcrumbCorrect(0, BcmVaFacilityMessages.HEADER_TITLE_TEXT));

        // Add the message for later editing
        bcm.bcmAddMessage().fillOutMessageFields(testMessage);
        bcm.bcmAddMessage().saveMessageExpectSuccess();
        bcm.bcmFacilityMessageConfirmation().dismissModal();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Edit a message, verifying that the breadcrumbs are accurate
        bcm.bcmVaFacilityMessages().editMessage(0);
        bcm.bcmAddMessage().waitForEditTitleHeader();
        Assert.assertTrue(bcm.bcmAddMessage().isCurrentBreadcrumbCorrect(testMessage.title));
        Assert.assertTrue(bcm.bcmAddMessage().isParentBreadcrumbCorrect(0, BcmVaFacilityMessages.HEADER_TITLE_TEXT));
    }
}