package com.agilex.common.selenium;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * Represents an individual page in your website.  Classes that override this
 * will have access to the driverutil object for interacting with the website.
 */
public abstract class WebPage {
	protected final DriverUtil driverutil;
	protected final WebSite webSite;
    public final static String EMPTY_STRING = "";
	
	public WebPage(DriverUtil driverutil, WebSite webSite) {
		this.driverutil = driverutil;
		this.webSite = webSite;
	}
	
	/**
	 * Should return the page title of html page that the class represents.
	 */
	public abstract String getPageTitle();
	
	/**
	 * Returns True if the page title in the concrete class matches the
	 * page title returned by the <code>Driver</code>.
	 *
	 * Override this method and return true if you want to ignore the
	 * page title or if you need to verify the title from another location
	 * such as a div element.
	 */
	public boolean verifyTitle() {
		return getPageTitle().equals(driverutil.getPageTitle());
	}

    public String createElementLocator(String attribute, String value) {
        return("//*[@" + attribute + "='" + value + "']");
    }

    // Shared methods
    public boolean areExpectedSelectOptionsPresent(String selectElement, List<String> expectedOptions) {
        List<String> gatheredOptions = driverutil.getSelectOptions(selectElement);

        // Compare the arrays
        if(gatheredOptions.size() != expectedOptions.size()) {
            return false;
        } else {
            for(int i=0; i<gatheredOptions.size(); i++) {
                if(!gatheredOptions.get(i).equals(expectedOptions.get(i))) { return false; }
            }
        }

        // If there were no mismatches, return true
        return true;
    }

    public boolean areExpectedSelectOptionsPresent(String selectElement, String[] expectedOptions) {
        return areExpectedSelectOptionsPresent(selectElement, Arrays.asList(expectedOptions));
    }

    public boolean areExpectedSelectOptionsPresent(String selectElement, Map<String, String> expectedOptions) {
        return areExpectedSelectOptionsPresent(selectElement, new ArrayList<String>(expectedOptions.values()));
    }
}
