package gov.va.mobilehealth.courier.core;


import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.datatype.joda.JodaModule;

import java.io.IOException;

public abstract class MongoDocumentBase {

    @JsonProperty("_id")
    protected String id;

    public MongoDocumentBase() {
    }

    public static CustomObjectMapper newCustomObjectMapper() {
        return new CustomObjectMapper();
    }

    public String getId() {
        return this.id;
    }

    public void setId(String id) {
        this.id = id;
    }

    // customer deserializer to map mongodb ObjectID to String for _id field
    // TODO: revisit later
    public static class ObjectIdDeserializer extends JsonDeserializer<String> {
        @Override
        public String deserialize(JsonParser jsonParser, DeserializationContext deserializationContext) throws IOException {
            ObjectCodec oc = jsonParser.getCodec();
            JsonNode node = oc.readTree(jsonParser);
            JsonNode id = node.get("$oid");
            if (id != null) {
                return id.textValue();
            } else {
                return node.textValue();
            }
        }
    }

    public static class CustomObjectMapper extends ObjectMapper {

        public CustomObjectMapper() {
            SimpleModule module = new SimpleModule("ObjectIdmodule");
            module.addDeserializer(String.class, new ObjectIdDeserializer());
          this.registerModule(module);
          this.registerModule(new JodaModule());
          //Do not emit dates as timestamp integers (make it more readable)
          this.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
        }

    }
}
