package gov.va.mobilehealth.courier.core;

import java.util.Date;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.Objects;

import org.hibernate.validator.constraints.NotEmpty;
import org.joda.time.LocalDate;
import org.joda.time.Years;

@JsonIgnoreProperties(ignoreUnknown = true)
public class Patient extends MongoDocumentBase {
    public static final int PATIENT_CONSENT = 1;
    public static final int STAFF_ASSISTED_CONSENT = 3;

    @JsonProperty
    private String firstName;

    @JsonProperty
    private String middleName;

    @JsonProperty
    private String lastName;

    @JsonProperty
    @NotEmpty
    private String dob;

    @JsonProperty
    private String ssn;

    @JsonProperty
    private String gender;

    @JsonProperty
    private boolean patientExistInCms;

    @JsonProperty
    private PatientPreference preference;

    @JsonProperty
    private UserIdentifier patientIdentifier;

    @JsonProperty
    private String lastNameInitial;

    @JsonProperty
    private String last4SSN;
    
    @JsonProperty
    private int consent;
    
    @JsonProperty
    private String consentDate;

    @JsonProperty
    private String consentTime;

    @JsonProperty
    private String facilityId;

    public Patient() {
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getMiddleName() {
        return middleName;
    }

    public void setMiddleName(String middleName) {
        this.middleName = middleName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getDob() {
        return dob;
    }

    public void setDob(String dob) {
        this.dob = dob;
    }

    public String getSsn() {
        return ssn;
    }

    public void setSsn(String ssn) {
        this.ssn = ssn;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public void setPatientExistInCms(boolean exist) {
        this.patientExistInCms = exist;
    }

    public PatientPreference getPreference() {
        return preference;
    }

    public void setPreference(PatientPreference preference) {
        this.preference = preference;
    }

    public UserIdentifier getPatientIdentifier() {
        return patientIdentifier;
    }

    public void setPatientIdentifier(UserIdentifier patientIdentifier) {
        this.patientIdentifier = patientIdentifier;
    }

    public String getLastNameInitial() {
        return lastNameInitial;
    }

    public void setLastNameInitial(String lastNameInitial) {
        this.lastNameInitial = lastNameInitial;
    }

    @JsonIgnore
    public void setLastNameInitialFromLastName(String lastName) {
        this.lastNameInitial = lastName.substring(0, 1).toUpperCase();
    }

    public String getLast4SSN() {
        return last4SSN;
    }

    public void setLast4SSN(String last4SSN) {
        this.last4SSN = last4SSN;
    }

    @JsonIgnore
    public void setLast4SSNFromSSN(String ssn) {
        this.last4SSN = ssn.substring(Math.max(ssn.length() - 4, 0));
    }

    public int getConsent() {
		return consent;
	}

	public void setConsent(int consent) {
		this.consent = consent;
	}

	public String getConsentDate() {
		return consentDate;
	}

	public void setConsentDate(String consentDate) {
		this.consentDate = consentDate;
	}

    public String getConsentTime() {
        return consentTime;
    }

    public void setConsentTime(String consentTime) {
        this.consentTime = consentTime;
    }

    public String getFacilityId() {
        return facilityId;
    }

    public void setFacilityId(String facilityId) {
        this.facilityId = facilityId;
    }

	@JsonProperty("age")
    public int getAgeFromDoB() {
        LocalDate birthdate = new LocalDate(this.dob);
        LocalDate now = new LocalDate();
        Years year = Years.yearsBetween(birthdate, now);
        return year.getYears();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Patient)) {
            return false;
        }

        Patient patient = (Patient) o;

        if (dob != null ? !dob.equals(patient.dob) : patient.dob != null) {
            return false;
        }
        if (firstName != null ? !firstName.equals(patient.firstName) : patient.firstName != null) {
            return false;
        }
        if (gender != null ? !gender.equals(patient.gender) : patient.gender != null) {
            return false;
        }
        if (lastName != null ? !lastName.equals(patient.lastName) : patient.lastName != null) {
            return false;
        }
        if (middleName != null ? !middleName.equals(patient.middleName) : patient.middleName != null) {
            return false;
        }
        if (preference != null ? !preference.equals(patient.preference) : patient.preference != null) {
            return false;
        }
        if (ssn != null ? !ssn.equals(patient.ssn) : patient.ssn != null) {
            return false;
        }
        if (facilityId != null ? !facilityId.equals(patient.facilityId) : patient.facilityId != null) {
            return false;
        }


        return true;
    }

    @Override
    public int hashCode() {
        int result = firstName != null ? firstName.hashCode() : 0;
        result = 31 * result + (middleName != null ? middleName.hashCode() : 0);
        result = 31 * result + (lastName != null ? lastName.hashCode() : 0);
        result = 31 * result + (dob != null ? dob.hashCode() : 0);
        result = 31 * result + (ssn != null ? ssn.hashCode() : 0);
        result = 31 * result + (gender != null ? gender.hashCode() : 0);
        result = 31 * result + (preference != null ? preference.hashCode() : 0);
        result = 31 * result + (facilityId != null ? facilityId.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("firstName", firstName)
            .add("middleName", middleName)
            .add("lastName", lastName)
            .add("dob", dob)
            .add("ssn", ssn)
            .add("gender", gender)
            .add("preference", preference)
            .add("facilityId", facilityId)
            .toString();
    }
}
