package gov.va.mobilehealth.courier.core;


import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.Objects;
import org.hibernate.validator.constraints.NotEmpty;

import javax.validation.constraints.NotNull;
import java.io.Serializable;

public class PatientMessage extends MongoDocumentBase implements Serializable {

  private static final long serialVersionUID = 1L;

  @JsonProperty    // patient phone number
  @NotEmpty
  private String source;

  @JsonProperty       // receiving phone number
  @NotEmpty
  private String destination;

  @JsonProperty
  private String text;


  @JsonProperty
  @NotNull
  private MessageType type;


  @JsonProperty
  private String networkSubmitTime;


  @JsonProperty
  private String clientSubmitTime;


  @JsonProperty
  private String patientId;

  @JsonProperty("alertLevel")
  private String alertLevel;

  @JsonProperty("serviceName")
  private String serviceName;

  @JsonProperty
  private String timeZone;

  @JsonProperty
  private String adjustedSubmitTime;
  
  @JsonProperty
  private String[] msgDeliveryType;

  public PatientMessage() {
  }

  public PatientMessage(String source, String destination) {
    this.source = source;
    this.destination = destination;
  }

  public String getSource() {
    return this.source;
  }

  public void setSource(String source) {
    this.source = source;
  }

  @JsonProperty("alertLevel")
  public String getAlertLevel() {
    return alertLevel;
  }

  @JsonProperty("alertLevel")
  public void setAlertLevel(String status) {
    this.alertLevel = status;
  }

  @JsonProperty("serviceName")
  public String getServiceName() {
    return serviceName;
  }

  @JsonProperty("serviceName")
  public void setServiceName(String serviceName) {
    this.serviceName = serviceName;
  }

  public String getDestination() {
    return this.destination;
  }

  public void setDestination(String destination) {
    this.destination = destination;
  }

  public String getText() {
    return this.text;
  }

  public void setText(String text) {
    this.text = text;
  }

  public MessageType getType() {
    return type;
  }

  @JsonIgnore
  public void setType(MessageType type) {
    this.type = type;
  }

  @JsonProperty("type")
  public void setTypeAsString(String type) {
    this.type = MessageType.getEnum(type);
  }

  public String getNetworkSubmitTime() {
    return this.networkSubmitTime;
  }

  public void setNetworkSubmitTime(String networkSubmitTime) {
    this.networkSubmitTime = networkSubmitTime;
  }

  public String getClientSubmitTime() {
    return clientSubmitTime;
  }

  public void setClientSubmitTime(String clientSubmitTime) {
    this.clientSubmitTime = clientSubmitTime;
  }

  public String getPatientId() {
    return patientId;
  }

  public void setPatientId(String patientId) {
    this.patientId = patientId;
  }

    public String getTimeZone() {
      return this.timeZone;
    }

    public void setTimeZone(String timeZone) {
      this.timeZone = timeZone;
    }

    public String getAdjustedSubmitTime() {
      return this.adjustedSubmitTime;
    }

    public void setAdjustedSubmitTime(String adjustedSubmitTime) {
      this.adjustedSubmitTime = adjustedSubmitTime;
    }
    
    public String[] getMsgDeliveryType() {
		return msgDeliveryType;
	}

	public void setMsgDeliveryType(String[] msgDeliveryType) {
		this.msgDeliveryType = msgDeliveryType;
	}

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof PatientMessage)) {
      return false;
    }

    PatientMessage that = (PatientMessage) o;

    if (id != null ? !source.equals(that.id) : that.id != null) {
      return false;
    }

    return true;
  }

  @Override
  public int hashCode() {
    return id != null ? id.hashCode() : 0;
  }

  @Override
  public String toString() {
    return Objects.toStringHelper(this)
      .add("source", source)
      .add("destination", destination)
      .add("text", text)
      .add("type", type)
      .add("networkSubmitTime", networkSubmitTime)
      .add("clientSubmitTime", clientSubmitTime)
      .add("patientId", patientId)
      .add("alertLevel", alertLevel)
      .add("serviceName", serviceName)
      .toString();
  }
}
