package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.va.mobilehealth.courier.core.protocol.ChangeEntry;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by shimk on 5/5/14.
 */
public class PatientPreferenceAuditTrail extends MongoDocumentBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientPreferenceAuditTrail.class);
    @JsonProperty
    private String content;

    @JsonProperty
    private String user;

    @JsonProperty
    private String time;

    @JsonProperty
    private String date;

    @JsonProperty
    private String patientId;

    @JsonProperty
    private List<PreferenceChangeEntry> changes = new ArrayList<PreferenceChangeEntry>();

    public PatientPreferenceAuditTrail() {
    }

    public static List<PreferenceChangeEntry> findChange(String fieldName, String oldValue, String newValue, List<PreferenceChangeEntry> foundSoFar) {
        if (newValue != null && !newValue.equals(oldValue)) {
            foundSoFar.add(new PreferenceChangeEntry(fieldName, oldValue, newValue));
        }
        return foundSoFar;
    }

    public static List<PreferenceChangeEntry> findChange(String fieldName, int oldValue, int newValue, List<PreferenceChangeEntry> foundSoFar) {
        if (newValue != oldValue) {
            foundSoFar.add(new PreferenceChangeEntry(fieldName, String.valueOf(oldValue), String.valueOf(newValue)));
        }
        return foundSoFar;
    }

    public static List<PreferenceChangeEntry> findChange(String fieldName, boolean oldValue, boolean newValue, List<PreferenceChangeEntry> foundSoFar) {
        if (newValue != oldValue) {
            foundSoFar.add(new PreferenceChangeEntry(fieldName, String.valueOf(oldValue), String.valueOf(newValue)));
        }
        return foundSoFar;
    }

    public static List<PreferenceChangeEntry> findChange(Object o, Object newOne, List<PreferenceChangeEntry> foundSoFar) {
        if (o == null) {
            o = new PatientPreference();
        }
        if (newOne == null) {
            newOne = new PatientPreference();
        }

        if (o != null && (o instanceof PatientPreference) && (newOne != null && (newOne instanceof PatientPreference))) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<PreferenceChangeEntry>();
            }
            PatientPreference oldObject = (PatientPreference) o;
            PatientPreference newObject = (PatientPreference) newOne;

            foundSoFar = findChange("homePhoneNumber", oldObject.getHomePhoneNumber(), newObject.getHomePhoneNumber(), foundSoFar);
            foundSoFar = findChange("phoneNumber", oldObject.getPhoneNumber(), newObject.getPhoneNumber(), foundSoFar);
            foundSoFar = findChange("timeZone", oldObject.getTimeZone(), newObject.getTimeZone(), foundSoFar);
            foundSoFar = findChange("email", oldObject.getEmail(), newObject.getEmail(), foundSoFar);
            if (newObject.getMessageDeliveryPreference() == null) {
                newObject.setMessageDeliveryPreference(new DeliveryPreference());
            }
            foundSoFar = newObject.getMessageDeliveryPreference().findChange("messageDeliveryPreference", oldObject.getMessageDeliveryPreference(), foundSoFar);
            if (newObject.getNotificationDeliveryPreference() == null) {
                newObject.setNotificationDeliveryPreference(new NotificationDeliveryPreference());
            }
            foundSoFar = newObject.getNotificationDeliveryPreference().findChange("notificationDeliveryPreference", oldObject.getNotificationDeliveryPreference(), foundSoFar);
            foundSoFar = findChange("receiveFacilityMessage", oldObject.isReceiveFacilityMessage(), newObject.isReceiveFacilityMessage(), foundSoFar);
            foundSoFar = findChange("receiveApptReminder", oldObject.isReceiveApptReminder(), newObject.isReceiveApptReminder(), foundSoFar);

        }
        return foundSoFar;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public String getPatientId() {
        return patientId;
    }

    public void setPatientId(String patientId) {
        this.patientId = patientId;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public String getTime() {
        return time;
    }

    public void setTime(String time) {
        this.time = time;
    }

    public String getDate() {
        return date;
    }

    public void setDate(String date) {
        this.date = date;
    }

    public List<PreferenceChangeEntry> getChanges() {
        return changes;
    }

    public void setChanges(List<PreferenceChangeEntry> changes) {
        this.changes = changes;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof PatientPreferenceAuditTrail)) {
            return false;
        }

        PatientPreferenceAuditTrail patientPreferenceAuditTrail = (PatientPreferenceAuditTrail) o;

        if (content != null ? !content.equals(patientPreferenceAuditTrail.content) : patientPreferenceAuditTrail.content != null) {
            return false;
        }
        if (time != null ? !time.equals(patientPreferenceAuditTrail.time) : patientPreferenceAuditTrail.time != null) {
            return false;
        }
        if (date != null ? !date.equals(patientPreferenceAuditTrail.date) : patientPreferenceAuditTrail.date != null) {
            return false;
        }
        if (patientId != null ? !patientId.equals(patientPreferenceAuditTrail.patientId) : patientPreferenceAuditTrail.patientId != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = content != null ? content.hashCode() : 0;
        result = 31 * result + (time != null ? time.hashCode() : 0);
        result = 31 * result + (date != null ? date.hashCode() : 0);
        result = 31 * result + (patientId != null ? patientId.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("content", content)
            .add("time", time)
            .add("date", date)
            .add("patientId", patientId)
            .toString();
    }

    public void addAuditTrail(Patient old, Patient newOne, String date, String time, String user, String patientId) {
        //get audit trail

        List<PreferenceChangeEntry> foundSoFar = new ArrayList<PreferenceChangeEntry>();

        if (old == null) {
            old = new Patient();
        }
        if (newOne == null) {
            newOne = new Patient();
        }

        if (old != null && (old instanceof Patient) && (newOne != null && (newOne instanceof Patient))) {
            foundSoFar = findChange("consent", old.getConsent(), newOne.getConsent(), foundSoFar);
            foundSoFar = findChange("consentDate", old.getConsentDate(), newOne.getConsentDate(), foundSoFar);
            foundSoFar = findChange("consentTime", old.getConsentTime(), newOne.getConsentTime(), foundSoFar);
            foundSoFar = findChange("facilityId", old.getFacilityId(), newOne.getFacilityId(), foundSoFar);
        }
        foundSoFar = findChange(old.getPreference(), newOne.getPreference(), foundSoFar);
        //LOGGER.debug("foundsofar" + foundSoFar.get(0).getFieldName() + " newval" + foundSoFar.get(0).getNewValue());
        PatientPreferenceAuditTrail patientPreferenceAuditTrail;
        if (foundSoFar.size() > 0) {
            // only if there is any change
        /* put comments and change back together
         * in one entry */
            // if (patientPreferenceAuditTrails.size() > old.patientPreferenceAuditTrails.size()) {

            this.setChanges(foundSoFar);
            this.setDate(date);
            this.setTime(time);
            this.setUser(user);
            this.setPatientId(patientId);

        }

    }

}
