package gov.va.mobilehealth.courier.core.protocol;

import java.util.ArrayList;
import java.util.List;

import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import gov.va.mobilehealth.courier.core.MongoDocumentBase;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "enabled",
    "name",
    BroadcastFacilityMessage.assigningAuthorityField,
    "createDate",
    "createBy"
})
public class BroadcastFacilityMessage extends MongoDocumentBase {


  public final static String assigningAuthorityField = "assigningAuthority";
  public final static String PROGRESS_PENDING = "Pending Delivery";
  public final static String PROGRESS_IN_PROGRESS = "Delivery In Progress";
  public final static String PROGRESS_DELIVERED= "Delivered";
  // create time
    @JsonProperty("createDate")
    String createDate;
    // create by
    @JsonProperty("createBy")
    String createBy;
    @JsonProperty("enabled")
    private boolean status;
    @JsonProperty("name")
    private String name;
    @JsonProperty("message")
    private String message;
    @JsonProperty("assigningAuthority")
    private String assigningAuthority;
    @JsonProperty("startDate")
    String startDate;
    @JsonProperty("endDate")
    String endDate;
    @JsonProperty("schedule")
    protected Schedule schedule;
    @JsonProperty("progress")
    String progress;
    @JsonProperty("comments")
    private Comment comments;
    @JsonProperty("createdDateUTC")
    private String createdDateUTC;
    // this field is added for convenience in calculating time
    // it is not used by client app
    @JsonProperty("timezone")
    private String timezone;

    public BroadcastFacilityMessage() {
    }

    @JsonProperty("enabled")
    public boolean isEnabled() {
        return status;
    }

    @JsonProperty("enabled")
    public void setEnabled(boolean enabled) {
        this.status = enabled;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("message")
    public String getMessage() {
        return message;
    }

    @JsonProperty("message")
    public void setMessage(String message) {
        this.message = message;
    }

    @JsonProperty(assigningAuthorityField)
    public String getAssigningAuthority() {
        return assigningAuthority;
    }

    @JsonProperty(assigningAuthorityField)
    public void setAssigningAuthority(String assigningAuthority) {
        this.assigningAuthority = assigningAuthority;
    }

    @JsonProperty("createDate")
    public String getCreateDate() {
        return createDate;
    }

    @JsonProperty("createDate")
    public void setCreateDate(String createDate) {
        this.createDate = createDate;
    }

    @JsonProperty("createBy")
    public String getCreateBy() {
        return createBy;
    }

    @JsonProperty("createBy")
    public void setCreateBy(String createBy) {
        this.createBy = createBy;
    }

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    public String getEndDate() {
        return endDate;
    }

    public void setEndDate(String endDate) {
        this.endDate = endDate;
    }

    public Schedule getSchedule() {
        return schedule;
    }

    public void setSchedule(Schedule schedule) {
        this.schedule = schedule;
    }

    public String getProgress() {
        return progress;
    }

    public void setProgress(String progress) {
        this.progress = progress;
    }

    public void setComments(Comment comments) {
        this.comments = comments;
    }

    public Comment getComments() {
        return comments;
    }

    public String getCreatedDateUTC() {
        return createdDateUTC;
    }

    public void setCreatedDateUTC(String createdDateUTC) {
        this.createdDateUTC = createdDateUTC;
    }

    public String getTimezone() {
        return timezone;
    }

    public void setTimezone(String timezone) {
        this.timezone = timezone;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof BroadcastFacilityMessage)) {
            return false;
        }

        BroadcastFacilityMessage facilityMessage = (BroadcastFacilityMessage) o;

        if (status != facilityMessage.status) {
            return false;
        }
        if (message != null ? !message.equals(facilityMessage.message) : facilityMessage.message != null) {
            return false;
        }
        if (name != null ? !name.equals(facilityMessage.name) : facilityMessage.name != null) {
            return false;
        }
        if (assigningAuthority != null ? !assigningAuthority.equals(facilityMessage.assigningAuthority) : facilityMessage.assigningAuthority != null) {
            return false;
        }
        if (createDate != null ? !createDate.equals(facilityMessage.createDate) : facilityMessage.createDate != null) {
            return false;
        }
        if (createBy != null ? !createBy.equals(facilityMessage.createBy) : facilityMessage.createBy != null) {
            return false;
        }
        if (startDate != null ? !startDate.equals(facilityMessage.startDate) : facilityMessage.startDate != null) {
            return false;
        }
        if (endDate != null ? !endDate.equals(facilityMessage.endDate) : facilityMessage.endDate != null) {
            return false;
        }
        if (schedule != null ? !schedule.equals(facilityMessage.schedule) : facilityMessage.schedule != null) {
            return false;
        }
        if (progress != null ? !progress.equals(facilityMessage.progress) : facilityMessage.progress != null) {
            return false;
        }
        if (comments != null ? !comments.equals(facilityMessage.comments) : facilityMessage.comments != null) {
            return false;
        }
        if (createdDateUTC != null ? !createdDateUTC.equals(facilityMessage.createdDateUTC) : facilityMessage.createdDateUTC != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = (status ? 1 : 0);
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (message != null ? message.hashCode() : 0);
        result = 31 * result + (assigningAuthority != null ? assigningAuthority.hashCode() : 0);

        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("name", name)
            .add("message", message)
            .add("assigningAuthority", assigningAuthority)
            .add("enabled", status)
            .add("createDate", createDate)
            .add("createBy", createBy)
            .add("startDate", startDate)
            .add("endDate", endDate)
            .add("schedule", schedule)
            .add("progress", progress)
            .add("comments", comments)
            .add("createdDateUTC", createdDateUTC)
            .toString();
    }
}
