package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.*;
import com.google.common.base.Objects;
import org.hibernate.validator.constraints.NotEmpty;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "_id",
    "name",
    "readings",
    "firstLevelMessage",
    "secondLevelMessage",
    "criticalMessage",
    "keywords",
    "pattern"
})
@JsonIgnoreProperties(ignoreUnknown = true)
public class CMSService implements CompareElement {

    @JsonIgnore
    public static final String KEYWORD_PATTERN_FORMAT = "(?i).*\\s*(%s)(\\s|\\d).*";

    @JsonProperty("_id")
    private String serviceId;
    @JsonProperty("name")
    private String name;
    @JsonProperty("readings")
    private List<Reading> readings = new ArrayList<Reading>();
    @JsonProperty("firstLevelMessage")
    private MessageElement firstLevelMessage;
    @JsonProperty("secondLevelMessage")
    private MessageElement secondLevelMessage;
    @JsonProperty("criticalMessage")
    private MessageElement criticalMessage;
    @JsonProperty("keywords")
    @NotEmpty
    private List<String> keywords = new ArrayList<String>();
    @JsonProperty("pattern")
    private String pattern;

    public static boolean validateKeywords(List<String> keywords) {
        //TODO: what do we need to validate?
        // one word?
        boolean valid = true;
        for (String k : keywords) {
            if (k.contains(" ")) {
                valid = false;
                break;
            }
        }
        return valid;
    }

    @JsonProperty("_id")
    public String getServiceId() {
        return serviceId;
    }

    @JsonProperty("_id")
    public void setServiceId(String serviceId) {
        this.serviceId = serviceId;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("readings")
    public List<Reading> getReadings() {
        return readings;
    }

    @JsonProperty("readings")
    public void setReadings(List<Reading> readings) {
        this.readings = readings;
    }

    @JsonProperty("firstLevelMessage")
    public MessageElement getFirstLevelMessage() {
        return firstLevelMessage;
    }

    @JsonProperty("firstLevelMessage")
    public void setFirstLevelMessage(MessageElement firstLevelMessage) {
        this.firstLevelMessage = firstLevelMessage;
    }

    @JsonProperty("secondLevelMessage")
    public MessageElement getSecondLevelMessage() {
        return secondLevelMessage;
    }

    @JsonProperty("secondLevelMessage")
    public void setSecondLevelMessage(MessageElement secondLevelMessage) {
        this.secondLevelMessage = secondLevelMessage;
    }

    @JsonProperty("criticalMessage")
    public MessageElement getCriticalMessage() {
        return criticalMessage;
    }

    @JsonProperty("criticalMessage")
    public void setCriticalMessage(MessageElement breachMessage) {
        this.criticalMessage = breachMessage;
    }

    @JsonProperty("pattern")
    public String getPattern() {
        return pattern;
    }

    @JsonProperty("pattern")
    public void setPattern(String pattern) {
        this.pattern = pattern;
    }

    @JsonIgnore
    public void setPatternFromKeywords(List<String> keywords) {
        int i;
        String kw = keywords.get(0);
        for (i = 1; i < keywords.size(); i++) {
            kw = kw + "|" + keywords.get(i);
        }
        this.pattern = String.format(KEYWORD_PATTERN_FORMAT, kw);
    }

    @JsonProperty("keywords")
    public List<String> getKeywords() {
        return keywords;
    }

    @JsonProperty("keywords")
    public void setKeywords(List<String> keywords) {
        this.keywords = keywords;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CMSService)) {
            return false;
        }

        CMSService service = (CMSService) o;

        if (criticalMessage != null ? !criticalMessage.equals(service.criticalMessage) : service.criticalMessage != null) {
            return false;
        }
        if (firstLevelMessage != null ? !firstLevelMessage.equals(service.firstLevelMessage) : service.firstLevelMessage != null) {
            return false;
        }
        if (name != null ? !name.equals(service.name) : service.name != null) {
            return false;
        }
        if (keywords != null ? !keywords.equals(service.keywords) : service.keywords != null) {
            return false;
        }
        if (readings != null ? !readings.equals(service.readings) : service.readings != null) {
            return false;
        }
        if (secondLevelMessage != null ? !secondLevelMessage.equals(service.secondLevelMessage) : service.secondLevelMessage != null) {
            return false;
        }
        if (serviceId != null ? !serviceId.equals(service.serviceId) : service.serviceId != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = serviceId != null ? serviceId.hashCode() : 0;
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (readings != null ? readings.hashCode() : 0);
        result = 31 * result + (firstLevelMessage != null ? firstLevelMessage.hashCode() : 0);
        result = 31 * result + (secondLevelMessage != null ? secondLevelMessage.hashCode() : 0);
        result = 31 * result + (criticalMessage != null ? criticalMessage.hashCode() : 0);
        result = 31 * result + (keywords != null ? keywords.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("serviceId", serviceId)
            .add("name", name)
            .add("readings", readings)
            .add("firstLevelMessage", firstLevelMessage)
            .add("secondLevelMessage", secondLevelMessage)
            .add("breachMessage", criticalMessage)
            .add("keywords", keywords)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new CMSService();
        }
        if (o != null && (o instanceof CMSService)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            CMSService oldObject = (CMSService) o;
            fieldName = fieldName + ".";

            foundSoFar = Template.findChange(fieldName + "name", oldObject.name, this.name, foundSoFar, source);
            //TODO: There is only one reading per protocol template
            // will revisit when we restructure the protocol/protocol template
            foundSoFar = Template.findChange(fieldName + "readings", oldObject.readings, this.readings, foundSoFar, source);
            if (this.firstLevelMessage == null) {
                this.firstLevelMessage = new MessageElement();
            }
            foundSoFar = this.firstLevelMessage.findChange(fieldName + "firstLevelMessage", oldObject.firstLevelMessage, foundSoFar, source);
            if (this.secondLevelMessage == null) {
                this.secondLevelMessage = new MessageElement();
            }
            foundSoFar = this.secondLevelMessage.findChange(fieldName + "secondLevelMessage", oldObject.secondLevelMessage, foundSoFar, source);
            if (this.criticalMessage == null) {
                this.criticalMessage = new MessageElement();
            }
            foundSoFar = this.criticalMessage.findChange(fieldName + "criticalMessage", oldObject.criticalMessage, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "keywords", oldObject.keywords, this.keywords, foundSoFar, source);
        }
        return foundSoFar;
    }

    public boolean sameElement(Object o) {
        if (!(o instanceof CMSService)) {
            return false;
        }

        CMSService service = (CMSService) o;
        if (serviceId != null ? !serviceId.equals(service.serviceId)
            : service.serviceId != null) {
            return false;
        }
        return true;
    }

    @JsonIgnore
    public String getElementIdentifier() {
        return this.name;
    }
}
