package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "below",
    "above"
})
public class MessageElement {

    @JsonProperty("below")
    private String below;
    @JsonProperty("above")
    private String above;

    public MessageElement() {
    }

    public MessageElement(String below, String above) {
        this.below = below;
        this.above = above;
    }

    @JsonProperty("below")
    public String getBelow() {
        return below;
    }

    @JsonProperty("below")
    public void setBelow(String below) {
        this.below = below;
    }

    @JsonProperty("above")
    public String getAbove() {
        return above;
    }

    @JsonProperty("above")
    public void setAbove(String above) {
        this.above = above;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof MessageElement)) {
            return false;
        }

        MessageElement that = (MessageElement) o;

        if (above != null ? !above.equals(that.above) : that.above != null) {
            return false;
        }

        if (below != null ? !below.equals(that.below) : that.below != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = below != null ? below.hashCode() : 0;
        result = 31 * result + (above != null ? above.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("below", below)
            .add("above", above)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new MessageElement();
        }
        if (o != null && (o instanceof MessageElement)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            MessageElement oldObject = (MessageElement) o;
            fieldName = fieldName + ".";

            foundSoFar = Template.findChange(fieldName + "below", oldObject.below, this.below, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "above", oldObject.above, this.above, foundSoFar, source);

        }
        return foundSoFar;
    }

}
