package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

// this is motivational template
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "status",
    "type",
    "name",
    "description",
    "serviceMessage",
    "schedule"
})
public class Motivational extends Template implements CompareElement {


    @JsonProperty("serviceMessage")
    private List<String> serviceMessage;
    // index of last message sent. Start from 1. 0 means no message has been sent
    private int lastMessageSent;

    public Motivational() {
    }

    @JsonIgnore
    @Override
    public void generateName() {
        this.name = this.type;
    }

    @JsonProperty("type")
    @Override
    public String getType() {
        return "Motivational/Educational";
    }

    @JsonProperty("type")
    @Override
    public void setType(String type) {
        this.type = "Motivational/Educational";
    }

    @JsonProperty("serviceMessage")
    public List<String> getServiceMessage() {
        return serviceMessage;
    }

    @JsonProperty("serviceMessage")
    public void setServiceMessage(List<String> serviceMessage) {
        this.serviceMessage = serviceMessage;
    }

    public int getLastMessageSent() {
        return this.lastMessageSent;
    }

    public void putLastMessageSent(int lastMessageSent) {
        this.lastMessageSent = lastMessageSent;
    }

    @JsonIgnore
    private void increaseLastMessageSent() {
        this.lastMessageSent++;
        if (this.lastMessageSent > this.serviceMessage.size()) {
            this.lastMessageSent = 1;
        }
    }

    @JsonIgnore
    public void resetLastMessageSent() {
        this.lastMessageSent = 0;
    }

    @JsonIgnore
    public String getNextMessage() {
        increaseLastMessageSent();
        return this.serviceMessage.get(this.lastMessageSent - 1);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Motivational)) {
            return false;
        }

        Motivational template = (Motivational) o;

        if (status != template.status) {
            return false;
        }
        if (description != null ? !description.equals(template.description) : template.description != null) {
            return false;
        }
        if (name != null ? !name.equals(template.name) : template.name != null) {
            return false;
        }
        if (schedule != null ? !schedule.equals(template.schedule) : template.schedule != null) {
            return false;
        }
        if (serviceMessage != null ? !serviceMessage.equals(template.serviceMessage) : template.serviceMessage != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = id != null ? id.hashCode() : 0;
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (description != null ? description.hashCode() : 0);
        result = 31 * result + (serviceMessage != null ? serviceMessage.hashCode() : 0);
        result = 31 * result + (schedule != null ? schedule.hashCode() : 0);

        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("name", name)
            .add("type", type)
            .add("description", description)
            .add("serviceMessage", serviceMessage)
            .add("schedule", schedule)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new Motivational();
        }
        if (o != null && (o instanceof Motivational)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            Motivational oldObject = (Motivational) o;
            fieldName = fieldName + ".";

            foundSoFar = findChange(fieldName + "status",
                oldObject.status != null ? oldObject.status.toString() : "",
                this.status != null ? this.status.toString() : "", foundSoFar, source);
            foundSoFar = findChange(fieldName + "name", oldObject.name, this.name, foundSoFar, source);
            foundSoFar = findChange(fieldName + "description", oldObject.description, this.description, foundSoFar, source);
            foundSoFar = findChange(fieldName + "serviceMessage", oldObject.serviceMessage, this.serviceMessage, foundSoFar, source);
            if (this.schedule == null) {
                this.schedule = new Schedule();
            }
            foundSoFar = this.schedule.findChange(fieldName + "schedule", oldObject.schedule, foundSoFar, source);
            foundSoFar = Motivational.findChange("startDate", oldObject.startDate, this.startDate, foundSoFar, source);
            foundSoFar = Motivational.findChange("endDate", oldObject.endDate, this.endDate, foundSoFar, source);
        }
        return foundSoFar;
    }

}
