package gov.va.mobilehealth.courier.core.protocol;


import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.Objects;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import org.hibernate.validator.constraints.NotEmpty;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PatientProtocol extends MongoDocumentBase {

  @JsonIgnore


  public static final List<PatientProtocolStatus> ACTIVE_PATIENT_PROTOCOL_STATUSES = Arrays.asList(PatientProtocolStatus.ACTIVE, PatientProtocolStatus.RESUME);
  public static final List<PatientProtocolStatus> ACTIVE_AND_PAUSED_PATIENT_PROTOCOL = Arrays.asList(PatientProtocolStatus.ACTIVE, PatientProtocolStatus.RESUME,PatientProtocolStatus.PAUSED);


  // patient Id
    @JsonProperty("patientId")
    @NotEmpty
    String patientId;

    // assigned time
    @JsonProperty("assignedDate")
    String assignedDate;

    // last edited time
    @JsonProperty("lastEditedDate")
    String lastEditedDate;
    // protocol template
    @NotNull
    @JsonProperty("protocol")
    Protocol protocol;
    @JsonProperty
    PatientProtocolStatus status;
    @JsonProperty("comments")
    private List<Comment> comments = new ArrayList<Comment>();

    public PatientProtocol() {
    }

    public PatientProtocol(String patientId, Protocol protocol) {
        this.patientId = patientId;
        this.protocol = protocol;
    }

    public String getPatientId() {
        return patientId;
    }

    public void setPatientId(String patientId) {
        this.patientId = patientId;
    }

    public String getAssignedDate() {
        return assignedDate;
    }

    public void setAssignedDate(String assignedDate) {
        this.assignedDate = assignedDate;
    }

    public String getLastEditedDate() {
        return lastEditedDate;
    }

    public void setLastEditedDate(String lastEditedDate) {
        this.lastEditedDate = lastEditedDate;
    }

    public List<Comment> getComments() {
        return comments;
    }

    public void setComments(List<Comment> comments) {
        this.comments = comments;
    }

    @JsonIgnore
    public void addComments(Comment comment) {
        this.comments.add(comment);
    }

    public Protocol getProtocol() {
        return protocol;
    }

    public void setProtocol(Protocol protocol) {
        this.protocol = protocol;
    }


    @JsonProperty("status")
    public void setTypeAsString(String type) {
        this.status = PatientProtocolStatus.getEnum(type);
    }

    public PatientProtocolStatus getStatus() {
        return status;
    }

    @JsonIgnore
    public void setStatus(PatientProtocolStatus status) {
        this.status = status;
    }

    @JsonIgnore
    public void setAllTemplateStatus(PatientProtocolStatus status) {
        if (protocol != null && protocol.getTemplates() != null) {
            for (Template t : protocol.getTemplates()) {
                t.setStatus(status);
            }
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof PatientProtocol)) {
            return false;
        }

        PatientProtocol pProtocol = (PatientProtocol) o;

        if (id != null ? !id.equals(pProtocol.id) : pProtocol.id != null) {
            return false;
        }
        if (patientId != null ? !patientId.equals(pProtocol.patientId) : pProtocol.patientId != null) {
            return false;
        }
        if (protocol != null ? !protocol.equals(pProtocol.protocol) : pProtocol.protocol != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        return id != null ? id.hashCode() : 0;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("id", id)
            .add("patientId", patientId)
            .add("protocol", protocol)
            .toString();
    }

    @JsonIgnore
    public void addAuditTrail(PatientProtocol old, String date, String time, String user) {
        //get audit trail
        List<ChangeEntry> foundSoFar = new ArrayList<ChangeEntry>();
        foundSoFar = findChange(old, foundSoFar, "");
        Comment c;
        if (foundSoFar.size() > 0) {
            // only if there is any change
        /* put comments and change back together
         * in one entry */
            if (comments.size() > old.getComments().size()) {
                c = comments.get(comments.size() - 1);
                c.setChanges(foundSoFar);
                c.setDate(date);
                c.setTime(time);
                c.setUser(user);
            } else {
                c = new Comment();
                c.setChanges(foundSoFar);
                c.setDate(date);
                c.setTime(time);
                c.setUser(user);
                comments.add(c);
            }
        }
    }

    public List<ChangeEntry> findChange(Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o != null && (o instanceof PatientProtocol)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            PatientProtocol oldObject = (PatientProtocol) o;

            foundSoFar = Template.findChange("status",
                oldObject.status != null ? oldObject.status.toString() : "",
                this.status != null ? this.status.toString() : "", foundSoFar, source);
            foundSoFar = this.protocol.findChange("protocol", (Object) oldObject.protocol, foundSoFar, source);
        }
        return foundSoFar;
    }
}
