package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import gov.va.mobilehealth.courier.core.MongoDocumentBase;

import org.hibernate.validator.constraints.NotEmpty;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "status",
    "name",
    "description",
    "area",
    "facility",
    "clinic",
    "createDate",
    "createBy",
    "comments",
    "templates"
})
public class Protocol extends MongoDocumentBase {


    // create time
    @JsonProperty("createDate")
    String createDate;
    // create by
    @JsonProperty("createBy")
    String createBy;
    @JsonProperty("status")
    private boolean status;
    @JsonProperty("name")
    private String name;
    @JsonProperty("description")
    private String description;
    @JsonProperty("area")
    private String area;
    @JsonProperty("facility")
    private String facility;
    @JsonProperty("clinic")
    private String clinic;
    @JsonProperty("comments")
    private List<Comment> comments = new ArrayList<Comment>();

    @JsonProperty("templates")
    @NotEmpty
    private List<Template> templates = new ArrayList<Template>();

    public Protocol() {
    }

    @JsonProperty("status")
    public boolean isStatus() {
        return status;
    }

    @JsonProperty("status")
    public void setStatus(boolean status) {
        this.status = status;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("description")
    public String getDescription() {
        return description;
    }

    @JsonProperty("description")
    public void setDescription(String description) {
        this.description = description;
    }

    @JsonProperty("area")
    public String getArea() {
        return area;
    }

    @JsonProperty("area")
    public void setArea(String area) {
        this.area = area;
    }

    @JsonProperty("facility")
    public String getFacility() {
        return facility;
    }

    @JsonProperty("facility")
    public void setFacility(String facility) {
        this.facility = facility;
    }

    @JsonProperty("clinic")
    public String getClinic() {
        return clinic;
    }

    @JsonProperty("clinic")
    public void setClinic(String clinic) {
        this.clinic = clinic;
    }

    @JsonProperty("createDate")
    public String getCreateDate() {
        return createDate;
    }

    @JsonProperty("createDate")
    public void setCreateDate(String createDate) {
        this.createDate = createDate;
    }

    @JsonProperty("createBy")
    public String getCreateBy() {
        return createBy;
    }

    @JsonProperty("createBy")
    public void setCreateBy(String createBy) {
        this.createBy = createBy;
    }

    @JsonProperty("comments")
    public List<Comment> getComments() {
        return comments;
    }

    @JsonProperty("comments")
    public void setComments(List<Comment> comments) {
        this.comments = comments;
    }

    @JsonProperty("templates")
    public List<Template> getTemplates() {
        return templates;
    }

    @JsonProperty("templates")
    public void setTemplates(List<Template> templates) {
        this.templates = templates;
    }

    @JsonIgnore
    public void addAuditTrail(Protocol old, String date, String time, String user) {
      //get audit trail
      List<ChangeEntry> foundSoFar = new ArrayList<ChangeEntry>();
      foundSoFar = findChange("protocol", old, foundSoFar, "");
      Comment c;
      if (foundSoFar.size() > 0) {
        // only if there is any change
        /* put comments and change back together
         * in one entry */
        if ((old != null) && (comments.size() > old.getComments().size())) {
          c = comments.get(comments.size() - 1);
          c.setChanges(foundSoFar);
          c.setDate(date);
          c.setTime(time);
          c.setUser(user);
        } else {
          c = new Comment();
          c.setChanges(foundSoFar);
          c.setDate(date);
          c.setTime(time);
          c.setUser(user);
          comments.add(c);
        }
      }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Protocol)) {
            return false;
        }

        Protocol protocol = (Protocol) o;

        if (status != protocol.status) {
            return false;
        }
        if (description != null ? !description.equals(protocol.description) : protocol.description != null) {
            return false;
        }
        if (name != null ? !name.equals(protocol.name) : protocol.name != null) {
            return false;
        }
        if (area != null ? !area.equals(protocol.area) : protocol.area != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = (status ? 1 : 0);
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (description != null ? description.hashCode() : 0);
        result = 31 * result + (area != null ? area.hashCode() : 0);

        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("name", name)
            .add("description", description)
            .add("area", area)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new Protocol();
        }
        if (o != null && (o instanceof Protocol)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            Protocol oldObject = (Protocol) o;
            fieldName = fieldName + ".";

            foundSoFar = Template.findChange(fieldName + "status", oldObject.status, this.status, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "name", oldObject.name, this.name, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "description", oldObject.description, this.description, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "area", oldObject.area, this.area, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "templates", oldObject.templates, this.templates, foundSoFar, source);
        }
        return foundSoFar;
    }
}
