package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "name",
    "valid",
    "alert",
    "critical",
    "unit"
})
public class Reading implements CompareElement {

    @JsonProperty("name")
    private String name;
    @JsonProperty("valid")
    private ReadingThreshold valid;
    @JsonProperty("alert")
    private ReadingThreshold alert;
    @JsonProperty("critical")
    private ReadingThreshold critical;
    @JsonProperty("unit")
    private String unit;

    public Reading() {
    }

    public Reading(String name, ReadingThreshold valid, ReadingThreshold alert) {
        this.name = name;
        this.valid = valid;
        this.alert = alert;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("unit")
    public String getUnit() {
        return unit;
    }

    @JsonProperty("unit")
    public void setUnit(String unit) {
        this.unit = unit;
    }

    @JsonProperty("valid")
    public ReadingThreshold getValid() {
        return valid;
    }

    @JsonProperty("valid")
    public void setValid(ReadingThreshold valid) {
        this.valid = valid;
    }

    @JsonProperty("alert")
    public ReadingThreshold getAlert() {
        return alert;
    }

    @JsonProperty("alert")
    public void setAlert(ReadingThreshold alert) {
        this.alert = alert;
    }

    @JsonProperty("critical")
    public ReadingThreshold getCritical() {
        return critical;
    }

    @JsonProperty("critical")
    public void setCritical(ReadingThreshold breach) {
        this.critical = breach;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Reading)) {
            return false;
        }

        Reading reading = (Reading) o;

        if (alert != null ? !alert.equals(reading.alert) : reading.alert != null) {
            return false;
        }
        if (name != null ? !name.equals(reading.name) : reading.name != null) {
            return false;
        }
        if (valid != null ? !valid.equals(reading.valid) : reading.valid != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = name != null ? name.hashCode() : 0;
        result = 31 * result + (valid != null ? valid.hashCode() : 0);
        result = 31 * result + (alert != null ? alert.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("name", name)
            .add("valid", valid)
            .add("alert", alert)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new Reading();
        }
        if (o != null && (o instanceof Reading)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            Reading oldObject = (Reading) o;
            fieldName = fieldName + ".";

            if (this.valid == null) {
                this.valid = new ReadingThreshold();
            }
            foundSoFar = this.valid.findChange(fieldName + "valid", oldObject.valid, foundSoFar, source);
            if (this.alert == null) {
                this.alert = new ReadingThreshold();
            }
            foundSoFar = this.alert.findChange(fieldName + "alert", oldObject.alert, foundSoFar, source);
            if (this.critical == null) {
                this.critical = new ReadingThreshold();
            }
            foundSoFar = this.critical.findChange(fieldName + "critical", oldObject.critical, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "unit", oldObject.unit, this.unit, foundSoFar, source);
        }
        return foundSoFar;
    }

    public boolean sameElement(Object o) {
        if (!(o instanceof Reading)) {
            return false;
        }

        Reading reading = (Reading) o;
        if (name != null ? !name.equals(reading.name) : reading.name != null) {
            return false;
        }
        return true;
    }

    @JsonIgnore
    public String getElementIdentifier() {
        return this.name;
    }
}
