package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "hour",
    "min",
    "meridian"
})
public class ScheduleTime {

    @JsonProperty("description")
    protected String description;
    @JsonProperty("hour")
    private int hour;
    @JsonProperty("min")
    private int min;
    @JsonProperty("meridian")
    private Meridian meridian;

    public ScheduleTime() {
    }

    @JsonProperty("hour")
    public int getHour() {
        return hour;
    }

    @JsonProperty("hour")
    public void setHour(int hour) {
        this.hour = hour;
    }

    @JsonProperty("min")
    public int getMin() {
        return min;
    }

    @JsonProperty("min")
    public void setMin(int min) {
        this.min = min;
    }

    @JsonProperty("meridian")
    public Meridian getMeridian() {
        return meridian;
    }

    @JsonProperty("meridian")
    public void setMeridian(Meridian meridian) {
        this.meridian = meridian;
    }

    @JsonProperty("description")
    public String getDescription() {
        return generateDescription();
    }

    @JsonProperty("description")
    public void setDescription(String description) {
        this.description = description;
    }

    @JsonIgnore
    public String generateDescription() {
        return "(At " + this.hour + ":" + String.format("%02d", this.min) + " " + this.meridian + ")";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ScheduleTime)) {
            return false;
        }

        ScheduleTime that = (ScheduleTime) o;

        if (hour != that.hour) {
            return false;
        }
        if (min != that.min) {
            return false;
        }
        if (meridian != that.meridian) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = hour;
        result = 31 * result + min;
        result = 31 * result + (meridian != null ? meridian.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("hour", hour)
            .add("min", min)
            .add("meridian", meridian)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new ScheduleTime();
        }
        if (o != null && (o instanceof ScheduleTime)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            ScheduleTime oldObject = (ScheduleTime) o;
            fieldName = fieldName + ".";

            foundSoFar = Template.findChange(fieldName + "hour", oldObject.hour, this.hour, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "min", oldObject.min, this.min, foundSoFar, source);
            ;
            foundSoFar = Template.findChange(fieldName + "unit",
                oldObject.meridian != null ? oldObject.meridian.toString() : "",
                this.meridian != null ? this.meridian.toString() : "", foundSoFar, source);
            ;
        }
        return foundSoFar;
    }
}
