package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.google.common.base.Joiner;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;

import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonTypeInfo(use = JsonTypeInfo.Id.CLASS, include = JsonTypeInfo.As.PROPERTY, property = "@class")
public abstract class Template extends MongoDocumentBase implements CompareElement {


    @JsonProperty("status")
    protected PatientProtocolStatus status;
    @JsonProperty("name")
    protected String name;
    @JsonProperty("description")
    protected String description;
    @JsonProperty("type")
    protected String type;
    @JsonProperty("schedule")
    protected Schedule schedule;
    @JsonProperty("facility")
    protected String facility;
    @JsonProperty("clinic")
    protected String clinic;
    // start time
    @JsonProperty("startDate")
    String startDate;

    // end time
    @JsonProperty("endDate")
    String endDate;


    public Template() {
    }

    public static List<ChangeEntry> findChange(String fieldName, String oldValue, String newValue, List<ChangeEntry> foundSoFar, String source) {
        if (newValue != null ? !newValue.equals(oldValue)
            : oldValue != null) {
            foundSoFar.add(new ChangeEntry(fieldName, oldValue, newValue, source));
        }
        return foundSoFar;
    }

    public static List<ChangeEntry> findChange(String fieldName, boolean oldValue, boolean newValue, List<ChangeEntry> foundSoFar, String source) {
        if (newValue != oldValue) {
            foundSoFar.add(new ChangeEntry(fieldName, String.valueOf(oldValue), String.valueOf(newValue), source));
        }
        return foundSoFar;
    }

    public static List<ChangeEntry> findChange(String fieldName, Float oldValue, Float newValue, List<ChangeEntry> foundSoFar, String source) {
        if (newValue != null ? !newValue.equals(oldValue)
            : oldValue != null) {
            foundSoFar
                    .add(new ChangeEntry(fieldName,
                            oldValue == null ? null : numberToString(oldValue
                                    .floatValue()),
                            newValue == null ? null : numberToString(newValue
                                    .floatValue()), source));
        }
        return foundSoFar;
    }

    protected static String numberToString(Float d) {
        if (d == null) {
            return null;
        } else {
            float value = d.floatValue();
            if (value == (int) value) {
                return String.format("%d", (int) value);
            } else {
                return String.format("%s", value);
            }
        }
    }
    
    protected static String numberToString(Integer d) {
        if (d == null) {
            return null;
        } else {
            return d.toString();
        }
    }

    public static List<ChangeEntry> findChange(String fieldName, Integer oldValue, Integer newValue, List<ChangeEntry> foundSoFar, String source) {
        if (newValue != null ? !newValue.equals(oldValue)
            : oldValue != null) {
            foundSoFar
                    .add(new ChangeEntry(fieldName,
                            oldValue == null ? null : numberToString(oldValue
                                    .intValue()),
                            newValue == null ? null : numberToString(newValue
                                    .intValue()), source));
        }
        return foundSoFar;
    }

    public static List<ChangeEntry> findChange(String fieldName, int oldValue, int newValue, List<ChangeEntry> foundSoFar, String source) {
        if (newValue != oldValue) {
            foundSoFar.add(new ChangeEntry(fieldName, String.valueOf(oldValue), String.valueOf(newValue), source));
        }
        return foundSoFar;
    }

    public static List<ChangeEntry> findChange(String fieldName, List<?> oldValue, List<?> newValue, List<ChangeEntry> foundSoFar, String source) {
        boolean oldEmpty = (oldValue == null) || (oldValue.size() == 0) ? true : false;
        boolean newEmpty = (newValue == null) || (newValue.size() == 0) ? true : false;

        if (newEmpty && oldEmpty) {
            return foundSoFar;
        }

        try {
            // TODO: assume both lists have stuff.
            // we don't handle the case that one list is empty, another has value now
            // will revisit
            if ((!newEmpty && newValue.get(0) instanceof String)
                || (!oldEmpty && oldValue.get(0) instanceof String)) {
                if (fieldName.compareTo("protocol.templates.serviceMessage") == 0) {
                    // Change for motivational message needs to map one box by one
                    // i.e. box1 changed from 'message1' to 'message11'
                    // box2 changed from 'message2' to 'message22', etc
                    int newSize = newValue == null ? 0 : newValue.size();
                    int oldSize = oldValue == null ? 0 : oldValue.size();
                    int minSize = newSize < oldSize ? newSize : oldSize;
                    for (int i = 0; i < minSize; i++) {
                        String newElem = (String)newValue.get(i);
                        String oldElem = (String)oldValue.get(i);
                        foundSoFar = findChange(fieldName, oldElem, newElem, foundSoFar, source + ", Field " + (i+1));
                    }
                    if (minSize == oldSize) {
                        // process the rest of new list
                        for (int i = minSize; i < newSize; i++) {
                            String newElem = (String)newValue.get(i);
                            foundSoFar = findChange(fieldName, "", newElem, foundSoFar, source + ", Field " + (i+1));
                        }
                    } else {
                        // process the rest of old list
                        for (int i = minSize; i < oldSize; i++) {
                            String oldElem = (String)oldValue.get(i);
                            foundSoFar = findChange(fieldName, oldElem, "", foundSoFar, source + ", Field " + (i+1));
                        }
                    }
                } else {
                    // List of Strings
                    Joiner joiner = Joiner.on(",").useForNull("");
     
                    String oldString = oldEmpty ? "" : joiner.join(oldValue);
                    String newString = newEmpty ? "" : joiner.join(newValue);
                    return findChange(fieldName, oldString, newString, foundSoFar, source);
                }
            } else if ((!newEmpty && newValue.get(0) instanceof CompareElement)
                || (!oldEmpty && oldValue.get(0) instanceof CompareElement)) {
                // if one of the list is empty
                if (oldEmpty) {
                    @SuppressWarnings("unchecked")
                    List<CompareElement> newList = (List<CompareElement>) newValue;
                    for (CompareElement e : newList) {
                        // take care the element which were added
                        e.findChange(fieldName, null, foundSoFar, 
                            source.compareTo("") == 0 
                              ? e.getElementIdentifier()
                                  : source + "," + e.getElementIdentifier());
                    }
                    return foundSoFar;
                }
                if (newEmpty) {
                    @SuppressWarnings("unchecked")
                    List<CompareElement> oldList = (List<CompareElement>) oldValue;
                    for (CompareElement e : oldList) {
                        // take care the element which were added
                        foundSoFar.add(new ChangeEntry(fieldName, e.getElementIdentifier(), "", source));
                    }
                    return foundSoFar;
                }
                // if both lists are not empty
                @SuppressWarnings("unchecked")
                List<CompareElement> oldList = (List<CompareElement>) oldValue;
                @SuppressWarnings("unchecked")
                List<CompareElement> newList = (List<CompareElement>) newValue;

                // List of CompareElements, we need to take care if the order is different.
                for (int i = 0; i < newList.size(); i++) {
                    CompareElement o1 = newList.get(i);
                    boolean find = false;
                    for (CompareElement o2 : oldList) {
                        if (o1.sameElement(o2)) {
                            // Service is still there, see if there is difference
                            // within the element
                            o1.findChange(fieldName, o2, foundSoFar,
                                source.compareTo("") == 0
                                    ? o1.getElementIdentifier()
                                    : source + "," + o1.getElementIdentifier()
                            );
                            find = true;
                            break;
                        }
                    }
                    if (!find) {
                        // take care the element which were added
                        o1.findChange(fieldName, null, foundSoFar, 
                            source.compareTo("") == 0 
                              ? o1.getElementIdentifier()
                                  : source + "," + o1.getElementIdentifier());
                    }
                }
                for (CompareElement o2 : oldList) {
                    boolean find = false;
                    for (CompareElement o1 : newList) {
                        if (o2.sameElement(o1)) {
                            find = true;
                            break;
                        }
                    }
                    if (!find) {
                        // take care the element which were deleted
                        foundSoFar.add(new ChangeEntry(fieldName, o2.getElementIdentifier(), "", source));
                    }
                }
            }
        } catch (NullPointerException e) {
        }

        return foundSoFar;
    }

    @JsonProperty("status")
    public void setTypeAsString(String type) {
        this.status = PatientProtocolStatus.getEnum(type);
    }

    public PatientProtocolStatus getStatus() {
        return status;
    }

    @JsonIgnore
    public void setStatus(PatientProtocolStatus status) {
        this.status = status;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonIgnore
    public abstract void generateName();

    @JsonProperty("description")
    public String getDescription() {
        return description;
    }

    @JsonProperty("description")
    public void setDescription(String description) {
        this.description = description;
    }

    @JsonIgnore
    public void generateDescription() {
        this.description = this.schedule.getScheduleType().generateDescription();
        this.description = this.description + " at " + this.schedule.getScheduleTime().generateDescription();
    }

    @JsonProperty("type")
    public abstract String getType();

    @JsonProperty("type")
    public abstract void setType(String type);

    @JsonProperty("schedule")
    public Schedule getSchedule() {
        return schedule;
    }

    @JsonProperty("schedule")
    public void setSchedule(Schedule schedule) {
        this.schedule = schedule;
    }

    @JsonProperty("facility")
    public String getFacility() {
        return facility;
    }

    @JsonProperty("facility")
    public void setFacility(String facility) {
        this.facility = facility;
    }

    @JsonProperty("clinic")
    public String getClinic() {
        return clinic;
    }

    @JsonProperty("clinic")
    public void setClinic(String clinic) {
        this.clinic = clinic;
    }

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    @JsonIgnore
    public String getDefaultStartDate() {
        DateTime dateTime = new DateTime();
        DateTimeFormatter fmt = ISODateTimeFormat.date();

        //ignore isStartTimeRelative and isEndTimeRelative field for now
//      if (protocol.getSchedule().isIsStartTimeRelative()) {
        if (schedule.getRelativeStartTime().getValue() > 0) {
            Period period = schedule.getRelativeStartTime().createPeriod();
            dateTime = dateTime.plus(period);
        }
        startDate = dateTime.toString(fmt);
        return startDate;
    }

    @JsonIgnore
    public String getDefaultEndDate() {
        if (startDate == null) {
            startDate = getDefaultStartDate();
        }
        DateTime dateTime = new DateTime(startDate);
        DateTimeFormatter fmt = ISODateTimeFormat.date();

//      if (protocol.getSchedule().isIsEndTimeRelative()) {
        if (schedule.getRelativeEndTime().getValue() > 0) {
            Period period = schedule.getRelativeEndTime().createPeriod();
            if (period != null) {
                dateTime = dateTime.plus(period);
            }
        }
        endDate = dateTime.toString(fmt);
        return endDate;
    }

    public String getEndDate() {
        return endDate;
    }

    public void setEndDate(String endDate) {
        this.endDate = endDate;
    }

    public boolean sameElement(Object o) {
        if (!(o instanceof Template)) {
            return false;
        }

        Template template = (Template) o;
        if (id != null ? !id.equals(template.id)
            : template.id != null) {
            return false;
        }
        return true;
    }

    @JsonIgnore
    public String getElementIdentifier() {
        return this.type + ": " + this.description;
    }

}
