package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.google.common.base.Objects;

import org.joda.time.Period;

import java.util.ArrayList;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "value",
    "unit"
})
public class TimeInterval {
    @JsonProperty("value")
    private Integer value;
    @JsonProperty("unit")
    @JsonDeserialize(using = TimeIntervalUnitDeserializer.class)
    private TimeIntervalUnit unit;

    public TimeInterval() {
    }

    public TimeInterval(TimeIntervalUnit unit, int value) {
        this.unit = unit;
        this.value = value;
    }

    @JsonProperty("value")
    public Integer getValue() {
        return value;
    }

    @JsonProperty("value")
    public void setValue(Integer value) {
        this.value = value;
    }

    @JsonProperty("unit")
    @JsonDeserialize(using = TimeIntervalUnitDeserializer.class)
    public TimeIntervalUnit getUnit() {
        return unit;
    }

    @JsonProperty("unit")
    @JsonDeserialize(using = TimeIntervalUnitDeserializer.class)
    public void setUnit(TimeIntervalUnit unit) {
        this.unit = unit;
    }

    public Period createPeriod() {
        Period period = null;

        switch (this.unit) {
            case MINUTE:
                period = Period.minutes(this.value);
                break;
            case DAY:
                period = Period.days(this.value);
                break;
            case HOUR:
                period = Period.hours(this.value);
                break;
            case MONTH:
                period = Period.months(this.value);
                break;
            case WEEK:
                period = Period.weeks(this.value);
                break;
            case YEAR:
                period = Period.years(this.value);
                break;
        }
        return period;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof TimeInterval)) {
            return false;
        }

        TimeInterval that = (TimeInterval) o;

        if (value != null ? !value.equals(that.value) : that.value != null) {
            return false;
        }
        if (unit != that.unit) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = value;
        result = 31 * result + (unit != null ? unit.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("value", value)
            .add("unit", unit)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new TimeInterval();
        }
        if (o != null && (o instanceof TimeInterval)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            TimeInterval oldObject = (TimeInterval) o;
            fieldName = fieldName + ".";

            foundSoFar = Template.findChange(fieldName + "value", oldObject.value, this.value, foundSoFar, source);
            foundSoFar = Template.findChange(fieldName + "unit",
                oldObject.unit != null ? oldObject.unit.toString() : "",
                this.unit != null ? this.unit.toString() : "", foundSoFar, source);
            ;
        }
        return foundSoFar;
    }

}
