package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

// this is vital template
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
    "status",
    "type",
    "name",
    "description",
    "pattern",
    "serviceMessage",
    "reminderMessage",
    "confirmationMessage",
    "service",
    "schedule"
})
public class Vital extends Template implements CompareElement {


    @JsonProperty("serviceMessage")
    private String serviceMessage;
    @JsonProperty("reminderMessage")
    private String reminderMessage;
    @JsonProperty("confirmationMessage")
    private String confirmationMessage;
    @JsonProperty("service")
    private CMSService service;

    public Vital() {
    }

    @JsonIgnore
    @Override
    public void generateName() {
        this.name = this.type + ": " + this.service.getName();
    }

    @JsonProperty("type")
    @Override
    public String getType() {
        return "Vitals";
    }

    @JsonProperty("type")
    @Override
    public void setType(String type) {
        this.type = "Vitals";
    }

    @JsonProperty("serviceMessage")
    public String getServiceMessage() {
        return serviceMessage;
    }

    @JsonProperty("serviceMessage")
    public void setServiceMessage(String serviceMessage) {
        this.serviceMessage = serviceMessage;
    }

    @JsonProperty("reminderMessage")
    public String getReminderMessage() {
        return reminderMessage;
    }

    @JsonProperty("reminderMessage")
    public void setReminderMessage(String reminderMessage) {
        this.reminderMessage = reminderMessage;
    }

    @JsonProperty("confirmationMessage")
    public String getConfirmationMessage() {
        return confirmationMessage;
    }

    @JsonProperty("confirmationMessage")
    public void setConfirmationMessage(String confirmationMessage) {
        this.confirmationMessage = confirmationMessage;
    }

    @JsonProperty("service")
    public CMSService getService() {
        return service;
    }

    @JsonProperty("service")
    public void setService(CMSService service) {
        this.service = service;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Vital)) {
            return false;
        }

        Vital template = (Vital) o;

        if (status != template.status) {
            return false;
        }
        if (confirmationMessage != null ? !confirmationMessage.equals(template.confirmationMessage) : template.confirmationMessage != null) {
            return false;
        }
        if (description != null ? !description.equals(template.description) : template.description != null) {
            return false;
        }
        if (name != null ? !name.equals(template.name) : template.name != null) {
            return false;
        }
        if (reminderMessage != null ? !reminderMessage.equals(template.reminderMessage) : template.reminderMessage != null) {
            return false;
        }
        if (schedule != null ? !schedule.equals(template.schedule) : template.schedule != null) {
            return false;
        }
        if (serviceMessage != null ? !serviceMessage.equals(template.serviceMessage) : template.serviceMessage != null) {
            return false;
        }
        if (service != null ? !service.equals(template.service) : template.service != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = id != null ? id.hashCode() : 0;
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (description != null ? description.hashCode() : 0);
        result = 31 * result + (serviceMessage != null ? serviceMessage.hashCode() : 0);
        result = 31 * result + (reminderMessage != null ? reminderMessage.hashCode() : 0);
        result = 31 * result + (confirmationMessage != null ? confirmationMessage.hashCode() : 0);
        result = 31 * result + (service != null ? service.hashCode() : 0);
        result = 31 * result + (schedule != null ? schedule.hashCode() : 0);

        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
            .add("name", name)
            .add("type", type)
            .add("description", description)
            .add("serviceMessage", serviceMessage)
            .add("reminderMessage", reminderMessage)
            .add("confirmationMessage", confirmationMessage)
            .add("service", service)
            .add("schedule", schedule)
            .toString();
    }

    public List<ChangeEntry> findChange(String fieldName, Object o, List<ChangeEntry> foundSoFar, String source) {
        if (o == null) {
            o = new Vital();
        }
        if (o != null && (o instanceof Vital)) {
            if (foundSoFar == null) {
                foundSoFar = new ArrayList<ChangeEntry>();
            }
            Vital oldObject = (Vital) o;
            fieldName = fieldName + ".";

            foundSoFar = findChange(fieldName + "status",
                oldObject.status != null ? oldObject.status.toString() : "",
                this.status != null ? this.status.toString() : "", foundSoFar, source);
            foundSoFar = findChange(fieldName + "name", oldObject.name, this.name, foundSoFar, source);
            foundSoFar = findChange(fieldName + "description", oldObject.description, this.description, foundSoFar, source);
            foundSoFar = findChange(fieldName + "serviceMessage", oldObject.serviceMessage, this.serviceMessage, foundSoFar, source);
            foundSoFar = findChange(fieldName + "reminderMessage", oldObject.reminderMessage, this.reminderMessage, foundSoFar, source);
            foundSoFar = findChange(fieldName + "confirmationMessage", oldObject.confirmationMessage, this.confirmationMessage, foundSoFar, source);
            if (this.service == null) {
                this.service = new CMSService();
            }
            foundSoFar = this.service.findChange(fieldName + "service", oldObject.service, foundSoFar, source);
            if (this.schedule == null) {
                this.schedule = new Schedule();
            }
            foundSoFar = this.schedule.findChange(fieldName + "schedule", oldObject.schedule, foundSoFar, source);
            foundSoFar = Vital.findChange("startDate", oldObject.startDate, this.startDate, foundSoFar, source);
            foundSoFar = Vital.findChange("endDate", oldObject.endDate, this.endDate, foundSoFar, source);
        }
        return foundSoFar;
    }

}
