package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class FacilityTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(FacilityTest.class);

    @Test
    public void serializesToJSON() throws Exception {
        final List<Facility> pList = getFacilityList();

        LOGGER.info(MAPPER.writeValueAsString(pList));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/facilityList.json"),
            new TypeReference<List<Facility>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(pList)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/facilityList.json"),
                new TypeReference<List<Facility>>() {
                }
            ))
        );
    }


    @Test
    public void deserializesFromJSON() throws Exception {
        final List<Facility> pList = getFacilityList();

        final List<Facility> pListRead =
            MAPPER.readValue(fixture("fixtures/facilityList.json"),
                new TypeReference<List<Facility>>() {
                }
            );

        int i = 0;
        for (Facility p : pListRead) {
            assertThat(p).isEqualTo(pList.get(i++));
        }

    }

    private List<Facility> getFacilityList() {
        List<Facility> pList = new ArrayList<Facility>();

        Facility p1 = new Facility();
        p1.setFacilityId("dfn-688");
        p1.setName("Washington-Washington-DC");
        p1.setTimezone("America/New_York");

        Facility p2 = new Facility();
        p2.setFacilityId("dfn-777");
        p2.setName("Test VAMC3-Chantilly-VA");
        p2.setTimezone("America/New_York");

        Facility p3 = new Facility();
        p3.setFacilityId("dfn-778");
        p3.setName("Test Timezone LA-CA");
        p3.setTimezone("America/Los_Angeles");

        pList.add(p1);
        pList.add(p2);
        pList.add(p3);

        return pList;
    }


}
