package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientIncomingMessageTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientIncomingMessageTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws Exception {
        final List<PatientIncomingMessage> pList = getPatientResponseList();

        LOGGER.info(MAPPER.writeValueAsString(pList));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/patientIncomingMessage.json"),
            new TypeReference<List<PatientIncomingMessage>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(pList)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/patientIncomingMessage.json"),
                new TypeReference<List<PatientIncomingMessage>>() {
                }
            ))
        );
    }


    @Test
    public void deserializesFromJSON() throws Exception {
        final List<PatientIncomingMessage> pList = getPatientResponseList();

        final List<PatientIncomingMessage> pListRead =
            MAPPER.readValue(fixture("fixtures/patientIncomingMessage.json"),
                new TypeReference<List<PatientIncomingMessage>>() {
                }
            );

        int i = 0;
        Set<ConstraintViolation<PatientIncomingMessage>> constraintViolations;
        for (PatientIncomingMessage p : pListRead) {
            assertThat(p).isEqualTo(pList.get(i++));

            constraintViolations = validator.validate(p);
            assertThat(constraintViolations.size()).isEqualTo(0);
        }

    }

    private List<PatientIncomingMessage> getPatientResponseList() {


        List<PatientIncomingMessage> patientResponses = new ArrayList<PatientIncomingMessage>();
        String[] msgDeliveryType = {"Annie"};
       
        PatientIncomingMessage patientResponse1 = new PatientIncomingMessage();
        patientResponse1.setSource("3011112222");
        patientResponse1.setDestination("2021112222");
        patientResponse1.setText("Hello Got the message");
        patientResponse1.setNetworkSubmitTime("2013-01-01 10:00:00");
        patientResponse1.setClientSubmitTime("2013-01-01 09:00:00");
        patientResponse1.setClientMessageId("testing");
        patientResponse1.setCostCentre("Centre");
        patientResponse1.setMssageId("8000");
        patientResponse1.setSegmentCount(1);
        patientResponse1.setReferenceNumber(10);
        patientResponse1.setSegmentNumber(15);
        patientResponse1.setType(MessageType.IN);
        patientResponse1.setPatientId("52b9f1519d16ccbfab2fcff1");
        patientResponse1.setMsgDeliveryType(msgDeliveryType);
        patientResponse1.setId(null);
        
        patientResponses.add(patientResponse1);

        PatientIncomingMessage patientResponse2 = new PatientIncomingMessage("7031112222", "2021112222");
        patientResponse2.setText("Hello");
        patientResponse2.setNetworkSubmitTime("2013-11-01 10:10:00");
        patientResponse2.setClientSubmitTime("2013-11-01 10:00:00");
        patientResponse2.setClientMessageId("testing");
        patientResponse2.setCostCentre("Centre");
        patientResponse2.setMssageId("9000");
        patientResponse2.setSegmentCount(2);
        patientResponse2.setReferenceNumber(20);
        patientResponse2.setSegmentNumber(25);
        patientResponse2.setType(MessageType.IN);
        patientResponse2.setPatientId("52b9f1519d16ccbfab2fcff1");
        patientResponse2.setMsgDeliveryType(msgDeliveryType);
        patientResponse2.setId(null);

        patientResponses.add(patientResponse2);

        return patientResponses;
    }
}
