package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientListTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientListTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws Exception {
        final List<Patient> pList = getPatientList();
        String str = fixture("fixtures/patientList.json");
        String s=MAPPER.writeValueAsString(pList);

        LOGGER.info(MAPPER.writeValueAsString(pList));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/patientList.json"),
            new TypeReference<List<Patient>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(pList)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/patientList.json"),
                new TypeReference<List<Patient>>() {
                }
            ))
        );
    }


    @Test
    public void deserializesFromJSON() throws Exception {
        final List<Patient> pList = getPatientList();

        final List<Patient> pListRead =
            MAPPER.readValue(fixture("fixtures/patientList.json"),
                new TypeReference<List<Patient>>() {
                }
            );

        int i = 0;
        Set<ConstraintViolation<Patient>> constraintViolations;
        for (Patient p : pListRead) {
            assertThat(p).isEqualTo(pList.get(i++));

            // validation for class
            constraintViolations = validator.validate(p);
            assertThat(constraintViolations.size()).isEqualTo(0);
        }

    }

    private List<Patient> getPatientList() {
        List<Patient> pList = new ArrayList<Patient>();

        PatientPreference pref = new PatientPreference();
        pref.setPhoneNumber("1234567890");
        pref.setHomePhoneNumber("3334447777");
        pref.setEmail("email@abc.com");
        DeliveryPreference deliveryPref = new DeliveryPreference();
        deliveryPref.setSendSms(true);
        pref.setMessageDeliveryPreference(deliveryPref);
        NotificationDeliveryPreference notificationDeliveryPref = new NotificationDeliveryPreference();
        notificationDeliveryPref.setSendSms(true);
        pref.setNotificationDeliveryPreference(notificationDeliveryPref);
        pref.setMessageStarted(true);
        pref.setMessagePaused(true);

        pref.setTimeZone("EST");
        Patient p1 = new Patient();
        p1.setFirstName("John");

        p1.setLastName("Smith");
        p1.setMiddleName("Henry");
        p1.setDob("1966-01-01");
        p1.setSsn("111-22-3333");
        p1.setPatientExistInCms(true);
        p1.setGender("male");
        p1.setLastNameInitial("S");
        p1.setLast4SSN("3333");
        p1.setPreference(pref);

        Patient p2 = new Patient();
        p2.setFirstName("Mary");
        p2.setLastName("Jones");
        p2.setDob("1949-05-01");

        Patient p3 = new Patient();
        p3.setFirstName("Bob");
        p3.setLastName("Johnson");
        p3.setDob("1980-07-04");

        pList.add(p1);
        pList.add(p2);
        pList.add(p3);

        return pList;
    }


}
