package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientOutgoingMessageTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientOutgoingMessageTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws Exception {
        final List<PatientOutgoingMessage> pList = getPatientMessageList();

        LOGGER.info(MAPPER.writeValueAsString(pList));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/patientOutgoingMessage.json"),
            new TypeReference<List<PatientOutgoingMessage>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(pList)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/patientOutgoingMessage.json"),
                new TypeReference<List<PatientOutgoingMessage>>() {
                }
            ))
        );
    }


    @Test
    public void deserializesFromJSON() throws Exception {
        final List<PatientOutgoingMessage> pList = getPatientMessageList();

        final List<PatientOutgoingMessage> pListRead =
            MAPPER.readValue(fixture("fixtures/patientOutgoingMessage.json"),
                new TypeReference<List<PatientOutgoingMessage>>() {
                }
            );

        int i = 0;
        Set<ConstraintViolation<PatientOutgoingMessage>> constraintViolations;
        for (PatientOutgoingMessage p : pListRead) {
            assertThat(p).isEqualTo(pList.get(i++));

            constraintViolations = validator.validate(p);
            assertThat(constraintViolations.size()).isEqualTo(0);
        }

    }

    private List<PatientOutgoingMessage> getPatientMessageList() {


        List<PatientOutgoingMessage> patientResponses = new ArrayList<PatientOutgoingMessage>();
        String[] msgDeliveryType = {"Annie","SMS"};


        PatientOutgoingMessage patientResponse1 = new PatientOutgoingMessage();
        patientResponse1.setSource("3011112222");
        patientResponse1.setDestination("2021112222");
        patientResponse1.setText("Hello Got the message");
        patientResponse1.setNetworkSubmitTime("2013-01-01 10:00:00");
        patientResponse1.setClientSubmitTime("2013-01-01 09:00:00");
        patientResponse1.setType(MessageType.OUT);
        patientResponse1.setMsgDeliveryType(msgDeliveryType);
        patientResponse1.setPatientId("52b9f1519d16ccbfab2fcff1");
       
        Message msg = new Message();
        msg.setMessageId("69879417");
        msg.setResponseCode("001");
        msg.setResponseString("OK");
        msg.setSuccessCode("0");
        msg.setId(null);
        List<Message> msgList = new ArrayList<Message>();
        msgList.add(msg);

        patientResponse1.setMessages(msgList);

        patientResponse1.setId(null);

        patientResponses.add(patientResponse1);

        PatientOutgoingMessage patientResponse2 = new PatientOutgoingMessage("7031112222", "2021112222");
        patientResponse2.setText("Hello");
        patientResponse2.setNetworkSubmitTime("2013-11-01 10:10:00");
        patientResponse2.setClientSubmitTime("2013-11-01 10:00:00");
        patientResponse2.setType(MessageType.OUT);
        patientResponse2.setMsgDeliveryType(msgDeliveryType);
        patientResponse2.setPatientId("52b9f1519d16ccbfab2fcff1");

        patientResponse2.setId(null);

        Message msg2 = new Message();
        msg2.setMessageId("69879417");
        msg2.setResponseCode("001");
        msg2.setResponseString("OK");
        msg2.setSuccessCode("0");
        msg2.setId(null);
        List<Message> msgList2 = new ArrayList<Message>();
        msgList2.add(msg2);

        patientResponse2.setMessages(msgList2);

        patientResponses.add(patientResponse2);

        return patientResponses;
    }
}
