package gov.va.mobilehealth.courier.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;


public class PatientResponseServiceDataTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientResponseServiceDataTest.class);

    @Test
    public void serializesToJSON() throws Exception {
        final List<PatientResponseServiceData> patientResponseServiceDataList = getPatientResponseServiceDataList();
        LOGGER.info(MAPPER.writeValueAsString(patientResponseServiceDataList));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/patientResponseServiceData.json"),
            new TypeReference<List<PatientResponseServiceData>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(patientResponseServiceDataList)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/patientResponseServiceData.json"),
                new TypeReference<List<PatientResponseServiceData>>() {
                }
            ))
        );
    }

    @Test
    public void deserializesFromJSON() throws Exception {
        final List<PatientResponseServiceData> patientResponseServiceDataList = getPatientResponseServiceDataList();

        final List<PatientResponseServiceData> patientResponseServiceDatasFixture =
            MAPPER.readValue(fixture("fixtures/patientResponseServiceData.json"),
                new TypeReference<List<PatientResponseServiceData>>() {
                }
            );

        for (PatientResponseServiceData patientResponseServiceDataFixture : patientResponseServiceDatasFixture) {
            assertThat(patientResponseServiceDataList.contains(patientResponseServiceDataFixture)).isTrue();
        }
    }

    public List<PatientResponseServiceData> getPatientResponseServiceDataList() {

        PatientResponseServiceData patientResponseServiceData = new PatientResponseServiceData();
        List<PatientResponseServiceData> patientResponseServiceDataList = new ArrayList<PatientResponseServiceData>();
        List<PatientReading> patientReadings = new ArrayList<PatientReading>();
        PatientServiceReading patientServiceReading = new PatientServiceReading();
        PatientReading patientReading = new PatientReading();

        patientResponseServiceData.setPatientId("52b9f1519d16ccbfab2fcff1");
        patientResponseServiceData.setPatientResponseReceivedTime("2014-03-01 10:00:00");
        patientServiceReading.setServiceName("Blood Pressure");
        patientServiceReading.setServiceId("52e829f9bf890cf9ee685d85");
        patientReading.setName("Systolic");
        Float tmpval = new Float(120);
        patientReading.setValue(tmpval);
        patientReading.setUnit("mmHg");
        patientReadings.add(patientReading);
        patientReading = new PatientReading();
        patientReading.setName("Diastolic");
        tmpval = new Float(80);
        patientReading.setValue(tmpval);
        patientReading.setUnit("mmHg");
        patientReadings.add(patientReading);

        patientServiceReading.setReadings(patientReadings);
        patientResponseServiceData.setPatientServiceReading(patientServiceReading);


        PatientResponseServiceData patientResponseServiceData2 = new PatientResponseServiceData();

        patientResponseServiceData2.setPatientId("52b9f1519d16ccbfab2fcff1");
        patientResponseServiceData2.setPatientResponseReceivedTime("2014-03-02 10:00:00");
        patientServiceReading = new PatientServiceReading();
        patientServiceReading.setServiceName("Blood Pressure");
        patientServiceReading.setServiceId("52e829f9bf890cf9ee685d85");

        patientReadings = new ArrayList<PatientReading>();

        patientReading = new PatientReading();
        patientReading.setName("Systolic");
        tmpval = new Float(130);
        patientReading.setValue(tmpval);
        patientReading.setUnit("mmHg");
        patientReadings.add(patientReading);
        patientReading = new PatientReading();
        patientReading.setName("Diastolic");
        tmpval = new Float(85);
        patientReading.setValue(tmpval);
        patientReading.setUnit("mmHg");
        patientReadings.add(patientReading);
        patientServiceReading.setReadings(patientReadings);
        patientResponseServiceData2.setPatientServiceReading(patientServiceReading);


        PatientResponseServiceData patientResponseServiceData3 = new PatientResponseServiceData();
        patientResponseServiceData3.setPatientId("52b9f1519d16ccbfab2fcff1");
        patientResponseServiceData3.setPatientResponseReceivedTime("2014-03-02 10:30:00");
        patientServiceReading = new PatientServiceReading();
        patientServiceReading.setServiceName("Pulse Oximetry (SpO2)");
        patientServiceReading.setServiceId("52e829f9bf890cf9ee685d86");

        patientReadings = new ArrayList<PatientReading>();

        patientReading = new PatientReading();
        patientReading.setName("SpO2");
        tmpval = new Float(70);
        patientReading.setValue(tmpval);
        patientReading.setUnit("% Sat");
        patientReadings.add(patientReading);
        patientServiceReading.setReadings(patientReadings);
        patientResponseServiceData3.setPatientServiceReading(patientServiceReading);


        patientResponseServiceDataList.add(patientResponseServiceData);
        patientResponseServiceDataList.add(patientResponseServiceData2);
        patientResponseServiceDataList.add(patientResponseServiceData3);


        return patientResponseServiceDataList;

    }

}
