package gov.va.mobilehealth.courier.core;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Validation;
import javax.validation.Validator;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws Exception {
        final Patient patient = getPatient();

        assertThat(
            MAPPER.writeValueAsString(patient)).isEqualTo(MAPPER.writeValueAsString(MAPPER.readValue(fixture("fixtures/patient.json"),
            new TypeReference<Patient>() {
            }
        )));

        LOGGER.info(MAPPER.writeValueAsString(patient));
    }

    @Test
    public void deserializesFromJSON() throws Exception {
        final Patient patient = getPatient();
        assertThat(
            MAPPER.readValue(fixture("fixtures/patient.json"), Patient.class)).
            isEqualTo(patient);
    }

    @Test
    public void lastNameInital() throws Exception {
        Patient patient = new Patient();
        
        patient.setLastNameInitialFromLastName("tom");
        assertThat(patient.getLastNameInitial()).isEqualTo("T");
        
        patient.setLastNameInitialFromLastName("James");
        assertThat(patient.getLastNameInitial()).isEqualTo("J");
    }

    @Test
    public void last4digitSococial() throws Exception {
        Patient patient = new Patient();
        
        patient.setLast4SSNFromSSN("111-22-3333");
        assertThat(patient.getLast4SSN()).isEqualTo("3333");
        
        patient.setLast4SSNFromSSN("222334444");
        assertThat(patient.getLast4SSN()).isEqualTo("4444");
        
        patient.setLast4SSNFromSSN("222");
        assertThat(patient.getLast4SSN()).isEqualTo("222");
    }

    private Patient getPatient() throws ParseException {

        PatientPreference pref = getPatientPreference();

        Patient patient = new Patient();
        patient.setFirstName("John");
        patient.setLastName("Smith");
        patient.setMiddleName("Henry");
        patient.setDob("1966-01-01");
        patient.setSsn("111-22-3333");
        patient.setPatientExistInCms(true);
        patient.setGender("male");
        patient.setLastNameInitialFromLastName(patient.getLastName());
        patient.setLast4SSNFromSSN(patient.getSsn());
        patient.setConsent(Patient.STAFF_ASSISTED_CONSENT);
        patient.setConsentDate("12/21/2012");
        patient.setPreference(pref);

        return patient;
    }


    private PatientPreference getPatientPreference() {
        PatientPreference patientPreference = new PatientPreference();
        patientPreference.setPhoneNumber("1234567890");
        patientPreference.setHomePhoneNumber("3334447777");
        patientPreference.setEmail("email@abc.com");
        patientPreference.setTimeZone("EST");
        DeliveryPreference deliveryPref = new DeliveryPreference();
        deliveryPref.setSendSms(true);
        patientPreference.setMessageDeliveryPreference(deliveryPref);
        NotificationDeliveryPreference notificationDeliveryPref = new NotificationDeliveryPreference();
        notificationDeliveryPref.setSendSms(true);
        patientPreference.setNotificationDeliveryPreference(notificationDeliveryPref);
        patientPreference.setMessageStarted(true);
        patientPreference.setMessagePaused(true);
        return patientPreference;
    }

}

