package gov.va.mobilehealth.courier.core;

import java.text.ParseException;
import java.util.Date;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class StaffRoleTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(StaffRoleTest.class);

    @Test
    public void serializesToJSON() throws Exception {
        final StaffRole staffRole = getStaffRole();

        assertThat(
            MAPPER.writeValueAsString(staffRole)).isEqualTo(MAPPER.writeValueAsString(MAPPER.readValue(fixture("fixtures/staffRole.json"),
            new TypeReference<StaffRole>() {
            }
        )));

        LOGGER.info(MAPPER.writeValueAsString(staffRole));
    }

    @Test
    public void deserializesFromJSON() throws Exception {
        final StaffRole staffRole = getStaffRole();
        assertThat(
            MAPPER.readValue(fixture("fixtures/staffRole.json"), StaffRole.class)).isEqualsToByComparingFields(staffRole);
    }

    private StaffRole getStaffRole() throws ParseException {

        UserIdentifier userIdentifier = getUserIdentifier();

        StaffRole staffRole = new StaffRole();
        String[] roles = {"admin"};
        staffRole.setRoles(roles);
        staffRole.setLastModifiedDate("12-1-2013");
        staffRole.setUserIdentifier(userIdentifier);;

        return staffRole;
    }

    private UserIdentifier getUserIdentifier() {
    	UserIdentifier userIdentifier = new UserIdentifier();
    	userIdentifier.setUniqueId("PATID1");
    	userIdentifier.setAssigningAuthority("EDIPI");
        return userIdentifier;
    }

}


