package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class FacilityMessageTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(FacilityMessageTest.class);

    @Test
    public void serializesToJSON() throws IOException {
        final BroadcastFacilityMessage expected = getBroadcastFacilityMessage();
        String s = MAPPER.writeValueAsString(expected);

        final BroadcastFacilityMessage given = MAPPER.readValue(fixture("fixtures/facilityMessage.json"), BroadcastFacilityMessage.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final BroadcastFacilityMessage expected = getBroadcastFacilityMessage();
        String s = MAPPER.writeValueAsString(expected);

        final BroadcastFacilityMessage given = MAPPER.readValue(fixture("fixtures/facilityMessage.json"), BroadcastFacilityMessage.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

    public BroadcastFacilityMessage getBroadcastFacilityMessage() {
        BroadcastFacilityMessage broadcastFacilityMessage = new BroadcastFacilityMessage();

        broadcastFacilityMessage.setName("Test Message");
        broadcastFacilityMessage.setEnabled(true);
        broadcastFacilityMessage.setMessage("This is a test broadcast facility message");
        broadcastFacilityMessage.setCreateBy("Vehu One");
        broadcastFacilityMessage.setCreateDate("2014-01-05");
        broadcastFacilityMessage.setAssigningAuthority("dfn-688");
        broadcastFacilityMessage.setStartDate("2014-10-01");
        broadcastFacilityMessage.setEndDate("2015-10-25");
        broadcastFacilityMessage.setProgress("Pending Delivery");

        Comment c1 = new Comment();
        c1.setContent("Disable");
        c1.setDate("2013-11-20");
        c1.setTime("12:05 PM");
        c1.setUser("ONE VEHU");
        broadcastFacilityMessage.setComments(c1);

        Schedule schedule = new Schedule();

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(8);
        scheduleTime.setMin(30);
        scheduleTime.setMeridian(Meridian.AM);

        schedule.setScheduleTime(scheduleTime);

        TypeEverySoMany scheduleType = new TypeEverySoMany();
        scheduleType.setName("Custom");
        scheduleType.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
        schedule.setScheduleType(scheduleType);

        broadcastFacilityMessage.setSchedule(schedule);
        return broadcastFacilityMessage;
    }
}
