package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientProtocolCompareTest {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolCompareTest.class);

  @Test
  public void compareTest() throws Exception {
    final List<PatientProtocol> patientProtocols = MAPPER.readValue(
      fixture("fixtures/patientProtocolCompare.json"),
      new TypeReference<List<PatientProtocol>>() {
      }
    );

    List<ChangeEntry> foundSoFar = patientProtocols.get(0).findChange(patientProtocols.get(1), null, "");

    List<ChangeEntry> expected = MAPPER.readValue(
      fixture("fixtures/patientProtocolCompareResult.json"),
      new TypeReference<List<ChangeEntry>>() {
      }
    );

    assertThat(foundSoFar).isEqualTo(expected);
  }

  @Test
  public void motivationProtocolCompareTest() throws Exception {
    final List<Protocol> protocols = MAPPER.readValue(
      fixture("fixtures/motivationalProtocolCompare.json"),
      new TypeReference<List<Protocol>>() {
      }
    );

    List<ChangeEntry> foundSoFar = protocols.get(0).findChange("protocol", protocols.get(1), null, "");
    
    List<ChangeEntry> expected = MAPPER.readValue(
      fixture("fixtures/motivationalProtocolCompareResult.json"),
      new TypeReference<List<ChangeEntry>>() {
      }
    );

    assertThat(foundSoFar).isEqualTo(expected);
  }

  @Test
  public void vitalProtocolDaysofWeekCompareTest() throws Exception {
    Protocol protocol = MAPPER.readValue(fixture("fixtures/protocol.json"), Protocol.class);

    List<ChangeEntry> foundSoFar = protocol.findChange("protocol", null, null, "");
    
    List<ChangeEntry> expected = MAPPER.readValue(
      fixture("fixtures/vitalProtocolCompareResult.json"),
      new TypeReference<List<ChangeEntry>>() {
      }
    );

    assertThat(foundSoFar).isEqualTo(expected);
  }

}
