package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class PatientProtocolTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws Exception {
        final List<PatientProtocol> patientProtocols = getPatientProtocols();

        LOGGER.info(MAPPER.writeValueAsString(patientProtocols));
        LOGGER.info(MAPPER.writeValueAsString(MAPPER.readValue(
            fixture("fixtures/patientProtocol.json"),
            new TypeReference<List<PatientProtocol>>() {
            }
        )));

        assertThat(MAPPER.writeValueAsString(patientProtocols)).isEqualTo(
            MAPPER.writeValueAsString(MAPPER.readValue(
                fixture("fixtures/patientProtocol.json"),
                new TypeReference<List<PatientProtocol>>() {
                }
            ))
        );
    }

    @Test
    public void deserializesFromJSON() throws Exception {
        final List<PatientProtocol> patientProtocols = getPatientProtocols();

        final List<PatientProtocol> patientProtocolsFixture =
            MAPPER.readValue(fixture("fixtures/patientProtocol.json"),
                new TypeReference<List<PatientProtocol>>() {
                }
            );

        for (PatientProtocol patientProtocolFixture : patientProtocolsFixture) {
            assertThat(patientProtocols.contains(patientProtocolFixture)).isTrue();
        }
    }

    @Test
    public void noConstraintViolationReturned() throws Exception {
        final List<PatientProtocol> patientProtocolsFixture =
            MAPPER.readValue(fixture("fixtures/patientProtocol.json"),
                new TypeReference<List<PatientProtocol>>() {
                }
            );

        Set<ConstraintViolation<PatientProtocol>> constraintViolations;

        for (PatientProtocol patientProtocolFixture : patientProtocolsFixture) {
            assertThat(validator.validate(patientProtocolFixture).size()).isZero();
        }
    }

    private List<PatientProtocol> getPatientProtocols() throws Exception {
        final List<PatientProtocol> patientProtocols = ProtocolTemplateBuilder.buildPatientProtocols();
        Comment comment = new Comment();
        comment.setContent("test message1");
        comment.setTime("2014-01-30");

        Comment comment2 = new Comment();
        comment2.setContent("test message2");
        comment2.setTime("2014-01-31");

        List<Comment> comments = new ArrayList<Comment>();
        comments.add(comment);
        comments.add(comment2);

        patientProtocols.get(0).setComments(comments);
        patientProtocols.get(0).setAssignedDate("2013-11-20");
        patientProtocols.get(0).setLastEditedDate("2014-11-20");
        patientProtocols.get(1).setAssignedDate("2013-11-20");
        patientProtocols.get(1).setLastEditedDate("2014-11-20");

        return patientProtocols;
    }

}
