package gov.va.mobilehealth.courier.core.protocol;

import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;

import java.util.ArrayList;
import java.util.List;

public class ProtocolTemplateBuilder {

    public static Template buildProtocolTemplateWithScheduleTypeOnce() {
        final Vital protocol = new Vital();

        List<String> l2 = new ArrayList<String>();
        l2.add("Blood");
        l2.add("BP");

        protocol.setName("Blood Pressure Protocol1");
        protocol.setDescription("This is blood pressure protocol with once schedule");
        protocol.setReminderMessage("reminder message");
        protocol.setStatus(PatientProtocolStatus.ACTIVE);

        CMSService bp = new CMSService();

        Reading sys = new Reading("systolic", new ReadingThreshold(70, 262), new ReadingThreshold(80, 140));
        sys.setUnit("mmHg");
        sys.setCritical(new ReadingThreshold(75, 230));
        Reading dia = new Reading("diastolic", new ReadingThreshold(50, 120), new ReadingThreshold(60, 90));
        dia.setUnit("mmHg");
        dia.setCritical(new ReadingThreshold(55, 110));

        List<Reading> readings = new ArrayList<Reading>();
        readings.add(sys);
        readings.add(dia);
        bp.setReadings(readings);
        bp.setKeywords(l2);
        bp.setPattern("(?i).*\\s*(bp|blood)(\\s|\\d).*");

        bp.setFirstLevelMessage((new MessageElement("below level message", "above level message")));
        bp.setSecondLevelMessage((new MessageElement("below level message 2", "above level message 2")));
        bp.setCriticalMessage((new MessageElement("this is low breach message", "this is high breach message")));

        protocol.setService(bp);

        Schedule schedule = new Schedule();

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(8);
        scheduleTime.setMin(30);
        scheduleTime.setMeridian(Meridian.AM);

        schedule.setScheduleTime(scheduleTime);

        ScheduleType scheduleType = new TypeOnce();
        scheduleType.setName("Once");
        schedule.setScheduleType(scheduleType);

        schedule.setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 240));
        schedule.setReminderNumber(0);
        schedule.setRelativeStartTime(new TimeInterval(TimeIntervalUnit.MINUTE, 0));
        schedule.setRelativeEndTime(new TimeInterval(TimeIntervalUnit.MONTH, 10));
        schedule.setBreach(new Breach());

        protocol.setSchedule(schedule);
        return protocol;
    }

    public static Template buildProtocolTemplateWithScheduleTypeDaysOfWeek() {
        final Vital protocol = new Vital();

        List<String> l2 = new ArrayList<String>();
        l2.add("Blood");
        l2.add("BP");

        protocol.setName("Blood Pressure Protocol2");
        protocol.setDescription("This is blood pressure protocol with days of week schedule");
        protocol.setReminderMessage("reminder message");
        protocol.setStatus(PatientProtocolStatus.ACTIVE);

        CMSService bp = new CMSService();

        Reading sys = new Reading("Systolic", new ReadingThreshold(70, 262), new ReadingThreshold(90, 200));
        Reading dia = new Reading("Diastolic", new ReadingThreshold(50, 120), new ReadingThreshold(70, 120));
        sys.setCritical(new ReadingThreshold(80, 230));
        dia.setCritical(new ReadingThreshold(60, 120));
        sys.setUnit("mmHg");
        dia.setUnit("mmHg");

        List<Reading> readings = new ArrayList<Reading>();
        readings.add(sys);
        readings.add(dia);
        bp.setReadings(readings);
        bp.setKeywords(l2);
        bp.setPattern("(?i).*\\s*(bp|blood)(\\s|\\d).*");

        bp.setFirstLevelMessage((new MessageElement("below level message", "above level message")));
        bp.setSecondLevelMessage((new MessageElement("below level message 2", "above level message 2")));
        bp.setCriticalMessage((new MessageElement("this is low breach message", "this is high breach message")));

        protocol.setService(bp);

        Schedule schedule = new Schedule();

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(8);
        scheduleTime.setMin(30);
        scheduleTime.setMeridian(Meridian.AM);

        schedule.setScheduleTime(scheduleTime);

        ScheduleType scheduleType = new TypeDaysOfWeek().withMonday(true).withThursday(true);
        scheduleType.setName("Specified Days");
        schedule.setScheduleType(scheduleType);

        schedule.setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 240));
        schedule.setReminderNumber(0);
        schedule.setRelativeStartTime(new TimeInterval(TimeIntervalUnit.MINUTE, 0));
        schedule.setRelativeEndTime(new TimeInterval(TimeIntervalUnit.MONTH, 10));
        schedule.setBreach(new Breach());

        protocol.setSchedule(schedule);
        return protocol;
    }

    public static Template buildProtocolTemplateWithScheduleTypeEverySoMany() {
        final Vital protocol = new Vital();

        List<String> l2 = new ArrayList<String>();
        l2.add("Blood");
        l2.add("BP");

        protocol.setName("Blood Pressure Protocol3");
        protocol.setDescription("This is blood pressure protocol with a custom schedule type");
        protocol.setReminderMessage("reminder message");
        protocol.setStatus(PatientProtocolStatus.ACTIVE);

        CMSService bp = new CMSService();

        Reading sys = new Reading("systolic", new ReadingThreshold(70, 262), new ReadingThreshold(80, 140));
        Reading dia = new Reading("diastolic", new ReadingThreshold(50, 120), new ReadingThreshold(60, 100));
        sys.setCritical(new ReadingThreshold(75, 230));
        dia.setCritical(new ReadingThreshold(55, 110));
        sys.setUnit("mmHg");
        dia.setUnit("mmHg");

        List<Reading> readings = new ArrayList<Reading>();
        readings.add(sys);
        readings.add(dia);
        bp.setReadings(readings);
        bp.setKeywords(l2);
        bp.setPattern("(?i).*\\s*(bp|blood)(\\s|\\d).*");

        bp.setFirstLevelMessage((new MessageElement("below level message", "above level message")));
        bp.setSecondLevelMessage((new MessageElement("below level message 2", "above level message 2")));
        bp.setCriticalMessage((new MessageElement("this is low breach message", "this is high breach message")));

        protocol.setService(bp);

        Schedule schedule = new Schedule();

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(8);
        scheduleTime.setMin(30);
        scheduleTime.setMeridian(Meridian.AM);

        schedule.setScheduleTime(scheduleTime);

        TypeEverySoMany scheduleType = new TypeEverySoMany();
        scheduleType.setName("Custom");
        scheduleType.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
        schedule.setScheduleType(scheduleType);

        schedule.setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 240));
        schedule.setReminderNumber(0);
        schedule.setRelativeStartTime(new TimeInterval(TimeIntervalUnit.MINUTE, 0));
        schedule.setRelativeEndTime(new TimeInterval(TimeIntervalUnit.MONTH, 10));
        schedule.setBreach(new Breach());

        protocol.setSchedule(schedule);
        return protocol;
    }


    public static Template buildProtocolTemplateMotivationalWithScheduleTypeEverySoMany() {
        final Motivational protocol = new Motivational();

        List<String> l2 = new ArrayList<String>();
        l2.add("service message1");
        l2.add("service message2");
        protocol.setServiceMessage(l2);

        protocol.setName("Blood Pressure Protocol3");
        protocol.setDescription("This is blood pressure protocol with a custom schedule type");
        protocol.setStatus(PatientProtocolStatus.ACTIVE);

        Schedule schedule = new Schedule();

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(8);
        scheduleTime.setMin(30);
        scheduleTime.setMeridian(Meridian.AM);

        schedule.setScheduleTime(scheduleTime);

        TypeEverySoMany scheduleType = new TypeEverySoMany();
        scheduleType.setName("Custom");
        scheduleType.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
        schedule.setScheduleType(scheduleType);

        schedule.setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 240));
        schedule.setReminderNumber(0);
        schedule.setRelativeStartTime(new TimeInterval(TimeIntervalUnit.MINUTE, 0));
        schedule.setRelativeEndTime(new TimeInterval(TimeIntervalUnit.MONTH, 10));
        schedule.setBreach(new Breach());

        protocol.setSchedule(schedule);
        return protocol;
    }

    public static PatientProtocol buildPatientProtocol(String id, Template template) {

        PatientProtocol p1 = new PatientProtocol();
        if (template == null) {
            template = buildProtocolTemplateWithScheduleTypeOnce();
        }
        p1.setPatientId(id);
        template.setStartDate("2013-11-25");
        template.setEndDate("2013-12-25");
        p1.setAssignedDate("2013-11-20");
        p1.setLastEditedDate("2014-11-20");
        p1.setStatus(PatientProtocolStatus.ACTIVE);

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol p = new Protocol();
        p.setTemplates(templates);
        p.setName("Blood Pressure");
        p1.setProtocol(p);

        return p1;
    }

    public static List<PatientProtocol> buildPatientProtocols() throws Exception {
        List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();

        PatientProtocol p2 = new PatientProtocol();
        Template bp2 = buildProtocolTemplateWithScheduleTypeOnce();
        p2.setPatientId("528a739eda06d04179851fa9");
        bp2.setStartDate("2013-11-20");
        bp2.setEndDate("2014-11-20");
        p2.setAssignedDate("2013-11-20");
        p2.setLastEditedDate("2014-11-20");
        p2.setStatus(PatientProtocolStatus.ACTIVE);

        Template bp3 = buildProtocolTemplateWithScheduleTypeOnce();
        bp3.setStartDate("2013-11-20");
        bp3.setEndDate("2014-11-20");
        bp3.setName("Blood Pressure Protocol2");

        List<Template> templates = new ArrayList<Template>();
        templates.add(bp2);
        templates.add(bp3);
        Protocol p = new Protocol();
        p.setTemplates(templates);
        p.setName("Blood Pressure");
        p2.setProtocol(p);

        PatientProtocol p1 = new PatientProtocol();
        Template bp1 = buildProtocolTemplateWithScheduleTypeOnce();
        bp1.setName("new name");
        p1.setPatientId("52a8f798da063c40a95fc9b6");
        bp1.setStartDate("2013-11-25");
        bp1.setEndDate("2013-12-25");
        p1.setAssignedDate("2013-11-20");
        p1.setLastEditedDate("2014-11-20");
        p1.setStatus(PatientProtocolStatus.EXPIRED);

        templates = new ArrayList<Template>();
        templates.add(bp1);
        p = new Protocol();
        p.setTemplates(templates);
        p.setName("Blood Pressure");
        p1.setProtocol(p);

        patientProtocol.add(p2);
        patientProtocol.add(p1);

        return patientProtocol;
    }
}
