package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Validation;
import javax.validation.Validator;
import java.io.IOException;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class ProtocolTemplateMotivationalTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolTemplateMotivationalTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateMotivationalWithScheduleTypeEverySoMany();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-motivational.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateMotivationalWithScheduleTypeEverySoMany();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-motivational.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

}
