/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Validation;
import javax.validation.Validator;
import java.io.IOException;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class ProtocolTemplateScheduleTypeEverySoManyTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolTemplateScheduleTypeEverySoManyTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeEverySoMany();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-schedule-type-every-so-many.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeEverySoMany();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-schedule-type-every-so-many.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

}
