package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.io.IOException;
import java.util.Set;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class ProtocolTemplateScheduleTypeOnceTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolTemplateScheduleTypeOnceTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-schedule-type-once.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final Template expected = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();

        final Template given = MAPPER.readValue(fixture("fixtures/protocol-template-schedule-type-once.json"), Template.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

    @Test
    public void noConstraintViolationReturned() throws Exception {
        Template template = MAPPER.readValue(fixture("fixtures/protocol-template-schedule-type-once.json"), Template.class);

        // validation for class
        Set<ConstraintViolation<Template>> constraintViolations = validator.validate(template);

        assertThat(constraintViolations.size()).isZero();
    }
}
