package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class ProtocolTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolTest.class);

    @Test
    public void serializesToJSON() throws IOException {
        final Protocol expected = getProtocol();

        final Protocol given = MAPPER.readValue(fixture("fixtures/protocol.json"), Protocol.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final Protocol expected = getProtocol();

        final Protocol given = MAPPER.readValue(fixture("fixtures/protocol.json"), Protocol.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

    public Protocol getProtocol() {
        Template t = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();
        List<Template> templates = new ArrayList<Template>();
        Protocol protocol = new Protocol();

        templates.add(t);

        protocol.setName("Blood Pressure Protocol1");
        protocol.setStatus(true);
        protocol.setDescription("This is blood pressure protocol with once schedule");
        protocol.setArea("Blood Pressure");
        protocol.setCreateBy("Vehu One");
        protocol.setCreateDate("2014-01-05");

        List<Comment> comments = new ArrayList<Comment>();
        Comment c1 = new Comment();
        c1.setContent("Disable");
        c1.setDate("2013-11-20");
        c1.setTime("12:05 PM");
        c1.setUser("ONE VEHU");

        Comment c2 = new Comment();
        c2.setContent("Enable protocol");
        c2.setDate("2013-11-20");
        c2.setTime("12:05 PM");
        c2.setUser("ONE VEHU");

        comments.add(c1);
        comments.add(c2);

        protocol.setComments(comments);
        protocol.setTemplates(templates);

        return protocol;
    }
}
