package gov.va.mobilehealth.courier.core.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Validation;
import javax.validation.Validator;

import java.io.IOException;
import static org.fest.assertions.api.Assertions.assertThat;

public class TimeIntervalTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(TimeIntervalTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws IOException {
        final TimeInterval expected = new TimeInterval();
        expected.setValue(10);
        expected.setUnit(TimeIntervalUnit.MINUTE);

        final TimeInterval given = MAPPER.readValue("{\"value\": 10,\"unit\": \"MINUTE\"}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void serializesToJSONWithEmptyString() throws IOException {
        final TimeInterval expected = new TimeInterval();
        expected.setValue(10);

        final TimeInterval given = MAPPER.readValue("{\"value\": 10,\"unit\": \"\"}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void serializesToJSONWithMissingFieldValue() throws IOException {
        final TimeInterval expected = new TimeInterval();
        expected.setUnit(TimeIntervalUnit.MINUTE);

        final TimeInterval given = MAPPER.readValue("{\"unit\": \"MINUTE\"}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void serializesToJSONWithMissingFieldValueAndEmptyString() throws IOException {
        final TimeInterval expected = new TimeInterval();
        
        final TimeInterval given = MAPPER.readValue("{\"unit\": \"\"}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
                MAPPER.writeValueAsString(given));
    }

    @Test
    public void serializesToJSONWithMissingFieldUnit() throws IOException {
        final TimeInterval expected = new TimeInterval();
        expected.setValue(10);

        final TimeInterval given = MAPPER.readValue("{\"value\": 10}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void serializesToJSONWithEmptyFields() throws IOException {
        final TimeInterval expected = new TimeInterval();
        final TimeInterval given = MAPPER.readValue("{\"value\": \"\",\"unit\": null}", TimeInterval.class);

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
                MAPPER.writeValueAsString(given));
    }
}
