package gov.va.mobilehealth.courier.bcm.appointments;

import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.bcm.appointments.db.AssessmentProfileFutureAppointmentsDAO;
import gov.va.mobilehealth.courier.bcm.appointments.db.AssessmentProfileFutureAppointmentsDAOTest;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.helper.QuartzScheduleFactoryTestHelper;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.junit.BeforeClass;
import org.junit.Test;
import org.quartz.SchedulerException;

import java.io.IOException;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertNotNull;

/**
 * PatientAppointmentReminderMessageScheduler is more of a coordinator of activities
 * so testing should be more integration focues, all the activies tied together work.
 * Details of issue on individual aspects (retrieving, mapping , data layer) should be
 * handled in respective test cases
 */
public class AppointmentReminderServiceTest {
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException, SchedulerException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection(AssessmentProfileFutureAppointmentsDAO.AppointmentsCollectionName).drop();
    QuartzScheduleFactoryTestHelper.initTestScheduler();
  }

    @Test
  public void canRefreshAssessmentProfileAppointments() throws IOException {
      AssessmentProfileFutureAppointmentsDAO dao = DAOFactory.createAssessmentProfileFutureAppointmentsDAO();

      //create 3 dummy appointments and save
      for (int i = 0; i < 4; i++) {
        saveDummyAppointment();
      }


      EdipiIdentifier edipiIdentifier = new EdipiIdentifier("100840");
      //sync current appointments from cds
      AppointmentReminderService worker = new AppointmentReminderService();
      List<AppointmentReminder> reminders = worker.queryAppointmentsAndCreateReminders(edipiIdentifier);

      //verify count in db
      assertEquals(3, dao.findByPatient(edipiIdentifier).size());

      assertNotNull(reminders);
    }

  @Test
  public void canRemoveExistingReminders(){
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("100840");
    AppointmentReminderService worker = new AppointmentReminderService();
    worker.removeExistingReminders(edipiIdentifier);
  }

  private void saveDummyAppointment() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();
    //create and save
    AssessmentProfileFutureAppointment appointment = AssessmentProfileFutureAppointmentsDAOTest.createAppointment();
    appointment.setPatientIcnIdentifier(new IcnIdentifier("100840"));
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier("100840"));
    dao.saveOrUpdateAppointment(appointment);
  }

}
