package gov.va.mobilehealth.courier.bcm.appointments;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import gov.va.mobilehealth.courier.bcm.appointments.datefilter.DateFilterFactory;
import org.junit.Assert;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertNotNull;

public class FutureAppointmentsServiceTest {

  @Test
  public void canQueryPatientFutureAppointments() {
    final Appointments appointments = getFutureKeptAppointmentsByIcn("100840");

    Assert.assertNotNull(appointments);
    Assert.assertEquals(3, appointments.size());
//    For now I'll trust the mapping by CDSLib is right, only if i have to do my own mapping or other issues arise...
  }

  private Appointments getFutureKeptAppointmentsByIcn(String icnIdentifier1) {
    final IcnIdentifier icnIdentifier = new IcnIdentifier(icnIdentifier1);
    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);

    return appointmentsService.getFutureAppointments(icnIdentifier, dateFilter);
  }

  private Appointments getAllAppointmentsByIcn(String icnIdentifier1) {
    final IcnIdentifier icnIdentifier = new IcnIdentifier(icnIdentifier1);
    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);

    return appointmentsService.getAllAppointments(icnIdentifier, dateFilter);
  }

  @Test
  public void canRetrieveAppointmentsForAnnieVet(){
    EdipiIdentifier edipi = new EdipiIdentifier("1099737332");

    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);
    final Appointments appointments  = appointmentsService.getFutureAppointments(edipi, dateFilter);

    Assert.assertNotNull(appointments);
    assertNotEquals(0, appointments.size());

  }

  @Test
  public void canRetrieveAppointmentsForAnnieVet100840(){
    EdipiIdentifier edipi = new EdipiIdentifier("100840");

    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);
    final Appointments appointments  = appointmentsService.getFutureAppointments(edipi, dateFilter);

    Assert.assertNotNull(appointments);
    assertEquals(3, appointments.size());

  }

  @Test
  public void canRetrieveAppointmentsForAnnieVetCMS0001(){
    EdipiIdentifier edipi = new EdipiIdentifier("CMS0001");

    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);
    final Appointments appointments  = appointmentsService.getFutureAppointments(edipi, dateFilter);

    Assert.assertNotNull(appointments);
    //There are 5 appointments defined but one is cancelled and never gets returned
    assertEquals(4, appointments.size());

  }
  @Test
  public void canRetrieveAppointmentsForAnnieVetCMS0002(){
    EdipiIdentifier edipi = new EdipiIdentifier("CMS0002");

    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);
    final Appointments appointments  = appointmentsService.getFutureAppointments(edipi, dateFilter);

    Assert.assertNotNull(appointments);
    //There are 3 appointments defined but one is cancelled and never gets returned
    assertEquals(2, appointments.size());

  }
  @Test
  public void canRetrieveAppointmentsForAnnieVetCMS0003(){
    EdipiIdentifier edipi = new EdipiIdentifier("CMS0003");

    final AppointmentsService appointmentsService = new AppointmentsService();
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(1);
    final Appointments appointments  = appointmentsService.getFutureAppointments(edipi, dateFilter);

    Assert.assertNotNull(appointments);
    assertEquals(2, appointments.size());

  }

  @Test
  public void allAppointmentsHaveStartDates(){
    final Appointments appointments = getFutureKeptAppointmentsByIcn("100840");

    for (Appointment appointment : appointments) {
      assertNotNull(appointment.getAppointmentStartDate());
    }
  }

  @Test
  public void onlyKeptAppointmentsReturned(){
    final Appointments allAppointments = getAllAppointmentsByIcn("123456");  //default repsponse
    final Appointments keptAppointments = getFutureKeptAppointmentsByIcn("123456");

    Assert.assertNotEquals(0, allAppointments.size());
    Assert.assertNotEquals(0, keptAppointments.size());
    Assert.assertTrue(allAppointments.size() > keptAppointments.size());  //we should be getting fewer kept

    //make sure our set of all appointments contains a non kept appointment
    Boolean nonKeptFound = false;
    for (Appointment allAppointment : allAppointments) {
      if (!allAppointment.getStatusCode().equals("R")){
        nonKeptFound = true;
        continue;
      }
    }
    assertEquals(true, nonKeptFound);

    //make sure all kept appointments are actually kept status
    for (Appointment keptAppointment : keptAppointments) {
      assertEquals("R", keptAppointment.getStatusCode());
    }
  }
}
