package gov.va.mobilehealth.courier.bcm.appointments;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * Created by clarkee on 10/27/14.
 */
public class MpiServiceTest {

  @Test
  public void canQueryMpiWithFacilityId(){
    MpiService mpiService = new MpiService();

//    graphing patient two
    PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "100840");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(patientIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("100840", icnIds.get(0).getIcnId());
  }

  @Test
  public void canQueryMpiWithEdipi(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("P001");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("1008593069V747998", icnIds.get(0).getIcnId());
  }

  /**
   * Queries using values in mock set to return the Gallow Younger Response
   */
  @Test
  public void canQueryMpiWithGallowYoungerResponse(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("0000000001");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("1008593069V747998", icnIds.get(0).getIcnId());
  }

  /**
   * Queries using values in mock set to return the Gallow Younger Edipi Response
   * which is a different response
   */

  @Test
  public void canQueryMpiWithGallowYoungerEdipiResponse(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("D123401");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("D123401", icnIds.get(0).getIcnId());
  }

  @Test
  public void canQueryMpiWithEdipi100840(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("100840");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("100840", icnIds.get(0).getIcnId());
  }

  @Test
  public void canQueryMpiWithEdipiRubenMoreno(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("1099737332");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("1011491687V235101", icnIds.get(0).getIcnId());
  }

  /**
   * Following 3 tests are for patients that we use to test appointment reminders, just making sure they show up as they should
   */
  @Test
  public void canQueryPatientsForTest1(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("CMS0001");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("CMS0001", icnIds.get(0).getIcnId());
  }
  @Test
  public void canQueryPatientsForTest2(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("CMS0002");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("CMS0002", icnIds.get(0).getIcnId());
  }
  @Test
  public void canQueryPatientsForTest3(){
    MpiService mpiService = new MpiService();

//    One Patient Response
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("CMS0003");
    List<IcnIdentifier> icnIds = mpiService.getIcnIdentifiers(edipiIdentifier);

    assertNotNull(icnIds);
    assertEquals(1, icnIds.size());

    assertEquals("CMS0003", icnIds.get(0).getIcnId());
  }

}
