package gov.va.mobilehealth.courier.bcm.appointments.db;

import com.google.common.base.Optional;
import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.bcm.appointments.AssessmentProfileFutureAppointment;
import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.bcm.appointments.IcnIdentifier;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;
import static org.junit.Assert.assertEquals;

public class AssessmentProfileFutureAppointmentsDAOTest {
  private static MongoClient mongoClient;
  public static String defaultIcnId = "I2301";
  public static String defaultEdipiId = "E2301";
  ;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection(AssessmentProfileFutureAppointmentsDAO.AppointmentsCollectionName).drop();
  }

  @Before
  public void setupEach() throws IOException {
    mongoClient.getDB("courier").getCollection(AssessmentProfileFutureAppointmentsDAO.AppointmentsCollectionName).drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }


  @Test
  public void insertFutureAppointment() throws Exception {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create
    AssessmentProfileFutureAppointment appointment = createAppointment();
    //save
    AssessmentProfileFutureAppointment savedAppointment = dao.saveOrUpdateAppointment(appointment);

    assertThat(dao.removeAssessmentProfileFutureAppointment(savedAppointment)).isEqualTo(1);

  }

  public static AssessmentProfileFutureAppointment createAppointment() {
    AssessmentProfileFutureAppointment appointment = new AssessmentProfileFutureAppointment();

    //Default setters
//    DateTime startDate = DateTime.parse("1972-12-28T12:00:01.000Z");
    DateTime startDate = DateTime.parse("01-03-2014", DateTimeFormat.forPattern("MM-dd-yyyy"));
    appointment.setStartDate(startDate);
    appointment.setFacility("Int Test Facility");
    appointment.setPatientIcnIdentifier(new IcnIdentifier(defaultIcnId));
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier(defaultEdipiId));

    return appointment;
  }

  @Test
  public void canFindAll() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create and save
    AssessmentProfileFutureAppointment appointment = createAppointment();

    AssessmentProfileFutureAppointment savedAppointment = dao.saveOrUpdateAppointment(appointment);

    //find and assert
    List<AssessmentProfileFutureAppointment> appointments = dao.findAll();
    assertThat(appointments.size()).isEqualTo(1);

    //delete to clean up
    dao.removeFromDB(savedAppointment.getId());
  }

  @Test
  public void findById() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create and save
    AssessmentProfileFutureAppointment appointment = createAppointment();

    AssessmentProfileFutureAppointment savedAppointment = dao.saveOrUpdateAppointment(appointment);

    //find and assert
    Optional<AssessmentProfileFutureAppointment> foundAppointments = dao.findById(savedAppointment.getId());

    assertThat(foundAppointments.isPresent()).isEqualTo(true);
    assertEquals(savedAppointment.getId(), foundAppointments.get().getId());
    //delete to clean up
    dao.removeFromDB(savedAppointment.getId());

  }

  @Test
  public void findByPatientIcn() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create and save
    AssessmentProfileFutureAppointment appointment = createAppointment();

    AssessmentProfileFutureAppointment savedMessage = dao.saveOrUpdateAppointment(appointment);

    //find and assert
    List<AssessmentProfileFutureAppointment> appointments = dao.findByPatient(new IcnIdentifier(defaultIcnId));
    assertThat(appointments.size()).isEqualTo(1);

    //find and assert
    appointments = dao.findByPatient(new IcnIdentifier("9999"));
    assertThat(appointments.size()).isEqualTo(0);

    //delete to clean up
    dao.removeFromDB(savedMessage.getId());
  }

  @Test
  public void findByPatientEdipi() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create and save
    AssessmentProfileFutureAppointment appointment = createAppointment();

    AssessmentProfileFutureAppointment savedMessage = dao.saveOrUpdateAppointment(appointment);

    //find and assert
    List<AssessmentProfileFutureAppointment> appointments = dao.findByPatient(new EdipiIdentifier(defaultEdipiId));
    assertThat(appointments.size()).isEqualTo(1);

    //find and assert
    appointments = dao.findByPatient(new EdipiIdentifier("9999"));
    assertThat(appointments.size()).isEqualTo(0);

    //delete to clean up
    dao.removeFromDB(savedMessage.getId());
  }

  @Test
  public void removeByPatientEdipi() throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = new AssessmentProfileFutureAppointmentsDAO();

    //create and save
    AssessmentProfileFutureAppointment appointment = createAppointment();

    dao.saveOrUpdateAppointment(appointment);

    //find and assert
    int result = dao.removeFromDB(new EdipiIdentifier(defaultEdipiId));
    assertEquals(1, result);
  }


}
