package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.helper.QuartzScheduleFactoryTestHelper;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.impl.triggers.CronTriggerImpl;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class InitiateAllAppointmentRemindersJobCreatorTest {
  private InitiateAllAppointmentRemindersJobCreator jobCreator;
  private Scheduler scheduler;

  @Before
  public void setup() throws SchedulerException {
    scheduler = QuartzScheduleFactoryTestHelper.initTestScheduler();

    jobCreator = new InitiateAllAppointmentRemindersJobCreator();
  }

  @After
  public void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
  }

  @Test
  public void canCreateJob() throws Exception {
    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    jobCreator.createJob();

    /**
     * check jobs
     */
    // check init and expire job scheduled
    //TODO: For now, work with entity id, may need to re-evaluate since we blow away the id's on each run...  may need to be patient id + original hdr appt id
    JobKey jobKey = JobKey.jobKey(InitiateAllAppointmentRemindersJobCreator.JOB_APPOINTMENT_INIT_NAME, InitiateAllAppointmentRemindersJobCreator.JOB_APPOINTMENT_INIT_GROUP);
    boolean JobScheduled = scheduler.checkExists(jobKey);
    Assert.assertTrue("JobScheduled not found", JobScheduled);

    List<? extends Trigger> triggersOfJob = scheduler.getTriggersOfJob(jobKey);
    Assert.assertEquals(1, triggersOfJob.size());
    Trigger trigger = triggersOfJob.get(0);

    //assert it is a chron trigger
    assertEquals(CronTriggerImpl.class, trigger.getClass());
    Date nextTriggerDate = trigger.getNextFireTime();
    //assert next trigger date is set
    assertNotNull(nextTriggerDate);


    scheduler.resumeAll();

/**
 * We are not validating the job actions, only that we successfully create the job
 */

    /**
     * run cleanup job, sleep
     */
    scheduler.deleteJob(jobKey);
  }


}
