package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.helper.QuartzScheduleFactoryTestHelper;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;

import java.io.IOException;

public class RefreshPatientAppointmentsJobCreatorTest {
  private RefreshPatientAppointmentsJobCreator jobCreator;
  private Scheduler scheduler;

  @Before
  public void setup() throws SchedulerException {
    scheduler = QuartzScheduleFactoryTestHelper.initTestScheduler();

    jobCreator = new RefreshPatientAppointmentsJobCreator();
  }

  @After
  public void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
  }

  @Test
  public void canCreateJob() throws Exception {
    /**
     * Setup, create entities
     */
    Patient patient = new Patient();
    patient.setId("234l2k3j42");
    UserIdentifier patientIdentifier = new UserIdentifier();
    patientIdentifier.setAssigningAuthority("edipi");
    patientIdentifier.setUniqueId("edipi123456");
    patient.setPatientIdentifier(patientIdentifier);

    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    jobCreator.createRefreshJob(patient);

    /**
     * check jobs
     */
    // check init and expire job scheduled
    //TODO: For now, work with entity id, may need to re-evaluate since we blow away the id's on each run...  may need to be patient id + original hdr appt id
    JobKey jobKey = JobKey.jobKey(patient.getId(), RefreshPatientAppointmentsJobCreator.JOB_APPOINTMENT_REFRESH_GROUP);
    boolean JobScheduled = scheduler.checkExists(jobKey);
    Assert.assertTrue("JobScheduled not found", JobScheduled);

    //TODO: Check meta data on the job...
    JobDetail jobDetail = scheduler.getJobDetail(jobKey);
    Assert.assertNotNull(jobDetail.getJobDataMap().getString(RefreshPatientAppointmentsJob.PatientEdipiIdKey));

    scheduler.resumeAll();

//    /**
//     * sleep
//     */
//    // wait for a minute to let the send message job run
//    Thread.sleep(10000);

/**
 * We are not validating the job actions, only that we successfully create the job
 */

    /**
     * run cleanup job, sleep
     */
    scheduler.deleteJob(jobKey);
  }

}
