package gov.va.mobilehealth.courier.db;

import com.google.common.base.Optional;
import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;


public class BroadcastFacilityMessagesDaoTest {

  private static MongoClient mongoClient;
  private final String assigningAuthority = "dfn-688";

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection(BroadcastFacilityMessageDAO.FacilityMessagesCollectionName).drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }


  @Test
  public void testSaveOrUpdateFacilityMessage() throws Exception {
    BroadcastFacilityMessageDAO dao = new BroadcastFacilityMessageDAO();

    //create
    BroadcastFacilityMessage message = createFacilityMessage();
    //save
    BroadcastFacilityMessage savedMessage = dao.saveOrUpdateFacilityMessage(message);

    assertThat(dao.removeBroadcastFacilityMessage(savedMessage)).isEqualTo(1);

  }

  public BroadcastFacilityMessage createFacilityMessage() {
    BroadcastFacilityMessage message = new BroadcastFacilityMessage();
    message.setName("Test Message");
    message.setMessage("Test Description");
    message.setAssigningAuthority(assigningAuthority);
    return message;
  }

  @Test
  public void canFindAll() throws IOException {
    BroadcastFacilityMessageDAO dao = new BroadcastFacilityMessageDAO();

    //create and save
    BroadcastFacilityMessage message = createFacilityMessage();

    BroadcastFacilityMessage savedMessage = dao.saveOrUpdateFacilityMessage(message);

    //find and assert
    List<BroadcastFacilityMessage> messages = dao.findAll();
    assertThat(messages.size()).isEqualTo(1);

    //delete to clean up
    dao.removeFromDB(savedMessage.getId());
  }

  @Test
  public void findById() throws IOException {
    BroadcastFacilityMessageDAO dao = new BroadcastFacilityMessageDAO();

    //create and save
    BroadcastFacilityMessage message = createFacilityMessage();

    BroadcastFacilityMessage savedMessage = dao.saveOrUpdateFacilityMessage(message);

    //find and assert
    Optional<BroadcastFacilityMessage> foundMessage = dao.findById(savedMessage.getId());

    assertThat(foundMessage.isPresent()).isEqualTo(true);

    //delete to clean up
    dao.removeFromDB(savedMessage.getId());

  }

  @Test
  public void findByAssigningAuthority() throws IOException {
    BroadcastFacilityMessageDAO dao = new BroadcastFacilityMessageDAO();

    //create and save
    BroadcastFacilityMessage message = createFacilityMessage();

    BroadcastFacilityMessage savedMessage = dao.saveOrUpdateFacilityMessage(message);

    //find and assert
    List<BroadcastFacilityMessage> messages = dao.findByAssigningAuthority(assigningAuthority);
    assertThat(messages.size()).isEqualTo(1);

    //find and assert
    messages = dao.findByAssigningAuthority("dfn-badId");
    assertThat(messages.size()).isEqualTo(0);

    //delete to clean up
    dao.removeFromDB(savedMessage.getId());
  }

}
