package gov.va.mobilehealth.courier.db;

import gov.va.mobilehealth.courier.core.protocol.Reading;
import gov.va.mobilehealth.courier.core.protocol.ReadingThreshold;
import gov.va.mobilehealth.courier.core.protocol.dictionary.Dictionary;
import gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary;
import gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import com.mongodb.MongoClient;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class DictionaryDaoTest {
  private static DictionaryDAO dao;
  private String id = "52e829f9bf890cf9ee685d8b";
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("dictionary").drop();

    dao = new DictionaryDAO();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canFindAll() throws Exception {

    Dictionary dictionary = new Dictionary();

    List<ServiceDictionary> services = new ArrayList<ServiceDictionary>();

    List<String> l = new ArrayList<String>();
    l.add("Blood");
    l.add("BP");

    ServiceDictionary bp = new ServiceDictionary();
    Reading sys = new Reading("Systolic", new ReadingThreshold(70, 262), new ReadingThreshold(80, 140));
    sys.setUnit("mmHg");
    Reading dia = new Reading("Diastolic", new ReadingThreshold(50, 120), new ReadingThreshold(60, 90));
    dia.setUnit("mmHg");

    List<Reading> readings = new ArrayList<Reading>();
    readings.add(sys);
    readings.add(dia);
    bp.setReadings(readings);

    bp.setKeywords(l);
    bp.setName("Blood Pressure");

    services.add(bp);
    dao.insertService(bp);

    TimeZone timeZone = new TimeZone();
    List<TimeZone> timeZones = new ArrayList<TimeZone>();

    timeZone.setValue("US/Alaska");
    timeZone.setDescription("Alaska (GMT-09:00)");
    timeZones.add(timeZone);


    dictionary.setServices(services);
    dictionary.setTimeZones(timeZones);
    dao.insertTimeZone(timeZone);

    dictionary = dao.findAll();
    services = dictionary.getServices();

    assertThat(services.size()).isEqualTo(1);

    List<TimeZone> timezones = dictionary.getTimeZones();

    assertThat(timezones.size()).isEqualTo(1);


  }


}
