package gov.va.mobilehealth.courier.db;

import com.google.common.base.Optional;
import com.mongodb.MongoClient;

import gov.va.mobilehealth.courier.core.Facility;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;


public class FacilityDaoTest {

  private static MongoClient mongoClient;
  private static List<Facility> facilityList;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    facilityList = getFacilityList();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canFindAll() throws IOException {
      FacilityDAO dao = new FacilityDAO();

      for (Facility f : facilityList) {
          dao.saveOrUpdate(f);
      }

      //find and assert
      List<Facility> result = dao.findAll();
      assertThat(result.size()).isEqualTo(2);

      //delete to clean up
      dao.removeFromDB(facilityList.get(0));
      dao.removeFromDB(facilityList.get(1));
  }

  @Test
  public void canFindById() throws IOException {
      FacilityDAO dao = new FacilityDAO();

      for (Facility f : facilityList) {
          dao.saveOrUpdate(f);
      }

      //find and assert
      Optional<Facility> result = dao.findById("dfn-688");

      assertThat(result.isPresent()).isEqualTo(true);

      //delete to clean up
      dao.removeFromDB(result.get());

  }

 
  private static List<Facility> getFacilityList() {
      List<Facility> pList = new ArrayList<Facility>();

      Facility p1 = new Facility();
      p1.setFacilityId("dfn-688");
      p1.setName("Washington-Washington-DC");
      p1.setTimezone("America/New_York");

      Facility p2 = new Facility();
      p2.setFacilityId("dfn-777");
      p2.setName("Test VAMC3-Chantilly-VA");
      p2.setTimezone("America/New_York");

      pList.add(p1);
      pList.add(p2);

      return pList;
  }


}
