package gov.va.mobilehealth.courier.db;

import static org.fest.assertions.api.Assertions.assertThat;
import gov.va.mobilehealth.courier.core.Message;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.PatientMessageHistory;
import gov.va.mobilehealth.courier.core.PatientOutgoingMessage;
import gov.va.mobilehealth.courier.resources.helper.PatientMessageHandler;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.mongodb.BasicDBObject;
import com.mongodb.MongoClient;

public class PatientMessageHistoryDaoTest {

	private static MongoClient mongoClient;
	private static final String patientId = "52b9f1519d16ccbfab2fcff1";

	@BeforeClass
	public static void setup() throws IOException {
		MongoDBConnection.init("mongodb://localhost", "courier");
		mongoClient = new MongoClient();
		mongoClient.getDB("courier").getCollection("patientMessages").drop();
	}

	@AfterClass
	public static void tearDown() {
		MongoDBConnection.close();
	}

	@Test
	public void testFindbyId() throws IOException {
		this.createPatientMessageData();
		PatientMessageHistoryDAO dao = new PatientMessageHistoryDAO();
		List<PatientMessageHistory> phMsgList = dao.findByPatientId(patientId, false);
		List<PatientMessageHistory> phMsgList1 = this.getPatientMsgHistList();
		assertThat(phMsgList.size()).isEqualTo(2);
		assertThat(phMsgList).isEqualTo(phMsgList1);
		this.cleanupTestData();

	}

	private void cleanupTestData() {
		mongoClient.getDB("courier").getCollection("patientMessages")
				.remove(new BasicDBObject().append("patientId", patientId));
	}

	private void createPatientMessageData() throws IOException {
		PatientMessageDAO pmDao = new PatientMessageDAO();
		PatientIncomingMessage patientMsg = this.getPatientIncomingMessage();
		pmDao.insertIncomingMessage(patientMsg);
		PatientOutgoingMessage patientMsg1 = this.getPatientOutgoingMessage();
		pmDao.insertOutgoingMessage(patientMsg1);
	}

	private List<PatientMessageHistory> getPatientMsgHistList() {
		List<PatientMessageHistory> pmhList = new ArrayList<PatientMessageHistory>();
		PatientMessageHistory pmh = new PatientMessageHistory();
		PatientMessageHistory pmh1 = new PatientMessageHistory();		
		pmh.setId("22225");
		pmh.setNetworkSubmitTime("2014-02-01 10:30:00");
		pmh.setText("Hello Clinic");
		pmh.setType(MessageType.IN.toString());
		pmhList.add(pmh);
		pmh1.setId("22227");
		pmh1.setNetworkSubmitTime("2013-12-01 10:10:00");
		pmh1.setText("Hello");
		pmh1.setType(MessageType.OUT.toString());
		pmhList.add(pmh1);

		return pmhList;
	}

	private PatientIncomingMessage getPatientIncomingMessage() {
		PatientIncomingMessage patientMsg = new PatientIncomingMessage(
				"7030001111", "7034442222");
		patientMsg.setId("22225");
		patientMsg.setClientSubmitTime("2014-02-01 10:00:00");
		patientMsg.setNetworkSubmitTime("2014-02-01 10:30:00");
		patientMsg.setText("Hello Clinic");
		patientMsg.setClientMessageId("testing");
		patientMsg.setCostCentre("Centre");
		patientMsg.setMssageId("8000");
		patientMsg.setDestination("201-222-3333");
		patientMsg.setSource("222-333-4444");
		patientMsg.setSegmentCount(1);
		patientMsg.setReferenceNumber(10);
		patientMsg.setSegmentNumber(15);
		patientMsg.setType(MessageType.IN);
		patientMsg.setPatientId(patientId);

		return patientMsg;
	}

	private PatientOutgoingMessage getPatientOutgoingMessage() {
		PatientOutgoingMessage patientMsg = new PatientOutgoingMessage(
				"7031112222", "2021112222");
		patientMsg.setId("22227");
		patientMsg.setText("Hello");
		patientMsg.setNetworkSubmitTime("2013-12-01 10:10:00");
		patientMsg.setClientSubmitTime("2013-12-01 10:00:00");
		patientMsg.setType(MessageType.OUT);
		patientMsg.setPatientId(patientId);
		String[] msgDeliveryType = {PatientMessageHandler.APP_NAME};
		patientMsg.setMsgDeliveryType(msgDeliveryType);

		Message msg = new Message();
		msg.setMessageId("69879417");
		msg.setResponseCode("001");
		msg.setResponseString("OK");
		msg.setSuccessCode("0");
		msg.setId(null);
		List<Message> msgList = new ArrayList<Message>();
		msgList.add(msg);

		patientMsg.setMessages(msgList);
		return patientMsg;
	}

}
