package gov.va.mobilehealth.courier.db;

import static org.fest.assertions.api.Assertions.assertThat;
import gov.va.mobilehealth.courier.core.StaffRoles;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.StaffRole;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.google.common.base.Optional;
import com.mongodb.MongoClient;

public class StaffRoleDaoTest {

	private static MongoClient mongoClient;

	@BeforeClass
	public static void setup() throws IOException {
		MongoDBConnection.init("mongodb://localhost", "courier");
		mongoClient = new MongoClient();
	}

	@Before
	public void setUp() throws Exception {
		mongoClient.getDB("courier").getCollection("patients").drop();
	}

	@AfterClass
	public static void tearDown() {
		MongoDBConnection.close();
	}

	@Test
	public void canCreateStaffRole() throws IOException{
		StaffRoleDAO dao = new StaffRoleDAO();

		StaffRole p1 = getStaffRole();
		dao.update(p1);

		dao.removeFromDB(p1.getId());
	}

	@Test
	public void findUserByUserIdentifier() throws IOException {
		StaffRoleDAO dao = new StaffRoleDAO();
		StaffRole p1 = getStaffRole();
		dao.update(p1);

		Optional<StaffRole> user= dao.findByUserIdentifier(p1.getUserIdentifier());

		assertThat(user.get().getUserIdentifier().getAssigningAuthority()).isEqualTo("EDIPI");
		assertThat(user.get().getUserIdentifier().getUniqueId()).isEqualTo("PATID01");

		dao.removeFromDB(p1.getId());
	}

    @Test
    public void findUserByUserIdentifierAndRoleTest() throws IOException {
        StaffRoleDAO dao = new StaffRoleDAO();
        StaffRole p1 = getStaffRole();
        dao.update(p1);

        Optional<StaffRole> user= dao.findByUserIdentifierAndRole(p1.getUserIdentifier(), StaffRoles.admin);

        assertThat(user.get().getUserIdentifier().getAssigningAuthority()).isEqualTo("EDIPI");
        assertThat(user.get().getUserIdentifier().getUniqueId()).isEqualTo("PATID01");

        user= dao.findByUserIdentifierAndRole(p1.getUserIdentifier(), StaffRoles.bcm_admin);
        assertThat(user.isPresent()).isFalse();

        dao.removeFromDB(p1.getId());
    }


	private StaffRole getStaffRole() {
		UserIdentifier userIdentifier = getUserIdentifier();

		StaffRole staffRole = new StaffRole();
		String[] roles = {"admin"};
		staffRole.setRoles(roles);

		staffRole.setUserIdentifier(userIdentifier);

		return staffRole;
	}


	private UserIdentifier getUserIdentifier() {
		UserIdentifier userIdentifier = new UserIdentifier();
		userIdentifier.setUniqueId("PATID01");
		userIdentifier.setAssigningAuthority("EDIPI");
		return userIdentifier;
	}

}
