package gov.va.mobilehealth.courier.db;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;
import com.mongodb.util.JSON;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.Message;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.PatientMessage;
import gov.va.mobilehealth.courier.core.PatientMessageHistory;
import gov.va.mobilehealth.courier.core.PatientOutgoingMessage;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.protocol.PatientResponseStatus;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import com.google.common.base.Optional;

import org.bson.types.ObjectId;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class StartPatientResponseDAOTest {
  private static MongoClient mongoClient;
  private static final String messageId1 = "73b9f1519d16ccbfab2fcff1";
  private static final String messageId2 = "89b9f1519d16ccbfab2fcff2";
  private static final String patientId = "52b9f1519d16ccbfab2fcff4";
  private static final String startResponse = " Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word 'HELP'.";


  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientMessages").drop();
    mongoClient.getDB("courier").getCollection("patients").drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canUpdatePatient() throws IOException {
    PatientDAO patientDAO = new PatientDAO();
    Patient p1 = getPatient();
    patientDAO.update(p1);
    Patient p3 = null;
    PatientIncomingMessage patientIncomingMessage = getPatientResponse(messageId1, patientId);
    if (patientIncomingMessage.getText().equals("Start")) {
      Optional<Patient> patient = patientDAO.findByMDN(patientIncomingMessage.getSource());
      if (patient.isPresent()) {
        p3 = patient.get();
        if (p3.getPreference() != null) {
          p3.getPreference().setMessageStarted(true);
          patientDAO.update(p3);
        }
      }
    }
    Optional<Patient> patient2 = patientDAO.findByMDN(patientIncomingMessage.getSource());
    if (patient2.isPresent()) {
      boolean isMessageStarted = false;
      p3 = patient2.get();
      isMessageStarted = p3.getPreference().isMessageStarted();
      assertThat(isMessageStarted).isEqualTo(true);
      patientDAO.removeFromDB(p3.getId());
    }
  }

  @Test
  public void canInsertStart() throws IOException {
    PatientDAO patientDAO = new PatientDAO();
    PatientMessageDAO dao = new PatientMessageDAO();
    Patient p1 = getPatient();
    patientDAO.update(p1);
    Patient p3 = null;
    String pId = "";
    Optional<Patient> patient = patientDAO.findByMDN(p1.getPreference().getPhoneNumber());
    if (patient.isPresent()) {

      pId = patient.get().getId();
      PatientIncomingMessage patientIncomingMessage = getPatientResponse(messageId1, pId);
      if (patientIncomingMessage.getText().equals("Start")) {
        Optional<Patient> patient2 = patientDAO.findByMDN(patientIncomingMessage.getSource());
        if (patient2.isPresent()) {
          p3 = patient2.get();
          if (p3.getPreference() != null) {
            p3.getPreference().setMessageStarted(true);
            patientDAO.update(p3);
          }
        }
      }
      insertPatientIncomingMessage(patientIncomingMessage);
      PatientOutgoingMessage p2 = getPatientOutgoingMessage(messageId2, pId);
      insertPatientOutgoingMessage(p2);

      Collection<PatientMessage> patientMessages = dao.findAll();
      assertThat(patientMessages.size()).isEqualTo(2);
      patientDAO.removeFromDB(p1.getId());
      mongoClient.getDB("courier").getCollection("patientMessages").drop();
    }

  }


  private PatientIncomingMessage getPatientResponse(String messageId, String patientId) {
    PatientIncomingMessage p1 = (new PatientIncomingMessage("2223334444",
            "2012223333"));
    p1.setId(messageId);
    p1.setClientSubmitTime("2014-02-28 13:00:10");
    p1.setNetworkSubmitTime("2014-02-28 12:10:10");
    p1.setText("Start");
    p1.setClientMessageId("testing");
    p1.setCostCentre("Centre");
    p1.setMssageId("8000");
    p1.setDestination("2012223333");
    p1.setSource("2223334444");
    p1.setSegmentCount(1);
    p1.setReferenceNumber(10);
    p1.setSegmentNumber(15);
    p1.setPatientId(patientId);
    p1.setType(MessageType.IN);

    return p1;
  }


  private PatientOutgoingMessage getPatientOutgoingMessage(String messageId, String patientId) {
    PatientOutgoingMessage patientMsg = new PatientOutgoingMessage(
            "2012223333", "2223334444");

    patientMsg.setText(startResponse);
    patientMsg.setNetworkSubmitTime("2014-02-28 13:10:00");
    patientMsg.setClientSubmitTime("2014-02-28 10:00:00");
    patientMsg.setType(MessageType.OUT);
    patientMsg.setPatientId(patientId);
    patientMsg.setId(messageId);
    patientMsg.setType(MessageType.OUT);

    Message msg = new Message();
    msg.setMessageId("69879417");
    msg.setResponseCode("001");
    msg.setResponseString("OK");
    msg.setSuccessCode("0");
    msg.setId(null);
    List<Message> msgList = new ArrayList<Message>();
    msgList.add(msg);

    patientMsg.setMessages(msgList);
    return patientMsg;
  }

  private void insertPatientIncomingMessage(PatientIncomingMessage patientResponse) {
    PatientMessageDAO dao = new PatientMessageDAO();

    try {
      dao.insertIncomingMessage(patientResponse);
    } catch (Exception exp) {

    }
  }

  private void insertPatientOutgoingMessage(PatientOutgoingMessage patientResponse) {
    PatientMessageDAO dao = new PatientMessageDAO();

    try {
      dao.insertOutgoingMessage(patientResponse);
    } catch (Exception exp) {

    }

  }
  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");
    p1.setLastNameInitial("S");
    p1.setLast4SSN("3333");


    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("2223334444");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }

}