package gov.va.mobilehealth.courier.db;

import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class TimeZoneDAOTest {

  private static MongoClient mongoClient;
  protected DBCollection collection;
  private String id = "528a739eda06d04179851fa9";
  private String id2 = "528a739eda06d04179851faa";
  ;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    MongoClient mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("timeZones").drop();

    TimeZoneDAO dao = new TimeZoneDAO();

  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canUpdateAndFindOne() throws Exception {
    TimeZone t1 = new TimeZone();
    t1.setId(id);
    t1.setValue("-09:00");
    t1.setDescription("Alaska (GMT-09:00)");

    TimeZoneDAO dao = new TimeZoneDAO();
    this.collection = MongoDBConnection.getDB().getCollection("timeZones");

    dao.save(t1);
    TimeZone t2 = new TimeZone();
    t2 = dao.findById(id).get();
    if (t2 != null) {
      if (t2.getValue() != null)
        assertThat(t2.getValue()).isEqualTo("-09:00");
      if (t2.getDescription() != null)
        assertThat(t2.getDescription()).isEqualTo("Alaska (GMT-09:00)");
      BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));
      collection.remove(searchQuery);

    }


  }

  @Test
  public void canInsertAndFindAll() throws Exception {
    List<TimeZone> timeZones = new ArrayList<TimeZone>();
    TimeZoneDAO dao = new TimeZoneDAO();
    this.collection = MongoDBConnection.getDB().getCollection("timeZones");
    timeZones = dao.findAll();
    TimeZone t1 = new TimeZone();

    TimeZone t2 = new TimeZone();

    dao.insert(t1);
    dao.insert(t2);

    assertThat(t1.getId()).isNotNull();
    assertThat(t2.getId()).isNotNull();

    List<TimeZone> list = new ArrayList<TimeZone>();
    list = dao.findAll();
    assertThat(list.size()).isEqualTo(timeZones.size() + 2);
    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(t1.getId()));
    collection.remove(searchQuery);
    searchQuery = new BasicDBObject("_id", new ObjectId(t2.getId()));
    collection.remove(searchQuery);


    List<TimeZone> refreshList = new ArrayList<TimeZone>();
    refreshList = dao.findAll();
    assertThat(timeZones.size()).isEqualTo(refreshList.size());
  }


}