package gov.va.mobilehealth.courier.helper;

import com.mongodb.MongoClient;

import gov.va.mobilehealth.courier.core.DeliveryPreference;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.resources.helper.OutgoingMessageProcessor;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.apache.camel.CamelContext;
import org.apache.camel.Exchange;
import org.apache.camel.impl.DefaultCamelContext;
import org.apache.camel.impl.DefaultExchange;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;

public class OutgoingMessageProcessorTest {

  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("dictionary").drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  //Currently Integration test, would like to get it to pure unit test
  //Depends on Mongo
  //Test against scenario that prevented SMS messages from being sent out correctly where the Destination
  //was being incorrectly assigned to 'Annie'
  @Test
  public void messageDestinationIsPhoneNumber() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    DeliveryPreference deliveryPref = new DeliveryPreference();
    deliveryPref.setSendSms(true);
    p1.getPreference().setMessageDeliveryPreference(deliveryPref);
    updatePatient(p1);



    // create a message
    CMSMessage originalMessage = new CMSMessage("source", p1, "Sample Text", MessageType.OUT);
//    CMSMessage originalMessage = new CMSMessage();
//    originalMessage.setText("Sample Text");
//    originalMessage.setPatientId(p1.getId());
//    originalMessage.setType(MessageType.OUT.toString());  //Not sure if this is right
//    originalMessage.setDestination(p1.getPreference().getPhoneNumber());



    OutgoingMessageProcessor processor = new OutgoingMessageProcessor(new MessageProcessorFake());
    CamelContext context = new DefaultCamelContext();

    Exchange exchange = new DefaultExchange(context);
    exchange.getIn().setBody(originalMessage);
    processor.process(exchange);

    CMSMessage sentMessage = processor.getLastMessage();
    Assert.assertNotNull(sentMessage);
    Assert.assertTrue(sentMessage.isForceSendViaSMS());
    Assert.assertEquals(p1.getPreference().getPhoneNumber(), sentMessage.getDestination());
  }

  private void updatePatient(Patient p1) throws IOException {
    PatientDAO dao = new PatientDAO();
    dao.update(p1);
  }

  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }

}
