package gov.va.mobilehealth.courier.resources.helper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.smsgateway.types.Message;
import gov.va.mobilehealth.courier.core.PatientMessage;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.types.Response;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.mongodb.MongoClient;

public class PatientMessageHandlerTest {

  PatientMessageHandler patientMessageHandler;
  private static MongoClient mongoClient;

  @Before
  public void setUp() throws Exception {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientMessages").drop();
    patientMessageHandler = new PatientMessageHandler();
  }

  @After
  public void tearDown() throws Exception {
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientMessages").drop();
  }

  @Test
  public void testProcessSingleMessageResult() throws IOException {
    CMSMessage cmsMsg = new CMSMessage("", createPatient(), "", MessageType.OUT);
//    cmsMsg.setType("Out");
    cmsMsg.setForceSendViaSMS(true);

    Response response = new Response();
    ArrayList<Message> list = new ArrayList<Message>();
    response.setMessageList(list);

    patientMessageHandler.processSingleMessageResult(cmsMsg, response);

    PatientMessageDAO patientMessageDAO = new PatientMessageDAO();
    List<PatientMessage> patientMessageList = patientMessageDAO.findAll();
    PatientMessage psMsg = patientMessageList.get(0);
    psMsg.getMsgDeliveryType();
    String[] expectedResult = {"Annie", "SMS"};
    Assert.assertArrayEquals(expectedResult, psMsg.getMsgDeliveryType());
  }

  public Patient createPatient() {
    Patient patient = new Patient();
    patient.setPreference(new PatientPreference());
    return patient;
  }

  @Test
  public void testProcessSingleMessageResultSecure() throws IOException {
    CMSMessage cmsMsg = new CMSMessage("", createPatient(), "", MessageType.OUT);
//    cmsMsg.setType("Out");
//    cmsMsg.setForceSendViaSMS(false);

    Response response = new Response();
    ArrayList<Message> list = new ArrayList<Message>();
    response.setMessageList(list);

    patientMessageHandler.processSingleMessageResult(cmsMsg, response);

    PatientMessageDAO patientMessageDAO = new PatientMessageDAO();
    List<PatientMessage> patientMessageList = patientMessageDAO.findAll();
    PatientMessage psMsg = patientMessageList.get(0);
    psMsg.getMsgDeliveryType();
    String[] expectedResult = {"Annie", null};
    Assert.assertArrayEquals(expectedResult, psMsg.getMsgDeliveryType());
  }

}
