package gov.va.mobilehealth.courier.resources.helper;

import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.aeonbits.owner.ConfigFactory;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

/**
 * Created by shimk on 9/5/14.
 */
public class ValidateUniquePhoneNumberTest {
  private static MongoClient mongoClient;
  private static ValidateUniquePhoneNumber validateUniquePhoneNumber;
  static ErrorMessageConfigure errorMessageConfig = getErrorMessageConfig();
  private static final Logger LOGGER = LoggerFactory.getLogger(ValidateUniquePhoneNumberTest.class);
  private static PatientDAO patientDAO;

  static ErrorMessageConfigure getErrorMessageConfig() {
    return ConfigFactory.create(ErrorMessageConfigure.class, System.getProperties(), System.getenv());
  }

  @Before
  public void setUp() throws Exception {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patients").drop();
    validateUniquePhoneNumber = new ValidateUniquePhoneNumber();
  }

  @After
  public void tearDown() throws Exception {
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientMessages").drop();
  }

  @Test
  public void throwsOnExistingPhoneNumber() throws IOException {
    patientDAO = getPatientDAO();
    Patient p1 = getPatient();
    patientDAO.update(p1);

    try {
      validateUniquePhoneNumber.checkUniquePhoneNumber(p1.getPreference().getPhoneNumber());
      fail("Expected an WebApplicationException to be thrown");

    } catch (WebApplicationException e) {
      Response response = e.getResponse();

      assertEquals(response.getStatus(), Response.Status.BAD_REQUEST.getStatusCode());

      ErrorResponse errorResponse = (ErrorResponse) response.getEntity();

      assertEquals(errorResponse.getMessage(), errorMessageConfig.duplicateMobilePhoneNumber());
      assertEquals(errorResponse.getCode(), ErrorCode.MSG_DUPLICATE_MOBILE_PHONE_NUMBER_FOUND);
    }
    patientDAO.removeFromDB(p1.getId());
  }

  @Test
  public void noExceptionOnUniquePhoneNumber() throws IOException {
    Patient p1 = getPatient();

    p1.getPreference().setPhoneNumber("1-222-555-8888");  //this phone number does not exist

    try {
      validateUniquePhoneNumber.checkUniquePhoneNumber(p1.getPreference().getPhoneNumber());

    } catch (WebApplicationException e) {
        fail("Should not have thrown any exception");
      Response response = e.getResponse();

      assertEquals(response.getStatus(), Response.Status.OK);
      }

  }

  private PatientDAO getPatientDAO(){
    PatientDAO patientDAO = new PatientDAO();
    return patientDAO;
  }
  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }
}
