package gov.va.mobilehealth.courier.bcm.appointments;

import gov.va.mobilehealth.courier.bcm.appointments.config.BcmAppointmentsConfig;
import org.aeonbits.owner.ConfigFactory;
import org.joda.time.DateTime;
import org.joda.time.Minutes;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by clarkee on 10/30/14.
 */
public class AppointmentReminderGenerator {
  public List<AppointmentReminder> createAppointmentReminders(List<AssessmentProfileFutureAppointment> appointments) {
    List<AppointmentReminder> reminders = new ArrayList<AppointmentReminder>();

    for (AssessmentProfileFutureAppointment appointment : appointments) {
      reminders.addAll(createAppointmentReminders(appointment));

    }
    return reminders;
  }


  public List<AppointmentReminder> createAppointmentReminders(AssessmentProfileFutureAppointment appointment) {
    List<AppointmentReminder> reminderList = new ArrayList<AppointmentReminder>();

    int minutes = Minutes.minutesBetween(DateTime.now(), appointment.getStartDate()).getMinutes();

    BcmAppointmentsConfig bcmApptConfig = getBcmApptConfig();
    List<Integer> reminderIntervals = bcmApptConfig.reminderIntervals();

    for (Integer reminderInterval : reminderIntervals) {
      int minutesInAdvance;
      minutesInAdvance = reminderInterval;
      if (minutes > minutesInAdvance){
        reminderList.add(createReminder(appointment, minutesInAdvance));
      }
    }
    return reminderList;
  }

  private AppointmentReminder createReminder(AssessmentProfileFutureAppointment appointment, int minutesInAdvance) {
    AppointmentReminder reminder = new AppointmentReminder();
    reminder.setDeliveryDateTime(appointment.getStartDate().minus(Minutes.minutes(minutesInAdvance)));
    reminder.setFutureAppointment(appointment);
    return reminder;
  }

  static BcmAppointmentsConfig getBcmApptConfig() {
    BcmAppointmentsConfig config = ConfigFactory.create(BcmAppointmentsConfig.class, System.getProperties(), System.getenv());
    return config;
  }

}
