package gov.va.mobilehealth.courier.bcm.appointments;

import com.agilex.healthcare.hdr.appointment.datalayer.AppointmentRepository;
import com.agilex.healthcare.hdr.config.HDRConfig;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import gov.va.mobilehealth.courier.bcm.appointments.config.BcmHdrConfig;
import org.aeonbits.owner.ConfigFactory;

import java.util.List;

/**
 * Created by clarkee on 10/27/14.
 */
public class AppointmentsService {
  private HDRConfig config;
  private static final String AppointmentStatusKeptScheduled = "R";

  public AppointmentsService() {
    //Translate Owner based config to config CDSLib requires
    BcmHdrConfig config = getBcmHdrConfig();
    this.config = new HDRConfig(config.hdrEndPointUrl(), config.hdrConnectionTimeout(), config.hdrReadTimeout(), config.hdrClientName());
  }

  static BcmHdrConfig getBcmHdrConfig() {
    BcmHdrConfig config = ConfigFactory.create(BcmHdrConfig.class, System.getProperties(), System.getenv());
    return config;
  }

  /**
   * Retrieve the future appointments for a patient
   * @param edipi
   * @param dateFilter
   * @return
   */
  public Appointments getFutureAppointments(EdipiIdentifier edipi, DateFilter dateFilter) {

    final MpiService mpiService = new MpiService();
    final AppointmentsService appointmentsService = new AppointmentsService();
    final Appointments appointments = new Appointments();

    List<IcnIdentifier> icnIdentifiers = mpiService.getIcnIdentifiers(edipi);

    for (IcnIdentifier icnIdentifier : icnIdentifiers) {
      appointments.addAll(appointmentsService.getFutureAppointments(icnIdentifier, dateFilter));
    }
    return appointments;
  }

  public com.agilex.healthcare.mobilehealthplatform.domain.Appointments getFutureAppointments(IcnIdentifier icnIdentifer, DateFilter dateFilter) {


    Appointments scheduledKeptAppointments = filterForKeptAppointments(getAllAppointments(icnIdentifer, dateFilter));
    return scheduledKeptAppointments;
  }

  private Appointments filterForKeptAppointments(Appointments allAppointments) {
    Appointments keptAppointments = new Appointments();
    for (Appointment appointment : allAppointments) {
      if (appointment.getStatusCode().equals(AppointmentStatusKeptScheduled)){
        keptAppointments.add(appointment);
      }
    }
    return keptAppointments;
  }

  public Appointments getAllAppointments(IcnIdentifier icnIdentifier, DateFilter dateFilter) {
    AppointmentRepository repo = new AppointmentRepository(config);
    PatientIdentifier patientIdentifier = icnIdentifier.getPatientIdentfier();
    return repo.fetchAppointmentsByPatient(patientIdentifier, dateFilter, true);

  }
}
