package gov.va.mobilehealth.courier.bcm.appointments;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.google.common.base.Objects;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import org.joda.time.DateTime;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({
  "facility",
  "startDate"
})
public class AssessmentProfileFutureAppointment extends MongoDocumentBase {

  @JsonProperty("localStartDate")
  DateTime startDate;

  @JsonProperty("facility")
  private String facility;

  public static final String PatientIcnIdentifierFieldName = "patientIcnIdentifier";
  @JsonProperty(PatientIcnIdentifierFieldName)
  private IcnIdentifier patientIcnIdentifier;

  public static final String PatientEdipiIdentifierFieldName = "patientEdipiIdentifier";
  @JsonProperty(PatientEdipiIdentifierFieldName)
  private EdipiIdentifier patientEdipiIdentifier;

  private static final String ClinicPhoneFieldName = "clinicPhone";
  @JsonProperty(ClinicPhoneFieldName)
  private String clinicPhone;

  public AssessmentProfileFutureAppointment() {
  }

  @JsonProperty("localStartDate")
  public DateTime getStartDate() {
    return startDate;
  }

  @JsonProperty("localStartDate")
  public void setStartDate(DateTime startDate) {
    this.startDate = startDate;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof AssessmentProfileFutureAppointment)) {
      return false;
    }

    AssessmentProfileFutureAppointment appointment = (AssessmentProfileFutureAppointment) o;

    if (startDate != null ? !startDate.equals(appointment.startDate) : appointment.startDate != null) {
      return false;
    }
    if (facility != null ? !facility.equals(appointment.facility) : appointment.facility != null) {
      return false;
    }
    if (patientIcnIdentifier != null ? !patientIcnIdentifier.equals(appointment.patientIcnIdentifier) : appointment.patientIcnIdentifier != null) {
      return false;
    }
    if (patientEdipiIdentifier != null ? !patientEdipiIdentifier.equals(appointment.patientEdipiIdentifier) : appointment.patientEdipiIdentifier != null) {
      return false;
    }
    if (clinicPhone != null ? !clinicPhone.equals(appointment.clinicPhone) : appointment.clinicPhone != null) {
      return false;
    }

    return true;
  }

  @Override
  public int hashCode() {
    int result = (startDate != null ? startDate.hashCode() : 0);
    result = 31 * result + (facility != null ? facility.hashCode() : 0);
    result = 31 * result + (patientIcnIdentifier != null ? patientIcnIdentifier.hashCode() : 0);
    result = 31 * result + (patientEdipiIdentifier != null ? patientEdipiIdentifier.hashCode() : 0);
    result = 31 * result + (clinicPhone != null ? clinicPhone.hashCode() : 0);

    return result;
  }

  @Override
  public String toString() {
    return Objects.toStringHelper(this)
      .add("startDate", startDate)
      .add("facility", facility)
      .add("patientIcnIdentifier", patientIcnIdentifier)
      .add("patientEdipiIdentifier", patientEdipiIdentifier)
      .add("clinicPhone", clinicPhone)
      .toString();
  }

  @JsonProperty("facility")
  public void setFacility(String facility) {
    this.facility = facility;
  }

  @JsonProperty("facility")
  public String getFacility() {
    return facility;
  }

  @JsonProperty(PatientIcnIdentifierFieldName)
  public void setPatientIcnIdentifier(IcnIdentifier patientIdentifier) {
    this.patientIcnIdentifier = patientIdentifier;
  }

  @JsonProperty(PatientIcnIdentifierFieldName)
  public IcnIdentifier getPatientIcnIdentifier() {
    return patientIcnIdentifier;
  }
  @JsonProperty(PatientEdipiIdentifierFieldName)
  public void setPatientEdipiIdentifier(EdipiIdentifier patientIdentifier) {
    this.patientEdipiIdentifier = patientIdentifier;
  }

  @JsonProperty(PatientEdipiIdentifierFieldName)
  public EdipiIdentifier getpatientEdipiIdentifier() {
    return patientEdipiIdentifier;
  }

  @JsonProperty(ClinicPhoneFieldName)
  public void setClinicPhone(String clinicPhone) {
    this.clinicPhone = clinicPhone;
  }

  @JsonProperty(ClinicPhoneFieldName)
  public String getClinicPhone() {
    return clinicPhone;
  }
}
