package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.utils.ServerConfig;
import org.aeonbits.owner.ConfigFactory;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.ee.servlet.QuartzInitializerListener;
import org.quartz.impl.StdSchedulerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

public class CmsQuartzListener  implements ServletContextListener {

  private ServletContextListener defaultQuartzListener = new QuartzInitializerListener();
  private ServletContextListener bcmQuartzListener = new BcmQuartzInitializerListener();

  private static final Logger LOGGER = LoggerFactory.getLogger(CmsQuartzListener.class);

  private Scheduler scheduler;
  private Scheduler bcmScheduler;

  static ServerConfig getServerConfig() {
    ServerConfig config = ConfigFactory.create(ServerConfig.class, System.getProperties(), System.getenv());
    return config;
  }

  @Override
  public void contextDestroyed(ServletContextEvent arg0) {
    ServerConfig serverConfig = getServerConfig();
    if (serverConfig.quartzBcmEnabled()){


      if (scheduler != null){
        try {
          LOGGER.debug("Shutting down Quartz Scheduler");
          scheduler.shutdown(true);
          LOGGER.debug("Shutting down Quartz Scheduler Complete");
        } catch (SchedulerException e) {
          LOGGER.debug("CmsQuartzListener Exception");
          LOGGER.debug(e.toString());
        }
      }

      bcmQuartzListener.contextDestroyed(arg0);

    }
    //Call base Quartz Implementation
    defaultQuartzListener.contextDestroyed(arg0);

  }

  @Override
  public void contextInitialized(ServletContextEvent ctx) {
    ServerConfig serverConfig = getServerConfig();

    //Call base Quartz implementation
    defaultQuartzListener.contextInitialized(ctx);
    initializeQuartzSchedulerFactory(ctx);

    if (serverConfig.quartzBcmEnabled()){
      bcmQuartzListener.contextInitialized(ctx);
      initializeBcmQuartzSchedulerFactory(ctx);
      initializeAppointmentReminderSchedule();
    }
  }

  private void initializeAppointmentReminderSchedule() {

    InitiateAllAppointmentRemindersJobCreator creator = new InitiateAllAppointmentRemindersJobCreator();
    creator.createJob();
  }

  private void initializeQuartzSchedulerFactory(ServletContextEvent ctx) {
    try {
      LOGGER.debug("Getting scheduler from ServletContext");
      scheduler = ((StdSchedulerFactory) ctx.getServletContext().getAttribute(QuartzInitializerListener.QUARTZ_FACTORY_KEY)).getScheduler();
      LOGGER.debug("Setting QuartzSchedulerFactory.setScheduler()");
      QuartzSchedulerFactory.setScheduler(scheduler);
      LOGGER.debug("DONE Setting QuartzSchedulerFactory.setScheduler()");
    } catch (SchedulerException e) {
      LOGGER.debug("CmsQuartzListener Exception");
      LOGGER.debug(e.toString());
    }
  }

  private void initializeBcmQuartzSchedulerFactory(ServletContextEvent ctx) {
    try {
      LOGGER.debug("Getting Bcm scheduler from ServletContext");
      bcmScheduler = ((StdSchedulerFactory) ctx.getServletContext().getAttribute(BcmQuartzInitializerListener.QUARTZ_FACTORY_KEY)).getScheduler();
      LOGGER.debug("Setting QuartzSchedulerFactory.setBcmScheduler()");
      QuartzSchedulerFactory.setBcmScheduler(bcmScheduler);
      LOGGER.debug("DONE Setting QuartzSchedulerFactory.setBcmScheduler()");
    } catch (SchedulerException e) {
      LOGGER.debug("CmsQuartzListener Exception");
      LOGGER.debug(e.toString());
    }
  }


}
