package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.bcm.appointments.config.BcmAppointmentsConfig;
import gov.va.mobilehealth.courier.scheduler.SchedulerBase;
import org.aeonbits.owner.ConfigFactory;
import org.quartz.Job;
import org.quartz.JobDetail;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.quartz.JobBuilder.newJob;

public class InitiateAllAppointmentRemindersJobCreator extends SchedulerBase {

  private static final Logger LOGGER = LoggerFactory.getLogger(InitiateAllAppointmentRemindersJobCreator.class);

  public static final String JOB_APPOINTMENT_INIT_NAME = "appointmentInitName";
  public static final String JOB_APPOINTMENT_INIT_GROUP = "appointmentInitGroup";
  public static final String JOB_APPOINTMENT_INIT_TRIGGER_GROUP = "appointmentInitGroup";

  protected Class<? extends Job> InitiateAllAppointmentRemindersJob;

  public InitiateAllAppointmentRemindersJobCreator() {
    this.scheduler = QuartzSchedulerFactory.getBcmScheduler();
    this.InitiateAllAppointmentRemindersJob = InitiateAllAppointmentRemindersJob.class;
  }


  protected Class<? extends Job> GetInitiateAllAppointmentRemindersJob() {
    return this.InitiateAllAppointmentRemindersJob;
  }

  static BcmAppointmentsConfig getBcmApptConfig() {
    BcmAppointmentsConfig config = ConfigFactory.create(BcmAppointmentsConfig.class, System.getProperties(), System.getenv());
    return config;
  }

  public void createJob() {

    //There should only be one instance of this job, hence the hardcoded name
    String jobName = JOB_APPOINTMENT_INIT_NAME;
    String jobGroup = JOB_APPOINTMENT_INIT_GROUP;
    String triggerGroup = JOB_APPOINTMENT_INIT_TRIGGER_GROUP;

    try {
      cleanJob(jobName, jobGroup, triggerGroup);

      JobDetail job = newJob(GetInitiateAllAppointmentRemindersJob())
        .withIdentity(jobName, jobGroup)
        .build();

      //TODO: Make chron time configurable
      BcmAppointmentsConfig config = getBcmApptConfig();
      String chronTime = config.appointmentInitChronSchedule();

      //TODO: Make Timezone configurable
      scheduleCronTriggerJob(job, jobName, jobGroup, chronTime, null, "America/New_York");

    } catch (SchedulerException e) {
      LOGGER.error("Scheduler exception", e);
    }

  }

}
