package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.List;

/**
 * Created by clarkee on 11/1/14.
 */
public class InitiateAllAppointmentRemindersWorker {
//  ProducerTemplate messageProducer;
  private static final Logger LOGGER = LoggerFactory.getLogger(InitiateAllAppointmentRemindersWorker.class);

  public InitiateAllAppointmentRemindersWorker() {
  }


  /**
   * Query for patients that have appointment reminder preferences
   * For each result, create a new Job/QueueItem
    * @throws IOException
   */
  public void execute() throws IOException{
    PatientDAO patientDAO = DAOFactory.createPatientDAO();

    List<Patient> patients = patientDAO.findAppointmentReminderPatients();

    for (Patient patient : patients) {
      createRefreshQueueItem(patient);
    }
  }

  private void createRefreshQueueItem(Patient patient) {
    //send message to outbound queue
    LOGGER.debug("Creating Refresh Patient Appointment queue item");
    ProducerTemplate producer = CamelProducerFactory.getRefreshPatientAppointmentsProducer();

    LOGGER.debug("got the Producer,sending the message");
    //Only need the patient edipi id
    try {
      producer.sendBody(patient.getPatientIdentifier().getUniqueId());
    } catch (Exception ex) {
      LOGGER.debug("Unable to create queue item");
      LOGGER.debug(String.valueOf(ex.getStackTrace()));
    }
  }
}
