package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.bcm.appointments.AppointmentReminder;
import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.scheduler.SchedulerBase;
import org.joda.time.Hours;
import org.quartz.Job;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.SchedulerException;
import org.quartz.impl.matchers.GroupMatcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.List;
import java.util.Set;

import static org.quartz.JobBuilder.newJob;

/**
 * Created by clarkee on 10/31/14.
 */
public class SendAppointmentReminderJobCreator extends SchedulerBase {
  private static final Logger LOGGER = LoggerFactory.getLogger(SendAppointmentReminderJobCreator.class);
  private static final String JOB_APPOINTMENT_REMINDER_GROUP_PREFIX = "futureAppointmentReminderGroup";
  public static final String JOB_APPOINTMENT_REMINDER_TRIGGER_GROUP = "futureAppointmentReminderTriggerGroup";

  protected Class<? extends Job> SendAppointmentReminderJobClass;

  public SendAppointmentReminderJobCreator() {

    this.scheduler = QuartzSchedulerFactory.getBcmScheduler();
    this.SendAppointmentReminderJobClass = SendAppointmentReminderJob.class;
  }


  protected Class<? extends Job> GetSendAppointmentReminderJobClass() {
    return this.SendAppointmentReminderJobClass;
  }

  public void createReminderJobs(List<AppointmentReminder> reminders, EdipiIdentifier edipiIdentifier) {
    cleanJobsForPatient(edipiIdentifier);
    for (int i = 0; i < reminders.size(); i++) {
      AppointmentReminder reminder = reminders.get(i);
      createReminderJob(reminder, getReminderJobName(reminder, i));
    }
  }

  public void cleanJobsForPatient(EdipiIdentifier edipiIdentifier) {
      try {
        String reminderJobGroupName = getReminderJobGroupName(edipiIdentifier);
        cleanJobsForPatient(reminderJobGroupName);
      } catch (SchedulerException e) {
        LOGGER.debug("Unable to clean Appointment Reminder job");
        LOGGER.debug(String.valueOf(e.getStackTrace()));
      }
  }

  private void cleanJobsForPatient(String reminderJobGroupName) throws SchedulerException {
    GroupMatcher<JobKey> matcher = GroupMatcher.groupEquals(reminderJobGroupName);
    Set<JobKey> jobKeys = scheduler.getJobKeys(matcher);
    for (JobKey jobKey : jobKeys) {
      cleanJob(jobKey.getName(), reminderJobGroupName, JOB_APPOINTMENT_REMINDER_TRIGGER_GROUP);
    }
  }

  private void createReminderJob(AppointmentReminder reminder, String reminderJobName) {

    String jobName = reminderJobName;
    String jobGroup = getReminderJobGroupName(reminder);

    //The cleanJob will only clean if the appointment id has not changed
//      cleanJob(jobName, jobGroup, triggerGroup);

    JobDetail job = newJob(GetSendAppointmentReminderJobClass())
      .withIdentity(jobName, jobGroup)
      .usingJobData(SendAppointmentReminderJob.AppointmentReminderMessageKey, reminder.getMessageContent())
      .usingJobData(SendAppointmentReminderJob.PatientEdipiIdKey, reminder.getFutureAppointment().getpatientEdipiIdentifier().getEdipiId())
      .build();

    //Use delivery date from reminder message as exact execution time
    Date scheduleTime = reminder.getDeliveryDateTime().toDate();
    //Give it 24 hours to execute
    Date scheduleEndTime = reminder.getDeliveryDateTime().plus(Hours.hours(24)).toDate();

    scheduleSimpleTriggerJob(job, jobName, jobGroup, scheduleTime, scheduleEndTime);

  }

  private String getReminderJobName(AppointmentReminder reminder, int i) {
    return reminder.getFutureAppointment().getId() + "_" + i;
  }

  public static String getReminderJobGroupName(AppointmentReminder reminder) {
    return JOB_APPOINTMENT_REMINDER_GROUP_PREFIX + reminder.getFutureAppointment().getpatientEdipiIdentifier().getEdipiId();
  }
  public static String getReminderJobGroupName(EdipiIdentifier edipiIdentifier) {
    return JOB_APPOINTMENT_REMINDER_GROUP_PREFIX + edipiIdentifier.getEdipiId();
  }

}
