package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

/**
 * Created by clarkee on 11/1/14.
 */
public class SendAppointmentReminderWorker {
  private final String edipiId;
  private final String message;
  ProducerTemplate messageProducer;
  private static final Logger LOGGER = LoggerFactory.getLogger(SendAppointmentReminderWorker.class);

  public SendAppointmentReminderWorker(ProducerTemplate messageProducer, String edipiId, String message){
    this.edipiId = edipiId;
    this.message = message;
    this.messageProducer = messageProducer;

  }
  public SendAppointmentReminderWorker(String edipiId, String message) {
    this.edipiId = edipiId;
    this.message = message;
    this.messageProducer = CamelProducerFactory.getOutgoingMsgProducer();
  }

  /**
   * Find the patient, create the message,and put it on the queue to send via app/sms
   * @throws IOException
   */
  public void execute() throws IOException{
    Patient patient;
    PatientDAO patientDAO = DAOFactory.createPatientDAO();
    PatientIdentifier patientIdentifier = (new EdipiIdentifier(edipiId)).getPatientIdentfier();
    Optional<Patient> result = patientDAO.findByPatientIdentifier(patientIdentifier);

    if (!result.isPresent()) {
      LOGGER.info("Patient Not Found: EDIPI " + edipiId);
      throw new RuntimeException("Patient Not Found: EDIPI " + edipiId);
    }
    else{
       patient = result.get();
    }

    CMSMessage cmsMsg = new CMSMessage(null, patient, this.message, MessageType.OUT);

    messageProducer.sendBody(cmsMsg);
  }
}
