package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.dictionary.Dictionary;
import gov.va.mobilehealth.courier.core.protocol.dictionary.DictionaryItem;
import gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary;
import gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;

public class DictionaryDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
  protected final DBCollection collection;

  public DictionaryDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("dictionary");
  }

  public String insertTimeZone(TimeZone dictionary) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(dictionary));

    collection.insert(dbObject);
    ObjectId oId = (ObjectId) dbObject.get("_id");
    dictionary.setId(oId.toString());
    return dictionary.getId();

  }

  public String insertService(ServiceDictionary dictionary) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(dictionary));

    collection.insert(dbObject);
    ObjectId oId = (ObjectId) dbObject.get("_id");
    dictionary.setId(oId.toString());
    return dictionary.getId();

  }

  public String insert(Dictionary dictionary) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(dictionary));

    collection.insert(dbObject);
    ObjectId oId = (ObjectId) dbObject.get("_id");
    dictionary.setId(oId.toString());
    return dictionary.getId();

  }

    /*public List<ServiceDictionary> findAll() throws IOException {
    List<ServiceDictionary> list = new ArrayList<ServiceDictionary>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        list.add(MAPPER.readValue(cursor.next().toString(), ServiceDictionary.class));
      }
    } finally {
      cursor.close();
    }

    return list;
  }   */

  public Dictionary findAll() throws IOException {
    Dictionary dictionary = new Dictionary();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        DictionaryItem dictionaryItem = MAPPER.readValue(cursor.next().toString(), DictionaryItem.class);


        if (dictionaryItem instanceof ServiceDictionary) {
          ServiceDictionary item = (ServiceDictionary) dictionaryItem;

          dictionary.getServices().add(item);
        } else if (dictionaryItem instanceof TimeZone) {
          TimeZone item = (TimeZone) dictionaryItem;
          dictionary.getTimeZones().add(item);
        }
      }
    } finally {
      cursor.close();
    }

    return dictionary;
  }
}
