package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.WriteResult;
import com.mongodb.util.JSON;

import gov.va.mobilehealth.courier.core.Facility;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FacilityDAO {
    private static final ObjectMapper MAPPER = MongoDocumentBase
      .newCustomObjectMapper();

    protected final DBCollection collection;
    private final String facilityMessagesCollection = "facility";

    public FacilityDAO() {

      this.collection = MongoDBConnection.getDB().getCollection(
        facilityMessagesCollection);
    }

    public Optional<Facility> findById(String id) throws IOException {
      DBObject dbObject = collection.findOne(new BasicDBObject("_id",
        id));

      Preconditions
        .checkNotNull(dbObject, "facility Id is null");

      return Optional.fromNullable(MAPPER.readValue(dbObject.toString(),
        Facility.class));
    }

    public List<Facility> findAll() throws IOException {
      List<Facility> messages = new ArrayList<Facility>();
      DBCursor cursor = collection.find();
      try {
        while (cursor.hasNext()) {
          messages.add(MAPPER.readValue(cursor.next().toString(),
                  Facility.class));
        }
      } finally {
        cursor.close();
      }

      return messages;
    }

    public int removeFromDB(Facility facility) throws IOException {

      if (null == facility || null == facility.getFacilityId()) {
        return -1;
      }
      BasicDBObject searchQuery = new BasicDBObject().append("_id", facility.getFacilityId());

      WriteResult wr = collection.remove(searchQuery);
      return wr.getN();
    }

    public Facility saveOrUpdate(Facility facility) throws IOException {
      ObjectId oId;

      DBObject dbObject = (DBObject) JSON.parse(MAPPER
        .writeValueAsString(facility));
      dbObject.put("_id", facility.getFacilityId());
      BasicDBObject searchQuery = new BasicDBObject().append("_id", facility.getFacilityId());
      collection.update(searchQuery, dbObject, true, false);
      return facility;

    }
  }
