package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Preconditions;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.*;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class PatientMessageDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();

  protected final DBCollection collection;

  public PatientMessageDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("patientMessages");
  }

  public List<PatientMessage> findAll() throws IOException {
    List<PatientMessage> patientResponses = new ArrayList<PatientMessage>();
    DBCursor cursor = collection.find();
    PatientMessage p;
    DBObject dbo;

    try {
      while (cursor.hasNext()) {
        dbo = cursor.next();
        if (MessageType.IN.toString().equalsIgnoreCase(dbo.get("type").toString())) {
          p = MAPPER.readValue(dbo.toString(), PatientIncomingMessage.class);
        } else {
          p = MAPPER.readValue(dbo.toString(), PatientOutgoingMessage.class);
        }
        //p.getAgeFromDoB();
        patientResponses.add(p);
      }
    } finally {
      cursor.close();
    }

    return patientResponses;
  }

  public void insertIncomingMessage(PatientIncomingMessage patientResponse) throws IOException {
    // use regular mapper here, since we do not want to use the MAPPER between
    // MongoDB objectID and string _id
    ObjectMapper regularMAPPER = Jackson.newObjectMapper();
    DBObject dbObject = (DBObject) JSON.parse(regularMAPPER
      .writeValueAsString(patientResponse));
    collection.insert(dbObject);
    if (patientResponse.getId() == null) {
      ObjectId oId = (ObjectId) dbObject.get("_id");
      patientResponse.setId(oId.toString());
    }
  }

  public void insertOutgoingMessage(PatientOutgoingMessage patientMessage) throws IOException {
    // use regular mapper here, since we do not want to use the MAPPER between
    // MongoDB objectID and string _id
    ObjectMapper regularMAPPER = Jackson.newObjectMapper();
    DBObject dbObject = (DBObject) JSON.parse(regularMAPPER
      .writeValueAsString(patientMessage));
    collection.insert(dbObject);
    if (patientMessage.getId() == null) {
      ObjectId oId = (ObjectId) dbObject.get("_id");
      patientMessage.setId(oId.toString());
    }
  }

  public void updateStatus(CMSMessage message) throws IOException {
    Preconditions.checkNotNull(message.getIncomingMessageId(), "protocol ID cannot be null");
    Preconditions.checkNotNull(message.getAlertLevel(), "status cannot be null");

    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(message.getIncomingMessageId()));

    BasicDBObject newDocument = new BasicDBObject("$set", new BasicDBObject()
      .append("alertLevel", message.getAlertLevel()).append("serviceName", message.getServiceName()));

    collection.update(searchQuery, newDocument);
  }

}
