package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.QueryBuilder;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.PatientMessageHistory;
import gov.va.mobilehealth.courier.resources.helper.PatientMessageHandler;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class PatientMessageHistoryDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();

  protected final DBCollection collection;

  public PatientMessageHistoryDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("patientMessages");
  }


  public List<PatientMessageHistory> findByPatientId(String patientId, boolean sortByDate) throws IOException {
    List<PatientMessageHistory> messageHists = new ArrayList<PatientMessageHistory>();
    BasicDBObject fields = new BasicDBObject();
    fields.put("type", 1);
    fields.put("text", 1);
    fields.put("networkSubmitTime", 1);
    fields.put("_id", 1);
    fields.put("alertLevel", 1);
    fields.put("serviceName", 1);
    fields.put("timeZone", 1);
    fields.put("adjustedSubmitTime", 1);
    fields.put("destination", 1);

    DBObject query = QueryBuilder.start("patientId").is(patientId).or(
            QueryBuilder.start().and("type").is(MessageType.OUT.toString()).
                and("msgDeliveryType").is(PatientMessageHandler.APP_NAME).get(),
            QueryBuilder.start("type").notEquals(MessageType.OUT.toString()).get()
         ).get();
    
    DBCursor cursor = collection.find(query, fields);
    
    if (sortByDate) {
        cursor.sort(new BasicDBObject("networkSubmitTime", 1).append("type", 1));
    }

    PatientMessageHistory p;

    try {
      while (cursor.hasNext()) {
        p = MAPPER.readValue(cursor.next().toString(), PatientMessageHistory.class);
        messageHists.add(p);
      }
    } finally {
      cursor.close();
    }

    return messageHists;
  }

}
