package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.PatientPreferenceAuditTrail;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


public class PatientPreferenceAuditTrailDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();

  protected final DBCollection collection;

  public PatientPreferenceAuditTrailDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("patientPreferenceAuditTrailData");
  }

  public List<PatientPreferenceAuditTrail> findByPatientIdentifier(UserIdentifier patientIdentifier) throws IOException {

    BasicDBObject whereQuery = new BasicDBObject();
    whereQuery.append("patientIdentifier.uniqueId", patientIdentifier.getUniqueId());
    whereQuery.append("patientIdentifier.assigningAuthority", patientIdentifier.getAssigningAuthority());
    return findByQuery(whereQuery);
  }


  public void save(PatientPreferenceAuditTrail patientPreferenceAuditTrail) throws IOException {
    //use regular mapper here, since we do not want to use the MAPPER between
    // MongoDB objectID and string _id
    ObjectMapper regularMAPPER = Jackson.newObjectMapper();
    DBObject dbObject = (DBObject) JSON.parse(regularMAPPER.writeValueAsString(patientPreferenceAuditTrail));
    collection.insert(dbObject);

  }

  protected List<PatientPreferenceAuditTrail> findByQuery(BasicDBObject query) throws IOException {
    List<PatientPreferenceAuditTrail> patientPreferenceAuditTrailList = new ArrayList<PatientPreferenceAuditTrail>();

    PatientPreferenceAuditTrail p;
    DBObject dbo;
    DBCursor cursor = collection.find(query);
    try {
      while (cursor.hasNext()) {
        dbo = cursor.next();
        p = MAPPER.readValue(dbo.toString(), PatientPreferenceAuditTrail.class);
        patientPreferenceAuditTrailList.add(p);
      }
    } finally {
      cursor.close();
    }

    return patientPreferenceAuditTrailList;
  }


}
