package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.*;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ProtocolDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase
    .newCustomObjectMapper();

  protected final DBCollection collection;

  public ProtocolDAO() {
    this.collection = MongoDBConnection.getDB().getCollection(
      "protocols");
  }

  public Optional<Protocol> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id",
      new ObjectId(id)));

    Preconditions
      .checkNotNull(dbObject, "template ID does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(),
      Protocol.class));
  }

  public Optional<Protocol> update(Protocol protocol) throws IOException {
    BasicDBObject dbObject = MAPPER.readValue(
      MAPPER.writeValueAsString(protocol), BasicDBObject.class);
    if (protocol.getId() != null) {
      dbObject.put("_id", new ObjectId(protocol.getId()));
    }
    collection.save(dbObject);

    return findById(protocol.getId());
  }

  public List<Protocol> findAll() throws IOException {
    List<Protocol> protocols = new ArrayList<Protocol>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        protocols.add(MAPPER.readValue(cursor.next().toString(),
          Protocol.class));
      }
    } finally {
      cursor.close();
    }

    return protocols;
  }

  public List<Protocol> findAllActive() throws IOException {
    List<Protocol> protocols = new ArrayList<Protocol>();
    BasicDBObject searchQuery = new BasicDBObject().append("status", true);
    DBCursor cursor = collection.find(searchQuery);
    try {
      while (cursor.hasNext()) {
        protocols.add(MAPPER.readValue(cursor.next().toString(),
          Protocol.class));
      }
    } finally {
      cursor.close();
    }

    return protocols;
  }

  public int removeProtocol(Protocol protocol) throws IOException {

    if (null == protocol || null == protocol.getId()) {
      return -1;
    }
    ObjectId oId = new ObjectId(protocol.getId());
    BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

    WriteResult wr = collection.remove(searchQuery);
    return wr.getN();
  }

  public Protocol saveOrUpdateProtocol(Protocol protocol) throws IOException {

    ObjectId oId;

    DBObject dbObject = (DBObject) JSON.parse(MAPPER
      .writeValueAsString(protocol));
    if (protocol.getId() != null) {
      oId = new ObjectId(protocol.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);
      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      protocol.setId(oId.toString());

    }
    return protocol;
  }
}
