package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.*;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ProtocolTemplateDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase
    .newCustomObjectMapper();

  protected final DBCollection collection;

  public ProtocolTemplateDAO() {
    this.collection = MongoDBConnection.getDB().getCollection(
      "protocolTemplates");
  }

  public Optional<Template> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id",
      new ObjectId(id)));

    Preconditions
      .checkNotNull(dbObject, "template ID does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(),
      Template.class));
  }

  public Optional<Template> update(Template template) throws IOException {
    BasicDBObject dbObject = MAPPER.readValue(
      MAPPER.writeValueAsString(template), BasicDBObject.class);
    if (template.getId() != null) {
      dbObject.put("_id", new ObjectId(template.getId()));
    }
    collection.save(dbObject);

    return findById(template.getId());
  }

  public List<Template> findAll() throws IOException {
    List<Template> protocols = new ArrayList<Template>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        protocols.add(MAPPER.readValue(cursor.next().toString(),
          Template.class));
      }
    } finally {
      cursor.close();
    }

    return protocols;
  }

  public List<Template> findAllActive() throws IOException {
    List<Template> protocols = new ArrayList<Template>();
    BasicDBObject searchQuery = new BasicDBObject().append("status", "Active");
    DBCursor cursor = collection.find(searchQuery);
    try {
      while (cursor.hasNext()) {
        protocols.add(MAPPER.readValue(cursor.next().toString(),
          Template.class));
      }
    } finally {
      cursor.close();
    }

    return protocols;
  }

  public int removeProtocol(Template template) throws IOException {

    if (null == template || null == template.getId()) {
      return -1;
    }
    ObjectId oId = new ObjectId(template.getId());
    BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

    WriteResult wr = collection.remove(searchQuery);
    return wr.getN();
  }

  public Template saveOrUpdateProtocol(Template template) throws IOException {

    ObjectId oId;

    DBObject dbObject = (DBObject) JSON.parse(MAPPER
      .writeValueAsString(template));
    if (template.getId() != null) {
      oId = new ObjectId(template.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);
      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      template.setId(oId.toString());

    }
    return template;
  }
}
