package gov.va.mobilehealth.courier.db;

import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.StaffRoles;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.StaffRole;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;

import org.bson.types.ObjectId;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;

public class StaffRoleDAO {
	  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
	  protected final DBCollection collection;
	  
	  public StaffRoleDAO() {
		    this.collection = MongoDBConnection.getDB().getCollection("staffRole");
		  }
	
	  public Optional<StaffRole> findByUserIdentifier(UserIdentifier userIdentifier) throws IOException {
		    BasicDBObject whereQuery = new BasicDBObject();
		    whereQuery.append("userIdentifier.uniqueId", userIdentifier.getUniqueId());
		    whereQuery.append("userIdentifier.assigningAuthority", userIdentifier.getAssigningAuthority());
		    
		    StaffRole staffRole = findOnebyQuery(whereQuery);

		    return Optional.fromNullable(staffRole);
		  }

    public Optional<StaffRole> findByUserIdentifierAndRole(
            UserIdentifier userIdentifier, StaffRoles role) throws IOException {
        BasicDBObject whereQuery = new BasicDBObject();
        whereQuery.append("userIdentifier.uniqueId", userIdentifier.getUniqueId());
        whereQuery.append("userIdentifier.assigningAuthority", userIdentifier.getAssigningAuthority());
        whereQuery.append("roles", role.name());

        StaffRole staffRole = findOnebyQuery(whereQuery);

        return Optional.fromNullable(staffRole);
    }
      
		  public StaffRole update(StaffRole staffRole) throws IOException {
		    ObjectId oId;

		    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(staffRole));
		    if (staffRole.getId() != null) {
		      oId = new ObjectId(staffRole.getId());
		      dbObject.put("_id", oId);
		      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

		      collection.update(searchQuery, dbObject);
		    } else {
		      collection.insert(dbObject);
		      oId = (ObjectId) dbObject.get("_id");
		      staffRole.setId(oId.toString());
		    }
		    return staffRole;
		  }
		  
		  public StaffRole findOnebyQuery(BasicDBObject query) throws IOException {
			  StaffRole staffRole = null;

			    DBObject dbObject = collection.findOne(query);

			    if (dbObject != null) {
			    	staffRole = MAPPER.readValue(dbObject.toString(), StaffRole.class);
			    }

			    return staffRole;
		  }  
		  
		  public void removeFromDB(String id) throws IOException {
			    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));
			    collection.remove(searchQuery);
			  }


}
