package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created with IntelliJ IDEA. User: shimk Date: 3/25/14 Time: 12:14 PM To change this template use File | Settings | File Templates.
 */
public class TimeZoneDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
  protected final DBCollection collection;

  public TimeZoneDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("timeZones");
  }

  public Optional<TimeZone> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id", new ObjectId(id)));

    Preconditions.checkNotNull(dbObject, "timezone does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(), TimeZone.class));
  }

  public List<TimeZone> findAll() throws IOException {
    List<TimeZone> list = new ArrayList<TimeZone>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        list.add(MAPPER.readValue(cursor.next().toString(), TimeZone.class));
      }
    } finally {
      cursor.close();
    }

    return list;
  }

  public String insert(TimeZone timeZone) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(timeZone));

    collection.insert(dbObject);
    ObjectId oId = (ObjectId) dbObject.get("_id");
    timeZone.setId(oId.toString());
    return timeZone.getId();

  }

  public String save(TimeZone timeZone) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(timeZone));
    if (timeZone.getId() != null) {
      dbObject.put("_id", new ObjectId(timeZone.getId()));
    }
    collection.save(dbObject);
    if (timeZone.getId() == null) {
      ObjectId oId = (ObjectId) dbObject.get("_id");
      timeZone.setId(oId.toString());
    }
    return timeZone.getId();
  }

  public void removeFromDB(String id) throws IOException {

    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));

    collection.remove(searchQuery);
  }
}
