package gov.va.mobilehealth.courier.protocol;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.PatientReading;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.protocol.CMSService;
import gov.va.mobilehealth.courier.core.protocol.PatientResponseStatus;
import gov.va.mobilehealth.courier.core.protocol.Reading;
import gov.va.mobilehealth.courier.core.protocol.Vital;

import java.util.Iterator;
import java.util.List;

public class AlertValidator extends ServiceReadingValidator {
  @Override
  // IMPORTANT: status field in patientResponseServiceData might be changed during validation
  public Optional<String> doValidate(PatientResponseServiceData patientResponseServiceData, Vital template) {

    List<PatientReading> patientReadings = patientResponseServiceData.getPatientServiceReading().getReadings();
    CMSService cmsService = template.getService();
    List<Reading> serviceReadings = cmsService.getReadings();

    Iterator<PatientReading> patientReadingIterator = patientReadings.iterator();
    Iterator<Reading> readingIterator = serviceReadings.iterator();
    while (patientReadingIterator.hasNext()) {

      PatientReading patientReading = patientReadingIterator.next();
      Reading reading = readingIterator.next();
      if (reading.getAlert() != null) {
        if (isHigh(patientReading, reading)) {
          patientResponseServiceData.setAlertLevel(PatientResponseStatus.WARNING.toString());
          return Optional.of(cmsService.getFirstLevelMessage().getAbove());
        } else if (isLow(patientReading, reading)) {
          patientResponseServiceData.setAlertLevel(PatientResponseStatus.WARNING.toString());
          return Optional.of(cmsService.getFirstLevelMessage().getBelow());
        }
      }
    }
    return Optional.absent();
  }

  private boolean isLow(PatientReading patientReading, Reading reading) {
    return reading.getAlert().getLow() != null && patientReading.getValue() <= reading.getAlert().getLow();
  }

  private boolean isHigh(PatientReading patientReading, Reading reading) {
    return reading.getAlert().getHigh() != null && patientReading.getValue() >= reading.getAlert().getHigh();
  }
}
