package gov.va.mobilehealth.courier.protocol;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.PatientReading;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.protocol.CMSService;
import gov.va.mobilehealth.courier.core.protocol.PatientResponseStatus;
import gov.va.mobilehealth.courier.core.protocol.Reading;
import gov.va.mobilehealth.courier.core.protocol.Vital;

import java.util.Iterator;
import java.util.List;

public class CriticalValidator extends ServiceReadingValidator {

  @Override
  // IMPORTANT: status field in patientResponseServiceData might be changed during validation
  public Optional<String> doValidate(PatientResponseServiceData patientResponseServiceData, Vital template) {

    List<PatientReading> patientReadings = patientResponseServiceData.getPatientServiceReading().getReadings();
    CMSService cmsService = template.getService();
    List<Reading> serviceReadings = cmsService.getReadings();

    Iterator<PatientReading> patientReadingIterator = patientReadings.iterator();
    Iterator<Reading> readingIterator = serviceReadings.iterator();
    while (patientReadingIterator.hasNext()) {
      PatientReading patientReading = patientReadingIterator.next();
      Reading reading = readingIterator.next();
      if (reading.getCritical() != null) {
        if (isCriticalHigh(patientReading, reading)) {
          patientResponseServiceData.setAlertLevel(PatientResponseStatus.CRITICAL.toString());
          return Optional.of(cmsService.getCriticalMessage().getAbove());
        } else if (isCriticalLow(patientReading, reading)) {
          patientResponseServiceData.setAlertLevel(PatientResponseStatus.CRITICAL.toString());
          return Optional.of(cmsService.getCriticalMessage().getBelow());
        }
      }
    }
    return Optional.absent();
  }

  private boolean isCriticalLow(PatientReading patientReading, Reading reading) {
    return reading.getCritical().getLow() != null && patientReading.getValue() <= reading.getCritical().getLow();
  }

  private boolean isCriticalHigh(PatientReading patientReading, Reading reading) {
    return reading.getCritical().getHigh() != null && patientReading.getValue() >= reading.getCritical().getHigh();
  }

}
