package gov.va.mobilehealth.courier.protocol;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.PatientReading;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.protocol.Reading;
import gov.va.mobilehealth.courier.core.protocol.Vital;

import java.util.Iterator;
import java.util.List;

public class ValidRangeValidator extends ServiceReadingValidator {
  public static final String INVALID_MESSAGE = "Thank you. Your readings are not in valid range, please try one more time.";

  @Override
  public Optional<String> doValidate(PatientResponseServiceData patientResponseServiceData, Vital template) {
    String message = null;

    List<PatientReading> patientReadings = patientResponseServiceData.getPatientServiceReading().getReadings();
    List<Reading> serviceReadings = template.getService().getReadings();

    Iterator<PatientReading> patientReadingIterator = patientReadings.iterator();
    Iterator<Reading> readingIterator = serviceReadings.iterator();
    while (patientReadingIterator.hasNext()) {
      PatientReading patientReading = patientReadingIterator.next();
      Reading reading = readingIterator.next();
      if ((reading.getValid() != null) && (isRangeInvalid(patientReading, reading))) {
        message = INVALID_MESSAGE;
      }
    }
    return Optional.fromNullable(message);
  }

  private boolean isRangeInvalid(PatientReading patientReading, Reading reading) {
    return (reading.getValid().getHigh() != null && patientReading.getValue() >= reading.getValid().getHigh()) ||
      (reading.getValid().getLow() != null && patientReading.getValue() <= reading.getValid().getLow());
  }
}
