package gov.va.mobilehealth.courier.resources;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.Facility;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.resources.helper.BadRequestException;
import gov.va.mobilehealth.courier.resources.helper.ErrorCode;

import org.codehaus.enunciate.Facet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.util.List;

@Service
@Path("facility")
@Produces(MediaType.APPLICATION_JSON)
@Facet(name = "private")
public class FacilityResource {

  private static final Logger LOGGER = LoggerFactory.getLogger(FacilityResource.class);

    /**
     * <pre>
     * Returns a facility object given an ID.
     *
     *      {
     *          "_id": "dfn-777",
     *          "name": "Test VAMC3-Chantilly-VA",
     *          "timezone": "America/New_York"
     *      }
     *
     * </pre>
     *
     * @param id
     * @return
     * @throws Exception
     */
  @Path("/{id}")
  @GET
  public Facility getBroadcastFacilityMessage(@Context UriInfo info, @Context HttpHeaders headers, @PathParam("id") String id) throws Exception {
      try {
          Optional<Facility> facility = DAOFactory.createFacilityDAO().findById(id);
          return facility.get();
      } catch (IllegalArgumentException ex) {
          throw new BadRequestException(ErrorCode.MSG_NOT_FOUND, "facility not found");
      }
  }

    /**
     * <pre>
     * Returns a list of all facilities.
     *
     *  [
     *      {
     *          "_id": "dfn-777",
     *          "name": "Test VAMC3-Chantilly-VA",
     *          "timezone": "America/New_York"
     *      },
     *      {
     *          "_id": "dfn-778",
     *          "name": "Test Timezone LA-CA",
     *          "timezone": "America/Los_Angeles"
     *      }
     *  ]
     *
     * </pre>
     *
     * @return A list of facilities.
     * @throws Exception
     */
  @GET
  public List<Facility> getAllFacility(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
      List<Facility> facility = DAOFactory.createFacilityDAO().findAll();
      return facility;
  }
}
