package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.core.AtomLink;
import gov.va.mobilehealth.courier.utils.authConfig.AuthConfig;

import org.aeonbits.owner.ConfigFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import java.util.ArrayList;

import org.codehaus.enunciate.Facet;

@Service
@Path("/resource-directory")
@Component
@Scope("request")
@Facet( name = "private")
public class ResourceDirectoryResource {

  protected static AuthConfig getAuthConfig() {
    return ConfigFactory.create(AuthConfig.class, System.getProperties(), System.getenv());
  }

  /**
   * Get a list of urls for authentication
   *
   * <pre>
   * request:
   *   cms/service/resource-directory/tokenurls
   *
   * response:
   * [
   *   {
   *     "rel": null,
   *     "title": "token",
   *     "href": "https://10.2.2.4:8090/AuthorizationServices/rest/token",
   *     "object-type": "AtomLink",
   *     "_id": "1"
   *   },
   *   {
   *     "rel": null,
   *     "title": "validateToken",
   *     "href": "https://10.2.2.4:8090/AuthorizationServices/rest/validateToken",
   *     "object-type": "AtomLink",
   *     "_id": "2"
   *   },
   *   {
   *     "rel": null,
   *     "title": "logout",
   *     "href": "https://10.2.2.4:8090/AuthorizationServices/logout?redirect_uri=https://192.168.33.2/cms/service/logout",
   *     "object-type": "AtomLink",
   *     "_id": "3"
   *   },
   *   {
   *     "rel": null,
   *     "title": "mhpSearch",
   *     "href": "https://10.2.2.4:8090/PatientViewerServices/rest/patients?searchstring=:searchstring",
   *     "object-type": "AtomLink",
   *     "_id": "4"
   *   },
   *   {
   *     "rel": null,
   *     "title": "resourceLastAccessed",
   *     "href": "https://10.2.2.4:8090/AuthorizationServices/rest/resourceLastAccessedTime",
   *     "object-type": "AtomLink",
   *     "_id": "5"
   *   }
   * ]
   * </pre>
   *
   */
  @Path("/tokenurls")
  @GET
  @Produces({"application/json", "application/xml"})
  public ArrayList<AtomLink> fetchResourceLink(@Context UriInfo info, @Context HttpHeaders headers) {
    AuthConfig authConfig = getAuthConfig();

    ArrayList<AtomLink> atomLinks = new ArrayList<AtomLink>();
    atomLinks.add(createLink(authConfig.clientTokenUri(), "token", "1"));
    atomLinks.add(createLink(authConfig.clientValidateTokenUri(), "validateToken", "2"));
    atomLinks.add(createLink(authConfig.clientLogoutUri(), "logout", "3"));
    atomLinks.add(createLink(authConfig.mhpSearchUrl(), "mhpSearch", "4"));
    atomLinks.add(createLink(authConfig.resourceLastAccessed(), "resourceLastAccessed", "5"));
    
    return atomLinks;
  }

  private AtomLink createLink(String uri, String title, String id) {
    AtomLink link = new AtomLink();
    link.setTitle(title);
    link.setHref(uri);
    link.setObject_type("AtomLink");
    link.set_id(id);
    return link;
  }
}
