package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.SMS;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;

import org.apache.camel.ProducerTemplate;
import org.codehaus.enunciate.Facet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.ServletContext;
import javax.validation.Valid;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

@Service
@Path("/sms")
@Consumes(MediaType.APPLICATION_JSON)
@Facet(name = "private")
public class SMSResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(SMSResource.class);
  private ProducerTemplate outgoingMsgProducer;

  /**
   * Test api to sent message to patient
   *
   * <pre>
   * request body:
   * {
   *     "message": "message",
   *     "phoneNumber": "1234567890"
   * }
   *
   * reponse:
   *  204 no content
   * </pre>
   *
   */
  @POST
  public void sendMessage(@Valid SMS sms, @Context ServletContext context, @Context UriInfo info, @Context HttpHeaders headers) {

    LOGGER.info("Received an sms: {}", sms);
    if (this.outgoingMsgProducer == null) {
        this.outgoingMsgProducer = (ProducerTemplate) context.getAttribute("camelOutgoingMsgProducer");
        LOGGER.info("got the producer : " + outgoingMsgProducer.toString());
    }

    Patient patient = new Patient();
    patient.setPreference(new PatientPreference());
    patient.getPreference().setPhoneNumber(sms.getPhoneNumber());

//    MessageProcessor processor = new MessageProcessor();
    CMSMessage msg = new CMSMessage(sms.getPhoneNumber(), patient, sms.getMessage(), MessageType.OUT);
//    CMSMessage msg = new CMSMessage();
//    msg.setSource(sms.getPhoneNumber());
//    msg.setText(sms.getMessage());
//    msg.setDestination(sms.getPhoneNumber());
//    msg.setType(MessageType.OUT.toString());
    // processor.sendSingleMessage(msg, null);
    LOGGER.info("SMSResource: Sending message to outbound queue");
    outgoingMsgProducer.sendBody(msg);
    LOGGER.info("SMSResource: Sending message to outbound queue");
  }
}
