package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.bcm.appointments.scheduler.BcmQuartzInitializerListener;
import gov.va.mobilehealth.courier.core.scheduler.QuartzJob;
import gov.va.mobilehealth.courier.scheduler.QuartzJobHandler;

import org.quartz.SchedulerException;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.ServletContext;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.util.List;

import org.codehaus.enunciate.Facet;

@Service
@Path("scheduled-jobs")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class SchedulingResource {
  @Context
  protected ServletContext context;

  public SchedulingResource() {

  }

  /**
   * API for query Quartz scheduler status. This API is for system debugging purpose
   *
   * <pre>
   * request:
   *    cms/service/scheduled-jobs
   *
   * reponse:
   *    Array of Quartz scheduled jobs
   * [
   *   {
   *     "name": "54259239e4b06c65459a3ddd_0",
   *     "group": "initProtocol",
   *     "jobClass": "gov.va.mobilehealth.courier.scheduler.PatientProtocolInitJob",
   *     "jobTriggerCronExpression": "0 0 8 ? * SUN,MON,TUE,WED,THU,FRI,SAT",
   *     "jobTriggerGroup": "initProtocol",
   *     "jobTriggerExpressionSummary": "seconds: 0\nminutes: 0\nhours: 8\ndaysOfMonth: ?\nmonths: *\ndaysOfWeek: 1,2,3,4,5,6,7\nlastdayOfWeek: false\nnearestWeekday: false\nNthDayOfWeek: 0\nlastdayOfMonth: false\nyears: *\n",
   *     "jobTriggerStartTime": "Sep 26, 2014 12:20:10 PM",
   *     "jobTriggerName": "54259239e4b06c65459a3ddd_0",
   *     "jobTriggerScheduleBuilderClass": "org.quartz.CronScheduleBuilder",
   *     "jobTriggerTimeZone": "America/New_York",
   *     "jobTriggerEndTime": "Mar 28, 2015 8:00:00 AM",
   *     "jobTriggerPreviousFireTime": "Sep 27, 2014 8:00:00 AM",
   *     "jobTriggerPriority": 5,
   *     "jobTriggerNextFireTime": "Sep 28, 2014 8:00:00 AM"
   *   }
   * ]
   *
   * </pre>
   *
   */
  @GET
  public List<QuartzJob> list(@Context UriInfo info, @Context HttpHeaders headers) throws SchedulerException {
    StdSchedulerFactory factory = (StdSchedulerFactory) context
      .getAttribute(BcmQuartzInitializerListener.QUARTZ_FACTORY_KEY);
    return (new QuartzJobHandler(factory.getScheduler())).list();
  }
}
