package gov.va.mobilehealth.courier.resources;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import gov.va.mobilehealth.courier.core.StaffRoles;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.StaffRole;
import gov.va.mobilehealth.courier.core.StaffUser;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.StaffRoleDAO;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

import org.codehaus.enunciate.Facet;
import org.springframework.stereotype.Service;

@Service
@Path("staff")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class StaffResource {
	private final StaffRoleDAO staffRoleDAO;

	public StaffResource() {
		this.staffRoleDAO = DAOFactory.createStaffRoleDAO();
	}

  /**
   * Get staff's user profile
   *
   * <pre>
   * request:
   *   cms/service/staff/userProfile
   *
   * response:
   *   staff's user profile
   *   
   * {
   *   "firstName": "SIX",
   *   "middleName": null,
   *   "lastName": "VEHU",
   *   "isAdmin": true,
   *   "isBcmAdmin": true,
   *   "isStaff": true
   * }
   * </pre>
   *
   */
	@Path("/userProfile")
	@GET
	public StaffUser getUserProfile(@Context UriInfo info, @Context HttpHeaders headers) throws IOException {
		StaffUser staffUser = new StaffUser();
		StaffRole staffRole = new StaffRole();
		boolean adminFlag = false;
        boolean bcmAdminFlag = false;
        boolean staffFlag = true;
		
		MhpUser user = MhpUserFactory.createFromSecurityContext();
		UserIdentifier userIdentifier = new UserIdentifier();
		userIdentifier.setAssigningAuthority(user.getUserIdentifier().getAssigningAuthority());
		userIdentifier.setUniqueId(user.getUserIdentifier().getUniqueId());
		
		staffUser.setFirstName(user.getFirstName());
		staffUser.setMiddleName(user.getMiddleInitials());
		staffUser.setLastName(user.getLastName());

		Optional<StaffRole> staffRoledb = staffRoleDAO.findByUserIdentifier(userIdentifier);
		if (staffRoledb.isPresent()) {
			staffRole = staffRoledb.get();
			String roles[] = staffRole.getRoles();
			List<String> roleList = Arrays.asList(roles);
			if ( roleList.contains(StaffRoles.admin.name())){
				adminFlag = true;
			}
            if ( roleList.contains(StaffRoles.bcm_admin.name())){
                bcmAdminFlag = true;
                if (!roleList.contains(StaffRoles.admin.name()) && !roleList.contains(StaffRoles.staff.name())) {
                    staffFlag = false;
                }
            }
		}	
		staffUser.setIsAdmin(adminFlag);
        staffUser.setIsBcmAdmin(bcmAdminFlag);
        staffUser.setIsStaff(staffFlag);
		return staffUser;
	}

}
