package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.core.AtomLink;
import gov.va.mobilehealth.courier.utils.authConfig.VetAuthConfig;

import org.aeonbits.owner.ConfigFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import java.util.ArrayList;

import org.codehaus.enunciate.Facet;

@Service
@Path("/vet-resource-directory")
@Component
@Scope("request")
@Facet( name = "private")
public class VetResourceDirectoryResource {

  protected static VetAuthConfig getVetAuthConfig() {
    return ConfigFactory.create(VetAuthConfig.class, System.getProperties(), System.getenv());
  }

  /**
   * Get a list of urls for veteran authentication
   *
   * <pre>
   * request:
   *   vet/service/vet-resource-directory/vet-auth
   *
   * response:
   * [
   *   {
   *     "rel": null,
   *     "title": "token",
   *     "href": "https://10.2.2.4/AuthorizationServices/rest/token",
   *     "object-type": "AtomLink",
   *     "_id": "1"
   *   },
   *   {
   *     "rel": null,
   *     "title": "validateToken",
   *     "href": "https://10.2.2.4/AuthorizationServices/rest/validateToken",
   *     "object-type": "AtomLink",
   *     "_id": "2"
   *   },
   *   {
   *     "rel": null,
   *     "title": "logout",
   *     "href": "https://10.2.2.4/ssoeproxy/logout?redirect_uri=https://192.168.33.2/vet/service/vetlogout",
   *     "object-type": "AtomLink",
   *     "_id": "3"
   *   },
   *   {
   *     "rel": null,
   *     "title": "resourceLastAccessed",
   *     "href": "https://10.2.2.4/AuthorizationServices/rest/resourceLastAccessedTime",
   *     "object-type": "AtomLink",
   *     "_id": "4"
   *   }
   * ]
   * </pre>
   *
   */
  @Path("/vet-auth")
  @GET
  @Produces({"application/json", "application/xml"})
  public ArrayList<AtomLink> fetchResourceLink(@Context UriInfo info, @Context HttpHeaders headers) {
    VetAuthConfig vetAuthConfig = getVetAuthConfig();

    ArrayList<AtomLink> atomLinks = new ArrayList<AtomLink>();
    atomLinks.add(createLink(vetAuthConfig.vetClientTokenUri(), "token", "1"));
    atomLinks.add(createLink(vetAuthConfig.vetClientValidateTokenUri(), "validateToken", "2"));
    atomLinks.add(createLink(vetAuthConfig.vetClientLogoutUri(), "logout", "3"));
    atomLinks.add(createLink(vetAuthConfig.resourceLastAccessed(), "resourceLastAccessed", "4"));
    atomLinks.add(createLink(vetAuthConfig.vetUserLinks(), "vetUserLinks", "5"));
    atomLinks.add(createLink(vetAuthConfig.vetRoaPage(), "vetRoaPage", "6"));

    return atomLinks;
  }

  private AtomLink createLink(String uri, String title, String id) {
    AtomLink link = new AtomLink();
    link.setTitle(title);
    link.setHref(uri);
    link.setObject_type("AtomLink");
    link.set_id(id);
    return link;
  }
}
