package gov.va.mobilehealth.courier.resources.helper;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.PatientOutgoingMessage;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageHandler;
import gov.va.mobilehealth.courier.smsgateway.types.Message;
import gov.va.mobilehealth.courier.smsgateway.types.Response;
import gov.va.mobilehealth.courier.utils.DateParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PatientMessageHandler implements MessageHandler {

  private static final Logger LOGGER = LoggerFactory.getLogger(PatientMessageHandler.class);

  private final PatientMessageDAO patientResponseDAO;
  public static final String APP_NAME = "Annie";
  public static final String SMS_MESSAGE_DELIVERY_TYPE = "SMS";

  public PatientMessageHandler() {
    this.patientResponseDAO = DAOFactory.createPatientResponseDAO();
  }

  public void processSingleMessageResult(CMSMessage cmsMsg, Response response) {
    PatientOutgoingMessage psMsg = new PatientOutgoingMessage();
    psMsg.setDestination(cmsMsg.getDestination());
    psMsg.setSource(cmsMsg.getSource());
    psMsg.setText(cmsMsg.getText());
    psMsg.setType(MessageType.getEnum(cmsMsg.getType()));
    psMsg.setPatientId(cmsMsg.getPatientId());
    psMsg.setServiceName(cmsMsg.getServiceName());

    String nowTime = DateParser.getNow(null, null);
    psMsg.setNetworkSubmitTime(DateParser.convertTimeZone(nowTime, null, "UTC", null));
    psMsg.setTimeZone(cmsMsg.getTimeZone());
    psMsg.setAdjustedSubmitTime(DateParser.convertTimeZone(nowTime, null, cmsMsg.getTimeZone(), null));
    String[] msgDeliveryType = new String[2];
    int deliveryTypeIndex = 0;
    
    if (cmsMsg.isForceSendViaAnnie()) {
        msgDeliveryType[deliveryTypeIndex++] = APP_NAME;
    }

    if (cmsMsg.isForceSendViaSMS()) {
      msgDeliveryType[deliveryTypeIndex] = SMS_MESSAGE_DELIVERY_TYPE;
      List<Message> list = response.getMessageList();
      List<gov.va.mobilehealth.courier.core.Message> pList = new ArrayList<gov.va.mobilehealth.courier.core.Message>();

      for (Message msg : list) {

        gov.va.mobilehealth.courier.core.Message m = new gov.va.mobilehealth.courier.core.Message();
        m.setMessageId(msg.getMessage_id());
        m.setResponseCode(msg.getResponse_code());
        m.setResponseString(msg.getResponse_string());
        m.setSuccessCode(msg.getSuccess_code());
        pList.add(m);
      }
      psMsg.setMessages(pList);
    }

    try {
      psMsg.setMsgDeliveryType(msgDeliveryType);
      patientResponseDAO.insertOutgoingMessage(psMsg);

    } catch (IOException e) {
      LOGGER.error("IOException", e);
    }


  }
}
