package gov.va.mobilehealth.courier.resources.helper;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.authConfig.AuthConfig;
import org.aeonbits.owner.ConfigFactory;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URL;


public class ProcessOptInMessage {
  private static final Logger LOGGER = LoggerFactory.getLogger(ProcessOptInMessage.class);
  private final PatientDAO patientDAO;
  private static final String OPT_IN_MESSAGE_NON_SECURE = "Hi, this is Annie.  Please text the word \"START\" to begin receiving messages from me. I look forward to hearing from you.";
  private static final String OPT_IN = "OPT_IN";
  private static final String NON_SECURE = "non-secure";
  static AuthConfig authConfig = getAuthConfig();
  public ProcessOptInMessage (){
    this.patientDAO = DAOFactory.createPatientDAO();
  }
  static AuthConfig getAuthConfig() {
    return ConfigFactory.create(AuthConfig.class, System.getProperties(), System.getenv());
  }
  public void sendOptInMessage(Patient patient) throws IOException {

    StringBuilder optInMessage_Secure = new StringBuilder("Hi, this is Annie.  Please text the word \"START\" to begin receiving messages from me. I look forward to hearing from you. ");

    URL anniUrl = new URL(authConfig.annieUrl());
    optInMessage_Secure.append(anniUrl);


    CMSMessage cmsMsg = new CMSMessage(OPT_IN, patient, null, MessageType.OUT);
    cmsMsg.setIncomingMessageId(null); // ask mark or samida
//    cmsMsg.setSource(OPT_IN);
//    cmsMsg.setDestination(patient.getPreference().getPhoneNumber());
//    cmsMsg.setType(MessageType.OUT.toString());
//    cmsMsg.setPatientId(patient.getId());
//    cmsMsg.setText(cmsMsg.getText());
    cmsMsg.setForceSendViaSMS(true);


    ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();
    if (patient.getPreference().getMessageDeliveryPreference().isSendSms()) {
      cmsMsg.setText(OPT_IN_MESSAGE_NON_SECURE);
    } else {
      cmsMsg.setText(optInMessage_Secure.toString().trim());

    }
   // patientDAO.update(patient);
    Optional<Patient> p1 = patientDAO.findByMDN(patient.getPreference().getPhoneNumber());
    if (p1.isPresent()) {
      Patient newPatient = p1.get();
      cmsMsg.setPatientId((newPatient.getId()));
      cmsMsg.setDestination(newPatient.getPreference().getPhoneNumber());
    }
    outgoingMsgProducer.sendBody(cmsMsg);
    //return
  }
}
