package gov.va.mobilehealth.courier.resources.helper;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;

/**
 * Created by clarkee on 7/24/14.
 */
public class ResponseMessageGenerator {

  public static final String HELP_STR = "These are commands to communicate with Annie\n  Start = Start Annie\n  Stop = Stop Annie\n  Pause = Temporarily pause messages from Annie\n  Resume = Resume messages from Annie";

  //TODO: There should be single location for this
  private static final String NON_SECURE_MESSAGE = "non-secure";
  public static final String PAUSE_ERROR = "I'm sorry; I don't recognize your command. Remember that you need to text me the \"PAUSE\" command before I can \"RESUME\" sending you messages. Thank you, Annie.  ";  // test
  public static final String PAUSE_RESPONSE = "You've chosen to temporarily pause messages. Let me know when you are ready to receive messages again by texting \"Resume\". I look forward to hearing from you.";
  public static final String RESUME_RESPONSE = "Welcome back! It is good to know that you have chosen to resume your messages. Let's get ready to get back on track. ";
  public static final String START_ERROR = "I'm sorry; I cannot recognize your message. Please remember that you need to text me the word \"START\" in order for me to appropriately respond to your messages.";
  public static final String START_RESPONSE = " Thank you for choosing Annie. You confirmed to start receiving messages from me. At any time you need assistance with the app, text the word \"HELP\".";
  public static final String STOP_RESPONSE = "I'm sorry to see you go. You have chosen to stop receiving messages. At any time you change your mind, text \"START\" to receive messages again. Thank you, Annie. ";

  public CMSMessage generatorStartResponse(PatientIncomingMessage patientResponse) {
    CMSMessage cmsMsg = new CMSMessage(patientResponse, "", MessageType.OUT);
    //TODO: Destination vs messageCommand
    cmsMsg.setPatientId(patientResponse.getPatientId());

    cmsMsg.setText(START_RESPONSE);
    return cmsMsg;
  }

  public CMSMessage generateHelpResponse(PatientIncomingMessage patientResponse, Patient patient) {
    CMSMessage cmsMsg = new CMSMessage(patientResponse, "", MessageType.OUT);
      //TODO: Destination vs messageCommand
    cmsMsg.setPatientId(patientResponse.getPatientId());
    cmsMsg.setMessageHelp(true);
    if (patient.getPreference() != null) {
      if (patient.getPreference().getMessageDeliveryPreference().isSendSms()) { // veteran sent by basic cell phone or smart phone,sending response to phone
        cmsMsg.setForceSendViaSMS(true);
      }
    }
    String helpMessage = getHelpResponse();
    cmsMsg.setText(helpMessage);
    return cmsMsg;
  }

  public String getHelpResponse() {
    StringBuilder helpStr = new StringBuilder(HELP_STR);

    return helpStr.toString();
  }

  public CMSMessage generateReadingBeforeStartResponse(PatientIncomingMessage patientResponse, String patientId) {
    CMSMessage cmsMsg = new CMSMessage(patientResponse, START_ERROR, MessageType.OUT);
    //TODO: Destination vs messageCommand
    cmsMsg.setPatientId(patientId);
    return cmsMsg;
  }

  public CMSMessage generateReadingResponse(PatientIncomingMessage patientResponse, String patientId) {
    CMSMessage cmsMsg = new CMSMessage(patientResponse, patientResponse.getText(), MessageType.OUT);
    //TODO: Destination vs messageCommand
    cmsMsg.setPatientId(patientId);
    cmsMsg.setAlertLevel(patientResponse.getAlertLevel());
    cmsMsg.setAdjustedSubmitTime(patientResponse.getAdjustedSubmitTime());
    cmsMsg.setNetworkSubmitTime(patientResponse.getNetworkSubmitTime());
    return cmsMsg;
  }


  private CMSMessage createMessageToSend(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = new CMSMessage(messageCommand, patient, "", MessageType.OUT);
    cmsMsg.setIncomingMessageId(patientResponse.getId());
    //TODO: Destination vs messageCommand
//    cmsMsg.setSource(messageCommand);
    cmsMsg.setDestination(patientResponse.getSource());
//    cmsMsg.setType(MessageType.OUT.toString());
//    cmsMsg.setPatientId(patient.getId());
    return cmsMsg;
  }

  public CMSMessage generatePauseStartErrorResponse(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(START_ERROR);
    return cmsMsg;
  }

  public CMSMessage generatePauseResponse(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(PAUSE_RESPONSE);
    return cmsMsg;
  }

  public CMSMessage generateResumeStartError(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(START_ERROR);
    return cmsMsg;
  }

  public CMSMessage generateResumePauseError(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(PAUSE_ERROR);
    return cmsMsg;
  }


  public CMSMessage genreateStopStartError(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(START_ERROR);
    return cmsMsg;
  }

  public CMSMessage generateStopResponse(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(STOP_RESPONSE);
    return cmsMsg;
  }

  public CMSMessage generateResumeResponse(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) {
    CMSMessage cmsMsg = createMessageToSend(patientResponse, patient, messageCommand);
    cmsMsg.setText(RESUME_RESPONSE);
    return cmsMsg;
  }
}
