package gov.va.mobilehealth.courier.scheduler;

import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.db.BroadcastFacilityMessageDAO;
import gov.va.mobilehealth.courier.db.DAOFactory;

import java.io.IOException;

public class FacilityMessageExpireJob implements Job {

  private static final Logger LOGGER = LoggerFactory.getLogger(FacilityMessageExpireJob.class);
  protected JobExecutionException jobException;

  @Override
  public void execute(JobExecutionContext context)
          throws JobExecutionException {
      String jobName = context.getJobDetail().getKey().getName();
      
      try {
      
          BroadcastFacilityMessageDAO broadcastFacilityMessageDAO = DAOFactory.createBroadcastFacilityMessageDAO();
    
          broadcastFacilityMessageDAO.updateMessageProgress(jobName, BroadcastFacilityMessage.PROGRESS_DELIVERED);
    
          BroadcastMessageScheduler scheduler = new BroadcastMessageScheduler(context.getScheduler());
          scheduler.cleanJob(jobName, BroadcastMessageScheduler.JOB_FIRST_GROUP,
                  BroadcastMessageScheduler.TRIGGER_FIRST_GROUP);
          
      } catch (Exception e) {
          
          LOGGER.error("Job execution exception", e);

          jobException = new JobExecutionException(e);
          // Set refire immediately to false for now. Need to 
          // revisit strategy in dealing with exceptions.
          jobException.setRefireImmediately(false);
          throw jobException;
      }
  }

}
