package gov.va.mobilehealth.courier.scheduler;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.protocol.Motivational;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.core.protocol.Vital;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import org.apache.camel.ProducerTemplate;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerContext;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


import java.io.*;

public class PatientProtocolInitJob extends PatientProtocolJob {

  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolInitJob.class);
  private PatientProtocol pProtocol;
  private Template template;


  @Override
  void doWork(JobExecutionContext context) throws IOException, JobExecutionException {
    CMSMessage message = getSingleMessage(context);

    //send message to outbound queue
    ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();

    message.setServiceName(pProtocol.getProtocol().getName());
    if (outgoingMsgProducer != null) {
      outgoingMsgProducer.sendBody(message);
    }

    // send SMS message
    //processor.sendSingleMessage(message, pmh);
    if (template instanceof Vital) {
      PatientProtocolScheduler scheduler = new PatientProtocolScheduler(context.getScheduler());

      scheduler.createReminders(pProtocol, templateIndex);
    }
  }

  protected CMSMessage getSingleMessage(JobExecutionContext context) throws IOException, JobExecutionException {
    // get patient protocol from protocol ID
    final Optional<PatientProtocol> protocol = patientProtocolDAO.findById(patientProtocolId);
    if (!protocol.isPresent()) {
      LOGGER.debug("Cannot find patient protocol: " + patientProtocolId);
      jobException = new JobExecutionException("PatientProtocolInitJob Cannot find patient protocol");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
//        PatientProtocol pProtocol = protocol.get();
    pProtocol = protocol.get();
    // get template
    template = pProtocol.getProtocol().getTemplates().get(templateIndex);

    // get patient from patient Id
    final Optional<Patient> pat = patientDAO.findById(pProtocol.getPatientId());
    if (!pat.isPresent()) {
      LOGGER.debug("Cannot find patient: " + pProtocol.getPatientId());
      jobException = new JobExecutionException("PatientProtocolInitJob Cannot find patient");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
    Patient patient = pat.get();

    CMSMessage cmsMsg = new CMSMessage(null, patient, "", MessageType.OUT);
//    cmsMsg.setDestination(patient.getPreference().getPhoneNumber());
//    cmsMsg.setType(MessageType.OUT.toString());
//    cmsMsg.setPatientId(patient.getId());
//    cmsMsg.setTimeZone(patient.getPreference().getTimeZone());
    String text = null;

    if (template instanceof Vital) {
      Vital vital = (Vital) template;
      text = vital.getServiceMessage();
    } else if (template instanceof Motivational) {
      Motivational motivational = (Motivational) template;
      text = motivational.getNextMessage();
      patientProtocolDAO.save(pProtocol);
    }
    cmsMsg.setText(text);
    return cmsMsg;
  }

}

