package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessor;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessorBase;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public abstract class PatientProtocolJob implements Job {
  // job parameter
  public static final String PROTOCOL_ID = "protocolID";
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolJob.class);
  protected PatientDAO patientDAO;
  protected PatientProtocolDAO patientProtocolDAO;
  protected JobExecutionException jobException;
  protected String jobName;
  protected String jobGroup;
  protected String patientProtocolId;
  protected int templateIndex;

  public void execute(JobExecutionContext context)
    throws JobExecutionException {

    try {
      patientDAO = DAOFactory.createPatientDAO();

      patientProtocolDAO = DAOFactory.createPatientProtocolDAO();

      jobName = context.getJobDetail().getKey().getName();

      String[] ids = PatientProtocolScheduler.splitJobName(this.jobName);
      patientProtocolId = ids[0];
      templateIndex = Integer.parseInt(ids[1]);

      doWork(context);

    } catch (Exception e) {
      LOGGER.error("Job execution exception", e);

      jobException = new JobExecutionException(e);
      // Set refire immediately to false for now. Need to 
      // revisit strategy in dealing with exceptions.
      jobException.setRefireImmediately(false);
      throw jobException;
    }

  }

  abstract void doWork(JobExecutionContext context) throws IOException, JobExecutionException;

  MessageProcessorBase getMessageProcessor() {
    return new MessageProcessor();
  }

}

