package gov.va.mobilehealth.courier.scheduler;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.resources.helper.PatientMessageHandler;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessorBase;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.DateParser;
import org.apache.camel.ProducerTemplate;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContext;
import javax.ws.rs.core.Context;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;


public class PatientProtocolLastJob extends PatientProtocolJob {
  public static final String REMINDER_MESSAGE1 = "reminderMessage";
  public static final String REMINDER_MESSAGE2 = "restOfReminder";
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolLastJob.class);
  private PatientProtocol pProtocol;
  @Context
  ServletContext servletContext;
  private Map<String, String> messageProp;

  @Override
  void doWork(JobExecutionContext context) throws IOException, JobExecutionException {


    CMSMessage cmsMsg = getSingleLastMessage(context);

    //send message to outbound queue
    ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();

    cmsMsg.setServiceName(pProtocol.getProtocol().getName());
    outgoingMsgProducer.sendBody(cmsMsg);

  }

  CMSMessage getSingleLastMessage(JobExecutionContext context) throws IOException, JobExecutionException {

    // get patient protocol from protocol ID
    final Optional<PatientProtocol> protocol = patientProtocolDAO.findById(patientProtocolId);

    if (!protocol.isPresent()) {
      LOGGER.debug("Cannot find patient protocol: " + patientProtocolId);
      jobException = new JobExecutionException("PatientProtocolLastJob Cannot find patient protocol");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
    pProtocol = protocol.get();
    // get template
    Template template = pProtocol.getProtocol().getTemplates().get(templateIndex);

    // get patient from patient Id
    final Optional<Patient> pat = patientDAO.findById(pProtocol.getPatientId());
    if (!pat.isPresent()) {
      LOGGER.debug("Cannot find patient: " + pProtocol.getPatientId());
      jobException = new JobExecutionException("PatientProtocolLastJob Cannot find patient");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
    Patient patient = pat.get();

    MessagePropertiesLoader loader = new MessagePropertiesLoader();
    messageProp = loader.loadProperties();

    Object[] params = new Object[4];
    params[0] = messageProp.get(REMINDER_MESSAGE1);
    params[1] = template.getDescription();
    params[2] = messageProp.get(REMINDER_MESSAGE2);

    String format = "yyyy-MM-dd";
    Date endDate = DateParser.parse(template.getEndDate(), format);

    params[3] = endDate;

    String result = MessageFormat.format(
            "{0} {1}. {2} {3,date}.",
            params[0], params[1], params[2], params[3]);

    CMSMessage cmsMsg = new CMSMessage(null, patient, result, MessageType.OUT);
//    cmsMsg.setText(result);
//    cmsMsg.setDestination(patient.getPreference().getPhoneNumber());
//    cmsMsg.setType(MessageType.OUT.toString());
//    cmsMsg.setPatientId(patient.getId());
//    cmsMsg.setTimeZone(patient.getPreference().getTimeZone());

    return cmsMsg;
  }

}

