package gov.va.mobilehealth.courier.scheduler;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.core.protocol.Vital;
import gov.va.mobilehealth.courier.resources.helper.PatientMessageHandler;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessorBase;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import org.apache.camel.ProducerTemplate;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class PatientProtocolReminderJob extends PatientProtocolJob {

  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolReminderJob.class);
  private PatientProtocol pProtocol;

  //@Context ServletContext servletContext;

  @Override
  void doWork(JobExecutionContext context) throws IOException, JobExecutionException {

    MessageProcessorBase processor;

    processor = getMessageProcessor();
    CMSMessage cmsMsg = getSingleReminderMessage(context);
    PatientMessageHandler pmh = new PatientMessageHandler();

    //send message to outbound queue
    LOGGER.debug("PatientProtocolReminderJob: Sending the message to Outbound queue");
    ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();
    LOGGER.debug("PatientProtocolReminderJob: got the Producer,sending the message");
    cmsMsg.setServiceName(pProtocol.getProtocol().getName());
    outgoingMsgProducer.sendBody(cmsMsg);

    //processor.sendSingleMessage(cmsMsg, pmh);
  }

  CMSMessage getSingleReminderMessage(JobExecutionContext context) throws IOException, JobExecutionException {
    // get patient protocol from protocol ID
    final Optional<PatientProtocol> protocol = patientProtocolDAO.findById(patientProtocolId);

    if (!protocol.isPresent()) {
      LOGGER.debug("Cannot find patient protocol: " + patientProtocolId);
      jobException = new JobExecutionException("PatientProtocolReminderJob Cannot find patient protocol");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
    pProtocol = protocol.get();
    // get template
    Template template = pProtocol.getProtocol().getTemplates().get(templateIndex);

    // get patient from patient Id
    final Optional<Patient> pat = patientDAO.findById(pProtocol.getPatientId());
    if (!pat.isPresent()) {
      LOGGER.debug("Cannot find patient: " + pProtocol.getPatientId());
      jobException = new JobExecutionException("PatientProtocolReminderJob Cannot find patient");
      //unschedule all triggers associated with this job
      // so that it does not run again
      jobException.setUnscheduleAllTriggers(true);
      throw jobException;
    }
    Patient patient = pat.get();

    CMSMessage cmsMsg = new CMSMessage(null, patient, null, MessageType.OUT);

    // TODO: need to re-evaluate to see if only vital need reminder message.
    // how about reminder template?
    if (template instanceof Vital) {
      Vital vital = (Vital) template;
      cmsMsg.setText(vital.getReminderMessage());
    }
//    cmsMsg.setDestination(patient.getPreference().getPhoneNumber());
//    cmsMsg.setType(MessageType.OUT.toString());
//    cmsMsg.setPatientId(patient.getId());
//    cmsMsg.setTimeZone(patient.getPreference().getTimeZone());

    return cmsMsg;
  }
}
