package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.protocol.Meridian;
import gov.va.mobilehealth.courier.core.protocol.Schedule;
import gov.va.mobilehealth.courier.core.protocol.ScheduleTime;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ProtocolChronGenerator {
  private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolChronGenerator.class);

  public static String getCronSchedule(Schedule schedule) {

    LOGGER.debug("schedule: {}", schedule);
    StringBuilder cronScheduleStr = new StringBuilder();
    cronScheduleStr.append("0 "); //seconds fields          sec min hour dayofmon month dow  year(optional)
    ScheduleTime scheduleTime = schedule.getScheduleTime();

    cronScheduleStr.append(scheduleTime.getMin());
    cronScheduleStr.append(" ");

    cronScheduleStr = convertHour(scheduleTime, cronScheduleStr);


    cronScheduleStr.append(" ? * ");     //? is for dau of month  and month

    cronScheduleStr.append(ProtocolChronGenerator.getScheduleDayOfWeek(schedule));      //returns day of week

    return cronScheduleStr.toString();
  }

  public static StringBuilder convertHour(ScheduleTime scheduleTime, StringBuilder cronScheduleStr) {

    if (scheduleTime.getMeridian().equals(Meridian.PM)) {
      if (scheduleTime.getHour() < 12) {
        cronScheduleStr.append(scheduleTime.getHour() + 12);
      } else {
        cronScheduleStr.append(scheduleTime.getHour());
      }

    } else {
      if (scheduleTime.getHour() == 12) {
        cronScheduleStr.append("0");

      } else {
        cronScheduleStr.append(scheduleTime.getHour());
      }
    }

    return cronScheduleStr;
  }

  public static String getStartDateTime(Template template) {
    return getStartDateTime(template.getStartDate(), template.getSchedule().getScheduleTime());
  }

  public static String getLastDateTime(Template template) {
    return getStartDateTime(template.getEndDate(), template.getSchedule().getScheduleTime());
  }

  public static String getStartDateTime(String date, ScheduleTime scheduleTime) {
    StringBuilder runTimeStr = new StringBuilder(date);
    runTimeStr.append(" ");
    runTimeStr = ProtocolChronGenerator.convertHour(scheduleTime, runTimeStr);

    runTimeStr.append(":");

    runTimeStr.append(scheduleTime.getMin());
    runTimeStr.append(":");
    runTimeStr.append("00");
    return runTimeStr.toString();
  }

  public static StringBuilder getScheduleDayOfWeek(Schedule schedule) {

    StringBuilder cronSchedule = new StringBuilder();
    TypeDaysOfWeek daysOfWeek = (TypeDaysOfWeek) schedule.getScheduleType();

    if (daysOfWeek.isSunday()) {
      cronSchedule.append("SUN");

    }
    if (daysOfWeek.isMonday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("MON");
    }
    if (daysOfWeek.isTuesday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("TUE");
    }
    if (daysOfWeek.isWednesday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("WED");
    }
    if (daysOfWeek.isThursday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("THU");
    }
    if (daysOfWeek.isFriday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("FRI");
    }
    if (daysOfWeek.isSaturday()) {
      if (cronSchedule.length() != 0) {
        cronSchedule.append(",");
      }

      cronSchedule.append("SAT");
    }

    return cronSchedule;

  }
}
