package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.scheduler.QuartzJob;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class QuartzJobHandler {

  private Scheduler scheduler;

  public QuartzJobHandler(Scheduler scheduler) {
    this.scheduler = scheduler;
  }

  protected static String formatDate(final Date date) {
    return (date == null) ? "" : DateFormat.getDateTimeInstance().format(date);
  }

  public List<QuartzJob> list() throws SchedulerException {
    List<QuartzJob> quartzJobs = new ArrayList<QuartzJob>();

    List<String> jobGroups = scheduler.getJobGroupNames();
    for (String jobGroup : jobGroups) {
      Set<JobKey> jobKeys = scheduler.getJobKeys(GroupMatcher.jobGroupEquals(jobGroup));
      for (JobKey jobKey : jobKeys) {
        QuartzJob quartzJob = new QuartzJob();

        quartzJob.setName(jobKey.getName());
        quartzJob.setGroup(jobKey.getGroup());

        JobDetail jobDetail = scheduler.getJobDetail(jobKey);
        quartzJob.setJobClass(jobDetail.getJobClass());

        List<? extends Trigger> triggers = scheduler.getTriggersOfJob(jobKey);
        for (Trigger trigger : triggers) {
          quartzJob.setAdditionalProperties("jobTriggerName", trigger.getKey().getName());
          quartzJob.setAdditionalProperties("jobTriggerGroup", trigger.getKey().getGroup());
          quartzJob.setAdditionalProperties("jobTriggerStartTime", formatDate(trigger.getStartTime()));
          quartzJob.setAdditionalProperties("jobTriggerEndTime", formatDate(trigger.getEndTime()));
          quartzJob.setAdditionalProperties("jobTriggerPreviousFireTime", formatDate(trigger.getPreviousFireTime()));
          quartzJob.setAdditionalProperties("jobTriggerNextFireTime", formatDate(trigger.getNextFireTime()));
          quartzJob.setAdditionalProperties("jobTriggerPriority", trigger.getPriority());
          quartzJob.setAdditionalProperties("jobTriggerScheduleBuilderClass", trigger.getScheduleBuilder().getClass());

          if (trigger instanceof CronTrigger) {
            CronTrigger cronTrigger = (CronTrigger) trigger;
            quartzJob.setAdditionalProperties("jobTriggerCronExpression", cronTrigger.getCronExpression());
            quartzJob.setAdditionalProperties("jobTriggerExpressionSummary", cronTrigger.getExpressionSummary());
            quartzJob.setAdditionalProperties("jobTriggerTimeZone", cronTrigger.getTimeZone());
          } else if (trigger instanceof SimpleTrigger) {
            SimpleTrigger simpleTrigger = (SimpleTrigger) trigger;
            quartzJob.setAdditionalProperties("jobTriggerRepeatCount", simpleTrigger.getRepeatCount());
            quartzJob.setAdditionalProperties("jobTriggerRepeatInterval", simpleTrigger.getRepeatInterval());
            quartzJob.setAdditionalProperties("jobTriggerTimesTriggered", simpleTrigger.getTimesTriggered());
          } else if (trigger instanceof CalendarIntervalTrigger) {
            CalendarIntervalTrigger calendarIntervalTrigger = (CalendarIntervalTrigger) trigger;
            quartzJob.setAdditionalProperties("jobTriggerTimesTriggered", calendarIntervalTrigger.getTimesTriggered());
            quartzJob.setAdditionalProperties("jobTriggerRepeatInterval", calendarIntervalTrigger.getRepeatInterval());
            quartzJob.setAdditionalProperties("jobTriggerRepeatIntervalUnit", calendarIntervalTrigger.getRepeatIntervalUnit());
            quartzJob.setAdditionalProperties("jobTriggerTimeZone", calendarIntervalTrigger.getTimeZone());
          }
        }

        quartzJobs.add(quartzJob);
      }
    }

    return quartzJobs;
  }
}
