package gov.va.mobilehealth.courier.scheduler;


import gov.va.mobilehealth.courier.core.protocol.TimeInterval;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

public abstract class SchedulerBase {
  private static final Logger LOGGER = LoggerFactory.getLogger(SchedulerBase.class);

  protected Scheduler scheduler;

  // same patient id is used for jobName for both job and trigger.
  protected void cleanJob(String jobName, String jobGroup, String triggerGroup) throws SchedulerException {
    if (scheduler.checkExists(JobKey.jobKey(jobName, jobGroup))) {
      scheduler.deleteJob(JobKey.jobKey(jobName, jobGroup));
    }
    if (scheduler.checkExists(TriggerKey.triggerKey(jobName, triggerGroup))) {
      scheduler.unscheduleJob(TriggerKey.triggerKey(jobName, triggerGroup));
    }

  }

  public void scheduleJob(JobDetail job, int triggerType, String jobName, String jobGroup, String schedStr, Date scheduledTime, Date scheduleEndTime, TimeInterval timeInterval, String timezone) {
    try {
      cleanJob(jobName, jobGroup, jobGroup);

      switch (triggerType) {
        case (1):
          Trigger trigger = TriggerFactory.buildSimpleTrigger(jobName, jobGroup, scheduledTime, scheduleEndTime);
          scheduler.scheduleJob(job, trigger);
          break;
        case (2):
          CronTrigger cronTrigger = TriggerFactory.buildCronTrigger(jobName, jobGroup, schedStr, scheduleEndTime, timezone);
          scheduler.scheduleJob(job, cronTrigger);
          break;
        case (3):
          CalendarIntervalTrigger calendarIntervalTrigger = TriggerFactory.buildCalendarTrigger(
            timeInterval.getValue(),
            scheduledTime,
            scheduleEndTime,
            jobGroup,
            jobName,
            timeInterval.getUnit(), timezone);
          scheduler.scheduleJob(job, calendarIntervalTrigger);
          break;
        case (4):
          break;
      }

    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }

  }

  public void scheduleSimpleTriggerJob(JobDetail job, String jobName, String jobGroup, Date scheduledTime, Date scheduleEndTime) {
    try {
      Trigger trigger = TriggerFactory.buildSimpleTrigger(jobName, jobGroup, scheduledTime, scheduleEndTime);
      scheduler.scheduleJob(job, trigger);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  public void scheduleCronTriggerJob(JobDetail job, String jobName, String jobGroup, String schedStr, Date scheduledEndTime, String timezone) {
    try {
      CronTrigger cronTrigger = TriggerFactory.buildCronTrigger(jobName, jobGroup, schedStr, scheduledEndTime, timezone);
      scheduler.scheduleJob(job, cronTrigger);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  public void scheduleCalendarIntervalTriggerJob(JobDetail job, String jobName, String jobGroup, Date scheduledTime, Date scheduleEndTime, TimeInterval timeInterval, String timezone) {
    try {
      CalendarIntervalTrigger calendarIntervalTrigger = TriggerFactory.buildCalendarTrigger(
        timeInterval.getValue(),
        scheduledTime,
        scheduleEndTime,
        jobGroup,
        jobName,
        timeInterval.getUnit(), timezone);
      scheduler.scheduleJob(job, calendarIntervalTrigger);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

}
