package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.protocol.TimeIntervalUnit;
import org.quartz.CalendarIntervalTrigger;
import org.quartz.CronTrigger;
import org.quartz.DateBuilder;
import org.quartz.SimpleTrigger;

import java.util.Date;
import java.util.TimeZone;

import static org.quartz.CalendarIntervalScheduleBuilder.calendarIntervalSchedule;
import static org.quartz.CronScheduleBuilder.cronSchedule;
import static org.quartz.TriggerBuilder.newTrigger;

public class TriggerFactory {

  private static DateBuilder.IntervalUnit getQuartzIntervalUnit(TimeIntervalUnit timeIntervalUnit) {
    switch (timeIntervalUnit) {
      case DAY:
        return DateBuilder.IntervalUnit.DAY;
      case WEEK:
        return DateBuilder.IntervalUnit.WEEK;
      case MONTH:
        return DateBuilder.IntervalUnit.MONTH;
      case YEAR:
        return DateBuilder.IntervalUnit.YEAR;
      case MINUTE:
        return DateBuilder.IntervalUnit.MINUTE;
      case HOUR:
        return DateBuilder.IntervalUnit.HOUR;

    }

    return null;
  }

  public static CalendarIntervalTrigger buildCalendarTrigger(int intervalVal, Date sDate, Date eDate, String triggerGroup, String triggerName, TimeIntervalUnit timeIntervalUnit, String timezone) {
    CalendarIntervalTrigger calendarIntervalTrigger = newTrigger()
      .withIdentity(triggerName, triggerGroup)

      .startAt(sDate)
      .endAt(eDate)// 15:00:00 tomorrow
      .withSchedule(calendarIntervalSchedule()
        .withInterval(intervalVal, getQuartzIntervalUnit(timeIntervalUnit)).inTimeZone(TimeZone.getTimeZone(timezone)))
        // .withInterval(intervalVal, getQuartzIntervalUnit(timeIntervalUnit)))

      .build();

    return calendarIntervalTrigger;
  }

  public static SimpleTrigger buildSimpleTrigger(String triggerName, String triggerGroup, Date scheduledTime, Date eDate) {

    SimpleTrigger simpleTrigger = (SimpleTrigger) newTrigger().withIdentity(triggerName, triggerGroup).startAt(scheduledTime).endAt(eDate).build();

    return simpleTrigger;

  }

  public static CronTrigger buildCronTrigger(String triggerName, String triggerGroup, String cronSchedStr, Date eDate, String timezone) {
    CronTrigger cronTrigger = newTrigger().withIdentity(triggerName, triggerGroup).withSchedule(cronSchedule(cronSchedStr).inTimeZone(TimeZone.getTimeZone(timezone))).endAt(eDate)
      //CronTrigger cronTrigger = newTrigger().withIdentity(triggerName, triggerGroup).withSchedule(cronSchedule(cronSchedStr)).endAt(eDate)
      .build();

    return cronTrigger;

  }


}
