package gov.va.mobilehealth.courier.utils;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

/**
 * Created with IntelliJ IDEA. User: clarkee Date: 12/4/13 Time: 12:57 PM To change this template use File | Settings | File Templates.
 */
public class DateParser {
  final static String DEFAULT_DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
  private static final Logger LOGGER = LoggerFactory.getLogger(DateParser.class);

  public static Date parseAttDate(String date) {
    return parse(date, DEFAULT_DATETIME_FORMAT);
  }

  public static Date parse(String date, String format) {
    SimpleDateFormat df = new SimpleDateFormat(format);
    Date dt;
    try {
      dt = df.parse(date);
    } catch (ParseException e) {
      throw new RuntimeException(e);
    }

    return dt;
  }

  public static String getCMSDateString(Date date) {
    if (date != null) {
      SimpleDateFormat df = new SimpleDateFormat(DEFAULT_DATETIME_FORMAT);
      return df.format(date);
    } else {
      return null;
    }
  }

  public static Date getEndofDate(Date date, String timezone) {
    if (null != date) {
      Calendar cal = Calendar.getInstance();
      cal.setTime(date);
      Calendar endOfDate = Calendar.getInstance();
      endOfDate.set(Calendar.YEAR, cal.get(Calendar.YEAR));
      endOfDate.set(Calendar.MONTH, cal.get(Calendar.MONTH));
      endOfDate.set(Calendar.DAY_OF_MONTH, cal.get(Calendar.DAY_OF_MONTH));
      endOfDate.set(Calendar.AM_PM, Calendar.AM);
      endOfDate.set(Calendar.HOUR, 23);
      endOfDate.set(Calendar.MINUTE, 59);
      endOfDate.set(Calendar.SECOND, 59);
      endOfDate.set(Calendar.MILLISECOND, 0);
      endOfDate.setTimeZone(TimeZone.getTimeZone(timezone));
      return endOfDate.getTime();
    } else {
      return null;
    }
  }

  public static boolean isDateValid(String date) {
    // match yyyy-mm-dd between 1900-01-01 and 2099-12-31
    String pattern = "(19|20)\\d\\d[-](0[1-9]|1[012])[-](0[1-9]|[12][0-9]|3[01])";
    if (date.matches(pattern)) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * get today's date in yyyy-MM-dd format
   *
   * @param timezone timezone, e.g., America/New_York, Europe/London
   * @return today's date in yyyy-MM-dd
   */
  public static String getTodaysDate(String timezone, String format) {
    return getNow(timezone, format != null ? format : "yyyy-MM-dd");
  }

  public static String getCurrentTime(String timezone, String format) {
    return getNow(timezone, format != null ? format : "h:mm a");
  }

  /**
   * get current time
   *
   * @param timezone timezone, default to system timezone
   * @param format time format, default to DEFAULT_DATETIME_FORMAT
   * @return now
   */
  public static String getNow(String timezone, String format) {
    if (format == null) {
      format = DEFAULT_DATETIME_FORMAT;
    }
    DateTime dateTime = new DateTime();
    DateTimeFormatter fmt = DateTimeFormat.forPattern(format);

    if (timezone != null) {
      dateTime = dateTime.withZone(DateTimeZone.forID(timezone));
    }
    String today = dateTime.toString(fmt);
    return today;
  }

  public static Date parseDateWithTimeZone(String date, String timezone) {
    SimpleDateFormat df = new SimpleDateFormat(DEFAULT_DATETIME_FORMAT);
    df.setTimeZone(TimeZone.getTimeZone(timezone));
    Date dt;
    try {
      dt = df.parse(date);
    } catch (ParseException e) {
      throw new RuntimeException(e);
    }

    return dt;
  }

  /**
   * convert timezone
   * @param date: input date string
   * @param fromTimezone: from timezone, default to system timezone
   * @param toTimezone: to timezone, default to system timezone
   * @param format: time format, default to DEFAULT_DATETIME_FORMAT
   * @return converted time string
   */
  public static String convertTimeZone(String date, String fromTimezone, String toTimezone, String format) {
    if (format == null) {
      format = DEFAULT_DATETIME_FORMAT;
    }
    if (fromTimezone == null) {
      fromTimezone = DateTimeZone.getDefault().getID();
    }
    if (toTimezone == null) {
      toTimezone = DateTimeZone.getDefault().getID();
    }
    DateTimeFormatter formatter = DateTimeFormat.forPattern(format);
    DateTime dateTime = formatter.withZone(DateTimeZone.forID(fromTimezone)).parseDateTime(date);
    String newDateTime = formatter.withZone(DateTimeZone.forID(toTimezone)).print(dateTime);
    return newDateTime;
  }

  public static String convertDateFormat(String inputDate, String origFormat) {
    DateFormat originalFormat = new SimpleDateFormat(origFormat, Locale.ENGLISH);
    DateFormat targetFormat = new SimpleDateFormat("yyyy-MM-dd");
    Date date = null;
    try {
      date = originalFormat.parse(inputDate);
    } catch (ParseException e) {
      throw new RuntimeException(e);
    }
    String formattedDate = targetFormat.format(date);

    return formattedDate;
  }
  
}
