package gov.va.mobilehealth.courier.utils;

import org.aeonbits.owner.ConfigFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

public class MongoClientInitializerListener implements ServletContextListener {
  private static final Logger LOGGER = LoggerFactory.getLogger(MongoClientInitializerListener.class);

  @Override
  public void contextInitialized(ServletContextEvent servletContextEvent) {
    LOGGER.debug("Initializing mongodb connection...");
    try {
      ServerConfig config = ConfigFactory.create(ServerConfig.class, System.getProperties(), System.getenv());

      MongoDBConnection.init(config.connectionString(), config.dbName());

      LOGGER.info("mongodb connection was initialized successfully.");
    } catch (Exception e) {
      LOGGER.error("Failed to initialize mongodb connection: " + e.getMessage(), e);
      throw new RuntimeException("Failed to initialize mongodb connection.", e);
    }
  }

  @Override
  public void contextDestroyed(ServletContextEvent servletContextEvent) {
    MongoDBConnection.close();
  }

}
