package gov.va.mobilehealth.courier.utils;

import java.text.MessageFormat;
import java.util.regex.Pattern;

public class SSNFormatter {

  public static final String SSN_PATTERN = "{0}-{1}-{2}";
  private static final Pattern PATTERN = Pattern.compile("^\\d{9}$");

  // thread safe anonymous subclass
  public static final ThreadLocal<MessageFormat> SSN_FORMAT = new ThreadLocal<MessageFormat>() {
    @Override
    protected MessageFormat initialValue() {
      return new MessageFormat(SSN_PATTERN);
    }
  };

  /**
   * format SSN from all numbers to xxx-xx-xxxx
   *
   * @param input number input like 111223333
   * @return formatted result for a valid input, i.e. 111-22-3333, or null if input invalid
   */
  public static String ssnFormatter(String input) {
    String ssn = null;
    if (aValidSSN(input)) {
      // grouping of 3-2-4
      String[] temp = {input.substring(0, 3),
        input.substring(3, 5), input.substring(5)};

      ssn = SSN_FORMAT.get().format(temp);
    }
    return ssn;
  }

  /**
   * Checks if a given string represents an SSN number
   *
   * @param input number input like 111223333
   */
  public static boolean aValidSSN(String input) {
    return PATTERN.matcher(input).matches();
  }
}
