package gov.va.mobilehealth.courier.utils.auth;

import gov.va.mobilehealth.courier.core.StaffRoles;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.StaffRole;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.StaffRoleDAO;

import java.io.IOException;
import java.util.Collection;

import org.aopalliance.intercept.MethodInvocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.AccessDecisionVoter;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.FilterInvocation;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

public class ClinicalAccessRoleVoter implements AccessDecisionVoter {
    private static final Logger LOGGER = LoggerFactory.getLogger(ClinicalAccessRoleVoter.class);

    private static final String PREFIX = "USER_";

    @Override
    public boolean supports(ConfigAttribute attribute) {
      if (attribute.getAttribute() != null){
        return attribute.getAttribute().startsWith(PREFIX);
      }
      else{
        return false;
      }
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean supports(Class clazz) {
        if (FilterInvocation.class.isAssignableFrom(clazz)) return true;
        return MethodInvocation.class.isAssignableFrom(clazz);
    }

    @Override
    public int vote(Authentication authentication, Object object, Collection attributes) {
        int rv = ACCESS_ABSTAIN;
        for(Object element: attributes) {
            ConfigAttribute attribute = (ConfigAttribute) element;
            if(this.supports(attribute)) {
                rv = ACCESS_DENIED;
                String role = attribute.getAttribute().substring(PREFIX.length());

                if("ADMIN".equalsIgnoreCase(role)) {
                    MhpUser user = MhpUserFactory.createFromAuthentication(authentication);
                    rv = checkAccess(user, StaffRoles.admin);
                } else if ("BCM_ADMIN".equalsIgnoreCase(role)) {
                    MhpUser user = MhpUserFactory.createFromAuthentication(authentication);
                    rv = checkAccess(user, StaffRoles.bcm_admin);
                }
             }
        }
        return rv;
    }


    private int checkAccess(MhpUser user, StaffRoles role) {
        // require for USER_ADMIN role, check if the current user has access
        StaffRoleDAO staffRoleDAO = DAOFactory.createStaffRoleDAO();
        UserIdentifier pi = new UserIdentifier().withUniqueId(
                user.getUserIdentifier().getUniqueId())
                .withAssigningAuthority(
                        user.getUserIdentifier()
                                .getAssigningAuthority());
        try {
            Optional<StaffRole> found = staffRoleDAO.findByUserIdentifierAndRole(pi, role);

            if (!found.isPresent()) {
                // cannot find in staff role db, default is staff role, access denied
                return ACCESS_DENIED;
            } else {
                return ACCESS_GRANTED;
            }

        } catch (IOException e) {
            // TODO Auto-generated catch block
            LOGGER.error("Staff role look up error " + e);
            return ACCESS_DENIED;
        }
    }

}
