package gov.va.mobilehealth.courier.veteran.resources;

import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.veteran.resources.helper.VeteranResourceHelper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.io.IOException;
import java.util.List;

import org.codehaus.enunciate.Facet;

@Service
@Path("my/protocols")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class VeteranScheduleResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(VeteranScheduleResource.class);

  /**
   * List veteran's active protocols
   *
   * <pre>
   * request:
   *   vet/service/my/protocols/active
   *
   * response:
   *   veteran's active protocols
   *   
   * [
   *   {
   *     "_id": "546502a80cf2ee82a2706cd4",
   *     "patientId": "54641521404e3e7fd989409e",
   *     "assignedDate": "2014-11-13",
   *     "lastEditedDate": "2014-11-13",
   *     "protocol": {
   *       "status": true,
   *       "name": "Diabetes Management - Self Foot Care",
   *       "description": "To support the Veteran who is living with Diabetes and engaging in self foot care on a regular schedule.  To provide an education overview of self foot care principals and practices.  Selection of Patients:  Those veterans who are Foot Risk Score 2 and 3.",
   *       "area": "Diabetes, Foot Care",
   *       "createDate": "System",
   *       "createBy": "ERICA BILEK",
   *       "comments": [],
   *       "templates": [
   *         {
   *           "@class": "gov.va.mobilehealth.courier.core.protocol.Motivational",
   *           "status": "Active",
   *           "type": "Motivational/Educational",
   *           "description": "Weekly Messages to Veteran",
   *           "serviceMessage": [
   *             "Are you having any problems inspecting your feet daily?  If so a mirror on a stick can be purchased at your medical supply store.   ",
   *             "Problems washing your feet daily? A sponge on a stick can be purchased at your local pharmacy.  ",
   *             "Do not walk barefoot-stepping on sharp objects could spell disaster.  Wear sandals, loafers, or mules that are easy to put on.  Slippers are not protective.",
   *             "Do your shoe shopping late in the afternoon or early evening.  Feet tend to swell during the day and this is best time to ensure a good fit.",
   *             "Foot powder on a daily basis in your shoes will help avoid fungal infections such as Athletes foot.",
   *             "Inspect your shoes before putting them on.  Children and grandchildren find them a wonderful hiding place for small toys.",
   *             "Do not soak your feet.  If you are having a problem call your foot care specialist for advice.",
   *             "Do not attempt to cut your own calluses or corns.  If they are painful call your foot care specialist for advice.",
   *             "Do not attempt to cut your own nails if they are thick and discolored.  You can use an emery board between visits to help reduce thickness."
   *           ],
   *           "schedule": {
   *             "acceptUnscheduledReading": false,
   *             "scheduleType": {
   *               "@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
   *               "name": "Custom",
   *               "description": "Every 1 week",
   *               "interval": {
   *                 "value": 1,
   *                 "unit": "WEEK"
   *               }
   *             },
   *             "scheduleTime": {
   *               "hour": 8,
   *               "min": 30,
   *               "meridian": "AM",
   *               "description": "(At 8:30 AM)"
   *             },
   *             "isStartTimeRelative": false,
   *             "relativeStartTime": {
   *               "value": 1,
   *               "unit": "DAY"
   *             },
   *             "isEndTimeRelative": false,
   *             "relativeEndTime": {
   *               "value": 12,
   *               "unit": "MONTH"
   *             },
   *             "breach": {
   *               "repeatBreachValues": false,
   *               "breachesPerRepeatingAlert": 0,
   *               "breachesDays": 0,
   *               "criticalPercentValues": false,
   *               "percentTriggersCriticalBreach": 0
   *             }
   *           },
   *           "lastMessageSent": 0,
   *           "_id": "540d935ee4b0a415f57f793f",
   *           "startDate": "2014-11-14",
   *           "endDate": "2015-11-14"
   *         }
   *       ],
   *       "_id": "540d935ee4b0a415f57f7940"
   *     },
   *     "status": "Active",
   *     "comments": [
   *       {
   *         "content": null,
   *         "user": "SIX VEHU",
   *         "time": "2:12 PM",
   *         "date": "2014-11-13",
   *         "changes": [
   *           {
   *             "fieldName": "status",
   *             "oldValue": "",
   *             "newValue": "Active",
   *             "source": ""
   *           }
   *         ]
   *       }
   *     ]
   *   }
   * ]
   * </pre>
   *
   */
  @Path("/active/")
  @GET
  public List<PatientProtocol> listActiveProtocols(@Context UriInfo info, @Context HttpHeaders headers) throws IOException {
    LOGGER.debug("Retrieving the list of active protocols");
    return getPatientProtocolDAO().findByPatientIdAndStatus(getCurrentpatientId(), PatientProtocol.ACTIVE_PATIENT_PROTOCOL_STATUSES);
  }

  protected PatientProtocolDAO getPatientProtocolDAO(){
    return DAOFactory.createPatientProtocolDAO();
  }

  protected String getCurrentpatientId() throws IOException {
    return new VeteranResourceHelper().getPatientId();
  }

}
