package gov.va.mobilehealth.courier.web;

import com.agilex.healthcare.mobilehealthplatform.security.AppUser;
import com.agilex.healthcare.utility.NullChecker;
import gov.va.mobilehealth.courier.utils.HttpSessionCollector;
import gov.va.mobilehealth.courier.utils.authConfig.VetAuthConfig;
import gov.va.mobilehealth.courier.utils.uriformatters.ResourceDirectoryBuilder;
import org.aeonbits.owner.ConfigFactory;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJacksonHttpMessageConverter;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.session.SessionInformation;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.security.oauth2.client.DefaultOAuth2ClientContext;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.security.oauth2.client.token.AccessTokenProvider;
import org.springframework.security.oauth2.client.token.AccessTokenProviderChain;
import org.springframework.security.oauth2.client.token.AccessTokenRequest;
import org.springframework.security.oauth2.client.token.DefaultAccessTokenRequest;
import org.springframework.security.oauth2.client.token.grant.client.ClientCredentialsAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.code.AuthorizationCodeAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.code.AuthorizationCodeResourceDetails;
import org.springframework.security.oauth2.client.token.grant.implicit.ImplicitAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.password.ResourceOwnerPasswordAccessTokenProvider;
import org.springframework.security.oauth2.common.AuthenticationScheme;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.ws.rs.core.UriBuilder;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Controller
@RequestMapping(value = "/service")
public class VeteranLoginLogoutController {
  private static final Logger LOGGER = LoggerFactory.getLogger(VeteranLoginLogoutController.class);
  private static final String KEY_AUTH_CODE = "code";
  private static final String KEY_GRANT_TYPE = "grant_type";
  private static final String KEY_SCOPE = "scope";
  private static final String KEY_REDIRECT_URI = "redirect_uri";
  private static final String KEY_ORIGINAL_REDIRECT_URI = "original_redirect_uri";
  private static final String KEY_CLIENT_ID = "client_id";
  private static final String KEY_STATE = "state";
  private static final String KEY_RESPONSE_TYPE = "response_type";
  private static final String KEY_CLIENT_SECRET = "client_secret";
  static VetAuthConfig authConfig = getAuthConfig();
  @Resource
  ResourceDirectoryBuilder resourceDirectoryBuilder;

  @Autowired(required = false)
  private SessionRegistry sessionRegistry;

  static VetAuthConfig getAuthConfig() {
    return ConfigFactory.create(VetAuthConfig.class, System.getProperties(), System.getenv());
  }

  @RequestMapping(value = "/vetoauthlogin", method = RequestMethod.GET)
  public void redirectToLoginPage(@RequestParam(value = "error", required = false) boolean error,
                                  @RequestParam(value = KEY_REDIRECT_URI, required = false) String originalRedirectUri,
                                  HttpServletRequest req, HttpServletResponse response) throws Exception {

    String baseUri = buildBaseUriString(req);

    UriBuilder authorizeUrlBuilder = UriBuilder.fromPath(authConfig.vetMhpAuthorizeUrl());

    //TODO: Add good logging, especially around any url building
    LOGGER.debug("Building redirectUri");
    String redirectUri = baseUri + "/vetoauthtoken";
    if (NullChecker.isNotNullish(originalRedirectUri)) {
      StringBuilder sb = new StringBuilder(redirectUri);
      sb.append("?");
      sb.append(KEY_ORIGINAL_REDIRECT_URI);
      sb.append("=");
      sb.append(originalRedirectUri);
      redirectUri = sb.toString();
    }

    LOGGER.debug("Building authorizeUrlBuilder");
    URI uri = authorizeUrlBuilder
      //TODO Don't use the MobileBlueButton values, need to have courier specific values registered with
      // AuthServices and utilize those
      .queryParam(KEY_RESPONSE_TYPE, KEY_AUTH_CODE)
      .queryParam(KEY_STATE, authConfig.oauthState())
      .queryParam(KEY_CLIENT_ID, authConfig.oauthClientId())
      .queryParam(KEY_REDIRECT_URI, redirectUri)
      .queryParam(KEY_SCOPE, authConfig.oauthScope())
      .build();

    LOGGER.debug("issuing redirect");
    redirect(response, uri.toString());
  }

  @RequestMapping(value = "/vetoauthtoken", method = RequestMethod.GET)
  public void getOauthToken(@RequestParam(value = KEY_AUTH_CODE, required = false) String code,
                            @RequestParam(value = KEY_ORIGINAL_REDIRECT_URI, required = false) String originalRedirectUri,
                            HttpServletRequest request, HttpServletResponse response) {

    String contextPath = NullChecker.isNotNullish(request.getContextPath()) ? request.getContextPath() : "";
    String redirectUri = NullChecker.isNotNullish(originalRedirectUri) ? buildOriginalRedirectUriForLogout(request) : contextPath + "/launchpad/";

    if (NullChecker.isNotNullish(code)) {
      LOGGER.debug("In Oauth token for code: " + code);
      SecurityContext ctx = SecurityContextHolder.getContext();
      ctx.setAuthentication(null);
      String token = getAccessToken(request, code, originalRedirectUri);
      redirectUri = redirectUri + "?token=" + token;
    } else {
      // START
      // the code below is not necessary when oauth login is completely handled by separate authoriziation service
      HttpSession session = request.getSession(false);
      if (session != null) {
        session.invalidate();
      }
      // END
    }
    redirect(response, redirectUri);
  }

  @RequestMapping(value = "/vetlogout", method = RequestMethod.GET)
  public void logout(HttpServletRequest request, HttpServletResponse response, ModelMap model) {
    invaliateSessionAndSecurityContext(request);
    String redirectUri = buildRedirectUri(request);
    redirect(response, redirectUri);
  }

  private String buildRedirectUri(HttpServletRequest request) {
    String baseUri = buildBaseUriString(request);
    String redirectUri = baseUri + "/vetoauthlogin";
    StringBuilder sb = new StringBuilder(redirectUri);
    sb.append("?");
    sb.append(KEY_REDIRECT_URI);
    sb.append("=");
    sb.append(buildOriginalRedirectUriForLogout(request));
    redirectUri = sb.toString();
    return redirectUri;
  }

  private void invaliateSessionAndSecurityContext(HttpServletRequest request) {

    Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

    if (authentication != null && authentication.getPrincipal() instanceof AppUser) {
      List<SessionInformation> allSessions = sessionRegistry.getAllSessions(authentication.getPrincipal(), false);
      LOGGER.info("Active Sessions for user : " + ((AppUser) authentication.getPrincipal()).getUsername() + " are : " + allSessions.size());
      for (SessionInformation sessionInformation : allSessions) {
        HttpSession httpSession = HttpSessionCollector.find(sessionInformation.getSessionId());
        if (httpSession != null) {
          LOGGER.info("invalidating http session Id : " + sessionInformation.getSessionId());
          httpSession.invalidate();
        }
        sessionInformation.expireNow();
        this.sessionRegistry.removeSessionInformation(sessionInformation.getSessionId());
      }
    }

    SecurityContextHolder.clearContext();
  }

  private String getAccessToken(HttpServletRequest req, String code, String originalRedirectUri) {
    //TODO Don't hardcode, hopefully this gets resolved through proxy config
    String baseUri = buildBaseUriString(req);

    String redirectUri = baseUri + "/vetoauthtoken";
    if (NullChecker.isNotNullish(originalRedirectUri)) {
      StringBuilder sb = new StringBuilder(redirectUri);
      sb.append("?");
      sb.append(KEY_ORIGINAL_REDIRECT_URI);
      sb.append("=");
      sb.append(originalRedirectUri);
      redirectUri = sb.toString();
    }

    MultiValueMap<String, String> formData = new LinkedMultiValueMap<String, String>();
    //TODO Use property file values
    formData.add(KEY_GRANT_TYPE, authConfig.oauthGrantType());
    formData.add(KEY_CLIENT_ID, authConfig.oauthClientId());
    formData.add(KEY_SCOPE, authConfig.oauthScope());
    formData.add(KEY_AUTH_CODE, code);
    formData.add(KEY_CLIENT_SECRET, authConfig.oauthClientSecret());
    formData.add(KEY_REDIRECT_URI, redirectUri);
    AuthorizationCodeResourceDetails resource = new AuthorizationCodeResourceDetails();

    resource.setAccessTokenUri(authConfig.vetMhpTokenUrl());
    resource.setClientId(authConfig.oauthClientId());

    resource.setScope(Arrays.asList(authConfig.oauthScope()));
    resource.setClientSecret(authConfig.oauthClientSecret());
    resource.setClientAuthenticationScheme(AuthenticationScheme.query);

    AccessTokenRequest request = new DefaultAccessTokenRequest();
    request.setAuthorizationCode(code);
    request.setPreservedState(new Object());
    request.setAll(formData.toSingleValueMap());

    OAuth2RestTemplate template = new OAuth2RestTemplate(resource, new DefaultOAuth2ClientContext(request));
    addJSONMessageConverter(template);

    OAuth2AccessToken accessToken = template.getAccessToken();

    return accessToken.getValue();
  }

  // This was added because of incompatability introduced by Yammer metrics library. Yammer metrics library has dependency on Jackson library and it was
  // interfering with object mapper
  private void addJSONMessageConverter(OAuth2RestTemplate template) {
    List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
    MappingJacksonHttpMessageConverter jsonMessageConverter = new MappingJacksonHttpMessageConverter();
    jsonMessageConverter.setObjectMapper(new ObjectMapper());
    messageConverters.add(jsonMessageConverter);

    AuthorizationCodeAccessTokenProvider authorizationCodeAccessTokenProvider = new AuthorizationCodeAccessTokenProvider();
    authorizationCodeAccessTokenProvider.setMessageConverters(messageConverters);

    AccessTokenProviderChain accessTokenProvider = new AccessTokenProviderChain(Arrays.<AccessTokenProvider>asList(
      authorizationCodeAccessTokenProvider, new ImplicitAccessTokenProvider(),
      new ResourceOwnerPasswordAccessTokenProvider(), new ClientCredentialsAccessTokenProvider()));

    template.setAccessTokenProvider(accessTokenProvider);
  }


  private String buildBaseUriString(HttpServletRequest req) {
    StringBuilder sb = new StringBuilder();
    sb.append("https");
    sb.append("://");

    sb.append(req.getServerName());

    sb.append("/vet/service");

    return sb.toString();
  }

  private String buildOriginalRedirectUriForLogout(HttpServletRequest req) {
    StringBuilder sb = new StringBuilder();
    sb.append("https");
    sb.append("://");

    sb.append(req.getServerName());

    sb.append("/vet/");

    return sb.toString();
  }

  private void redirect(HttpServletResponse response, String redirectUri) {
    try {
      response.sendRedirect(redirectUri);
    } catch (IOException e) {
      throw new RuntimeException("Exception while redirecting to " + redirectUri, e);
    }
  }

}
