package gov.va.mobilehealth.courier.resources;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.restassured.http.ContentType;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.utils.RESTIntTestConfig;
import gov.va.mobilehealth.courier.utils.StaffSecurityProxy;
import org.aeonbits.owner.ConfigFactory;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.hamcrest.Matchers.hasItem;
import static org.hamcrest.core.IsEqual.equalTo;


public class PatientUpdateResourceIntTest {

  protected static StaffSecurityProxy staffSecurityProxy;
  protected static RESTIntTestConfig config;

  private static final Logger LOGGER = LoggerFactory.getLogger(PatientUpdateResourceIntTest.class);
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

  @BeforeClass
  public static void initSecurity() {

    staffSecurityProxy = new StaffSecurityProxy();

    staffSecurityProxy.initAccessToken();
    config = ConfigFactory.create(RESTIntTestConfig.class, System.getProperties(), System.getenv());
  }

  @AfterClass
  public static void closeSecurityConnection() {
    staffSecurityProxy.logoutAndCloseConnection();
  }

  @Test
  public void searchPatient() {


    staffSecurityProxy.addAccessToken().
      given().
      queryParam("type", 1).
      queryParam("value", "patient49").
      when().
      get("/service/patients/search").
      then().
      log().
      everything().
      contentType(ContentType.JSON).
      statusCode(200).
      body("lastName", hasItem("patient49")).
      body("ssn", hasItem("999-99-9949"));
  }

  @Test
  public void registerPatient() {

    String patient = "{" +
      "\"firstName\": \"Integration\"," +
      "\"middleName\": \"Test\"," +
      "\"lastName\": \"Patient\"," +
      "\"ssn\": \"123-45-6789\"," +
      "\"dob\": \"04/07/1935\"," +
      "\"gender\": \"male\"," +
      "\"age\": \"79\"," +
      "\"patientIdentifier\": {" +
      "  \"uniqueId\": \"123\"," +
      "  \"assigningAuthority\": \"dfn-688\"" +
      "}," +
      "\"facilityId\": \"dfn-688\"," +
      "\"patientExistInCms\": false," +
      "\"preference\": {" +
      "  \"timeZone\": \"Atlantic/Bermuda\"," +
      "  \"phoneNumber\": \"11234567891\"," +
      "  \"messageDeliveryPreference\": {" +
      "    \"sendAnnie\": true," +
      "    \"sendSms\": true" +
      "  }," +
      "  \"notificationDeliveryPreference\": {" +
      "    \"sendAnnie\": false," +
      "    \"sendSms\": true" +
      "  }," +
      "  \"receiveFacilityMessage\": true," +
      "  \"receiveApptReminder\": true" +
      "}," +
      "\"consent\": 3" +
      "}";

    staffSecurityProxy.addAccessToken().
      given().
      contentType(ContentType.JSON).
      body(patient).
      when().
      put("/service/patients").
      then().
      log().
      everything().
      contentType(ContentType.JSON).
      statusCode(200).
      body("firstName", equalTo("Integration")).
      body("middleName", equalTo("Test")).
      body("lastName", equalTo("Patient")).
      body("ssn", equalTo("123-45-6789")).
      body("facilityId", equalTo("dfn-688")).
      body("consent", equalTo(3));

  }

  @Test
  public void updatePatient() throws JsonProcessingException {

    String patient = "{" +
      "  \"_id\": \"53fba91e0cf23303d3ef97ba\"," +
      "  \"firstName\": \"Integration\"," +
      "  \"middleName\": \"Test\"," +
      "  \"lastName\": \"Patient\"," +
      "  \"dob\": \"05/05/1900\"," +
      "  \"ssn\": \"123-45-6789\"," +
      "  \"gender\": \"male\"," +
      "  \"facilityId\": \"dfn-777\"," +
      "  \"patientExistInCms\": true," +
      "  \"preference\": {" +
      "    \"messageStarted\": true," +
      "    \"messageStopped\": false," +
      "    \"messagePaused\": false," +
      "    \"phoneNumber\": \"17031115555\"," +
      "    \"homePhoneNumber\": \"17037652255\"," +
      "    \"email\": \"zpatient@testmail.com\"," +
      "    \"timeZone\": \"America/New_York\"," +
      "    \"messageDeliveryPreference\": {" +
      "      \"sendAnnie\": true," +
      "      \"sendSms\": true" +
      "    }," +
      "    \"notificationDeliveryPreference\": {" +
      "      \"sendAnnie\": false," +
      "      \"sendSms\": true" +
      "    }," +
      "    \"receiveFacilityMessage\": false," +
      "    \"receiveApptReminder\": false" +
      "  }," +
      "  \"patientIdentifier\": {" +
      "    \"uniqueId\": \"123\"," +
      "    \"assigningAuthority\": \"EDIPI\"" +
      "  }," +
      "  \"consent\": 3," +
      "  \"consentDate\": \"2014-04-30\"," +
      "  \"age\": 114" +
      "}";

    staffSecurityProxy.addAccessToken().
      given().
      contentType(ContentType.JSON).
      body(patient).
      when().
      put("/service/patients").
      then().
      statusCode(200).
      body("_id", equalTo("53fba91e0cf23303d3ef97ba")).
      body("firstName", equalTo("Integration")).
      body("middleName", equalTo("Test")).
      body("lastName", equalTo("Patient")).
      body("dob", equalTo("1900-05-05")).
      body("ssn", equalTo("123-45-6789")).
      body("gender", equalTo("male")).
      body("facilityId", equalTo("dfn-777")).
      body("patientExistInCms", equalTo(true)).
      body("consent", equalTo(3)).
      body("consentDate", equalTo("2014-04-30")).
      body("age", equalTo(114)).
      body("preference.messageStarted", equalTo(true)).
      body("preference.messageStopped", equalTo(false)).
      body("preference.messagePaused", equalTo(false)).
      body("preference.phoneNumber", equalTo("17031115555")).
      body("preference.homePhoneNumber", equalTo("17037652255")).
      body("preference.email", equalTo("zpatient@testmail.com")).
      body("preference.timeZone", equalTo("America/New_York")).
      body("preference.messageDeliveryPreference.sendAnnie", equalTo(true)).
      body("preference.messageDeliveryPreference.sendSms", equalTo(true)).
      body("preference.notificationDeliveryPreference.sendAnnie", equalTo(false)).
      body("preference.notificationDeliveryPreference.sendSms", equalTo(true)).
      body("preference.receiveFacilityMessage", equalTo(false)).
      body("preference.receiveApptReminder", equalTo(false)).
      body("patientIdentifier.uniqueId", equalTo("123")).
      body("patientIdentifier.assigningAuthority", equalTo("EDIPI"));
  }

  @Test
  public void updatePatientIncorrectFormat() {

    String patient = "{" +
      "  \"_id\": \"53fba91e0cf23303d3ef97ba\"," +
      "  \"firstName\": \"zztest\"," +
      "  \"middleName\": null," +
      "  \"lastName\": \"patient55\"," +
      "  \"dob\": \"05/05/1900\"," +
      "  \"ssn\": \"123-45-6789\"," +
      "  \"gender\": \"male\"," +
      "  \"patientExistInCms\": true," +
      "  \"preference\": {" +
      "    \"messageStarted\": true," +
      "    \"messageStopped\": false," +
      "    \"messagePaused\": false," +
      "    \"phoneNumber\": \"1703111sdf\"," +
      "    \"homePhoneNumber\": \"17037652255\"," +
      "    \"email\": \"zpatient@testmail.com\"," +
      "    \"timeZone\": \"America/New_York\"" +
      "  }," +
      "  \"patientIdentifier\": {" +
      "    \"uniqueId\": \"123\"," +
      "    \"assigningAuthority\": \"EDIPI\"" +
      "  }," +
      "  \"lastNameInitial\": \"P\"," +
      "  \"last4SSN\": \"9955\"," +
      "  \"consent\": 3," +
      "  \"consentDate\": \"2014-04-30\"," +
      "  \"age\": 114" +
      "}";

    staffSecurityProxy.addAccessToken().
      given().
      contentType(ContentType.JSON).
      body(patient).
      when().
      put("/service/patients").
      then().statusCode(400);

  }

  @Test
  public void updatePatientWithDuplicateMobile() {



    String patient = "{" +
            "  \"_id\": \"53fba91e0cf23303d3ef97ba\"," +
            "  \"firstName\": \"zztest\"," +
            "  \"middleName\": null," +
            "  \"lastName\": \"patient55\"," +
            "  \"dob\": \"05/05/1900\"," +
            "  \"ssn\": \"123-45-6789\"," +
            "  \"gender\": \"male\"," +
            "  \"patientExistInCms\": true," +
            "  \"preference\": {" +
            "    \"messageStarted\": true," +
            "    \"messageStopped\": false," +
            "    \"messagePaused\": false," +
            "    \"phoneNumber\": \"17033334444\"," +  //this number exists
            "    \"homePhoneNumber\": \"17037652255\"," +
            "    \"email\": \"zpatient@testmail.com\"," +
            "    \"timeZone\": \"America/New_York\"" +
            "  }," +
            "  \"patientIdentifier\": {" +
            "    \"uniqueId\": \"123\"," +
            "    \"assigningAuthority\": \"EDIPI\"" +
            "  }," +
            "  \"lastNameInitial\": \"P\"," +
            "  \"last4SSN\": \"9955\"," +
            "  \"consent\": 3," +
            "  \"consentDate\": \"2014-04-30\"," +
            "  \"age\": 114" +
            "}";

    staffSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON).
            body(patient).
            when().
            put("/service/patients").
            then().statusCode(400)
            .body("code", equalTo(1004))
            .body("message", equalTo("Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX."));
  }

}
