package gov.va.mobilehealth.courier.resources;

import java.io.IOException;
import java.util.List;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.restassured.http.ContentType;
import com.jayway.restassured.response.Response;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.core.protocol.Vital;
import gov.va.mobilehealth.courier.utils.StaffSecurityProxy;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;


public class ProtocolUpdateResourceIntTest {

    protected static StaffSecurityProxy staffSecurityProxy;
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @BeforeClass
    public static void initSecurity() {

        staffSecurityProxy = new StaffSecurityProxy();
        staffSecurityProxy.initAccessToken();
    }

    @AfterClass
    public static void closeSecurityConnection() {
        staffSecurityProxy.logoutAndCloseConnection();
    }

    @Test
    public void createProtocol() throws IOException {
        String protocol = fixture("fixtures/protocolResIntTest.json");

        // Post the new protocol
        staffSecurityProxy.addAccessToken().given()
                .contentType(ContentType.JSON).body(protocol).when()
                .put("/service/protocols/new").then().log().everything()
                .statusCode(204);

        // Get the list of protocol included the new protocol just created
        Response res = staffSecurityProxy.addAccessToken().given()
        .when()
        .get("/service/protocols").then().log().everything()
        .contentType(ContentType.JSON)
        .statusCode(200)
        .extract()
        .response();
        
        final List<Protocol> protocolReturned =
                MAPPER.readValue(res.asString(),
                    new TypeReference<List<Protocol>>() {
                    }
                );
        
        // Verify the new protocol is there
        Protocol find = null;
        for (Protocol p : protocolReturned) {
            if (p.getName().equals("Integration test")) {
                find = p;
                break;
            }
        }
        
        assertThat(find).isNotNull();
        
        // Verify some fields of the new protocl
        Vital template = (Vital)find.getTemplates().get(0);
        assertThat(template.getSchedule().getScheduleType().getName()).isEqualTo("Once");
        assertThat(template.getSchedule().getReminderNumber()).isNull();
        assertThat(template.getService().getName()).isEqualTo("Blood Glucose");
        assertThat(template.getService().getReadings().get(0).getValid().getLow()).isEqualTo(30);
        assertThat(template.getService().getReadings().get(0).getAlert().getLow()).isNull();
        assertThat(template.getService().getReadings().get(0).getAlert().getHigh()).isNull();
    }
}
