package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.utils.VeteranSecurityProxy;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.restassured.http.ContentType;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.utils.RESTIntTestConfig;
import org.aeonbits.owner.ConfigFactory;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.hamcrest.Matchers.hasItem;
import static org.hamcrest.core.IsEqual.equalTo;


public class RegistrationResourceIntTest {

  protected static VeteranSecurityProxy veteranSecurityProxy;
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  protected static RESTIntTestConfig config;
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientUpdateResourceIntTest.class);

  @BeforeClass
  public static void initSecurity() {

    config = ConfigFactory.create(RESTIntTestConfig.class, System.getProperties(), System.getenv());

    veteranSecurityProxy = new VeteranSecurityProxy();
    veteranSecurityProxy.initAccessToken();

  }

  @AfterClass
  public static void closeSecurityConnection() {
    veteranSecurityProxy.logoutAndCloseConnection();
  }


  @Test
  public void updatePatient() throws JsonProcessingException {

//    String patient = {"_id":"53bc3ef20cf2636be815cb27","firstName":"zztest","middleName":null,"lastName":"patient55","dob":"05/05/1900","ssn":"999-99-9955","gender":"male","patientExistInCms":true,"preference":{"messageStarted":true,"messageStopped":false,"messagePaused":false,"phoneNumber":"17031110055","homePhoneNumber":"17037650055","messagingPreference":"secure","email":"zpatient@testmail.com","timeZone":"America/New_York","phoneType":"smart","basicPhoneNonSecure":false},"patientIdentifier":{"uniqueId":"PATID55","assigningAuthority":"EDIPI"},"lastNameInitial":"P","last4SSN":"9955","consent":true,"consentDate":"2014-04-30","age":114}

    String patient = "{\"_id\":\"53bc3ef20cf2636be815cb27\"," +
            "\"firstName\":\"zztest\"," +
            "\"middleName\":null," +
            "\"lastName\":\"patient55\"," +
            "\"dob\":\"05/05/1900\"," +
            "\"ssn\":\"999-99-9955\"," +
            "\"gender\":\"male\"," +
            "\"patientExistInCms\":true," +
            "\"facilityId\": \"dfn-777\"," +
            "\"preference\":{" +
            "\"messageStarted\":true," +
            "\"messageStopped\":false," +
            "\"messagePaused\":false," +
            "\"phoneNumber\":\"17031110055\"," +
            "\"homePhoneNumber\":\"17037650055\"," +
            "\"email\":\"zpatient@testmail.com\"," +
            "\"timeZone\":\"America/New_York\"," +
            "  \"messageDeliveryPreference\": {" +
            "    \"sendAnnie\": true," +
            "    \"sendSms\": true" +
            "  }," +
            "  \"notificationDeliveryPreference\": {" +
            "    \"sendAnnie\": false," +
            "    \"sendSms\": true" +
            "  }," +
            "\"receiveFacilityMessage\": false," +
            "\"receiveApptReminder\": false" +
            "}," +
            "\"patientIdentifier\":{" +
            "\"uniqueId\":\"PATID55\"," +
            "\"assigningAuthority\":\"EDIPI\"}," +
            "\"lastNameInitial\":\"P\"," +
            "\"last4SSN\":\"9955\"," +
            "\"consent\":3," +
            "\"consentDate\":\"2014-04-30\"," +
            "\"age\":114}";

    veteranSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON)
            .body(patient).
            when().
            put("/service/registration").
            then().
            statusCode(204);

    //next to verify updated values since it does not return patient in put method
    veteranSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON).
            expect().
            body("firstName", equalTo("zztest")).
            body("middleName", equalTo(null)).
            body("lastName", equalTo("patient55")).
            body("dob",equalTo("1900-05-05")).
            body("ssn", equalTo("999-99-9955")).
            body("gender", equalTo("MALE")).
            body("facilityId", equalTo("dfn-777")).
            body("patientExistInCms", equalTo(true)).
            body("preference.messageStarted", equalTo(true)).
            body("preference.messageStopped", equalTo(false)).
            body("preference.messagePaused", equalTo(false)).
            body("preference.phoneNumber",equalTo("17031110055")).
            body("preference.homePhoneNumber",equalTo("17037650055")).
            body("preference.email", equalTo("zpatient@testmail.com")).
            body("preference.timeZone", equalTo("America/New_York")).
            body("preference.messageDeliveryPreference.sendAnnie", equalTo(true)).
            body("preference.messageDeliveryPreference.sendSms", equalTo(true)).
            body("preference.notificationDeliveryPreference.sendAnnie", equalTo(false)).
            body("preference.notificationDeliveryPreference.sendSms", equalTo(true)).
            body("preference.receiveFacilityMessage", equalTo(false)).
            body("preference.receiveApptReminder", equalTo(false)).
            body("patientIdentifier.uniqueId", equalTo("PATID55")).
            body("patientIdentifier.assigningAuthority", equalTo("EDIPI")).
            body("lastNameInitial",equalTo("P")).
            body("last4SSN",equalTo("9955")).
            body("consent", equalTo(3)).
            body("consentDate",equalTo("2014-04-30")).
            body("age",equalTo(114)).
            when().
            get("/service/registration/userProfile").
            then().
            log().
            everything().
            contentType(ContentType.JSON).
            statusCode(200);


  }

  @Test
  public void findPatient() {
// this information whould match updatePatient info
    veteranSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON).
            expect().
            body("firstName", equalTo("zztest")).
            body("middleName", equalTo(null)).
            body("lastName", equalTo("patient55")).
            body("dob",equalTo("1900-05-05")).
            body("ssn", equalTo("999-99-9955")).
            body("gender", equalTo("MALE")).
            body("patientExistInCms", equalTo(true)).
            body("preference.messageStarted", equalTo(true)).
            body("preference.messageStopped", equalTo(false)).
            body("preference.messagePaused", equalTo(false)).
            body("preference.phoneNumber",equalTo("17031110055")).
            body("preference.homePhoneNumber",equalTo("17037650055")).
            body("preference.email", equalTo("zpatient@testmail.com")).
            body("preference.timeZone", equalTo("America/New_York")).
            body("preference.messageDeliveryPreference.sendAnnie", equalTo(true)).
            body("preference.messageDeliveryPreference.sendSms", equalTo(false)).
            body("preference.notificationDeliveryPreference.sendAnnie", equalTo(false)).
            body("preference.notificationDeliveryPreference.sendSms", equalTo(true)).
            body("patientIdentifier.uniqueId", equalTo("PATID55")).
            body("patientIdentifier.assigningAuthority", equalTo("EDIPI")).
            body("lastNameInitial",equalTo("P")).
            body("last4SSN",equalTo("9955")).
            body("consent", equalTo(3)).
            body("consentDate",equalTo("2014-04-30")).
            body("age",equalTo(114)).
            when().
            get("/service/registration/userProfile").
            then().
            log().
            everything().
            contentType(ContentType.JSON).
            statusCode(200);


  }


  //
  @Test
  public void updatePatientIncorrectFormat() {

    String patient = "{" +
            "  \"_id\": \"53bc3ef20cf2636be815cb27\"," +
            "  \"firstName\": \"zztest\"," +
            "  \"middleName\": null," +
            "  \"lastName\": \"patient55\"," +
            "  \"dob\": \"05/05/1900\"," +
            "  \"ssn\": \"999-99-9955\"," +
            "  \"gender\": \"MALE\"," +
            "  \"patientExistInCms\": true," +
            "  \"preference\": {" +
            "    \"messageStarted\": true," +
            "    \"messageStopped\": false," +
            "    \"messagePaused\": false," +
            "    \"phoneNumber\": \"1703111sdf\"," +
            "    \"homePhoneNumber\": \"17037652255\"," +
            "    \"email\": \"zpatient@testmail.com\"," +
            "    \"timeZone\": \"America/New_York\"" +
            "  }," +
            "  \"patientIdentifier\": {" +
            "    \"uniqueId\": \"123\"," +
            "    \"assigningAuthority\": \"EDIPI\"" +
            "  }," +
            "  \"lastNameInitial\": \"P\"," +
            "  \"last4SSN\": \"9955\"," +
            "  \"consent\": 3," +
            "  \"consentDate\": \"2014-04-30\"," +
            "  \"age\": 114" +
            "}";

    veteranSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON).
            body(patient).
            when().
            put("/service/registration").
            then().statusCode(400);

  }
  @Test
  public void updatePatientWithDuplicateMobile() {

    String patient ="{\"_id\":\"53fba91e0cf23303d3ef97ba\"," +
            "\"firstName\":\"zztest\"," +
            "\"middleName\":null," +
            "\"lastName\":\"patient55\"," +
            "\"dob\":\"05/05/1900\"," +
            "\"ssn\":\"999-99-9955\"," +
            "\"gender\":\"\"," +
            "\"patientExistInCms\":true," +
            "\"preference\":{" +
            "\"messageStarted\":false," +
            "\"messageStopped\":false," +
            "\"messagePaused\":false," +
            "\"phoneNumber\":\"17033334444\"," +
            "\"homePhoneNumber\":null," +
            "\"email\":null," +
            "\"timeZone\":\"America/New_York\"" +
            "}," +
            "\"patientIdentifier\":{" +
            "\"uniqueId\":\"PATID55\"," +
            "\"assigningAuthority\":\"EDIPI\"}," +
            "\"lastNameInitial\":\"P\"," +
            "\"last4SSN\":\"9955\"," +
            "\"consent\":3," +
            "\"consentDate\":\"2014-09-04\"," +
            "\"age\":114}";

    veteranSecurityProxy.addAccessToken().
            given().
            contentType(ContentType.JSON).
            body(patient).
            when().
            put("/service/registration").
            then().statusCode(400)
            .body("code", equalTo(1004))
            .body("message", equalTo("Please check the mobile phone number entered. Our records show it is currently in use. If you still have a problem, contact our Help Desk at XXXX."));
  }
}
