package gov.va.mobilehealth.courier.utils;

import com.agilex.healthcare.mobilehealthplatform.security.Client;
import com.agilex.healthcare.mobilehealthplatform.security.ClientIdentification;
import com.agilex.healthcare.mobilehealthplatform.security.User;
import com.agilex.healthcare.mobilehealthplatform.security.infrastructure.HttpClient;
import com.agilex.healthcare.mobilehealthplatform.security.infrastructure.Value;
import com.jayway.restassured.specification.RequestSpecification;
import org.aeonbits.owner.ConfigFactory;

import static com.jayway.restassured.RestAssured.given;

public class StaffSecurityProxy {

  protected RESTIntTestConfig config;

  private static final String STAFF_AUTHORIZE_BASE_URI = "/AuthorizationServices/provider";

  protected static HttpClient httpClient;
  protected Client client;
  protected User user;

  private Value accessToken;

  public StaffSecurityProxy() {
    config = ConfigFactory.create(RESTIntTestConfig.class, System.getProperties(), System.getenv());

    httpClient = new HttpClient(config.oauthStaffBaseURI());
    ClientIdentification clientIdentification = new ClientIdentification(config.oauthClientSecret(), config.oauthClientId(), config.oauthStaffRedirectURI());
    client = Client.createClient(httpClient, clientIdentification, STAFF_AUTHORIZE_BASE_URI);
  }

  public void initAccessToken() {
    String userId = config.oauthStaffAccessCode();
    String pass = config.oauthStaffVerifyCode();
    String facilityCode = config.oauthFacilityId();
    String facilityName = config.oauthFacilityName();

    initAccessToken(userId, pass, facilityCode, facilityName);
  }

  public void initAccessToken(String userId, String pass, String facilityCode, String facilityName) {
    user = new User(httpClient, userId, pass, facilityCode, facilityName, STAFF_AUTHORIZE_BASE_URI);

    getAccessToken();
  }

  private void getAccessToken() {
    Value code = client.authorize(user);
    accessToken = client.getAccessToken(code);
  }

  public RequestSpecification addAccessToken() {
    return given().
      baseUri(config.baseURI()).
      port(config.port()).
      basePath(config.baseStaffPath()).
      relaxedHTTPSValidation().
      header("Authorization", "Bearer " + accessToken.toString());
  }

  public void logoutAndCloseConnection() {
    client.logoutAndCloseConnection(user, accessToken);
  }

}
