package gov.va.mobilehealth.courier.utils;

import com.agilex.healthcare.mobilehealthplatform.security.Client;
import com.agilex.healthcare.mobilehealthplatform.security.ClientIdentification;
import com.agilex.healthcare.mobilehealthplatform.security.User;
import com.agilex.healthcare.mobilehealthplatform.security.infrastructure.HttpClient;
import com.agilex.healthcare.mobilehealthplatform.security.infrastructure.Value;
import com.jayway.restassured.specification.RequestSpecification;
import gov.va.mobilehealth.courier.resources.PatientUpdateResourceIntTest;
import gov.va.mobilehealth.courier.utils.RESTIntTestConfig;
import org.aeonbits.owner.ConfigFactory;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.jayway.restassured.RestAssured.given;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Add a Description.  Should include class purpose (why not what) and assumptions.
 */
public class VeteranSecurityProxy {

    private static final String HOST_URI = "https://hatest.agilexhealth.com";
    private static final String REDIRECT_URI = "https://hatest.agilexhealth.com/launchpad/";
    private static final String VETERAN_AUTHORIZE_BASE_URI = "/ssoeproxy/veteran";
    private static final String VETERAN_SECURITY_BASE_URI = "/ssoeproxy";
    protected RESTIntTestConfig config;
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientUpdateResourceIntTest.class);
    protected static HttpClient httpClient;
    protected Client client;
    protected User veteran;
    private Value accessToken;

  public VeteranSecurityProxy() {
    config = ConfigFactory.create(RESTIntTestConfig.class, System.getProperties(), System.getenv());
    httpClient = new HttpClient(config.oauthVetBaseURI());

    ClientIdentification clientIdentification = new ClientIdentification("TESTLAUNCHPAD", "launchPad", config.oauthVetRedirectURI());
    client = Client.createClient(httpClient, clientIdentification, config.oauthVetAuthrizeBaseUri());
    initAccessToken();
  }



  public void initAccessToken() {
    veteran = new User(httpClient, "zztest.patient55", "pass", config.oauthVetSecurityBaseUri());
    LOGGER.debug("veteran");
    LOGGER.debug(veteran.toString());
    getAccessToken();
  }


  private void getAccessToken() {
    Value code = client.authorize(veteran);
    LOGGER.debug("code  too   ");
    LOGGER.debug(code.toString());
    accessToken = client.getAccessToken(code);
  }

  public RequestSpecification addAccessToken() {
    return given().
            baseUri(config.baseURI()).
            port(config.portVA()).
            basePath(config.baseVetPath()).
            relaxedHTTPSValidation().
            header("Authorization", "Bearer " + accessToken.toString());
  }

  public void logoutAndCloseConnection() {
    client.logoutAndCloseConnection(veteran, accessToken);
  }


////    @Test
//    public void getAccessTokenForInvalidVeteran() {
//        HttpClient httpClient = new HttpClient(HOST_URI);
//
//        ClientIdentification clientIdentification = new ClientIdentification("TESTLAUNCHPAD", "launchPad", REDIRECT_URI);
//        Client client = Client.createClient(httpClient, clientIdentification, VETERAN_AUTHORIZE_BASE_URI);
//        User veteran = new User(httpClient, "invalid_patient", "pass", VETERAN_SECURITY_BASE_URI);
//
//        try {
//            Value code = client.authorize(veteran);
//        } catch (RuntimeException r) {
//            assertEquals("Unable to log in user.  Make sure user credentials are correct.", r.getMessage());
//        }
//    }

}
