package gov.va.mobilehealth.courier.bcm.appointments;

import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertEquals;

public class AppointmentReminderGeneratorTest {


  private AssessmentProfileFutureAppointment createAppointment(int minutesInFuture) {
    AssessmentProfileFutureAppointment appointment = new AssessmentProfileFutureAppointment();
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier("123401"));
    DateTime startDate = DateTime.now().plusMinutes(minutesInFuture);
//    appointment.setStartDate(DateTime.parse("10/24/2014 10:00:00", DateTimeFormat.forPattern("MM/dd/yyyy HH:mm:ss")));
    appointment.setStartDate(startDate);
    appointment.setFacility("Test Facility");
    return appointment;
  }

  @Test
  public void canGenerateRemindersForAppointmentIn1Minutes(){
    //create appointment more than 5 Minutes ahead
    AssessmentProfileFutureAppointment appointment = createAppointment(1);
    List<AppointmentReminder> appointmentReminders = getReminders(appointment);


    //assert two reminders for 3 and 5 Minutes in advance
    assertEquals(0, appointmentReminders.size());
  }

  private List<AppointmentReminder> getReminders(AssessmentProfileFutureAppointment appointment) {
    //generator reminders
    AppointmentReminderGenerator gen = new AppointmentReminderGenerator();
    return gen.createAppointmentReminders(appointment);
  }

  @Test
  public void canGenerateRemindersForAppointmentIn2Minutes(){
    //create appointment more than 5 Minutes ahead
    AssessmentProfileFutureAppointment appointment = createAppointment(2);

    //generator reminders
    List<AppointmentReminder> appointmentReminders = getReminders(appointment);

    //assert two reminders for 3 and 5 Minutes in advance
    assertEquals(1, appointmentReminders.size());
  }
  @Test
  public void canGenerateRemindersForAppointmentIn4Minutes(){
    AssessmentProfileFutureAppointment appointment = createAppointment(5);

    //generator reminders
    List<AppointmentReminder> appointmentReminders = getReminders(appointment);

    //assert two reminders for 1 and 3 Minutes in advance
    assertEquals(2, appointmentReminders.size());
  }

  @Test
  public void reminderMessageHasCorrectDeliverySchedule(){
    AssessmentProfileFutureAppointment appointment = createAppointment(2);

    DateTime expectedDeliveryDateTime = appointment.getStartDate().minus(Minutes.minutes(1));

    //generator reminders
    List<AppointmentReminder> appointmentReminders = getReminders(appointment);

    //assert two reminders for 1 Minutes in advance
    assertEquals(1, appointmentReminders.size());

    AppointmentReminder reminder = appointmentReminders.get(0);

    assertEquals(expectedDeliveryDateTime, reminder.getDeliveryDateTime());
    assertEquals(appointment, reminder.getFutureAppointment());
  }
}
