package gov.va.mobilehealth.courier.bcm.appointments;

import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.junit.Assert;
import org.junit.Test;

public class AppointmentReminderTest {

  @Test
  public void testGetMessageContent() throws Exception {
   AppointmentReminder reminder = createAppointmentReminder();
    reminder.getFutureAppointment().setStartDate(new DateTime(2014, 10, 24, 10, 00, 00));
   String message = reminder.getMessageContent();
    String expectedMessage = "On 10/24 at 10:00 AM you have an appointment at Test Facility. If you have questions or to cancel call 800-293-8262. Thanks, Annie";
    Assert.assertEquals(expectedMessage, message);
  }

  @Test
  public void testGetMessageContentWithPmAppointment() throws Exception {
   AppointmentReminder reminder = createAppointmentReminder();
    reminder.getFutureAppointment().setStartDate(new DateTime(2014, 10, 24, 14, 30, 00));
   String message = reminder.getMessageContent();
    String expectedMessage = "On 10/24 at 02:30 PM you have an appointment at Test Facility. If you have questions or to cancel call 800-293-8262. Thanks, Annie";
    Assert.assertEquals(expectedMessage, message);
  }

  private AppointmentReminder createAppointmentReminder(){
    AppointmentReminder reminder = new AppointmentReminder();
    reminder.setFutureAppointment(createFutureAppointment());
    //10 seconds in the future?
    reminder.setDeliveryDateTime(DateTime.now().plus(Seconds.seconds(3)));
    return reminder;
  }
  private AssessmentProfileFutureAppointment createFutureAppointment() {
    AssessmentProfileFutureAppointment appointment = new AssessmentProfileFutureAppointment();
    appointment.setId("asd9f87asdf87asdf98a0sdfg8as0g");
    appointment.setFacility("Test Facility");
    appointment.setStartDate(DateTime.now());  //evaluate what time we want here
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier("100840"));
    appointment.setPatientIcnIdentifier(new IcnIdentifier("100840"));
    appointment.setClinicPhone("800-293-8262");
    return appointment;
  }

}
