package gov.va.mobilehealth.courier.bcm.appointments;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import org.joda.time.DateTime;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class AssessmentProfileFutureAppointmentTest {
  //Use the mapper configured for MongoDocumentBase
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
  private static final Logger LOGGER = LoggerFactory.getLogger(AssessmentProfileFutureAppointmentTest.class);

  @Test
  public void serializesToJSON() throws IOException {
    final AssessmentProfileFutureAppointment appointment = getAssessmentProfileFutureAppointment();
    String serializedAppointment = MAPPER.writeValueAsString(appointment);

    final AssessmentProfileFutureAppointment deserializedAppointment = MAPPER.readValue(fixture("fixtures/futureAppointment.json"), AssessmentProfileFutureAppointment.class);

    String serDeserAppointment = MAPPER.writeValueAsString(deserializedAppointment);
    LOGGER.info(serializedAppointment);
    LOGGER.info(serDeserAppointment);

    assertThat(serializedAppointment).isEqualTo(
      serDeserAppointment);
  }
  @Test
  public void deserializesFromJSON() throws IOException {
    final AssessmentProfileFutureAppointment appointment = getAssessmentProfileFutureAppointment();
    String serializedAppointment = MAPPER.writeValueAsString(appointment);

    final AssessmentProfileFutureAppointment deserializedAppointment = MAPPER.readValue(fixture("fixtures/futureAppointment.json"), AssessmentProfileFutureAppointment.class);

    LOGGER.info(serializedAppointment);
    String serDeserAppointment = MAPPER.writeValueAsString(deserializedAppointment);
    LOGGER.info(serDeserAppointment);

    assertThat(deserializedAppointment).isEqualTo(appointment);
  }

  @Test
  public void serializesToJSONCorrectDateFormat() throws IOException {
    final AssessmentProfileFutureAppointment appointment = getAssessmentProfileFutureAppointment();
    String serializedAppointment = MAPPER.writeValueAsString(appointment);

    long startDateLong = appointment.getStartDate().toDate().getTime();
    assertThat(serializedAppointment).doesNotContain(Long.toString(startDateLong));
  }

  public AssessmentProfileFutureAppointment getAssessmentProfileFutureAppointment() {
    AssessmentProfileFutureAppointment appointment = new AssessmentProfileFutureAppointment();

    appointment.setFacility("Test Facility");

    DateTime startDate = DateTime.parse("1972-12-28T12:00:01.000Z");
    appointment.setStartDate(startDate);
    appointment.setPatientIcnIdentifier(new IcnIdentifier("Test Icn Id"));
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier("Test Edipi Id"));
    appointment.setClinicPhone("123-456-7890");

    return appointment;
  }
}
