package gov.va.mobilehealth.courier.bcm.appointments;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.utility.DateHelper;
import org.joda.time.DateTime;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class FutureAppointmentMapperTest {

  @Test
  public void canMapCdsAppointmentsToFutureAppointments() {
    Appointment appointment = new Appointment();
    appointment.setAppointmentStartDate(DateHelper.parseDateTime("10/24/2014 10:00:00 EDT"));
    appointment.setClinicName("Test Clinic Name");
    appointment.setFacilityName("Test Facility Name");
    appointment.setPatientIdentifier("ICN", "12301");
    appointment.setStatus("pending");
    appointment.setStatusCode("R");
    appointment.setClinicPhone("123-456-7890");

    FutureAppointmentMapper mapper = new FutureAppointmentMapper();
    EdipiIdentifier edipiIdentifier = new EdipiIdentifier("456");
    AssessmentProfileFutureAppointment futureAppointment = mapper.mapAppointment(edipiIdentifier, appointment);

    assertNotNull(futureAppointment);
    assertEquals(new DateTime(2014, 10, 24, 10, 0, 0), futureAppointment.getStartDate());
    assertEquals("Test Facility Name", futureAppointment.getFacility());
    assertEquals(new IcnIdentifier("12301"), futureAppointment.getPatientIcnIdentifier());
    assertEquals(new EdipiIdentifier("456"), futureAppointment.getpatientEdipiIdentifier());
    assertEquals("123-456-7890", futureAppointment.getClinicPhone());
  }
}
