package gov.va.mobilehealth.courier.protocol;

import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.ProtocolTemplateBuilder;
import gov.va.mobilehealth.courier.helper.MockMessageProducer;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

@RunWith(Parameterized.class)
public class MessageProcessorMatcherTest {

  private static final List<PatientProtocol> patientProtocols = Arrays.asList(
      ProtocolTemplateBuilder.buildPatientProtocol("patientId", ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce()),
      ProtocolTemplateBuilder.buildPatientProtocol("patientId", ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeDaysOfWeek()),
      ProtocolTemplateBuilder.buildPatientProtocol("patientId", ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeEverySoMany()));
  private String messageText;

  private boolean expectedMatch;

  public MessageProcessorMatcherTest(String messageText, boolean expectedMatch) {
    this.messageText = messageText;
    this.expectedMatch = expectedMatch;
  }

  @Parameterized.Parameters(name = "messageText: {0}, expectedMatch: {1} ")
  public static Collection<Object[]> data() throws Exception {
    return Arrays.asList(new Object[][]{
        {"BP 123/99", true},
        {"BLood 123/99", true},
        {"My blood pressure today is 123 on 99", true},
        {"Bld 1/99", false}
    });
  }

  @Test
  public void findMatchProtocolReturnsExpectedPatientProtocol() throws Exception {
    MessageProcessorService msgService = new MessageProcessorService(new MockMessageProducer());

    assertThat(msgService.findMatchProtocol(patientProtocols, messageText).isPresent()).isEqualTo(expectedMatch);
  }

}
