package gov.va.mobilehealth.courier.resources.helper;

import gov.va.mobilehealth.courier.core.DeliveryPreference;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.NotificationDeliveryPreference;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;

import static org.fest.assertions.api.Assertions.assertThat;

public class OutgoingMessageProcessorTest {

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");

  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void generateSmsNoticeTest() throws Exception {
      // create a patient
      Patient patient = new Patient();
      patient.setFirstName("John");
      patient.setLastName("Smith");

      PatientPreference pref = new PatientPreference();
      pref.setPhoneNumber("1-123-456-7890");
      NotificationDeliveryPreference notificationDeliveryPref = new NotificationDeliveryPreference();
      notificationDeliveryPref.setSendSms(true);
      pref.setNotificationDeliveryPreference(notificationDeliveryPref);
      patient.setPreference(pref);

      // create a message
      CMSMessage originalMessage = new CMSMessage("", patient, "", MessageType.OUT);
//      originalMessage.setForceSendViaSMS(false);

      OutgoingMessageProcessor processor = new OutgoingMessageProcessor(); 

      // 1. patient agrees to receive sms notification
      assertThat(processor.generateSmsNotice(originalMessage, patient)).isNotNull();

      // 2. patient has smart phone, original message go through SMS
      originalMessage.setForceSendViaSMS(true);
      assertThat(processor.generateSmsNotice(originalMessage, patient)).isNull();

      // 3. patient does not agree to receive sms notification
      notificationDeliveryPref.setSendSms(false);
      patient.getPreference().setNotificationDeliveryPreference(notificationDeliveryPref);
      assertThat(processor.generateSmsNotice(originalMessage, patient)).isNull();

      // 3. patient does not have smart phone, original message go through SMS
      originalMessage.setForceSendViaSMS(false);
      assertThat(processor.generateSmsNotice(originalMessage, patient)).isNull();
  }
  @Test
  public void changePreferenceTest() throws Exception {

    Patient patient = new Patient();
    patient.setFirstName("John");
    patient.setLastName("Smith");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");
    NotificationDeliveryPreference notificationDeliveryPref = new NotificationDeliveryPreference();
    notificationDeliveryPref.setSendSms(true);
    pref.setNotificationDeliveryPreference(notificationDeliveryPref);
    patient.setPreference(pref);

    // create a message
    CMSMessage originalMessage = new CMSMessage(null, patient, "", MessageType.OUT);
//    originalMessage.setForceSendViaSMS(false);

    OutgoingMessageProcessor processor = new OutgoingMessageProcessor();
    assertThat(processor.generateSmsNotice(originalMessage, patient)).isNotNull();
    notificationDeliveryPref.setSendSms(false);
    patient.getPreference().setNotificationDeliveryPreference(notificationDeliveryPref);
    originalMessage.setForceSendViaSMS(true);
    assertThat(processor.generateSmsNotice(originalMessage, patient)).isNull();

  }
  @Test
  public void validateDestination() throws Exception{

    Patient patient = getPatient();
    patient.getPreference().setMessageStarted(false);
    DeliveryPreference deliveryPref = new DeliveryPreference();
    deliveryPref.setSendSms(false);
    patient.getPreference().setMessageDeliveryPreference(deliveryPref);

    CMSMessage originalMessage = new CMSMessage("", patient, "", MessageType.OUT);
//    originalMessage.setForceSendViaSMS(false);

    OutgoingMessageProcessor processor = new OutgoingMessageProcessor();

    assertThat(processor.setMessageDestination(originalMessage).getDestination()).isEqualTo(IncomingMessageProcessor.APP_NAME);



  }
  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }



}
