package gov.va.mobilehealth.courier.resources.helper;

import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class ResponseMessageGeneratorTest {

  ResponseMessageGenerator generator;
  @Before
  public void Setup(){
    generator = new ResponseMessageGenerator();
  }
  @Test
  public void testGeneratorStartResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    CMSMessage message = generator.generatorStartResponse(patientResponse);

    assertLastMessage(message, patient, patientResponse, patientResponse.getDestination(), ResponseMessageGenerator.START_RESPONSE);
  }

  @Test
  public void testGenerateHelpResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    CMSMessage message = generator.generateHelpResponse(patientResponse, patient);

    assertLastMessage(message, patient, patientResponse, patientResponse.getDestination(), new ResponseMessageGenerator().getHelpResponse());

  }

  @Test
  public void testGenerateReadingBeforeStartResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    CMSMessage message = generator.generateReadingBeforeStartResponse(patientResponse, patient.getId());

    assertLastMessage(message, patient, patientResponse, patientResponse.getDestination(), ResponseMessageGenerator.START_ERROR);

  }

  @Test
  public void testGenerateReadingResponse() throws Exception {
  }

  @Test
  public void testGeneratePauseStartErrorResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    CMSMessage message = generator.generatePauseStartErrorResponse(patientResponse, patient, responseType);
//    imp.pauseProtocol(patientResponse, patient, responseType);

    assertLastMessage(message, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);

  }

  @Test
  public void testGeneratePauseResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    CMSMessage message = generator.generatePauseResponse(patientResponse, patient, responseType);
//    imp.pauseProtocol(patientResponse, patient, responseType);

    assertLastMessage(message, patient, patientResponse, responseType, ResponseMessageGenerator.PAUSE_RESPONSE);

  }

  @Test
  public void testGenerateResumeStartError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    CMSMessage message = generator.generateResumeStartError(patientResponse, patient, responseType);

    assertLastMessage(message, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);

  }

  @Test
  public void testGenerateResumePauseError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    p1.getPreference().setMessagePaused(false);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    CMSMessage message = generator.generateResumePauseError(patientResponse, patient, responseType);

    assertLastMessage(message, patient, patientResponse, responseType, ResponseMessageGenerator.PAUSE_ERROR);
  }

  @Test
  public void testGenreateStopStartError() throws Exception {

  }

  @Test
  public void testGenerateStopResponse() throws Exception {

  }

  @Test
  public void testGenerateResumeResponse() throws Exception {

  }

  private void assertLastMessage(CMSMessage message, Patient patient, PatientIncomingMessage expectedSource, String responseType, String expectedMessageText) {
    //assert lastMessage values
    assertCommonMessageTraits2(patient, expectedSource, responseType, message);
    Assert.assertEquals("Text", expectedMessageText, message.getText());
  }

  private void assertCommonMessageTraits2(Patient patient, PatientIncomingMessage patientResponse, String expectedSource, CMSMessage lastMessage) {
    Assert.assertNotNull(lastMessage);

    Assert.assertEquals("IncomingMessageId", patientResponse.getId(), lastMessage.getIncomingMessageId());
    Assert.assertEquals("Source", expectedSource, lastMessage.getSource());
    Assert.assertEquals("Destination", patientResponse.getSource(), lastMessage.getDestination());
    Assert.assertEquals("Type", "Out", lastMessage.getType());
    Assert.assertEquals("PatientId", patient.getId(), lastMessage.getPatientId());
    Assert.assertEquals("SendSMS", false, lastMessage.isForceSendViaSMS());
  }

  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }

  PatientIncomingMessage createPatientIncomingMessage(Patient patient){
    PatientIncomingMessage message = new PatientIncomingMessage("", "");
//    message.setId("123");
    message.setSource("messageSource");
    message.setDestination("destination");

    //TODO: Sometimes patientId off of message is used, sometimes off of passed in patient
    message.setPatientId(patient.getId());

    return message;
  }

}
