package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.utils.DateParser;

import org.junit.Assert;
import org.junit.Test;
import org.quartz.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import static org.quartz.CalendarIntervalScheduleBuilder.calendarIntervalSchedule;
import static org.quartz.CronScheduleBuilder.cronSchedule;
import static org.quartz.TriggerBuilder.newTrigger;


/**
 * Created with IntelliJ IDEA.
 * User: shimk
 * Date: 12/12/13
 * Time: 5:03 PM
 * To change this template use File | Settings | File Templates.
 */
public class ProtocolChronGeneratorTest {

    public static  final String triggerName = "initProtocol";
    public static  final String triggerGroup = "initProtocol";
    public static  final String jobGroup = "initProtocol";
    @Test
    public void canConvertAMTime() throws Exception {
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setHour(5);

        StringBuilder sb = new StringBuilder();
        sb = ProtocolChronGenerator.convertHour(scheduleTime, sb);

        Assert.assertEquals("5", sb.toString());
    }

    @Test
    public void canConvertPMTime() throws Exception {
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setMeridian(Meridian.PM);
        scheduleTime.setHour(5);

        StringBuilder sb = new StringBuilder();
        sb = ProtocolChronGenerator.convertHour(scheduleTime, sb);

        Assert.assertEquals("17", sb.toString());
    }

    @Test
    public void canConvert12AM() throws Exception {
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setHour(12);

        StringBuilder sb = new StringBuilder();
        sb = ProtocolChronGenerator.convertHour(scheduleTime, sb);

        Assert.assertEquals("0", sb.toString());
    }

    @Test
    public void canConvertDaysOfWeek(){
        Vital protocol = new Vital();

        TypeDaysOfWeek daysOfWeek = new TypeDaysOfWeek();
        daysOfWeek.setName("DaysOfWeek");
        daysOfWeek.setMonday(true);
        daysOfWeek.setFriday(true);
        daysOfWeek.setSaturday(true);
        Schedule schedule = new Schedule();
        protocol.setSchedule(schedule);
        protocol.getSchedule().setScheduleType(daysOfWeek);

        StringBuilder sb = ProtocolChronGenerator.getScheduleDayOfWeek(protocol.getSchedule());

        Assert.assertEquals("MON,FRI,SAT", sb.toString());

    }

    @Test
    public void canCreateChronJobDescriptor(){
        Vital protocol = new Vital();
        TypeDaysOfWeek daysOfWeek = new TypeDaysOfWeek();
        daysOfWeek.setName("DaysOfWeek");
        daysOfWeek.setSaturday(true);
        Schedule schedule = new Schedule();
        protocol.setSchedule(schedule);
        protocol.getSchedule().setScheduleType(daysOfWeek);

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);
        protocol.getSchedule().setScheduleTime(scheduleTime);

        String chron = ProtocolChronGenerator.getCronSchedule(protocol.getSchedule());
//        sec min hour dayofmon month dow  year(optional)
        Assert.assertEquals("0 15 10 ? * SAT", chron);
    }

    @Test
    public void canCreateChronTriggerforTypeDaysOfWeek(){
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        Vital protocol = new Vital();


        TypeDaysOfWeek daysOfWeek = new TypeDaysOfWeek();
        daysOfWeek.setName("DaysOfWeek");
        daysOfWeek.setSaturday(true);
        daysOfWeek.setMonday(true);
        Schedule schedule = new Schedule();
        schedule.setScheduleType(daysOfWeek);


        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);
        schedule.setScheduleTime(scheduleTime);
        protocol.setSchedule(schedule);


        String cronSchedStr = ProtocolChronGenerator.getCronSchedule(protocol.getSchedule());

        String timeZone="America/Chicago";
		CronTrigger cronTrigger = newTrigger().withIdentity(triggerName, triggerGroup).withSchedule(cronSchedule(cronSchedStr).inTimeZone(TimeZone.getTimeZone(timeZone)))
                .build();
//        sec min hour dayofmon month dow  year(optional)
        Assert.assertEquals("0 15 10 ? * MON,SAT",cronTrigger.getCronExpression());
        Assert.assertEquals("Central Standard Time",cronTrigger.getTimeZone().getDisplayName());

    }

    @Test
    public void canCreateRuntime(){

        Vital template = new Vital();
        ScheduleType once = new TypeOnce();
        once.setName("Once");
        Schedule schedule = new Schedule();
        template.setSchedule(schedule);
        template.getSchedule().setScheduleType(once);

        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);
        template.getSchedule().setScheduleTime(scheduleTime);
        template.setStartDate("2013-11-12");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);
        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template) ;

       // String chron = ProtocolChronGenerator.getCronSchedule();
//        sec min hour dayofmon month dow  year(optional)
       Assert.assertEquals("2013-11-12 10:15:00", runTimeStr);
    }

    @Test
    public void canSimpleTrigger(){
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        Vital template = new Vital();
        String timeZone="US/Pacific";
        template.setStartDate("2014-01-22");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);

        Schedule schedule = new Schedule();
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);

        schedule.setScheduleTime(scheduleTime);


        TypeOnce typeOnce = new TypeOnce();
        schedule.setScheduleType(typeOnce);
        template.setSchedule(schedule);
        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template);
        
		Date sDate = DateParser.parseDateWithTimeZone(runTimeStr,timeZone);

        Assert.assertEquals("2014-01-22 10:15:00", runTimeStr);
        
        Assert.assertEquals("Wed Jan 22 13:15:00 EST 2014", sDate.toString());
        
        SimpleTrigger simpleTrigger =  (SimpleTrigger) newTrigger().withIdentity(triggerName, triggerGroup).startAt(sDate).build();

        Assert.assertEquals(sDate,simpleTrigger.getStartTime());

    }


    @Test
    public void canCalendarIntervalTriggerByDay() {
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        Vital template = new Vital();
        String timeZone="America/New_York";
        template.setStartDate("2014-01-22");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);

        Schedule schedule = new Schedule();
        TypeEverySoMany typeEverySoMany = new TypeEverySoMany();
        TimeInterval timeInterval = new TimeInterval();
        timeInterval.setUnit(TimeIntervalUnit.DAY);
        timeInterval.setValue(2);
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);

        // Schedule schedule = new Schedule();
        schedule.setScheduleTime(scheduleTime);
        typeEverySoMany.setInterval(timeInterval);
        schedule.setScheduleType(typeEverySoMany);
        template.setSchedule(schedule);


        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template);
        Date sDate = DateParser.parseDateWithTimeZone(runTimeStr,timeZone);
        Assert.assertEquals("Wed Jan 22 10:15:00 EST 2014", sDate.toString());


        CalendarIntervalTrigger calendarTrigger = newTrigger()
                .withIdentity(triggerName, triggerGroup)
                .startAt(sDate) // 15:00:00 tomorrow
                .withSchedule(calendarIntervalSchedule()
                .inTimeZone(TimeZone.getTimeZone(timeZone))
                .withInterval(timeInterval.getValue(), DateBuilder.IntervalUnit.DAY)) // interval is set in calendar days
                .build();


        Assert.assertEquals(2, calendarTrigger.getRepeatInterval());
        Assert.assertEquals(DateBuilder.IntervalUnit.DAY, calendarTrigger.getRepeatIntervalUnit());
        
        Assert.assertEquals("Wed Jan 22 10:15:00 EST 2014",calendarTrigger.getStartTime().toString());
    }

    @Test
    public void canCalendarIntervalTriggerByWeek() {
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        Vital template = new Vital();
        String timeZone="America/Los_Angeles";
        template.setStartDate("2014-01-22");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);

        Schedule schedule = new Schedule();
        TypeEverySoMany typeEverySoMany = new TypeEverySoMany();
        TimeInterval timeInterval = new TimeInterval();
        timeInterval.setUnit(TimeIntervalUnit.WEEK);
        timeInterval.setValue(2);
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);

        // Schedule schedule = new Schedule();
        schedule.setScheduleTime(scheduleTime);
        typeEverySoMany.setInterval(timeInterval);
        schedule.setScheduleType(typeEverySoMany);
        template.setSchedule(schedule);


        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template);
        Date sDate = DateParser.parseDateWithTimeZone(runTimeStr,timeZone);


        CalendarIntervalTrigger calendarTrigger = newTrigger()
                .withIdentity(triggerName, triggerGroup)
                .startAt(sDate) // 15:00:00 tomorrow
                .withSchedule(calendarIntervalSchedule()
                .inTimeZone(TimeZone.getTimeZone(timeZone))
                .withInterval(timeInterval.getValue(), DateBuilder.IntervalUnit.WEEK)) // interval is set in calendar days
                .build();


        Assert.assertEquals(2, calendarTrigger.getRepeatInterval());
        Assert.assertEquals(DateBuilder.IntervalUnit.WEEK, calendarTrigger.getRepeatIntervalUnit());
        Assert.assertEquals("Wed Jan 22 13:15:00 EST 2014", sDate.toString());

    }

    @Test
    public void canCalendarIntervalTriggerByMonth() {
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        Vital template = new Vital();
        String timeZone="America/Denver";
        template.setStartDate("2014-01-22");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);

        Schedule schedule = new Schedule();
        TypeEverySoMany typeEverySoMany = new TypeEverySoMany();
        TimeInterval timeInterval = new TimeInterval();
        timeInterval.setUnit(TimeIntervalUnit.MONTH);
        timeInterval.setValue(2);
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);

        // Schedule schedule = new Schedule();
        schedule.setScheduleTime(scheduleTime);
        typeEverySoMany.setInterval(timeInterval);
        schedule.setScheduleType(typeEverySoMany);
        template.setSchedule(schedule);


        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template);
        Date sDate = DateParser.parseDateWithTimeZone(runTimeStr,timeZone);


        CalendarIntervalTrigger calendarTrigger = newTrigger()
                .withIdentity(triggerName, triggerGroup)
                .startAt(sDate) // 15:00:00 tomorrow
                .withSchedule(calendarIntervalSchedule()
                .inTimeZone(TimeZone.getTimeZone(timeZone))
                .withInterval(timeInterval.getValue(), DateBuilder.IntervalUnit.MONTH)) // interval is set in calendar days
                .build();


        Assert.assertEquals(2, calendarTrigger.getRepeatInterval());
        Assert.assertEquals(DateBuilder.IntervalUnit.MONTH, calendarTrigger.getRepeatIntervalUnit());
        Assert.assertEquals("Wed Jan 22 12:15:00 EST 2014", sDate.toString());

    }
    @Test
    public void canCalendarIntervalTriggerByYear() {
        String triggerName = "initProtocol";
        String triggerGroup = "initProtocol";
        String jobGroup = "initProtocol";
        String cronScheduleStr = "";
        String timeZone="America/Chicago";

        Vital template = new Vital();
        template.setStartDate("2014-01-22");

        List<Template> templates = new ArrayList<Template>();
        templates.add(template);
        Protocol protocol = new Protocol();
        protocol.setTemplates(templates);
        protocol.setName("Blood Pressure");

        PatientProtocol patientProtocol = new PatientProtocol("patien01", protocol);
        String jobName = patientProtocol.getId();

        Schedule schedule = new Schedule();
        TypeEverySoMany typeEverySoMany = new TypeEverySoMany();
        TimeInterval timeInterval = new TimeInterval();
        timeInterval.setUnit(TimeIntervalUnit.YEAR);
        timeInterval.setValue(2);
        ScheduleTime scheduleTime = new ScheduleTime();
        scheduleTime.setHour(10);
        scheduleTime.setMeridian(Meridian.AM);
        scheduleTime.setMin(15);

        // Schedule schedule = new Schedule();
        schedule.setScheduleTime(scheduleTime);
        typeEverySoMany.setInterval(timeInterval);
        schedule.setScheduleType(typeEverySoMany);
        template.setSchedule(schedule);


        patientProtocol.setProtocol(protocol);
        String runTimeStr = ProtocolChronGenerator.getStartDateTime(template);
        Date sDate = DateParser.parseDateWithTimeZone(runTimeStr,timeZone);

        CalendarIntervalTrigger calendarTrigger = newTrigger()
                .withIdentity(triggerName, triggerGroup)
                .startAt(sDate) // 15:00:00 tomorrow
                .withSchedule(calendarIntervalSchedule()
                .inTimeZone(TimeZone.getTimeZone(timeZone))
                .withInterval(timeInterval.getValue(), DateBuilder.IntervalUnit.YEAR)) // interval is set in calendar days
                .build();


        Assert.assertEquals(2, calendarTrigger.getRepeatInterval());
        Assert.assertEquals(DateBuilder.IntervalUnit.YEAR, calendarTrigger.getRepeatIntervalUnit());
        Assert.assertEquals("Wed Jan 22 11:15:00 EST 2014", sDate.toString());
        //PatientProtocolScheduler.scheduleJob(job, 1, jobName, jobGroup, cronScheduleStr,scheduleTime,timeInterval);

    }



//    @Test
//    public void testConvertHour() throws Exception {
//
//    }
//
//    @Test
//    public void testGetScheduleDayOfWeek() throws Exception {
//
//    }
}
