package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.scheduler.QuartzJob;

import org.joda.time.LocalDateTime;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.quartz.*;
import org.quartz.impl.JobDetailImpl;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.triggers.SimpleTriggerImpl;

import static org.fest.assertions.api.Assertions.assertThat;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class QuartzJobHandlerTest {
    private static Scheduler scheduler;

    @Before
    public void setUp() throws Exception {
        scheduler = new StdSchedulerFactory("quartz_test.properties").getScheduler();
        //scheduler.start();
    }

    @After
    public void tearDown() throws Exception {
        scheduler.shutdown(false);
    }

    @Test
    public void testList() throws Exception {
        String expectedJobName = "runMeJobName";
        String expectedJobGroup = "runMeJobGroup";
        Class<? extends Job> expectedJobClass = RunMeJob.class;

        //specify your sceduler task details
        JobDetailImpl job = new JobDetailImpl();
        job.setName(expectedJobName);
        job.setGroup(expectedJobGroup);
        job.setJobClass(expectedJobClass);

        //configure the scheduler time
        SimpleTriggerImpl trigger = new SimpleTriggerImpl();
        trigger.setName("runMeJobTesting");
        trigger.setStartTime(new Date(System.currentTimeMillis() + 1000));
        trigger.setRepeatCount(SimpleTrigger.REPEAT_INDEFINITELY);
        trigger.setRepeatInterval(30000);

        scheduler.scheduleJob(job, trigger);

        QuartzJobHandler quartzJobHandler = new QuartzJobHandler(scheduler);

        List<QuartzJob> quartzJobs = quartzJobHandler.list();

        assertThat(quartzJobs.size()).isEqualTo(1);

        QuartzJob quartzJob = quartzJobs.get(0);
        assertThat(quartzJob.getGroup()).isEqualTo(expectedJobGroup);
        assertThat(quartzJob.getName()).isEqualTo(expectedJobName);
        assertThat(quartzJob.getJobClass()).isEqualTo(expectedJobClass);

        Map<String, Object> additionalProperties = quartzJob.getAdditionalProperties();

        System.out.println(additionalProperties.get("jobTriggerName").toString());

        assertThat(additionalProperties.get("jobTriggerName")).isEqualTo("runMeJobTesting");
        assertThat(additionalProperties.get("jobTriggerStartTime")).isEqualTo(QuartzJobHandler.formatDate(trigger.getStartTime()));
        assertThat(additionalProperties.get("jobTriggerEndTime")).isEqualTo(QuartzJobHandler.formatDate(trigger.getEndTime()));
        assertThat(additionalProperties.get("jobTriggerPreviousFireTime")).isEqualTo(QuartzJobHandler.formatDate(trigger.getPreviousFireTime()));
        assertThat(additionalProperties.get("jobTriggerNextFireTime")).isEqualTo(QuartzJobHandler.formatDate(trigger.getNextFireTime()));
        assertThat(additionalProperties.get("jobTriggerPriority")).isEqualTo(trigger.getPriority());
        assertThat(additionalProperties.get("jobTriggerScheduleBuilderClass")).isEqualTo(trigger.getScheduleBuilder().getClass());
    }

    class RunMeJob implements Job {
        @Override
        public void execute(JobExecutionContext context) throws JobExecutionException {

        }
    }
}
