package gov.va.mobilehealth.courier.utils;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created with IntelliJ IDEA.
 * User: clarkee
 * Date: 12/4/13
 * Time: 12:58 PM
 * To change this template use File | Settings | File Templates.
 */
public class DateParserTest {
	
    @Test
    public void canParse() {
        String dateString = "2011-11-16 10:26:13";
        DateTime expectedDateTime = new DateTime(2011, 11, 16, 10, 26, 13);

        Assert.assertEquals(expectedDateTime.toDate(), DateParser.parseAttDate(dateString));
    }
    

    @Test
    public void testGetEndofDate() {
        
        DateTime expectedDateTime1 = new DateTime(2014, 11, 16, 10, 26, 13);
        
        DateTime expectedDateTime2 = new DateTime(2014, 11, 16, 23, 59, 59);
        
        Date endOfDate = DateParser.getEndofDate(expectedDateTime1.toDate(),"US/Eastern");
        
        Assert.assertEquals(expectedDateTime2.toDate(), endOfDate);
        
    }
    
    @Test
    public void testparseDateWithTimeZone() {
        
        String date = "2010-05-23 09:01:02";
        String timeZone = "US/Pacific";
        
        String expectedDate = "Sun May 23 12:01:02 EDT 2010";
        
        //TimeZone.setDefault(TimeZone.getTimeZone("PST"));
        
        Date outputDate = DateParser.parseDateWithTimeZone(date, timeZone);
        
        Assert.assertEquals(expectedDate, outputDate.toString());
        
    }
    
    @Test
    public void testParseDate1() {
        
        String inputDate = "May 05, 1900";
        
        String expectedDate = "1900-05-05";

        String outputDate = DateParser.convertDateFormat(inputDate, "MMM dd, yyyy");
        
        Assert.assertEquals(expectedDate, outputDate.toString());
        
    }
    
    @Test
    public void testParseDate2() {
        
        String inputDate = "05/05/1900";
        
        String expectedDate = "1900-05-05";

        String outputDate = DateParser.convertDateFormat(inputDate, "MM/dd/yyyy");
        
        Assert.assertEquals(expectedDate, outputDate.toString());
        
    }
    
    @Test
    public void testTimezoneChange() {
        
        String inputDate = "2010-01-23 09:01:02";
        String fromTimeZone = "America/New_York";
        String toTimeZone = "America/Chicago";
        String outputDate = DateParser.convertTimeZone(inputDate, fromTimeZone, toTimeZone, null);
        Assert.assertEquals("2010-01-23 08:01:02", outputDate);
        
        inputDate = "2010-01-23 09:01:02";
        fromTimeZone = "America/New_York";
        toTimeZone = "UTC";
        outputDate = DateParser.convertTimeZone(inputDate, fromTimeZone, toTimeZone, null);
        Assert.assertEquals("2010-01-23 14:01:02", outputDate);
        
        inputDate = "2010-06-23 09:01:02";
        fromTimeZone = "UTC";
        toTimeZone = "America/New_York";
        outputDate = DateParser.convertTimeZone(inputDate, fromTimeZone, toTimeZone, null);
        Assert.assertEquals("2010-06-23 05:01:02", outputDate);
    }
}
