include_recipe "frontend-cms::frontend"

node.include_attribute('apache2::default')

node.set['frontend-cms']['fqdn'] = 'www.cms.dev'
node.set['apache']['listen_ports'] = [443]

directory "/etc/httpd/ssl" do
  action :create
  recursive true
end

execute "create-private-key" do
  command "openssl genrsa 1024 > #{node['apache']['ssl_cert_key_file']}"
  not_if "test -f #{node['apache']['ssl_cert_key_file']}"
end

execute "create-certficate" do
  command %Q{openssl req -new -x509 -newkey rsa:1024 -key #{node['apache']['ssl_cert_key_file']} -out #{node['apache']['ssl_cert_file']} -days 365 <<EOF
US
VA
Chantilly
Agilex
Healthcare
#{node['frontend-cms']['fqdn']}
jay.flowers@agilex.com
EOF}
  not_if "test -f #{node['apache']['ssl_cert_file']}"
end

include_recipe "apache2"
include_recipe "apache2::mod_cgi"
include_recipe "apache2::mod_ssl"
include_recipe "apache2::mod_proxy"
include_recipe "apache2::mod_proxy_http"
include_recipe "apache2::mod_proxy_ajp"
include_recipe "apache2::mod_headers"


def get_filename_for_url(url)
  filename = File.basename(url)
  if url.include?('service/local/artifact/maven/content')
    resp = HTTParty.head(url).response
    if resp.header['Content-Disposition'] =~ /.*filename="(.*).*"/
      filename = $1.gsub("\\", "") 
    else
      raise "Could not obtain filename of latest artifact from url #{url}"
    end 
  elsif url.include?('service/local/artifact/maven/redirect')
    begin
      resp = HTTParty.head(url, { :no_follow => true})
    rescue HTTParty::RedirectionTooDeep => e
      filename = File.basename(e.response.header['Location'])
    end 
  end 
  return filename
end 

node['frontend-cms']['frontend'].each do |f|
  f.default[:filename] = get_filename_for_url(f[:source])
end

web_app "frontend_config" do
  template "frontend.conf.erb"
  frontend_apps node['frontend-cms']['frontend']
  backend_apps node['frontend-cms']['backend']
  document_root node['apache']['root_dir']
  ssl_cert_file node['apache']['ssl_cert_file']
  ssl_cert_key_file node['apache']['ssl_cert_key_file']
  ssl_cert_chain_file node['apache']['ssl_cert_chain_file'] if node['apache'].has_key?('ssl_cert_chain_file')
end
