# Cookbook Name:: apache_mbb_homes
# Recipe:: frontend
#
# Copyright 2013, Agilex
#
# All rights reserved - Do Not Redistribute
#

chef_gem "httparty" do
  version '0.11.0'
end
require 'rubygems'
gem "httparty"
require 'httparty'

def fail_on_duplicate_filenames
  filenames = node["frontend-cms"]["frontend"].collect{ |spec| File.basename(spec['source']) }.sort

  last_filename = nil
  filenames.each do |filename|
    if filename == last_filename
      raise RuntimeError.new("Found duplicate frontend webapps in node['frontend-cms']['frontend']! #{filename}")
    end
    
    last_filename = filename
  end
end

def get_filename_for_url(url)
  filename = File.basename(url)
  if url.include?('service/local/artifact/maven/content')
    resp = HTTParty.head(url).response
    if resp.header['Content-Disposition'] =~ /.*filename="(.*).*"/
      filename = $1.gsub("\\", "") 
    else
      raise "Could not obtain filename of latest artifact from url #{url}"
    end 
  elsif url.include?('service/local/artifact/maven/redirect')
    begin
      resp = HTTParty.head(url, { :no_follow => true})
    rescue HTTParty::RedirectionTooDeep => e
      filename = File.basename(e.response.header['Location'])
    end 
  end 
  return filename
end 

fail_on_duplicate_filenames

node["frontend-cms"]["frontend"].each do |spec|
  filename = get_filename_for_url(spec['source'])
  cached_file_path = "#{Chef::Config[:file_cache_path]}/#{filename}"
  extraction_path = "/srv/webapps/#{filename.gsub('.zip', '')}"

  if spec["source"].start_with?("http")
    remote_file cached_file_path do
      source spec["source"]
      checksum open("#{spec['source']}.sha1", {ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE}).string
    end
  else
    file cached_file_path do
      content IO.read(spec['source'])
    end
  end

  directory extraction_path do
    action :create
    recursive true
  end
  
  bash "Unzip archive #{cached_file_path}" do
    code <<-EOF
      cd #{extraction_path}

      rm -rf *
      rm -rf .*

      /usr/bin/unzip #{cached_file_path}
    EOF

    action :run
  end
end
