require "chefspec"
require "tempfile"

describe "frontend-cms::frontend" do
  describe "default attributes" do 
    let(:chef_run){ ChefSpec::ChefRunner.new(platform: "centos", version: "6.3") }

    it "should set node['frontend-cms']['frontend'] to an empty array" do
      chef_run.converge("frontend-cms::frontend");
  
      expect(chef_run.node['frontend-cms']['frontend']).to be_empty
    end
  end

  describe "Obtaining zip file" do
    let(:chef_run){ ChefSpec::ChefRunner.new(platform: "centos", version: "6.3") }
  
    before :each do
      Chef::Config[:file_cache_path] = '/var/cache/chef'
  
      chef_run.node.set["frontend-cms"]["frontend"] = [ {
          "context" => "/claims",
          "source"  => "https://nexus.agilex-dev.com/path/to/file.zip"
        }, {
          "context" => "/jobs",
          "source"  => "/path/to/file2.zip"
        }
      ]
  
      method = IO.method(:read)
      IO.stub(:read).and_return do |arg|
        method.call(arg)
      end
      IO.stub(:read).with("/path/to/file2.zip").and_return("Some content")
    end
  
    it "should download urls to the chef file cache" do
      chef_run.converge("frontend-cms::frontend");
  
      expect(chef_run.resources.collect{ |r| r.to_s}).to include("remote_file[/var/cache/chef/file.zip]")
    end
  
    it "should copy files to the chef file cache if a local path is provided" do
      chef_run.converge("frontend-cms::frontend");
  
      expect(chef_run.resources.collect{ |r| r.to_s}).to include("file[/var/cache/chef/file2.zip]")
    end
  
    it "should fail if there are duplicate basenames" do
      chef_run.node["frontend-cms"]["frontend"].push({
        "context" => "/claims2",
        "source"  => "https://nexus.agilex-dev.com/another/path/to/file.zip"
      })
      expect{chef_run.converge("frontend-cms::frontend")}.to raise_error
    end
  end

  describe "Unzipping UI assets archive" do
    let(:chef_run){ ChefSpec::ChefRunner.new(platform: "centos", version: "6.3") }
  
    before :each do
      Chef::Config[:file_cache_path] = '/var/cache/chef'
  
      chef_run.node.set["frontend-cms"]["frontend"] = [ {
          "context" => "/claims",
          "source"  => "https://nexus.agilex-dev.com/path/to/claims-1.0.0.249.zip"
        }
      ]
    end
  
    it "should create a directory for the webapp" do
      chef_run.converge("frontend-cms::frontend");
  
      expect(chef_run).to create_directory("/srv/webapps/claims-1.0.0.249")
    end

    it "should unzip the contents of the zip file" do
      chef_run.converge("frontend-cms::frontend");
  
      expect(chef_run.resources.collect{ |r| r.to_s}).to include("bash[Unzip archive /var/cache/chef/claims-1.0.0.249.zip]")
    end
  end
end
