#
# Cookbook Name:: weblogic
# Recipe:: zip_distro
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

include_recipe node['weblogic']['java_flavor']

Chef::Log.warn("No download url set for weblogic installer.") unless node['weblogic']['installer_url']

mw_home = node['weblogic']['mw_home']
wl_home = node['weblogic']['wl_home']

case node['platform_family']
  when "centos", "rhel", "debian", "ubuntu"

    bash 'start-development-domain' do
      cwd wl_home
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code 'nohup $MW_HOME/server1/startWebLogic.sh < /dev/null &> /dev/null &'
      only_if { ::File.exists?("#{wl_home}/config/config.xml") }
    end

    ruby_block "block_until_WebLogic is operational" do
      block do
        1.upto(100).each { |c|
          Chef::Log.info('Waiting for WebLogic to start:' + c.to_s)

          if `curl -sL -w "%{http_code}" http://localhost:7001/console -o dev/null | grep '200'` =~ /200/
            break
          else
            sleep 5
          end
        }
      end
    end

    bash 'startNodeManager' do
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code <<-EOH
    nohup $MW_HOME/server1/bin/startNodeManager.sh< /dev/null &> /dev/null &
      EOH
    end
end
