#
# Cookbook Name:: weblogic
# Recipe:: zip_distro
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

include_recipe node['weblogic']['java_flavor']

Chef::Log.warn("No download url set for weblogic installer.") unless node['weblogic']['installer_url']

src_filename = "wls.zip"
src_filepath = "#{Chef::Config['file_cache_path']}/#{src_filename}"
extract_path = "#{Chef::Config['file_cache_path']}/weblogic_package"
mw_home = node['weblogic']['mw_home']
wl_home = node['weblogic']['wl_home']

case node['platform_family']
  when "centos", "rhel", "debian", "ubuntu"

    bash 'import-vamf-certs' do
      code <<-EOH
      keytool -importcert -storepass changeit -trustcacerts -noprompt \
              -file /cms-project/infrastructure2/vagrant/vamf-provisionless/vamf-weblogic-httpd-certificate.pem \
              -alias haweblogichttp -keystore $JAVA_HOME/jre/lib/security/cacerts || true

      keytool -importcert -storepass changeit -trustcacerts -noprompt \
              -file /cms-project/infrastructure2/vagrant/vamf-provisionless/vamf-weblogic-certificate.pem \
              -alias haweblogicwl -keystore $JAVA_HOME/jre/lib/security/cacerts || true
      EOH
    end

    package "unzip" do
      :install
    end

    directory mw_home do
      recursive true
      owner node['weblogic']['owner']
      group node['weblogic']['group']
      action :create
    end

    remote_file src_filepath do
      source node['weblogic']['installer_url']
      owner node['weblogic']['owner']
      group node['weblogic']['owner']
      mode 00644
    end

    bash 'extract_weblogic' do
      cwd ::File.dirname(src_filepath)
      code <<-EOH
    unzip #{src_filename} -d #{extract_path}
    mv #{extract_path}/*/* #{mw_home}
    chown -R #{node['weblogic']['owner']}:#{node['weblogic']['group']} #{mw_home}
      EOH
      not_if { ::File.exists?(extract_path) }
    end

    bash 'fix_permission' do
      code "chown -R #{node['weblogic']['owner']}:#{node['weblogic']['group']} #{mw_home}"
    end

    include_recipe 'weblogic::set_mw_home'

    bash 'run-weblogic-config' do
      cwd mw_home
      code <<-EOH
    $MW_HOME/configure.sh -silent
      EOH
      creates "#{mw_home}/wlserver/.product.properties"
    end

    directory wl_home do
      recursive true
      owner node['weblogic']['owner']
      group node['weblogic']['group']
      action :create
    end

    bash 'create-development-domain' do
      cwd wl_home
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code <<-EOH
. $MW_HOME/wlserver/server/bin/setWLSEnv.sh
$JAVA_HOME/bin/java $JAVA_OPTIONS -Xmx512m -XX:MaxPermSize=256m -Djava.security.egd=file:/dev/./urandom -Dweblogic.management.allowPasswordEcho=true -Dweblogic.management.GenerateDefaultConfig=true -Dweblogic.management.username=weblogic -Dweblogic.management.password=weblogic1 weblogic.Server > $MW_HOME/weblogic_install.log 2>&1 &
      EOH
      not_if { ::File.exists?("#{wl_home}/config/config.xml") }
    end

    bash 'start-development-domain' do
      cwd wl_home
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code 'nohup $MW_HOME/server1/startWebLogic.sh < /dev/null &> /dev/null &'
      only_if { ::File.exists?("#{wl_home}/config/config.xml") }
    end

    ruby_block "block_until_WebLogic is operational" do
      block do
        1.upto(100).each { |c|
          Chef::Log.info('Waiting for WebLogic to start:' + c.to_s)

          if `curl -sL -w "%{http_code}" http://localhost:7001/console -o dev/null | grep '200'` =~ /200/
            break
          else
            sleep 20
          end
        }
      end
    end

    bash 'startNodeManager' do
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code <<-EOH
    nohup $MW_HOME/server1/bin/startNodeManager.sh< /dev/null &> /dev/null &
      EOH
    end

    bash 'create-JMS-Resources' do
      cwd wl_home
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code '. $MW_HOME/wlserver/server/bin/setWLSEnv.sh && $JAVA_HOME/bin/java weblogic.WLST /vagrant/wlst_annie.py'
    end

    bash 'create-datasource' do
      cwd wl_home
      user node['weblogic']['owner']
      group node['weblogic']['group']
      code '. $MW_HOME/wlserver/server/bin/setWLSEnv.sh && $JAVA_HOME/bin/java weblogic.WLST /vagrant/wlst_create_datasource.py'
    end
end
