REM
REM The script will create tablespace, schema, and table that are required by the Courier project.
REM The script is expected to be run by a Database Admin sysdba account.
REM The script expects two input values:
REM the first value is the password for QUARTZ user
REM and the seond value is the password for QUARTZ_USER user.
REM Example: "sqlplus / as sysdba @courier_dbinstall_r1.sql quartzPwd quartzUserPwd"
REM

set pagesize 0
set linesize 500
set trimspool on
spool courier_dbinstall_r1.log

WHENEVER SQLERROR EXIT SQL.SQLCODE
-- create tablespace
CREATE TABLESPACE QUARTZ_DATA
  DATAFILE '/u01/apps/oracle/oradata/quartz_data01.dbf' SIZE 1024M reuse autoextend on next 1024M;
CREATE TABLESPACE QUARTZ_IDX
  DATAFILE '/u01/apps/oracle/oradata/quartz_idx01.dbf' SIZE 1024M reuse autoextend on next 1024M;

-- create user
CREATE USER QUARTZ IDENTIFIED BY &1
 DEFAULT TABLESPACE QUARTZ_DATA
 QUOTA unlimited ON QUARTZ_DATA
 QUOTA unlimited ON QUARTZ_IDX;

grant create table to QUARTZ;

CREATE USER QUARTZ_USER IDENTIFIED BY &2
 DEFAULT TABLESPACE QUARTZ_DATA
 QUOTA unlimited ON QUARTZ_DATA
 QUOTA unlimited ON QUARTZ_IDX;

grant create session to QUARTZ_USER;
CREATE ROLE quartz_app_role;
grant quartz_app_role to QUARTZ_USER;

CREATE TABLE QUARTZ.qrtz_job_details
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    JOB_NAME  VARCHAR2(200) NOT NULL,
    JOB_GROUP VARCHAR2(200) NOT NULL,
    DESCRIPTION VARCHAR2(250) NULL,
    JOB_CLASS_NAME   VARCHAR2(250) NOT NULL,
    IS_DURABLE VARCHAR2(1) NOT NULL,
    IS_NONCONCURRENT VARCHAR2(1) NOT NULL,
    IS_UPDATE_DATA VARCHAR2(1) NOT NULL,
    REQUESTS_RECOVERY VARCHAR2(1) NOT NULL,
    JOB_DATA BLOB NULL,
    CONSTRAINT QRTZ_JOB_DETAILS_PK PRIMARY KEY (SCHED_NAME,JOB_NAME,JOB_GROUP)
)TABLESPACE QUARTZ_DATA ;

CREATE TABLE QUARTZ.qrtz_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    JOB_NAME  VARCHAR2(200) NOT NULL,
    JOB_GROUP VARCHAR2(200) NOT NULL,
    DESCRIPTION VARCHAR2(250) NULL,
    NEXT_FIRE_TIME NUMBER(13) NULL,
    PREV_FIRE_TIME NUMBER(13) NULL,
    PRIORITY NUMBER(13) NULL,
    TRIGGER_STATE VARCHAR2(16) NOT NULL,
    TRIGGER_TYPE VARCHAR2(8) NOT NULL,
    START_TIME NUMBER(13) NOT NULL,
    END_TIME NUMBER(13) NULL,
    CALENDAR_NAME VARCHAR2(200) NULL,
    MISFIRE_INSTR NUMBER(2) NULL,
    JOB_DATA BLOB NULL,
    CONSTRAINT QRTZ_TRIGGERS_PK PRIMARY KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP),
    CONSTRAINT QRTZ_TRIGGER_TO_JOBS_FK FOREIGN KEY (SCHED_NAME,JOB_NAME,JOB_GROUP)
      REFERENCES QUARTZ.QRTZ_JOB_DETAILS(SCHED_NAME,JOB_NAME,JOB_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_simple_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    REPEAT_COUNT NUMBER(7) NOT NULL,
    REPEAT_INTERVAL NUMBER(12) NOT NULL,
    TIMES_TRIGGERED NUMBER(10) NOT NULL,
    CONSTRAINT QRTZ_SIMPLE_TRIG_PK PRIMARY KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP),
    CONSTRAINT QRTZ_SIMPLE_TRIG_TO_TRIG_FK FOREIGN KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
	REFERENCES QUARTZ.QRTZ_TRIGGERS(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_cron_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    CRON_EXPRESSION VARCHAR2(120) NOT NULL,
    TIME_ZONE_ID VARCHAR2(80),
    CONSTRAINT QRTZ_CRON_TRIG_PK PRIMARY KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP),
    CONSTRAINT QRTZ_CRON_TRIG_TO_TRIG_FK FOREIGN KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
      REFERENCES QUARTZ.QRTZ_TRIGGERS(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_simprop_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    STR_PROP_1 VARCHAR2(512) NULL,
    STR_PROP_2 VARCHAR2(512) NULL,
    STR_PROP_3 VARCHAR2(512) NULL,
    INT_PROP_1 NUMBER(10) NULL,
    INT_PROP_2 NUMBER(10) NULL,
    LONG_PROP_1 NUMBER(13) NULL,
    LONG_PROP_2 NUMBER(13) NULL,
    DEC_PROP_1 NUMERIC(13,4) NULL,
    DEC_PROP_2 NUMERIC(13,4) NULL,
    BOOL_PROP_1 VARCHAR2(1) NULL,
    BOOL_PROP_2 VARCHAR2(1) NULL,
    CONSTRAINT QRTZ_SIMPROP_TRIG_PK PRIMARY KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP),
    CONSTRAINT QRTZ_SIMPROP_TRIG_TO_TRIG_FK FOREIGN KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
      REFERENCES QUARTZ.QRTZ_TRIGGERS(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_blob_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    BLOB_DATA BLOB NULL,
    CONSTRAINT QRTZ_BLOB_TRIG_PK PRIMARY KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP),
    CONSTRAINT QRTZ_BLOB_TRIG_TO_TRIG_FK FOREIGN KEY (SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
        REFERENCES QUARTZ.QRTZ_TRIGGERS(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_calendars
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    CALENDAR_NAME  VARCHAR2(200) NOT NULL,
    CALENDAR BLOB NOT NULL,
    CONSTRAINT QRTZ_CALENDARS_PK PRIMARY KEY (SCHED_NAME,CALENDAR_NAME)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_paused_trigger_grps
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    TRIGGER_GROUP  VARCHAR2(200) NOT NULL,
    CONSTRAINT QRTZ_PAUSED_TRIG_GRPS_PK PRIMARY KEY (SCHED_NAME,TRIGGER_GROUP)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_fired_triggers
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    ENTRY_ID VARCHAR2(95) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    INSTANCE_NAME VARCHAR2(200) NOT NULL,
    FIRED_TIME NUMBER(13) NOT NULL,
    SCHED_TIME NUMBER(13) NOT NULL,
    PRIORITY NUMBER(13) NOT NULL,
    STATE VARCHAR2(16) NOT NULL,
    JOB_NAME VARCHAR2(200) NULL,
    JOB_GROUP VARCHAR2(200) NULL,
    IS_NONCONCURRENT VARCHAR2(1) NULL,
    REQUESTS_RECOVERY VARCHAR2(1) NULL,
    CONSTRAINT QRTZ_FIRED_TRIGGER_PK PRIMARY KEY (SCHED_NAME,ENTRY_ID)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_scheduler_state
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    INSTANCE_NAME VARCHAR2(200) NOT NULL,
    LAST_CHECKIN_TIME NUMBER(13) NOT NULL,
    CHECKIN_INTERVAL NUMBER(13) NOT NULL,
    CONSTRAINT QRTZ_SCHEDULER_STATE_PK PRIMARY KEY (SCHED_NAME,INSTANCE_NAME)
)TABLESPACE QUARTZ_DATA;

CREATE TABLE QUARTZ.qrtz_locks
  (
    SCHED_NAME VARCHAR2(120) NOT NULL,
    LOCK_NAME  VARCHAR2(40) NOT NULL,
    CONSTRAINT QRTZ_LOCKS_PK PRIMARY KEY (SCHED_NAME,LOCK_NAME)
)TABLESPACE QUARTZ_DATA;

create index QUARTZ.idx_qrtz_j_req_recovery on QUARTZ.qrtz_job_details(SCHED_NAME,REQUESTS_RECOVERY) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_j_grp on QUARTZ.qrtz_job_details(SCHED_NAME,JOB_GROUP)  TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_j on QUARTZ.qrtz_triggers(SCHED_NAME,JOB_NAME,JOB_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_jg on QUARTZ.qrtz_triggers(SCHED_NAME,JOB_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_c on QUARTZ.qrtz_triggers(SCHED_NAME,CALENDAR_NAME) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_g on QUARTZ.qrtz_triggers(SCHED_NAME,TRIGGER_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_state on QUARTZ.qrtz_triggers(SCHED_NAME,TRIGGER_STATE) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_n_state on QUARTZ.qrtz_triggers(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP,TRIGGER_STATE) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_n_g_state on QUARTZ.qrtz_triggers(SCHED_NAME,TRIGGER_GROUP,TRIGGER_STATE) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_next_fire_time on QUARTZ.qrtz_triggers(SCHED_NAME,NEXT_FIRE_TIME) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_nft_st on QUARTZ.qrtz_triggers(SCHED_NAME,TRIGGER_STATE,NEXT_FIRE_TIME) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_nft_misfire on QUARTZ.qrtz_triggers(SCHED_NAME,MISFIRE_INSTR,NEXT_FIRE_TIME) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_nft_st_misfire on QUARTZ.qrtz_triggers(SCHED_NAME,MISFIRE_INSTR,NEXT_FIRE_TIME,TRIGGER_STATE) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_t_nft_st_misfire_grp on QUARTZ.qrtz_triggers(SCHED_NAME,MISFIRE_INSTR,NEXT_FIRE_TIME,TRIGGER_GROUP,TRIGGER_STATE) TABLESPACE QUARTZ_IDX;

create index QUARTZ.idx_qrtz_ft_trig_inst_name on QUARTZ.qrtz_fired_triggers(SCHED_NAME,INSTANCE_NAME) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_ft_inst_job_req_rcvry on QUARTZ.qrtz_fired_triggers(SCHED_NAME,INSTANCE_NAME,REQUESTS_RECOVERY) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_ft_j_g on QUARTZ.qrtz_fired_triggers(SCHED_NAME,JOB_NAME,JOB_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_ft_jg on QUARTZ.qrtz_fired_triggers(SCHED_NAME,JOB_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_ft_t_g on QUARTZ.qrtz_fired_triggers(SCHED_NAME,TRIGGER_NAME,TRIGGER_GROUP) TABLESPACE QUARTZ_IDX;
create index QUARTZ.idx_qrtz_ft_tg on QUARTZ.qrtz_fired_triggers(SCHED_NAME,TRIGGER_GROUP) TABLESPACE QUARTZ_IDX;

grant select, insert, update, delete on QUARTZ.qrtz_job_details to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_simple_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_cron_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_simprop_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_blob_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_calendars to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_paused_trigger_grps to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_fired_triggers to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_scheduler_state to quartz_app_role;
grant select, insert, update, delete on QUARTZ.qrtz_locks to quartz_app_role;

CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_job_details FOR QUARTZ.qrtz_job_details;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_triggers FOR QUARTZ.qrtz_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_simple_triggers FOR QUARTZ.qrtz_simple_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_cron_triggers FOR QUARTZ.qrtz_cron_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_simprop_triggers FOR QUARTZ.qrtz_simprop_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_blob_triggers FOR QUARTZ.qrtz_blob_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_calendars FOR QUARTZ.qrtz_calendars;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_paused_trigger_grps FOR QUARTZ.qrtz_paused_trigger_grps;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_fired_triggers FOR QUARTZ.qrtz_fired_triggers;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_scheduler_state FOR QUARTZ.qrtz_scheduler_state;
CREATE OR REPLACE SYNONYM QUARTZ_USER.qrtz_locks FOR QUARTZ.qrtz_locks;

spool off;
