package gov.va.mobilehealth.courier.db;

import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.db.userpermission.UserPermissionDAO;
import gov.va.mobilehealth.courier.db.userpermission.UserRole;
import org.junit.Assert;
import org.junit.Test;

import java.net.UnknownHostException;
import java.util.List;

/**
 * Created by tuoe on 6/26/14.
 */
public class UserPermissionDAOTest {
    @Test
    public void testAnValidDAO ()
    {
        UserPermissionDAO dao = null;
        try {
            dao = UserPermissionDAO.getInstance("mongodb://localhost:27017", "courier");
        } catch (UnknownHostException e) {
            Assert.fail("Unknown host exception");
        }
        Assert.assertNotNull(dao);

    }
    @Test
    public void testDAOListAdd ()
    {
        UserPermissionDAO dao = null;
        try {
            dao = UserPermissionDAO.getInstance("mongodb://localhost:27017", "courier");
            dao.dropCollection();
            UserIdentifier userID = new UserIdentifier();
            userID.setUniqueId("101");
            userID.setAssigningAuthority("edit");
            dao.insertUserPermission(userID, true, true);

            List<UserRole> existings = dao.listAllRecords();
            Assert.assertEquals("collection size should be one", existings.size(), 1);
            Assert.assertEquals("The only user's ID should be 101", existings.get(0).getUserID().getUniqueId(), String.valueOf(101));
            Assert.assertEquals("The only user's authorization should be edit", existings.get(0).getUserID().getAssigningAuthority(), "edit");
            Assert.assertTrue("The only user should be admin", existings.get(0).getRoles().contains("admin"));
            Assert.assertTrue("The only user should be bcm admin", existings.get(0).getRoles().contains("bcm_admin"));

        } catch (UnknownHostException e) {
            Assert.fail("Unknown host exception");
        }

    }

    @Test
    public void testDAOEdit ()
    {
        UserPermissionDAO dao = null;
        try {
            dao = UserPermissionDAO.getInstance("mongodb://localhost:27017", "courier");
            dao.dropCollection();
            UserIdentifier userID = new UserIdentifier();
            userID.setUniqueId("101");
            userID.setAssigningAuthority("edit");
            dao.insertUserPermission(userID, true, true);
            dao.updateUserPermission(userID, false, false);
            List<UserRole> existings = dao.listAllRecords();
            Assert.assertEquals("collection size should be one", existings.size(), 1);
            Assert.assertEquals("The only user's ID should be 101", existings.get(0).getUserID().getUniqueId(), String.valueOf(101));
            Assert.assertEquals("The only user's authorization should be edit", existings.get(0).getUserID().getAssigningAuthority(), "edit");
            Assert.assertTrue("The only user should not be admin", !existings.get(0).getRoles().contains("admin"));
            Assert.assertTrue("The only user should not be admin", !existings.get(0).getRoles().contains("bcm_admin"));

        } catch (UnknownHostException e) {
            Assert.fail("Unknown host exception");
        }

    }

    @Test
    public void testDAOSearch ()
    {
        UserPermissionDAO dao = null;
        try {
            dao = UserPermissionDAO.getInstance("mongodb://localhost:27017", "courier");
            dao.dropCollection();
            UserIdentifier userID = new UserIdentifier();
            userID.setUniqueId("101");
            userID.setAssigningAuthority("edit");
            dao.insertUserPermission(userID, true, true);

            UserRole userFromDB = dao.findExstingUserPermission(userID);
            Assert.assertEquals("The only user's ID should be 101", userFromDB.getUserID().getUniqueId(), String.valueOf(101));
            Assert.assertEquals("The only user's authorization should be edit",userFromDB.getUserID().getAssigningAuthority(), "edit");
            Assert.assertTrue("The only user should be admin", userFromDB.getRoles().contains("admin"));

        } catch (UnknownHostException e) {
            Assert.fail("Unknown host exception");
        }

    }

}
