package gov.va.mobilehealth.courier.db.userpermission;

import com.mongodb.*;
import gov.va.mobilehealth.courier.core.UserIdentifier;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * Created by tuoe on 6/26/14.
 */
public class UserPermissionDAO

{
    public static final String S_USER_PERMISSION_COLLECTION = "staffRole";
    public static final String S_KEY_USER_IDENTIFIER = "userIdentifier";
    public static final String S_UNIQUE_ID = "uniqueId";
    public static final String S_ASSIGNING_AUTHORITY = "assigningAuthority";
    public static final String S_ROLES = "roles";
    public static final String S_LAST_MODIFIED = "lastModifiedDate";
    private static UserPermissionDAO S_INSTANCE;
    private DB m_db;
    private final MongoClient m_mongoClient;

    private UserPermissionDAO (String p_connectionURL, String p_db) throws UnknownHostException {
        MongoClientURI uri = new MongoClientURI(p_connectionURL);
        m_mongoClient = new MongoClient(uri);
        m_db = m_mongoClient.getDB(p_db);
    }

    /**
     * Sychroized get instance to make sure only one DAO available
     *
     * @param p_connectionURL
     * @param p_db
     * @return
     * @throws UnknownHostException
     */
    public static synchronized UserPermissionDAO getInstance (String p_connectionURL, String p_db) throws UnknownHostException {
        if (S_INSTANCE == null)
            S_INSTANCE = new UserPermissionDAO(p_connectionURL, p_db);
        return S_INSTANCE;
    }

    public void dropCollection ()
    {
        m_db.getCollection(S_USER_PERMISSION_COLLECTION).drop();
    }

    /**
     * Retreive all the records in the collection
     *
     * @return
     */
    public List<UserRole> listAllRecords ()
    {
        List<UserRole> result = new ArrayList<UserRole>();
        DBCollection table = m_db.getCollection(S_USER_PERMISSION_COLLECTION);
        DBCursor cursor = table.find();
        cursor.limit(Integer.MAX_VALUE);
        while (cursor.hasNext())
        {
            DBObject eachDocument = cursor.next();
            UserRole eachRecord = getUserRole(eachDocument);
            result.add(eachRecord);
        }
        return result;
    }

    private UserRole getUserRole(DBObject p_eachDocument) {
        if (p_eachDocument == null)
            return null;
        BasicDBObject l_userID = (BasicDBObject) p_eachDocument.get(S_KEY_USER_IDENTIFIER);
        UserIdentifier uid = new UserIdentifier();
        uid.setUniqueId(l_userID.get(S_UNIQUE_ID).toString());
        uid.setAssigningAuthority(l_userID.get(S_ASSIGNING_AUTHORITY).toString());
        List l_roles = (List) p_eachDocument.get(S_ROLES);
        Date l_date = (Date) p_eachDocument.get(S_LAST_MODIFIED);

        return new UserRole(uid, l_roles, l_date);
    }

    /**
     * update existing role
     *
     * @param p_userID
     * @param setAdmin - set as admin or staff
     * @param setBcm - set as bcm admin or not
     * @return
     */
    public int updateUserPermission(UserIdentifier p_userID, boolean setAdmin, boolean setBcm)
    {
        DBCollection table = m_db.getCollection(S_USER_PERMISSION_COLLECTION);

        BasicDBObject query = new BasicDBObject();
        query.put(S_KEY_USER_IDENTIFIER+"."+S_UNIQUE_ID, p_userID.getUniqueId());
        query.put(S_KEY_USER_IDENTIFIER+"."+S_ASSIGNING_AUTHORITY, p_userID.getAssigningAuthority());

        BasicDBObject newDocument = new BasicDBObject();

        List<String> roles = new ArrayList<String>();
        roles.add(setAdmin ? UserRole.Role.admin.toString() : UserRole.Role.staff.toString());
        if( setBcm == true )
          roles.add(UserRole.Role.bcm_admin.toString());

        newDocument.put(S_ROLES, roles);
        newDocument.put(S_LAST_MODIFIED, new Date());

        BasicDBObject updateObj = new BasicDBObject();
        updateObj.put("$set", newDocument);

        WriteResult result = table.update(query, updateObj);
        WriteConcern concern = result.getLastConcern();

        return 1;
    }
    /**
     * update existing role
     *
     * @param p_userID
     * @param setAdmin - set as admin or staff
     * @param setBcm - set as bcm or not
     * @return
     */
    public int insertUserPermission(UserIdentifier p_userID, boolean setAdmin, boolean setBcm) {
        DBCollection table = m_db.getCollection(S_USER_PERMISSION_COLLECTION);
        BasicDBObject newDocument = new BasicDBObject();
        BasicDBObject idObject = new BasicDBObject();
        idObject.put(S_UNIQUE_ID, p_userID.getUniqueId());
        idObject.put(S_ASSIGNING_AUTHORITY, p_userID.getAssigningAuthority());
        List<String> roles = new ArrayList<String>();
        roles.add(setAdmin ? UserRole.Role.admin.toString() : UserRole.Role.staff.toString());
        if( setBcm == true )
          roles.add(UserRole.Role.bcm_admin.toString());
        newDocument.put(S_ROLES, roles);
        newDocument.put(S_KEY_USER_IDENTIFIER, idObject);
        newDocument.put(S_LAST_MODIFIED, new Date());
        table.insert(newDocument);
        return 1;
    }

    /**
     * retrieve the existing record based on the unique user ID
     * @param p_userID
     * @return
     */
    public UserRole findExstingUserPermission(UserIdentifier p_userID)
    {
        DBCollection table = m_db.getCollection(S_USER_PERMISSION_COLLECTION);
        BasicDBObject query = new BasicDBObject();
        query.put(S_KEY_USER_IDENTIFIER+"."+S_UNIQUE_ID, p_userID.getUniqueId());
        query.put(S_KEY_USER_IDENTIFIER+"."+S_ASSIGNING_AUTHORITY, p_userID.getAssigningAuthority());
        DBObject existing = table.findOne(query);
        return getUserRole(existing);

    }

    @Override
    //close down the connection to make sure no connection leak
    protected void finalize() throws Throwable {
        m_mongoClient.close();
        super.finalize();
    }
}
