package gov.va.mobilehealth.courier.db.userpermission;

import gov.va.mobilehealth.courier.core.UserIdentifier;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;
import java.util.Properties;
import java.util.Scanner;

/**
 * Created by tuoe on 6/26/14.
 * Updated by samantha domville oct 2 2014 - added BCM admin role.
 */
public class UserPermissionManager
{
    private static String S_HOST;
    private static String S_PORT;
    private static String S_DB;


    private UserPermissionDAO m_dao;

    /**
     * Entry point of the program
     * @param args
     */
    public static void main(String[] args) {
        new UserPermissionManager().manageAdmin();
    }

    /**
     * Main method to start up the menu and
     */
    public void manageAdmin ()
    {
        try {
            String l_connectionURL = System.getProperty("connectionURL");
            System.out.println("connection URL --> " + l_connectionURL);
            String l_db = System.getProperty("db");
            System.out.println("DB --> " + l_db);
            m_dao = UserPermissionDAO.getInstance (l_connectionURL, l_db);
        } catch (UnknownHostException e) {
            System.out.println ("Can not connect to mongo DB, please make sure you use the correct connection settings.");
            System.exit(2);
        }
        while (true)
        {
            printOutMenu ();
            Scanner in = new Scanner(System.in);
            String choice = in.nextLine();
            if (!correctOperationChoice(choice))
                System.out.println("Please make sure only enter 1-4");
            else
            {
                int p_int = Integer.parseInt(choice);

                switch (p_int)
                {
                    case 1:
                        listAllRecords();
                        break;
                    case 2:
                        saveRecord(true);
                        break;
                    case 3:
                        saveRecord(false);
                        break;
                    case 4:
                        System.out.println("Thanks! Bye");
                        System.exit(1);
                        break;
                }
            }
        }
    }

    /**
     * Handles both insert and update
     * @param p_insert
     */
    private void saveRecord(boolean p_insert) {
        System.out.print("User ID: ");
        Scanner in = new Scanner(System.in);
        String uniqueID = in.nextLine();
        if (uniqueID == null || uniqueID.equalsIgnoreCase(""))
        {
            System.out.println("Invalid input");
            return;
        }
        System.out.print("Authorization (case sensitive): ");
        String authorization = in.nextLine();

        if (authorization == null || authorization.equalsIgnoreCase(""))
        {
            System.out.println("Invalid input");
            return;
        }
        UserIdentifier userID = null;
        try
        {
            userID = new UserIdentifier ();
            userID.setUniqueId(uniqueID);
            userID.setAssigningAuthority(authorization);
        }
        catch (Exception e)
        {
            System.out.println("Invalid user ID! It needs to be an integer");
            return;
        }

        System.out.print("Set as clinical admin? (enter \'yes\' or \'no\') ");
        String admin = in.nextLine();
        if (admin == null || (!admin.equalsIgnoreCase("yes") && !admin.equalsIgnoreCase("no")))
        {
          System.out.println("Invalid input");
          return;
        }

        System.out.print("Set as BCM admin? (enter 'yes' or 'no') ");
        String bcm = in.nextLine();
        if (bcm == null || (!bcm.equalsIgnoreCase("yes") && !bcm.equalsIgnoreCase("no")))
        {
          System.out.println("Invalid input");
          return;
        }

        UserRole l_exstingUserPermission = m_dao.findExstingUserPermission(userID);

        if (!p_insert)
        {
            if (l_exstingUserPermission != null) {
                m_dao.updateUserPermission(userID, admin.equalsIgnoreCase("yes"), bcm.equalsIgnoreCase("yes"));
                System.out.println("One record updated!");
            }
            else
                System.out.println("There is no record with the user ID. Do you mean to insert?");
        }
        else if (l_exstingUserPermission == null)
        {
            m_dao.insertUserPermission(userID, admin.equalsIgnoreCase("yes"), bcm.equalsIgnoreCase("yes"));
            System.out.println("One record inserted!");
        }
        else
            System.out.println("There is already a record with the same user ID. Do you mean to update?");
    }

    /**
     * Retrieve the existing records
     */
    private void listAllRecords() {

        List<UserRole> l_roles = m_dao.listAllRecords();
        int l_userPortionLengh = lengthOfLongestUserID (l_roles) + 13;
        int l_adminPortionLength = 32;
        int l_lastModifyPortionLength = 5;
        int spacesToPrint = 0;

        int totalLength = l_userPortionLengh + l_adminPortionLength + l_lastModifyPortionLength;

        System.out.println();
        print("-", totalLength+15, true);
        System.out.println();
        print("User ID", l_userPortionLengh, false);
        print("Role(s)", l_adminPortionLength, false);
        print("Last Modified", l_lastModifyPortionLength, false); System.out.println();
        print("-", totalLength+15, true);

        if (l_roles == null || l_roles.size() == 0) {
            System.out.println();
            System.out.println("No records found");
        }
        else {
            for (UserRole eachRole : l_roles) {
                List rolesList = eachRole.getRoles();
                System.out.println();

                print(eachRole.getUserID().getUniqueId() + "(" + eachRole.getUserID().getAssigningAuthority() + ")", l_userPortionLengh, false);

                for( int i = 0; i < rolesList.size(); i++ )
                {
                  System.out.print(rolesList.get(i));
                  System.out.print(" ");
                }

                if(rolesList.contains(UserRole.Role.bcm_admin.toString()) && rolesList.size() > 1)
                  spacesToPrint = l_lastModifyPortionLength;
                else if(rolesList.contains(UserRole.Role.bcm_admin.toString()) && rolesList.size() == 1)
                  spacesToPrint = l_lastModifyPortionLength + 6;
                else
                  spacesToPrint = l_lastModifyPortionLength + 10;
                print("", spacesToPrint, false);

                print(eachRole.getLastModified() == null ? "N/A" : eachRole.getLastModified().toString(), l_lastModifyPortionLength, false);
            }
            System.out.println();
        }
        print("-", totalLength+15, true);
        System.out.println();

    }

    /**
     * Get the longest length of the ID combo with consideration of parentheses
     *
     * @param p_userRoles
     * @return
     */
    private int lengthOfLongestUserID (List<UserRole> p_userRoles)
    {
        int longest = 7;
        for (UserRole l_userRole : p_userRoles)
        {
            int l_id_length = l_userRole.getUserID().getUniqueId().length();
            int l_aa_length = l_userRole.getUserID().getAssigningAuthority().length();

            if (longest < l_id_length + l_aa_length)
                longest = l_aa_length + l_id_length;
        }
        return longest + 2;
    }

    /**
     * The heavy lifting method to do the real print. The logic is that
     *
     * 1. If repeat, just loop through and print the content in a row
     * 2. If not repeat, just print the port and rest in empty space
     *
     * @param p_content
     * @param p_length
     * @param p_repeat
     */
    private void print (String p_content, int p_length, boolean p_repeat)
    {
        if (p_repeat)
            for (int i = 0; i< p_length; i++)
                System.out.print(p_content);
        else
        {
            System.out.print(p_content);

            for (int i =0; i < p_length-p_content.length(); i++)
                System.out.print(" ");
        }

    }


    /**
     * Make sure the operation enters 1-4
     * @param p_choice
     * @return
     */
    private boolean correctOperationChoice (String p_choice)
    {
        try
        {
          if (p_choice == null
            ||
            p_choice.isEmpty()
            ||
            (Integer.parseInt(p_choice) <= 0 || Integer.parseInt(p_choice) > 4))
            return false;
        } catch (NumberFormatException e){
          System.out.println("Invalid input");
          return false;
        }
        return true;
    }

    private void printOutMenu ()
    {
        System.out.println("#################################");
        System.out.println("1. List existing user admin roles");
        System.out.println("2. Insert new user admin role");
        System.out.println("3. Update existing admin role");
        System.out.println("4. Exit");
        System.out.println("#################################");
        System.out.print("Please enter 1-4: ");

    }
}

