#########################################################################################
# Stories:   CMS-2010
#
# Tests:      CMS-4136, CMS-4137
#
# Acceptance Criteria: CMS-3970, CMS-3993, CMS-3994, CMS-4099, CMS-4100, CMS-4101, CMS-4102, CMS-4103
#
#########################################################################################

require 'spec_helper'
require 'error_messages'


describe "CMS-2010 Validation of fields when adding or editing a vitals template" do

  wait = Selenium::WebDriver::Wait.new(:timeout => 10) # seconds

  context "As an Admin I can Validate The Schedule Settings Fields" do

    today_date = DateTime.now.strftime('%m/%d/%Y')

    it "Navigate to the Admin home page" do
      page = PatientSearchPage.new(@browser)

      page.goto_admin_page
      page = AdminPage.new(@browser)
    end

    it "Validate Vitals with no fields entered" do
      error_messages = Error_Messages.new
      @protocolName = 'Vital Validation Protocol'
      @focusArea = 'Vital, Validation, Focus, Area'
      @description = 'This is a new protocol created for Vital field Validation'

      protocol = Protocol.new
      protocol.focusArea = @focusArea
      protocol.description = @description
      protocol.newTemplateName = @protocolName

      page = ListTemplatesPage.new(@browser)
      page.waitForLoadingToClear

      page.create_new_protocol(protocol)

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_schedule_settings_element.click
      vitals.custom_readings_alerts_settings_element.click
      vitals.custom_message_settings_element.click

      page.finish_form
      expect(vitals.custom_empty_error_summary).to eq("Error\nWe're sorry, there was an error with the protocol submission. This form cannot be empty. Please fill in the required fields.")
      expect(vitals.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))
    end

    it 'Validate Vitals Schedule Field' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      vitals = page.custom_list[0]
      vitals.custom_schedule='Custom'
      page.finish_form

      expect(vitals.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
      expect(vitals.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
      expect(vitals.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_time_between_each_cycle = '6'
      page.finish_form

      expect(vitals.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_time_between_each_cycle_units = 'Week'
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_schedule='Once'
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_schedule='Specified Days'
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))
      expect(vitals.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      vitals.check_custom_protocol_mon
      vitals.check_custom_protocol_wed
      vitals.check_custom_protocol_fri
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

    end

    it 'Validate Vitals Schedule Time Settings Fields' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)

      vitals = page.custom_list[0]
      @whattimehour = '9'
      @whattimemin = '15'
      @whattimeampm = 'PM'
      vitals.custom_start_time_hour = @whattimehour
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_start_time_minutes = @whattimemin
      page.finish_form

      expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_am_pm = @whattimeampm
      page.finish_form

      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

    end

    it 'Validate Vitals Start/End Time Settings Fields' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)

      vitals = page.custom_list[0]

      vitals.custom_start_period = '2'
      page.finish_form

      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_end_period = '5'
      page.finish_form

      expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_select_start_period = 'Hour'
      page.finish_form

      expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      vitals.custom_select_end_period = 'Month'
      page.finish_form

      expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
      expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

    end


  end


  context "As an Admin I can Validate The Message Settings Fields" do

    it 'Validate Vitals Message Settings Fields' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_service_message = 'Here is the service message for this protocol'
      vitals.custom_reminder_message = 'First reminder message'
      page.finish_form

      expect(vitals.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
      expect(vitals.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_service_message = ''
      vitals.custom_reminder_message = ''
      page.finish_form

      expect(vitals.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

      vitals.custom_service_message = 'Here is the second service message for this protocol'
      vitals.custom_reminder_message = 'Second reminder message'
      vitals.custom_reminder_time_between = '23'
      page.finish_form

      expect(vitals.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_schedule = 'Minute'
      page.finish_form

      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_sent = '25'
      vitals.custom_reminder_message = ''
      page.finish_form

      expect(vitals.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

      vitals.custom_reminder_message = 'Third reminder message'
      page.finish_form

    end

  end

  context "As an Admin I can Validate The Readings and Alerts Fields are not empty" do

    it 'Validate Readings and Alerts fields contain required values' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_template_measurement = 'Blood Pressure'
      page.finish_form

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      vitals.custom_lowest_valid_reading2 = ''
      vitals.custom_critical_low_level_alert2 = ''
      vitals.custom_low_level_alert2 = ''
      page.finish_form

      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      #Parameter 2 fields:
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))


      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '80'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'

      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      vitals.custom_highest_valid_reading2 = ''
      vitals.custom_critical_high_level_alert2 = ''
      vitals.custom_high_level_alert2 = ''
      page.finish_form

      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))
      #Parameter 2 fields:
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.finish_form

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      vitals.custom_lowest_valid_reading2 = ''
      vitals.custom_critical_low_level_alert2 = ''
      vitals.custom_low_level_alert2 = ''
      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      vitals.custom_highest_valid_reading2 = ''
      vitals.custom_critical_high_level_alert2 = ''
      vitals.custom_high_level_alert2 = ''
      vitals.custom_valid_reading_message = 'Valid Blood Pressure Protocol Readings'
      vitals.custom_critical_low_level_alert_message = 'Critical Low Level Alert Message'
      vitals.custom_critical_high_level_alert_message = 'Critical High Level Alert Message'
      vitals.custom_low_level_alert_message = 'Low Level Alert Message'
      vitals.custom_high_level_alert_message = 'High Level Alert Message'
      page.finish

      page.waitForLoadingToClear
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

      #Parameter 2 fields:
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

    end

    it 'Validate Valid Readings values are accurate in relation to each other' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '270'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '80'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.finish_form
#lowest valid equal to highest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '76'
      page.finish_form
      #lowest valid greater than crticial low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading = '81'
      page.finish_form
      #lowest valid greater than alert low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '141'
      page.finish_form
      #lowest valid greater than alert high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '201'
      page.finish_form
      #lowest valid greater than crticial high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_highest_valid_reading = '65'
      page.finish_form
#highest valid less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '199'
      page.finish_form
#highest valid less than critical high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading = '139'
      page.finish_form
#highest valid less than alert high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '79'
      page.finish_form
#highest valid less than alert low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '74'
      page.finish_form
#highest valid less than critical low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '270'
      vitals.custom_lowest_valid_reading2 = '141'
      page.finish_form
#lowest valid 2 greater than highest valid 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '56'
      page.finish_form
#lowest valid 2 greater than critical low 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '61'
      page.finish_form
      #lowest valid 2 greater than low alert 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '91'
      page.finish_form
      #lowest valid 2 greater than alert high 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '101'
      page.finish_form
      #lowest valid 2 greater than critical high 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_highest_valid_reading2 = '35'
      page.finish_form
      #highest valid 2 less than lowest valid 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '99'
      page.finish_form
      #highest valid 2 less than critical high 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '89'
      page.finish_form
      #highest valid 2 less than alert high 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '59'
      page.finish_form
      #highest valid 2 less than alert low 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '54'
      page.finish_form
      #highest valid 2 less than critical low 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '140'
    end


    it 'Validate Critical Readings values are accurate in relation to each other' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '69'
      vitals.custom_low_level_alert = '80'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.finish_form
#critical low less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert = '81'
      page.finish_form
#critical low greater than low alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '141'
      page.finish_form
#critical low greater than high alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '201'
      page.finish_form
#critical low greater than critical high
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '271'
      page.finish_form
#critical low greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_critical_high_level_alert = '69'
      page.finish_form
      #critical high less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '74'
      page.finish_form
      #critical high less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '79'
      page.finish_form
      #critical high less than alert low
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '139'
      page.finish_form
      #critical high less than alert high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '271'
      page.finish_form
      #critical high greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_critical_low_level_alert2 = '39'
      page.finish_form
      #critical low2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert2 = '61'
      page.finish_form
      #critical low2 greater than low alert2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '91'
      page.finish_form
      #critical low2 greater than high alert2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '101'
      page.finish_form
      #critical low2 greater than critical high2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '141'
      page.finish_form
      #critical low2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_critical_high_level_alert2 = '39'
      page.finish_form
      #critical high2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '54'
      page.finish_form
      #critical high2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '59'
      page.finish_form
      #critical high2 less than alert low2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '89'
      page.finish_form
      #critical high2 less than alert high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '141'
      page.finish_form
      #critical high2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert2 = '100'
    end

    it 'Validate Alert Readings values are accurate in relation to each other' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '69'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.finish_form

      #low alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '74'
      page.finish_form
      #low alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '141'
      page.finish_form
      #low alert greater than alert high
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '201'
      page.finish_form
      #low alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '271'
      page.finish_form
      #low alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '80'
      vitals.custom_high_level_alert = '69'
      page.finish_form
      #high alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '74'
      page.finish_form
      #high alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '79'
      page.finish_form
      #high alert less than alert low
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert = '201'
      page.finish_form
      #high alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '271'
      page.finish_form
      #high alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '140'
      vitals.custom_low_level_alert2 = '39'
      page.finish_form
      #low alert2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert2 = '54'
      page.finish_form
      #low alert2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert2 = '91'
      page.finish_form
      #low alert2 greater than alert high2
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '101'
      page.finish_form
      #low alert2 greater than critical high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '141'
      page.finish_form
      #low alert2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '60'
      vitals.custom_high_level_alert2 = '39'
      page.finish_form
      #high alert2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert2 = '54'
      page.finish_form
      #high alert2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert2 = '59'
      page.finish_form
      #high alert2 less than alert low2
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert2 = '101'
      page.finish_form
      #high alert2 greater than critical high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert2 = '141'
      page.finish_form
      #high alert2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert2 = '90'
      page.finish_form
      page.dismiss_modal
    end

  end
 context "Validate fields when editing a vitals protocol" do

    it 'Select newly created protocol and edit' do
      page = ListTemplatesPage.new(@browser)

      @list = page.protocol_list
      @index = page.get_index_by_text(@list, 'Vital Validation Protocol')
      protocol = page.protocol_list[@index].edit_view

    end

    it ' Validate Schedule Settings fields when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      vitals = page.custom_list[0]

      page.waitForLoadingToClear
      vitals.custom_template_name_element.click
      vitals.custom_schedule_settings_element.click

      vitals.uncheck_custom_protocol_mon
      vitals.uncheck_custom_protocol_wed
      vitals.uncheck_custom_protocol_fri
      vitals.custom_start_period = ''
      vitals.custom_end_period = ''

      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      vitals.check_custom_protocol_tues
      vitals.check_custom_protocol_sat
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals.custom_start_period = '8'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals.custom_end_period = '9'

    end


   it 'Validate Message Settings fields when editing' do
     error_messages = Error_Messages.new
     page = TemplateEditPage.new(@browser)
     page.waitForLoadingToClear

     vitals = page.custom_list[0]
     vitals.custom_message_settings_element.click
     vitals.custom_service_message = ''
     page.save_form_with_bottom_button

     expect(vitals.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

     vitals.custom_service_message = 'Here is the EDITED service message for this protocol'
     vitals.custom_reminder_message = ''
     vitals.custom_reminder_time_between = ''
     vitals.custom_reminder_time_sent = ''

     page.save_form_with_bottom_button

     expect(vitals.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

     vitals.custom_reminder_message = 'Edited Reminder Message'

     page.save_form_with_bottom_button
     expect(vitals.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
     expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

     vitals.custom_reminder_time_between = '4'

     page.save_form_with_bottom_button
     expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

     vitals.custom_reminder_time_sent = '8'

   end


   it 'Validate Missing Readings and Alerts when editing' do
     error_messages = Error_Messages.new

     page = TemplateEditPage.new(@browser)
     page.waitForLoadingToClear

     vitals = page.custom_list[0]
     vitals.custom_readings_alerts_settings_element.click

     vitals.custom_valid_reading_message = ''
     vitals.custom_critical_low_level_alert_message = ''
     vitals.custom_critical_high_level_alert_message = ''
     vitals.custom_low_level_alert_message = ''
     vitals.custom_high_level_alert_message = ''
     page.save_form_with_bottom_button

     expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
     expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
     expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
     expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
     expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

     vitals.custom_lowest_valid_reading = ''
     vitals.custom_critical_low_level_alert = ''
     vitals.custom_low_level_alert = ''
     vitals.custom_lowest_valid_reading2 = ''
     vitals.custom_critical_low_level_alert2 = ''
     vitals.custom_low_level_alert2 = ''
     page.save_form_with_bottom_button

     expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
     expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
     expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
     expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
     expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
     expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
     expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))
     #Parameter 2 fields:
     expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
     expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
     expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))

     vitals.custom_lowest_valid_reading = '70'
     vitals.custom_critical_low_level_alert = '75'
     vitals.custom_low_level_alert = '80'
     vitals.custom_lowest_valid_reading2 = '40'
     vitals.custom_critical_low_level_alert2 = '55'
     vitals.custom_low_level_alert2 = '60'

     vitals.custom_highest_valid_reading = ''
     vitals.custom_critical_high_level_alert = ''
     vitals.custom_high_level_alert = ''
     vitals.custom_highest_valid_reading2 = ''
     vitals.custom_critical_high_level_alert2 = ''
     vitals.custom_high_level_alert2 = ''
     page.save_form_with_bottom_button

     expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
     expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
     expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
     expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
     expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
     expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
     expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
     expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))
     #Parameter 2 fields:
     expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
     expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
     expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

     vitals.custom_highest_valid_reading = '270'
     vitals.custom_critical_high_level_alert = '200'
     vitals.custom_high_level_alert = '140'
     vitals.custom_highest_valid_reading2 = '140'
     vitals.custom_critical_high_level_alert2 = '100'
     vitals.custom_high_level_alert2 = '90'
     page.save_form_with_bottom_button

     expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
     expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
     expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
     expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
     expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

     vitals.custom_lowest_valid_reading = ''
     vitals.custom_critical_low_level_alert = ''
     vitals.custom_low_level_alert = ''
     vitals.custom_lowest_valid_reading2 = ''
     vitals.custom_critical_low_level_alert2 = ''
     vitals.custom_low_level_alert2 = ''
     vitals.custom_highest_valid_reading = ''
     vitals.custom_critical_high_level_alert = ''
     vitals.custom_high_level_alert = ''
     vitals.custom_highest_valid_reading2 = ''
     vitals.custom_critical_high_level_alert2 = ''
     vitals.custom_high_level_alert2 = ''
     vitals.custom_valid_reading_message = 'Valid Blood Pressure Protocol Readings'
     vitals.custom_critical_low_level_alert_message = 'Critical Low Level Alert Message'
     vitals.custom_critical_high_level_alert_message = 'Critical High Level Alert Message'
     vitals.custom_low_level_alert_message = 'Low Level Alert Message'
     vitals.custom_high_level_alert_message = 'High Level Alert Message'
     page.save_form_with_bottom_button

     expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
     expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
     expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
     expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
     expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
     expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

     #Parameter 2 fields:
     expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
     expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
     expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
     expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
     expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
     expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

   end


    it 'Validate Valid Readings values are accurate in relation to each other when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '270'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '80'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid equal to highest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '76'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than crticial low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading = '81'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '141'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '201'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than crticial high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_highest_valid_reading = '65'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '199'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than critical high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading = '139'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '79'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '74'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than critical low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '270'
      vitals.custom_lowest_valid_reading2 = '141'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid 2 greater than highest valid 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '56'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid 2 greater than critical low 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '61'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid 2 greater than low alert 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '91'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid 2 greater than alert high 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '101'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid 2 greater than critical high 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_highest_valid_reading2 = '35'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid 2 less than lowest valid 2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '99'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid 2 less than critical high 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '89'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid 2 less than alert high 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '59'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid 2 less than alert low 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '54'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid 2 less than critical low 2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading2 = '140'
    end


    it 'Validate Critical Readings values are accurate in relation to each other when editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '69'
      vitals.custom_low_level_alert = '80'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.save_bottom
      page.waitForLoadingToClear
#critical low less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert = '81'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than low alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '141'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than high alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '201'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than critical high
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '271'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_critical_high_level_alert = '69'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '74'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '79'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than alert low
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '139'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than alert high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '271'
      page.save_bottom
      page.waitForLoadingToClear
#critical high greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_critical_low_level_alert2 = '39'
      page.save_bottom
      page.waitForLoadingToClear
#critical low2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert2 = '61'
      page.save_bottom
      page.waitForLoadingToClear
#critical low2 greater than low alert2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '91'
      page.save_bottom
      page.waitForLoadingToClear
#critical low2 greater than high alert2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '101'
      page.save_bottom
      page.waitForLoadingToClear
#critical low2 greater than critical high2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '141'
      page.save_bottom
      page.waitForLoadingToClear
#critical low2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_critical_high_level_alert2 = '39'
      page.save_bottom
      page.waitForLoadingToClear
#critical high2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '54'
      page.save_bottom
      page.waitForLoadingToClear
#critical high2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '59'
      page.save_bottom
      page.waitForLoadingToClear
#critical high2 less than alert low2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '89'
      page.save_bottom
      page.waitForLoadingToClear
#critical high2 less than alert high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert2 = '141'
      page.save_bottom
      page.waitForLoadingToClear
#critical high2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert2 = '100'
    end


    it 'Validate Alert Readings values are accurate in relation to each other when editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '69'
      vitals.custom_lowest_valid_reading2 = '40'
      vitals.custom_critical_low_level_alert2 = '55'
      vitals.custom_low_level_alert2 = '60'
      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      vitals.custom_highest_valid_reading2 = '140'
      vitals.custom_critical_high_level_alert2 = '100'
      vitals.custom_high_level_alert2 = '90'
      page.save_bottom
      page.waitForLoadingToClear

      #low alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '74'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '141'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than alert high
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '201'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '271'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '80'
      vitals.custom_high_level_alert = '69'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '74'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '79'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than alert low
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert = '201'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '271'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '140'
      vitals.custom_low_level_alert2 = '39'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert2 = '54'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert2 = '91'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert2 greater than alert high2
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '101'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert2 greater than critical high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '141'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert2 = '60'
      vitals.custom_high_level_alert2 = '39'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert2 less than lowest valid2
      expect(vitals.custom_lowest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert2 = '54'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert2 less than critical low2
      expect(vitals.custom_critical_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert2 = '59'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert2 less than alert low2
      expect(vitals.custom_low_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert2 = '101'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert2 greater than critical high2
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert2 = '141'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert2 greater than highest valid2
      expect(vitals.custom_highest_valid_reading2_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert2_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert2 = '90'
      page.save_bottom
      page.waitForLoadingToClear
      page.dismiss_modal
      end

    end
    context 'Verify Field Validations When Creating a Custom Protocol With Two Vitals Templates' do

      it 'Create Custom Template With Two Vitals' do
        error_messages = Error_Messages.new
        @templateName = 'Custom Validation Protocol'
        @description = 'Custom Protocol to validate two vitals templates'
        @focusArea = 'Vitals, Custom, Field Validation'

        protocol = Protocol.new
        protocol.focusArea = @focusArea
        protocol.description = @description
        protocol.newTemplateName = @templateName

        page = ListTemplatesPage.new(@browser)

        page.create_custom_protocol(protocol)

        page= AddNewProtocolPage.new(@browser)
        page.check_vitals_checkbox
        page.vitals_quantity = '2'

        page.next

        page = TemplateEditPage.new(@browser)
        page.waitForLoadingToClear

        vitals = page.custom_list[0]
        vitals.custom_vitals_element.click
        vitals.custom_schedule_settings_element.click
        vitals.custom_readings_alerts_settings_element.click
        vitals.custom_message_settings_element.click

        vitals2 = page.custom_list[1]
        vitals2.custom_vitals_element.click
        vitals2.custom_schedule_settings_element.click
        vitals2.custom_readings_alerts_settings_element.click
        vitals2.custom_message_settings_element.click

        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        expect(vitals2.custom_schedule_error).to eq(error_messages.get_schedule_error_message(:show_schedule_error => true))
        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))
      end


      it 'Validate Vitals Schedule Field for the first template' do
        error_messages = Error_Messages.new
        page = TemplateEditPage.new(@browser)
        vitals = page.custom_list[0]
        vitals.custom_schedule='Custom'
        page.finish
        page.waitForLoadingToClear
        expect(vitals.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
        expect(vitals.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
        expect(vitals.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_time_between_each_cycle = '6'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_time_between_each_cycle_units = 'Week'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_schedule='Once'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_schedule='Specified Days'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))
        expect(vitals.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

        vitals.check_custom_protocol_mon
        vitals.check_custom_protocol_wed
        vitals.check_custom_protocol_fri
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end

      it 'Validate Vitals Schedule Field for the second template' do
        error_messages = Error_Messages.new
        page = TemplateEditPage.new(@browser)
        vitals2 = page.custom_list[1]
        vitals2.custom_schedule='Custom'
        page.finish
        expect(vitals2.custom_error_summary).to eq("Error\nPlease correct errors on the form below.")
        expect(vitals2.custom_time_between_each_cycle_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_error => true))
        expect(vitals2.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_time_between_each_cycle = '2'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_time_between_each_cycle_units_error).to eq(error_messages.get_schedule_error_message(:show_time_between_each_cycle_select_error => true))
        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_time_between_each_cycle_units = 'Day'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_schedule='Once'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_schedule='Specified Days'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))
        expect(vitals2.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

        vitals2.check_custom_protocol_tues
        vitals2.check_custom_protocol_thurs
        vitals2.check_custom_protocol_sat
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end


      it 'Validate Vitals Schedule Time Settings Fields for the first template' do
        error_messages = Error_Messages.new
        page = TemplateEditPage.new(@browser)

        vitals = page.custom_list[0]
        @whattimehour = '9'
        @whattimemin = '15'
        @whattimeampm = 'PM'
        vitals.custom_start_time_hour = @whattimehour
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_start_time_minutes = @whattimemin
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_am_pm = @whattimeampm
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end


      it 'Validate Vitals Schedule Time Settings Fields for the second template' do
        error_messages = Error_Messages.new
        page = TemplateEditPage.new(@browser)

        vitals2 = page.custom_list[1]
        @whattimehour = '6'
        @whattimemin = '45'
        @whattimeampm = 'AM'
        vitals2.custom_start_time_hour = @whattimehour
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_start_time_minutes = @whattimemin
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_service_message_start_time_error).to eq(error_messages.get_schedule_error_message(:show_service_message_start_time_error => true))
        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_am_pm = @whattimeampm
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end

      it 'Validate Vitals Start/End Time Settings Fields for the first template' do
        error_messages = Error_Messages.new

        page = TemplateEditPage.new(@browser)

        vitals = page.custom_list[0]

        vitals.custom_start_period = '2'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_end_period = '5'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_select_start_period = 'Hour'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals.custom_select_end_period = 'Month'
        page.finish
        page.waitForLoadingToClear

        expect(vitals.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end

      it 'Validate Vitals Start/End Time Settings Fields for the second template' do
        error_messages = Error_Messages.new

        page = TemplateEditPage.new(@browser)

        vitals2 = page.custom_list[1]

        vitals2.custom_start_period = '4'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_end_period = '8'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_select_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_select_error => true))
        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_select_start_period = 'Minute'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_select_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_select_error => true))
        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

        vitals2.custom_select_end_period = 'Year'
        page.finish
        page.waitForLoadingToClear

        expect(vitals2.custom_template_measurement_error).to eq(error_messages.get_schedule_error_message(:show_measurement_error => true))
        expect(vitals2.custom_service_message_error).to eq(error_messages.get_schedule_error_message(:show_service_message_error => true))

      end

    end

  context "As an Admin I can Validate The Message Settings Fields of a Custom Protocol with Two Templates" do

    it 'Validate Vitals Message Settings Fields for the first template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_service_message = 'Here is the service message for the first template'
      vitals.custom_reminder_message = 'First Template, First reminder message'
      page.finish_form

      expect(vitals.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
      expect(vitals.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_service_message = ''
      vitals.custom_reminder_message = ''
      page.finish_form

      expect(vitals.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

      vitals.custom_service_message = 'Here is the second service message for the first templatel'
      vitals.custom_reminder_message = 'First Template, Second reminder message'
      vitals.custom_reminder_time_between = '23'
      page.finish_form

      expect(vitals.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_schedule = 'Minute'
      page.finish_form

      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_sent = '25'
      vitals.custom_reminder_message = ''
      page.finish_form

      expect(vitals.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

      vitals.custom_reminder_message = 'First Template,m Third reminder message'
      page.finish_form

    end

    it 'Validate Vitals Message Settings Fields for the second template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]
      vitals2.custom_service_message = 'Here is the service message for the second template'
      vitals2.custom_reminder_message = 'Second Template, First reminder message'
      page.finish_form

      expect(vitals2.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
      expect(vitals2.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals2.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals2.custom_service_message = ''
      vitals2.custom_reminder_message = ''
      page.finish_form

      expect(vitals2.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

      vitals2.custom_service_message = 'Here is the second service message for the second template'
      vitals2.custom_reminder_message = 'Second Template, Second reminder message'
      vitals2.custom_reminder_time_between = '23'
      page.finish_form

      expect(vitals2.custom_reminder_time_schedule_error).to eq(error_messages.get_message_error_message(:show_reminder_time_schedule_error => true))
      expect(vitals2.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals2.custom_reminder_time_schedule = 'Minute'
      page.finish_form

      expect(vitals2.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals2.custom_reminder_time_sent = '25'
      vitals2.custom_reminder_message = ''
      page.finish_form

      expect(vitals2.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

      vitals2.custom_reminder_message = 'Second Template, Third reminder message'
      page.finish_form

    end

      end
  context "As an Admin I can Validate The Readings and Alerts Fields are Valid in Multiple Templates" do

    it 'Validate Readings and Alerts fields contain required values for the first template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_template_measurement = 'Pulse Oximetry (SpO2)'
      page.finish_form

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      page.finish_form

      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_low_level_alert = '70'

      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      page.finish_form

      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.finish_form

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      vitals.custom_valid_reading_message = 'Valid Pulse Oxemitry Protocol Readings'
      vitals.custom_critical_low_level_alert_message = 'Critical Low Level Alert Message'
      vitals.custom_critical_high_level_alert_message = 'Critical High Level Alert Message'
      vitals.custom_low_level_alert_message = 'Low Level Alert Message'
      vitals.custom_high_level_alert_message = 'High Level Alert Message'
      page.finish_form

      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

    end

    it 'Validate Readings and Alerts fields contain required values for the second template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]

      vitals2.custom_template_measurement = 'Blood Glucose'
      page.finish_form

      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = ''
      vitals2.custom_low_level_alert = ''
      page.finish_form

      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = '60'
      vitals2.custom_low_level_alert = '70'

      vitals2.custom_highest_valid_reading = ''
      vitals2.custom_high_level_alert = ''
      page.finish_form

      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_highest_valid_reading = '100'
      vitals2.custom_high_level_alert = '90'
      page.finish_form

      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = ''
      vitals2.custom_low_level_alert = ''
      vitals2.custom_highest_valid_reading = ''
      vitals2.custom_high_level_alert = ''
      vitals2.custom_valid_reading_message = 'Valid Blood Glucose Protocol Readings'
      vitals2.custom_low_level_alert_message = 'Low Level Alert Message'
      vitals2.custom_high_level_alert_message = 'High Level Alert Message'
      page.finish_form

      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

    end


    it 'Validate Valid Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '100'
      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_low_level_alert = '70'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
       page.finish_form
#lowest valid equal to highest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '66'
      page.finish_form
#lowest valid greater than crticial low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading = '71'
      page.finish_form
#lowest valid greater than alert low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '91'
      page.finish_form
#lowest valid greater than alert high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '96'
      page.finish_form
#lowest valid greater than crticial high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_highest_valid_reading = '59'
      page.finish_form
#highest valid less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '94'
      page.finish_form
#highest valid less than critical high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading = '89'
      page.finish_form
#highest valid less than alert high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '69'
      page.finish_form
#highest valid less than alert low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '64'
      page.finish_form
#highest valid less than critical low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '100'

    end

    it 'Validate Valid Readings values are accurate in relation to each other for the second template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]

      vitals2.custom_lowest_valid_reading = '300'
      vitals2.custom_low_level_alert = '40'
      vitals2.custom_highest_valid_reading = '300'
      vitals2.custom_high_level_alert = '200'
      page.finish_form
#lowest valid equal to highest valid
      #expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '41'
      page.finish_form
#lowest valid greater than alert low
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '201'
      page.finish_form
#lowest valid greater than alert high
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '30'
      vitals2.custom_highest_valid_reading = '29'
      page.finish_form
#highest valid less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '199'
      page.finish_form
#highest valid less than alert high
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '39'
      page.finish_form
#highest valid less than alert low
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '300'

    end

    it 'Validate Critical Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_readings_alerts_settings_element.click

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_critical_low_level_alert = '59'
      vitals.custom_low_level_alert = '70'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.finish_form
#critical low less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert = '71'
      page.finish_form
#critical low greater than low alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '91'
      page.finish_form
#critical low greater than high alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '96'
      page.finish_form
#critical low greater than critical high
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '101'
      page.finish_form
#critical low greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_critical_high_level_alert = '59'
      page.finish_form
#critical high less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '64'
      page.finish_form
#critical high less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '69'
      page.finish_form
#critical high less than alert low
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '89'
      page.finish_form
#critical high less than alert high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '101'
      page.finish_form
#critical high greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert = '95'
    end

    it 'Validate Alert Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_low_level_alert = '59'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.finish_form

      #low alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '64'
      page.finish_form
      #low alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '91'
      page.finish_form
      #low alert greater than alert high
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '96'
      page.finish_form
      #low alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '101'
      page.finish_form
      #low alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '70'
      vitals.custom_high_level_alert = '59'
      page.finish_form
      #high alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '64'
      page.finish_form
      #high alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '69'
      page.finish_form
      #high alert less than alert low
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert = '96'
      page.finish_form
      #high alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '101'
      page.finish_form
      #high alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '90'

    end

    it 'Validate Alert Readings values are accurate in relation to each other for the second template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]
      vitals2.custom_readings_alerts_settings_element.click

      vitals2.custom_lowest_valid_reading = '30'
      vitals2.custom_low_level_alert = '29'
      vitals2.custom_highest_valid_reading = '300'
      vitals2.custom_high_level_alert = '200'
      page.finish_form

      #low alert less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals2.custom_low_level_alert = '201'
      page.finish_form
      #low alert greater than alert high
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_low_level_alert = '301'
      page.finish_form
      #low alert greater than highest valid
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_low_level_alert = '40'
      vitals2.custom_high_level_alert = '29'
      page.finish_form
      #high alert less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true))

      vitals2.custom_high_level_alert = '39'
      page.finish_form
      #high alert less than alert low
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_high_level_alert = '301'
      page.finish_form
      #high alert greater than highest valid
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_high_level_alert = '200'
      page.finish_form
      page.dismiss_modal
    end
  end

  context 'As an Admin I can Validate Custom the edting of protocols with two vital templates' do

    it 'Edit Custom Template With Two Vitals' do
      page = ListTemplatesPage.new(@browser)

      @list = page.protocol_list
      @index = page.get_index_by_text(@list, 'Custom Validation Protocol')
      protocol = page.protocol_list[@index].edit_view

    end


    it ' Validate Schedule Settings fields for the first template when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      vitals = page.custom_list[0]

      page.waitForLoadingToClear
      vitals.custom_template_name_element.click
      vitals.custom_schedule_settings_element.click
      vitals.custom_readings_alerts_settings_element.click
      vitals.custom_message_settings_element.click

      vitals.uncheck_custom_protocol_mon
      vitals.uncheck_custom_protocol_wed
      vitals.uncheck_custom_protocol_fri
      vitals.custom_start_period = ''
      vitals.custom_end_period = ''

      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      vitals.check_custom_protocol_tues
      vitals.check_custom_protocol_sat
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals.custom_start_period = '8'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals.custom_end_period = '9'

    end

    it ' Validate Schedule Settings fields for the second template when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      vitals2 = page.custom_list[1]

      page.waitForLoadingToClear
      vitals2.custom_template_name_element.click
      vitals2.custom_schedule_settings_element.click
      vitals2.custom_readings_alerts_settings_element.click
      vitals2.custom_message_settings_element.click

      vitals2.uncheck_custom_protocol_tues
      vitals2.uncheck_custom_protocol_thurs
      vitals2.uncheck_custom_protocol_sat
      vitals2.custom_start_period = ''
      vitals2.custom_end_period = ''

      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))
      expect(vitals2.custom_select_days_error).to eq(error_messages.get_schedule_error_message(:show_select_days_error => true))

      vitals2.check_custom_protocol_mon
      vitals2.check_custom_protocol_fri
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_start_period_error).to eq(error_messages.get_schedule_error_message(:show_start_period_error => true))
      expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals2.custom_start_period = '8'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_end_period_error).to eq(error_messages.get_schedule_error_message(:show_end_period_error => true))

      vitals2.custom_end_period = '9'

    end

    it 'Validate Message Settings fields for the first template when editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_message_settings_element.click
      vitals.custom_service_message = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

      vitals.custom_service_message = 'Here is the EDITED service message for this protocol'
      vitals.custom_reminder_message = ''
      vitals.custom_reminder_time_between = ''
      vitals.custom_reminder_time_sent = ''

      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

      vitals.custom_reminder_message = 'Edited Reminder Message'

      page.save_bottom
      page.waitForLoadingToClear
      expect(vitals.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_between = '4'

      page.save_bottom
      page.waitForLoadingToClear
      expect(vitals.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals.custom_reminder_time_sent = '8'

    end

    it 'Validate Message Settings fields for the second template when editing' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]
      vitals2.custom_message_settings_element.click
      vitals2.custom_service_message = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_service_message_error).to eq(error_messages.get_message_error_message(:show_service_message_error => true))

      vitals2.custom_service_message = 'Here is the EDITED service message for this protocol'
      vitals2.custom_reminder_message = ''
      vitals2.custom_reminder_time_between = ''
      vitals2.custom_reminder_time_sent = ''

      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_reminder_message_error).to eq(error_messages.get_message_error_message(:show_reminder_message_error => true))

      vitals2.custom_reminder_message = 'Edited Reminder Message'

      page.save_bottom
      page.waitForLoadingToClear
      expect(vitals2.custom_reminder_time_between_error).to eq(error_messages.get_message_error_message(:show_reminder_time_between_error => true))
      expect(vitals2.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals2.custom_reminder_time_between = '4'

      page.save_bottom
      page.waitForLoadingToClear
      expect(vitals2.custom_reminder_time_sent_error).to eq(error_messages.get_message_error_message(:show_reminder_time_sent_error => true))

      vitals2.custom_reminder_time_sent = '8'

    end


    it 'Validate Missing Readings and Alerts for the first template when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_readings_alerts_settings_element.click
      vitals.custom_valid_reading_message = ''
      vitals.custom_critical_low_level_alert_message = ''
      vitals.custom_critical_high_level_alert_message = ''
      vitals.custom_low_level_alert_message = ''
      vitals.custom_high_level_alert_message = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = '70'
      vitals.custom_critical_low_level_alert = '75'
      vitals.custom_low_level_alert = '80'

      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_highest_valid_reading = '270'
      vitals.custom_critical_high_level_alert = '200'
      vitals.custom_high_level_alert = '140'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals.custom_critical_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_low_level_alert_message_error => true))
      expect(vitals.custom_critical_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_critical_high_level_alert_message_error => true))
      expect(vitals.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals.custom_lowest_valid_reading = ''
      vitals.custom_critical_low_level_alert = ''
      vitals.custom_low_level_alert = ''
      vitals.custom_highest_valid_reading = ''
      vitals.custom_critical_high_level_alert = ''
      vitals.custom_high_level_alert = ''
      vitals.custom_valid_reading_message = 'Valid Blood Pressure Protocol Readings'
      vitals.custom_critical_low_level_alert_message = 'Critical Low Level Alert Message'
      vitals.custom_critical_high_level_alert_message = 'Critical High Level Alert Message'
      vitals.custom_low_level_alert_message = 'Low Level Alert Message'
      vitals.custom_high_level_alert_message = 'High Level Alert Message'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_missing_message => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_missing_message => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

    end

    it 'Validate Missing Readings and Alerts for the second template when editing' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]
      vitals2.custom_readings_alerts_settings_element.click
      vitals2.custom_valid_reading_message = ''
      vitals2.custom_low_level_alert_message = ''
      vitals2.custom_high_level_alert_message = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = ''
      vitals2.custom_low_level_alert = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = '70'
      vitals2.custom_low_level_alert = '80'

      vitals2.custom_highest_valid_reading = ''
      vitals2.custom_high_level_alert = ''
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_highest_valid_reading = '270'
      vitals2.custom_high_level_alert = '140'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_valid_reading_message_error).to eq(error_messages.get_readings_error_message(:show_valid_reading_message_error => true))
      expect(vitals2.custom_low_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_low_level_alert_message_error => true))
      expect(vitals2.custom_high_level_alert_message_error).to eq(error_messages.get_readings_error_message(:show_high_level_alert_message_error => true))

      vitals2.custom_lowest_valid_reading = ''
      vitals2.custom_low_level_alert = ''
      vitals2.custom_highest_valid_reading = ''
      vitals2.custom_high_level_alert = ''
      vitals2.custom_valid_reading_message = 'Valid Blood Pressure Protocol Readings'
      vitals2.custom_low_level_alert_message = 'Low Level Alert Message'
      vitals2.custom_high_level_alert_message = 'High Level Alert Message'
      page.save_bottom
      page.waitForLoadingToClear

      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_missing_message => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_missing_message => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_missing_message => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_missing_message => true))

    end

    it 'Validate Valid Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '100'
      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_low_level_alert = '70'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid equal to highest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '66'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than crticial low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_lowest_valid_reading = '71'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert low
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '91'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '96'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than crticial high
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_CH => true,  :show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_LV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_highest_valid_reading = '59'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '94'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than critical high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_highest_valid_reading = '89'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert high
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '69'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '64'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than critical low
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_CH => true, :show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals.custom_highest_valid_reading = '100'

    end

    it 'Validate Valid Readings values are accurate in relation to each other for the second template' do
      error_messages = Error_Messages.new

      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]

      vitals2.custom_lowest_valid_reading = '300'
      vitals2.custom_low_level_alert = '40'
      vitals2.custom_highest_valid_reading = '300'
      vitals2.custom_high_level_alert = '200'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid equal to highest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true, :show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '41'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert low
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '201'
      page.save_bottom
      page.waitForLoadingToClear
#lowest valid greater than alert high
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true, :show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_LV => true))

      vitals2.custom_lowest_valid_reading = '30'
      vitals2.custom_highest_valid_reading = '29'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_HV => true))
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_LV => true, :show_highest_valid_vs_AL => true, :show_highest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '199'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert high
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '39'
      page.save_bottom
      page.waitForLoadingToClear
#highest valid less than alert low
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true, :show_highest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_highest_valid_reading = '300'

    end


    it 'Validate Critical Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]
      vitals.custom_readings_alerts_settings_element.click

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_critical_low_level_alert = '59'
      vitals.custom_low_level_alert = '70'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.save_bottom
      page.waitForLoadingToClear
#critical low less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_LV => true))

      vitals.custom_critical_low_level_alert = '71'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than low alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '91'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than high alert
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '96'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than critical high
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '101'
      page.save_bottom
      page.waitForLoadingToClear
#critical low greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_HV => true, :show_critical_low_vs_CH => true, :show_critical_low_vs_AL => true, :show_critical_low_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CL => true))

      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_critical_high_level_alert = '59'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_CH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_LV => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '64'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_CL => true, :show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '69'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than alert low
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true, :show_critical_high_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '89'
      page.save_bottom
      page.waitForLoadingToClear
#critical high less than alert high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_critical_high_level_alert = '101'
      page.save_bottom
      page.waitForLoadingToClear
#critical high greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_CH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_HV => true))

      vitals.custom_critical_high_level_alert = '95'
    end


    it 'Validate Alert Readings values are accurate in relation to each other for the first template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals = page.custom_list[0]

      vitals.custom_lowest_valid_reading = '60'
      vitals.custom_critical_low_level_alert = '65'
      vitals.custom_low_level_alert = '59'
      vitals.custom_highest_valid_reading = '100'
      vitals.custom_critical_high_level_alert = '95'
      vitals.custom_high_level_alert = '90'
      page.save_bottom
      page.waitForLoadingToClear

      #low alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true, :show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '64'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_CL => true))

      vitals.custom_low_level_alert = '91'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than alert high
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '96'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '101'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AL => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true, :show_low_alert_vs_CH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_low_level_alert = '70'
      vitals.custom_high_level_alert = '59'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than lowest valid
      expect(vitals.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '64'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than critical low
      expect(vitals.custom_critical_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_low_vs_AH => true))
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_CL => true))

      vitals.custom_high_level_alert = '69'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than alert low
      expect(vitals.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals.custom_high_level_alert = '96'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert greater than critical high
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '101'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert greater than highest valid
      expect(vitals.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals.custom_critical_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_critical_high_vs_AH => true))
      expect(vitals.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true, :show_high_alert_vs_CH => true))

      vitals.custom_high_level_alert = '90'

    end


    it 'Validate Alert Readings values are accurate in relation to each other for the second template' do
      error_messages = Error_Messages.new
      page = TemplateEditPage.new(@browser)
      page.waitForLoadingToClear

      vitals2 = page.custom_list[1]
      vitals2.custom_readings_alerts_settings_element.click

      vitals2.custom_lowest_valid_reading = '30'
      vitals2.custom_low_level_alert = '29'
      vitals2.custom_highest_valid_reading = '300'
      vitals2.custom_high_level_alert = '200'
      page.save_bottom
      page.waitForLoadingToClear

      #low alert less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_LV => true))

      vitals2.custom_low_level_alert = '201'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than alert high
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_low_level_alert = '301'
      page.save_bottom
      page.waitForLoadingToClear
      #low alert greater than highest valid
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AL => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_HV => true, :show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_low_level_alert = '40'
      vitals2.custom_high_level_alert = '29'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than lowest valid
      expect(vitals2.custom_lowest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_lowest_valid_vs_AH => true))
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true, :show_high_alert_vs_LV => true))

      vitals2.custom_high_level_alert = '39'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert less than alert low
      expect(vitals2.custom_low_level_alert_error).to eq(error_messages.get_readings_error_message(:show_low_alert_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_AL => true))

      vitals2.custom_high_level_alert = '301'
      page.save_bottom
      page.waitForLoadingToClear
      #high alert greater than highest valid
      expect(vitals2.custom_highest_valid_reading_error).to eq(error_messages.get_readings_error_message(:show_highest_valid_vs_AH => true))
      expect(vitals2.custom_high_level_alert_error).to eq(error_messages.get_readings_error_message(:show_high_alert_vs_HV => true))

      vitals2.custom_high_level_alert = '200'
      page.save_bottom
      page.waitForLoadingToClear
      page.dismiss_modal

      page = ListTemplatesPage.new(@browser)
      page.GoToStaffSearch
    end
  end
end



